/*******************************************************************************

        copyright:      Copyright (c) 2007 Peter Triller. All rights reserved

        license:        BSD style: $(LICENSE)

        version:        Initial release: Sept 2007, 2010

        authors:        Peter, Moritz Warning

		Contains the Unicode Data files converted to structs and simple
		accessor functions. This file is generated by a Perl script. All
		necessary changes should be made in the script, not in this file. 

*******************************************************************************/

module tango.text.UnicodeData;

struct UnicodeData
{
	enum GeneralCategory
    {
        Lu = 1, //  Letter, Uppercase
        Ll = 1 <<  1, //  Letter, Lowercase
        Lt = 1 <<  2, //  Letter, Titlecase
        Lm = 1 <<  3, //  Letter, Modifier
        Lo = 1 <<  4, //  Letter, Other
        Mn = 1 <<  5, //  Mark, Nonspacing
        Mc = 1 <<  6, //  Mark, Spacing Combining
        Me = 1 <<  7, //  Mark, Enclosing
        Nd = 1 <<  8, //  Number, Decimal Digit
        Nl = 1 <<  9, //  Number, Letter
        No = 1 << 10, //  Number, Other
        Pc = 1 << 11, //  Punctuation, Connector
        Pd = 1 << 12, //  Punctuation, Dash
        Ps = 1 << 13, //  Punctuation, Open
        Pe = 1 << 14, //  Punctuation, Close
        Pi = 1 << 15, //  Punctuation, Initial quote (may behave like Ps or Pe depending on usage)
        Pf = 1 << 16, //  Punctuation, Final quote (may behave like Ps or Pe depending on usage)
        Po = 1 << 17, //  Punctuation, Other
        Sm = 1 << 18, //  Symbol, Math
        Sc = 1 << 19, //  Symbol, Currency
        Sk = 1 << 20, //  Symbol, Modifier
        So = 1 << 21, //  Symbol, Other
        Zs = 1 << 22, //  Separator, Space
        Zl = 1 << 23, //  Separator, Line
        Zp = 1 << 24, //  Separator, Paragraph
        Cc = 1 << 25, //  Other, Control
        Cf = 1 << 26, //  Other, Format
        Cs = 1 << 27, //  Other, Surrogate
        Co = 1 << 28, //  Other, Private Use
        Cn = 1 << 29, //  Other, Not Assigned (no characters in the file have this property)
        SpecialMapping = 1 << 30 // Special Bit for detection of specialMappings
    }
    
    
    enum BidiClass
    {
        L, //  Left-to-Right
        LRE, //  Left-to-Right Embedding
        LRO, //  Left-to-Right Override
        R, //  Right-to-Left
        AL, //  Right-to-Left Arabic
        RLE, //  Right-to-Left Embedding
        RLO, //  Right-to-Left Override
        PDF, //  Pop Directional Format
        EN, //  European Number
        ES, //  European Number Separator
        ET, //  European Number Terminator
        AN, //  Arabic Number
        CS, //  Common Number Separator
        NSM, //  Non-Spacing Mark
        BN, //  Boundary Neutral
        B, //  Paragraph Separator
        S, //  Segment Separator
        WS, //  Whitespace
        ON //  Other Neutrals
    }
    
    enum DecompositionType
    {
        None, // Custom type signaling no Decomposition
        Font, //    A font variant (e.g. a blackletter form).
        NoBreak, //    A no-break version of a space or hyphen.
        Initial, //    An initial presentation form (Arabic).
        Medial, //    A medial presentation form (Arabic).
        Final, //    A final presentation form (Arabic).
        Isolated, //    An isolated presentation form (Arabic).
        Circle, //    An encircled form.
        Super, //    A superscript form.
        Sub, //    A subscript form.
        Vertical, //    A vertical layout presentation form.
        Wide, //    A wide (or zenkaku) compatibility character.
        Narrow, //    A narrow (or hankaku) compatibility character.
        Small, //    A small variant form (CNS compatibility).
        Square, //    A CJK squared font variant.
        Fraction, //    A vulgar fraction form.
        Compat //    Otherwise unspecified compatibility character.
    }
    
    enum BidiMirrored
    {
        Y, // Yes
        N  // No
    }
    
    dchar code;
    
//    char[] name;
    
    GeneralCategory generalCategory;
    
//    short canonicalCombiningClass;
    
    //TODO the defaults are not yet set correctly
//    BidiClass bidiClass;
    
    //TODO end
//    DecompositionType decompositionType;
//    dchar[] decompositionMapping;
    
    // TODO Check handling
//    int numeric_1;
//    int numeric_2;
//    double numeric_3;
    
    // TODO end
//    BidiMirrored bidiMirrored;
//    char[] unicode1Name;
//    char [] isoComment;

    dchar simpleUpperCaseMapping;
    dchar simpleLowerCaseMapping;
    dchar simpleTitleCaseMapping;
}

struct SpecialCaseData
{
    dchar code;
    dchar[] upperCaseMapping;
    dchar[] lowerCaseMapping;
    dchar[] titleCaseMapping;
}

struct FoldingCaseData
{
	dchar code;
	dchar[] mapping;
}

private int binary_search(T)(T[] array, dchar code)
{
    uint lo = 0;
    uint hi = array.length;

    while (lo < hi)
    {
        uint mid = lo + (hi - lo) / 2;
        if (code > array[mid].code)
            lo = mid + 1;
        else if (code < array[mid].code)
            hi = mid;
        else
            return mid;
    }
    return -1;
}

UnicodeData* getUnicodeData(dchar code)
{
    //shortcut
    if(code >= 0x0 && code <= 0x36F)
        return &internalUnicodeData[cast(size_t) code - 0x0];
    
    auto idx = binary_search(internalUnicodeData, code);
    return idx >= 0 ? &internalUnicodeData[idx] : null;
}

SpecialCaseData* getSpecialCaseData(dchar code)
{
    auto idx = binary_search(internalSpecialCaseData, code);
    return idx >= 0 ? &internalSpecialCaseData[idx] : null;
}

FoldingCaseData* getFoldingCaseData(dchar code)
{
    //shortcut
    if(code >= 0x41 && code <= 0x5A)
        return &internalFoldingCaseData[cast(size_t) code - 0x41];
    
    auto idx = binary_search(internalFoldingCaseData, code);
    return idx >= 0 ? &internalFoldingCaseData[idx] : null;
}


//shortcuts, make source file smaller
private const
   Lu = UnicodeData.GeneralCategory.Lu,
   Ll = UnicodeData.GeneralCategory.Ll,
   Lt = UnicodeData.GeneralCategory.Lt,
   Lm = UnicodeData.GeneralCategory.Lm,
   Lo = UnicodeData.GeneralCategory.Lo,
   Mn = UnicodeData.GeneralCategory.Mn,
   Mc = UnicodeData.GeneralCategory.Mc,
   Me = UnicodeData.GeneralCategory.Me,
   Nd = UnicodeData.GeneralCategory.Nd,
   Nl = UnicodeData.GeneralCategory.Nl,
   No = UnicodeData.GeneralCategory.No,
   Pc = UnicodeData.GeneralCategory.Pc,
   Pd = UnicodeData.GeneralCategory.Pd,
   Ps = UnicodeData.GeneralCategory.Ps,
   Pe = UnicodeData.GeneralCategory.Pe,
   Pi = UnicodeData.GeneralCategory.Pi,
   Pf = UnicodeData.GeneralCategory.Pf,
   Po = UnicodeData.GeneralCategory.Po,
   Sm = UnicodeData.GeneralCategory.Sm,
   Sc = UnicodeData.GeneralCategory.Sc,
   Sk = UnicodeData.GeneralCategory.Sk,
   So = UnicodeData.GeneralCategory.So,
   Zs = UnicodeData.GeneralCategory.Zs,
   Zl = UnicodeData.GeneralCategory.Zl,
   Zp = UnicodeData.GeneralCategory.Zp,
   Cc = UnicodeData.GeneralCategory.Cc,
   Cf = UnicodeData.GeneralCategory.Cf,
   Cs = UnicodeData.GeneralCategory.Cs,
   Co = UnicodeData.GeneralCategory.Co,
   Cn = UnicodeData.GeneralCategory.Cn,
   SM = UnicodeData.GeneralCategory.SpecialMapping;

private {
UnicodeData internalUnicodeData[] = [
//{<code>, <generalCategory>, <simpleUpperCaseMapping>,
//<simpleLowerCaseMapping>, <simpleTitleCaseMapping>}
{0x0,Cc,0x0,0x0,0x0},
{0x1,Cc,0x1,0x1,0x1},
{0x2,Cc,0x2,0x2,0x2},
{0x3,Cc,0x3,0x3,0x3},
{0x4,Cc,0x4,0x4,0x4},
{0x5,Cc,0x5,0x5,0x5},
{0x6,Cc,0x6,0x6,0x6},
{0x7,Cc,0x7,0x7,0x7},
{0x8,Cc,0x8,0x8,0x8},
{0x9,Cc,0x9,0x9,0x9},
{0xA,Cc,0xA,0xA,0xA},
{0xB,Cc,0xB,0xB,0xB},
{0xC,Cc,0xC,0xC,0xC},
{0xD,Cc,0xD,0xD,0xD},
{0xE,Cc,0xE,0xE,0xE},
{0xF,Cc,0xF,0xF,0xF},
{0x10,Cc,0x10,0x10,0x10},
{0x11,Cc,0x11,0x11,0x11},
{0x12,Cc,0x12,0x12,0x12},
{0x13,Cc,0x13,0x13,0x13},
{0x14,Cc,0x14,0x14,0x14},
{0x15,Cc,0x15,0x15,0x15},
{0x16,Cc,0x16,0x16,0x16},
{0x17,Cc,0x17,0x17,0x17},
{0x18,Cc,0x18,0x18,0x18},
{0x19,Cc,0x19,0x19,0x19},
{0x1A,Cc,0x1A,0x1A,0x1A},
{0x1B,Cc,0x1B,0x1B,0x1B},
{0x1C,Cc,0x1C,0x1C,0x1C},
{0x1D,Cc,0x1D,0x1D,0x1D},
{0x1E,Cc,0x1E,0x1E,0x1E},
{0x1F,Cc,0x1F,0x1F,0x1F},
{0x20,Zs,0x20,0x20,0x20},
{0x21,Po,0x21,0x21,0x21},
{0x22,Po,0x22,0x22,0x22},
{0x23,Po,0x23,0x23,0x23},
{0x24,Sc,0x24,0x24,0x24},
{0x25,Po,0x25,0x25,0x25},
{0x26,Po,0x26,0x26,0x26},
{0x27,Po,0x27,0x27,0x27},
{0x28,Ps,0x28,0x28,0x28},
{0x29,Pe,0x29,0x29,0x29},
{0x2A,Po,0x2A,0x2A,0x2A},
{0x2B,Sm,0x2B,0x2B,0x2B},
{0x2C,Po,0x2C,0x2C,0x2C},
{0x2D,Pd,0x2D,0x2D,0x2D},
{0x2E,Po,0x2E,0x2E,0x2E},
{0x2F,Po,0x2F,0x2F,0x2F},
{0x30,Nd,0x30,0x30,0x30},
{0x31,Nd,0x31,0x31,0x31},
{0x32,Nd,0x32,0x32,0x32},
{0x33,Nd,0x33,0x33,0x33},
{0x34,Nd,0x34,0x34,0x34},
{0x35,Nd,0x35,0x35,0x35},
{0x36,Nd,0x36,0x36,0x36},
{0x37,Nd,0x37,0x37,0x37},
{0x38,Nd,0x38,0x38,0x38},
{0x39,Nd,0x39,0x39,0x39},
{0x3A,Po,0x3A,0x3A,0x3A},
{0x3B,Po,0x3B,0x3B,0x3B},
{0x3C,Sm,0x3C,0x3C,0x3C},
{0x3D,Sm,0x3D,0x3D,0x3D},
{0x3E,Sm,0x3E,0x3E,0x3E},
{0x3F,Po,0x3F,0x3F,0x3F},
{0x40,Po,0x40,0x40,0x40},
{0x41,Lu,0x41,0x61,0x41},
{0x42,Lu,0x42,0x62,0x42},
{0x43,Lu,0x43,0x63,0x43},
{0x44,Lu,0x44,0x64,0x44},
{0x45,Lu,0x45,0x65,0x45},
{0x46,Lu,0x46,0x66,0x46},
{0x47,Lu,0x47,0x67,0x47},
{0x48,Lu,0x48,0x68,0x48},
{0x49,Lu,0x49,0x69,0x49},
{0x4A,Lu,0x4A,0x6A,0x4A},
{0x4B,Lu,0x4B,0x6B,0x4B},
{0x4C,Lu,0x4C,0x6C,0x4C},
{0x4D,Lu,0x4D,0x6D,0x4D},
{0x4E,Lu,0x4E,0x6E,0x4E},
{0x4F,Lu,0x4F,0x6F,0x4F},
{0x50,Lu,0x50,0x70,0x50},
{0x51,Lu,0x51,0x71,0x51},
{0x52,Lu,0x52,0x72,0x52},
{0x53,Lu,0x53,0x73,0x53},
{0x54,Lu,0x54,0x74,0x54},
{0x55,Lu,0x55,0x75,0x55},
{0x56,Lu,0x56,0x76,0x56},
{0x57,Lu,0x57,0x77,0x57},
{0x58,Lu,0x58,0x78,0x58},
{0x59,Lu,0x59,0x79,0x59},
{0x5A,Lu,0x5A,0x7A,0x5A},
{0x5B,Ps,0x5B,0x5B,0x5B},
{0x5C,Po,0x5C,0x5C,0x5C},
{0x5D,Pe,0x5D,0x5D,0x5D},
{0x5E,Sk,0x5E,0x5E,0x5E},
{0x5F,Pc,0x5F,0x5F,0x5F},
{0x60,Sk,0x60,0x60,0x60},
{0x61,Ll,0x41,0x61,0x41},
{0x62,Ll,0x42,0x62,0x42},
{0x63,Ll,0x43,0x63,0x43},
{0x64,Ll,0x44,0x64,0x44},
{0x65,Ll,0x45,0x65,0x45},
{0x66,Ll,0x46,0x66,0x46},
{0x67,Ll,0x47,0x67,0x47},
{0x68,Ll,0x48,0x68,0x48},
{0x69,Ll,0x49,0x69,0x49},
{0x6A,Ll,0x4A,0x6A,0x4A},
{0x6B,Ll,0x4B,0x6B,0x4B},
{0x6C,Ll,0x4C,0x6C,0x4C},
{0x6D,Ll,0x4D,0x6D,0x4D},
{0x6E,Ll,0x4E,0x6E,0x4E},
{0x6F,Ll,0x4F,0x6F,0x4F},
{0x70,Ll,0x50,0x70,0x50},
{0x71,Ll,0x51,0x71,0x51},
{0x72,Ll,0x52,0x72,0x52},
{0x73,Ll,0x53,0x73,0x53},
{0x74,Ll,0x54,0x74,0x54},
{0x75,Ll,0x55,0x75,0x55},
{0x76,Ll,0x56,0x76,0x56},
{0x77,Ll,0x57,0x77,0x57},
{0x78,Ll,0x58,0x78,0x58},
{0x79,Ll,0x59,0x79,0x59},
{0x7A,Ll,0x5A,0x7A,0x5A},
{0x7B,Ps,0x7B,0x7B,0x7B},
{0x7C,Sm,0x7C,0x7C,0x7C},
{0x7D,Pe,0x7D,0x7D,0x7D},
{0x7E,Sm,0x7E,0x7E,0x7E},
{0x7F,Cc,0x7F,0x7F,0x7F},
{0x80,Cc,0x80,0x80,0x80},
{0x81,Cc,0x81,0x81,0x81},
{0x82,Cc,0x82,0x82,0x82},
{0x83,Cc,0x83,0x83,0x83},
{0x84,Cc,0x84,0x84,0x84},
{0x85,Cc,0x85,0x85,0x85},
{0x86,Cc,0x86,0x86,0x86},
{0x87,Cc,0x87,0x87,0x87},
{0x88,Cc,0x88,0x88,0x88},
{0x89,Cc,0x89,0x89,0x89},
{0x8A,Cc,0x8A,0x8A,0x8A},
{0x8B,Cc,0x8B,0x8B,0x8B},
{0x8C,Cc,0x8C,0x8C,0x8C},
{0x8D,Cc,0x8D,0x8D,0x8D},
{0x8E,Cc,0x8E,0x8E,0x8E},
{0x8F,Cc,0x8F,0x8F,0x8F},
{0x90,Cc,0x90,0x90,0x90},
{0x91,Cc,0x91,0x91,0x91},
{0x92,Cc,0x92,0x92,0x92},
{0x93,Cc,0x93,0x93,0x93},
{0x94,Cc,0x94,0x94,0x94},
{0x95,Cc,0x95,0x95,0x95},
{0x96,Cc,0x96,0x96,0x96},
{0x97,Cc,0x97,0x97,0x97},
{0x98,Cc,0x98,0x98,0x98},
{0x99,Cc,0x99,0x99,0x99},
{0x9A,Cc,0x9A,0x9A,0x9A},
{0x9B,Cc,0x9B,0x9B,0x9B},
{0x9C,Cc,0x9C,0x9C,0x9C},
{0x9D,Cc,0x9D,0x9D,0x9D},
{0x9E,Cc,0x9E,0x9E,0x9E},
{0x9F,Cc,0x9F,0x9F,0x9F},
{0xA0,Zs,0xA0,0xA0,0xA0},
{0xA1,Po,0xA1,0xA1,0xA1},
{0xA2,Sc,0xA2,0xA2,0xA2},
{0xA3,Sc,0xA3,0xA3,0xA3},
{0xA4,Sc,0xA4,0xA4,0xA4},
{0xA5,Sc,0xA5,0xA5,0xA5},
{0xA6,So,0xA6,0xA6,0xA6},
{0xA7,So,0xA7,0xA7,0xA7},
{0xA8,Sk,0xA8,0xA8,0xA8},
{0xA9,So,0xA9,0xA9,0xA9},
{0xAA,Ll,0xAA,0xAA,0xAA},
{0xAB,Pi,0xAB,0xAB,0xAB},
{0xAC,Sm,0xAC,0xAC,0xAC},
{0xAD,Cf,0xAD,0xAD,0xAD},
{0xAE,So,0xAE,0xAE,0xAE},
{0xAF,Sk,0xAF,0xAF,0xAF},
{0xB0,So,0xB0,0xB0,0xB0},
{0xB1,Sm,0xB1,0xB1,0xB1},
{0xB2,No,0xB2,0xB2,0xB2},
{0xB3,No,0xB3,0xB3,0xB3},
{0xB4,Sk,0xB4,0xB4,0xB4},
{0xB5,Ll,0x39C,0xB5,0x39C},
{0xB6,So,0xB6,0xB6,0xB6},
{0xB7,Po,0xB7,0xB7,0xB7},
{0xB8,Sk,0xB8,0xB8,0xB8},
{0xB9,No,0xB9,0xB9,0xB9},
{0xBA,Ll,0xBA,0xBA,0xBA},
{0xBB,Pf,0xBB,0xBB,0xBB},
{0xBC,No,0xBC,0xBC,0xBC},
{0xBD,No,0xBD,0xBD,0xBD},
{0xBE,No,0xBE,0xBE,0xBE},
{0xBF,Po,0xBF,0xBF,0xBF},
{0xC0,Lu,0xC0,0xE0,0xC0},
{0xC1,Lu,0xC1,0xE1,0xC1},
{0xC2,Lu,0xC2,0xE2,0xC2},
{0xC3,Lu,0xC3,0xE3,0xC3},
{0xC4,Lu,0xC4,0xE4,0xC4},
{0xC5,Lu,0xC5,0xE5,0xC5},
{0xC6,Lu,0xC6,0xE6,0xC6},
{0xC7,Lu,0xC7,0xE7,0xC7},
{0xC8,Lu,0xC8,0xE8,0xC8},
{0xC9,Lu,0xC9,0xE9,0xC9},
{0xCA,Lu,0xCA,0xEA,0xCA},
{0xCB,Lu,0xCB,0xEB,0xCB},
{0xCC,Lu,0xCC,0xEC,0xCC},
{0xCD,Lu,0xCD,0xED,0xCD},
{0xCE,Lu,0xCE,0xEE,0xCE},
{0xCF,Lu,0xCF,0xEF,0xCF},
{0xD0,Lu,0xD0,0xF0,0xD0},
{0xD1,Lu,0xD1,0xF1,0xD1},
{0xD2,Lu,0xD2,0xF2,0xD2},
{0xD3,Lu,0xD3,0xF3,0xD3},
{0xD4,Lu,0xD4,0xF4,0xD4},
{0xD5,Lu,0xD5,0xF5,0xD5},
{0xD6,Lu,0xD6,0xF6,0xD6},
{0xD7,Sm,0xD7,0xD7,0xD7},
{0xD8,Lu,0xD8,0xF8,0xD8},
{0xD9,Lu,0xD9,0xF9,0xD9},
{0xDA,Lu,0xDA,0xFA,0xDA},
{0xDB,Lu,0xDB,0xFB,0xDB},
{0xDC,Lu,0xDC,0xFC,0xDC},
{0xDD,Lu,0xDD,0xFD,0xDD},
{0xDE,Lu,0xDE,0xFE,0xDE},
{0xDF,Ll | SM,0xDF,0xDF,0xDF},
{0xE0,Ll,0xC0,0xE0,0xC0},
{0xE1,Ll,0xC1,0xE1,0xC1},
{0xE2,Ll,0xC2,0xE2,0xC2},
{0xE3,Ll,0xC3,0xE3,0xC3},
{0xE4,Ll,0xC4,0xE4,0xC4},
{0xE5,Ll,0xC5,0xE5,0xC5},
{0xE6,Ll,0xC6,0xE6,0xC6},
{0xE7,Ll,0xC7,0xE7,0xC7},
{0xE8,Ll,0xC8,0xE8,0xC8},
{0xE9,Ll,0xC9,0xE9,0xC9},
{0xEA,Ll,0xCA,0xEA,0xCA},
{0xEB,Ll,0xCB,0xEB,0xCB},
{0xEC,Ll,0xCC,0xEC,0xCC},
{0xED,Ll,0xCD,0xED,0xCD},
{0xEE,Ll,0xCE,0xEE,0xCE},
{0xEF,Ll,0xCF,0xEF,0xCF},
{0xF0,Ll,0xD0,0xF0,0xD0},
{0xF1,Ll,0xD1,0xF1,0xD1},
{0xF2,Ll,0xD2,0xF2,0xD2},
{0xF3,Ll,0xD3,0xF3,0xD3},
{0xF4,Ll,0xD4,0xF4,0xD4},
{0xF5,Ll,0xD5,0xF5,0xD5},
{0xF6,Ll,0xD6,0xF6,0xD6},
{0xF7,Sm,0xF7,0xF7,0xF7},
{0xF8,Ll,0xD8,0xF8,0xD8},
{0xF9,Ll,0xD9,0xF9,0xD9},
{0xFA,Ll,0xDA,0xFA,0xDA},
{0xFB,Ll,0xDB,0xFB,0xDB},
{0xFC,Ll,0xDC,0xFC,0xDC},
{0xFD,Ll,0xDD,0xFD,0xDD},
{0xFE,Ll,0xDE,0xFE,0xDE},
{0xFF,Ll,0x178,0xFF,0x178},
{0x100,Lu,0x100,0x101,0x100},
{0x101,Ll,0x100,0x101,0x100},
{0x102,Lu,0x102,0x103,0x102},
{0x103,Ll,0x102,0x103,0x102},
{0x104,Lu,0x104,0x105,0x104},
{0x105,Ll,0x104,0x105,0x104},
{0x106,Lu,0x106,0x107,0x106},
{0x107,Ll,0x106,0x107,0x106},
{0x108,Lu,0x108,0x109,0x108},
{0x109,Ll,0x108,0x109,0x108},
{0x10A,Lu,0x10A,0x10B,0x10A},
{0x10B,Ll,0x10A,0x10B,0x10A},
{0x10C,Lu,0x10C,0x10D,0x10C},
{0x10D,Ll,0x10C,0x10D,0x10C},
{0x10E,Lu,0x10E,0x10F,0x10E},
{0x10F,Ll,0x10E,0x10F,0x10E},
{0x110,Lu,0x110,0x111,0x110},
{0x111,Ll,0x110,0x111,0x110},
{0x112,Lu,0x112,0x113,0x112},
{0x113,Ll,0x112,0x113,0x112},
{0x114,Lu,0x114,0x115,0x114},
{0x115,Ll,0x114,0x115,0x114},
{0x116,Lu,0x116,0x117,0x116},
{0x117,Ll,0x116,0x117,0x116},
{0x118,Lu,0x118,0x119,0x118},
{0x119,Ll,0x118,0x119,0x118},
{0x11A,Lu,0x11A,0x11B,0x11A},
{0x11B,Ll,0x11A,0x11B,0x11A},
{0x11C,Lu,0x11C,0x11D,0x11C},
{0x11D,Ll,0x11C,0x11D,0x11C},
{0x11E,Lu,0x11E,0x11F,0x11E},
{0x11F,Ll,0x11E,0x11F,0x11E},
{0x120,Lu,0x120,0x121,0x120},
{0x121,Ll,0x120,0x121,0x120},
{0x122,Lu,0x122,0x123,0x122},
{0x123,Ll,0x122,0x123,0x122},
{0x124,Lu,0x124,0x125,0x124},
{0x125,Ll,0x124,0x125,0x124},
{0x126,Lu,0x126,0x127,0x126},
{0x127,Ll,0x126,0x127,0x126},
{0x128,Lu,0x128,0x129,0x128},
{0x129,Ll,0x128,0x129,0x128},
{0x12A,Lu,0x12A,0x12B,0x12A},
{0x12B,Ll,0x12A,0x12B,0x12A},
{0x12C,Lu,0x12C,0x12D,0x12C},
{0x12D,Ll,0x12C,0x12D,0x12C},
{0x12E,Lu,0x12E,0x12F,0x12E},
{0x12F,Ll,0x12E,0x12F,0x12E},
{0x130,Lu | SM,0x130,0x69,0x130},
{0x131,Ll,0x49,0x131,0x49},
{0x132,Lu,0x132,0x133,0x132},
{0x133,Ll,0x132,0x133,0x132},
{0x134,Lu,0x134,0x135,0x134},
{0x135,Ll,0x134,0x135,0x134},
{0x136,Lu,0x136,0x137,0x136},
{0x137,Ll,0x136,0x137,0x136},
{0x138,Ll,0x138,0x138,0x138},
{0x139,Lu,0x139,0x13A,0x139},
{0x13A,Ll,0x139,0x13A,0x139},
{0x13B,Lu,0x13B,0x13C,0x13B},
{0x13C,Ll,0x13B,0x13C,0x13B},
{0x13D,Lu,0x13D,0x13E,0x13D},
{0x13E,Ll,0x13D,0x13E,0x13D},
{0x13F,Lu,0x13F,0x140,0x13F},
{0x140,Ll,0x13F,0x140,0x13F},
{0x141,Lu,0x141,0x142,0x141},
{0x142,Ll,0x141,0x142,0x141},
{0x143,Lu,0x143,0x144,0x143},
{0x144,Ll,0x143,0x144,0x143},
{0x145,Lu,0x145,0x146,0x145},
{0x146,Ll,0x145,0x146,0x145},
{0x147,Lu,0x147,0x148,0x147},
{0x148,Ll,0x147,0x148,0x147},
{0x149,Ll | SM,0x149,0x149,0x149},
{0x14A,Lu,0x14A,0x14B,0x14A},
{0x14B,Ll,0x14A,0x14B,0x14A},
{0x14C,Lu,0x14C,0x14D,0x14C},
{0x14D,Ll,0x14C,0x14D,0x14C},
{0x14E,Lu,0x14E,0x14F,0x14E},
{0x14F,Ll,0x14E,0x14F,0x14E},
{0x150,Lu,0x150,0x151,0x150},
{0x151,Ll,0x150,0x151,0x150},
{0x152,Lu,0x152,0x153,0x152},
{0x153,Ll,0x152,0x153,0x152},
{0x154,Lu,0x154,0x155,0x154},
{0x155,Ll,0x154,0x155,0x154},
{0x156,Lu,0x156,0x157,0x156},
{0x157,Ll,0x156,0x157,0x156},
{0x158,Lu,0x158,0x159,0x158},
{0x159,Ll,0x158,0x159,0x158},
{0x15A,Lu,0x15A,0x15B,0x15A},
{0x15B,Ll,0x15A,0x15B,0x15A},
{0x15C,Lu,0x15C,0x15D,0x15C},
{0x15D,Ll,0x15C,0x15D,0x15C},
{0x15E,Lu,0x15E,0x15F,0x15E},
{0x15F,Ll,0x15E,0x15F,0x15E},
{0x160,Lu,0x160,0x161,0x160},
{0x161,Ll,0x160,0x161,0x160},
{0x162,Lu,0x162,0x163,0x162},
{0x163,Ll,0x162,0x163,0x162},
{0x164,Lu,0x164,0x165,0x164},
{0x165,Ll,0x164,0x165,0x164},
{0x166,Lu,0x166,0x167,0x166},
{0x167,Ll,0x166,0x167,0x166},
{0x168,Lu,0x168,0x169,0x168},
{0x169,Ll,0x168,0x169,0x168},
{0x16A,Lu,0x16A,0x16B,0x16A},
{0x16B,Ll,0x16A,0x16B,0x16A},
{0x16C,Lu,0x16C,0x16D,0x16C},
{0x16D,Ll,0x16C,0x16D,0x16C},
{0x16E,Lu,0x16E,0x16F,0x16E},
{0x16F,Ll,0x16E,0x16F,0x16E},
{0x170,Lu,0x170,0x171,0x170},
{0x171,Ll,0x170,0x171,0x170},
{0x172,Lu,0x172,0x173,0x172},
{0x173,Ll,0x172,0x173,0x172},
{0x174,Lu,0x174,0x175,0x174},
{0x175,Ll,0x174,0x175,0x174},
{0x176,Lu,0x176,0x177,0x176},
{0x177,Ll,0x176,0x177,0x176},
{0x178,Lu,0x178,0xFF,0x178},
{0x179,Lu,0x179,0x17A,0x179},
{0x17A,Ll,0x179,0x17A,0x179},
{0x17B,Lu,0x17B,0x17C,0x17B},
{0x17C,Ll,0x17B,0x17C,0x17B},
{0x17D,Lu,0x17D,0x17E,0x17D},
{0x17E,Ll,0x17D,0x17E,0x17D},
{0x17F,Ll,0x53,0x17F,0x53},
{0x180,Ll,0x243,0x180,0x243},
{0x181,Lu,0x181,0x253,0x181},
{0x182,Lu,0x182,0x183,0x182},
{0x183,Ll,0x182,0x183,0x182},
{0x184,Lu,0x184,0x185,0x184},
{0x185,Ll,0x184,0x185,0x184},
{0x186,Lu,0x186,0x254,0x186},
{0x187,Lu,0x187,0x188,0x187},
{0x188,Ll,0x187,0x188,0x187},
{0x189,Lu,0x189,0x256,0x189},
{0x18A,Lu,0x18A,0x257,0x18A},
{0x18B,Lu,0x18B,0x18C,0x18B},
{0x18C,Ll,0x18B,0x18C,0x18B},
{0x18D,Ll,0x18D,0x18D,0x18D},
{0x18E,Lu,0x18E,0x1DD,0x18E},
{0x18F,Lu,0x18F,0x259,0x18F},
{0x190,Lu,0x190,0x25B,0x190},
{0x191,Lu,0x191,0x192,0x191},
{0x192,Ll,0x191,0x192,0x191},
{0x193,Lu,0x193,0x260,0x193},
{0x194,Lu,0x194,0x263,0x194},
{0x195,Ll,0x1F6,0x195,0x1F6},
{0x196,Lu,0x196,0x269,0x196},
{0x197,Lu,0x197,0x268,0x197},
{0x198,Lu,0x198,0x199,0x198},
{0x199,Ll,0x198,0x199,0x198},
{0x19A,Ll,0x23D,0x19A,0x23D},
{0x19B,Ll,0x19B,0x19B,0x19B},
{0x19C,Lu,0x19C,0x26F,0x19C},
{0x19D,Lu,0x19D,0x272,0x19D},
{0x19E,Ll,0x220,0x19E,0x220},
{0x19F,Lu,0x19F,0x275,0x19F},
{0x1A0,Lu,0x1A0,0x1A1,0x1A0},
{0x1A1,Ll,0x1A0,0x1A1,0x1A0},
{0x1A2,Lu,0x1A2,0x1A3,0x1A2},
{0x1A3,Ll,0x1A2,0x1A3,0x1A2},
{0x1A4,Lu,0x1A4,0x1A5,0x1A4},
{0x1A5,Ll,0x1A4,0x1A5,0x1A4},
{0x1A6,Lu,0x1A6,0x280,0x1A6},
{0x1A7,Lu,0x1A7,0x1A8,0x1A7},
{0x1A8,Ll,0x1A7,0x1A8,0x1A7},
{0x1A9,Lu,0x1A9,0x283,0x1A9},
{0x1AA,Ll,0x1AA,0x1AA,0x1AA},
{0x1AB,Ll,0x1AB,0x1AB,0x1AB},
{0x1AC,Lu,0x1AC,0x1AD,0x1AC},
{0x1AD,Ll,0x1AC,0x1AD,0x1AC},
{0x1AE,Lu,0x1AE,0x288,0x1AE},
{0x1AF,Lu,0x1AF,0x1B0,0x1AF},
{0x1B0,Ll,0x1AF,0x1B0,0x1AF},
{0x1B1,Lu,0x1B1,0x28A,0x1B1},
{0x1B2,Lu,0x1B2,0x28B,0x1B2},
{0x1B3,Lu,0x1B3,0x1B4,0x1B3},
{0x1B4,Ll,0x1B3,0x1B4,0x1B3},
{0x1B5,Lu,0x1B5,0x1B6,0x1B5},
{0x1B6,Ll,0x1B5,0x1B6,0x1B5},
{0x1B7,Lu,0x1B7,0x292,0x1B7},
{0x1B8,Lu,0x1B8,0x1B9,0x1B8},
{0x1B9,Ll,0x1B8,0x1B9,0x1B8},
{0x1BA,Ll,0x1BA,0x1BA,0x1BA},
{0x1BB,Lo,0x1BB,0x1BB,0x1BB},
{0x1BC,Lu,0x1BC,0x1BD,0x1BC},
{0x1BD,Ll,0x1BC,0x1BD,0x1BC},
{0x1BE,Ll,0x1BE,0x1BE,0x1BE},
{0x1BF,Ll,0x1F7,0x1BF,0x1F7},
{0x1C0,Lo,0x1C0,0x1C0,0x1C0},
{0x1C1,Lo,0x1C1,0x1C1,0x1C1},
{0x1C2,Lo,0x1C2,0x1C2,0x1C2},
{0x1C3,Lo,0x1C3,0x1C3,0x1C3},
{0x1C4,Lu,0x1C4,0x1C6,0x1C5},
{0x1C5,Lt,0x1C4,0x1C6,0x1C5},
{0x1C6,Ll,0x1C4,0x1C6,0x1C5},
{0x1C7,Lu,0x1C7,0x1C9,0x1C8},
{0x1C8,Lt,0x1C7,0x1C9,0x1C8},
{0x1C9,Ll,0x1C7,0x1C9,0x1C8},
{0x1CA,Lu,0x1CA,0x1CC,0x1CB},
{0x1CB,Lt,0x1CA,0x1CC,0x1CB},
{0x1CC,Ll,0x1CA,0x1CC,0x1CB},
{0x1CD,Lu,0x1CD,0x1CE,0x1CD},
{0x1CE,Ll,0x1CD,0x1CE,0x1CD},
{0x1CF,Lu,0x1CF,0x1D0,0x1CF},
{0x1D0,Ll,0x1CF,0x1D0,0x1CF},
{0x1D1,Lu,0x1D1,0x1D2,0x1D1},
{0x1D2,Ll,0x1D1,0x1D2,0x1D1},
{0x1D3,Lu,0x1D3,0x1D4,0x1D3},
{0x1D4,Ll,0x1D3,0x1D4,0x1D3},
{0x1D5,Lu,0x1D5,0x1D6,0x1D5},
{0x1D6,Ll,0x1D5,0x1D6,0x1D5},
{0x1D7,Lu,0x1D7,0x1D8,0x1D7},
{0x1D8,Ll,0x1D7,0x1D8,0x1D7},
{0x1D9,Lu,0x1D9,0x1DA,0x1D9},
{0x1DA,Ll,0x1D9,0x1DA,0x1D9},
{0x1DB,Lu,0x1DB,0x1DC,0x1DB},
{0x1DC,Ll,0x1DB,0x1DC,0x1DB},
{0x1DD,Ll,0x18E,0x1DD,0x18E},
{0x1DE,Lu,0x1DE,0x1DF,0x1DE},
{0x1DF,Ll,0x1DE,0x1DF,0x1DE},
{0x1E0,Lu,0x1E0,0x1E1,0x1E0},
{0x1E1,Ll,0x1E0,0x1E1,0x1E0},
{0x1E2,Lu,0x1E2,0x1E3,0x1E2},
{0x1E3,Ll,0x1E2,0x1E3,0x1E2},
{0x1E4,Lu,0x1E4,0x1E5,0x1E4},
{0x1E5,Ll,0x1E4,0x1E5,0x1E4},
{0x1E6,Lu,0x1E6,0x1E7,0x1E6},
{0x1E7,Ll,0x1E6,0x1E7,0x1E6},
{0x1E8,Lu,0x1E8,0x1E9,0x1E8},
{0x1E9,Ll,0x1E8,0x1E9,0x1E8},
{0x1EA,Lu,0x1EA,0x1EB,0x1EA},
{0x1EB,Ll,0x1EA,0x1EB,0x1EA},
{0x1EC,Lu,0x1EC,0x1ED,0x1EC},
{0x1ED,Ll,0x1EC,0x1ED,0x1EC},
{0x1EE,Lu,0x1EE,0x1EF,0x1EE},
{0x1EF,Ll,0x1EE,0x1EF,0x1EE},
{0x1F0,Ll | SM,0x1F0,0x1F0,0x1F0},
{0x1F1,Lu,0x1F1,0x1F3,0x1F2},
{0x1F2,Lt,0x1F1,0x1F3,0x1F2},
{0x1F3,Ll,0x1F1,0x1F3,0x1F2},
{0x1F4,Lu,0x1F4,0x1F5,0x1F4},
{0x1F5,Ll,0x1F4,0x1F5,0x1F4},
{0x1F6,Lu,0x1F6,0x195,0x1F6},
{0x1F7,Lu,0x1F7,0x1BF,0x1F7},
{0x1F8,Lu,0x1F8,0x1F9,0x1F8},
{0x1F9,Ll,0x1F8,0x1F9,0x1F8},
{0x1FA,Lu,0x1FA,0x1FB,0x1FA},
{0x1FB,Ll,0x1FA,0x1FB,0x1FA},
{0x1FC,Lu,0x1FC,0x1FD,0x1FC},
{0x1FD,Ll,0x1FC,0x1FD,0x1FC},
{0x1FE,Lu,0x1FE,0x1FF,0x1FE},
{0x1FF,Ll,0x1FE,0x1FF,0x1FE},
{0x200,Lu,0x200,0x201,0x200},
{0x201,Ll,0x200,0x201,0x200},
{0x202,Lu,0x202,0x203,0x202},
{0x203,Ll,0x202,0x203,0x202},
{0x204,Lu,0x204,0x205,0x204},
{0x205,Ll,0x204,0x205,0x204},
{0x206,Lu,0x206,0x207,0x206},
{0x207,Ll,0x206,0x207,0x206},
{0x208,Lu,0x208,0x209,0x208},
{0x209,Ll,0x208,0x209,0x208},
{0x20A,Lu,0x20A,0x20B,0x20A},
{0x20B,Ll,0x20A,0x20B,0x20A},
{0x20C,Lu,0x20C,0x20D,0x20C},
{0x20D,Ll,0x20C,0x20D,0x20C},
{0x20E,Lu,0x20E,0x20F,0x20E},
{0x20F,Ll,0x20E,0x20F,0x20E},
{0x210,Lu,0x210,0x211,0x210},
{0x211,Ll,0x210,0x211,0x210},
{0x212,Lu,0x212,0x213,0x212},
{0x213,Ll,0x212,0x213,0x212},
{0x214,Lu,0x214,0x215,0x214},
{0x215,Ll,0x214,0x215,0x214},
{0x216,Lu,0x216,0x217,0x216},
{0x217,Ll,0x216,0x217,0x216},
{0x218,Lu,0x218,0x219,0x218},
{0x219,Ll,0x218,0x219,0x218},
{0x21A,Lu,0x21A,0x21B,0x21A},
{0x21B,Ll,0x21A,0x21B,0x21A},
{0x21C,Lu,0x21C,0x21D,0x21C},
{0x21D,Ll,0x21C,0x21D,0x21C},
{0x21E,Lu,0x21E,0x21F,0x21E},
{0x21F,Ll,0x21E,0x21F,0x21E},
{0x220,Lu,0x220,0x19E,0x220},
{0x221,Ll,0x221,0x221,0x221},
{0x222,Lu,0x222,0x223,0x222},
{0x223,Ll,0x222,0x223,0x222},
{0x224,Lu,0x224,0x225,0x224},
{0x225,Ll,0x224,0x225,0x224},
{0x226,Lu,0x226,0x227,0x226},
{0x227,Ll,0x226,0x227,0x226},
{0x228,Lu,0x228,0x229,0x228},
{0x229,Ll,0x228,0x229,0x228},
{0x22A,Lu,0x22A,0x22B,0x22A},
{0x22B,Ll,0x22A,0x22B,0x22A},
{0x22C,Lu,0x22C,0x22D,0x22C},
{0x22D,Ll,0x22C,0x22D,0x22C},
{0x22E,Lu,0x22E,0x22F,0x22E},
{0x22F,Ll,0x22E,0x22F,0x22E},
{0x230,Lu,0x230,0x231,0x230},
{0x231,Ll,0x230,0x231,0x230},
{0x232,Lu,0x232,0x233,0x232},
{0x233,Ll,0x232,0x233,0x232},
{0x234,Ll,0x234,0x234,0x234},
{0x235,Ll,0x235,0x235,0x235},
{0x236,Ll,0x236,0x236,0x236},
{0x237,Ll,0x237,0x237,0x237},
{0x238,Ll,0x238,0x238,0x238},
{0x239,Ll,0x239,0x239,0x239},
{0x23A,Lu,0x23A,0x2C65,0x23A},
{0x23B,Lu,0x23B,0x23C,0x23B},
{0x23C,Ll,0x23B,0x23C,0x23B},
{0x23D,Lu,0x23D,0x19A,0x23D},
{0x23E,Lu,0x23E,0x2C66,0x23E},
{0x23F,Ll,0x23F,0x23F,0x23F},
{0x240,Ll,0x240,0x240,0x240},
{0x241,Lu,0x241,0x242,0x241},
{0x242,Ll,0x241,0x242,0x241},
{0x243,Lu,0x243,0x180,0x243},
{0x244,Lu,0x244,0x289,0x244},
{0x245,Lu,0x245,0x28C,0x245},
{0x246,Lu,0x246,0x247,0x246},
{0x247,Ll,0x246,0x247,0x246},
{0x248,Lu,0x248,0x249,0x248},
{0x249,Ll,0x248,0x249,0x248},
{0x24A,Lu,0x24A,0x24B,0x24A},
{0x24B,Ll,0x24A,0x24B,0x24A},
{0x24C,Lu,0x24C,0x24D,0x24C},
{0x24D,Ll,0x24C,0x24D,0x24C},
{0x24E,Lu,0x24E,0x24F,0x24E},
{0x24F,Ll,0x24E,0x24F,0x24E},
{0x250,Ll,0x250,0x250,0x250},
{0x251,Ll,0x251,0x251,0x251},
{0x252,Ll,0x252,0x252,0x252},
{0x253,Ll,0x181,0x253,0x181},
{0x254,Ll,0x186,0x254,0x186},
{0x255,Ll,0x255,0x255,0x255},
{0x256,Ll,0x189,0x256,0x189},
{0x257,Ll,0x18A,0x257,0x18A},
{0x258,Ll,0x258,0x258,0x258},
{0x259,Ll,0x18F,0x259,0x18F},
{0x25A,Ll,0x25A,0x25A,0x25A},
{0x25B,Ll,0x190,0x25B,0x190},
{0x25C,Ll,0x25C,0x25C,0x25C},
{0x25D,Ll,0x25D,0x25D,0x25D},
{0x25E,Ll,0x25E,0x25E,0x25E},
{0x25F,Ll,0x25F,0x25F,0x25F},
{0x260,Ll,0x193,0x260,0x193},
{0x261,Ll,0x261,0x261,0x261},
{0x262,Ll,0x262,0x262,0x262},
{0x263,Ll,0x194,0x263,0x194},
{0x264,Ll,0x264,0x264,0x264},
{0x265,Ll,0x265,0x265,0x265},
{0x266,Ll,0x266,0x266,0x266},
{0x267,Ll,0x267,0x267,0x267},
{0x268,Ll,0x197,0x268,0x197},
{0x269,Ll,0x196,0x269,0x196},
{0x26A,Ll,0x26A,0x26A,0x26A},
{0x26B,Ll,0x2C62,0x26B,0x2C62},
{0x26C,Ll,0x26C,0x26C,0x26C},
{0x26D,Ll,0x26D,0x26D,0x26D},
{0x26E,Ll,0x26E,0x26E,0x26E},
{0x26F,Ll,0x19C,0x26F,0x19C},
{0x270,Ll,0x270,0x270,0x270},
{0x271,Ll,0x271,0x271,0x271},
{0x272,Ll,0x19D,0x272,0x19D},
{0x273,Ll,0x273,0x273,0x273},
{0x274,Ll,0x274,0x274,0x274},
{0x275,Ll,0x19F,0x275,0x19F},
{0x276,Ll,0x276,0x276,0x276},
{0x277,Ll,0x277,0x277,0x277},
{0x278,Ll,0x278,0x278,0x278},
{0x279,Ll,0x279,0x279,0x279},
{0x27A,Ll,0x27A,0x27A,0x27A},
{0x27B,Ll,0x27B,0x27B,0x27B},
{0x27C,Ll,0x27C,0x27C,0x27C},
{0x27D,Ll,0x2C64,0x27D,0x2C64},
{0x27E,Ll,0x27E,0x27E,0x27E},
{0x27F,Ll,0x27F,0x27F,0x27F},
{0x280,Ll,0x1A6,0x280,0x1A6},
{0x281,Ll,0x281,0x281,0x281},
{0x282,Ll,0x282,0x282,0x282},
{0x283,Ll,0x1A9,0x283,0x1A9},
{0x284,Ll,0x284,0x284,0x284},
{0x285,Ll,0x285,0x285,0x285},
{0x286,Ll,0x286,0x286,0x286},
{0x287,Ll,0x287,0x287,0x287},
{0x288,Ll,0x1AE,0x288,0x1AE},
{0x289,Ll,0x244,0x289,0x244},
{0x28A,Ll,0x1B1,0x28A,0x1B1},
{0x28B,Ll,0x1B2,0x28B,0x1B2},
{0x28C,Ll,0x245,0x28C,0x245},
{0x28D,Ll,0x28D,0x28D,0x28D},
{0x28E,Ll,0x28E,0x28E,0x28E},
{0x28F,Ll,0x28F,0x28F,0x28F},
{0x290,Ll,0x290,0x290,0x290},
{0x291,Ll,0x291,0x291,0x291},
{0x292,Ll,0x1B7,0x292,0x1B7},
{0x293,Ll,0x293,0x293,0x293},
{0x294,Lo,0x294,0x294,0x294},
{0x295,Ll,0x295,0x295,0x295},
{0x296,Ll,0x296,0x296,0x296},
{0x297,Ll,0x297,0x297,0x297},
{0x298,Ll,0x298,0x298,0x298},
{0x299,Ll,0x299,0x299,0x299},
{0x29A,Ll,0x29A,0x29A,0x29A},
{0x29B,Ll,0x29B,0x29B,0x29B},
{0x29C,Ll,0x29C,0x29C,0x29C},
{0x29D,Ll,0x29D,0x29D,0x29D},
{0x29E,Ll,0x29E,0x29E,0x29E},
{0x29F,Ll,0x29F,0x29F,0x29F},
{0x2A0,Ll,0x2A0,0x2A0,0x2A0},
{0x2A1,Ll,0x2A1,0x2A1,0x2A1},
{0x2A2,Ll,0x2A2,0x2A2,0x2A2},
{0x2A3,Ll,0x2A3,0x2A3,0x2A3},
{0x2A4,Ll,0x2A4,0x2A4,0x2A4},
{0x2A5,Ll,0x2A5,0x2A5,0x2A5},
{0x2A6,Ll,0x2A6,0x2A6,0x2A6},
{0x2A7,Ll,0x2A7,0x2A7,0x2A7},
{0x2A8,Ll,0x2A8,0x2A8,0x2A8},
{0x2A9,Ll,0x2A9,0x2A9,0x2A9},
{0x2AA,Ll,0x2AA,0x2AA,0x2AA},
{0x2AB,Ll,0x2AB,0x2AB,0x2AB},
{0x2AC,Ll,0x2AC,0x2AC,0x2AC},
{0x2AD,Ll,0x2AD,0x2AD,0x2AD},
{0x2AE,Ll,0x2AE,0x2AE,0x2AE},
{0x2AF,Ll,0x2AF,0x2AF,0x2AF},
{0x2B0,Lm,0x2B0,0x2B0,0x2B0},
{0x2B1,Lm,0x2B1,0x2B1,0x2B1},
{0x2B2,Lm,0x2B2,0x2B2,0x2B2},
{0x2B3,Lm,0x2B3,0x2B3,0x2B3},
{0x2B4,Lm,0x2B4,0x2B4,0x2B4},
{0x2B5,Lm,0x2B5,0x2B5,0x2B5},
{0x2B6,Lm,0x2B6,0x2B6,0x2B6},
{0x2B7,Lm,0x2B7,0x2B7,0x2B7},
{0x2B8,Lm,0x2B8,0x2B8,0x2B8},
{0x2B9,Lm,0x2B9,0x2B9,0x2B9},
{0x2BA,Lm,0x2BA,0x2BA,0x2BA},
{0x2BB,Lm,0x2BB,0x2BB,0x2BB},
{0x2BC,Lm,0x2BC,0x2BC,0x2BC},
{0x2BD,Lm,0x2BD,0x2BD,0x2BD},
{0x2BE,Lm,0x2BE,0x2BE,0x2BE},
{0x2BF,Lm,0x2BF,0x2BF,0x2BF},
{0x2C0,Lm,0x2C0,0x2C0,0x2C0},
{0x2C1,Lm,0x2C1,0x2C1,0x2C1},
{0x2C2,Sk,0x2C2,0x2C2,0x2C2},
{0x2C3,Sk,0x2C3,0x2C3,0x2C3},
{0x2C4,Sk,0x2C4,0x2C4,0x2C4},
{0x2C5,Sk,0x2C5,0x2C5,0x2C5},
{0x2C6,Lm,0x2C6,0x2C6,0x2C6},
{0x2C7,Lm,0x2C7,0x2C7,0x2C7},
{0x2C8,Lm,0x2C8,0x2C8,0x2C8},
{0x2C9,Lm,0x2C9,0x2C9,0x2C9},
{0x2CA,Lm,0x2CA,0x2CA,0x2CA},
{0x2CB,Lm,0x2CB,0x2CB,0x2CB},
{0x2CC,Lm,0x2CC,0x2CC,0x2CC},
{0x2CD,Lm,0x2CD,0x2CD,0x2CD},
{0x2CE,Lm,0x2CE,0x2CE,0x2CE},
{0x2CF,Lm,0x2CF,0x2CF,0x2CF},
{0x2D0,Lm,0x2D0,0x2D0,0x2D0},
{0x2D1,Lm,0x2D1,0x2D1,0x2D1},
{0x2D2,Sk,0x2D2,0x2D2,0x2D2},
{0x2D3,Sk,0x2D3,0x2D3,0x2D3},
{0x2D4,Sk,0x2D4,0x2D4,0x2D4},
{0x2D5,Sk,0x2D5,0x2D5,0x2D5},
{0x2D6,Sk,0x2D6,0x2D6,0x2D6},
{0x2D7,Sk,0x2D7,0x2D7,0x2D7},
{0x2D8,Sk,0x2D8,0x2D8,0x2D8},
{0x2D9,Sk,0x2D9,0x2D9,0x2D9},
{0x2DA,Sk,0x2DA,0x2DA,0x2DA},
{0x2DB,Sk,0x2DB,0x2DB,0x2DB},
{0x2DC,Sk,0x2DC,0x2DC,0x2DC},
{0x2DD,Sk,0x2DD,0x2DD,0x2DD},
{0x2DE,Sk,0x2DE,0x2DE,0x2DE},
{0x2DF,Sk,0x2DF,0x2DF,0x2DF},
{0x2E0,Lm,0x2E0,0x2E0,0x2E0},
{0x2E1,Lm,0x2E1,0x2E1,0x2E1},
{0x2E2,Lm,0x2E2,0x2E2,0x2E2},
{0x2E3,Lm,0x2E3,0x2E3,0x2E3},
{0x2E4,Lm,0x2E4,0x2E4,0x2E4},
{0x2E5,Sk,0x2E5,0x2E5,0x2E5},
{0x2E6,Sk,0x2E6,0x2E6,0x2E6},
{0x2E7,Sk,0x2E7,0x2E7,0x2E7},
{0x2E8,Sk,0x2E8,0x2E8,0x2E8},
{0x2E9,Sk,0x2E9,0x2E9,0x2E9},
{0x2EA,Sk,0x2EA,0x2EA,0x2EA},
{0x2EB,Sk,0x2EB,0x2EB,0x2EB},
{0x2EC,Sk,0x2EC,0x2EC,0x2EC},
{0x2ED,Sk,0x2ED,0x2ED,0x2ED},
{0x2EE,Lm,0x2EE,0x2EE,0x2EE},
{0x2EF,Sk,0x2EF,0x2EF,0x2EF},
{0x2F0,Sk,0x2F0,0x2F0,0x2F0},
{0x2F1,Sk,0x2F1,0x2F1,0x2F1},
{0x2F2,Sk,0x2F2,0x2F2,0x2F2},
{0x2F3,Sk,0x2F3,0x2F3,0x2F3},
{0x2F4,Sk,0x2F4,0x2F4,0x2F4},
{0x2F5,Sk,0x2F5,0x2F5,0x2F5},
{0x2F6,Sk,0x2F6,0x2F6,0x2F6},
{0x2F7,Sk,0x2F7,0x2F7,0x2F7},
{0x2F8,Sk,0x2F8,0x2F8,0x2F8},
{0x2F9,Sk,0x2F9,0x2F9,0x2F9},
{0x2FA,Sk,0x2FA,0x2FA,0x2FA},
{0x2FB,Sk,0x2FB,0x2FB,0x2FB},
{0x2FC,Sk,0x2FC,0x2FC,0x2FC},
{0x2FD,Sk,0x2FD,0x2FD,0x2FD},
{0x2FE,Sk,0x2FE,0x2FE,0x2FE},
{0x2FF,Sk,0x2FF,0x2FF,0x2FF},
{0x300,Mn,0x300,0x300,0x300},
{0x301,Mn,0x301,0x301,0x301},
{0x302,Mn,0x302,0x302,0x302},
{0x303,Mn,0x303,0x303,0x303},
{0x304,Mn,0x304,0x304,0x304},
{0x305,Mn,0x305,0x305,0x305},
{0x306,Mn,0x306,0x306,0x306},
{0x307,Mn,0x307,0x307,0x307},
{0x308,Mn,0x308,0x308,0x308},
{0x309,Mn,0x309,0x309,0x309},
{0x30A,Mn,0x30A,0x30A,0x30A},
{0x30B,Mn,0x30B,0x30B,0x30B},
{0x30C,Mn,0x30C,0x30C,0x30C},
{0x30D,Mn,0x30D,0x30D,0x30D},
{0x30E,Mn,0x30E,0x30E,0x30E},
{0x30F,Mn,0x30F,0x30F,0x30F},
{0x310,Mn,0x310,0x310,0x310},
{0x311,Mn,0x311,0x311,0x311},
{0x312,Mn,0x312,0x312,0x312},
{0x313,Mn,0x313,0x313,0x313},
{0x314,Mn,0x314,0x314,0x314},
{0x315,Mn,0x315,0x315,0x315},
{0x316,Mn,0x316,0x316,0x316},
{0x317,Mn,0x317,0x317,0x317},
{0x318,Mn,0x318,0x318,0x318},
{0x319,Mn,0x319,0x319,0x319},
{0x31A,Mn,0x31A,0x31A,0x31A},
{0x31B,Mn,0x31B,0x31B,0x31B},
{0x31C,Mn,0x31C,0x31C,0x31C},
{0x31D,Mn,0x31D,0x31D,0x31D},
{0x31E,Mn,0x31E,0x31E,0x31E},
{0x31F,Mn,0x31F,0x31F,0x31F},
{0x320,Mn,0x320,0x320,0x320},
{0x321,Mn,0x321,0x321,0x321},
{0x322,Mn,0x322,0x322,0x322},
{0x323,Mn,0x323,0x323,0x323},
{0x324,Mn,0x324,0x324,0x324},
{0x325,Mn,0x325,0x325,0x325},
{0x326,Mn,0x326,0x326,0x326},
{0x327,Mn,0x327,0x327,0x327},
{0x328,Mn,0x328,0x328,0x328},
{0x329,Mn,0x329,0x329,0x329},
{0x32A,Mn,0x32A,0x32A,0x32A},
{0x32B,Mn,0x32B,0x32B,0x32B},
{0x32C,Mn,0x32C,0x32C,0x32C},
{0x32D,Mn,0x32D,0x32D,0x32D},
{0x32E,Mn,0x32E,0x32E,0x32E},
{0x32F,Mn,0x32F,0x32F,0x32F},
{0x330,Mn,0x330,0x330,0x330},
{0x331,Mn,0x331,0x331,0x331},
{0x332,Mn,0x332,0x332,0x332},
{0x333,Mn,0x333,0x333,0x333},
{0x334,Mn,0x334,0x334,0x334},
{0x335,Mn,0x335,0x335,0x335},
{0x336,Mn,0x336,0x336,0x336},
{0x337,Mn,0x337,0x337,0x337},
{0x338,Mn,0x338,0x338,0x338},
{0x339,Mn,0x339,0x339,0x339},
{0x33A,Mn,0x33A,0x33A,0x33A},
{0x33B,Mn,0x33B,0x33B,0x33B},
{0x33C,Mn,0x33C,0x33C,0x33C},
{0x33D,Mn,0x33D,0x33D,0x33D},
{0x33E,Mn,0x33E,0x33E,0x33E},
{0x33F,Mn,0x33F,0x33F,0x33F},
{0x340,Mn,0x340,0x340,0x340},
{0x341,Mn,0x341,0x341,0x341},
{0x342,Mn,0x342,0x342,0x342},
{0x343,Mn,0x343,0x343,0x343},
{0x344,Mn,0x344,0x344,0x344},
{0x345,Mn,0x399,0x345,0x399},
{0x346,Mn,0x346,0x346,0x346},
{0x347,Mn,0x347,0x347,0x347},
{0x348,Mn,0x348,0x348,0x348},
{0x349,Mn,0x349,0x349,0x349},
{0x34A,Mn,0x34A,0x34A,0x34A},
{0x34B,Mn,0x34B,0x34B,0x34B},
{0x34C,Mn,0x34C,0x34C,0x34C},
{0x34D,Mn,0x34D,0x34D,0x34D},
{0x34E,Mn,0x34E,0x34E,0x34E},
{0x34F,Mn,0x34F,0x34F,0x34F},
{0x350,Mn,0x350,0x350,0x350},
{0x351,Mn,0x351,0x351,0x351},
{0x352,Mn,0x352,0x352,0x352},
{0x353,Mn,0x353,0x353,0x353},
{0x354,Mn,0x354,0x354,0x354},
{0x355,Mn,0x355,0x355,0x355},
{0x356,Mn,0x356,0x356,0x356},
{0x357,Mn,0x357,0x357,0x357},
{0x358,Mn,0x358,0x358,0x358},
{0x359,Mn,0x359,0x359,0x359},
{0x35A,Mn,0x35A,0x35A,0x35A},
{0x35B,Mn,0x35B,0x35B,0x35B},
{0x35C,Mn,0x35C,0x35C,0x35C},
{0x35D,Mn,0x35D,0x35D,0x35D},
{0x35E,Mn,0x35E,0x35E,0x35E},
{0x35F,Mn,0x35F,0x35F,0x35F},
{0x360,Mn,0x360,0x360,0x360},
{0x361,Mn,0x361,0x361,0x361},
{0x362,Mn,0x362,0x362,0x362},
{0x363,Mn,0x363,0x363,0x363},
{0x364,Mn,0x364,0x364,0x364},
{0x365,Mn,0x365,0x365,0x365},
{0x366,Mn,0x366,0x366,0x366},
{0x367,Mn,0x367,0x367,0x367},
{0x368,Mn,0x368,0x368,0x368},
{0x369,Mn,0x369,0x369,0x369},
{0x36A,Mn,0x36A,0x36A,0x36A},
{0x36B,Mn,0x36B,0x36B,0x36B},
{0x36C,Mn,0x36C,0x36C,0x36C},
{0x36D,Mn,0x36D,0x36D,0x36D},
{0x36E,Mn,0x36E,0x36E,0x36E},
{0x36F,Mn,0x36F,0x36F,0x36F},
{0x374,Sk,0x374,0x374,0x374},
{0x375,Sk,0x375,0x375,0x375},
{0x37A,Lm,0x37A,0x37A,0x37A},
{0x37B,Ll,0x3FD,0x37B,0x3FD},
{0x37C,Ll,0x3FE,0x37C,0x3FE},
{0x37D,Ll,0x3FF,0x37D,0x3FF},
{0x37E,Po,0x37E,0x37E,0x37E},
{0x384,Sk,0x384,0x384,0x384},
{0x385,Sk,0x385,0x385,0x385},
{0x386,Lu,0x386,0x3AC,0x386},
{0x387,Po,0x387,0x387,0x387},
{0x388,Lu,0x388,0x3AD,0x388},
{0x389,Lu,0x389,0x3AE,0x389},
{0x38A,Lu,0x38A,0x3AF,0x38A},
{0x38C,Lu,0x38C,0x3CC,0x38C},
{0x38E,Lu,0x38E,0x3CD,0x38E},
{0x38F,Lu,0x38F,0x3CE,0x38F},
{0x390,Ll | SM,0x390,0x390,0x390},
{0x391,Lu,0x391,0x3B1,0x391},
{0x392,Lu,0x392,0x3B2,0x392},
{0x393,Lu,0x393,0x3B3,0x393},
{0x394,Lu,0x394,0x3B4,0x394},
{0x395,Lu,0x395,0x3B5,0x395},
{0x396,Lu,0x396,0x3B6,0x396},
{0x397,Lu,0x397,0x3B7,0x397},
{0x398,Lu,0x398,0x3B8,0x398},
{0x399,Lu,0x399,0x3B9,0x399},
{0x39A,Lu,0x39A,0x3BA,0x39A},
{0x39B,Lu,0x39B,0x3BB,0x39B},
{0x39C,Lu,0x39C,0x3BC,0x39C},
{0x39D,Lu,0x39D,0x3BD,0x39D},
{0x39E,Lu,0x39E,0x3BE,0x39E},
{0x39F,Lu,0x39F,0x3BF,0x39F},
{0x3A0,Lu,0x3A0,0x3C0,0x3A0},
{0x3A1,Lu,0x3A1,0x3C1,0x3A1},
{0x3A3,Lu,0x3A3,0x3C3,0x3A3},
{0x3A4,Lu,0x3A4,0x3C4,0x3A4},
{0x3A5,Lu,0x3A5,0x3C5,0x3A5},
{0x3A6,Lu,0x3A6,0x3C6,0x3A6},
{0x3A7,Lu,0x3A7,0x3C7,0x3A7},
{0x3A8,Lu,0x3A8,0x3C8,0x3A8},
{0x3A9,Lu,0x3A9,0x3C9,0x3A9},
{0x3AA,Lu,0x3AA,0x3CA,0x3AA},
{0x3AB,Lu,0x3AB,0x3CB,0x3AB},
{0x3AC,Ll,0x386,0x3AC,0x386},
{0x3AD,Ll,0x388,0x3AD,0x388},
{0x3AE,Ll,0x389,0x3AE,0x389},
{0x3AF,Ll,0x38A,0x3AF,0x38A},
{0x3B0,Ll | SM,0x3B0,0x3B0,0x3B0},
{0x3B1,Ll,0x391,0x3B1,0x391},
{0x3B2,Ll,0x392,0x3B2,0x392},
{0x3B3,Ll,0x393,0x3B3,0x393},
{0x3B4,Ll,0x394,0x3B4,0x394},
{0x3B5,Ll,0x395,0x3B5,0x395},
{0x3B6,Ll,0x396,0x3B6,0x396},
{0x3B7,Ll,0x397,0x3B7,0x397},
{0x3B8,Ll,0x398,0x3B8,0x398},
{0x3B9,Ll,0x399,0x3B9,0x399},
{0x3BA,Ll,0x39A,0x3BA,0x39A},
{0x3BB,Ll,0x39B,0x3BB,0x39B},
{0x3BC,Ll,0x39C,0x3BC,0x39C},
{0x3BD,Ll,0x39D,0x3BD,0x39D},
{0x3BE,Ll,0x39E,0x3BE,0x39E},
{0x3BF,Ll,0x39F,0x3BF,0x39F},
{0x3C0,Ll,0x3A0,0x3C0,0x3A0},
{0x3C1,Ll,0x3A1,0x3C1,0x3A1},
{0x3C2,Ll,0x3A3,0x3C2,0x3A3},
{0x3C3,Ll,0x3A3,0x3C3,0x3A3},
{0x3C4,Ll,0x3A4,0x3C4,0x3A4},
{0x3C5,Ll,0x3A5,0x3C5,0x3A5},
{0x3C6,Ll,0x3A6,0x3C6,0x3A6},
{0x3C7,Ll,0x3A7,0x3C7,0x3A7},
{0x3C8,Ll,0x3A8,0x3C8,0x3A8},
{0x3C9,Ll,0x3A9,0x3C9,0x3A9},
{0x3CA,Ll,0x3AA,0x3CA,0x3AA},
{0x3CB,Ll,0x3AB,0x3CB,0x3AB},
{0x3CC,Ll,0x38C,0x3CC,0x38C},
{0x3CD,Ll,0x38E,0x3CD,0x38E},
{0x3CE,Ll,0x38F,0x3CE,0x38F},
{0x3D0,Ll,0x392,0x3D0,0x392},
{0x3D1,Ll,0x398,0x3D1,0x398},
{0x3D2,Lu,0x3D2,0x3D2,0x3D2},
{0x3D3,Lu,0x3D3,0x3D3,0x3D3},
{0x3D4,Lu,0x3D4,0x3D4,0x3D4},
{0x3D5,Ll,0x3A6,0x3D5,0x3A6},
{0x3D6,Ll,0x3A0,0x3D6,0x3A0},
{0x3D7,Ll,0x3D7,0x3D7,0x3D7},
{0x3D8,Lu,0x3D8,0x3D9,0x3D8},
{0x3D9,Ll,0x3D8,0x3D9,0x3D8},
{0x3DA,Lu,0x3DA,0x3DB,0x3DA},
{0x3DB,Ll,0x3DA,0x3DB,0x3DA},
{0x3DC,Lu,0x3DC,0x3DD,0x3DC},
{0x3DD,Ll,0x3DC,0x3DD,0x3DC},
{0x3DE,Lu,0x3DE,0x3DF,0x3DE},
{0x3DF,Ll,0x3DE,0x3DF,0x3DE},
{0x3E0,Lu,0x3E0,0x3E1,0x3E0},
{0x3E1,Ll,0x3E0,0x3E1,0x3E0},
{0x3E2,Lu,0x3E2,0x3E3,0x3E2},
{0x3E3,Ll,0x3E2,0x3E3,0x3E2},
{0x3E4,Lu,0x3E4,0x3E5,0x3E4},
{0x3E5,Ll,0x3E4,0x3E5,0x3E4},
{0x3E6,Lu,0x3E6,0x3E7,0x3E6},
{0x3E7,Ll,0x3E6,0x3E7,0x3E6},
{0x3E8,Lu,0x3E8,0x3E9,0x3E8},
{0x3E9,Ll,0x3E8,0x3E9,0x3E8},
{0x3EA,Lu,0x3EA,0x3EB,0x3EA},
{0x3EB,Ll,0x3EA,0x3EB,0x3EA},
{0x3EC,Lu,0x3EC,0x3ED,0x3EC},
{0x3ED,Ll,0x3EC,0x3ED,0x3EC},
{0x3EE,Lu,0x3EE,0x3EF,0x3EE},
{0x3EF,Ll,0x3EE,0x3EF,0x3EE},
{0x3F0,Ll,0x39A,0x3F0,0x39A},
{0x3F1,Ll,0x3A1,0x3F1,0x3A1},
{0x3F2,Ll,0x3F9,0x3F2,0x3F9},
{0x3F3,Ll,0x3F3,0x3F3,0x3F3},
{0x3F4,Lu,0x3F4,0x3B8,0x3F4},
{0x3F5,Ll,0x395,0x3F5,0x395},
{0x3F6,Sm,0x3F6,0x3F6,0x3F6},
{0x3F7,Lu,0x3F7,0x3F8,0x3F7},
{0x3F8,Ll,0x3F7,0x3F8,0x3F7},
{0x3F9,Lu,0x3F9,0x3F2,0x3F9},
{0x3FA,Lu,0x3FA,0x3FB,0x3FA},
{0x3FB,Ll,0x3FA,0x3FB,0x3FA},
{0x3FC,Ll,0x3FC,0x3FC,0x3FC},
{0x3FD,Lu,0x3FD,0x37B,0x3FD},
{0x3FE,Lu,0x3FE,0x37C,0x3FE},
{0x3FF,Lu,0x3FF,0x37D,0x3FF},
{0x400,Lu,0x400,0x450,0x400},
{0x401,Lu,0x401,0x451,0x401},
{0x402,Lu,0x402,0x452,0x402},
{0x403,Lu,0x403,0x453,0x403},
{0x404,Lu,0x404,0x454,0x404},
{0x405,Lu,0x405,0x455,0x405},
{0x406,Lu,0x406,0x456,0x406},
{0x407,Lu,0x407,0x457,0x407},
{0x408,Lu,0x408,0x458,0x408},
{0x409,Lu,0x409,0x459,0x409},
{0x40A,Lu,0x40A,0x45A,0x40A},
{0x40B,Lu,0x40B,0x45B,0x40B},
{0x40C,Lu,0x40C,0x45C,0x40C},
{0x40D,Lu,0x40D,0x45D,0x40D},
{0x40E,Lu,0x40E,0x45E,0x40E},
{0x40F,Lu,0x40F,0x45F,0x40F},
{0x410,Lu,0x410,0x430,0x410},
{0x411,Lu,0x411,0x431,0x411},
{0x412,Lu,0x412,0x432,0x412},
{0x413,Lu,0x413,0x433,0x413},
{0x414,Lu,0x414,0x434,0x414},
{0x415,Lu,0x415,0x435,0x415},
{0x416,Lu,0x416,0x436,0x416},
{0x417,Lu,0x417,0x437,0x417},
{0x418,Lu,0x418,0x438,0x418},
{0x419,Lu,0x419,0x439,0x419},
{0x41A,Lu,0x41A,0x43A,0x41A},
{0x41B,Lu,0x41B,0x43B,0x41B},
{0x41C,Lu,0x41C,0x43C,0x41C},
{0x41D,Lu,0x41D,0x43D,0x41D},
{0x41E,Lu,0x41E,0x43E,0x41E},
{0x41F,Lu,0x41F,0x43F,0x41F},
{0x420,Lu,0x420,0x440,0x420},
{0x421,Lu,0x421,0x441,0x421},
{0x422,Lu,0x422,0x442,0x422},
{0x423,Lu,0x423,0x443,0x423},
{0x424,Lu,0x424,0x444,0x424},
{0x425,Lu,0x425,0x445,0x425},
{0x426,Lu,0x426,0x446,0x426},
{0x427,Lu,0x427,0x447,0x427},
{0x428,Lu,0x428,0x448,0x428},
{0x429,Lu,0x429,0x449,0x429},
{0x42A,Lu,0x42A,0x44A,0x42A},
{0x42B,Lu,0x42B,0x44B,0x42B},
{0x42C,Lu,0x42C,0x44C,0x42C},
{0x42D,Lu,0x42D,0x44D,0x42D},
{0x42E,Lu,0x42E,0x44E,0x42E},
{0x42F,Lu,0x42F,0x44F,0x42F},
{0x430,Ll,0x410,0x430,0x410},
{0x431,Ll,0x411,0x431,0x411},
{0x432,Ll,0x412,0x432,0x412},
{0x433,Ll,0x413,0x433,0x413},
{0x434,Ll,0x414,0x434,0x414},
{0x435,Ll,0x415,0x435,0x415},
{0x436,Ll,0x416,0x436,0x416},
{0x437,Ll,0x417,0x437,0x417},
{0x438,Ll,0x418,0x438,0x418},
{0x439,Ll,0x419,0x439,0x419},
{0x43A,Ll,0x41A,0x43A,0x41A},
{0x43B,Ll,0x41B,0x43B,0x41B},
{0x43C,Ll,0x41C,0x43C,0x41C},
{0x43D,Ll,0x41D,0x43D,0x41D},
{0x43E,Ll,0x41E,0x43E,0x41E},
{0x43F,Ll,0x41F,0x43F,0x41F},
{0x440,Ll,0x420,0x440,0x420},
{0x441,Ll,0x421,0x441,0x421},
{0x442,Ll,0x422,0x442,0x422},
{0x443,Ll,0x423,0x443,0x423},
{0x444,Ll,0x424,0x444,0x424},
{0x445,Ll,0x425,0x445,0x425},
{0x446,Ll,0x426,0x446,0x426},
{0x447,Ll,0x427,0x447,0x427},
{0x448,Ll,0x428,0x448,0x428},
{0x449,Ll,0x429,0x449,0x429},
{0x44A,Ll,0x42A,0x44A,0x42A},
{0x44B,Ll,0x42B,0x44B,0x42B},
{0x44C,Ll,0x42C,0x44C,0x42C},
{0x44D,Ll,0x42D,0x44D,0x42D},
{0x44E,Ll,0x42E,0x44E,0x42E},
{0x44F,Ll,0x42F,0x44F,0x42F},
{0x450,Ll,0x400,0x450,0x400},
{0x451,Ll,0x401,0x451,0x401},
{0x452,Ll,0x402,0x452,0x402},
{0x453,Ll,0x403,0x453,0x403},
{0x454,Ll,0x404,0x454,0x404},
{0x455,Ll,0x405,0x455,0x405},
{0x456,Ll,0x406,0x456,0x406},
{0x457,Ll,0x407,0x457,0x407},
{0x458,Ll,0x408,0x458,0x408},
{0x459,Ll,0x409,0x459,0x409},
{0x45A,Ll,0x40A,0x45A,0x40A},
{0x45B,Ll,0x40B,0x45B,0x40B},
{0x45C,Ll,0x40C,0x45C,0x40C},
{0x45D,Ll,0x40D,0x45D,0x40D},
{0x45E,Ll,0x40E,0x45E,0x40E},
{0x45F,Ll,0x40F,0x45F,0x40F},
{0x460,Lu,0x460,0x461,0x460},
{0x461,Ll,0x460,0x461,0x460},
{0x462,Lu,0x462,0x463,0x462},
{0x463,Ll,0x462,0x463,0x462},
{0x464,Lu,0x464,0x465,0x464},
{0x465,Ll,0x464,0x465,0x464},
{0x466,Lu,0x466,0x467,0x466},
{0x467,Ll,0x466,0x467,0x466},
{0x468,Lu,0x468,0x469,0x468},
{0x469,Ll,0x468,0x469,0x468},
{0x46A,Lu,0x46A,0x46B,0x46A},
{0x46B,Ll,0x46A,0x46B,0x46A},
{0x46C,Lu,0x46C,0x46D,0x46C},
{0x46D,Ll,0x46C,0x46D,0x46C},
{0x46E,Lu,0x46E,0x46F,0x46E},
{0x46F,Ll,0x46E,0x46F,0x46E},
{0x470,Lu,0x470,0x471,0x470},
{0x471,Ll,0x470,0x471,0x470},
{0x472,Lu,0x472,0x473,0x472},
{0x473,Ll,0x472,0x473,0x472},
{0x474,Lu,0x474,0x475,0x474},
{0x475,Ll,0x474,0x475,0x474},
{0x476,Lu,0x476,0x477,0x476},
{0x477,Ll,0x476,0x477,0x476},
{0x478,Lu,0x478,0x479,0x478},
{0x479,Ll,0x478,0x479,0x478},
{0x47A,Lu,0x47A,0x47B,0x47A},
{0x47B,Ll,0x47A,0x47B,0x47A},
{0x47C,Lu,0x47C,0x47D,0x47C},
{0x47D,Ll,0x47C,0x47D,0x47C},
{0x47E,Lu,0x47E,0x47F,0x47E},
{0x47F,Ll,0x47E,0x47F,0x47E},
{0x480,Lu,0x480,0x481,0x480},
{0x481,Ll,0x480,0x481,0x480},
{0x482,So,0x482,0x482,0x482},
{0x483,Mn,0x483,0x483,0x483},
{0x484,Mn,0x484,0x484,0x484},
{0x485,Mn,0x485,0x485,0x485},
{0x486,Mn,0x486,0x486,0x486},
{0x488,Me,0x488,0x488,0x488},
{0x489,Me,0x489,0x489,0x489},
{0x48A,Lu,0x48A,0x48B,0x48A},
{0x48B,Ll,0x48A,0x48B,0x48A},
{0x48C,Lu,0x48C,0x48D,0x48C},
{0x48D,Ll,0x48C,0x48D,0x48C},
{0x48E,Lu,0x48E,0x48F,0x48E},
{0x48F,Ll,0x48E,0x48F,0x48E},
{0x490,Lu,0x490,0x491,0x490},
{0x491,Ll,0x490,0x491,0x490},
{0x492,Lu,0x492,0x493,0x492},
{0x493,Ll,0x492,0x493,0x492},
{0x494,Lu,0x494,0x495,0x494},
{0x495,Ll,0x494,0x495,0x494},
{0x496,Lu,0x496,0x497,0x496},
{0x497,Ll,0x496,0x497,0x496},
{0x498,Lu,0x498,0x499,0x498},
{0x499,Ll,0x498,0x499,0x498},
{0x49A,Lu,0x49A,0x49B,0x49A},
{0x49B,Ll,0x49A,0x49B,0x49A},
{0x49C,Lu,0x49C,0x49D,0x49C},
{0x49D,Ll,0x49C,0x49D,0x49C},
{0x49E,Lu,0x49E,0x49F,0x49E},
{0x49F,Ll,0x49E,0x49F,0x49E},
{0x4A0,Lu,0x4A0,0x4A1,0x4A0},
{0x4A1,Ll,0x4A0,0x4A1,0x4A0},
{0x4A2,Lu,0x4A2,0x4A3,0x4A2},
{0x4A3,Ll,0x4A2,0x4A3,0x4A2},
{0x4A4,Lu,0x4A4,0x4A5,0x4A4},
{0x4A5,Ll,0x4A4,0x4A5,0x4A4},
{0x4A6,Lu,0x4A6,0x4A7,0x4A6},
{0x4A7,Ll,0x4A6,0x4A7,0x4A6},
{0x4A8,Lu,0x4A8,0x4A9,0x4A8},
{0x4A9,Ll,0x4A8,0x4A9,0x4A8},
{0x4AA,Lu,0x4AA,0x4AB,0x4AA},
{0x4AB,Ll,0x4AA,0x4AB,0x4AA},
{0x4AC,Lu,0x4AC,0x4AD,0x4AC},
{0x4AD,Ll,0x4AC,0x4AD,0x4AC},
{0x4AE,Lu,0x4AE,0x4AF,0x4AE},
{0x4AF,Ll,0x4AE,0x4AF,0x4AE},
{0x4B0,Lu,0x4B0,0x4B1,0x4B0},
{0x4B1,Ll,0x4B0,0x4B1,0x4B0},
{0x4B2,Lu,0x4B2,0x4B3,0x4B2},
{0x4B3,Ll,0x4B2,0x4B3,0x4B2},
{0x4B4,Lu,0x4B4,0x4B5,0x4B4},
{0x4B5,Ll,0x4B4,0x4B5,0x4B4},
{0x4B6,Lu,0x4B6,0x4B7,0x4B6},
{0x4B7,Ll,0x4B6,0x4B7,0x4B6},
{0x4B8,Lu,0x4B8,0x4B9,0x4B8},
{0x4B9,Ll,0x4B8,0x4B9,0x4B8},
{0x4BA,Lu,0x4BA,0x4BB,0x4BA},
{0x4BB,Ll,0x4BA,0x4BB,0x4BA},
{0x4BC,Lu,0x4BC,0x4BD,0x4BC},
{0x4BD,Ll,0x4BC,0x4BD,0x4BC},
{0x4BE,Lu,0x4BE,0x4BF,0x4BE},
{0x4BF,Ll,0x4BE,0x4BF,0x4BE},
{0x4C0,Lu,0x4C0,0x4CF,0x4C0},
{0x4C1,Lu,0x4C1,0x4C2,0x4C1},
{0x4C2,Ll,0x4C1,0x4C2,0x4C1},
{0x4C3,Lu,0x4C3,0x4C4,0x4C3},
{0x4C4,Ll,0x4C3,0x4C4,0x4C3},
{0x4C5,Lu,0x4C5,0x4C6,0x4C5},
{0x4C6,Ll,0x4C5,0x4C6,0x4C5},
{0x4C7,Lu,0x4C7,0x4C8,0x4C7},
{0x4C8,Ll,0x4C7,0x4C8,0x4C7},
{0x4C9,Lu,0x4C9,0x4CA,0x4C9},
{0x4CA,Ll,0x4C9,0x4CA,0x4C9},
{0x4CB,Lu,0x4CB,0x4CC,0x4CB},
{0x4CC,Ll,0x4CB,0x4CC,0x4CB},
{0x4CD,Lu,0x4CD,0x4CE,0x4CD},
{0x4CE,Ll,0x4CD,0x4CE,0x4CD},
{0x4CF,Ll,0x4C0,0x4CF,0x4C0},
{0x4D0,Lu,0x4D0,0x4D1,0x4D0},
{0x4D1,Ll,0x4D0,0x4D1,0x4D0},
{0x4D2,Lu,0x4D2,0x4D3,0x4D2},
{0x4D3,Ll,0x4D2,0x4D3,0x4D2},
{0x4D4,Lu,0x4D4,0x4D5,0x4D4},
{0x4D5,Ll,0x4D4,0x4D5,0x4D4},
{0x4D6,Lu,0x4D6,0x4D7,0x4D6},
{0x4D7,Ll,0x4D6,0x4D7,0x4D6},
{0x4D8,Lu,0x4D8,0x4D9,0x4D8},
{0x4D9,Ll,0x4D8,0x4D9,0x4D8},
{0x4DA,Lu,0x4DA,0x4DB,0x4DA},
{0x4DB,Ll,0x4DA,0x4DB,0x4DA},
{0x4DC,Lu,0x4DC,0x4DD,0x4DC},
{0x4DD,Ll,0x4DC,0x4DD,0x4DC},
{0x4DE,Lu,0x4DE,0x4DF,0x4DE},
{0x4DF,Ll,0x4DE,0x4DF,0x4DE},
{0x4E0,Lu,0x4E0,0x4E1,0x4E0},
{0x4E1,Ll,0x4E0,0x4E1,0x4E0},
{0x4E2,Lu,0x4E2,0x4E3,0x4E2},
{0x4E3,Ll,0x4E2,0x4E3,0x4E2},
{0x4E4,Lu,0x4E4,0x4E5,0x4E4},
{0x4E5,Ll,0x4E4,0x4E5,0x4E4},
{0x4E6,Lu,0x4E6,0x4E7,0x4E6},
{0x4E7,Ll,0x4E6,0x4E7,0x4E6},
{0x4E8,Lu,0x4E8,0x4E9,0x4E8},
{0x4E9,Ll,0x4E8,0x4E9,0x4E8},
{0x4EA,Lu,0x4EA,0x4EB,0x4EA},
{0x4EB,Ll,0x4EA,0x4EB,0x4EA},
{0x4EC,Lu,0x4EC,0x4ED,0x4EC},
{0x4ED,Ll,0x4EC,0x4ED,0x4EC},
{0x4EE,Lu,0x4EE,0x4EF,0x4EE},
{0x4EF,Ll,0x4EE,0x4EF,0x4EE},
{0x4F0,Lu,0x4F0,0x4F1,0x4F0},
{0x4F1,Ll,0x4F0,0x4F1,0x4F0},
{0x4F2,Lu,0x4F2,0x4F3,0x4F2},
{0x4F3,Ll,0x4F2,0x4F3,0x4F2},
{0x4F4,Lu,0x4F4,0x4F5,0x4F4},
{0x4F5,Ll,0x4F4,0x4F5,0x4F4},
{0x4F6,Lu,0x4F6,0x4F7,0x4F6},
{0x4F7,Ll,0x4F6,0x4F7,0x4F6},
{0x4F8,Lu,0x4F8,0x4F9,0x4F8},
{0x4F9,Ll,0x4F8,0x4F9,0x4F8},
{0x4FA,Lu,0x4FA,0x4FB,0x4FA},
{0x4FB,Ll,0x4FA,0x4FB,0x4FA},
{0x4FC,Lu,0x4FC,0x4FD,0x4FC},
{0x4FD,Ll,0x4FC,0x4FD,0x4FC},
{0x4FE,Lu,0x4FE,0x4FF,0x4FE},
{0x4FF,Ll,0x4FE,0x4FF,0x4FE},
{0x500,Lu,0x500,0x501,0x500},
{0x501,Ll,0x500,0x501,0x500},
{0x502,Lu,0x502,0x503,0x502},
{0x503,Ll,0x502,0x503,0x502},
{0x504,Lu,0x504,0x505,0x504},
{0x505,Ll,0x504,0x505,0x504},
{0x506,Lu,0x506,0x507,0x506},
{0x507,Ll,0x506,0x507,0x506},
{0x508,Lu,0x508,0x509,0x508},
{0x509,Ll,0x508,0x509,0x508},
{0x50A,Lu,0x50A,0x50B,0x50A},
{0x50B,Ll,0x50A,0x50B,0x50A},
{0x50C,Lu,0x50C,0x50D,0x50C},
{0x50D,Ll,0x50C,0x50D,0x50C},
{0x50E,Lu,0x50E,0x50F,0x50E},
{0x50F,Ll,0x50E,0x50F,0x50E},
{0x510,Lu,0x510,0x511,0x510},
{0x511,Ll,0x510,0x511,0x510},
{0x512,Lu,0x512,0x513,0x512},
{0x513,Ll,0x512,0x513,0x512},
{0x531,Lu,0x531,0x561,0x531},
{0x532,Lu,0x532,0x562,0x532},
{0x533,Lu,0x533,0x563,0x533},
{0x534,Lu,0x534,0x564,0x534},
{0x535,Lu,0x535,0x565,0x535},
{0x536,Lu,0x536,0x566,0x536},
{0x537,Lu,0x537,0x567,0x537},
{0x538,Lu,0x538,0x568,0x538},
{0x539,Lu,0x539,0x569,0x539},
{0x53A,Lu,0x53A,0x56A,0x53A},
{0x53B,Lu,0x53B,0x56B,0x53B},
{0x53C,Lu,0x53C,0x56C,0x53C},
{0x53D,Lu,0x53D,0x56D,0x53D},
{0x53E,Lu,0x53E,0x56E,0x53E},
{0x53F,Lu,0x53F,0x56F,0x53F},
{0x540,Lu,0x540,0x570,0x540},
{0x541,Lu,0x541,0x571,0x541},
{0x542,Lu,0x542,0x572,0x542},
{0x543,Lu,0x543,0x573,0x543},
{0x544,Lu,0x544,0x574,0x544},
{0x545,Lu,0x545,0x575,0x545},
{0x546,Lu,0x546,0x576,0x546},
{0x547,Lu,0x547,0x577,0x547},
{0x548,Lu,0x548,0x578,0x548},
{0x549,Lu,0x549,0x579,0x549},
{0x54A,Lu,0x54A,0x57A,0x54A},
{0x54B,Lu,0x54B,0x57B,0x54B},
{0x54C,Lu,0x54C,0x57C,0x54C},
{0x54D,Lu,0x54D,0x57D,0x54D},
{0x54E,Lu,0x54E,0x57E,0x54E},
{0x54F,Lu,0x54F,0x57F,0x54F},
{0x550,Lu,0x550,0x580,0x550},
{0x551,Lu,0x551,0x581,0x551},
{0x552,Lu,0x552,0x582,0x552},
{0x553,Lu,0x553,0x583,0x553},
{0x554,Lu,0x554,0x584,0x554},
{0x555,Lu,0x555,0x585,0x555},
{0x556,Lu,0x556,0x586,0x556},
{0x559,Lm,0x559,0x559,0x559},
{0x55A,Po,0x55A,0x55A,0x55A},
{0x55B,Po,0x55B,0x55B,0x55B},
{0x55C,Po,0x55C,0x55C,0x55C},
{0x55D,Po,0x55D,0x55D,0x55D},
{0x55E,Po,0x55E,0x55E,0x55E},
{0x55F,Po,0x55F,0x55F,0x55F},
{0x561,Ll,0x531,0x561,0x531},
{0x562,Ll,0x532,0x562,0x532},
{0x563,Ll,0x533,0x563,0x533},
{0x564,Ll,0x534,0x564,0x534},
{0x565,Ll,0x535,0x565,0x535},
{0x566,Ll,0x536,0x566,0x536},
{0x567,Ll,0x537,0x567,0x537},
{0x568,Ll,0x538,0x568,0x538},
{0x569,Ll,0x539,0x569,0x539},
{0x56A,Ll,0x53A,0x56A,0x53A},
{0x56B,Ll,0x53B,0x56B,0x53B},
{0x56C,Ll,0x53C,0x56C,0x53C},
{0x56D,Ll,0x53D,0x56D,0x53D},
{0x56E,Ll,0x53E,0x56E,0x53E},
{0x56F,Ll,0x53F,0x56F,0x53F},
{0x570,Ll,0x540,0x570,0x540},
{0x571,Ll,0x541,0x571,0x541},
{0x572,Ll,0x542,0x572,0x542},
{0x573,Ll,0x543,0x573,0x543},
{0x574,Ll,0x544,0x574,0x544},
{0x575,Ll,0x545,0x575,0x545},
{0x576,Ll,0x546,0x576,0x546},
{0x577,Ll,0x547,0x577,0x547},
{0x578,Ll,0x548,0x578,0x548},
{0x579,Ll,0x549,0x579,0x549},
{0x57A,Ll,0x54A,0x57A,0x54A},
{0x57B,Ll,0x54B,0x57B,0x54B},
{0x57C,Ll,0x54C,0x57C,0x54C},
{0x57D,Ll,0x54D,0x57D,0x54D},
{0x57E,Ll,0x54E,0x57E,0x54E},
{0x57F,Ll,0x54F,0x57F,0x54F},
{0x580,Ll,0x550,0x580,0x550},
{0x581,Ll,0x551,0x581,0x551},
{0x582,Ll,0x552,0x582,0x552},
{0x583,Ll,0x553,0x583,0x553},
{0x584,Ll,0x554,0x584,0x554},
{0x585,Ll,0x555,0x585,0x555},
{0x586,Ll,0x556,0x586,0x556},
{0x587,Ll | SM,0x587,0x587,0x587},
{0x589,Po,0x589,0x589,0x589},
{0x58A,Pd,0x58A,0x58A,0x58A},
{0x591,Mn,0x591,0x591,0x591},
{0x592,Mn,0x592,0x592,0x592},
{0x593,Mn,0x593,0x593,0x593},
{0x594,Mn,0x594,0x594,0x594},
{0x595,Mn,0x595,0x595,0x595},
{0x596,Mn,0x596,0x596,0x596},
{0x597,Mn,0x597,0x597,0x597},
{0x598,Mn,0x598,0x598,0x598},
{0x599,Mn,0x599,0x599,0x599},
{0x59A,Mn,0x59A,0x59A,0x59A},
{0x59B,Mn,0x59B,0x59B,0x59B},
{0x59C,Mn,0x59C,0x59C,0x59C},
{0x59D,Mn,0x59D,0x59D,0x59D},
{0x59E,Mn,0x59E,0x59E,0x59E},
{0x59F,Mn,0x59F,0x59F,0x59F},
{0x5A0,Mn,0x5A0,0x5A0,0x5A0},
{0x5A1,Mn,0x5A1,0x5A1,0x5A1},
{0x5A2,Mn,0x5A2,0x5A2,0x5A2},
{0x5A3,Mn,0x5A3,0x5A3,0x5A3},
{0x5A4,Mn,0x5A4,0x5A4,0x5A4},
{0x5A5,Mn,0x5A5,0x5A5,0x5A5},
{0x5A6,Mn,0x5A6,0x5A6,0x5A6},
{0x5A7,Mn,0x5A7,0x5A7,0x5A7},
{0x5A8,Mn,0x5A8,0x5A8,0x5A8},
{0x5A9,Mn,0x5A9,0x5A9,0x5A9},
{0x5AA,Mn,0x5AA,0x5AA,0x5AA},
{0x5AB,Mn,0x5AB,0x5AB,0x5AB},
{0x5AC,Mn,0x5AC,0x5AC,0x5AC},
{0x5AD,Mn,0x5AD,0x5AD,0x5AD},
{0x5AE,Mn,0x5AE,0x5AE,0x5AE},
{0x5AF,Mn,0x5AF,0x5AF,0x5AF},
{0x5B0,Mn,0x5B0,0x5B0,0x5B0},
{0x5B1,Mn,0x5B1,0x5B1,0x5B1},
{0x5B2,Mn,0x5B2,0x5B2,0x5B2},
{0x5B3,Mn,0x5B3,0x5B3,0x5B3},
{0x5B4,Mn,0x5B4,0x5B4,0x5B4},
{0x5B5,Mn,0x5B5,0x5B5,0x5B5},
{0x5B6,Mn,0x5B6,0x5B6,0x5B6},
{0x5B7,Mn,0x5B7,0x5B7,0x5B7},
{0x5B8,Mn,0x5B8,0x5B8,0x5B8},
{0x5B9,Mn,0x5B9,0x5B9,0x5B9},
{0x5BA,Mn,0x5BA,0x5BA,0x5BA},
{0x5BB,Mn,0x5BB,0x5BB,0x5BB},
{0x5BC,Mn,0x5BC,0x5BC,0x5BC},
{0x5BD,Mn,0x5BD,0x5BD,0x5BD},
{0x5BE,Po,0x5BE,0x5BE,0x5BE},
{0x5BF,Mn,0x5BF,0x5BF,0x5BF},
{0x5C0,Po,0x5C0,0x5C0,0x5C0},
{0x5C1,Mn,0x5C1,0x5C1,0x5C1},
{0x5C2,Mn,0x5C2,0x5C2,0x5C2},
{0x5C3,Po,0x5C3,0x5C3,0x5C3},
{0x5C4,Mn,0x5C4,0x5C4,0x5C4},
{0x5C5,Mn,0x5C5,0x5C5,0x5C5},
{0x5C6,Po,0x5C6,0x5C6,0x5C6},
{0x5C7,Mn,0x5C7,0x5C7,0x5C7},
{0x5D0,Lo,0x5D0,0x5D0,0x5D0},
{0x5D1,Lo,0x5D1,0x5D1,0x5D1},
{0x5D2,Lo,0x5D2,0x5D2,0x5D2},
{0x5D3,Lo,0x5D3,0x5D3,0x5D3},
{0x5D4,Lo,0x5D4,0x5D4,0x5D4},
{0x5D5,Lo,0x5D5,0x5D5,0x5D5},
{0x5D6,Lo,0x5D6,0x5D6,0x5D6},
{0x5D7,Lo,0x5D7,0x5D7,0x5D7},
{0x5D8,Lo,0x5D8,0x5D8,0x5D8},
{0x5D9,Lo,0x5D9,0x5D9,0x5D9},
{0x5DA,Lo,0x5DA,0x5DA,0x5DA},
{0x5DB,Lo,0x5DB,0x5DB,0x5DB},
{0x5DC,Lo,0x5DC,0x5DC,0x5DC},
{0x5DD,Lo,0x5DD,0x5DD,0x5DD},
{0x5DE,Lo,0x5DE,0x5DE,0x5DE},
{0x5DF,Lo,0x5DF,0x5DF,0x5DF},
{0x5E0,Lo,0x5E0,0x5E0,0x5E0},
{0x5E1,Lo,0x5E1,0x5E1,0x5E1},
{0x5E2,Lo,0x5E2,0x5E2,0x5E2},
{0x5E3,Lo,0x5E3,0x5E3,0x5E3},
{0x5E4,Lo,0x5E4,0x5E4,0x5E4},
{0x5E5,Lo,0x5E5,0x5E5,0x5E5},
{0x5E6,Lo,0x5E6,0x5E6,0x5E6},
{0x5E7,Lo,0x5E7,0x5E7,0x5E7},
{0x5E8,Lo,0x5E8,0x5E8,0x5E8},
{0x5E9,Lo,0x5E9,0x5E9,0x5E9},
{0x5EA,Lo,0x5EA,0x5EA,0x5EA},
{0x5F0,Lo,0x5F0,0x5F0,0x5F0},
{0x5F1,Lo,0x5F1,0x5F1,0x5F1},
{0x5F2,Lo,0x5F2,0x5F2,0x5F2},
{0x5F3,Po,0x5F3,0x5F3,0x5F3},
{0x5F4,Po,0x5F4,0x5F4,0x5F4},
{0x600,Cf,0x600,0x600,0x600},
{0x601,Cf,0x601,0x601,0x601},
{0x602,Cf,0x602,0x602,0x602},
{0x603,Cf,0x603,0x603,0x603},
{0x60B,Sc,0x60B,0x60B,0x60B},
{0x60C,Po,0x60C,0x60C,0x60C},
{0x60D,Po,0x60D,0x60D,0x60D},
{0x60E,So,0x60E,0x60E,0x60E},
{0x60F,So,0x60F,0x60F,0x60F},
{0x610,Mn,0x610,0x610,0x610},
{0x611,Mn,0x611,0x611,0x611},
{0x612,Mn,0x612,0x612,0x612},
{0x613,Mn,0x613,0x613,0x613},
{0x614,Mn,0x614,0x614,0x614},
{0x615,Mn,0x615,0x615,0x615},
{0x61B,Po,0x61B,0x61B,0x61B},
{0x61E,Po,0x61E,0x61E,0x61E},
{0x61F,Po,0x61F,0x61F,0x61F},
{0x621,Lo,0x621,0x621,0x621},
{0x622,Lo,0x622,0x622,0x622},
{0x623,Lo,0x623,0x623,0x623},
{0x624,Lo,0x624,0x624,0x624},
{0x625,Lo,0x625,0x625,0x625},
{0x626,Lo,0x626,0x626,0x626},
{0x627,Lo,0x627,0x627,0x627},
{0x628,Lo,0x628,0x628,0x628},
{0x629,Lo,0x629,0x629,0x629},
{0x62A,Lo,0x62A,0x62A,0x62A},
{0x62B,Lo,0x62B,0x62B,0x62B},
{0x62C,Lo,0x62C,0x62C,0x62C},
{0x62D,Lo,0x62D,0x62D,0x62D},
{0x62E,Lo,0x62E,0x62E,0x62E},
{0x62F,Lo,0x62F,0x62F,0x62F},
{0x630,Lo,0x630,0x630,0x630},
{0x631,Lo,0x631,0x631,0x631},
{0x632,Lo,0x632,0x632,0x632},
{0x633,Lo,0x633,0x633,0x633},
{0x634,Lo,0x634,0x634,0x634},
{0x635,Lo,0x635,0x635,0x635},
{0x636,Lo,0x636,0x636,0x636},
{0x637,Lo,0x637,0x637,0x637},
{0x638,Lo,0x638,0x638,0x638},
{0x639,Lo,0x639,0x639,0x639},
{0x63A,Lo,0x63A,0x63A,0x63A},
{0x640,Lm,0x640,0x640,0x640},
{0x641,Lo,0x641,0x641,0x641},
{0x642,Lo,0x642,0x642,0x642},
{0x643,Lo,0x643,0x643,0x643},
{0x644,Lo,0x644,0x644,0x644},
{0x645,Lo,0x645,0x645,0x645},
{0x646,Lo,0x646,0x646,0x646},
{0x647,Lo,0x647,0x647,0x647},
{0x648,Lo,0x648,0x648,0x648},
{0x649,Lo,0x649,0x649,0x649},
{0x64A,Lo,0x64A,0x64A,0x64A},
{0x64B,Mn,0x64B,0x64B,0x64B},
{0x64C,Mn,0x64C,0x64C,0x64C},
{0x64D,Mn,0x64D,0x64D,0x64D},
{0x64E,Mn,0x64E,0x64E,0x64E},
{0x64F,Mn,0x64F,0x64F,0x64F},
{0x650,Mn,0x650,0x650,0x650},
{0x651,Mn,0x651,0x651,0x651},
{0x652,Mn,0x652,0x652,0x652},
{0x653,Mn,0x653,0x653,0x653},
{0x654,Mn,0x654,0x654,0x654},
{0x655,Mn,0x655,0x655,0x655},
{0x656,Mn,0x656,0x656,0x656},
{0x657,Mn,0x657,0x657,0x657},
{0x658,Mn,0x658,0x658,0x658},
{0x659,Mn,0x659,0x659,0x659},
{0x65A,Mn,0x65A,0x65A,0x65A},
{0x65B,Mn,0x65B,0x65B,0x65B},
{0x65C,Mn,0x65C,0x65C,0x65C},
{0x65D,Mn,0x65D,0x65D,0x65D},
{0x65E,Mn,0x65E,0x65E,0x65E},
{0x660,Nd,0x660,0x660,0x660},
{0x661,Nd,0x661,0x661,0x661},
{0x662,Nd,0x662,0x662,0x662},
{0x663,Nd,0x663,0x663,0x663},
{0x664,Nd,0x664,0x664,0x664},
{0x665,Nd,0x665,0x665,0x665},
{0x666,Nd,0x666,0x666,0x666},
{0x667,Nd,0x667,0x667,0x667},
{0x668,Nd,0x668,0x668,0x668},
{0x669,Nd,0x669,0x669,0x669},
{0x66A,Po,0x66A,0x66A,0x66A},
{0x66B,Po,0x66B,0x66B,0x66B},
{0x66C,Po,0x66C,0x66C,0x66C},
{0x66D,Po,0x66D,0x66D,0x66D},
{0x66E,Lo,0x66E,0x66E,0x66E},
{0x66F,Lo,0x66F,0x66F,0x66F},
{0x670,Mn,0x670,0x670,0x670},
{0x671,Lo,0x671,0x671,0x671},
{0x672,Lo,0x672,0x672,0x672},
{0x673,Lo,0x673,0x673,0x673},
{0x674,Lo,0x674,0x674,0x674},
{0x675,Lo,0x675,0x675,0x675},
{0x676,Lo,0x676,0x676,0x676},
{0x677,Lo,0x677,0x677,0x677},
{0x678,Lo,0x678,0x678,0x678},
{0x679,Lo,0x679,0x679,0x679},
{0x67A,Lo,0x67A,0x67A,0x67A},
{0x67B,Lo,0x67B,0x67B,0x67B},
{0x67C,Lo,0x67C,0x67C,0x67C},
{0x67D,Lo,0x67D,0x67D,0x67D},
{0x67E,Lo,0x67E,0x67E,0x67E},
{0x67F,Lo,0x67F,0x67F,0x67F},
{0x680,Lo,0x680,0x680,0x680},
{0x681,Lo,0x681,0x681,0x681},
{0x682,Lo,0x682,0x682,0x682},
{0x683,Lo,0x683,0x683,0x683},
{0x684,Lo,0x684,0x684,0x684},
{0x685,Lo,0x685,0x685,0x685},
{0x686,Lo,0x686,0x686,0x686},
{0x687,Lo,0x687,0x687,0x687},
{0x688,Lo,0x688,0x688,0x688},
{0x689,Lo,0x689,0x689,0x689},
{0x68A,Lo,0x68A,0x68A,0x68A},
{0x68B,Lo,0x68B,0x68B,0x68B},
{0x68C,Lo,0x68C,0x68C,0x68C},
{0x68D,Lo,0x68D,0x68D,0x68D},
{0x68E,Lo,0x68E,0x68E,0x68E},
{0x68F,Lo,0x68F,0x68F,0x68F},
{0x690,Lo,0x690,0x690,0x690},
{0x691,Lo,0x691,0x691,0x691},
{0x692,Lo,0x692,0x692,0x692},
{0x693,Lo,0x693,0x693,0x693},
{0x694,Lo,0x694,0x694,0x694},
{0x695,Lo,0x695,0x695,0x695},
{0x696,Lo,0x696,0x696,0x696},
{0x697,Lo,0x697,0x697,0x697},
{0x698,Lo,0x698,0x698,0x698},
{0x699,Lo,0x699,0x699,0x699},
{0x69A,Lo,0x69A,0x69A,0x69A},
{0x69B,Lo,0x69B,0x69B,0x69B},
{0x69C,Lo,0x69C,0x69C,0x69C},
{0x69D,Lo,0x69D,0x69D,0x69D},
{0x69E,Lo,0x69E,0x69E,0x69E},
{0x69F,Lo,0x69F,0x69F,0x69F},
{0x6A0,Lo,0x6A0,0x6A0,0x6A0},
{0x6A1,Lo,0x6A1,0x6A1,0x6A1},
{0x6A2,Lo,0x6A2,0x6A2,0x6A2},
{0x6A3,Lo,0x6A3,0x6A3,0x6A3},
{0x6A4,Lo,0x6A4,0x6A4,0x6A4},
{0x6A5,Lo,0x6A5,0x6A5,0x6A5},
{0x6A6,Lo,0x6A6,0x6A6,0x6A6},
{0x6A7,Lo,0x6A7,0x6A7,0x6A7},
{0x6A8,Lo,0x6A8,0x6A8,0x6A8},
{0x6A9,Lo,0x6A9,0x6A9,0x6A9},
{0x6AA,Lo,0x6AA,0x6AA,0x6AA},
{0x6AB,Lo,0x6AB,0x6AB,0x6AB},
{0x6AC,Lo,0x6AC,0x6AC,0x6AC},
{0x6AD,Lo,0x6AD,0x6AD,0x6AD},
{0x6AE,Lo,0x6AE,0x6AE,0x6AE},
{0x6AF,Lo,0x6AF,0x6AF,0x6AF},
{0x6B0,Lo,0x6B0,0x6B0,0x6B0},
{0x6B1,Lo,0x6B1,0x6B1,0x6B1},
{0x6B2,Lo,0x6B2,0x6B2,0x6B2},
{0x6B3,Lo,0x6B3,0x6B3,0x6B3},
{0x6B4,Lo,0x6B4,0x6B4,0x6B4},
{0x6B5,Lo,0x6B5,0x6B5,0x6B5},
{0x6B6,Lo,0x6B6,0x6B6,0x6B6},
{0x6B7,Lo,0x6B7,0x6B7,0x6B7},
{0x6B8,Lo,0x6B8,0x6B8,0x6B8},
{0x6B9,Lo,0x6B9,0x6B9,0x6B9},
{0x6BA,Lo,0x6BA,0x6BA,0x6BA},
{0x6BB,Lo,0x6BB,0x6BB,0x6BB},
{0x6BC,Lo,0x6BC,0x6BC,0x6BC},
{0x6BD,Lo,0x6BD,0x6BD,0x6BD},
{0x6BE,Lo,0x6BE,0x6BE,0x6BE},
{0x6BF,Lo,0x6BF,0x6BF,0x6BF},
{0x6C0,Lo,0x6C0,0x6C0,0x6C0},
{0x6C1,Lo,0x6C1,0x6C1,0x6C1},
{0x6C2,Lo,0x6C2,0x6C2,0x6C2},
{0x6C3,Lo,0x6C3,0x6C3,0x6C3},
{0x6C4,Lo,0x6C4,0x6C4,0x6C4},
{0x6C5,Lo,0x6C5,0x6C5,0x6C5},
{0x6C6,Lo,0x6C6,0x6C6,0x6C6},
{0x6C7,Lo,0x6C7,0x6C7,0x6C7},
{0x6C8,Lo,0x6C8,0x6C8,0x6C8},
{0x6C9,Lo,0x6C9,0x6C9,0x6C9},
{0x6CA,Lo,0x6CA,0x6CA,0x6CA},
{0x6CB,Lo,0x6CB,0x6CB,0x6CB},
{0x6CC,Lo,0x6CC,0x6CC,0x6CC},
{0x6CD,Lo,0x6CD,0x6CD,0x6CD},
{0x6CE,Lo,0x6CE,0x6CE,0x6CE},
{0x6CF,Lo,0x6CF,0x6CF,0x6CF},
{0x6D0,Lo,0x6D0,0x6D0,0x6D0},
{0x6D1,Lo,0x6D1,0x6D1,0x6D1},
{0x6D2,Lo,0x6D2,0x6D2,0x6D2},
{0x6D3,Lo,0x6D3,0x6D3,0x6D3},
{0x6D4,Po,0x6D4,0x6D4,0x6D4},
{0x6D5,Lo,0x6D5,0x6D5,0x6D5},
{0x6D6,Mn,0x6D6,0x6D6,0x6D6},
{0x6D7,Mn,0x6D7,0x6D7,0x6D7},
{0x6D8,Mn,0x6D8,0x6D8,0x6D8},
{0x6D9,Mn,0x6D9,0x6D9,0x6D9},
{0x6DA,Mn,0x6DA,0x6DA,0x6DA},
{0x6DB,Mn,0x6DB,0x6DB,0x6DB},
{0x6DC,Mn,0x6DC,0x6DC,0x6DC},
{0x6DD,Cf,0x6DD,0x6DD,0x6DD},
{0x6DE,Me,0x6DE,0x6DE,0x6DE},
{0x6DF,Mn,0x6DF,0x6DF,0x6DF},
{0x6E0,Mn,0x6E0,0x6E0,0x6E0},
{0x6E1,Mn,0x6E1,0x6E1,0x6E1},
{0x6E2,Mn,0x6E2,0x6E2,0x6E2},
{0x6E3,Mn,0x6E3,0x6E3,0x6E3},
{0x6E4,Mn,0x6E4,0x6E4,0x6E4},
{0x6E5,Lm,0x6E5,0x6E5,0x6E5},
{0x6E6,Lm,0x6E6,0x6E6,0x6E6},
{0x6E7,Mn,0x6E7,0x6E7,0x6E7},
{0x6E8,Mn,0x6E8,0x6E8,0x6E8},
{0x6E9,So,0x6E9,0x6E9,0x6E9},
{0x6EA,Mn,0x6EA,0x6EA,0x6EA},
{0x6EB,Mn,0x6EB,0x6EB,0x6EB},
{0x6EC,Mn,0x6EC,0x6EC,0x6EC},
{0x6ED,Mn,0x6ED,0x6ED,0x6ED},
{0x6EE,Lo,0x6EE,0x6EE,0x6EE},
{0x6EF,Lo,0x6EF,0x6EF,0x6EF},
{0x6F0,Nd,0x6F0,0x6F0,0x6F0},
{0x6F1,Nd,0x6F1,0x6F1,0x6F1},
{0x6F2,Nd,0x6F2,0x6F2,0x6F2},
{0x6F3,Nd,0x6F3,0x6F3,0x6F3},
{0x6F4,Nd,0x6F4,0x6F4,0x6F4},
{0x6F5,Nd,0x6F5,0x6F5,0x6F5},
{0x6F6,Nd,0x6F6,0x6F6,0x6F6},
{0x6F7,Nd,0x6F7,0x6F7,0x6F7},
{0x6F8,Nd,0x6F8,0x6F8,0x6F8},
{0x6F9,Nd,0x6F9,0x6F9,0x6F9},
{0x6FA,Lo,0x6FA,0x6FA,0x6FA},
{0x6FB,Lo,0x6FB,0x6FB,0x6FB},
{0x6FC,Lo,0x6FC,0x6FC,0x6FC},
{0x6FD,So,0x6FD,0x6FD,0x6FD},
{0x6FE,So,0x6FE,0x6FE,0x6FE},
{0x6FF,Lo,0x6FF,0x6FF,0x6FF},
{0x700,Po,0x700,0x700,0x700},
{0x701,Po,0x701,0x701,0x701},
{0x702,Po,0x702,0x702,0x702},
{0x703,Po,0x703,0x703,0x703},
{0x704,Po,0x704,0x704,0x704},
{0x705,Po,0x705,0x705,0x705},
{0x706,Po,0x706,0x706,0x706},
{0x707,Po,0x707,0x707,0x707},
{0x708,Po,0x708,0x708,0x708},
{0x709,Po,0x709,0x709,0x709},
{0x70A,Po,0x70A,0x70A,0x70A},
{0x70B,Po,0x70B,0x70B,0x70B},
{0x70C,Po,0x70C,0x70C,0x70C},
{0x70D,Po,0x70D,0x70D,0x70D},
{0x70F,Cf,0x70F,0x70F,0x70F},
{0x710,Lo,0x710,0x710,0x710},
{0x711,Mn,0x711,0x711,0x711},
{0x712,Lo,0x712,0x712,0x712},
{0x713,Lo,0x713,0x713,0x713},
{0x714,Lo,0x714,0x714,0x714},
{0x715,Lo,0x715,0x715,0x715},
{0x716,Lo,0x716,0x716,0x716},
{0x717,Lo,0x717,0x717,0x717},
{0x718,Lo,0x718,0x718,0x718},
{0x719,Lo,0x719,0x719,0x719},
{0x71A,Lo,0x71A,0x71A,0x71A},
{0x71B,Lo,0x71B,0x71B,0x71B},
{0x71C,Lo,0x71C,0x71C,0x71C},
{0x71D,Lo,0x71D,0x71D,0x71D},
{0x71E,Lo,0x71E,0x71E,0x71E},
{0x71F,Lo,0x71F,0x71F,0x71F},
{0x720,Lo,0x720,0x720,0x720},
{0x721,Lo,0x721,0x721,0x721},
{0x722,Lo,0x722,0x722,0x722},
{0x723,Lo,0x723,0x723,0x723},
{0x724,Lo,0x724,0x724,0x724},
{0x725,Lo,0x725,0x725,0x725},
{0x726,Lo,0x726,0x726,0x726},
{0x727,Lo,0x727,0x727,0x727},
{0x728,Lo,0x728,0x728,0x728},
{0x729,Lo,0x729,0x729,0x729},
{0x72A,Lo,0x72A,0x72A,0x72A},
{0x72B,Lo,0x72B,0x72B,0x72B},
{0x72C,Lo,0x72C,0x72C,0x72C},
{0x72D,Lo,0x72D,0x72D,0x72D},
{0x72E,Lo,0x72E,0x72E,0x72E},
{0x72F,Lo,0x72F,0x72F,0x72F},
{0x730,Mn,0x730,0x730,0x730},
{0x731,Mn,0x731,0x731,0x731},
{0x732,Mn,0x732,0x732,0x732},
{0x733,Mn,0x733,0x733,0x733},
{0x734,Mn,0x734,0x734,0x734},
{0x735,Mn,0x735,0x735,0x735},
{0x736,Mn,0x736,0x736,0x736},
{0x737,Mn,0x737,0x737,0x737},
{0x738,Mn,0x738,0x738,0x738},
{0x739,Mn,0x739,0x739,0x739},
{0x73A,Mn,0x73A,0x73A,0x73A},
{0x73B,Mn,0x73B,0x73B,0x73B},
{0x73C,Mn,0x73C,0x73C,0x73C},
{0x73D,Mn,0x73D,0x73D,0x73D},
{0x73E,Mn,0x73E,0x73E,0x73E},
{0x73F,Mn,0x73F,0x73F,0x73F},
{0x740,Mn,0x740,0x740,0x740},
{0x741,Mn,0x741,0x741,0x741},
{0x742,Mn,0x742,0x742,0x742},
{0x743,Mn,0x743,0x743,0x743},
{0x744,Mn,0x744,0x744,0x744},
{0x745,Mn,0x745,0x745,0x745},
{0x746,Mn,0x746,0x746,0x746},
{0x747,Mn,0x747,0x747,0x747},
{0x748,Mn,0x748,0x748,0x748},
{0x749,Mn,0x749,0x749,0x749},
{0x74A,Mn,0x74A,0x74A,0x74A},
{0x74D,Lo,0x74D,0x74D,0x74D},
{0x74E,Lo,0x74E,0x74E,0x74E},
{0x74F,Lo,0x74F,0x74F,0x74F},
{0x750,Lo,0x750,0x750,0x750},
{0x751,Lo,0x751,0x751,0x751},
{0x752,Lo,0x752,0x752,0x752},
{0x753,Lo,0x753,0x753,0x753},
{0x754,Lo,0x754,0x754,0x754},
{0x755,Lo,0x755,0x755,0x755},
{0x756,Lo,0x756,0x756,0x756},
{0x757,Lo,0x757,0x757,0x757},
{0x758,Lo,0x758,0x758,0x758},
{0x759,Lo,0x759,0x759,0x759},
{0x75A,Lo,0x75A,0x75A,0x75A},
{0x75B,Lo,0x75B,0x75B,0x75B},
{0x75C,Lo,0x75C,0x75C,0x75C},
{0x75D,Lo,0x75D,0x75D,0x75D},
{0x75E,Lo,0x75E,0x75E,0x75E},
{0x75F,Lo,0x75F,0x75F,0x75F},
{0x760,Lo,0x760,0x760,0x760},
{0x761,Lo,0x761,0x761,0x761},
{0x762,Lo,0x762,0x762,0x762},
{0x763,Lo,0x763,0x763,0x763},
{0x764,Lo,0x764,0x764,0x764},
{0x765,Lo,0x765,0x765,0x765},
{0x766,Lo,0x766,0x766,0x766},
{0x767,Lo,0x767,0x767,0x767},
{0x768,Lo,0x768,0x768,0x768},
{0x769,Lo,0x769,0x769,0x769},
{0x76A,Lo,0x76A,0x76A,0x76A},
{0x76B,Lo,0x76B,0x76B,0x76B},
{0x76C,Lo,0x76C,0x76C,0x76C},
{0x76D,Lo,0x76D,0x76D,0x76D},
{0x780,Lo,0x780,0x780,0x780},
{0x781,Lo,0x781,0x781,0x781},
{0x782,Lo,0x782,0x782,0x782},
{0x783,Lo,0x783,0x783,0x783},
{0x784,Lo,0x784,0x784,0x784},
{0x785,Lo,0x785,0x785,0x785},
{0x786,Lo,0x786,0x786,0x786},
{0x787,Lo,0x787,0x787,0x787},
{0x788,Lo,0x788,0x788,0x788},
{0x789,Lo,0x789,0x789,0x789},
{0x78A,Lo,0x78A,0x78A,0x78A},
{0x78B,Lo,0x78B,0x78B,0x78B},
{0x78C,Lo,0x78C,0x78C,0x78C},
{0x78D,Lo,0x78D,0x78D,0x78D},
{0x78E,Lo,0x78E,0x78E,0x78E},
{0x78F,Lo,0x78F,0x78F,0x78F},
{0x790,Lo,0x790,0x790,0x790},
{0x791,Lo,0x791,0x791,0x791},
{0x792,Lo,0x792,0x792,0x792},
{0x793,Lo,0x793,0x793,0x793},
{0x794,Lo,0x794,0x794,0x794},
{0x795,Lo,0x795,0x795,0x795},
{0x796,Lo,0x796,0x796,0x796},
{0x797,Lo,0x797,0x797,0x797},
{0x798,Lo,0x798,0x798,0x798},
{0x799,Lo,0x799,0x799,0x799},
{0x79A,Lo,0x79A,0x79A,0x79A},
{0x79B,Lo,0x79B,0x79B,0x79B},
{0x79C,Lo,0x79C,0x79C,0x79C},
{0x79D,Lo,0x79D,0x79D,0x79D},
{0x79E,Lo,0x79E,0x79E,0x79E},
{0x79F,Lo,0x79F,0x79F,0x79F},
{0x7A0,Lo,0x7A0,0x7A0,0x7A0},
{0x7A1,Lo,0x7A1,0x7A1,0x7A1},
{0x7A2,Lo,0x7A2,0x7A2,0x7A2},
{0x7A3,Lo,0x7A3,0x7A3,0x7A3},
{0x7A4,Lo,0x7A4,0x7A4,0x7A4},
{0x7A5,Lo,0x7A5,0x7A5,0x7A5},
{0x7A6,Mn,0x7A6,0x7A6,0x7A6},
{0x7A7,Mn,0x7A7,0x7A7,0x7A7},
{0x7A8,Mn,0x7A8,0x7A8,0x7A8},
{0x7A9,Mn,0x7A9,0x7A9,0x7A9},
{0x7AA,Mn,0x7AA,0x7AA,0x7AA},
{0x7AB,Mn,0x7AB,0x7AB,0x7AB},
{0x7AC,Mn,0x7AC,0x7AC,0x7AC},
{0x7AD,Mn,0x7AD,0x7AD,0x7AD},
{0x7AE,Mn,0x7AE,0x7AE,0x7AE},
{0x7AF,Mn,0x7AF,0x7AF,0x7AF},
{0x7B0,Mn,0x7B0,0x7B0,0x7B0},
{0x7B1,Lo,0x7B1,0x7B1,0x7B1},
{0x7C0,Nd,0x7C0,0x7C0,0x7C0},
{0x7C1,Nd,0x7C1,0x7C1,0x7C1},
{0x7C2,Nd,0x7C2,0x7C2,0x7C2},
{0x7C3,Nd,0x7C3,0x7C3,0x7C3},
{0x7C4,Nd,0x7C4,0x7C4,0x7C4},
{0x7C5,Nd,0x7C5,0x7C5,0x7C5},
{0x7C6,Nd,0x7C6,0x7C6,0x7C6},
{0x7C7,Nd,0x7C7,0x7C7,0x7C7},
{0x7C8,Nd,0x7C8,0x7C8,0x7C8},
{0x7C9,Nd,0x7C9,0x7C9,0x7C9},
{0x7CA,Lo,0x7CA,0x7CA,0x7CA},
{0x7CB,Lo,0x7CB,0x7CB,0x7CB},
{0x7CC,Lo,0x7CC,0x7CC,0x7CC},
{0x7CD,Lo,0x7CD,0x7CD,0x7CD},
{0x7CE,Lo,0x7CE,0x7CE,0x7CE},
{0x7CF,Lo,0x7CF,0x7CF,0x7CF},
{0x7D0,Lo,0x7D0,0x7D0,0x7D0},
{0x7D1,Lo,0x7D1,0x7D1,0x7D1},
{0x7D2,Lo,0x7D2,0x7D2,0x7D2},
{0x7D3,Lo,0x7D3,0x7D3,0x7D3},
{0x7D4,Lo,0x7D4,0x7D4,0x7D4},
{0x7D5,Lo,0x7D5,0x7D5,0x7D5},
{0x7D6,Lo,0x7D6,0x7D6,0x7D6},
{0x7D7,Lo,0x7D7,0x7D7,0x7D7},
{0x7D8,Lo,0x7D8,0x7D8,0x7D8},
{0x7D9,Lo,0x7D9,0x7D9,0x7D9},
{0x7DA,Lo,0x7DA,0x7DA,0x7DA},
{0x7DB,Lo,0x7DB,0x7DB,0x7DB},
{0x7DC,Lo,0x7DC,0x7DC,0x7DC},
{0x7DD,Lo,0x7DD,0x7DD,0x7DD},
{0x7DE,Lo,0x7DE,0x7DE,0x7DE},
{0x7DF,Lo,0x7DF,0x7DF,0x7DF},
{0x7E0,Lo,0x7E0,0x7E0,0x7E0},
{0x7E1,Lo,0x7E1,0x7E1,0x7E1},
{0x7E2,Lo,0x7E2,0x7E2,0x7E2},
{0x7E3,Lo,0x7E3,0x7E3,0x7E3},
{0x7E4,Lo,0x7E4,0x7E4,0x7E4},
{0x7E5,Lo,0x7E5,0x7E5,0x7E5},
{0x7E6,Lo,0x7E6,0x7E6,0x7E6},
{0x7E7,Lo,0x7E7,0x7E7,0x7E7},
{0x7E8,Lo,0x7E8,0x7E8,0x7E8},
{0x7E9,Lo,0x7E9,0x7E9,0x7E9},
{0x7EA,Lo,0x7EA,0x7EA,0x7EA},
{0x7EB,Mn,0x7EB,0x7EB,0x7EB},
{0x7EC,Mn,0x7EC,0x7EC,0x7EC},
{0x7ED,Mn,0x7ED,0x7ED,0x7ED},
{0x7EE,Mn,0x7EE,0x7EE,0x7EE},
{0x7EF,Mn,0x7EF,0x7EF,0x7EF},
{0x7F0,Mn,0x7F0,0x7F0,0x7F0},
{0x7F1,Mn,0x7F1,0x7F1,0x7F1},
{0x7F2,Mn,0x7F2,0x7F2,0x7F2},
{0x7F3,Mn,0x7F3,0x7F3,0x7F3},
{0x7F4,Lm,0x7F4,0x7F4,0x7F4},
{0x7F5,Lm,0x7F5,0x7F5,0x7F5},
{0x7F6,So,0x7F6,0x7F6,0x7F6},
{0x7F7,Po,0x7F7,0x7F7,0x7F7},
{0x7F8,Po,0x7F8,0x7F8,0x7F8},
{0x7F9,Po,0x7F9,0x7F9,0x7F9},
{0x7FA,Lm,0x7FA,0x7FA,0x7FA},
{0x901,Mn,0x901,0x901,0x901},
{0x902,Mn,0x902,0x902,0x902},
{0x903,Mc,0x903,0x903,0x903},
{0x904,Lo,0x904,0x904,0x904},
{0x905,Lo,0x905,0x905,0x905},
{0x906,Lo,0x906,0x906,0x906},
{0x907,Lo,0x907,0x907,0x907},
{0x908,Lo,0x908,0x908,0x908},
{0x909,Lo,0x909,0x909,0x909},
{0x90A,Lo,0x90A,0x90A,0x90A},
{0x90B,Lo,0x90B,0x90B,0x90B},
{0x90C,Lo,0x90C,0x90C,0x90C},
{0x90D,Lo,0x90D,0x90D,0x90D},
{0x90E,Lo,0x90E,0x90E,0x90E},
{0x90F,Lo,0x90F,0x90F,0x90F},
{0x910,Lo,0x910,0x910,0x910},
{0x911,Lo,0x911,0x911,0x911},
{0x912,Lo,0x912,0x912,0x912},
{0x913,Lo,0x913,0x913,0x913},
{0x914,Lo,0x914,0x914,0x914},
{0x915,Lo,0x915,0x915,0x915},
{0x916,Lo,0x916,0x916,0x916},
{0x917,Lo,0x917,0x917,0x917},
{0x918,Lo,0x918,0x918,0x918},
{0x919,Lo,0x919,0x919,0x919},
{0x91A,Lo,0x91A,0x91A,0x91A},
{0x91B,Lo,0x91B,0x91B,0x91B},
{0x91C,Lo,0x91C,0x91C,0x91C},
{0x91D,Lo,0x91D,0x91D,0x91D},
{0x91E,Lo,0x91E,0x91E,0x91E},
{0x91F,Lo,0x91F,0x91F,0x91F},
{0x920,Lo,0x920,0x920,0x920},
{0x921,Lo,0x921,0x921,0x921},
{0x922,Lo,0x922,0x922,0x922},
{0x923,Lo,0x923,0x923,0x923},
{0x924,Lo,0x924,0x924,0x924},
{0x925,Lo,0x925,0x925,0x925},
{0x926,Lo,0x926,0x926,0x926},
{0x927,Lo,0x927,0x927,0x927},
{0x928,Lo,0x928,0x928,0x928},
{0x929,Lo,0x929,0x929,0x929},
{0x92A,Lo,0x92A,0x92A,0x92A},
{0x92B,Lo,0x92B,0x92B,0x92B},
{0x92C,Lo,0x92C,0x92C,0x92C},
{0x92D,Lo,0x92D,0x92D,0x92D},
{0x92E,Lo,0x92E,0x92E,0x92E},
{0x92F,Lo,0x92F,0x92F,0x92F},
{0x930,Lo,0x930,0x930,0x930},
{0x931,Lo,0x931,0x931,0x931},
{0x932,Lo,0x932,0x932,0x932},
{0x933,Lo,0x933,0x933,0x933},
{0x934,Lo,0x934,0x934,0x934},
{0x935,Lo,0x935,0x935,0x935},
{0x936,Lo,0x936,0x936,0x936},
{0x937,Lo,0x937,0x937,0x937},
{0x938,Lo,0x938,0x938,0x938},
{0x939,Lo,0x939,0x939,0x939},
{0x93C,Mn,0x93C,0x93C,0x93C},
{0x93D,Lo,0x93D,0x93D,0x93D},
{0x93E,Mc,0x93E,0x93E,0x93E},
{0x93F,Mc,0x93F,0x93F,0x93F},
{0x940,Mc,0x940,0x940,0x940},
{0x941,Mn,0x941,0x941,0x941},
{0x942,Mn,0x942,0x942,0x942},
{0x943,Mn,0x943,0x943,0x943},
{0x944,Mn,0x944,0x944,0x944},
{0x945,Mn,0x945,0x945,0x945},
{0x946,Mn,0x946,0x946,0x946},
{0x947,Mn,0x947,0x947,0x947},
{0x948,Mn,0x948,0x948,0x948},
{0x949,Mc,0x949,0x949,0x949},
{0x94A,Mc,0x94A,0x94A,0x94A},
{0x94B,Mc,0x94B,0x94B,0x94B},
{0x94C,Mc,0x94C,0x94C,0x94C},
{0x94D,Mn,0x94D,0x94D,0x94D},
{0x950,Lo,0x950,0x950,0x950},
{0x951,Mn,0x951,0x951,0x951},
{0x952,Mn,0x952,0x952,0x952},
{0x953,Mn,0x953,0x953,0x953},
{0x954,Mn,0x954,0x954,0x954},
{0x958,Lo,0x958,0x958,0x958},
{0x959,Lo,0x959,0x959,0x959},
{0x95A,Lo,0x95A,0x95A,0x95A},
{0x95B,Lo,0x95B,0x95B,0x95B},
{0x95C,Lo,0x95C,0x95C,0x95C},
{0x95D,Lo,0x95D,0x95D,0x95D},
{0x95E,Lo,0x95E,0x95E,0x95E},
{0x95F,Lo,0x95F,0x95F,0x95F},
{0x960,Lo,0x960,0x960,0x960},
{0x961,Lo,0x961,0x961,0x961},
{0x962,Mn,0x962,0x962,0x962},
{0x963,Mn,0x963,0x963,0x963},
{0x964,Po,0x964,0x964,0x964},
{0x965,Po,0x965,0x965,0x965},
{0x966,Nd,0x966,0x966,0x966},
{0x967,Nd,0x967,0x967,0x967},
{0x968,Nd,0x968,0x968,0x968},
{0x969,Nd,0x969,0x969,0x969},
{0x96A,Nd,0x96A,0x96A,0x96A},
{0x96B,Nd,0x96B,0x96B,0x96B},
{0x96C,Nd,0x96C,0x96C,0x96C},
{0x96D,Nd,0x96D,0x96D,0x96D},
{0x96E,Nd,0x96E,0x96E,0x96E},
{0x96F,Nd,0x96F,0x96F,0x96F},
{0x970,Po,0x970,0x970,0x970},
{0x97B,Lo,0x97B,0x97B,0x97B},
{0x97C,Lo,0x97C,0x97C,0x97C},
{0x97D,Lo,0x97D,0x97D,0x97D},
{0x97E,Lo,0x97E,0x97E,0x97E},
{0x97F,Lo,0x97F,0x97F,0x97F},
{0x981,Mn,0x981,0x981,0x981},
{0x982,Mc,0x982,0x982,0x982},
{0x983,Mc,0x983,0x983,0x983},
{0x985,Lo,0x985,0x985,0x985},
{0x986,Lo,0x986,0x986,0x986},
{0x987,Lo,0x987,0x987,0x987},
{0x988,Lo,0x988,0x988,0x988},
{0x989,Lo,0x989,0x989,0x989},
{0x98A,Lo,0x98A,0x98A,0x98A},
{0x98B,Lo,0x98B,0x98B,0x98B},
{0x98C,Lo,0x98C,0x98C,0x98C},
{0x98F,Lo,0x98F,0x98F,0x98F},
{0x990,Lo,0x990,0x990,0x990},
{0x993,Lo,0x993,0x993,0x993},
{0x994,Lo,0x994,0x994,0x994},
{0x995,Lo,0x995,0x995,0x995},
{0x996,Lo,0x996,0x996,0x996},
{0x997,Lo,0x997,0x997,0x997},
{0x998,Lo,0x998,0x998,0x998},
{0x999,Lo,0x999,0x999,0x999},
{0x99A,Lo,0x99A,0x99A,0x99A},
{0x99B,Lo,0x99B,0x99B,0x99B},
{0x99C,Lo,0x99C,0x99C,0x99C},
{0x99D,Lo,0x99D,0x99D,0x99D},
{0x99E,Lo,0x99E,0x99E,0x99E},
{0x99F,Lo,0x99F,0x99F,0x99F},
{0x9A0,Lo,0x9A0,0x9A0,0x9A0},
{0x9A1,Lo,0x9A1,0x9A1,0x9A1},
{0x9A2,Lo,0x9A2,0x9A2,0x9A2},
{0x9A3,Lo,0x9A3,0x9A3,0x9A3},
{0x9A4,Lo,0x9A4,0x9A4,0x9A4},
{0x9A5,Lo,0x9A5,0x9A5,0x9A5},
{0x9A6,Lo,0x9A6,0x9A6,0x9A6},
{0x9A7,Lo,0x9A7,0x9A7,0x9A7},
{0x9A8,Lo,0x9A8,0x9A8,0x9A8},
{0x9AA,Lo,0x9AA,0x9AA,0x9AA},
{0x9AB,Lo,0x9AB,0x9AB,0x9AB},
{0x9AC,Lo,0x9AC,0x9AC,0x9AC},
{0x9AD,Lo,0x9AD,0x9AD,0x9AD},
{0x9AE,Lo,0x9AE,0x9AE,0x9AE},
{0x9AF,Lo,0x9AF,0x9AF,0x9AF},
{0x9B0,Lo,0x9B0,0x9B0,0x9B0},
{0x9B2,Lo,0x9B2,0x9B2,0x9B2},
{0x9B6,Lo,0x9B6,0x9B6,0x9B6},
{0x9B7,Lo,0x9B7,0x9B7,0x9B7},
{0x9B8,Lo,0x9B8,0x9B8,0x9B8},
{0x9B9,Lo,0x9B9,0x9B9,0x9B9},
{0x9BC,Mn,0x9BC,0x9BC,0x9BC},
{0x9BD,Lo,0x9BD,0x9BD,0x9BD},
{0x9BE,Mc,0x9BE,0x9BE,0x9BE},
{0x9BF,Mc,0x9BF,0x9BF,0x9BF},
{0x9C0,Mc,0x9C0,0x9C0,0x9C0},
{0x9C1,Mn,0x9C1,0x9C1,0x9C1},
{0x9C2,Mn,0x9C2,0x9C2,0x9C2},
{0x9C3,Mn,0x9C3,0x9C3,0x9C3},
{0x9C4,Mn,0x9C4,0x9C4,0x9C4},
{0x9C7,Mc,0x9C7,0x9C7,0x9C7},
{0x9C8,Mc,0x9C8,0x9C8,0x9C8},
{0x9CB,Mc,0x9CB,0x9CB,0x9CB},
{0x9CC,Mc,0x9CC,0x9CC,0x9CC},
{0x9CD,Mn,0x9CD,0x9CD,0x9CD},
{0x9CE,Lo,0x9CE,0x9CE,0x9CE},
{0x9D7,Mc,0x9D7,0x9D7,0x9D7},
{0x9DC,Lo,0x9DC,0x9DC,0x9DC},
{0x9DD,Lo,0x9DD,0x9DD,0x9DD},
{0x9DF,Lo,0x9DF,0x9DF,0x9DF},
{0x9E0,Lo,0x9E0,0x9E0,0x9E0},
{0x9E1,Lo,0x9E1,0x9E1,0x9E1},
{0x9E2,Mn,0x9E2,0x9E2,0x9E2},
{0x9E3,Mn,0x9E3,0x9E3,0x9E3},
{0x9E6,Nd,0x9E6,0x9E6,0x9E6},
{0x9E7,Nd,0x9E7,0x9E7,0x9E7},
{0x9E8,Nd,0x9E8,0x9E8,0x9E8},
{0x9E9,Nd,0x9E9,0x9E9,0x9E9},
{0x9EA,Nd,0x9EA,0x9EA,0x9EA},
{0x9EB,Nd,0x9EB,0x9EB,0x9EB},
{0x9EC,Nd,0x9EC,0x9EC,0x9EC},
{0x9ED,Nd,0x9ED,0x9ED,0x9ED},
{0x9EE,Nd,0x9EE,0x9EE,0x9EE},
{0x9EF,Nd,0x9EF,0x9EF,0x9EF},
{0x9F0,Lo,0x9F0,0x9F0,0x9F0},
{0x9F1,Lo,0x9F1,0x9F1,0x9F1},
{0x9F2,Sc,0x9F2,0x9F2,0x9F2},
{0x9F3,Sc,0x9F3,0x9F3,0x9F3},
{0x9F4,No,0x9F4,0x9F4,0x9F4},
{0x9F5,No,0x9F5,0x9F5,0x9F5},
{0x9F6,No,0x9F6,0x9F6,0x9F6},
{0x9F7,No,0x9F7,0x9F7,0x9F7},
{0x9F8,No,0x9F8,0x9F8,0x9F8},
{0x9F9,No,0x9F9,0x9F9,0x9F9},
{0x9FA,So,0x9FA,0x9FA,0x9FA},
{0xA01,Mn,0xA01,0xA01,0xA01},
{0xA02,Mn,0xA02,0xA02,0xA02},
{0xA03,Mc,0xA03,0xA03,0xA03},
{0xA05,Lo,0xA05,0xA05,0xA05},
{0xA06,Lo,0xA06,0xA06,0xA06},
{0xA07,Lo,0xA07,0xA07,0xA07},
{0xA08,Lo,0xA08,0xA08,0xA08},
{0xA09,Lo,0xA09,0xA09,0xA09},
{0xA0A,Lo,0xA0A,0xA0A,0xA0A},
{0xA0F,Lo,0xA0F,0xA0F,0xA0F},
{0xA10,Lo,0xA10,0xA10,0xA10},
{0xA13,Lo,0xA13,0xA13,0xA13},
{0xA14,Lo,0xA14,0xA14,0xA14},
{0xA15,Lo,0xA15,0xA15,0xA15},
{0xA16,Lo,0xA16,0xA16,0xA16},
{0xA17,Lo,0xA17,0xA17,0xA17},
{0xA18,Lo,0xA18,0xA18,0xA18},
{0xA19,Lo,0xA19,0xA19,0xA19},
{0xA1A,Lo,0xA1A,0xA1A,0xA1A},
{0xA1B,Lo,0xA1B,0xA1B,0xA1B},
{0xA1C,Lo,0xA1C,0xA1C,0xA1C},
{0xA1D,Lo,0xA1D,0xA1D,0xA1D},
{0xA1E,Lo,0xA1E,0xA1E,0xA1E},
{0xA1F,Lo,0xA1F,0xA1F,0xA1F},
{0xA20,Lo,0xA20,0xA20,0xA20},
{0xA21,Lo,0xA21,0xA21,0xA21},
{0xA22,Lo,0xA22,0xA22,0xA22},
{0xA23,Lo,0xA23,0xA23,0xA23},
{0xA24,Lo,0xA24,0xA24,0xA24},
{0xA25,Lo,0xA25,0xA25,0xA25},
{0xA26,Lo,0xA26,0xA26,0xA26},
{0xA27,Lo,0xA27,0xA27,0xA27},
{0xA28,Lo,0xA28,0xA28,0xA28},
{0xA2A,Lo,0xA2A,0xA2A,0xA2A},
{0xA2B,Lo,0xA2B,0xA2B,0xA2B},
{0xA2C,Lo,0xA2C,0xA2C,0xA2C},
{0xA2D,Lo,0xA2D,0xA2D,0xA2D},
{0xA2E,Lo,0xA2E,0xA2E,0xA2E},
{0xA2F,Lo,0xA2F,0xA2F,0xA2F},
{0xA30,Lo,0xA30,0xA30,0xA30},
{0xA32,Lo,0xA32,0xA32,0xA32},
{0xA33,Lo,0xA33,0xA33,0xA33},
{0xA35,Lo,0xA35,0xA35,0xA35},
{0xA36,Lo,0xA36,0xA36,0xA36},
{0xA38,Lo,0xA38,0xA38,0xA38},
{0xA39,Lo,0xA39,0xA39,0xA39},
{0xA3C,Mn,0xA3C,0xA3C,0xA3C},
{0xA3E,Mc,0xA3E,0xA3E,0xA3E},
{0xA3F,Mc,0xA3F,0xA3F,0xA3F},
{0xA40,Mc,0xA40,0xA40,0xA40},
{0xA41,Mn,0xA41,0xA41,0xA41},
{0xA42,Mn,0xA42,0xA42,0xA42},
{0xA47,Mn,0xA47,0xA47,0xA47},
{0xA48,Mn,0xA48,0xA48,0xA48},
{0xA4B,Mn,0xA4B,0xA4B,0xA4B},
{0xA4C,Mn,0xA4C,0xA4C,0xA4C},
{0xA4D,Mn,0xA4D,0xA4D,0xA4D},
{0xA59,Lo,0xA59,0xA59,0xA59},
{0xA5A,Lo,0xA5A,0xA5A,0xA5A},
{0xA5B,Lo,0xA5B,0xA5B,0xA5B},
{0xA5C,Lo,0xA5C,0xA5C,0xA5C},
{0xA5E,Lo,0xA5E,0xA5E,0xA5E},
{0xA66,Nd,0xA66,0xA66,0xA66},
{0xA67,Nd,0xA67,0xA67,0xA67},
{0xA68,Nd,0xA68,0xA68,0xA68},
{0xA69,Nd,0xA69,0xA69,0xA69},
{0xA6A,Nd,0xA6A,0xA6A,0xA6A},
{0xA6B,Nd,0xA6B,0xA6B,0xA6B},
{0xA6C,Nd,0xA6C,0xA6C,0xA6C},
{0xA6D,Nd,0xA6D,0xA6D,0xA6D},
{0xA6E,Nd,0xA6E,0xA6E,0xA6E},
{0xA6F,Nd,0xA6F,0xA6F,0xA6F},
{0xA70,Mn,0xA70,0xA70,0xA70},
{0xA71,Mn,0xA71,0xA71,0xA71},
{0xA72,Lo,0xA72,0xA72,0xA72},
{0xA73,Lo,0xA73,0xA73,0xA73},
{0xA74,Lo,0xA74,0xA74,0xA74},
{0xA81,Mn,0xA81,0xA81,0xA81},
{0xA82,Mn,0xA82,0xA82,0xA82},
{0xA83,Mc,0xA83,0xA83,0xA83},
{0xA85,Lo,0xA85,0xA85,0xA85},
{0xA86,Lo,0xA86,0xA86,0xA86},
{0xA87,Lo,0xA87,0xA87,0xA87},
{0xA88,Lo,0xA88,0xA88,0xA88},
{0xA89,Lo,0xA89,0xA89,0xA89},
{0xA8A,Lo,0xA8A,0xA8A,0xA8A},
{0xA8B,Lo,0xA8B,0xA8B,0xA8B},
{0xA8C,Lo,0xA8C,0xA8C,0xA8C},
{0xA8D,Lo,0xA8D,0xA8D,0xA8D},
{0xA8F,Lo,0xA8F,0xA8F,0xA8F},
{0xA90,Lo,0xA90,0xA90,0xA90},
{0xA91,Lo,0xA91,0xA91,0xA91},
{0xA93,Lo,0xA93,0xA93,0xA93},
{0xA94,Lo,0xA94,0xA94,0xA94},
{0xA95,Lo,0xA95,0xA95,0xA95},
{0xA96,Lo,0xA96,0xA96,0xA96},
{0xA97,Lo,0xA97,0xA97,0xA97},
{0xA98,Lo,0xA98,0xA98,0xA98},
{0xA99,Lo,0xA99,0xA99,0xA99},
{0xA9A,Lo,0xA9A,0xA9A,0xA9A},
{0xA9B,Lo,0xA9B,0xA9B,0xA9B},
{0xA9C,Lo,0xA9C,0xA9C,0xA9C},
{0xA9D,Lo,0xA9D,0xA9D,0xA9D},
{0xA9E,Lo,0xA9E,0xA9E,0xA9E},
{0xA9F,Lo,0xA9F,0xA9F,0xA9F},
{0xAA0,Lo,0xAA0,0xAA0,0xAA0},
{0xAA1,Lo,0xAA1,0xAA1,0xAA1},
{0xAA2,Lo,0xAA2,0xAA2,0xAA2},
{0xAA3,Lo,0xAA3,0xAA3,0xAA3},
{0xAA4,Lo,0xAA4,0xAA4,0xAA4},
{0xAA5,Lo,0xAA5,0xAA5,0xAA5},
{0xAA6,Lo,0xAA6,0xAA6,0xAA6},
{0xAA7,Lo,0xAA7,0xAA7,0xAA7},
{0xAA8,Lo,0xAA8,0xAA8,0xAA8},
{0xAAA,Lo,0xAAA,0xAAA,0xAAA},
{0xAAB,Lo,0xAAB,0xAAB,0xAAB},
{0xAAC,Lo,0xAAC,0xAAC,0xAAC},
{0xAAD,Lo,0xAAD,0xAAD,0xAAD},
{0xAAE,Lo,0xAAE,0xAAE,0xAAE},
{0xAAF,Lo,0xAAF,0xAAF,0xAAF},
{0xAB0,Lo,0xAB0,0xAB0,0xAB0},
{0xAB2,Lo,0xAB2,0xAB2,0xAB2},
{0xAB3,Lo,0xAB3,0xAB3,0xAB3},
{0xAB5,Lo,0xAB5,0xAB5,0xAB5},
{0xAB6,Lo,0xAB6,0xAB6,0xAB6},
{0xAB7,Lo,0xAB7,0xAB7,0xAB7},
{0xAB8,Lo,0xAB8,0xAB8,0xAB8},
{0xAB9,Lo,0xAB9,0xAB9,0xAB9},
{0xABC,Mn,0xABC,0xABC,0xABC},
{0xABD,Lo,0xABD,0xABD,0xABD},
{0xABE,Mc,0xABE,0xABE,0xABE},
{0xABF,Mc,0xABF,0xABF,0xABF},
{0xAC0,Mc,0xAC0,0xAC0,0xAC0},
{0xAC1,Mn,0xAC1,0xAC1,0xAC1},
{0xAC2,Mn,0xAC2,0xAC2,0xAC2},
{0xAC3,Mn,0xAC3,0xAC3,0xAC3},
{0xAC4,Mn,0xAC4,0xAC4,0xAC4},
{0xAC5,Mn,0xAC5,0xAC5,0xAC5},
{0xAC7,Mn,0xAC7,0xAC7,0xAC7},
{0xAC8,Mn,0xAC8,0xAC8,0xAC8},
{0xAC9,Mc,0xAC9,0xAC9,0xAC9},
{0xACB,Mc,0xACB,0xACB,0xACB},
{0xACC,Mc,0xACC,0xACC,0xACC},
{0xACD,Mn,0xACD,0xACD,0xACD},
{0xAD0,Lo,0xAD0,0xAD0,0xAD0},
{0xAE0,Lo,0xAE0,0xAE0,0xAE0},
{0xAE1,Lo,0xAE1,0xAE1,0xAE1},
{0xAE2,Mn,0xAE2,0xAE2,0xAE2},
{0xAE3,Mn,0xAE3,0xAE3,0xAE3},
{0xAE6,Nd,0xAE6,0xAE6,0xAE6},
{0xAE7,Nd,0xAE7,0xAE7,0xAE7},
{0xAE8,Nd,0xAE8,0xAE8,0xAE8},
{0xAE9,Nd,0xAE9,0xAE9,0xAE9},
{0xAEA,Nd,0xAEA,0xAEA,0xAEA},
{0xAEB,Nd,0xAEB,0xAEB,0xAEB},
{0xAEC,Nd,0xAEC,0xAEC,0xAEC},
{0xAED,Nd,0xAED,0xAED,0xAED},
{0xAEE,Nd,0xAEE,0xAEE,0xAEE},
{0xAEF,Nd,0xAEF,0xAEF,0xAEF},
{0xAF1,Sc,0xAF1,0xAF1,0xAF1},
{0xB01,Mn,0xB01,0xB01,0xB01},
{0xB02,Mc,0xB02,0xB02,0xB02},
{0xB03,Mc,0xB03,0xB03,0xB03},
{0xB05,Lo,0xB05,0xB05,0xB05},
{0xB06,Lo,0xB06,0xB06,0xB06},
{0xB07,Lo,0xB07,0xB07,0xB07},
{0xB08,Lo,0xB08,0xB08,0xB08},
{0xB09,Lo,0xB09,0xB09,0xB09},
{0xB0A,Lo,0xB0A,0xB0A,0xB0A},
{0xB0B,Lo,0xB0B,0xB0B,0xB0B},
{0xB0C,Lo,0xB0C,0xB0C,0xB0C},
{0xB0F,Lo,0xB0F,0xB0F,0xB0F},
{0xB10,Lo,0xB10,0xB10,0xB10},
{0xB13,Lo,0xB13,0xB13,0xB13},
{0xB14,Lo,0xB14,0xB14,0xB14},
{0xB15,Lo,0xB15,0xB15,0xB15},
{0xB16,Lo,0xB16,0xB16,0xB16},
{0xB17,Lo,0xB17,0xB17,0xB17},
{0xB18,Lo,0xB18,0xB18,0xB18},
{0xB19,Lo,0xB19,0xB19,0xB19},
{0xB1A,Lo,0xB1A,0xB1A,0xB1A},
{0xB1B,Lo,0xB1B,0xB1B,0xB1B},
{0xB1C,Lo,0xB1C,0xB1C,0xB1C},
{0xB1D,Lo,0xB1D,0xB1D,0xB1D},
{0xB1E,Lo,0xB1E,0xB1E,0xB1E},
{0xB1F,Lo,0xB1F,0xB1F,0xB1F},
{0xB20,Lo,0xB20,0xB20,0xB20},
{0xB21,Lo,0xB21,0xB21,0xB21},
{0xB22,Lo,0xB22,0xB22,0xB22},
{0xB23,Lo,0xB23,0xB23,0xB23},
{0xB24,Lo,0xB24,0xB24,0xB24},
{0xB25,Lo,0xB25,0xB25,0xB25},
{0xB26,Lo,0xB26,0xB26,0xB26},
{0xB27,Lo,0xB27,0xB27,0xB27},
{0xB28,Lo,0xB28,0xB28,0xB28},
{0xB2A,Lo,0xB2A,0xB2A,0xB2A},
{0xB2B,Lo,0xB2B,0xB2B,0xB2B},
{0xB2C,Lo,0xB2C,0xB2C,0xB2C},
{0xB2D,Lo,0xB2D,0xB2D,0xB2D},
{0xB2E,Lo,0xB2E,0xB2E,0xB2E},
{0xB2F,Lo,0xB2F,0xB2F,0xB2F},
{0xB30,Lo,0xB30,0xB30,0xB30},
{0xB32,Lo,0xB32,0xB32,0xB32},
{0xB33,Lo,0xB33,0xB33,0xB33},
{0xB35,Lo,0xB35,0xB35,0xB35},
{0xB36,Lo,0xB36,0xB36,0xB36},
{0xB37,Lo,0xB37,0xB37,0xB37},
{0xB38,Lo,0xB38,0xB38,0xB38},
{0xB39,Lo,0xB39,0xB39,0xB39},
{0xB3C,Mn,0xB3C,0xB3C,0xB3C},
{0xB3D,Lo,0xB3D,0xB3D,0xB3D},
{0xB3E,Mc,0xB3E,0xB3E,0xB3E},
{0xB3F,Mn,0xB3F,0xB3F,0xB3F},
{0xB40,Mc,0xB40,0xB40,0xB40},
{0xB41,Mn,0xB41,0xB41,0xB41},
{0xB42,Mn,0xB42,0xB42,0xB42},
{0xB43,Mn,0xB43,0xB43,0xB43},
{0xB47,Mc,0xB47,0xB47,0xB47},
{0xB48,Mc,0xB48,0xB48,0xB48},
{0xB4B,Mc,0xB4B,0xB4B,0xB4B},
{0xB4C,Mc,0xB4C,0xB4C,0xB4C},
{0xB4D,Mn,0xB4D,0xB4D,0xB4D},
{0xB56,Mn,0xB56,0xB56,0xB56},
{0xB57,Mc,0xB57,0xB57,0xB57},
{0xB5C,Lo,0xB5C,0xB5C,0xB5C},
{0xB5D,Lo,0xB5D,0xB5D,0xB5D},
{0xB5F,Lo,0xB5F,0xB5F,0xB5F},
{0xB60,Lo,0xB60,0xB60,0xB60},
{0xB61,Lo,0xB61,0xB61,0xB61},
{0xB66,Nd,0xB66,0xB66,0xB66},
{0xB67,Nd,0xB67,0xB67,0xB67},
{0xB68,Nd,0xB68,0xB68,0xB68},
{0xB69,Nd,0xB69,0xB69,0xB69},
{0xB6A,Nd,0xB6A,0xB6A,0xB6A},
{0xB6B,Nd,0xB6B,0xB6B,0xB6B},
{0xB6C,Nd,0xB6C,0xB6C,0xB6C},
{0xB6D,Nd,0xB6D,0xB6D,0xB6D},
{0xB6E,Nd,0xB6E,0xB6E,0xB6E},
{0xB6F,Nd,0xB6F,0xB6F,0xB6F},
{0xB70,So,0xB70,0xB70,0xB70},
{0xB71,Lo,0xB71,0xB71,0xB71},
{0xB82,Mn,0xB82,0xB82,0xB82},
{0xB83,Lo,0xB83,0xB83,0xB83},
{0xB85,Lo,0xB85,0xB85,0xB85},
{0xB86,Lo,0xB86,0xB86,0xB86},
{0xB87,Lo,0xB87,0xB87,0xB87},
{0xB88,Lo,0xB88,0xB88,0xB88},
{0xB89,Lo,0xB89,0xB89,0xB89},
{0xB8A,Lo,0xB8A,0xB8A,0xB8A},
{0xB8E,Lo,0xB8E,0xB8E,0xB8E},
{0xB8F,Lo,0xB8F,0xB8F,0xB8F},
{0xB90,Lo,0xB90,0xB90,0xB90},
{0xB92,Lo,0xB92,0xB92,0xB92},
{0xB93,Lo,0xB93,0xB93,0xB93},
{0xB94,Lo,0xB94,0xB94,0xB94},
{0xB95,Lo,0xB95,0xB95,0xB95},
{0xB99,Lo,0xB99,0xB99,0xB99},
{0xB9A,Lo,0xB9A,0xB9A,0xB9A},
{0xB9C,Lo,0xB9C,0xB9C,0xB9C},
{0xB9E,Lo,0xB9E,0xB9E,0xB9E},
{0xB9F,Lo,0xB9F,0xB9F,0xB9F},
{0xBA3,Lo,0xBA3,0xBA3,0xBA3},
{0xBA4,Lo,0xBA4,0xBA4,0xBA4},
{0xBA8,Lo,0xBA8,0xBA8,0xBA8},
{0xBA9,Lo,0xBA9,0xBA9,0xBA9},
{0xBAA,Lo,0xBAA,0xBAA,0xBAA},
{0xBAE,Lo,0xBAE,0xBAE,0xBAE},
{0xBAF,Lo,0xBAF,0xBAF,0xBAF},
{0xBB0,Lo,0xBB0,0xBB0,0xBB0},
{0xBB1,Lo,0xBB1,0xBB1,0xBB1},
{0xBB2,Lo,0xBB2,0xBB2,0xBB2},
{0xBB3,Lo,0xBB3,0xBB3,0xBB3},
{0xBB4,Lo,0xBB4,0xBB4,0xBB4},
{0xBB5,Lo,0xBB5,0xBB5,0xBB5},
{0xBB6,Lo,0xBB6,0xBB6,0xBB6},
{0xBB7,Lo,0xBB7,0xBB7,0xBB7},
{0xBB8,Lo,0xBB8,0xBB8,0xBB8},
{0xBB9,Lo,0xBB9,0xBB9,0xBB9},
{0xBBE,Mc,0xBBE,0xBBE,0xBBE},
{0xBBF,Mc,0xBBF,0xBBF,0xBBF},
{0xBC0,Mn,0xBC0,0xBC0,0xBC0},
{0xBC1,Mc,0xBC1,0xBC1,0xBC1},
{0xBC2,Mc,0xBC2,0xBC2,0xBC2},
{0xBC6,Mc,0xBC6,0xBC6,0xBC6},
{0xBC7,Mc,0xBC7,0xBC7,0xBC7},
{0xBC8,Mc,0xBC8,0xBC8,0xBC8},
{0xBCA,Mc,0xBCA,0xBCA,0xBCA},
{0xBCB,Mc,0xBCB,0xBCB,0xBCB},
{0xBCC,Mc,0xBCC,0xBCC,0xBCC},
{0xBCD,Mn,0xBCD,0xBCD,0xBCD},
{0xBD7,Mc,0xBD7,0xBD7,0xBD7},
{0xBE6,Nd,0xBE6,0xBE6,0xBE6},
{0xBE7,Nd,0xBE7,0xBE7,0xBE7},
{0xBE8,Nd,0xBE8,0xBE8,0xBE8},
{0xBE9,Nd,0xBE9,0xBE9,0xBE9},
{0xBEA,Nd,0xBEA,0xBEA,0xBEA},
{0xBEB,Nd,0xBEB,0xBEB,0xBEB},
{0xBEC,Nd,0xBEC,0xBEC,0xBEC},
{0xBED,Nd,0xBED,0xBED,0xBED},
{0xBEE,Nd,0xBEE,0xBEE,0xBEE},
{0xBEF,Nd,0xBEF,0xBEF,0xBEF},
{0xBF0,No,0xBF0,0xBF0,0xBF0},
{0xBF1,No,0xBF1,0xBF1,0xBF1},
{0xBF2,No,0xBF2,0xBF2,0xBF2},
{0xBF3,So,0xBF3,0xBF3,0xBF3},
{0xBF4,So,0xBF4,0xBF4,0xBF4},
{0xBF5,So,0xBF5,0xBF5,0xBF5},
{0xBF6,So,0xBF6,0xBF6,0xBF6},
{0xBF7,So,0xBF7,0xBF7,0xBF7},
{0xBF8,So,0xBF8,0xBF8,0xBF8},
{0xBF9,Sc,0xBF9,0xBF9,0xBF9},
{0xBFA,So,0xBFA,0xBFA,0xBFA},
{0xC01,Mc,0xC01,0xC01,0xC01},
{0xC02,Mc,0xC02,0xC02,0xC02},
{0xC03,Mc,0xC03,0xC03,0xC03},
{0xC05,Lo,0xC05,0xC05,0xC05},
{0xC06,Lo,0xC06,0xC06,0xC06},
{0xC07,Lo,0xC07,0xC07,0xC07},
{0xC08,Lo,0xC08,0xC08,0xC08},
{0xC09,Lo,0xC09,0xC09,0xC09},
{0xC0A,Lo,0xC0A,0xC0A,0xC0A},
{0xC0B,Lo,0xC0B,0xC0B,0xC0B},
{0xC0C,Lo,0xC0C,0xC0C,0xC0C},
{0xC0E,Lo,0xC0E,0xC0E,0xC0E},
{0xC0F,Lo,0xC0F,0xC0F,0xC0F},
{0xC10,Lo,0xC10,0xC10,0xC10},
{0xC12,Lo,0xC12,0xC12,0xC12},
{0xC13,Lo,0xC13,0xC13,0xC13},
{0xC14,Lo,0xC14,0xC14,0xC14},
{0xC15,Lo,0xC15,0xC15,0xC15},
{0xC16,Lo,0xC16,0xC16,0xC16},
{0xC17,Lo,0xC17,0xC17,0xC17},
{0xC18,Lo,0xC18,0xC18,0xC18},
{0xC19,Lo,0xC19,0xC19,0xC19},
{0xC1A,Lo,0xC1A,0xC1A,0xC1A},
{0xC1B,Lo,0xC1B,0xC1B,0xC1B},
{0xC1C,Lo,0xC1C,0xC1C,0xC1C},
{0xC1D,Lo,0xC1D,0xC1D,0xC1D},
{0xC1E,Lo,0xC1E,0xC1E,0xC1E},
{0xC1F,Lo,0xC1F,0xC1F,0xC1F},
{0xC20,Lo,0xC20,0xC20,0xC20},
{0xC21,Lo,0xC21,0xC21,0xC21},
{0xC22,Lo,0xC22,0xC22,0xC22},
{0xC23,Lo,0xC23,0xC23,0xC23},
{0xC24,Lo,0xC24,0xC24,0xC24},
{0xC25,Lo,0xC25,0xC25,0xC25},
{0xC26,Lo,0xC26,0xC26,0xC26},
{0xC27,Lo,0xC27,0xC27,0xC27},
{0xC28,Lo,0xC28,0xC28,0xC28},
{0xC2A,Lo,0xC2A,0xC2A,0xC2A},
{0xC2B,Lo,0xC2B,0xC2B,0xC2B},
{0xC2C,Lo,0xC2C,0xC2C,0xC2C},
{0xC2D,Lo,0xC2D,0xC2D,0xC2D},
{0xC2E,Lo,0xC2E,0xC2E,0xC2E},
{0xC2F,Lo,0xC2F,0xC2F,0xC2F},
{0xC30,Lo,0xC30,0xC30,0xC30},
{0xC31,Lo,0xC31,0xC31,0xC31},
{0xC32,Lo,0xC32,0xC32,0xC32},
{0xC33,Lo,0xC33,0xC33,0xC33},
{0xC35,Lo,0xC35,0xC35,0xC35},
{0xC36,Lo,0xC36,0xC36,0xC36},
{0xC37,Lo,0xC37,0xC37,0xC37},
{0xC38,Lo,0xC38,0xC38,0xC38},
{0xC39,Lo,0xC39,0xC39,0xC39},
{0xC3E,Mn,0xC3E,0xC3E,0xC3E},
{0xC3F,Mn,0xC3F,0xC3F,0xC3F},
{0xC40,Mn,0xC40,0xC40,0xC40},
{0xC41,Mc,0xC41,0xC41,0xC41},
{0xC42,Mc,0xC42,0xC42,0xC42},
{0xC43,Mc,0xC43,0xC43,0xC43},
{0xC44,Mc,0xC44,0xC44,0xC44},
{0xC46,Mn,0xC46,0xC46,0xC46},
{0xC47,Mn,0xC47,0xC47,0xC47},
{0xC48,Mn,0xC48,0xC48,0xC48},
{0xC4A,Mn,0xC4A,0xC4A,0xC4A},
{0xC4B,Mn,0xC4B,0xC4B,0xC4B},
{0xC4C,Mn,0xC4C,0xC4C,0xC4C},
{0xC4D,Mn,0xC4D,0xC4D,0xC4D},
{0xC55,Mn,0xC55,0xC55,0xC55},
{0xC56,Mn,0xC56,0xC56,0xC56},
{0xC60,Lo,0xC60,0xC60,0xC60},
{0xC61,Lo,0xC61,0xC61,0xC61},
{0xC66,Nd,0xC66,0xC66,0xC66},
{0xC67,Nd,0xC67,0xC67,0xC67},
{0xC68,Nd,0xC68,0xC68,0xC68},
{0xC69,Nd,0xC69,0xC69,0xC69},
{0xC6A,Nd,0xC6A,0xC6A,0xC6A},
{0xC6B,Nd,0xC6B,0xC6B,0xC6B},
{0xC6C,Nd,0xC6C,0xC6C,0xC6C},
{0xC6D,Nd,0xC6D,0xC6D,0xC6D},
{0xC6E,Nd,0xC6E,0xC6E,0xC6E},
{0xC6F,Nd,0xC6F,0xC6F,0xC6F},
{0xC82,Mc,0xC82,0xC82,0xC82},
{0xC83,Mc,0xC83,0xC83,0xC83},
{0xC85,Lo,0xC85,0xC85,0xC85},
{0xC86,Lo,0xC86,0xC86,0xC86},
{0xC87,Lo,0xC87,0xC87,0xC87},
{0xC88,Lo,0xC88,0xC88,0xC88},
{0xC89,Lo,0xC89,0xC89,0xC89},
{0xC8A,Lo,0xC8A,0xC8A,0xC8A},
{0xC8B,Lo,0xC8B,0xC8B,0xC8B},
{0xC8C,Lo,0xC8C,0xC8C,0xC8C},
{0xC8E,Lo,0xC8E,0xC8E,0xC8E},
{0xC8F,Lo,0xC8F,0xC8F,0xC8F},
{0xC90,Lo,0xC90,0xC90,0xC90},
{0xC92,Lo,0xC92,0xC92,0xC92},
{0xC93,Lo,0xC93,0xC93,0xC93},
{0xC94,Lo,0xC94,0xC94,0xC94},
{0xC95,Lo,0xC95,0xC95,0xC95},
{0xC96,Lo,0xC96,0xC96,0xC96},
{0xC97,Lo,0xC97,0xC97,0xC97},
{0xC98,Lo,0xC98,0xC98,0xC98},
{0xC99,Lo,0xC99,0xC99,0xC99},
{0xC9A,Lo,0xC9A,0xC9A,0xC9A},
{0xC9B,Lo,0xC9B,0xC9B,0xC9B},
{0xC9C,Lo,0xC9C,0xC9C,0xC9C},
{0xC9D,Lo,0xC9D,0xC9D,0xC9D},
{0xC9E,Lo,0xC9E,0xC9E,0xC9E},
{0xC9F,Lo,0xC9F,0xC9F,0xC9F},
{0xCA0,Lo,0xCA0,0xCA0,0xCA0},
{0xCA1,Lo,0xCA1,0xCA1,0xCA1},
{0xCA2,Lo,0xCA2,0xCA2,0xCA2},
{0xCA3,Lo,0xCA3,0xCA3,0xCA3},
{0xCA4,Lo,0xCA4,0xCA4,0xCA4},
{0xCA5,Lo,0xCA5,0xCA5,0xCA5},
{0xCA6,Lo,0xCA6,0xCA6,0xCA6},
{0xCA7,Lo,0xCA7,0xCA7,0xCA7},
{0xCA8,Lo,0xCA8,0xCA8,0xCA8},
{0xCAA,Lo,0xCAA,0xCAA,0xCAA},
{0xCAB,Lo,0xCAB,0xCAB,0xCAB},
{0xCAC,Lo,0xCAC,0xCAC,0xCAC},
{0xCAD,Lo,0xCAD,0xCAD,0xCAD},
{0xCAE,Lo,0xCAE,0xCAE,0xCAE},
{0xCAF,Lo,0xCAF,0xCAF,0xCAF},
{0xCB0,Lo,0xCB0,0xCB0,0xCB0},
{0xCB1,Lo,0xCB1,0xCB1,0xCB1},
{0xCB2,Lo,0xCB2,0xCB2,0xCB2},
{0xCB3,Lo,0xCB3,0xCB3,0xCB3},
{0xCB5,Lo,0xCB5,0xCB5,0xCB5},
{0xCB6,Lo,0xCB6,0xCB6,0xCB6},
{0xCB7,Lo,0xCB7,0xCB7,0xCB7},
{0xCB8,Lo,0xCB8,0xCB8,0xCB8},
{0xCB9,Lo,0xCB9,0xCB9,0xCB9},
{0xCBC,Mn,0xCBC,0xCBC,0xCBC},
{0xCBD,Lo,0xCBD,0xCBD,0xCBD},
{0xCBE,Mc,0xCBE,0xCBE,0xCBE},
{0xCBF,Mn,0xCBF,0xCBF,0xCBF},
{0xCC0,Mc,0xCC0,0xCC0,0xCC0},
{0xCC1,Mc,0xCC1,0xCC1,0xCC1},
{0xCC2,Mc,0xCC2,0xCC2,0xCC2},
{0xCC3,Mc,0xCC3,0xCC3,0xCC3},
{0xCC4,Mc,0xCC4,0xCC4,0xCC4},
{0xCC6,Mn,0xCC6,0xCC6,0xCC6},
{0xCC7,Mc,0xCC7,0xCC7,0xCC7},
{0xCC8,Mc,0xCC8,0xCC8,0xCC8},
{0xCCA,Mc,0xCCA,0xCCA,0xCCA},
{0xCCB,Mc,0xCCB,0xCCB,0xCCB},
{0xCCC,Mn,0xCCC,0xCCC,0xCCC},
{0xCCD,Mn,0xCCD,0xCCD,0xCCD},
{0xCD5,Mc,0xCD5,0xCD5,0xCD5},
{0xCD6,Mc,0xCD6,0xCD6,0xCD6},
{0xCDE,Lo,0xCDE,0xCDE,0xCDE},
{0xCE0,Lo,0xCE0,0xCE0,0xCE0},
{0xCE1,Lo,0xCE1,0xCE1,0xCE1},
{0xCE2,Mn,0xCE2,0xCE2,0xCE2},
{0xCE3,Mn,0xCE3,0xCE3,0xCE3},
{0xCE6,Nd,0xCE6,0xCE6,0xCE6},
{0xCE7,Nd,0xCE7,0xCE7,0xCE7},
{0xCE8,Nd,0xCE8,0xCE8,0xCE8},
{0xCE9,Nd,0xCE9,0xCE9,0xCE9},
{0xCEA,Nd,0xCEA,0xCEA,0xCEA},
{0xCEB,Nd,0xCEB,0xCEB,0xCEB},
{0xCEC,Nd,0xCEC,0xCEC,0xCEC},
{0xCED,Nd,0xCED,0xCED,0xCED},
{0xCEE,Nd,0xCEE,0xCEE,0xCEE},
{0xCEF,Nd,0xCEF,0xCEF,0xCEF},
{0xCF1,So,0xCF1,0xCF1,0xCF1},
{0xCF2,So,0xCF2,0xCF2,0xCF2},
{0xD02,Mc,0xD02,0xD02,0xD02},
{0xD03,Mc,0xD03,0xD03,0xD03},
{0xD05,Lo,0xD05,0xD05,0xD05},
{0xD06,Lo,0xD06,0xD06,0xD06},
{0xD07,Lo,0xD07,0xD07,0xD07},
{0xD08,Lo,0xD08,0xD08,0xD08},
{0xD09,Lo,0xD09,0xD09,0xD09},
{0xD0A,Lo,0xD0A,0xD0A,0xD0A},
{0xD0B,Lo,0xD0B,0xD0B,0xD0B},
{0xD0C,Lo,0xD0C,0xD0C,0xD0C},
{0xD0E,Lo,0xD0E,0xD0E,0xD0E},
{0xD0F,Lo,0xD0F,0xD0F,0xD0F},
{0xD10,Lo,0xD10,0xD10,0xD10},
{0xD12,Lo,0xD12,0xD12,0xD12},
{0xD13,Lo,0xD13,0xD13,0xD13},
{0xD14,Lo,0xD14,0xD14,0xD14},
{0xD15,Lo,0xD15,0xD15,0xD15},
{0xD16,Lo,0xD16,0xD16,0xD16},
{0xD17,Lo,0xD17,0xD17,0xD17},
{0xD18,Lo,0xD18,0xD18,0xD18},
{0xD19,Lo,0xD19,0xD19,0xD19},
{0xD1A,Lo,0xD1A,0xD1A,0xD1A},
{0xD1B,Lo,0xD1B,0xD1B,0xD1B},
{0xD1C,Lo,0xD1C,0xD1C,0xD1C},
{0xD1D,Lo,0xD1D,0xD1D,0xD1D},
{0xD1E,Lo,0xD1E,0xD1E,0xD1E},
{0xD1F,Lo,0xD1F,0xD1F,0xD1F},
{0xD20,Lo,0xD20,0xD20,0xD20},
{0xD21,Lo,0xD21,0xD21,0xD21},
{0xD22,Lo,0xD22,0xD22,0xD22},
{0xD23,Lo,0xD23,0xD23,0xD23},
{0xD24,Lo,0xD24,0xD24,0xD24},
{0xD25,Lo,0xD25,0xD25,0xD25},
{0xD26,Lo,0xD26,0xD26,0xD26},
{0xD27,Lo,0xD27,0xD27,0xD27},
{0xD28,Lo,0xD28,0xD28,0xD28},
{0xD2A,Lo,0xD2A,0xD2A,0xD2A},
{0xD2B,Lo,0xD2B,0xD2B,0xD2B},
{0xD2C,Lo,0xD2C,0xD2C,0xD2C},
{0xD2D,Lo,0xD2D,0xD2D,0xD2D},
{0xD2E,Lo,0xD2E,0xD2E,0xD2E},
{0xD2F,Lo,0xD2F,0xD2F,0xD2F},
{0xD30,Lo,0xD30,0xD30,0xD30},
{0xD31,Lo,0xD31,0xD31,0xD31},
{0xD32,Lo,0xD32,0xD32,0xD32},
{0xD33,Lo,0xD33,0xD33,0xD33},
{0xD34,Lo,0xD34,0xD34,0xD34},
{0xD35,Lo,0xD35,0xD35,0xD35},
{0xD36,Lo,0xD36,0xD36,0xD36},
{0xD37,Lo,0xD37,0xD37,0xD37},
{0xD38,Lo,0xD38,0xD38,0xD38},
{0xD39,Lo,0xD39,0xD39,0xD39},
{0xD3E,Mc,0xD3E,0xD3E,0xD3E},
{0xD3F,Mc,0xD3F,0xD3F,0xD3F},
{0xD40,Mc,0xD40,0xD40,0xD40},
{0xD41,Mn,0xD41,0xD41,0xD41},
{0xD42,Mn,0xD42,0xD42,0xD42},
{0xD43,Mn,0xD43,0xD43,0xD43},
{0xD46,Mc,0xD46,0xD46,0xD46},
{0xD47,Mc,0xD47,0xD47,0xD47},
{0xD48,Mc,0xD48,0xD48,0xD48},
{0xD4A,Mc,0xD4A,0xD4A,0xD4A},
{0xD4B,Mc,0xD4B,0xD4B,0xD4B},
{0xD4C,Mc,0xD4C,0xD4C,0xD4C},
{0xD4D,Mn,0xD4D,0xD4D,0xD4D},
{0xD57,Mc,0xD57,0xD57,0xD57},
{0xD60,Lo,0xD60,0xD60,0xD60},
{0xD61,Lo,0xD61,0xD61,0xD61},
{0xD66,Nd,0xD66,0xD66,0xD66},
{0xD67,Nd,0xD67,0xD67,0xD67},
{0xD68,Nd,0xD68,0xD68,0xD68},
{0xD69,Nd,0xD69,0xD69,0xD69},
{0xD6A,Nd,0xD6A,0xD6A,0xD6A},
{0xD6B,Nd,0xD6B,0xD6B,0xD6B},
{0xD6C,Nd,0xD6C,0xD6C,0xD6C},
{0xD6D,Nd,0xD6D,0xD6D,0xD6D},
{0xD6E,Nd,0xD6E,0xD6E,0xD6E},
{0xD6F,Nd,0xD6F,0xD6F,0xD6F},
{0xD82,Mc,0xD82,0xD82,0xD82},
{0xD83,Mc,0xD83,0xD83,0xD83},
{0xD85,Lo,0xD85,0xD85,0xD85},
{0xD86,Lo,0xD86,0xD86,0xD86},
{0xD87,Lo,0xD87,0xD87,0xD87},
{0xD88,Lo,0xD88,0xD88,0xD88},
{0xD89,Lo,0xD89,0xD89,0xD89},
{0xD8A,Lo,0xD8A,0xD8A,0xD8A},
{0xD8B,Lo,0xD8B,0xD8B,0xD8B},
{0xD8C,Lo,0xD8C,0xD8C,0xD8C},
{0xD8D,Lo,0xD8D,0xD8D,0xD8D},
{0xD8E,Lo,0xD8E,0xD8E,0xD8E},
{0xD8F,Lo,0xD8F,0xD8F,0xD8F},
{0xD90,Lo,0xD90,0xD90,0xD90},
{0xD91,Lo,0xD91,0xD91,0xD91},
{0xD92,Lo,0xD92,0xD92,0xD92},
{0xD93,Lo,0xD93,0xD93,0xD93},
{0xD94,Lo,0xD94,0xD94,0xD94},
{0xD95,Lo,0xD95,0xD95,0xD95},
{0xD96,Lo,0xD96,0xD96,0xD96},
{0xD9A,Lo,0xD9A,0xD9A,0xD9A},
{0xD9B,Lo,0xD9B,0xD9B,0xD9B},
{0xD9C,Lo,0xD9C,0xD9C,0xD9C},
{0xD9D,Lo,0xD9D,0xD9D,0xD9D},
{0xD9E,Lo,0xD9E,0xD9E,0xD9E},
{0xD9F,Lo,0xD9F,0xD9F,0xD9F},
{0xDA0,Lo,0xDA0,0xDA0,0xDA0},
{0xDA1,Lo,0xDA1,0xDA1,0xDA1},
{0xDA2,Lo,0xDA2,0xDA2,0xDA2},
{0xDA3,Lo,0xDA3,0xDA3,0xDA3},
{0xDA4,Lo,0xDA4,0xDA4,0xDA4},
{0xDA5,Lo,0xDA5,0xDA5,0xDA5},
{0xDA6,Lo,0xDA6,0xDA6,0xDA6},
{0xDA7,Lo,0xDA7,0xDA7,0xDA7},
{0xDA8,Lo,0xDA8,0xDA8,0xDA8},
{0xDA9,Lo,0xDA9,0xDA9,0xDA9},
{0xDAA,Lo,0xDAA,0xDAA,0xDAA},
{0xDAB,Lo,0xDAB,0xDAB,0xDAB},
{0xDAC,Lo,0xDAC,0xDAC,0xDAC},
{0xDAD,Lo,0xDAD,0xDAD,0xDAD},
{0xDAE,Lo,0xDAE,0xDAE,0xDAE},
{0xDAF,Lo,0xDAF,0xDAF,0xDAF},
{0xDB0,Lo,0xDB0,0xDB0,0xDB0},
{0xDB1,Lo,0xDB1,0xDB1,0xDB1},
{0xDB3,Lo,0xDB3,0xDB3,0xDB3},
{0xDB4,Lo,0xDB4,0xDB4,0xDB4},
{0xDB5,Lo,0xDB5,0xDB5,0xDB5},
{0xDB6,Lo,0xDB6,0xDB6,0xDB6},
{0xDB7,Lo,0xDB7,0xDB7,0xDB7},
{0xDB8,Lo,0xDB8,0xDB8,0xDB8},
{0xDB9,Lo,0xDB9,0xDB9,0xDB9},
{0xDBA,Lo,0xDBA,0xDBA,0xDBA},
{0xDBB,Lo,0xDBB,0xDBB,0xDBB},
{0xDBD,Lo,0xDBD,0xDBD,0xDBD},
{0xDC0,Lo,0xDC0,0xDC0,0xDC0},
{0xDC1,Lo,0xDC1,0xDC1,0xDC1},
{0xDC2,Lo,0xDC2,0xDC2,0xDC2},
{0xDC3,Lo,0xDC3,0xDC3,0xDC3},
{0xDC4,Lo,0xDC4,0xDC4,0xDC4},
{0xDC5,Lo,0xDC5,0xDC5,0xDC5},
{0xDC6,Lo,0xDC6,0xDC6,0xDC6},
{0xDCA,Mn,0xDCA,0xDCA,0xDCA},
{0xDCF,Mc,0xDCF,0xDCF,0xDCF},
{0xDD0,Mc,0xDD0,0xDD0,0xDD0},
{0xDD1,Mc,0xDD1,0xDD1,0xDD1},
{0xDD2,Mn,0xDD2,0xDD2,0xDD2},
{0xDD3,Mn,0xDD3,0xDD3,0xDD3},
{0xDD4,Mn,0xDD4,0xDD4,0xDD4},
{0xDD6,Mn,0xDD6,0xDD6,0xDD6},
{0xDD8,Mc,0xDD8,0xDD8,0xDD8},
{0xDD9,Mc,0xDD9,0xDD9,0xDD9},
{0xDDA,Mc,0xDDA,0xDDA,0xDDA},
{0xDDB,Mc,0xDDB,0xDDB,0xDDB},
{0xDDC,Mc,0xDDC,0xDDC,0xDDC},
{0xDDD,Mc,0xDDD,0xDDD,0xDDD},
{0xDDE,Mc,0xDDE,0xDDE,0xDDE},
{0xDDF,Mc,0xDDF,0xDDF,0xDDF},
{0xDF2,Mc,0xDF2,0xDF2,0xDF2},
{0xDF3,Mc,0xDF3,0xDF3,0xDF3},
{0xDF4,Po,0xDF4,0xDF4,0xDF4},
{0xE01,Lo,0xE01,0xE01,0xE01},
{0xE02,Lo,0xE02,0xE02,0xE02},
{0xE03,Lo,0xE03,0xE03,0xE03},
{0xE04,Lo,0xE04,0xE04,0xE04},
{0xE05,Lo,0xE05,0xE05,0xE05},
{0xE06,Lo,0xE06,0xE06,0xE06},
{0xE07,Lo,0xE07,0xE07,0xE07},
{0xE08,Lo,0xE08,0xE08,0xE08},
{0xE09,Lo,0xE09,0xE09,0xE09},
{0xE0A,Lo,0xE0A,0xE0A,0xE0A},
{0xE0B,Lo,0xE0B,0xE0B,0xE0B},
{0xE0C,Lo,0xE0C,0xE0C,0xE0C},
{0xE0D,Lo,0xE0D,0xE0D,0xE0D},
{0xE0E,Lo,0xE0E,0xE0E,0xE0E},
{0xE0F,Lo,0xE0F,0xE0F,0xE0F},
{0xE10,Lo,0xE10,0xE10,0xE10},
{0xE11,Lo,0xE11,0xE11,0xE11},
{0xE12,Lo,0xE12,0xE12,0xE12},
{0xE13,Lo,0xE13,0xE13,0xE13},
{0xE14,Lo,0xE14,0xE14,0xE14},
{0xE15,Lo,0xE15,0xE15,0xE15},
{0xE16,Lo,0xE16,0xE16,0xE16},
{0xE17,Lo,0xE17,0xE17,0xE17},
{0xE18,Lo,0xE18,0xE18,0xE18},
{0xE19,Lo,0xE19,0xE19,0xE19},
{0xE1A,Lo,0xE1A,0xE1A,0xE1A},
{0xE1B,Lo,0xE1B,0xE1B,0xE1B},
{0xE1C,Lo,0xE1C,0xE1C,0xE1C},
{0xE1D,Lo,0xE1D,0xE1D,0xE1D},
{0xE1E,Lo,0xE1E,0xE1E,0xE1E},
{0xE1F,Lo,0xE1F,0xE1F,0xE1F},
{0xE20,Lo,0xE20,0xE20,0xE20},
{0xE21,Lo,0xE21,0xE21,0xE21},
{0xE22,Lo,0xE22,0xE22,0xE22},
{0xE23,Lo,0xE23,0xE23,0xE23},
{0xE24,Lo,0xE24,0xE24,0xE24},
{0xE25,Lo,0xE25,0xE25,0xE25},
{0xE26,Lo,0xE26,0xE26,0xE26},
{0xE27,Lo,0xE27,0xE27,0xE27},
{0xE28,Lo,0xE28,0xE28,0xE28},
{0xE29,Lo,0xE29,0xE29,0xE29},
{0xE2A,Lo,0xE2A,0xE2A,0xE2A},
{0xE2B,Lo,0xE2B,0xE2B,0xE2B},
{0xE2C,Lo,0xE2C,0xE2C,0xE2C},
{0xE2D,Lo,0xE2D,0xE2D,0xE2D},
{0xE2E,Lo,0xE2E,0xE2E,0xE2E},
{0xE2F,Lo,0xE2F,0xE2F,0xE2F},
{0xE30,Lo,0xE30,0xE30,0xE30},
{0xE31,Mn,0xE31,0xE31,0xE31},
{0xE32,Lo,0xE32,0xE32,0xE32},
{0xE33,Lo,0xE33,0xE33,0xE33},
{0xE34,Mn,0xE34,0xE34,0xE34},
{0xE35,Mn,0xE35,0xE35,0xE35},
{0xE36,Mn,0xE36,0xE36,0xE36},
{0xE37,Mn,0xE37,0xE37,0xE37},
{0xE38,Mn,0xE38,0xE38,0xE38},
{0xE39,Mn,0xE39,0xE39,0xE39},
{0xE3A,Mn,0xE3A,0xE3A,0xE3A},
{0xE3F,Sc,0xE3F,0xE3F,0xE3F},
{0xE40,Lo,0xE40,0xE40,0xE40},
{0xE41,Lo,0xE41,0xE41,0xE41},
{0xE42,Lo,0xE42,0xE42,0xE42},
{0xE43,Lo,0xE43,0xE43,0xE43},
{0xE44,Lo,0xE44,0xE44,0xE44},
{0xE45,Lo,0xE45,0xE45,0xE45},
{0xE46,Lm,0xE46,0xE46,0xE46},
{0xE47,Mn,0xE47,0xE47,0xE47},
{0xE48,Mn,0xE48,0xE48,0xE48},
{0xE49,Mn,0xE49,0xE49,0xE49},
{0xE4A,Mn,0xE4A,0xE4A,0xE4A},
{0xE4B,Mn,0xE4B,0xE4B,0xE4B},
{0xE4C,Mn,0xE4C,0xE4C,0xE4C},
{0xE4D,Mn,0xE4D,0xE4D,0xE4D},
{0xE4E,Mn,0xE4E,0xE4E,0xE4E},
{0xE4F,Po,0xE4F,0xE4F,0xE4F},
{0xE50,Nd,0xE50,0xE50,0xE50},
{0xE51,Nd,0xE51,0xE51,0xE51},
{0xE52,Nd,0xE52,0xE52,0xE52},
{0xE53,Nd,0xE53,0xE53,0xE53},
{0xE54,Nd,0xE54,0xE54,0xE54},
{0xE55,Nd,0xE55,0xE55,0xE55},
{0xE56,Nd,0xE56,0xE56,0xE56},
{0xE57,Nd,0xE57,0xE57,0xE57},
{0xE58,Nd,0xE58,0xE58,0xE58},
{0xE59,Nd,0xE59,0xE59,0xE59},
{0xE5A,Po,0xE5A,0xE5A,0xE5A},
{0xE5B,Po,0xE5B,0xE5B,0xE5B},
{0xE81,Lo,0xE81,0xE81,0xE81},
{0xE82,Lo,0xE82,0xE82,0xE82},
{0xE84,Lo,0xE84,0xE84,0xE84},
{0xE87,Lo,0xE87,0xE87,0xE87},
{0xE88,Lo,0xE88,0xE88,0xE88},
{0xE8A,Lo,0xE8A,0xE8A,0xE8A},
{0xE8D,Lo,0xE8D,0xE8D,0xE8D},
{0xE94,Lo,0xE94,0xE94,0xE94},
{0xE95,Lo,0xE95,0xE95,0xE95},
{0xE96,Lo,0xE96,0xE96,0xE96},
{0xE97,Lo,0xE97,0xE97,0xE97},
{0xE99,Lo,0xE99,0xE99,0xE99},
{0xE9A,Lo,0xE9A,0xE9A,0xE9A},
{0xE9B,Lo,0xE9B,0xE9B,0xE9B},
{0xE9C,Lo,0xE9C,0xE9C,0xE9C},
{0xE9D,Lo,0xE9D,0xE9D,0xE9D},
{0xE9E,Lo,0xE9E,0xE9E,0xE9E},
{0xE9F,Lo,0xE9F,0xE9F,0xE9F},
{0xEA1,Lo,0xEA1,0xEA1,0xEA1},
{0xEA2,Lo,0xEA2,0xEA2,0xEA2},
{0xEA3,Lo,0xEA3,0xEA3,0xEA3},
{0xEA5,Lo,0xEA5,0xEA5,0xEA5},
{0xEA7,Lo,0xEA7,0xEA7,0xEA7},
{0xEAA,Lo,0xEAA,0xEAA,0xEAA},
{0xEAB,Lo,0xEAB,0xEAB,0xEAB},
{0xEAD,Lo,0xEAD,0xEAD,0xEAD},
{0xEAE,Lo,0xEAE,0xEAE,0xEAE},
{0xEAF,Lo,0xEAF,0xEAF,0xEAF},
{0xEB0,Lo,0xEB0,0xEB0,0xEB0},
{0xEB1,Mn,0xEB1,0xEB1,0xEB1},
{0xEB2,Lo,0xEB2,0xEB2,0xEB2},
{0xEB3,Lo,0xEB3,0xEB3,0xEB3},
{0xEB4,Mn,0xEB4,0xEB4,0xEB4},
{0xEB5,Mn,0xEB5,0xEB5,0xEB5},
{0xEB6,Mn,0xEB6,0xEB6,0xEB6},
{0xEB7,Mn,0xEB7,0xEB7,0xEB7},
{0xEB8,Mn,0xEB8,0xEB8,0xEB8},
{0xEB9,Mn,0xEB9,0xEB9,0xEB9},
{0xEBB,Mn,0xEBB,0xEBB,0xEBB},
{0xEBC,Mn,0xEBC,0xEBC,0xEBC},
{0xEBD,Lo,0xEBD,0xEBD,0xEBD},
{0xEC0,Lo,0xEC0,0xEC0,0xEC0},
{0xEC1,Lo,0xEC1,0xEC1,0xEC1},
{0xEC2,Lo,0xEC2,0xEC2,0xEC2},
{0xEC3,Lo,0xEC3,0xEC3,0xEC3},
{0xEC4,Lo,0xEC4,0xEC4,0xEC4},
{0xEC6,Lm,0xEC6,0xEC6,0xEC6},
{0xEC8,Mn,0xEC8,0xEC8,0xEC8},
{0xEC9,Mn,0xEC9,0xEC9,0xEC9},
{0xECA,Mn,0xECA,0xECA,0xECA},
{0xECB,Mn,0xECB,0xECB,0xECB},
{0xECC,Mn,0xECC,0xECC,0xECC},
{0xECD,Mn,0xECD,0xECD,0xECD},
{0xED0,Nd,0xED0,0xED0,0xED0},
{0xED1,Nd,0xED1,0xED1,0xED1},
{0xED2,Nd,0xED2,0xED2,0xED2},
{0xED3,Nd,0xED3,0xED3,0xED3},
{0xED4,Nd,0xED4,0xED4,0xED4},
{0xED5,Nd,0xED5,0xED5,0xED5},
{0xED6,Nd,0xED6,0xED6,0xED6},
{0xED7,Nd,0xED7,0xED7,0xED7},
{0xED8,Nd,0xED8,0xED8,0xED8},
{0xED9,Nd,0xED9,0xED9,0xED9},
{0xEDC,Lo,0xEDC,0xEDC,0xEDC},
{0xEDD,Lo,0xEDD,0xEDD,0xEDD},
{0xF00,Lo,0xF00,0xF00,0xF00},
{0xF01,So,0xF01,0xF01,0xF01},
{0xF02,So,0xF02,0xF02,0xF02},
{0xF03,So,0xF03,0xF03,0xF03},
{0xF04,Po,0xF04,0xF04,0xF04},
{0xF05,Po,0xF05,0xF05,0xF05},
{0xF06,Po,0xF06,0xF06,0xF06},
{0xF07,Po,0xF07,0xF07,0xF07},
{0xF08,Po,0xF08,0xF08,0xF08},
{0xF09,Po,0xF09,0xF09,0xF09},
{0xF0A,Po,0xF0A,0xF0A,0xF0A},
{0xF0B,Po,0xF0B,0xF0B,0xF0B},
{0xF0C,Po,0xF0C,0xF0C,0xF0C},
{0xF0D,Po,0xF0D,0xF0D,0xF0D},
{0xF0E,Po,0xF0E,0xF0E,0xF0E},
{0xF0F,Po,0xF0F,0xF0F,0xF0F},
{0xF10,Po,0xF10,0xF10,0xF10},
{0xF11,Po,0xF11,0xF11,0xF11},
{0xF12,Po,0xF12,0xF12,0xF12},
{0xF13,So,0xF13,0xF13,0xF13},
{0xF14,So,0xF14,0xF14,0xF14},
{0xF15,So,0xF15,0xF15,0xF15},
{0xF16,So,0xF16,0xF16,0xF16},
{0xF17,So,0xF17,0xF17,0xF17},
{0xF18,Mn,0xF18,0xF18,0xF18},
{0xF19,Mn,0xF19,0xF19,0xF19},
{0xF1A,So,0xF1A,0xF1A,0xF1A},
{0xF1B,So,0xF1B,0xF1B,0xF1B},
{0xF1C,So,0xF1C,0xF1C,0xF1C},
{0xF1D,So,0xF1D,0xF1D,0xF1D},
{0xF1E,So,0xF1E,0xF1E,0xF1E},
{0xF1F,So,0xF1F,0xF1F,0xF1F},
{0xF20,Nd,0xF20,0xF20,0xF20},
{0xF21,Nd,0xF21,0xF21,0xF21},
{0xF22,Nd,0xF22,0xF22,0xF22},
{0xF23,Nd,0xF23,0xF23,0xF23},
{0xF24,Nd,0xF24,0xF24,0xF24},
{0xF25,Nd,0xF25,0xF25,0xF25},
{0xF26,Nd,0xF26,0xF26,0xF26},
{0xF27,Nd,0xF27,0xF27,0xF27},
{0xF28,Nd,0xF28,0xF28,0xF28},
{0xF29,Nd,0xF29,0xF29,0xF29},
{0xF2A,No,0xF2A,0xF2A,0xF2A},
{0xF2B,No,0xF2B,0xF2B,0xF2B},
{0xF2C,No,0xF2C,0xF2C,0xF2C},
{0xF2D,No,0xF2D,0xF2D,0xF2D},
{0xF2E,No,0xF2E,0xF2E,0xF2E},
{0xF2F,No,0xF2F,0xF2F,0xF2F},
{0xF30,No,0xF30,0xF30,0xF30},
{0xF31,No,0xF31,0xF31,0xF31},
{0xF32,No,0xF32,0xF32,0xF32},
{0xF33,No,0xF33,0xF33,0xF33},
{0xF34,So,0xF34,0xF34,0xF34},
{0xF35,Mn,0xF35,0xF35,0xF35},
{0xF36,So,0xF36,0xF36,0xF36},
{0xF37,Mn,0xF37,0xF37,0xF37},
{0xF38,So,0xF38,0xF38,0xF38},
{0xF39,Mn,0xF39,0xF39,0xF39},
{0xF3A,Ps,0xF3A,0xF3A,0xF3A},
{0xF3B,Pe,0xF3B,0xF3B,0xF3B},
{0xF3C,Ps,0xF3C,0xF3C,0xF3C},
{0xF3D,Pe,0xF3D,0xF3D,0xF3D},
{0xF3E,Mc,0xF3E,0xF3E,0xF3E},
{0xF3F,Mc,0xF3F,0xF3F,0xF3F},
{0xF40,Lo,0xF40,0xF40,0xF40},
{0xF41,Lo,0xF41,0xF41,0xF41},
{0xF42,Lo,0xF42,0xF42,0xF42},
{0xF43,Lo,0xF43,0xF43,0xF43},
{0xF44,Lo,0xF44,0xF44,0xF44},
{0xF45,Lo,0xF45,0xF45,0xF45},
{0xF46,Lo,0xF46,0xF46,0xF46},
{0xF47,Lo,0xF47,0xF47,0xF47},
{0xF49,Lo,0xF49,0xF49,0xF49},
{0xF4A,Lo,0xF4A,0xF4A,0xF4A},
{0xF4B,Lo,0xF4B,0xF4B,0xF4B},
{0xF4C,Lo,0xF4C,0xF4C,0xF4C},
{0xF4D,Lo,0xF4D,0xF4D,0xF4D},
{0xF4E,Lo,0xF4E,0xF4E,0xF4E},
{0xF4F,Lo,0xF4F,0xF4F,0xF4F},
{0xF50,Lo,0xF50,0xF50,0xF50},
{0xF51,Lo,0xF51,0xF51,0xF51},
{0xF52,Lo,0xF52,0xF52,0xF52},
{0xF53,Lo,0xF53,0xF53,0xF53},
{0xF54,Lo,0xF54,0xF54,0xF54},
{0xF55,Lo,0xF55,0xF55,0xF55},
{0xF56,Lo,0xF56,0xF56,0xF56},
{0xF57,Lo,0xF57,0xF57,0xF57},
{0xF58,Lo,0xF58,0xF58,0xF58},
{0xF59,Lo,0xF59,0xF59,0xF59},
{0xF5A,Lo,0xF5A,0xF5A,0xF5A},
{0xF5B,Lo,0xF5B,0xF5B,0xF5B},
{0xF5C,Lo,0xF5C,0xF5C,0xF5C},
{0xF5D,Lo,0xF5D,0xF5D,0xF5D},
{0xF5E,Lo,0xF5E,0xF5E,0xF5E},
{0xF5F,Lo,0xF5F,0xF5F,0xF5F},
{0xF60,Lo,0xF60,0xF60,0xF60},
{0xF61,Lo,0xF61,0xF61,0xF61},
{0xF62,Lo,0xF62,0xF62,0xF62},
{0xF63,Lo,0xF63,0xF63,0xF63},
{0xF64,Lo,0xF64,0xF64,0xF64},
{0xF65,Lo,0xF65,0xF65,0xF65},
{0xF66,Lo,0xF66,0xF66,0xF66},
{0xF67,Lo,0xF67,0xF67,0xF67},
{0xF68,Lo,0xF68,0xF68,0xF68},
{0xF69,Lo,0xF69,0xF69,0xF69},
{0xF6A,Lo,0xF6A,0xF6A,0xF6A},
{0xF71,Mn,0xF71,0xF71,0xF71},
{0xF72,Mn,0xF72,0xF72,0xF72},
{0xF73,Mn,0xF73,0xF73,0xF73},
{0xF74,Mn,0xF74,0xF74,0xF74},
{0xF75,Mn,0xF75,0xF75,0xF75},
{0xF76,Mn,0xF76,0xF76,0xF76},
{0xF77,Mn,0xF77,0xF77,0xF77},
{0xF78,Mn,0xF78,0xF78,0xF78},
{0xF79,Mn,0xF79,0xF79,0xF79},
{0xF7A,Mn,0xF7A,0xF7A,0xF7A},
{0xF7B,Mn,0xF7B,0xF7B,0xF7B},
{0xF7C,Mn,0xF7C,0xF7C,0xF7C},
{0xF7D,Mn,0xF7D,0xF7D,0xF7D},
{0xF7E,Mn,0xF7E,0xF7E,0xF7E},
{0xF7F,Mc,0xF7F,0xF7F,0xF7F},
{0xF80,Mn,0xF80,0xF80,0xF80},
{0xF81,Mn,0xF81,0xF81,0xF81},
{0xF82,Mn,0xF82,0xF82,0xF82},
{0xF83,Mn,0xF83,0xF83,0xF83},
{0xF84,Mn,0xF84,0xF84,0xF84},
{0xF85,Po,0xF85,0xF85,0xF85},
{0xF86,Mn,0xF86,0xF86,0xF86},
{0xF87,Mn,0xF87,0xF87,0xF87},
{0xF88,Lo,0xF88,0xF88,0xF88},
{0xF89,Lo,0xF89,0xF89,0xF89},
{0xF8A,Lo,0xF8A,0xF8A,0xF8A},
{0xF8B,Lo,0xF8B,0xF8B,0xF8B},
{0xF90,Mn,0xF90,0xF90,0xF90},
{0xF91,Mn,0xF91,0xF91,0xF91},
{0xF92,Mn,0xF92,0xF92,0xF92},
{0xF93,Mn,0xF93,0xF93,0xF93},
{0xF94,Mn,0xF94,0xF94,0xF94},
{0xF95,Mn,0xF95,0xF95,0xF95},
{0xF96,Mn,0xF96,0xF96,0xF96},
{0xF97,Mn,0xF97,0xF97,0xF97},
{0xF99,Mn,0xF99,0xF99,0xF99},
{0xF9A,Mn,0xF9A,0xF9A,0xF9A},
{0xF9B,Mn,0xF9B,0xF9B,0xF9B},
{0xF9C,Mn,0xF9C,0xF9C,0xF9C},
{0xF9D,Mn,0xF9D,0xF9D,0xF9D},
{0xF9E,Mn,0xF9E,0xF9E,0xF9E},
{0xF9F,Mn,0xF9F,0xF9F,0xF9F},
{0xFA0,Mn,0xFA0,0xFA0,0xFA0},
{0xFA1,Mn,0xFA1,0xFA1,0xFA1},
{0xFA2,Mn,0xFA2,0xFA2,0xFA2},
{0xFA3,Mn,0xFA3,0xFA3,0xFA3},
{0xFA4,Mn,0xFA4,0xFA4,0xFA4},
{0xFA5,Mn,0xFA5,0xFA5,0xFA5},
{0xFA6,Mn,0xFA6,0xFA6,0xFA6},
{0xFA7,Mn,0xFA7,0xFA7,0xFA7},
{0xFA8,Mn,0xFA8,0xFA8,0xFA8},
{0xFA9,Mn,0xFA9,0xFA9,0xFA9},
{0xFAA,Mn,0xFAA,0xFAA,0xFAA},
{0xFAB,Mn,0xFAB,0xFAB,0xFAB},
{0xFAC,Mn,0xFAC,0xFAC,0xFAC},
{0xFAD,Mn,0xFAD,0xFAD,0xFAD},
{0xFAE,Mn,0xFAE,0xFAE,0xFAE},
{0xFAF,Mn,0xFAF,0xFAF,0xFAF},
{0xFB0,Mn,0xFB0,0xFB0,0xFB0},
{0xFB1,Mn,0xFB1,0xFB1,0xFB1},
{0xFB2,Mn,0xFB2,0xFB2,0xFB2},
{0xFB3,Mn,0xFB3,0xFB3,0xFB3},
{0xFB4,Mn,0xFB4,0xFB4,0xFB4},
{0xFB5,Mn,0xFB5,0xFB5,0xFB5},
{0xFB6,Mn,0xFB6,0xFB6,0xFB6},
{0xFB7,Mn,0xFB7,0xFB7,0xFB7},
{0xFB8,Mn,0xFB8,0xFB8,0xFB8},
{0xFB9,Mn,0xFB9,0xFB9,0xFB9},
{0xFBA,Mn,0xFBA,0xFBA,0xFBA},
{0xFBB,Mn,0xFBB,0xFBB,0xFBB},
{0xFBC,Mn,0xFBC,0xFBC,0xFBC},
{0xFBE,So,0xFBE,0xFBE,0xFBE},
{0xFBF,So,0xFBF,0xFBF,0xFBF},
{0xFC0,So,0xFC0,0xFC0,0xFC0},
{0xFC1,So,0xFC1,0xFC1,0xFC1},
{0xFC2,So,0xFC2,0xFC2,0xFC2},
{0xFC3,So,0xFC3,0xFC3,0xFC3},
{0xFC4,So,0xFC4,0xFC4,0xFC4},
{0xFC5,So,0xFC5,0xFC5,0xFC5},
{0xFC6,Mn,0xFC6,0xFC6,0xFC6},
{0xFC7,So,0xFC7,0xFC7,0xFC7},
{0xFC8,So,0xFC8,0xFC8,0xFC8},
{0xFC9,So,0xFC9,0xFC9,0xFC9},
{0xFCA,So,0xFCA,0xFCA,0xFCA},
{0xFCB,So,0xFCB,0xFCB,0xFCB},
{0xFCC,So,0xFCC,0xFCC,0xFCC},
{0xFCF,So,0xFCF,0xFCF,0xFCF},
{0xFD0,Po,0xFD0,0xFD0,0xFD0},
{0xFD1,Po,0xFD1,0xFD1,0xFD1},
{0x1000,Lo,0x1000,0x1000,0x1000},
{0x1001,Lo,0x1001,0x1001,0x1001},
{0x1002,Lo,0x1002,0x1002,0x1002},
{0x1003,Lo,0x1003,0x1003,0x1003},
{0x1004,Lo,0x1004,0x1004,0x1004},
{0x1005,Lo,0x1005,0x1005,0x1005},
{0x1006,Lo,0x1006,0x1006,0x1006},
{0x1007,Lo,0x1007,0x1007,0x1007},
{0x1008,Lo,0x1008,0x1008,0x1008},
{0x1009,Lo,0x1009,0x1009,0x1009},
{0x100A,Lo,0x100A,0x100A,0x100A},
{0x100B,Lo,0x100B,0x100B,0x100B},
{0x100C,Lo,0x100C,0x100C,0x100C},
{0x100D,Lo,0x100D,0x100D,0x100D},
{0x100E,Lo,0x100E,0x100E,0x100E},
{0x100F,Lo,0x100F,0x100F,0x100F},
{0x1010,Lo,0x1010,0x1010,0x1010},
{0x1011,Lo,0x1011,0x1011,0x1011},
{0x1012,Lo,0x1012,0x1012,0x1012},
{0x1013,Lo,0x1013,0x1013,0x1013},
{0x1014,Lo,0x1014,0x1014,0x1014},
{0x1015,Lo,0x1015,0x1015,0x1015},
{0x1016,Lo,0x1016,0x1016,0x1016},
{0x1017,Lo,0x1017,0x1017,0x1017},
{0x1018,Lo,0x1018,0x1018,0x1018},
{0x1019,Lo,0x1019,0x1019,0x1019},
{0x101A,Lo,0x101A,0x101A,0x101A},
{0x101B,Lo,0x101B,0x101B,0x101B},
{0x101C,Lo,0x101C,0x101C,0x101C},
{0x101D,Lo,0x101D,0x101D,0x101D},
{0x101E,Lo,0x101E,0x101E,0x101E},
{0x101F,Lo,0x101F,0x101F,0x101F},
{0x1020,Lo,0x1020,0x1020,0x1020},
{0x1021,Lo,0x1021,0x1021,0x1021},
{0x1023,Lo,0x1023,0x1023,0x1023},
{0x1024,Lo,0x1024,0x1024,0x1024},
{0x1025,Lo,0x1025,0x1025,0x1025},
{0x1026,Lo,0x1026,0x1026,0x1026},
{0x1027,Lo,0x1027,0x1027,0x1027},
{0x1029,Lo,0x1029,0x1029,0x1029},
{0x102A,Lo,0x102A,0x102A,0x102A},
{0x102C,Mc,0x102C,0x102C,0x102C},
{0x102D,Mn,0x102D,0x102D,0x102D},
{0x102E,Mn,0x102E,0x102E,0x102E},
{0x102F,Mn,0x102F,0x102F,0x102F},
{0x1030,Mn,0x1030,0x1030,0x1030},
{0x1031,Mc,0x1031,0x1031,0x1031},
{0x1032,Mn,0x1032,0x1032,0x1032},
{0x1036,Mn,0x1036,0x1036,0x1036},
{0x1037,Mn,0x1037,0x1037,0x1037},
{0x1038,Mc,0x1038,0x1038,0x1038},
{0x1039,Mn,0x1039,0x1039,0x1039},
{0x1040,Nd,0x1040,0x1040,0x1040},
{0x1041,Nd,0x1041,0x1041,0x1041},
{0x1042,Nd,0x1042,0x1042,0x1042},
{0x1043,Nd,0x1043,0x1043,0x1043},
{0x1044,Nd,0x1044,0x1044,0x1044},
{0x1045,Nd,0x1045,0x1045,0x1045},
{0x1046,Nd,0x1046,0x1046,0x1046},
{0x1047,Nd,0x1047,0x1047,0x1047},
{0x1048,Nd,0x1048,0x1048,0x1048},
{0x1049,Nd,0x1049,0x1049,0x1049},
{0x104A,Po,0x104A,0x104A,0x104A},
{0x104B,Po,0x104B,0x104B,0x104B},
{0x104C,Po,0x104C,0x104C,0x104C},
{0x104D,Po,0x104D,0x104D,0x104D},
{0x104E,Po,0x104E,0x104E,0x104E},
{0x104F,Po,0x104F,0x104F,0x104F},
{0x1050,Lo,0x1050,0x1050,0x1050},
{0x1051,Lo,0x1051,0x1051,0x1051},
{0x1052,Lo,0x1052,0x1052,0x1052},
{0x1053,Lo,0x1053,0x1053,0x1053},
{0x1054,Lo,0x1054,0x1054,0x1054},
{0x1055,Lo,0x1055,0x1055,0x1055},
{0x1056,Mc,0x1056,0x1056,0x1056},
{0x1057,Mc,0x1057,0x1057,0x1057},
{0x1058,Mn,0x1058,0x1058,0x1058},
{0x1059,Mn,0x1059,0x1059,0x1059},
{0x10A0,Lu,0x10A0,0x2D00,0x10A0},
{0x10A1,Lu,0x10A1,0x2D01,0x10A1},
{0x10A2,Lu,0x10A2,0x2D02,0x10A2},
{0x10A3,Lu,0x10A3,0x2D03,0x10A3},
{0x10A4,Lu,0x10A4,0x2D04,0x10A4},
{0x10A5,Lu,0x10A5,0x2D05,0x10A5},
{0x10A6,Lu,0x10A6,0x2D06,0x10A6},
{0x10A7,Lu,0x10A7,0x2D07,0x10A7},
{0x10A8,Lu,0x10A8,0x2D08,0x10A8},
{0x10A9,Lu,0x10A9,0x2D09,0x10A9},
{0x10AA,Lu,0x10AA,0x2D0A,0x10AA},
{0x10AB,Lu,0x10AB,0x2D0B,0x10AB},
{0x10AC,Lu,0x10AC,0x2D0C,0x10AC},
{0x10AD,Lu,0x10AD,0x2D0D,0x10AD},
{0x10AE,Lu,0x10AE,0x2D0E,0x10AE},
{0x10AF,Lu,0x10AF,0x2D0F,0x10AF},
{0x10B0,Lu,0x10B0,0x2D10,0x10B0},
{0x10B1,Lu,0x10B1,0x2D11,0x10B1},
{0x10B2,Lu,0x10B2,0x2D12,0x10B2},
{0x10B3,Lu,0x10B3,0x2D13,0x10B3},
{0x10B4,Lu,0x10B4,0x2D14,0x10B4},
{0x10B5,Lu,0x10B5,0x2D15,0x10B5},
{0x10B6,Lu,0x10B6,0x2D16,0x10B6},
{0x10B7,Lu,0x10B7,0x2D17,0x10B7},
{0x10B8,Lu,0x10B8,0x2D18,0x10B8},
{0x10B9,Lu,0x10B9,0x2D19,0x10B9},
{0x10BA,Lu,0x10BA,0x2D1A,0x10BA},
{0x10BB,Lu,0x10BB,0x2D1B,0x10BB},
{0x10BC,Lu,0x10BC,0x2D1C,0x10BC},
{0x10BD,Lu,0x10BD,0x2D1D,0x10BD},
{0x10BE,Lu,0x10BE,0x2D1E,0x10BE},
{0x10BF,Lu,0x10BF,0x2D1F,0x10BF},
{0x10C0,Lu,0x10C0,0x2D20,0x10C0},
{0x10C1,Lu,0x10C1,0x2D21,0x10C1},
{0x10C2,Lu,0x10C2,0x2D22,0x10C2},
{0x10C3,Lu,0x10C3,0x2D23,0x10C3},
{0x10C4,Lu,0x10C4,0x2D24,0x10C4},
{0x10C5,Lu,0x10C5,0x2D25,0x10C5},
{0x10D0,Lo,0x10D0,0x10D0,0x10D0},
{0x10D1,Lo,0x10D1,0x10D1,0x10D1},
{0x10D2,Lo,0x10D2,0x10D2,0x10D2},
{0x10D3,Lo,0x10D3,0x10D3,0x10D3},
{0x10D4,Lo,0x10D4,0x10D4,0x10D4},
{0x10D5,Lo,0x10D5,0x10D5,0x10D5},
{0x10D6,Lo,0x10D6,0x10D6,0x10D6},
{0x10D7,Lo,0x10D7,0x10D7,0x10D7},
{0x10D8,Lo,0x10D8,0x10D8,0x10D8},
{0x10D9,Lo,0x10D9,0x10D9,0x10D9},
{0x10DA,Lo,0x10DA,0x10DA,0x10DA},
{0x10DB,Lo,0x10DB,0x10DB,0x10DB},
{0x10DC,Lo,0x10DC,0x10DC,0x10DC},
{0x10DD,Lo,0x10DD,0x10DD,0x10DD},
{0x10DE,Lo,0x10DE,0x10DE,0x10DE},
{0x10DF,Lo,0x10DF,0x10DF,0x10DF},
{0x10E0,Lo,0x10E0,0x10E0,0x10E0},
{0x10E1,Lo,0x10E1,0x10E1,0x10E1},
{0x10E2,Lo,0x10E2,0x10E2,0x10E2},
{0x10E3,Lo,0x10E3,0x10E3,0x10E3},
{0x10E4,Lo,0x10E4,0x10E4,0x10E4},
{0x10E5,Lo,0x10E5,0x10E5,0x10E5},
{0x10E6,Lo,0x10E6,0x10E6,0x10E6},
{0x10E7,Lo,0x10E7,0x10E7,0x10E7},
{0x10E8,Lo,0x10E8,0x10E8,0x10E8},
{0x10E9,Lo,0x10E9,0x10E9,0x10E9},
{0x10EA,Lo,0x10EA,0x10EA,0x10EA},
{0x10EB,Lo,0x10EB,0x10EB,0x10EB},
{0x10EC,Lo,0x10EC,0x10EC,0x10EC},
{0x10ED,Lo,0x10ED,0x10ED,0x10ED},
{0x10EE,Lo,0x10EE,0x10EE,0x10EE},
{0x10EF,Lo,0x10EF,0x10EF,0x10EF},
{0x10F0,Lo,0x10F0,0x10F0,0x10F0},
{0x10F1,Lo,0x10F1,0x10F1,0x10F1},
{0x10F2,Lo,0x10F2,0x10F2,0x10F2},
{0x10F3,Lo,0x10F3,0x10F3,0x10F3},
{0x10F4,Lo,0x10F4,0x10F4,0x10F4},
{0x10F5,Lo,0x10F5,0x10F5,0x10F5},
{0x10F6,Lo,0x10F6,0x10F6,0x10F6},
{0x10F7,Lo,0x10F7,0x10F7,0x10F7},
{0x10F8,Lo,0x10F8,0x10F8,0x10F8},
{0x10F9,Lo,0x10F9,0x10F9,0x10F9},
{0x10FA,Lo,0x10FA,0x10FA,0x10FA},
{0x10FB,Po,0x10FB,0x10FB,0x10FB},
{0x10FC,Lm,0x10FC,0x10FC,0x10FC},
{0x1100,Lo,0x1100,0x1100,0x1100},
{0x1101,Lo,0x1101,0x1101,0x1101},
{0x1102,Lo,0x1102,0x1102,0x1102},
{0x1103,Lo,0x1103,0x1103,0x1103},
{0x1104,Lo,0x1104,0x1104,0x1104},
{0x1105,Lo,0x1105,0x1105,0x1105},
{0x1106,Lo,0x1106,0x1106,0x1106},
{0x1107,Lo,0x1107,0x1107,0x1107},
{0x1108,Lo,0x1108,0x1108,0x1108},
{0x1109,Lo,0x1109,0x1109,0x1109},
{0x110A,Lo,0x110A,0x110A,0x110A},
{0x110B,Lo,0x110B,0x110B,0x110B},
{0x110C,Lo,0x110C,0x110C,0x110C},
{0x110D,Lo,0x110D,0x110D,0x110D},
{0x110E,Lo,0x110E,0x110E,0x110E},
{0x110F,Lo,0x110F,0x110F,0x110F},
{0x1110,Lo,0x1110,0x1110,0x1110},
{0x1111,Lo,0x1111,0x1111,0x1111},
{0x1112,Lo,0x1112,0x1112,0x1112},
{0x1113,Lo,0x1113,0x1113,0x1113},
{0x1114,Lo,0x1114,0x1114,0x1114},
{0x1115,Lo,0x1115,0x1115,0x1115},
{0x1116,Lo,0x1116,0x1116,0x1116},
{0x1117,Lo,0x1117,0x1117,0x1117},
{0x1118,Lo,0x1118,0x1118,0x1118},
{0x1119,Lo,0x1119,0x1119,0x1119},
{0x111A,Lo,0x111A,0x111A,0x111A},
{0x111B,Lo,0x111B,0x111B,0x111B},
{0x111C,Lo,0x111C,0x111C,0x111C},
{0x111D,Lo,0x111D,0x111D,0x111D},
{0x111E,Lo,0x111E,0x111E,0x111E},
{0x111F,Lo,0x111F,0x111F,0x111F},
{0x1120,Lo,0x1120,0x1120,0x1120},
{0x1121,Lo,0x1121,0x1121,0x1121},
{0x1122,Lo,0x1122,0x1122,0x1122},
{0x1123,Lo,0x1123,0x1123,0x1123},
{0x1124,Lo,0x1124,0x1124,0x1124},
{0x1125,Lo,0x1125,0x1125,0x1125},
{0x1126,Lo,0x1126,0x1126,0x1126},
{0x1127,Lo,0x1127,0x1127,0x1127},
{0x1128,Lo,0x1128,0x1128,0x1128},
{0x1129,Lo,0x1129,0x1129,0x1129},
{0x112A,Lo,0x112A,0x112A,0x112A},
{0x112B,Lo,0x112B,0x112B,0x112B},
{0x112C,Lo,0x112C,0x112C,0x112C},
{0x112D,Lo,0x112D,0x112D,0x112D},
{0x112E,Lo,0x112E,0x112E,0x112E},
{0x112F,Lo,0x112F,0x112F,0x112F},
{0x1130,Lo,0x1130,0x1130,0x1130},
{0x1131,Lo,0x1131,0x1131,0x1131},
{0x1132,Lo,0x1132,0x1132,0x1132},
{0x1133,Lo,0x1133,0x1133,0x1133},
{0x1134,Lo,0x1134,0x1134,0x1134},
{0x1135,Lo,0x1135,0x1135,0x1135},
{0x1136,Lo,0x1136,0x1136,0x1136},
{0x1137,Lo,0x1137,0x1137,0x1137},
{0x1138,Lo,0x1138,0x1138,0x1138},
{0x1139,Lo,0x1139,0x1139,0x1139},
{0x113A,Lo,0x113A,0x113A,0x113A},
{0x113B,Lo,0x113B,0x113B,0x113B},
{0x113C,Lo,0x113C,0x113C,0x113C},
{0x113D,Lo,0x113D,0x113D,0x113D},
{0x113E,Lo,0x113E,0x113E,0x113E},
{0x113F,Lo,0x113F,0x113F,0x113F},
{0x1140,Lo,0x1140,0x1140,0x1140},
{0x1141,Lo,0x1141,0x1141,0x1141},
{0x1142,Lo,0x1142,0x1142,0x1142},
{0x1143,Lo,0x1143,0x1143,0x1143},
{0x1144,Lo,0x1144,0x1144,0x1144},
{0x1145,Lo,0x1145,0x1145,0x1145},
{0x1146,Lo,0x1146,0x1146,0x1146},
{0x1147,Lo,0x1147,0x1147,0x1147},
{0x1148,Lo,0x1148,0x1148,0x1148},
{0x1149,Lo,0x1149,0x1149,0x1149},
{0x114A,Lo,0x114A,0x114A,0x114A},
{0x114B,Lo,0x114B,0x114B,0x114B},
{0x114C,Lo,0x114C,0x114C,0x114C},
{0x114D,Lo,0x114D,0x114D,0x114D},
{0x114E,Lo,0x114E,0x114E,0x114E},
{0x114F,Lo,0x114F,0x114F,0x114F},
{0x1150,Lo,0x1150,0x1150,0x1150},
{0x1151,Lo,0x1151,0x1151,0x1151},
{0x1152,Lo,0x1152,0x1152,0x1152},
{0x1153,Lo,0x1153,0x1153,0x1153},
{0x1154,Lo,0x1154,0x1154,0x1154},
{0x1155,Lo,0x1155,0x1155,0x1155},
{0x1156,Lo,0x1156,0x1156,0x1156},
{0x1157,Lo,0x1157,0x1157,0x1157},
{0x1158,Lo,0x1158,0x1158,0x1158},
{0x1159,Lo,0x1159,0x1159,0x1159},
{0x115F,Lo,0x115F,0x115F,0x115F},
{0x1160,Lo,0x1160,0x1160,0x1160},
{0x1161,Lo,0x1161,0x1161,0x1161},
{0x1162,Lo,0x1162,0x1162,0x1162},
{0x1163,Lo,0x1163,0x1163,0x1163},
{0x1164,Lo,0x1164,0x1164,0x1164},
{0x1165,Lo,0x1165,0x1165,0x1165},
{0x1166,Lo,0x1166,0x1166,0x1166},
{0x1167,Lo,0x1167,0x1167,0x1167},
{0x1168,Lo,0x1168,0x1168,0x1168},
{0x1169,Lo,0x1169,0x1169,0x1169},
{0x116A,Lo,0x116A,0x116A,0x116A},
{0x116B,Lo,0x116B,0x116B,0x116B},
{0x116C,Lo,0x116C,0x116C,0x116C},
{0x116D,Lo,0x116D,0x116D,0x116D},
{0x116E,Lo,0x116E,0x116E,0x116E},
{0x116F,Lo,0x116F,0x116F,0x116F},
{0x1170,Lo,0x1170,0x1170,0x1170},
{0x1171,Lo,0x1171,0x1171,0x1171},
{0x1172,Lo,0x1172,0x1172,0x1172},
{0x1173,Lo,0x1173,0x1173,0x1173},
{0x1174,Lo,0x1174,0x1174,0x1174},
{0x1175,Lo,0x1175,0x1175,0x1175},
{0x1176,Lo,0x1176,0x1176,0x1176},
{0x1177,Lo,0x1177,0x1177,0x1177},
{0x1178,Lo,0x1178,0x1178,0x1178},
{0x1179,Lo,0x1179,0x1179,0x1179},
{0x117A,Lo,0x117A,0x117A,0x117A},
{0x117B,Lo,0x117B,0x117B,0x117B},
{0x117C,Lo,0x117C,0x117C,0x117C},
{0x117D,Lo,0x117D,0x117D,0x117D},
{0x117E,Lo,0x117E,0x117E,0x117E},
{0x117F,Lo,0x117F,0x117F,0x117F},
{0x1180,Lo,0x1180,0x1180,0x1180},
{0x1181,Lo,0x1181,0x1181,0x1181},
{0x1182,Lo,0x1182,0x1182,0x1182},
{0x1183,Lo,0x1183,0x1183,0x1183},
{0x1184,Lo,0x1184,0x1184,0x1184},
{0x1185,Lo,0x1185,0x1185,0x1185},
{0x1186,Lo,0x1186,0x1186,0x1186},
{0x1187,Lo,0x1187,0x1187,0x1187},
{0x1188,Lo,0x1188,0x1188,0x1188},
{0x1189,Lo,0x1189,0x1189,0x1189},
{0x118A,Lo,0x118A,0x118A,0x118A},
{0x118B,Lo,0x118B,0x118B,0x118B},
{0x118C,Lo,0x118C,0x118C,0x118C},
{0x118D,Lo,0x118D,0x118D,0x118D},
{0x118E,Lo,0x118E,0x118E,0x118E},
{0x118F,Lo,0x118F,0x118F,0x118F},
{0x1190,Lo,0x1190,0x1190,0x1190},
{0x1191,Lo,0x1191,0x1191,0x1191},
{0x1192,Lo,0x1192,0x1192,0x1192},
{0x1193,Lo,0x1193,0x1193,0x1193},
{0x1194,Lo,0x1194,0x1194,0x1194},
{0x1195,Lo,0x1195,0x1195,0x1195},
{0x1196,Lo,0x1196,0x1196,0x1196},
{0x1197,Lo,0x1197,0x1197,0x1197},
{0x1198,Lo,0x1198,0x1198,0x1198},
{0x1199,Lo,0x1199,0x1199,0x1199},
{0x119A,Lo,0x119A,0x119A,0x119A},
{0x119B,Lo,0x119B,0x119B,0x119B},
{0x119C,Lo,0x119C,0x119C,0x119C},
{0x119D,Lo,0x119D,0x119D,0x119D},
{0x119E,Lo,0x119E,0x119E,0x119E},
{0x119F,Lo,0x119F,0x119F,0x119F},
{0x11A0,Lo,0x11A0,0x11A0,0x11A0},
{0x11A1,Lo,0x11A1,0x11A1,0x11A1},
{0x11A2,Lo,0x11A2,0x11A2,0x11A2},
{0x11A8,Lo,0x11A8,0x11A8,0x11A8},
{0x11A9,Lo,0x11A9,0x11A9,0x11A9},
{0x11AA,Lo,0x11AA,0x11AA,0x11AA},
{0x11AB,Lo,0x11AB,0x11AB,0x11AB},
{0x11AC,Lo,0x11AC,0x11AC,0x11AC},
{0x11AD,Lo,0x11AD,0x11AD,0x11AD},
{0x11AE,Lo,0x11AE,0x11AE,0x11AE},
{0x11AF,Lo,0x11AF,0x11AF,0x11AF},
{0x11B0,Lo,0x11B0,0x11B0,0x11B0},
{0x11B1,Lo,0x11B1,0x11B1,0x11B1},
{0x11B2,Lo,0x11B2,0x11B2,0x11B2},
{0x11B3,Lo,0x11B3,0x11B3,0x11B3},
{0x11B4,Lo,0x11B4,0x11B4,0x11B4},
{0x11B5,Lo,0x11B5,0x11B5,0x11B5},
{0x11B6,Lo,0x11B6,0x11B6,0x11B6},
{0x11B7,Lo,0x11B7,0x11B7,0x11B7},
{0x11B8,Lo,0x11B8,0x11B8,0x11B8},
{0x11B9,Lo,0x11B9,0x11B9,0x11B9},
{0x11BA,Lo,0x11BA,0x11BA,0x11BA},
{0x11BB,Lo,0x11BB,0x11BB,0x11BB},
{0x11BC,Lo,0x11BC,0x11BC,0x11BC},
{0x11BD,Lo,0x11BD,0x11BD,0x11BD},
{0x11BE,Lo,0x11BE,0x11BE,0x11BE},
{0x11BF,Lo,0x11BF,0x11BF,0x11BF},
{0x11C0,Lo,0x11C0,0x11C0,0x11C0},
{0x11C1,Lo,0x11C1,0x11C1,0x11C1},
{0x11C2,Lo,0x11C2,0x11C2,0x11C2},
{0x11C3,Lo,0x11C3,0x11C3,0x11C3},
{0x11C4,Lo,0x11C4,0x11C4,0x11C4},
{0x11C5,Lo,0x11C5,0x11C5,0x11C5},
{0x11C6,Lo,0x11C6,0x11C6,0x11C6},
{0x11C7,Lo,0x11C7,0x11C7,0x11C7},
{0x11C8,Lo,0x11C8,0x11C8,0x11C8},
{0x11C9,Lo,0x11C9,0x11C9,0x11C9},
{0x11CA,Lo,0x11CA,0x11CA,0x11CA},
{0x11CB,Lo,0x11CB,0x11CB,0x11CB},
{0x11CC,Lo,0x11CC,0x11CC,0x11CC},
{0x11CD,Lo,0x11CD,0x11CD,0x11CD},
{0x11CE,Lo,0x11CE,0x11CE,0x11CE},
{0x11CF,Lo,0x11CF,0x11CF,0x11CF},
{0x11D0,Lo,0x11D0,0x11D0,0x11D0},
{0x11D1,Lo,0x11D1,0x11D1,0x11D1},
{0x11D2,Lo,0x11D2,0x11D2,0x11D2},
{0x11D3,Lo,0x11D3,0x11D3,0x11D3},
{0x11D4,Lo,0x11D4,0x11D4,0x11D4},
{0x11D5,Lo,0x11D5,0x11D5,0x11D5},
{0x11D6,Lo,0x11D6,0x11D6,0x11D6},
{0x11D7,Lo,0x11D7,0x11D7,0x11D7},
{0x11D8,Lo,0x11D8,0x11D8,0x11D8},
{0x11D9,Lo,0x11D9,0x11D9,0x11D9},
{0x11DA,Lo,0x11DA,0x11DA,0x11DA},
{0x11DB,Lo,0x11DB,0x11DB,0x11DB},
{0x11DC,Lo,0x11DC,0x11DC,0x11DC},
{0x11DD,Lo,0x11DD,0x11DD,0x11DD},
{0x11DE,Lo,0x11DE,0x11DE,0x11DE},
{0x11DF,Lo,0x11DF,0x11DF,0x11DF},
{0x11E0,Lo,0x11E0,0x11E0,0x11E0},
{0x11E1,Lo,0x11E1,0x11E1,0x11E1},
{0x11E2,Lo,0x11E2,0x11E2,0x11E2},
{0x11E3,Lo,0x11E3,0x11E3,0x11E3},
{0x11E4,Lo,0x11E4,0x11E4,0x11E4},
{0x11E5,Lo,0x11E5,0x11E5,0x11E5},
{0x11E6,Lo,0x11E6,0x11E6,0x11E6},
{0x11E7,Lo,0x11E7,0x11E7,0x11E7},
{0x11E8,Lo,0x11E8,0x11E8,0x11E8},
{0x11E9,Lo,0x11E9,0x11E9,0x11E9},
{0x11EA,Lo,0x11EA,0x11EA,0x11EA},
{0x11EB,Lo,0x11EB,0x11EB,0x11EB},
{0x11EC,Lo,0x11EC,0x11EC,0x11EC},
{0x11ED,Lo,0x11ED,0x11ED,0x11ED},
{0x11EE,Lo,0x11EE,0x11EE,0x11EE},
{0x11EF,Lo,0x11EF,0x11EF,0x11EF},
{0x11F0,Lo,0x11F0,0x11F0,0x11F0},
{0x11F1,Lo,0x11F1,0x11F1,0x11F1},
{0x11F2,Lo,0x11F2,0x11F2,0x11F2},
{0x11F3,Lo,0x11F3,0x11F3,0x11F3},
{0x11F4,Lo,0x11F4,0x11F4,0x11F4},
{0x11F5,Lo,0x11F5,0x11F5,0x11F5},
{0x11F6,Lo,0x11F6,0x11F6,0x11F6},
{0x11F7,Lo,0x11F7,0x11F7,0x11F7},
{0x11F8,Lo,0x11F8,0x11F8,0x11F8},
{0x11F9,Lo,0x11F9,0x11F9,0x11F9},
{0x1200,Lo,0x1200,0x1200,0x1200},
{0x1201,Lo,0x1201,0x1201,0x1201},
{0x1202,Lo,0x1202,0x1202,0x1202},
{0x1203,Lo,0x1203,0x1203,0x1203},
{0x1204,Lo,0x1204,0x1204,0x1204},
{0x1205,Lo,0x1205,0x1205,0x1205},
{0x1206,Lo,0x1206,0x1206,0x1206},
{0x1207,Lo,0x1207,0x1207,0x1207},
{0x1208,Lo,0x1208,0x1208,0x1208},
{0x1209,Lo,0x1209,0x1209,0x1209},
{0x120A,Lo,0x120A,0x120A,0x120A},
{0x120B,Lo,0x120B,0x120B,0x120B},
{0x120C,Lo,0x120C,0x120C,0x120C},
{0x120D,Lo,0x120D,0x120D,0x120D},
{0x120E,Lo,0x120E,0x120E,0x120E},
{0x120F,Lo,0x120F,0x120F,0x120F},
{0x1210,Lo,0x1210,0x1210,0x1210},
{0x1211,Lo,0x1211,0x1211,0x1211},
{0x1212,Lo,0x1212,0x1212,0x1212},
{0x1213,Lo,0x1213,0x1213,0x1213},
{0x1214,Lo,0x1214,0x1214,0x1214},
{0x1215,Lo,0x1215,0x1215,0x1215},
{0x1216,Lo,0x1216,0x1216,0x1216},
{0x1217,Lo,0x1217,0x1217,0x1217},
{0x1218,Lo,0x1218,0x1218,0x1218},
{0x1219,Lo,0x1219,0x1219,0x1219},
{0x121A,Lo,0x121A,0x121A,0x121A},
{0x121B,Lo,0x121B,0x121B,0x121B},
{0x121C,Lo,0x121C,0x121C,0x121C},
{0x121D,Lo,0x121D,0x121D,0x121D},
{0x121E,Lo,0x121E,0x121E,0x121E},
{0x121F,Lo,0x121F,0x121F,0x121F},
{0x1220,Lo,0x1220,0x1220,0x1220},
{0x1221,Lo,0x1221,0x1221,0x1221},
{0x1222,Lo,0x1222,0x1222,0x1222},
{0x1223,Lo,0x1223,0x1223,0x1223},
{0x1224,Lo,0x1224,0x1224,0x1224},
{0x1225,Lo,0x1225,0x1225,0x1225},
{0x1226,Lo,0x1226,0x1226,0x1226},
{0x1227,Lo,0x1227,0x1227,0x1227},
{0x1228,Lo,0x1228,0x1228,0x1228},
{0x1229,Lo,0x1229,0x1229,0x1229},
{0x122A,Lo,0x122A,0x122A,0x122A},
{0x122B,Lo,0x122B,0x122B,0x122B},
{0x122C,Lo,0x122C,0x122C,0x122C},
{0x122D,Lo,0x122D,0x122D,0x122D},
{0x122E,Lo,0x122E,0x122E,0x122E},
{0x122F,Lo,0x122F,0x122F,0x122F},
{0x1230,Lo,0x1230,0x1230,0x1230},
{0x1231,Lo,0x1231,0x1231,0x1231},
{0x1232,Lo,0x1232,0x1232,0x1232},
{0x1233,Lo,0x1233,0x1233,0x1233},
{0x1234,Lo,0x1234,0x1234,0x1234},
{0x1235,Lo,0x1235,0x1235,0x1235},
{0x1236,Lo,0x1236,0x1236,0x1236},
{0x1237,Lo,0x1237,0x1237,0x1237},
{0x1238,Lo,0x1238,0x1238,0x1238},
{0x1239,Lo,0x1239,0x1239,0x1239},
{0x123A,Lo,0x123A,0x123A,0x123A},
{0x123B,Lo,0x123B,0x123B,0x123B},
{0x123C,Lo,0x123C,0x123C,0x123C},
{0x123D,Lo,0x123D,0x123D,0x123D},
{0x123E,Lo,0x123E,0x123E,0x123E},
{0x123F,Lo,0x123F,0x123F,0x123F},
{0x1240,Lo,0x1240,0x1240,0x1240},
{0x1241,Lo,0x1241,0x1241,0x1241},
{0x1242,Lo,0x1242,0x1242,0x1242},
{0x1243,Lo,0x1243,0x1243,0x1243},
{0x1244,Lo,0x1244,0x1244,0x1244},
{0x1245,Lo,0x1245,0x1245,0x1245},
{0x1246,Lo,0x1246,0x1246,0x1246},
{0x1247,Lo,0x1247,0x1247,0x1247},
{0x1248,Lo,0x1248,0x1248,0x1248},
{0x124A,Lo,0x124A,0x124A,0x124A},
{0x124B,Lo,0x124B,0x124B,0x124B},
{0x124C,Lo,0x124C,0x124C,0x124C},
{0x124D,Lo,0x124D,0x124D,0x124D},
{0x1250,Lo,0x1250,0x1250,0x1250},
{0x1251,Lo,0x1251,0x1251,0x1251},
{0x1252,Lo,0x1252,0x1252,0x1252},
{0x1253,Lo,0x1253,0x1253,0x1253},
{0x1254,Lo,0x1254,0x1254,0x1254},
{0x1255,Lo,0x1255,0x1255,0x1255},
{0x1256,Lo,0x1256,0x1256,0x1256},
{0x1258,Lo,0x1258,0x1258,0x1258},
{0x125A,Lo,0x125A,0x125A,0x125A},
{0x125B,Lo,0x125B,0x125B,0x125B},
{0x125C,Lo,0x125C,0x125C,0x125C},
{0x125D,Lo,0x125D,0x125D,0x125D},
{0x1260,Lo,0x1260,0x1260,0x1260},
{0x1261,Lo,0x1261,0x1261,0x1261},
{0x1262,Lo,0x1262,0x1262,0x1262},
{0x1263,Lo,0x1263,0x1263,0x1263},
{0x1264,Lo,0x1264,0x1264,0x1264},
{0x1265,Lo,0x1265,0x1265,0x1265},
{0x1266,Lo,0x1266,0x1266,0x1266},
{0x1267,Lo,0x1267,0x1267,0x1267},
{0x1268,Lo,0x1268,0x1268,0x1268},
{0x1269,Lo,0x1269,0x1269,0x1269},
{0x126A,Lo,0x126A,0x126A,0x126A},
{0x126B,Lo,0x126B,0x126B,0x126B},
{0x126C,Lo,0x126C,0x126C,0x126C},
{0x126D,Lo,0x126D,0x126D,0x126D},
{0x126E,Lo,0x126E,0x126E,0x126E},
{0x126F,Lo,0x126F,0x126F,0x126F},
{0x1270,Lo,0x1270,0x1270,0x1270},
{0x1271,Lo,0x1271,0x1271,0x1271},
{0x1272,Lo,0x1272,0x1272,0x1272},
{0x1273,Lo,0x1273,0x1273,0x1273},
{0x1274,Lo,0x1274,0x1274,0x1274},
{0x1275,Lo,0x1275,0x1275,0x1275},
{0x1276,Lo,0x1276,0x1276,0x1276},
{0x1277,Lo,0x1277,0x1277,0x1277},
{0x1278,Lo,0x1278,0x1278,0x1278},
{0x1279,Lo,0x1279,0x1279,0x1279},
{0x127A,Lo,0x127A,0x127A,0x127A},
{0x127B,Lo,0x127B,0x127B,0x127B},
{0x127C,Lo,0x127C,0x127C,0x127C},
{0x127D,Lo,0x127D,0x127D,0x127D},
{0x127E,Lo,0x127E,0x127E,0x127E},
{0x127F,Lo,0x127F,0x127F,0x127F},
{0x1280,Lo,0x1280,0x1280,0x1280},
{0x1281,Lo,0x1281,0x1281,0x1281},
{0x1282,Lo,0x1282,0x1282,0x1282},
{0x1283,Lo,0x1283,0x1283,0x1283},
{0x1284,Lo,0x1284,0x1284,0x1284},
{0x1285,Lo,0x1285,0x1285,0x1285},
{0x1286,Lo,0x1286,0x1286,0x1286},
{0x1287,Lo,0x1287,0x1287,0x1287},
{0x1288,Lo,0x1288,0x1288,0x1288},
{0x128A,Lo,0x128A,0x128A,0x128A},
{0x128B,Lo,0x128B,0x128B,0x128B},
{0x128C,Lo,0x128C,0x128C,0x128C},
{0x128D,Lo,0x128D,0x128D,0x128D},
{0x1290,Lo,0x1290,0x1290,0x1290},
{0x1291,Lo,0x1291,0x1291,0x1291},
{0x1292,Lo,0x1292,0x1292,0x1292},
{0x1293,Lo,0x1293,0x1293,0x1293},
{0x1294,Lo,0x1294,0x1294,0x1294},
{0x1295,Lo,0x1295,0x1295,0x1295},
{0x1296,Lo,0x1296,0x1296,0x1296},
{0x1297,Lo,0x1297,0x1297,0x1297},
{0x1298,Lo,0x1298,0x1298,0x1298},
{0x1299,Lo,0x1299,0x1299,0x1299},
{0x129A,Lo,0x129A,0x129A,0x129A},
{0x129B,Lo,0x129B,0x129B,0x129B},
{0x129C,Lo,0x129C,0x129C,0x129C},
{0x129D,Lo,0x129D,0x129D,0x129D},
{0x129E,Lo,0x129E,0x129E,0x129E},
{0x129F,Lo,0x129F,0x129F,0x129F},
{0x12A0,Lo,0x12A0,0x12A0,0x12A0},
{0x12A1,Lo,0x12A1,0x12A1,0x12A1},
{0x12A2,Lo,0x12A2,0x12A2,0x12A2},
{0x12A3,Lo,0x12A3,0x12A3,0x12A3},
{0x12A4,Lo,0x12A4,0x12A4,0x12A4},
{0x12A5,Lo,0x12A5,0x12A5,0x12A5},
{0x12A6,Lo,0x12A6,0x12A6,0x12A6},
{0x12A7,Lo,0x12A7,0x12A7,0x12A7},
{0x12A8,Lo,0x12A8,0x12A8,0x12A8},
{0x12A9,Lo,0x12A9,0x12A9,0x12A9},
{0x12AA,Lo,0x12AA,0x12AA,0x12AA},
{0x12AB,Lo,0x12AB,0x12AB,0x12AB},
{0x12AC,Lo,0x12AC,0x12AC,0x12AC},
{0x12AD,Lo,0x12AD,0x12AD,0x12AD},
{0x12AE,Lo,0x12AE,0x12AE,0x12AE},
{0x12AF,Lo,0x12AF,0x12AF,0x12AF},
{0x12B0,Lo,0x12B0,0x12B0,0x12B0},
{0x12B2,Lo,0x12B2,0x12B2,0x12B2},
{0x12B3,Lo,0x12B3,0x12B3,0x12B3},
{0x12B4,Lo,0x12B4,0x12B4,0x12B4},
{0x12B5,Lo,0x12B5,0x12B5,0x12B5},
{0x12B8,Lo,0x12B8,0x12B8,0x12B8},
{0x12B9,Lo,0x12B9,0x12B9,0x12B9},
{0x12BA,Lo,0x12BA,0x12BA,0x12BA},
{0x12BB,Lo,0x12BB,0x12BB,0x12BB},
{0x12BC,Lo,0x12BC,0x12BC,0x12BC},
{0x12BD,Lo,0x12BD,0x12BD,0x12BD},
{0x12BE,Lo,0x12BE,0x12BE,0x12BE},
{0x12C0,Lo,0x12C0,0x12C0,0x12C0},
{0x12C2,Lo,0x12C2,0x12C2,0x12C2},
{0x12C3,Lo,0x12C3,0x12C3,0x12C3},
{0x12C4,Lo,0x12C4,0x12C4,0x12C4},
{0x12C5,Lo,0x12C5,0x12C5,0x12C5},
{0x12C8,Lo,0x12C8,0x12C8,0x12C8},
{0x12C9,Lo,0x12C9,0x12C9,0x12C9},
{0x12CA,Lo,0x12CA,0x12CA,0x12CA},
{0x12CB,Lo,0x12CB,0x12CB,0x12CB},
{0x12CC,Lo,0x12CC,0x12CC,0x12CC},
{0x12CD,Lo,0x12CD,0x12CD,0x12CD},
{0x12CE,Lo,0x12CE,0x12CE,0x12CE},
{0x12CF,Lo,0x12CF,0x12CF,0x12CF},
{0x12D0,Lo,0x12D0,0x12D0,0x12D0},
{0x12D1,Lo,0x12D1,0x12D1,0x12D1},
{0x12D2,Lo,0x12D2,0x12D2,0x12D2},
{0x12D3,Lo,0x12D3,0x12D3,0x12D3},
{0x12D4,Lo,0x12D4,0x12D4,0x12D4},
{0x12D5,Lo,0x12D5,0x12D5,0x12D5},
{0x12D6,Lo,0x12D6,0x12D6,0x12D6},
{0x12D8,Lo,0x12D8,0x12D8,0x12D8},
{0x12D9,Lo,0x12D9,0x12D9,0x12D9},
{0x12DA,Lo,0x12DA,0x12DA,0x12DA},
{0x12DB,Lo,0x12DB,0x12DB,0x12DB},
{0x12DC,Lo,0x12DC,0x12DC,0x12DC},
{0x12DD,Lo,0x12DD,0x12DD,0x12DD},
{0x12DE,Lo,0x12DE,0x12DE,0x12DE},
{0x12DF,Lo,0x12DF,0x12DF,0x12DF},
{0x12E0,Lo,0x12E0,0x12E0,0x12E0},
{0x12E1,Lo,0x12E1,0x12E1,0x12E1},
{0x12E2,Lo,0x12E2,0x12E2,0x12E2},
{0x12E3,Lo,0x12E3,0x12E3,0x12E3},
{0x12E4,Lo,0x12E4,0x12E4,0x12E4},
{0x12E5,Lo,0x12E5,0x12E5,0x12E5},
{0x12E6,Lo,0x12E6,0x12E6,0x12E6},
{0x12E7,Lo,0x12E7,0x12E7,0x12E7},
{0x12E8,Lo,0x12E8,0x12E8,0x12E8},
{0x12E9,Lo,0x12E9,0x12E9,0x12E9},
{0x12EA,Lo,0x12EA,0x12EA,0x12EA},
{0x12EB,Lo,0x12EB,0x12EB,0x12EB},
{0x12EC,Lo,0x12EC,0x12EC,0x12EC},
{0x12ED,Lo,0x12ED,0x12ED,0x12ED},
{0x12EE,Lo,0x12EE,0x12EE,0x12EE},
{0x12EF,Lo,0x12EF,0x12EF,0x12EF},
{0x12F0,Lo,0x12F0,0x12F0,0x12F0},
{0x12F1,Lo,0x12F1,0x12F1,0x12F1},
{0x12F2,Lo,0x12F2,0x12F2,0x12F2},
{0x12F3,Lo,0x12F3,0x12F3,0x12F3},
{0x12F4,Lo,0x12F4,0x12F4,0x12F4},
{0x12F5,Lo,0x12F5,0x12F5,0x12F5},
{0x12F6,Lo,0x12F6,0x12F6,0x12F6},
{0x12F7,Lo,0x12F7,0x12F7,0x12F7},
{0x12F8,Lo,0x12F8,0x12F8,0x12F8},
{0x12F9,Lo,0x12F9,0x12F9,0x12F9},
{0x12FA,Lo,0x12FA,0x12FA,0x12FA},
{0x12FB,Lo,0x12FB,0x12FB,0x12FB},
{0x12FC,Lo,0x12FC,0x12FC,0x12FC},
{0x12FD,Lo,0x12FD,0x12FD,0x12FD},
{0x12FE,Lo,0x12FE,0x12FE,0x12FE},
{0x12FF,Lo,0x12FF,0x12FF,0x12FF},
{0x1300,Lo,0x1300,0x1300,0x1300},
{0x1301,Lo,0x1301,0x1301,0x1301},
{0x1302,Lo,0x1302,0x1302,0x1302},
{0x1303,Lo,0x1303,0x1303,0x1303},
{0x1304,Lo,0x1304,0x1304,0x1304},
{0x1305,Lo,0x1305,0x1305,0x1305},
{0x1306,Lo,0x1306,0x1306,0x1306},
{0x1307,Lo,0x1307,0x1307,0x1307},
{0x1308,Lo,0x1308,0x1308,0x1308},
{0x1309,Lo,0x1309,0x1309,0x1309},
{0x130A,Lo,0x130A,0x130A,0x130A},
{0x130B,Lo,0x130B,0x130B,0x130B},
{0x130C,Lo,0x130C,0x130C,0x130C},
{0x130D,Lo,0x130D,0x130D,0x130D},
{0x130E,Lo,0x130E,0x130E,0x130E},
{0x130F,Lo,0x130F,0x130F,0x130F},
{0x1310,Lo,0x1310,0x1310,0x1310},
{0x1312,Lo,0x1312,0x1312,0x1312},
{0x1313,Lo,0x1313,0x1313,0x1313},
{0x1314,Lo,0x1314,0x1314,0x1314},
{0x1315,Lo,0x1315,0x1315,0x1315},
{0x1318,Lo,0x1318,0x1318,0x1318},
{0x1319,Lo,0x1319,0x1319,0x1319},
{0x131A,Lo,0x131A,0x131A,0x131A},
{0x131B,Lo,0x131B,0x131B,0x131B},
{0x131C,Lo,0x131C,0x131C,0x131C},
{0x131D,Lo,0x131D,0x131D,0x131D},
{0x131E,Lo,0x131E,0x131E,0x131E},
{0x131F,Lo,0x131F,0x131F,0x131F},
{0x1320,Lo,0x1320,0x1320,0x1320},
{0x1321,Lo,0x1321,0x1321,0x1321},
{0x1322,Lo,0x1322,0x1322,0x1322},
{0x1323,Lo,0x1323,0x1323,0x1323},
{0x1324,Lo,0x1324,0x1324,0x1324},
{0x1325,Lo,0x1325,0x1325,0x1325},
{0x1326,Lo,0x1326,0x1326,0x1326},
{0x1327,Lo,0x1327,0x1327,0x1327},
{0x1328,Lo,0x1328,0x1328,0x1328},
{0x1329,Lo,0x1329,0x1329,0x1329},
{0x132A,Lo,0x132A,0x132A,0x132A},
{0x132B,Lo,0x132B,0x132B,0x132B},
{0x132C,Lo,0x132C,0x132C,0x132C},
{0x132D,Lo,0x132D,0x132D,0x132D},
{0x132E,Lo,0x132E,0x132E,0x132E},
{0x132F,Lo,0x132F,0x132F,0x132F},
{0x1330,Lo,0x1330,0x1330,0x1330},
{0x1331,Lo,0x1331,0x1331,0x1331},
{0x1332,Lo,0x1332,0x1332,0x1332},
{0x1333,Lo,0x1333,0x1333,0x1333},
{0x1334,Lo,0x1334,0x1334,0x1334},
{0x1335,Lo,0x1335,0x1335,0x1335},
{0x1336,Lo,0x1336,0x1336,0x1336},
{0x1337,Lo,0x1337,0x1337,0x1337},
{0x1338,Lo,0x1338,0x1338,0x1338},
{0x1339,Lo,0x1339,0x1339,0x1339},
{0x133A,Lo,0x133A,0x133A,0x133A},
{0x133B,Lo,0x133B,0x133B,0x133B},
{0x133C,Lo,0x133C,0x133C,0x133C},
{0x133D,Lo,0x133D,0x133D,0x133D},
{0x133E,Lo,0x133E,0x133E,0x133E},
{0x133F,Lo,0x133F,0x133F,0x133F},
{0x1340,Lo,0x1340,0x1340,0x1340},
{0x1341,Lo,0x1341,0x1341,0x1341},
{0x1342,Lo,0x1342,0x1342,0x1342},
{0x1343,Lo,0x1343,0x1343,0x1343},
{0x1344,Lo,0x1344,0x1344,0x1344},
{0x1345,Lo,0x1345,0x1345,0x1345},
{0x1346,Lo,0x1346,0x1346,0x1346},
{0x1347,Lo,0x1347,0x1347,0x1347},
{0x1348,Lo,0x1348,0x1348,0x1348},
{0x1349,Lo,0x1349,0x1349,0x1349},
{0x134A,Lo,0x134A,0x134A,0x134A},
{0x134B,Lo,0x134B,0x134B,0x134B},
{0x134C,Lo,0x134C,0x134C,0x134C},
{0x134D,Lo,0x134D,0x134D,0x134D},
{0x134E,Lo,0x134E,0x134E,0x134E},
{0x134F,Lo,0x134F,0x134F,0x134F},
{0x1350,Lo,0x1350,0x1350,0x1350},
{0x1351,Lo,0x1351,0x1351,0x1351},
{0x1352,Lo,0x1352,0x1352,0x1352},
{0x1353,Lo,0x1353,0x1353,0x1353},
{0x1354,Lo,0x1354,0x1354,0x1354},
{0x1355,Lo,0x1355,0x1355,0x1355},
{0x1356,Lo,0x1356,0x1356,0x1356},
{0x1357,Lo,0x1357,0x1357,0x1357},
{0x1358,Lo,0x1358,0x1358,0x1358},
{0x1359,Lo,0x1359,0x1359,0x1359},
{0x135A,Lo,0x135A,0x135A,0x135A},
{0x135F,Mn,0x135F,0x135F,0x135F},
{0x1360,So,0x1360,0x1360,0x1360},
{0x1361,Po,0x1361,0x1361,0x1361},
{0x1362,Po,0x1362,0x1362,0x1362},
{0x1363,Po,0x1363,0x1363,0x1363},
{0x1364,Po,0x1364,0x1364,0x1364},
{0x1365,Po,0x1365,0x1365,0x1365},
{0x1366,Po,0x1366,0x1366,0x1366},
{0x1367,Po,0x1367,0x1367,0x1367},
{0x1368,Po,0x1368,0x1368,0x1368},
{0x1369,No,0x1369,0x1369,0x1369},
{0x136A,No,0x136A,0x136A,0x136A},
{0x136B,No,0x136B,0x136B,0x136B},
{0x136C,No,0x136C,0x136C,0x136C},
{0x136D,No,0x136D,0x136D,0x136D},
{0x136E,No,0x136E,0x136E,0x136E},
{0x136F,No,0x136F,0x136F,0x136F},
{0x1370,No,0x1370,0x1370,0x1370},
{0x1371,No,0x1371,0x1371,0x1371},
{0x1372,No,0x1372,0x1372,0x1372},
{0x1373,No,0x1373,0x1373,0x1373},
{0x1374,No,0x1374,0x1374,0x1374},
{0x1375,No,0x1375,0x1375,0x1375},
{0x1376,No,0x1376,0x1376,0x1376},
{0x1377,No,0x1377,0x1377,0x1377},
{0x1378,No,0x1378,0x1378,0x1378},
{0x1379,No,0x1379,0x1379,0x1379},
{0x137A,No,0x137A,0x137A,0x137A},
{0x137B,No,0x137B,0x137B,0x137B},
{0x137C,No,0x137C,0x137C,0x137C},
{0x1380,Lo,0x1380,0x1380,0x1380},
{0x1381,Lo,0x1381,0x1381,0x1381},
{0x1382,Lo,0x1382,0x1382,0x1382},
{0x1383,Lo,0x1383,0x1383,0x1383},
{0x1384,Lo,0x1384,0x1384,0x1384},
{0x1385,Lo,0x1385,0x1385,0x1385},
{0x1386,Lo,0x1386,0x1386,0x1386},
{0x1387,Lo,0x1387,0x1387,0x1387},
{0x1388,Lo,0x1388,0x1388,0x1388},
{0x1389,Lo,0x1389,0x1389,0x1389},
{0x138A,Lo,0x138A,0x138A,0x138A},
{0x138B,Lo,0x138B,0x138B,0x138B},
{0x138C,Lo,0x138C,0x138C,0x138C},
{0x138D,Lo,0x138D,0x138D,0x138D},
{0x138E,Lo,0x138E,0x138E,0x138E},
{0x138F,Lo,0x138F,0x138F,0x138F},
{0x1390,So,0x1390,0x1390,0x1390},
{0x1391,So,0x1391,0x1391,0x1391},
{0x1392,So,0x1392,0x1392,0x1392},
{0x1393,So,0x1393,0x1393,0x1393},
{0x1394,So,0x1394,0x1394,0x1394},
{0x1395,So,0x1395,0x1395,0x1395},
{0x1396,So,0x1396,0x1396,0x1396},
{0x1397,So,0x1397,0x1397,0x1397},
{0x1398,So,0x1398,0x1398,0x1398},
{0x1399,So,0x1399,0x1399,0x1399},
{0x13A0,Lo,0x13A0,0x13A0,0x13A0},
{0x13A1,Lo,0x13A1,0x13A1,0x13A1},
{0x13A2,Lo,0x13A2,0x13A2,0x13A2},
{0x13A3,Lo,0x13A3,0x13A3,0x13A3},
{0x13A4,Lo,0x13A4,0x13A4,0x13A4},
{0x13A5,Lo,0x13A5,0x13A5,0x13A5},
{0x13A6,Lo,0x13A6,0x13A6,0x13A6},
{0x13A7,Lo,0x13A7,0x13A7,0x13A7},
{0x13A8,Lo,0x13A8,0x13A8,0x13A8},
{0x13A9,Lo,0x13A9,0x13A9,0x13A9},
{0x13AA,Lo,0x13AA,0x13AA,0x13AA},
{0x13AB,Lo,0x13AB,0x13AB,0x13AB},
{0x13AC,Lo,0x13AC,0x13AC,0x13AC},
{0x13AD,Lo,0x13AD,0x13AD,0x13AD},
{0x13AE,Lo,0x13AE,0x13AE,0x13AE},
{0x13AF,Lo,0x13AF,0x13AF,0x13AF},
{0x13B0,Lo,0x13B0,0x13B0,0x13B0},
{0x13B1,Lo,0x13B1,0x13B1,0x13B1},
{0x13B2,Lo,0x13B2,0x13B2,0x13B2},
{0x13B3,Lo,0x13B3,0x13B3,0x13B3},
{0x13B4,Lo,0x13B4,0x13B4,0x13B4},
{0x13B5,Lo,0x13B5,0x13B5,0x13B5},
{0x13B6,Lo,0x13B6,0x13B6,0x13B6},
{0x13B7,Lo,0x13B7,0x13B7,0x13B7},
{0x13B8,Lo,0x13B8,0x13B8,0x13B8},
{0x13B9,Lo,0x13B9,0x13B9,0x13B9},
{0x13BA,Lo,0x13BA,0x13BA,0x13BA},
{0x13BB,Lo,0x13BB,0x13BB,0x13BB},
{0x13BC,Lo,0x13BC,0x13BC,0x13BC},
{0x13BD,Lo,0x13BD,0x13BD,0x13BD},
{0x13BE,Lo,0x13BE,0x13BE,0x13BE},
{0x13BF,Lo,0x13BF,0x13BF,0x13BF},
{0x13C0,Lo,0x13C0,0x13C0,0x13C0},
{0x13C1,Lo,0x13C1,0x13C1,0x13C1},
{0x13C2,Lo,0x13C2,0x13C2,0x13C2},
{0x13C3,Lo,0x13C3,0x13C3,0x13C3},
{0x13C4,Lo,0x13C4,0x13C4,0x13C4},
{0x13C5,Lo,0x13C5,0x13C5,0x13C5},
{0x13C6,Lo,0x13C6,0x13C6,0x13C6},
{0x13C7,Lo,0x13C7,0x13C7,0x13C7},
{0x13C8,Lo,0x13C8,0x13C8,0x13C8},
{0x13C9,Lo,0x13C9,0x13C9,0x13C9},
{0x13CA,Lo,0x13CA,0x13CA,0x13CA},
{0x13CB,Lo,0x13CB,0x13CB,0x13CB},
{0x13CC,Lo,0x13CC,0x13CC,0x13CC},
{0x13CD,Lo,0x13CD,0x13CD,0x13CD},
{0x13CE,Lo,0x13CE,0x13CE,0x13CE},
{0x13CF,Lo,0x13CF,0x13CF,0x13CF},
{0x13D0,Lo,0x13D0,0x13D0,0x13D0},
{0x13D1,Lo,0x13D1,0x13D1,0x13D1},
{0x13D2,Lo,0x13D2,0x13D2,0x13D2},
{0x13D3,Lo,0x13D3,0x13D3,0x13D3},
{0x13D4,Lo,0x13D4,0x13D4,0x13D4},
{0x13D5,Lo,0x13D5,0x13D5,0x13D5},
{0x13D6,Lo,0x13D6,0x13D6,0x13D6},
{0x13D7,Lo,0x13D7,0x13D7,0x13D7},
{0x13D8,Lo,0x13D8,0x13D8,0x13D8},
{0x13D9,Lo,0x13D9,0x13D9,0x13D9},
{0x13DA,Lo,0x13DA,0x13DA,0x13DA},
{0x13DB,Lo,0x13DB,0x13DB,0x13DB},
{0x13DC,Lo,0x13DC,0x13DC,0x13DC},
{0x13DD,Lo,0x13DD,0x13DD,0x13DD},
{0x13DE,Lo,0x13DE,0x13DE,0x13DE},
{0x13DF,Lo,0x13DF,0x13DF,0x13DF},
{0x13E0,Lo,0x13E0,0x13E0,0x13E0},
{0x13E1,Lo,0x13E1,0x13E1,0x13E1},
{0x13E2,Lo,0x13E2,0x13E2,0x13E2},
{0x13E3,Lo,0x13E3,0x13E3,0x13E3},
{0x13E4,Lo,0x13E4,0x13E4,0x13E4},
{0x13E5,Lo,0x13E5,0x13E5,0x13E5},
{0x13E6,Lo,0x13E6,0x13E6,0x13E6},
{0x13E7,Lo,0x13E7,0x13E7,0x13E7},
{0x13E8,Lo,0x13E8,0x13E8,0x13E8},
{0x13E9,Lo,0x13E9,0x13E9,0x13E9},
{0x13EA,Lo,0x13EA,0x13EA,0x13EA},
{0x13EB,Lo,0x13EB,0x13EB,0x13EB},
{0x13EC,Lo,0x13EC,0x13EC,0x13EC},
{0x13ED,Lo,0x13ED,0x13ED,0x13ED},
{0x13EE,Lo,0x13EE,0x13EE,0x13EE},
{0x13EF,Lo,0x13EF,0x13EF,0x13EF},
{0x13F0,Lo,0x13F0,0x13F0,0x13F0},
{0x13F1,Lo,0x13F1,0x13F1,0x13F1},
{0x13F2,Lo,0x13F2,0x13F2,0x13F2},
{0x13F3,Lo,0x13F3,0x13F3,0x13F3},
{0x13F4,Lo,0x13F4,0x13F4,0x13F4},
{0x1401,Lo,0x1401,0x1401,0x1401},
{0x1402,Lo,0x1402,0x1402,0x1402},
{0x1403,Lo,0x1403,0x1403,0x1403},
{0x1404,Lo,0x1404,0x1404,0x1404},
{0x1405,Lo,0x1405,0x1405,0x1405},
{0x1406,Lo,0x1406,0x1406,0x1406},
{0x1407,Lo,0x1407,0x1407,0x1407},
{0x1408,Lo,0x1408,0x1408,0x1408},
{0x1409,Lo,0x1409,0x1409,0x1409},
{0x140A,Lo,0x140A,0x140A,0x140A},
{0x140B,Lo,0x140B,0x140B,0x140B},
{0x140C,Lo,0x140C,0x140C,0x140C},
{0x140D,Lo,0x140D,0x140D,0x140D},
{0x140E,Lo,0x140E,0x140E,0x140E},
{0x140F,Lo,0x140F,0x140F,0x140F},
{0x1410,Lo,0x1410,0x1410,0x1410},
{0x1411,Lo,0x1411,0x1411,0x1411},
{0x1412,Lo,0x1412,0x1412,0x1412},
{0x1413,Lo,0x1413,0x1413,0x1413},
{0x1414,Lo,0x1414,0x1414,0x1414},
{0x1415,Lo,0x1415,0x1415,0x1415},
{0x1416,Lo,0x1416,0x1416,0x1416},
{0x1417,Lo,0x1417,0x1417,0x1417},
{0x1418,Lo,0x1418,0x1418,0x1418},
{0x1419,Lo,0x1419,0x1419,0x1419},
{0x141A,Lo,0x141A,0x141A,0x141A},
{0x141B,Lo,0x141B,0x141B,0x141B},
{0x141C,Lo,0x141C,0x141C,0x141C},
{0x141D,Lo,0x141D,0x141D,0x141D},
{0x141E,Lo,0x141E,0x141E,0x141E},
{0x141F,Lo,0x141F,0x141F,0x141F},
{0x1420,Lo,0x1420,0x1420,0x1420},
{0x1421,Lo,0x1421,0x1421,0x1421},
{0x1422,Lo,0x1422,0x1422,0x1422},
{0x1423,Lo,0x1423,0x1423,0x1423},
{0x1424,Lo,0x1424,0x1424,0x1424},
{0x1425,Lo,0x1425,0x1425,0x1425},
{0x1426,Lo,0x1426,0x1426,0x1426},
{0x1427,Lo,0x1427,0x1427,0x1427},
{0x1428,Lo,0x1428,0x1428,0x1428},
{0x1429,Lo,0x1429,0x1429,0x1429},
{0x142A,Lo,0x142A,0x142A,0x142A},
{0x142B,Lo,0x142B,0x142B,0x142B},
{0x142C,Lo,0x142C,0x142C,0x142C},
{0x142D,Lo,0x142D,0x142D,0x142D},
{0x142E,Lo,0x142E,0x142E,0x142E},
{0x142F,Lo,0x142F,0x142F,0x142F},
{0x1430,Lo,0x1430,0x1430,0x1430},
{0x1431,Lo,0x1431,0x1431,0x1431},
{0x1432,Lo,0x1432,0x1432,0x1432},
{0x1433,Lo,0x1433,0x1433,0x1433},
{0x1434,Lo,0x1434,0x1434,0x1434},
{0x1435,Lo,0x1435,0x1435,0x1435},
{0x1436,Lo,0x1436,0x1436,0x1436},
{0x1437,Lo,0x1437,0x1437,0x1437},
{0x1438,Lo,0x1438,0x1438,0x1438},
{0x1439,Lo,0x1439,0x1439,0x1439},
{0x143A,Lo,0x143A,0x143A,0x143A},
{0x143B,Lo,0x143B,0x143B,0x143B},
{0x143C,Lo,0x143C,0x143C,0x143C},
{0x143D,Lo,0x143D,0x143D,0x143D},
{0x143E,Lo,0x143E,0x143E,0x143E},
{0x143F,Lo,0x143F,0x143F,0x143F},
{0x1440,Lo,0x1440,0x1440,0x1440},
{0x1441,Lo,0x1441,0x1441,0x1441},
{0x1442,Lo,0x1442,0x1442,0x1442},
{0x1443,Lo,0x1443,0x1443,0x1443},
{0x1444,Lo,0x1444,0x1444,0x1444},
{0x1445,Lo,0x1445,0x1445,0x1445},
{0x1446,Lo,0x1446,0x1446,0x1446},
{0x1447,Lo,0x1447,0x1447,0x1447},
{0x1448,Lo,0x1448,0x1448,0x1448},
{0x1449,Lo,0x1449,0x1449,0x1449},
{0x144A,Lo,0x144A,0x144A,0x144A},
{0x144B,Lo,0x144B,0x144B,0x144B},
{0x144C,Lo,0x144C,0x144C,0x144C},
{0x144D,Lo,0x144D,0x144D,0x144D},
{0x144E,Lo,0x144E,0x144E,0x144E},
{0x144F,Lo,0x144F,0x144F,0x144F},
{0x1450,Lo,0x1450,0x1450,0x1450},
{0x1451,Lo,0x1451,0x1451,0x1451},
{0x1452,Lo,0x1452,0x1452,0x1452},
{0x1453,Lo,0x1453,0x1453,0x1453},
{0x1454,Lo,0x1454,0x1454,0x1454},
{0x1455,Lo,0x1455,0x1455,0x1455},
{0x1456,Lo,0x1456,0x1456,0x1456},
{0x1457,Lo,0x1457,0x1457,0x1457},
{0x1458,Lo,0x1458,0x1458,0x1458},
{0x1459,Lo,0x1459,0x1459,0x1459},
{0x145A,Lo,0x145A,0x145A,0x145A},
{0x145B,Lo,0x145B,0x145B,0x145B},
{0x145C,Lo,0x145C,0x145C,0x145C},
{0x145D,Lo,0x145D,0x145D,0x145D},
{0x145E,Lo,0x145E,0x145E,0x145E},
{0x145F,Lo,0x145F,0x145F,0x145F},
{0x1460,Lo,0x1460,0x1460,0x1460},
{0x1461,Lo,0x1461,0x1461,0x1461},
{0x1462,Lo,0x1462,0x1462,0x1462},
{0x1463,Lo,0x1463,0x1463,0x1463},
{0x1464,Lo,0x1464,0x1464,0x1464},
{0x1465,Lo,0x1465,0x1465,0x1465},
{0x1466,Lo,0x1466,0x1466,0x1466},
{0x1467,Lo,0x1467,0x1467,0x1467},
{0x1468,Lo,0x1468,0x1468,0x1468},
{0x1469,Lo,0x1469,0x1469,0x1469},
{0x146A,Lo,0x146A,0x146A,0x146A},
{0x146B,Lo,0x146B,0x146B,0x146B},
{0x146C,Lo,0x146C,0x146C,0x146C},
{0x146D,Lo,0x146D,0x146D,0x146D},
{0x146E,Lo,0x146E,0x146E,0x146E},
{0x146F,Lo,0x146F,0x146F,0x146F},
{0x1470,Lo,0x1470,0x1470,0x1470},
{0x1471,Lo,0x1471,0x1471,0x1471},
{0x1472,Lo,0x1472,0x1472,0x1472},
{0x1473,Lo,0x1473,0x1473,0x1473},
{0x1474,Lo,0x1474,0x1474,0x1474},
{0x1475,Lo,0x1475,0x1475,0x1475},
{0x1476,Lo,0x1476,0x1476,0x1476},
{0x1477,Lo,0x1477,0x1477,0x1477},
{0x1478,Lo,0x1478,0x1478,0x1478},
{0x1479,Lo,0x1479,0x1479,0x1479},
{0x147A,Lo,0x147A,0x147A,0x147A},
{0x147B,Lo,0x147B,0x147B,0x147B},
{0x147C,Lo,0x147C,0x147C,0x147C},
{0x147D,Lo,0x147D,0x147D,0x147D},
{0x147E,Lo,0x147E,0x147E,0x147E},
{0x147F,Lo,0x147F,0x147F,0x147F},
{0x1480,Lo,0x1480,0x1480,0x1480},
{0x1481,Lo,0x1481,0x1481,0x1481},
{0x1482,Lo,0x1482,0x1482,0x1482},
{0x1483,Lo,0x1483,0x1483,0x1483},
{0x1484,Lo,0x1484,0x1484,0x1484},
{0x1485,Lo,0x1485,0x1485,0x1485},
{0x1486,Lo,0x1486,0x1486,0x1486},
{0x1487,Lo,0x1487,0x1487,0x1487},
{0x1488,Lo,0x1488,0x1488,0x1488},
{0x1489,Lo,0x1489,0x1489,0x1489},
{0x148A,Lo,0x148A,0x148A,0x148A},
{0x148B,Lo,0x148B,0x148B,0x148B},
{0x148C,Lo,0x148C,0x148C,0x148C},
{0x148D,Lo,0x148D,0x148D,0x148D},
{0x148E,Lo,0x148E,0x148E,0x148E},
{0x148F,Lo,0x148F,0x148F,0x148F},
{0x1490,Lo,0x1490,0x1490,0x1490},
{0x1491,Lo,0x1491,0x1491,0x1491},
{0x1492,Lo,0x1492,0x1492,0x1492},
{0x1493,Lo,0x1493,0x1493,0x1493},
{0x1494,Lo,0x1494,0x1494,0x1494},
{0x1495,Lo,0x1495,0x1495,0x1495},
{0x1496,Lo,0x1496,0x1496,0x1496},
{0x1497,Lo,0x1497,0x1497,0x1497},
{0x1498,Lo,0x1498,0x1498,0x1498},
{0x1499,Lo,0x1499,0x1499,0x1499},
{0x149A,Lo,0x149A,0x149A,0x149A},
{0x149B,Lo,0x149B,0x149B,0x149B},
{0x149C,Lo,0x149C,0x149C,0x149C},
{0x149D,Lo,0x149D,0x149D,0x149D},
{0x149E,Lo,0x149E,0x149E,0x149E},
{0x149F,Lo,0x149F,0x149F,0x149F},
{0x14A0,Lo,0x14A0,0x14A0,0x14A0},
{0x14A1,Lo,0x14A1,0x14A1,0x14A1},
{0x14A2,Lo,0x14A2,0x14A2,0x14A2},
{0x14A3,Lo,0x14A3,0x14A3,0x14A3},
{0x14A4,Lo,0x14A4,0x14A4,0x14A4},
{0x14A5,Lo,0x14A5,0x14A5,0x14A5},
{0x14A6,Lo,0x14A6,0x14A6,0x14A6},
{0x14A7,Lo,0x14A7,0x14A7,0x14A7},
{0x14A8,Lo,0x14A8,0x14A8,0x14A8},
{0x14A9,Lo,0x14A9,0x14A9,0x14A9},
{0x14AA,Lo,0x14AA,0x14AA,0x14AA},
{0x14AB,Lo,0x14AB,0x14AB,0x14AB},
{0x14AC,Lo,0x14AC,0x14AC,0x14AC},
{0x14AD,Lo,0x14AD,0x14AD,0x14AD},
{0x14AE,Lo,0x14AE,0x14AE,0x14AE},
{0x14AF,Lo,0x14AF,0x14AF,0x14AF},
{0x14B0,Lo,0x14B0,0x14B0,0x14B0},
{0x14B1,Lo,0x14B1,0x14B1,0x14B1},
{0x14B2,Lo,0x14B2,0x14B2,0x14B2},
{0x14B3,Lo,0x14B3,0x14B3,0x14B3},
{0x14B4,Lo,0x14B4,0x14B4,0x14B4},
{0x14B5,Lo,0x14B5,0x14B5,0x14B5},
{0x14B6,Lo,0x14B6,0x14B6,0x14B6},
{0x14B7,Lo,0x14B7,0x14B7,0x14B7},
{0x14B8,Lo,0x14B8,0x14B8,0x14B8},
{0x14B9,Lo,0x14B9,0x14B9,0x14B9},
{0x14BA,Lo,0x14BA,0x14BA,0x14BA},
{0x14BB,Lo,0x14BB,0x14BB,0x14BB},
{0x14BC,Lo,0x14BC,0x14BC,0x14BC},
{0x14BD,Lo,0x14BD,0x14BD,0x14BD},
{0x14BE,Lo,0x14BE,0x14BE,0x14BE},
{0x14BF,Lo,0x14BF,0x14BF,0x14BF},
{0x14C0,Lo,0x14C0,0x14C0,0x14C0},
{0x14C1,Lo,0x14C1,0x14C1,0x14C1},
{0x14C2,Lo,0x14C2,0x14C2,0x14C2},
{0x14C3,Lo,0x14C3,0x14C3,0x14C3},
{0x14C4,Lo,0x14C4,0x14C4,0x14C4},
{0x14C5,Lo,0x14C5,0x14C5,0x14C5},
{0x14C6,Lo,0x14C6,0x14C6,0x14C6},
{0x14C7,Lo,0x14C7,0x14C7,0x14C7},
{0x14C8,Lo,0x14C8,0x14C8,0x14C8},
{0x14C9,Lo,0x14C9,0x14C9,0x14C9},
{0x14CA,Lo,0x14CA,0x14CA,0x14CA},
{0x14CB,Lo,0x14CB,0x14CB,0x14CB},
{0x14CC,Lo,0x14CC,0x14CC,0x14CC},
{0x14CD,Lo,0x14CD,0x14CD,0x14CD},
{0x14CE,Lo,0x14CE,0x14CE,0x14CE},
{0x14CF,Lo,0x14CF,0x14CF,0x14CF},
{0x14D0,Lo,0x14D0,0x14D0,0x14D0},
{0x14D1,Lo,0x14D1,0x14D1,0x14D1},
{0x14D2,Lo,0x14D2,0x14D2,0x14D2},
{0x14D3,Lo,0x14D3,0x14D3,0x14D3},
{0x14D4,Lo,0x14D4,0x14D4,0x14D4},
{0x14D5,Lo,0x14D5,0x14D5,0x14D5},
{0x14D6,Lo,0x14D6,0x14D6,0x14D6},
{0x14D7,Lo,0x14D7,0x14D7,0x14D7},
{0x14D8,Lo,0x14D8,0x14D8,0x14D8},
{0x14D9,Lo,0x14D9,0x14D9,0x14D9},
{0x14DA,Lo,0x14DA,0x14DA,0x14DA},
{0x14DB,Lo,0x14DB,0x14DB,0x14DB},
{0x14DC,Lo,0x14DC,0x14DC,0x14DC},
{0x14DD,Lo,0x14DD,0x14DD,0x14DD},
{0x14DE,Lo,0x14DE,0x14DE,0x14DE},
{0x14DF,Lo,0x14DF,0x14DF,0x14DF},
{0x14E0,Lo,0x14E0,0x14E0,0x14E0},
{0x14E1,Lo,0x14E1,0x14E1,0x14E1},
{0x14E2,Lo,0x14E2,0x14E2,0x14E2},
{0x14E3,Lo,0x14E3,0x14E3,0x14E3},
{0x14E4,Lo,0x14E4,0x14E4,0x14E4},
{0x14E5,Lo,0x14E5,0x14E5,0x14E5},
{0x14E6,Lo,0x14E6,0x14E6,0x14E6},
{0x14E7,Lo,0x14E7,0x14E7,0x14E7},
{0x14E8,Lo,0x14E8,0x14E8,0x14E8},
{0x14E9,Lo,0x14E9,0x14E9,0x14E9},
{0x14EA,Lo,0x14EA,0x14EA,0x14EA},
{0x14EB,Lo,0x14EB,0x14EB,0x14EB},
{0x14EC,Lo,0x14EC,0x14EC,0x14EC},
{0x14ED,Lo,0x14ED,0x14ED,0x14ED},
{0x14EE,Lo,0x14EE,0x14EE,0x14EE},
{0x14EF,Lo,0x14EF,0x14EF,0x14EF},
{0x14F0,Lo,0x14F0,0x14F0,0x14F0},
{0x14F1,Lo,0x14F1,0x14F1,0x14F1},
{0x14F2,Lo,0x14F2,0x14F2,0x14F2},
{0x14F3,Lo,0x14F3,0x14F3,0x14F3},
{0x14F4,Lo,0x14F4,0x14F4,0x14F4},
{0x14F5,Lo,0x14F5,0x14F5,0x14F5},
{0x14F6,Lo,0x14F6,0x14F6,0x14F6},
{0x14F7,Lo,0x14F7,0x14F7,0x14F7},
{0x14F8,Lo,0x14F8,0x14F8,0x14F8},
{0x14F9,Lo,0x14F9,0x14F9,0x14F9},
{0x14FA,Lo,0x14FA,0x14FA,0x14FA},
{0x14FB,Lo,0x14FB,0x14FB,0x14FB},
{0x14FC,Lo,0x14FC,0x14FC,0x14FC},
{0x14FD,Lo,0x14FD,0x14FD,0x14FD},
{0x14FE,Lo,0x14FE,0x14FE,0x14FE},
{0x14FF,Lo,0x14FF,0x14FF,0x14FF},
{0x1500,Lo,0x1500,0x1500,0x1500},
{0x1501,Lo,0x1501,0x1501,0x1501},
{0x1502,Lo,0x1502,0x1502,0x1502},
{0x1503,Lo,0x1503,0x1503,0x1503},
{0x1504,Lo,0x1504,0x1504,0x1504},
{0x1505,Lo,0x1505,0x1505,0x1505},
{0x1506,Lo,0x1506,0x1506,0x1506},
{0x1507,Lo,0x1507,0x1507,0x1507},
{0x1508,Lo,0x1508,0x1508,0x1508},
{0x1509,Lo,0x1509,0x1509,0x1509},
{0x150A,Lo,0x150A,0x150A,0x150A},
{0x150B,Lo,0x150B,0x150B,0x150B},
{0x150C,Lo,0x150C,0x150C,0x150C},
{0x150D,Lo,0x150D,0x150D,0x150D},
{0x150E,Lo,0x150E,0x150E,0x150E},
{0x150F,Lo,0x150F,0x150F,0x150F},
{0x1510,Lo,0x1510,0x1510,0x1510},
{0x1511,Lo,0x1511,0x1511,0x1511},
{0x1512,Lo,0x1512,0x1512,0x1512},
{0x1513,Lo,0x1513,0x1513,0x1513},
{0x1514,Lo,0x1514,0x1514,0x1514},
{0x1515,Lo,0x1515,0x1515,0x1515},
{0x1516,Lo,0x1516,0x1516,0x1516},
{0x1517,Lo,0x1517,0x1517,0x1517},
{0x1518,Lo,0x1518,0x1518,0x1518},
{0x1519,Lo,0x1519,0x1519,0x1519},
{0x151A,Lo,0x151A,0x151A,0x151A},
{0x151B,Lo,0x151B,0x151B,0x151B},
{0x151C,Lo,0x151C,0x151C,0x151C},
{0x151D,Lo,0x151D,0x151D,0x151D},
{0x151E,Lo,0x151E,0x151E,0x151E},
{0x151F,Lo,0x151F,0x151F,0x151F},
{0x1520,Lo,0x1520,0x1520,0x1520},
{0x1521,Lo,0x1521,0x1521,0x1521},
{0x1522,Lo,0x1522,0x1522,0x1522},
{0x1523,Lo,0x1523,0x1523,0x1523},
{0x1524,Lo,0x1524,0x1524,0x1524},
{0x1525,Lo,0x1525,0x1525,0x1525},
{0x1526,Lo,0x1526,0x1526,0x1526},
{0x1527,Lo,0x1527,0x1527,0x1527},
{0x1528,Lo,0x1528,0x1528,0x1528},
{0x1529,Lo,0x1529,0x1529,0x1529},
{0x152A,Lo,0x152A,0x152A,0x152A},
{0x152B,Lo,0x152B,0x152B,0x152B},
{0x152C,Lo,0x152C,0x152C,0x152C},
{0x152D,Lo,0x152D,0x152D,0x152D},
{0x152E,Lo,0x152E,0x152E,0x152E},
{0x152F,Lo,0x152F,0x152F,0x152F},
{0x1530,Lo,0x1530,0x1530,0x1530},
{0x1531,Lo,0x1531,0x1531,0x1531},
{0x1532,Lo,0x1532,0x1532,0x1532},
{0x1533,Lo,0x1533,0x1533,0x1533},
{0x1534,Lo,0x1534,0x1534,0x1534},
{0x1535,Lo,0x1535,0x1535,0x1535},
{0x1536,Lo,0x1536,0x1536,0x1536},
{0x1537,Lo,0x1537,0x1537,0x1537},
{0x1538,Lo,0x1538,0x1538,0x1538},
{0x1539,Lo,0x1539,0x1539,0x1539},
{0x153A,Lo,0x153A,0x153A,0x153A},
{0x153B,Lo,0x153B,0x153B,0x153B},
{0x153C,Lo,0x153C,0x153C,0x153C},
{0x153D,Lo,0x153D,0x153D,0x153D},
{0x153E,Lo,0x153E,0x153E,0x153E},
{0x153F,Lo,0x153F,0x153F,0x153F},
{0x1540,Lo,0x1540,0x1540,0x1540},
{0x1541,Lo,0x1541,0x1541,0x1541},
{0x1542,Lo,0x1542,0x1542,0x1542},
{0x1543,Lo,0x1543,0x1543,0x1543},
{0x1544,Lo,0x1544,0x1544,0x1544},
{0x1545,Lo,0x1545,0x1545,0x1545},
{0x1546,Lo,0x1546,0x1546,0x1546},
{0x1547,Lo,0x1547,0x1547,0x1547},
{0x1548,Lo,0x1548,0x1548,0x1548},
{0x1549,Lo,0x1549,0x1549,0x1549},
{0x154A,Lo,0x154A,0x154A,0x154A},
{0x154B,Lo,0x154B,0x154B,0x154B},
{0x154C,Lo,0x154C,0x154C,0x154C},
{0x154D,Lo,0x154D,0x154D,0x154D},
{0x154E,Lo,0x154E,0x154E,0x154E},
{0x154F,Lo,0x154F,0x154F,0x154F},
{0x1550,Lo,0x1550,0x1550,0x1550},
{0x1551,Lo,0x1551,0x1551,0x1551},
{0x1552,Lo,0x1552,0x1552,0x1552},
{0x1553,Lo,0x1553,0x1553,0x1553},
{0x1554,Lo,0x1554,0x1554,0x1554},
{0x1555,Lo,0x1555,0x1555,0x1555},
{0x1556,Lo,0x1556,0x1556,0x1556},
{0x1557,Lo,0x1557,0x1557,0x1557},
{0x1558,Lo,0x1558,0x1558,0x1558},
{0x1559,Lo,0x1559,0x1559,0x1559},
{0x155A,Lo,0x155A,0x155A,0x155A},
{0x155B,Lo,0x155B,0x155B,0x155B},
{0x155C,Lo,0x155C,0x155C,0x155C},
{0x155D,Lo,0x155D,0x155D,0x155D},
{0x155E,Lo,0x155E,0x155E,0x155E},
{0x155F,Lo,0x155F,0x155F,0x155F},
{0x1560,Lo,0x1560,0x1560,0x1560},
{0x1561,Lo,0x1561,0x1561,0x1561},
{0x1562,Lo,0x1562,0x1562,0x1562},
{0x1563,Lo,0x1563,0x1563,0x1563},
{0x1564,Lo,0x1564,0x1564,0x1564},
{0x1565,Lo,0x1565,0x1565,0x1565},
{0x1566,Lo,0x1566,0x1566,0x1566},
{0x1567,Lo,0x1567,0x1567,0x1567},
{0x1568,Lo,0x1568,0x1568,0x1568},
{0x1569,Lo,0x1569,0x1569,0x1569},
{0x156A,Lo,0x156A,0x156A,0x156A},
{0x156B,Lo,0x156B,0x156B,0x156B},
{0x156C,Lo,0x156C,0x156C,0x156C},
{0x156D,Lo,0x156D,0x156D,0x156D},
{0x156E,Lo,0x156E,0x156E,0x156E},
{0x156F,Lo,0x156F,0x156F,0x156F},
{0x1570,Lo,0x1570,0x1570,0x1570},
{0x1571,Lo,0x1571,0x1571,0x1571},
{0x1572,Lo,0x1572,0x1572,0x1572},
{0x1573,Lo,0x1573,0x1573,0x1573},
{0x1574,Lo,0x1574,0x1574,0x1574},
{0x1575,Lo,0x1575,0x1575,0x1575},
{0x1576,Lo,0x1576,0x1576,0x1576},
{0x1577,Lo,0x1577,0x1577,0x1577},
{0x1578,Lo,0x1578,0x1578,0x1578},
{0x1579,Lo,0x1579,0x1579,0x1579},
{0x157A,Lo,0x157A,0x157A,0x157A},
{0x157B,Lo,0x157B,0x157B,0x157B},
{0x157C,Lo,0x157C,0x157C,0x157C},
{0x157D,Lo,0x157D,0x157D,0x157D},
{0x157E,Lo,0x157E,0x157E,0x157E},
{0x157F,Lo,0x157F,0x157F,0x157F},
{0x1580,Lo,0x1580,0x1580,0x1580},
{0x1581,Lo,0x1581,0x1581,0x1581},
{0x1582,Lo,0x1582,0x1582,0x1582},
{0x1583,Lo,0x1583,0x1583,0x1583},
{0x1584,Lo,0x1584,0x1584,0x1584},
{0x1585,Lo,0x1585,0x1585,0x1585},
{0x1586,Lo,0x1586,0x1586,0x1586},
{0x1587,Lo,0x1587,0x1587,0x1587},
{0x1588,Lo,0x1588,0x1588,0x1588},
{0x1589,Lo,0x1589,0x1589,0x1589},
{0x158A,Lo,0x158A,0x158A,0x158A},
{0x158B,Lo,0x158B,0x158B,0x158B},
{0x158C,Lo,0x158C,0x158C,0x158C},
{0x158D,Lo,0x158D,0x158D,0x158D},
{0x158E,Lo,0x158E,0x158E,0x158E},
{0x158F,Lo,0x158F,0x158F,0x158F},
{0x1590,Lo,0x1590,0x1590,0x1590},
{0x1591,Lo,0x1591,0x1591,0x1591},
{0x1592,Lo,0x1592,0x1592,0x1592},
{0x1593,Lo,0x1593,0x1593,0x1593},
{0x1594,Lo,0x1594,0x1594,0x1594},
{0x1595,Lo,0x1595,0x1595,0x1595},
{0x1596,Lo,0x1596,0x1596,0x1596},
{0x1597,Lo,0x1597,0x1597,0x1597},
{0x1598,Lo,0x1598,0x1598,0x1598},
{0x1599,Lo,0x1599,0x1599,0x1599},
{0x159A,Lo,0x159A,0x159A,0x159A},
{0x159B,Lo,0x159B,0x159B,0x159B},
{0x159C,Lo,0x159C,0x159C,0x159C},
{0x159D,Lo,0x159D,0x159D,0x159D},
{0x159E,Lo,0x159E,0x159E,0x159E},
{0x159F,Lo,0x159F,0x159F,0x159F},
{0x15A0,Lo,0x15A0,0x15A0,0x15A0},
{0x15A1,Lo,0x15A1,0x15A1,0x15A1},
{0x15A2,Lo,0x15A2,0x15A2,0x15A2},
{0x15A3,Lo,0x15A3,0x15A3,0x15A3},
{0x15A4,Lo,0x15A4,0x15A4,0x15A4},
{0x15A5,Lo,0x15A5,0x15A5,0x15A5},
{0x15A6,Lo,0x15A6,0x15A6,0x15A6},
{0x15A7,Lo,0x15A7,0x15A7,0x15A7},
{0x15A8,Lo,0x15A8,0x15A8,0x15A8},
{0x15A9,Lo,0x15A9,0x15A9,0x15A9},
{0x15AA,Lo,0x15AA,0x15AA,0x15AA},
{0x15AB,Lo,0x15AB,0x15AB,0x15AB},
{0x15AC,Lo,0x15AC,0x15AC,0x15AC},
{0x15AD,Lo,0x15AD,0x15AD,0x15AD},
{0x15AE,Lo,0x15AE,0x15AE,0x15AE},
{0x15AF,Lo,0x15AF,0x15AF,0x15AF},
{0x15B0,Lo,0x15B0,0x15B0,0x15B0},
{0x15B1,Lo,0x15B1,0x15B1,0x15B1},
{0x15B2,Lo,0x15B2,0x15B2,0x15B2},
{0x15B3,Lo,0x15B3,0x15B3,0x15B3},
{0x15B4,Lo,0x15B4,0x15B4,0x15B4},
{0x15B5,Lo,0x15B5,0x15B5,0x15B5},
{0x15B6,Lo,0x15B6,0x15B6,0x15B6},
{0x15B7,Lo,0x15B7,0x15B7,0x15B7},
{0x15B8,Lo,0x15B8,0x15B8,0x15B8},
{0x15B9,Lo,0x15B9,0x15B9,0x15B9},
{0x15BA,Lo,0x15BA,0x15BA,0x15BA},
{0x15BB,Lo,0x15BB,0x15BB,0x15BB},
{0x15BC,Lo,0x15BC,0x15BC,0x15BC},
{0x15BD,Lo,0x15BD,0x15BD,0x15BD},
{0x15BE,Lo,0x15BE,0x15BE,0x15BE},
{0x15BF,Lo,0x15BF,0x15BF,0x15BF},
{0x15C0,Lo,0x15C0,0x15C0,0x15C0},
{0x15C1,Lo,0x15C1,0x15C1,0x15C1},
{0x15C2,Lo,0x15C2,0x15C2,0x15C2},
{0x15C3,Lo,0x15C3,0x15C3,0x15C3},
{0x15C4,Lo,0x15C4,0x15C4,0x15C4},
{0x15C5,Lo,0x15C5,0x15C5,0x15C5},
{0x15C6,Lo,0x15C6,0x15C6,0x15C6},
{0x15C7,Lo,0x15C7,0x15C7,0x15C7},
{0x15C8,Lo,0x15C8,0x15C8,0x15C8},
{0x15C9,Lo,0x15C9,0x15C9,0x15C9},
{0x15CA,Lo,0x15CA,0x15CA,0x15CA},
{0x15CB,Lo,0x15CB,0x15CB,0x15CB},
{0x15CC,Lo,0x15CC,0x15CC,0x15CC},
{0x15CD,Lo,0x15CD,0x15CD,0x15CD},
{0x15CE,Lo,0x15CE,0x15CE,0x15CE},
{0x15CF,Lo,0x15CF,0x15CF,0x15CF},
{0x15D0,Lo,0x15D0,0x15D0,0x15D0},
{0x15D1,Lo,0x15D1,0x15D1,0x15D1},
{0x15D2,Lo,0x15D2,0x15D2,0x15D2},
{0x15D3,Lo,0x15D3,0x15D3,0x15D3},
{0x15D4,Lo,0x15D4,0x15D4,0x15D4},
{0x15D5,Lo,0x15D5,0x15D5,0x15D5},
{0x15D6,Lo,0x15D6,0x15D6,0x15D6},
{0x15D7,Lo,0x15D7,0x15D7,0x15D7},
{0x15D8,Lo,0x15D8,0x15D8,0x15D8},
{0x15D9,Lo,0x15D9,0x15D9,0x15D9},
{0x15DA,Lo,0x15DA,0x15DA,0x15DA},
{0x15DB,Lo,0x15DB,0x15DB,0x15DB},
{0x15DC,Lo,0x15DC,0x15DC,0x15DC},
{0x15DD,Lo,0x15DD,0x15DD,0x15DD},
{0x15DE,Lo,0x15DE,0x15DE,0x15DE},
{0x15DF,Lo,0x15DF,0x15DF,0x15DF},
{0x15E0,Lo,0x15E0,0x15E0,0x15E0},
{0x15E1,Lo,0x15E1,0x15E1,0x15E1},
{0x15E2,Lo,0x15E2,0x15E2,0x15E2},
{0x15E3,Lo,0x15E3,0x15E3,0x15E3},
{0x15E4,Lo,0x15E4,0x15E4,0x15E4},
{0x15E5,Lo,0x15E5,0x15E5,0x15E5},
{0x15E6,Lo,0x15E6,0x15E6,0x15E6},
{0x15E7,Lo,0x15E7,0x15E7,0x15E7},
{0x15E8,Lo,0x15E8,0x15E8,0x15E8},
{0x15E9,Lo,0x15E9,0x15E9,0x15E9},
{0x15EA,Lo,0x15EA,0x15EA,0x15EA},
{0x15EB,Lo,0x15EB,0x15EB,0x15EB},
{0x15EC,Lo,0x15EC,0x15EC,0x15EC},
{0x15ED,Lo,0x15ED,0x15ED,0x15ED},
{0x15EE,Lo,0x15EE,0x15EE,0x15EE},
{0x15EF,Lo,0x15EF,0x15EF,0x15EF},
{0x15F0,Lo,0x15F0,0x15F0,0x15F0},
{0x15F1,Lo,0x15F1,0x15F1,0x15F1},
{0x15F2,Lo,0x15F2,0x15F2,0x15F2},
{0x15F3,Lo,0x15F3,0x15F3,0x15F3},
{0x15F4,Lo,0x15F4,0x15F4,0x15F4},
{0x15F5,Lo,0x15F5,0x15F5,0x15F5},
{0x15F6,Lo,0x15F6,0x15F6,0x15F6},
{0x15F7,Lo,0x15F7,0x15F7,0x15F7},
{0x15F8,Lo,0x15F8,0x15F8,0x15F8},
{0x15F9,Lo,0x15F9,0x15F9,0x15F9},
{0x15FA,Lo,0x15FA,0x15FA,0x15FA},
{0x15FB,Lo,0x15FB,0x15FB,0x15FB},
{0x15FC,Lo,0x15FC,0x15FC,0x15FC},
{0x15FD,Lo,0x15FD,0x15FD,0x15FD},
{0x15FE,Lo,0x15FE,0x15FE,0x15FE},
{0x15FF,Lo,0x15FF,0x15FF,0x15FF},
{0x1600,Lo,0x1600,0x1600,0x1600},
{0x1601,Lo,0x1601,0x1601,0x1601},
{0x1602,Lo,0x1602,0x1602,0x1602},
{0x1603,Lo,0x1603,0x1603,0x1603},
{0x1604,Lo,0x1604,0x1604,0x1604},
{0x1605,Lo,0x1605,0x1605,0x1605},
{0x1606,Lo,0x1606,0x1606,0x1606},
{0x1607,Lo,0x1607,0x1607,0x1607},
{0x1608,Lo,0x1608,0x1608,0x1608},
{0x1609,Lo,0x1609,0x1609,0x1609},
{0x160A,Lo,0x160A,0x160A,0x160A},
{0x160B,Lo,0x160B,0x160B,0x160B},
{0x160C,Lo,0x160C,0x160C,0x160C},
{0x160D,Lo,0x160D,0x160D,0x160D},
{0x160E,Lo,0x160E,0x160E,0x160E},
{0x160F,Lo,0x160F,0x160F,0x160F},
{0x1610,Lo,0x1610,0x1610,0x1610},
{0x1611,Lo,0x1611,0x1611,0x1611},
{0x1612,Lo,0x1612,0x1612,0x1612},
{0x1613,Lo,0x1613,0x1613,0x1613},
{0x1614,Lo,0x1614,0x1614,0x1614},
{0x1615,Lo,0x1615,0x1615,0x1615},
{0x1616,Lo,0x1616,0x1616,0x1616},
{0x1617,Lo,0x1617,0x1617,0x1617},
{0x1618,Lo,0x1618,0x1618,0x1618},
{0x1619,Lo,0x1619,0x1619,0x1619},
{0x161A,Lo,0x161A,0x161A,0x161A},
{0x161B,Lo,0x161B,0x161B,0x161B},
{0x161C,Lo,0x161C,0x161C,0x161C},
{0x161D,Lo,0x161D,0x161D,0x161D},
{0x161E,Lo,0x161E,0x161E,0x161E},
{0x161F,Lo,0x161F,0x161F,0x161F},
{0x1620,Lo,0x1620,0x1620,0x1620},
{0x1621,Lo,0x1621,0x1621,0x1621},
{0x1622,Lo,0x1622,0x1622,0x1622},
{0x1623,Lo,0x1623,0x1623,0x1623},
{0x1624,Lo,0x1624,0x1624,0x1624},
{0x1625,Lo,0x1625,0x1625,0x1625},
{0x1626,Lo,0x1626,0x1626,0x1626},
{0x1627,Lo,0x1627,0x1627,0x1627},
{0x1628,Lo,0x1628,0x1628,0x1628},
{0x1629,Lo,0x1629,0x1629,0x1629},
{0x162A,Lo,0x162A,0x162A,0x162A},
{0x162B,Lo,0x162B,0x162B,0x162B},
{0x162C,Lo,0x162C,0x162C,0x162C},
{0x162D,Lo,0x162D,0x162D,0x162D},
{0x162E,Lo,0x162E,0x162E,0x162E},
{0x162F,Lo,0x162F,0x162F,0x162F},
{0x1630,Lo,0x1630,0x1630,0x1630},
{0x1631,Lo,0x1631,0x1631,0x1631},
{0x1632,Lo,0x1632,0x1632,0x1632},
{0x1633,Lo,0x1633,0x1633,0x1633},
{0x1634,Lo,0x1634,0x1634,0x1634},
{0x1635,Lo,0x1635,0x1635,0x1635},
{0x1636,Lo,0x1636,0x1636,0x1636},
{0x1637,Lo,0x1637,0x1637,0x1637},
{0x1638,Lo,0x1638,0x1638,0x1638},
{0x1639,Lo,0x1639,0x1639,0x1639},
{0x163A,Lo,0x163A,0x163A,0x163A},
{0x163B,Lo,0x163B,0x163B,0x163B},
{0x163C,Lo,0x163C,0x163C,0x163C},
{0x163D,Lo,0x163D,0x163D,0x163D},
{0x163E,Lo,0x163E,0x163E,0x163E},
{0x163F,Lo,0x163F,0x163F,0x163F},
{0x1640,Lo,0x1640,0x1640,0x1640},
{0x1641,Lo,0x1641,0x1641,0x1641},
{0x1642,Lo,0x1642,0x1642,0x1642},
{0x1643,Lo,0x1643,0x1643,0x1643},
{0x1644,Lo,0x1644,0x1644,0x1644},
{0x1645,Lo,0x1645,0x1645,0x1645},
{0x1646,Lo,0x1646,0x1646,0x1646},
{0x1647,Lo,0x1647,0x1647,0x1647},
{0x1648,Lo,0x1648,0x1648,0x1648},
{0x1649,Lo,0x1649,0x1649,0x1649},
{0x164A,Lo,0x164A,0x164A,0x164A},
{0x164B,Lo,0x164B,0x164B,0x164B},
{0x164C,Lo,0x164C,0x164C,0x164C},
{0x164D,Lo,0x164D,0x164D,0x164D},
{0x164E,Lo,0x164E,0x164E,0x164E},
{0x164F,Lo,0x164F,0x164F,0x164F},
{0x1650,Lo,0x1650,0x1650,0x1650},
{0x1651,Lo,0x1651,0x1651,0x1651},
{0x1652,Lo,0x1652,0x1652,0x1652},
{0x1653,Lo,0x1653,0x1653,0x1653},
{0x1654,Lo,0x1654,0x1654,0x1654},
{0x1655,Lo,0x1655,0x1655,0x1655},
{0x1656,Lo,0x1656,0x1656,0x1656},
{0x1657,Lo,0x1657,0x1657,0x1657},
{0x1658,Lo,0x1658,0x1658,0x1658},
{0x1659,Lo,0x1659,0x1659,0x1659},
{0x165A,Lo,0x165A,0x165A,0x165A},
{0x165B,Lo,0x165B,0x165B,0x165B},
{0x165C,Lo,0x165C,0x165C,0x165C},
{0x165D,Lo,0x165D,0x165D,0x165D},
{0x165E,Lo,0x165E,0x165E,0x165E},
{0x165F,Lo,0x165F,0x165F,0x165F},
{0x1660,Lo,0x1660,0x1660,0x1660},
{0x1661,Lo,0x1661,0x1661,0x1661},
{0x1662,Lo,0x1662,0x1662,0x1662},
{0x1663,Lo,0x1663,0x1663,0x1663},
{0x1664,Lo,0x1664,0x1664,0x1664},
{0x1665,Lo,0x1665,0x1665,0x1665},
{0x1666,Lo,0x1666,0x1666,0x1666},
{0x1667,Lo,0x1667,0x1667,0x1667},
{0x1668,Lo,0x1668,0x1668,0x1668},
{0x1669,Lo,0x1669,0x1669,0x1669},
{0x166A,Lo,0x166A,0x166A,0x166A},
{0x166B,Lo,0x166B,0x166B,0x166B},
{0x166C,Lo,0x166C,0x166C,0x166C},
{0x166D,Po,0x166D,0x166D,0x166D},
{0x166E,Po,0x166E,0x166E,0x166E},
{0x166F,Lo,0x166F,0x166F,0x166F},
{0x1670,Lo,0x1670,0x1670,0x1670},
{0x1671,Lo,0x1671,0x1671,0x1671},
{0x1672,Lo,0x1672,0x1672,0x1672},
{0x1673,Lo,0x1673,0x1673,0x1673},
{0x1674,Lo,0x1674,0x1674,0x1674},
{0x1675,Lo,0x1675,0x1675,0x1675},
{0x1676,Lo,0x1676,0x1676,0x1676},
{0x1680,Zs,0x1680,0x1680,0x1680},
{0x1681,Lo,0x1681,0x1681,0x1681},
{0x1682,Lo,0x1682,0x1682,0x1682},
{0x1683,Lo,0x1683,0x1683,0x1683},
{0x1684,Lo,0x1684,0x1684,0x1684},
{0x1685,Lo,0x1685,0x1685,0x1685},
{0x1686,Lo,0x1686,0x1686,0x1686},
{0x1687,Lo,0x1687,0x1687,0x1687},
{0x1688,Lo,0x1688,0x1688,0x1688},
{0x1689,Lo,0x1689,0x1689,0x1689},
{0x168A,Lo,0x168A,0x168A,0x168A},
{0x168B,Lo,0x168B,0x168B,0x168B},
{0x168C,Lo,0x168C,0x168C,0x168C},
{0x168D,Lo,0x168D,0x168D,0x168D},
{0x168E,Lo,0x168E,0x168E,0x168E},
{0x168F,Lo,0x168F,0x168F,0x168F},
{0x1690,Lo,0x1690,0x1690,0x1690},
{0x1691,Lo,0x1691,0x1691,0x1691},
{0x1692,Lo,0x1692,0x1692,0x1692},
{0x1693,Lo,0x1693,0x1693,0x1693},
{0x1694,Lo,0x1694,0x1694,0x1694},
{0x1695,Lo,0x1695,0x1695,0x1695},
{0x1696,Lo,0x1696,0x1696,0x1696},
{0x1697,Lo,0x1697,0x1697,0x1697},
{0x1698,Lo,0x1698,0x1698,0x1698},
{0x1699,Lo,0x1699,0x1699,0x1699},
{0x169A,Lo,0x169A,0x169A,0x169A},
{0x169B,Ps,0x169B,0x169B,0x169B},
{0x169C,Pe,0x169C,0x169C,0x169C},
{0x16A0,Lo,0x16A0,0x16A0,0x16A0},
{0x16A1,Lo,0x16A1,0x16A1,0x16A1},
{0x16A2,Lo,0x16A2,0x16A2,0x16A2},
{0x16A3,Lo,0x16A3,0x16A3,0x16A3},
{0x16A4,Lo,0x16A4,0x16A4,0x16A4},
{0x16A5,Lo,0x16A5,0x16A5,0x16A5},
{0x16A6,Lo,0x16A6,0x16A6,0x16A6},
{0x16A7,Lo,0x16A7,0x16A7,0x16A7},
{0x16A8,Lo,0x16A8,0x16A8,0x16A8},
{0x16A9,Lo,0x16A9,0x16A9,0x16A9},
{0x16AA,Lo,0x16AA,0x16AA,0x16AA},
{0x16AB,Lo,0x16AB,0x16AB,0x16AB},
{0x16AC,Lo,0x16AC,0x16AC,0x16AC},
{0x16AD,Lo,0x16AD,0x16AD,0x16AD},
{0x16AE,Lo,0x16AE,0x16AE,0x16AE},
{0x16AF,Lo,0x16AF,0x16AF,0x16AF},
{0x16B0,Lo,0x16B0,0x16B0,0x16B0},
{0x16B1,Lo,0x16B1,0x16B1,0x16B1},
{0x16B2,Lo,0x16B2,0x16B2,0x16B2},
{0x16B3,Lo,0x16B3,0x16B3,0x16B3},
{0x16B4,Lo,0x16B4,0x16B4,0x16B4},
{0x16B5,Lo,0x16B5,0x16B5,0x16B5},
{0x16B6,Lo,0x16B6,0x16B6,0x16B6},
{0x16B7,Lo,0x16B7,0x16B7,0x16B7},
{0x16B8,Lo,0x16B8,0x16B8,0x16B8},
{0x16B9,Lo,0x16B9,0x16B9,0x16B9},
{0x16BA,Lo,0x16BA,0x16BA,0x16BA},
{0x16BB,Lo,0x16BB,0x16BB,0x16BB},
{0x16BC,Lo,0x16BC,0x16BC,0x16BC},
{0x16BD,Lo,0x16BD,0x16BD,0x16BD},
{0x16BE,Lo,0x16BE,0x16BE,0x16BE},
{0x16BF,Lo,0x16BF,0x16BF,0x16BF},
{0x16C0,Lo,0x16C0,0x16C0,0x16C0},
{0x16C1,Lo,0x16C1,0x16C1,0x16C1},
{0x16C2,Lo,0x16C2,0x16C2,0x16C2},
{0x16C3,Lo,0x16C3,0x16C3,0x16C3},
{0x16C4,Lo,0x16C4,0x16C4,0x16C4},
{0x16C5,Lo,0x16C5,0x16C5,0x16C5},
{0x16C6,Lo,0x16C6,0x16C6,0x16C6},
{0x16C7,Lo,0x16C7,0x16C7,0x16C7},
{0x16C8,Lo,0x16C8,0x16C8,0x16C8},
{0x16C9,Lo,0x16C9,0x16C9,0x16C9},
{0x16CA,Lo,0x16CA,0x16CA,0x16CA},
{0x16CB,Lo,0x16CB,0x16CB,0x16CB},
{0x16CC,Lo,0x16CC,0x16CC,0x16CC},
{0x16CD,Lo,0x16CD,0x16CD,0x16CD},
{0x16CE,Lo,0x16CE,0x16CE,0x16CE},
{0x16CF,Lo,0x16CF,0x16CF,0x16CF},
{0x16D0,Lo,0x16D0,0x16D0,0x16D0},
{0x16D1,Lo,0x16D1,0x16D1,0x16D1},
{0x16D2,Lo,0x16D2,0x16D2,0x16D2},
{0x16D3,Lo,0x16D3,0x16D3,0x16D3},
{0x16D4,Lo,0x16D4,0x16D4,0x16D4},
{0x16D5,Lo,0x16D5,0x16D5,0x16D5},
{0x16D6,Lo,0x16D6,0x16D6,0x16D6},
{0x16D7,Lo,0x16D7,0x16D7,0x16D7},
{0x16D8,Lo,0x16D8,0x16D8,0x16D8},
{0x16D9,Lo,0x16D9,0x16D9,0x16D9},
{0x16DA,Lo,0x16DA,0x16DA,0x16DA},
{0x16DB,Lo,0x16DB,0x16DB,0x16DB},
{0x16DC,Lo,0x16DC,0x16DC,0x16DC},
{0x16DD,Lo,0x16DD,0x16DD,0x16DD},
{0x16DE,Lo,0x16DE,0x16DE,0x16DE},
{0x16DF,Lo,0x16DF,0x16DF,0x16DF},
{0x16E0,Lo,0x16E0,0x16E0,0x16E0},
{0x16E1,Lo,0x16E1,0x16E1,0x16E1},
{0x16E2,Lo,0x16E2,0x16E2,0x16E2},
{0x16E3,Lo,0x16E3,0x16E3,0x16E3},
{0x16E4,Lo,0x16E4,0x16E4,0x16E4},
{0x16E5,Lo,0x16E5,0x16E5,0x16E5},
{0x16E6,Lo,0x16E6,0x16E6,0x16E6},
{0x16E7,Lo,0x16E7,0x16E7,0x16E7},
{0x16E8,Lo,0x16E8,0x16E8,0x16E8},
{0x16E9,Lo,0x16E9,0x16E9,0x16E9},
{0x16EA,Lo,0x16EA,0x16EA,0x16EA},
{0x16EB,Po,0x16EB,0x16EB,0x16EB},
{0x16EC,Po,0x16EC,0x16EC,0x16EC},
{0x16ED,Po,0x16ED,0x16ED,0x16ED},
{0x16EE,Nl,0x16EE,0x16EE,0x16EE},
{0x16EF,Nl,0x16EF,0x16EF,0x16EF},
{0x16F0,Nl,0x16F0,0x16F0,0x16F0},
{0x1700,Lo,0x1700,0x1700,0x1700},
{0x1701,Lo,0x1701,0x1701,0x1701},
{0x1702,Lo,0x1702,0x1702,0x1702},
{0x1703,Lo,0x1703,0x1703,0x1703},
{0x1704,Lo,0x1704,0x1704,0x1704},
{0x1705,Lo,0x1705,0x1705,0x1705},
{0x1706,Lo,0x1706,0x1706,0x1706},
{0x1707,Lo,0x1707,0x1707,0x1707},
{0x1708,Lo,0x1708,0x1708,0x1708},
{0x1709,Lo,0x1709,0x1709,0x1709},
{0x170A,Lo,0x170A,0x170A,0x170A},
{0x170B,Lo,0x170B,0x170B,0x170B},
{0x170C,Lo,0x170C,0x170C,0x170C},
{0x170E,Lo,0x170E,0x170E,0x170E},
{0x170F,Lo,0x170F,0x170F,0x170F},
{0x1710,Lo,0x1710,0x1710,0x1710},
{0x1711,Lo,0x1711,0x1711,0x1711},
{0x1712,Mn,0x1712,0x1712,0x1712},
{0x1713,Mn,0x1713,0x1713,0x1713},
{0x1714,Mn,0x1714,0x1714,0x1714},
{0x1720,Lo,0x1720,0x1720,0x1720},
{0x1721,Lo,0x1721,0x1721,0x1721},
{0x1722,Lo,0x1722,0x1722,0x1722},
{0x1723,Lo,0x1723,0x1723,0x1723},
{0x1724,Lo,0x1724,0x1724,0x1724},
{0x1725,Lo,0x1725,0x1725,0x1725},
{0x1726,Lo,0x1726,0x1726,0x1726},
{0x1727,Lo,0x1727,0x1727,0x1727},
{0x1728,Lo,0x1728,0x1728,0x1728},
{0x1729,Lo,0x1729,0x1729,0x1729},
{0x172A,Lo,0x172A,0x172A,0x172A},
{0x172B,Lo,0x172B,0x172B,0x172B},
{0x172C,Lo,0x172C,0x172C,0x172C},
{0x172D,Lo,0x172D,0x172D,0x172D},
{0x172E,Lo,0x172E,0x172E,0x172E},
{0x172F,Lo,0x172F,0x172F,0x172F},
{0x1730,Lo,0x1730,0x1730,0x1730},
{0x1731,Lo,0x1731,0x1731,0x1731},
{0x1732,Mn,0x1732,0x1732,0x1732},
{0x1733,Mn,0x1733,0x1733,0x1733},
{0x1734,Mn,0x1734,0x1734,0x1734},
{0x1735,Po,0x1735,0x1735,0x1735},
{0x1736,Po,0x1736,0x1736,0x1736},
{0x1740,Lo,0x1740,0x1740,0x1740},
{0x1741,Lo,0x1741,0x1741,0x1741},
{0x1742,Lo,0x1742,0x1742,0x1742},
{0x1743,Lo,0x1743,0x1743,0x1743},
{0x1744,Lo,0x1744,0x1744,0x1744},
{0x1745,Lo,0x1745,0x1745,0x1745},
{0x1746,Lo,0x1746,0x1746,0x1746},
{0x1747,Lo,0x1747,0x1747,0x1747},
{0x1748,Lo,0x1748,0x1748,0x1748},
{0x1749,Lo,0x1749,0x1749,0x1749},
{0x174A,Lo,0x174A,0x174A,0x174A},
{0x174B,Lo,0x174B,0x174B,0x174B},
{0x174C,Lo,0x174C,0x174C,0x174C},
{0x174D,Lo,0x174D,0x174D,0x174D},
{0x174E,Lo,0x174E,0x174E,0x174E},
{0x174F,Lo,0x174F,0x174F,0x174F},
{0x1750,Lo,0x1750,0x1750,0x1750},
{0x1751,Lo,0x1751,0x1751,0x1751},
{0x1752,Mn,0x1752,0x1752,0x1752},
{0x1753,Mn,0x1753,0x1753,0x1753},
{0x1760,Lo,0x1760,0x1760,0x1760},
{0x1761,Lo,0x1761,0x1761,0x1761},
{0x1762,Lo,0x1762,0x1762,0x1762},
{0x1763,Lo,0x1763,0x1763,0x1763},
{0x1764,Lo,0x1764,0x1764,0x1764},
{0x1765,Lo,0x1765,0x1765,0x1765},
{0x1766,Lo,0x1766,0x1766,0x1766},
{0x1767,Lo,0x1767,0x1767,0x1767},
{0x1768,Lo,0x1768,0x1768,0x1768},
{0x1769,Lo,0x1769,0x1769,0x1769},
{0x176A,Lo,0x176A,0x176A,0x176A},
{0x176B,Lo,0x176B,0x176B,0x176B},
{0x176C,Lo,0x176C,0x176C,0x176C},
{0x176E,Lo,0x176E,0x176E,0x176E},
{0x176F,Lo,0x176F,0x176F,0x176F},
{0x1770,Lo,0x1770,0x1770,0x1770},
{0x1772,Mn,0x1772,0x1772,0x1772},
{0x1773,Mn,0x1773,0x1773,0x1773},
{0x1780,Lo,0x1780,0x1780,0x1780},
{0x1781,Lo,0x1781,0x1781,0x1781},
{0x1782,Lo,0x1782,0x1782,0x1782},
{0x1783,Lo,0x1783,0x1783,0x1783},
{0x1784,Lo,0x1784,0x1784,0x1784},
{0x1785,Lo,0x1785,0x1785,0x1785},
{0x1786,Lo,0x1786,0x1786,0x1786},
{0x1787,Lo,0x1787,0x1787,0x1787},
{0x1788,Lo,0x1788,0x1788,0x1788},
{0x1789,Lo,0x1789,0x1789,0x1789},
{0x178A,Lo,0x178A,0x178A,0x178A},
{0x178B,Lo,0x178B,0x178B,0x178B},
{0x178C,Lo,0x178C,0x178C,0x178C},
{0x178D,Lo,0x178D,0x178D,0x178D},
{0x178E,Lo,0x178E,0x178E,0x178E},
{0x178F,Lo,0x178F,0x178F,0x178F},
{0x1790,Lo,0x1790,0x1790,0x1790},
{0x1791,Lo,0x1791,0x1791,0x1791},
{0x1792,Lo,0x1792,0x1792,0x1792},
{0x1793,Lo,0x1793,0x1793,0x1793},
{0x1794,Lo,0x1794,0x1794,0x1794},
{0x1795,Lo,0x1795,0x1795,0x1795},
{0x1796,Lo,0x1796,0x1796,0x1796},
{0x1797,Lo,0x1797,0x1797,0x1797},
{0x1798,Lo,0x1798,0x1798,0x1798},
{0x1799,Lo,0x1799,0x1799,0x1799},
{0x179A,Lo,0x179A,0x179A,0x179A},
{0x179B,Lo,0x179B,0x179B,0x179B},
{0x179C,Lo,0x179C,0x179C,0x179C},
{0x179D,Lo,0x179D,0x179D,0x179D},
{0x179E,Lo,0x179E,0x179E,0x179E},
{0x179F,Lo,0x179F,0x179F,0x179F},
{0x17A0,Lo,0x17A0,0x17A0,0x17A0},
{0x17A1,Lo,0x17A1,0x17A1,0x17A1},
{0x17A2,Lo,0x17A2,0x17A2,0x17A2},
{0x17A3,Lo,0x17A3,0x17A3,0x17A3},
{0x17A4,Lo,0x17A4,0x17A4,0x17A4},
{0x17A5,Lo,0x17A5,0x17A5,0x17A5},
{0x17A6,Lo,0x17A6,0x17A6,0x17A6},
{0x17A7,Lo,0x17A7,0x17A7,0x17A7},
{0x17A8,Lo,0x17A8,0x17A8,0x17A8},
{0x17A9,Lo,0x17A9,0x17A9,0x17A9},
{0x17AA,Lo,0x17AA,0x17AA,0x17AA},
{0x17AB,Lo,0x17AB,0x17AB,0x17AB},
{0x17AC,Lo,0x17AC,0x17AC,0x17AC},
{0x17AD,Lo,0x17AD,0x17AD,0x17AD},
{0x17AE,Lo,0x17AE,0x17AE,0x17AE},
{0x17AF,Lo,0x17AF,0x17AF,0x17AF},
{0x17B0,Lo,0x17B0,0x17B0,0x17B0},
{0x17B1,Lo,0x17B1,0x17B1,0x17B1},
{0x17B2,Lo,0x17B2,0x17B2,0x17B2},
{0x17B3,Lo,0x17B3,0x17B3,0x17B3},
{0x17B4,Cf,0x17B4,0x17B4,0x17B4},
{0x17B5,Cf,0x17B5,0x17B5,0x17B5},
{0x17B6,Mc,0x17B6,0x17B6,0x17B6},
{0x17B7,Mn,0x17B7,0x17B7,0x17B7},
{0x17B8,Mn,0x17B8,0x17B8,0x17B8},
{0x17B9,Mn,0x17B9,0x17B9,0x17B9},
{0x17BA,Mn,0x17BA,0x17BA,0x17BA},
{0x17BB,Mn,0x17BB,0x17BB,0x17BB},
{0x17BC,Mn,0x17BC,0x17BC,0x17BC},
{0x17BD,Mn,0x17BD,0x17BD,0x17BD},
{0x17BE,Mc,0x17BE,0x17BE,0x17BE},
{0x17BF,Mc,0x17BF,0x17BF,0x17BF},
{0x17C0,Mc,0x17C0,0x17C0,0x17C0},
{0x17C1,Mc,0x17C1,0x17C1,0x17C1},
{0x17C2,Mc,0x17C2,0x17C2,0x17C2},
{0x17C3,Mc,0x17C3,0x17C3,0x17C3},
{0x17C4,Mc,0x17C4,0x17C4,0x17C4},
{0x17C5,Mc,0x17C5,0x17C5,0x17C5},
{0x17C6,Mn,0x17C6,0x17C6,0x17C6},
{0x17C7,Mc,0x17C7,0x17C7,0x17C7},
{0x17C8,Mc,0x17C8,0x17C8,0x17C8},
{0x17C9,Mn,0x17C9,0x17C9,0x17C9},
{0x17CA,Mn,0x17CA,0x17CA,0x17CA},
{0x17CB,Mn,0x17CB,0x17CB,0x17CB},
{0x17CC,Mn,0x17CC,0x17CC,0x17CC},
{0x17CD,Mn,0x17CD,0x17CD,0x17CD},
{0x17CE,Mn,0x17CE,0x17CE,0x17CE},
{0x17CF,Mn,0x17CF,0x17CF,0x17CF},
{0x17D0,Mn,0x17D0,0x17D0,0x17D0},
{0x17D1,Mn,0x17D1,0x17D1,0x17D1},
{0x17D2,Mn,0x17D2,0x17D2,0x17D2},
{0x17D3,Mn,0x17D3,0x17D3,0x17D3},
{0x17D4,Po,0x17D4,0x17D4,0x17D4},
{0x17D5,Po,0x17D5,0x17D5,0x17D5},
{0x17D6,Po,0x17D6,0x17D6,0x17D6},
{0x17D7,Lm,0x17D7,0x17D7,0x17D7},
{0x17D8,Po,0x17D8,0x17D8,0x17D8},
{0x17D9,Po,0x17D9,0x17D9,0x17D9},
{0x17DA,Po,0x17DA,0x17DA,0x17DA},
{0x17DB,Sc,0x17DB,0x17DB,0x17DB},
{0x17DC,Lo,0x17DC,0x17DC,0x17DC},
{0x17DD,Mn,0x17DD,0x17DD,0x17DD},
{0x17E0,Nd,0x17E0,0x17E0,0x17E0},
{0x17E1,Nd,0x17E1,0x17E1,0x17E1},
{0x17E2,Nd,0x17E2,0x17E2,0x17E2},
{0x17E3,Nd,0x17E3,0x17E3,0x17E3},
{0x17E4,Nd,0x17E4,0x17E4,0x17E4},
{0x17E5,Nd,0x17E5,0x17E5,0x17E5},
{0x17E6,Nd,0x17E6,0x17E6,0x17E6},
{0x17E7,Nd,0x17E7,0x17E7,0x17E7},
{0x17E8,Nd,0x17E8,0x17E8,0x17E8},
{0x17E9,Nd,0x17E9,0x17E9,0x17E9},
{0x17F0,No,0x17F0,0x17F0,0x17F0},
{0x17F1,No,0x17F1,0x17F1,0x17F1},
{0x17F2,No,0x17F2,0x17F2,0x17F2},
{0x17F3,No,0x17F3,0x17F3,0x17F3},
{0x17F4,No,0x17F4,0x17F4,0x17F4},
{0x17F5,No,0x17F5,0x17F5,0x17F5},
{0x17F6,No,0x17F6,0x17F6,0x17F6},
{0x17F7,No,0x17F7,0x17F7,0x17F7},
{0x17F8,No,0x17F8,0x17F8,0x17F8},
{0x17F9,No,0x17F9,0x17F9,0x17F9},
{0x1800,Po,0x1800,0x1800,0x1800},
{0x1801,Po,0x1801,0x1801,0x1801},
{0x1802,Po,0x1802,0x1802,0x1802},
{0x1803,Po,0x1803,0x1803,0x1803},
{0x1804,Po,0x1804,0x1804,0x1804},
{0x1805,Po,0x1805,0x1805,0x1805},
{0x1806,Pd,0x1806,0x1806,0x1806},
{0x1807,Po,0x1807,0x1807,0x1807},
{0x1808,Po,0x1808,0x1808,0x1808},
{0x1809,Po,0x1809,0x1809,0x1809},
{0x180A,Po,0x180A,0x180A,0x180A},
{0x180B,Mn,0x180B,0x180B,0x180B},
{0x180C,Mn,0x180C,0x180C,0x180C},
{0x180D,Mn,0x180D,0x180D,0x180D},
{0x180E,Zs,0x180E,0x180E,0x180E},
{0x1810,Nd,0x1810,0x1810,0x1810},
{0x1811,Nd,0x1811,0x1811,0x1811},
{0x1812,Nd,0x1812,0x1812,0x1812},
{0x1813,Nd,0x1813,0x1813,0x1813},
{0x1814,Nd,0x1814,0x1814,0x1814},
{0x1815,Nd,0x1815,0x1815,0x1815},
{0x1816,Nd,0x1816,0x1816,0x1816},
{0x1817,Nd,0x1817,0x1817,0x1817},
{0x1818,Nd,0x1818,0x1818,0x1818},
{0x1819,Nd,0x1819,0x1819,0x1819},
{0x1820,Lo,0x1820,0x1820,0x1820},
{0x1821,Lo,0x1821,0x1821,0x1821},
{0x1822,Lo,0x1822,0x1822,0x1822},
{0x1823,Lo,0x1823,0x1823,0x1823},
{0x1824,Lo,0x1824,0x1824,0x1824},
{0x1825,Lo,0x1825,0x1825,0x1825},
{0x1826,Lo,0x1826,0x1826,0x1826},
{0x1827,Lo,0x1827,0x1827,0x1827},
{0x1828,Lo,0x1828,0x1828,0x1828},
{0x1829,Lo,0x1829,0x1829,0x1829},
{0x182A,Lo,0x182A,0x182A,0x182A},
{0x182B,Lo,0x182B,0x182B,0x182B},
{0x182C,Lo,0x182C,0x182C,0x182C},
{0x182D,Lo,0x182D,0x182D,0x182D},
{0x182E,Lo,0x182E,0x182E,0x182E},
{0x182F,Lo,0x182F,0x182F,0x182F},
{0x1830,Lo,0x1830,0x1830,0x1830},
{0x1831,Lo,0x1831,0x1831,0x1831},
{0x1832,Lo,0x1832,0x1832,0x1832},
{0x1833,Lo,0x1833,0x1833,0x1833},
{0x1834,Lo,0x1834,0x1834,0x1834},
{0x1835,Lo,0x1835,0x1835,0x1835},
{0x1836,Lo,0x1836,0x1836,0x1836},
{0x1837,Lo,0x1837,0x1837,0x1837},
{0x1838,Lo,0x1838,0x1838,0x1838},
{0x1839,Lo,0x1839,0x1839,0x1839},
{0x183A,Lo,0x183A,0x183A,0x183A},
{0x183B,Lo,0x183B,0x183B,0x183B},
{0x183C,Lo,0x183C,0x183C,0x183C},
{0x183D,Lo,0x183D,0x183D,0x183D},
{0x183E,Lo,0x183E,0x183E,0x183E},
{0x183F,Lo,0x183F,0x183F,0x183F},
{0x1840,Lo,0x1840,0x1840,0x1840},
{0x1841,Lo,0x1841,0x1841,0x1841},
{0x1842,Lo,0x1842,0x1842,0x1842},
{0x1843,Lm,0x1843,0x1843,0x1843},
{0x1844,Lo,0x1844,0x1844,0x1844},
{0x1845,Lo,0x1845,0x1845,0x1845},
{0x1846,Lo,0x1846,0x1846,0x1846},
{0x1847,Lo,0x1847,0x1847,0x1847},
{0x1848,Lo,0x1848,0x1848,0x1848},
{0x1849,Lo,0x1849,0x1849,0x1849},
{0x184A,Lo,0x184A,0x184A,0x184A},
{0x184B,Lo,0x184B,0x184B,0x184B},
{0x184C,Lo,0x184C,0x184C,0x184C},
{0x184D,Lo,0x184D,0x184D,0x184D},
{0x184E,Lo,0x184E,0x184E,0x184E},
{0x184F,Lo,0x184F,0x184F,0x184F},
{0x1850,Lo,0x1850,0x1850,0x1850},
{0x1851,Lo,0x1851,0x1851,0x1851},
{0x1852,Lo,0x1852,0x1852,0x1852},
{0x1853,Lo,0x1853,0x1853,0x1853},
{0x1854,Lo,0x1854,0x1854,0x1854},
{0x1855,Lo,0x1855,0x1855,0x1855},
{0x1856,Lo,0x1856,0x1856,0x1856},
{0x1857,Lo,0x1857,0x1857,0x1857},
{0x1858,Lo,0x1858,0x1858,0x1858},
{0x1859,Lo,0x1859,0x1859,0x1859},
{0x185A,Lo,0x185A,0x185A,0x185A},
{0x185B,Lo,0x185B,0x185B,0x185B},
{0x185C,Lo,0x185C,0x185C,0x185C},
{0x185D,Lo,0x185D,0x185D,0x185D},
{0x185E,Lo,0x185E,0x185E,0x185E},
{0x185F,Lo,0x185F,0x185F,0x185F},
{0x1860,Lo,0x1860,0x1860,0x1860},
{0x1861,Lo,0x1861,0x1861,0x1861},
{0x1862,Lo,0x1862,0x1862,0x1862},
{0x1863,Lo,0x1863,0x1863,0x1863},
{0x1864,Lo,0x1864,0x1864,0x1864},
{0x1865,Lo,0x1865,0x1865,0x1865},
{0x1866,Lo,0x1866,0x1866,0x1866},
{0x1867,Lo,0x1867,0x1867,0x1867},
{0x1868,Lo,0x1868,0x1868,0x1868},
{0x1869,Lo,0x1869,0x1869,0x1869},
{0x186A,Lo,0x186A,0x186A,0x186A},
{0x186B,Lo,0x186B,0x186B,0x186B},
{0x186C,Lo,0x186C,0x186C,0x186C},
{0x186D,Lo,0x186D,0x186D,0x186D},
{0x186E,Lo,0x186E,0x186E,0x186E},
{0x186F,Lo,0x186F,0x186F,0x186F},
{0x1870,Lo,0x1870,0x1870,0x1870},
{0x1871,Lo,0x1871,0x1871,0x1871},
{0x1872,Lo,0x1872,0x1872,0x1872},
{0x1873,Lo,0x1873,0x1873,0x1873},
{0x1874,Lo,0x1874,0x1874,0x1874},
{0x1875,Lo,0x1875,0x1875,0x1875},
{0x1876,Lo,0x1876,0x1876,0x1876},
{0x1877,Lo,0x1877,0x1877,0x1877},
{0x1880,Lo,0x1880,0x1880,0x1880},
{0x1881,Lo,0x1881,0x1881,0x1881},
{0x1882,Lo,0x1882,0x1882,0x1882},
{0x1883,Lo,0x1883,0x1883,0x1883},
{0x1884,Lo,0x1884,0x1884,0x1884},
{0x1885,Lo,0x1885,0x1885,0x1885},
{0x1886,Lo,0x1886,0x1886,0x1886},
{0x1887,Lo,0x1887,0x1887,0x1887},
{0x1888,Lo,0x1888,0x1888,0x1888},
{0x1889,Lo,0x1889,0x1889,0x1889},
{0x188A,Lo,0x188A,0x188A,0x188A},
{0x188B,Lo,0x188B,0x188B,0x188B},
{0x188C,Lo,0x188C,0x188C,0x188C},
{0x188D,Lo,0x188D,0x188D,0x188D},
{0x188E,Lo,0x188E,0x188E,0x188E},
{0x188F,Lo,0x188F,0x188F,0x188F},
{0x1890,Lo,0x1890,0x1890,0x1890},
{0x1891,Lo,0x1891,0x1891,0x1891},
{0x1892,Lo,0x1892,0x1892,0x1892},
{0x1893,Lo,0x1893,0x1893,0x1893},
{0x1894,Lo,0x1894,0x1894,0x1894},
{0x1895,Lo,0x1895,0x1895,0x1895},
{0x1896,Lo,0x1896,0x1896,0x1896},
{0x1897,Lo,0x1897,0x1897,0x1897},
{0x1898,Lo,0x1898,0x1898,0x1898},
{0x1899,Lo,0x1899,0x1899,0x1899},
{0x189A,Lo,0x189A,0x189A,0x189A},
{0x189B,Lo,0x189B,0x189B,0x189B},
{0x189C,Lo,0x189C,0x189C,0x189C},
{0x189D,Lo,0x189D,0x189D,0x189D},
{0x189E,Lo,0x189E,0x189E,0x189E},
{0x189F,Lo,0x189F,0x189F,0x189F},
{0x18A0,Lo,0x18A0,0x18A0,0x18A0},
{0x18A1,Lo,0x18A1,0x18A1,0x18A1},
{0x18A2,Lo,0x18A2,0x18A2,0x18A2},
{0x18A3,Lo,0x18A3,0x18A3,0x18A3},
{0x18A4,Lo,0x18A4,0x18A4,0x18A4},
{0x18A5,Lo,0x18A5,0x18A5,0x18A5},
{0x18A6,Lo,0x18A6,0x18A6,0x18A6},
{0x18A7,Lo,0x18A7,0x18A7,0x18A7},
{0x18A8,Lo,0x18A8,0x18A8,0x18A8},
{0x18A9,Mn,0x18A9,0x18A9,0x18A9},
{0x1900,Lo,0x1900,0x1900,0x1900},
{0x1901,Lo,0x1901,0x1901,0x1901},
{0x1902,Lo,0x1902,0x1902,0x1902},
{0x1903,Lo,0x1903,0x1903,0x1903},
{0x1904,Lo,0x1904,0x1904,0x1904},
{0x1905,Lo,0x1905,0x1905,0x1905},
{0x1906,Lo,0x1906,0x1906,0x1906},
{0x1907,Lo,0x1907,0x1907,0x1907},
{0x1908,Lo,0x1908,0x1908,0x1908},
{0x1909,Lo,0x1909,0x1909,0x1909},
{0x190A,Lo,0x190A,0x190A,0x190A},
{0x190B,Lo,0x190B,0x190B,0x190B},
{0x190C,Lo,0x190C,0x190C,0x190C},
{0x190D,Lo,0x190D,0x190D,0x190D},
{0x190E,Lo,0x190E,0x190E,0x190E},
{0x190F,Lo,0x190F,0x190F,0x190F},
{0x1910,Lo,0x1910,0x1910,0x1910},
{0x1911,Lo,0x1911,0x1911,0x1911},
{0x1912,Lo,0x1912,0x1912,0x1912},
{0x1913,Lo,0x1913,0x1913,0x1913},
{0x1914,Lo,0x1914,0x1914,0x1914},
{0x1915,Lo,0x1915,0x1915,0x1915},
{0x1916,Lo,0x1916,0x1916,0x1916},
{0x1917,Lo,0x1917,0x1917,0x1917},
{0x1918,Lo,0x1918,0x1918,0x1918},
{0x1919,Lo,0x1919,0x1919,0x1919},
{0x191A,Lo,0x191A,0x191A,0x191A},
{0x191B,Lo,0x191B,0x191B,0x191B},
{0x191C,Lo,0x191C,0x191C,0x191C},
{0x1920,Mn,0x1920,0x1920,0x1920},
{0x1921,Mn,0x1921,0x1921,0x1921},
{0x1922,Mn,0x1922,0x1922,0x1922},
{0x1923,Mc,0x1923,0x1923,0x1923},
{0x1924,Mc,0x1924,0x1924,0x1924},
{0x1925,Mc,0x1925,0x1925,0x1925},
{0x1926,Mc,0x1926,0x1926,0x1926},
{0x1927,Mn,0x1927,0x1927,0x1927},
{0x1928,Mn,0x1928,0x1928,0x1928},
{0x1929,Mc,0x1929,0x1929,0x1929},
{0x192A,Mc,0x192A,0x192A,0x192A},
{0x192B,Mc,0x192B,0x192B,0x192B},
{0x1930,Mc,0x1930,0x1930,0x1930},
{0x1931,Mc,0x1931,0x1931,0x1931},
{0x1932,Mn,0x1932,0x1932,0x1932},
{0x1933,Mc,0x1933,0x1933,0x1933},
{0x1934,Mc,0x1934,0x1934,0x1934},
{0x1935,Mc,0x1935,0x1935,0x1935},
{0x1936,Mc,0x1936,0x1936,0x1936},
{0x1937,Mc,0x1937,0x1937,0x1937},
{0x1938,Mc,0x1938,0x1938,0x1938},
{0x1939,Mn,0x1939,0x1939,0x1939},
{0x193A,Mn,0x193A,0x193A,0x193A},
{0x193B,Mn,0x193B,0x193B,0x193B},
{0x1940,So,0x1940,0x1940,0x1940},
{0x1944,Po,0x1944,0x1944,0x1944},
{0x1945,Po,0x1945,0x1945,0x1945},
{0x1946,Nd,0x1946,0x1946,0x1946},
{0x1947,Nd,0x1947,0x1947,0x1947},
{0x1948,Nd,0x1948,0x1948,0x1948},
{0x1949,Nd,0x1949,0x1949,0x1949},
{0x194A,Nd,0x194A,0x194A,0x194A},
{0x194B,Nd,0x194B,0x194B,0x194B},
{0x194C,Nd,0x194C,0x194C,0x194C},
{0x194D,Nd,0x194D,0x194D,0x194D},
{0x194E,Nd,0x194E,0x194E,0x194E},
{0x194F,Nd,0x194F,0x194F,0x194F},
{0x1950,Lo,0x1950,0x1950,0x1950},
{0x1951,Lo,0x1951,0x1951,0x1951},
{0x1952,Lo,0x1952,0x1952,0x1952},
{0x1953,Lo,0x1953,0x1953,0x1953},
{0x1954,Lo,0x1954,0x1954,0x1954},
{0x1955,Lo,0x1955,0x1955,0x1955},
{0x1956,Lo,0x1956,0x1956,0x1956},
{0x1957,Lo,0x1957,0x1957,0x1957},
{0x1958,Lo,0x1958,0x1958,0x1958},
{0x1959,Lo,0x1959,0x1959,0x1959},
{0x195A,Lo,0x195A,0x195A,0x195A},
{0x195B,Lo,0x195B,0x195B,0x195B},
{0x195C,Lo,0x195C,0x195C,0x195C},
{0x195D,Lo,0x195D,0x195D,0x195D},
{0x195E,Lo,0x195E,0x195E,0x195E},
{0x195F,Lo,0x195F,0x195F,0x195F},
{0x1960,Lo,0x1960,0x1960,0x1960},
{0x1961,Lo,0x1961,0x1961,0x1961},
{0x1962,Lo,0x1962,0x1962,0x1962},
{0x1963,Lo,0x1963,0x1963,0x1963},
{0x1964,Lo,0x1964,0x1964,0x1964},
{0x1965,Lo,0x1965,0x1965,0x1965},
{0x1966,Lo,0x1966,0x1966,0x1966},
{0x1967,Lo,0x1967,0x1967,0x1967},
{0x1968,Lo,0x1968,0x1968,0x1968},
{0x1969,Lo,0x1969,0x1969,0x1969},
{0x196A,Lo,0x196A,0x196A,0x196A},
{0x196B,Lo,0x196B,0x196B,0x196B},
{0x196C,Lo,0x196C,0x196C,0x196C},
{0x196D,Lo,0x196D,0x196D,0x196D},
{0x1970,Lo,0x1970,0x1970,0x1970},
{0x1971,Lo,0x1971,0x1971,0x1971},
{0x1972,Lo,0x1972,0x1972,0x1972},
{0x1973,Lo,0x1973,0x1973,0x1973},
{0x1974,Lo,0x1974,0x1974,0x1974},
{0x1980,Lo,0x1980,0x1980,0x1980},
{0x1981,Lo,0x1981,0x1981,0x1981},
{0x1982,Lo,0x1982,0x1982,0x1982},
{0x1983,Lo,0x1983,0x1983,0x1983},
{0x1984,Lo,0x1984,0x1984,0x1984},
{0x1985,Lo,0x1985,0x1985,0x1985},
{0x1986,Lo,0x1986,0x1986,0x1986},
{0x1987,Lo,0x1987,0x1987,0x1987},
{0x1988,Lo,0x1988,0x1988,0x1988},
{0x1989,Lo,0x1989,0x1989,0x1989},
{0x198A,Lo,0x198A,0x198A,0x198A},
{0x198B,Lo,0x198B,0x198B,0x198B},
{0x198C,Lo,0x198C,0x198C,0x198C},
{0x198D,Lo,0x198D,0x198D,0x198D},
{0x198E,Lo,0x198E,0x198E,0x198E},
{0x198F,Lo,0x198F,0x198F,0x198F},
{0x1990,Lo,0x1990,0x1990,0x1990},
{0x1991,Lo,0x1991,0x1991,0x1991},
{0x1992,Lo,0x1992,0x1992,0x1992},
{0x1993,Lo,0x1993,0x1993,0x1993},
{0x1994,Lo,0x1994,0x1994,0x1994},
{0x1995,Lo,0x1995,0x1995,0x1995},
{0x1996,Lo,0x1996,0x1996,0x1996},
{0x1997,Lo,0x1997,0x1997,0x1997},
{0x1998,Lo,0x1998,0x1998,0x1998},
{0x1999,Lo,0x1999,0x1999,0x1999},
{0x199A,Lo,0x199A,0x199A,0x199A},
{0x199B,Lo,0x199B,0x199B,0x199B},
{0x199C,Lo,0x199C,0x199C,0x199C},
{0x199D,Lo,0x199D,0x199D,0x199D},
{0x199E,Lo,0x199E,0x199E,0x199E},
{0x199F,Lo,0x199F,0x199F,0x199F},
{0x19A0,Lo,0x19A0,0x19A0,0x19A0},
{0x19A1,Lo,0x19A1,0x19A1,0x19A1},
{0x19A2,Lo,0x19A2,0x19A2,0x19A2},
{0x19A3,Lo,0x19A3,0x19A3,0x19A3},
{0x19A4,Lo,0x19A4,0x19A4,0x19A4},
{0x19A5,Lo,0x19A5,0x19A5,0x19A5},
{0x19A6,Lo,0x19A6,0x19A6,0x19A6},
{0x19A7,Lo,0x19A7,0x19A7,0x19A7},
{0x19A8,Lo,0x19A8,0x19A8,0x19A8},
{0x19A9,Lo,0x19A9,0x19A9,0x19A9},
{0x19B0,Mc,0x19B0,0x19B0,0x19B0},
{0x19B1,Mc,0x19B1,0x19B1,0x19B1},
{0x19B2,Mc,0x19B2,0x19B2,0x19B2},
{0x19B3,Mc,0x19B3,0x19B3,0x19B3},
{0x19B4,Mc,0x19B4,0x19B4,0x19B4},
{0x19B5,Mc,0x19B5,0x19B5,0x19B5},
{0x19B6,Mc,0x19B6,0x19B6,0x19B6},
{0x19B7,Mc,0x19B7,0x19B7,0x19B7},
{0x19B8,Mc,0x19B8,0x19B8,0x19B8},
{0x19B9,Mc,0x19B9,0x19B9,0x19B9},
{0x19BA,Mc,0x19BA,0x19BA,0x19BA},
{0x19BB,Mc,0x19BB,0x19BB,0x19BB},
{0x19BC,Mc,0x19BC,0x19BC,0x19BC},
{0x19BD,Mc,0x19BD,0x19BD,0x19BD},
{0x19BE,Mc,0x19BE,0x19BE,0x19BE},
{0x19BF,Mc,0x19BF,0x19BF,0x19BF},
{0x19C0,Mc,0x19C0,0x19C0,0x19C0},
{0x19C1,Lo,0x19C1,0x19C1,0x19C1},
{0x19C2,Lo,0x19C2,0x19C2,0x19C2},
{0x19C3,Lo,0x19C3,0x19C3,0x19C3},
{0x19C4,Lo,0x19C4,0x19C4,0x19C4},
{0x19C5,Lo,0x19C5,0x19C5,0x19C5},
{0x19C6,Lo,0x19C6,0x19C6,0x19C6},
{0x19C7,Lo,0x19C7,0x19C7,0x19C7},
{0x19C8,Mc,0x19C8,0x19C8,0x19C8},
{0x19C9,Mc,0x19C9,0x19C9,0x19C9},
{0x19D0,Nd,0x19D0,0x19D0,0x19D0},
{0x19D1,Nd,0x19D1,0x19D1,0x19D1},
{0x19D2,Nd,0x19D2,0x19D2,0x19D2},
{0x19D3,Nd,0x19D3,0x19D3,0x19D3},
{0x19D4,Nd,0x19D4,0x19D4,0x19D4},
{0x19D5,Nd,0x19D5,0x19D5,0x19D5},
{0x19D6,Nd,0x19D6,0x19D6,0x19D6},
{0x19D7,Nd,0x19D7,0x19D7,0x19D7},
{0x19D8,Nd,0x19D8,0x19D8,0x19D8},
{0x19D9,Nd,0x19D9,0x19D9,0x19D9},
{0x19DE,Po,0x19DE,0x19DE,0x19DE},
{0x19DF,Po,0x19DF,0x19DF,0x19DF},
{0x19E0,So,0x19E0,0x19E0,0x19E0},
{0x19E1,So,0x19E1,0x19E1,0x19E1},
{0x19E2,So,0x19E2,0x19E2,0x19E2},
{0x19E3,So,0x19E3,0x19E3,0x19E3},
{0x19E4,So,0x19E4,0x19E4,0x19E4},
{0x19E5,So,0x19E5,0x19E5,0x19E5},
{0x19E6,So,0x19E6,0x19E6,0x19E6},
{0x19E7,So,0x19E7,0x19E7,0x19E7},
{0x19E8,So,0x19E8,0x19E8,0x19E8},
{0x19E9,So,0x19E9,0x19E9,0x19E9},
{0x19EA,So,0x19EA,0x19EA,0x19EA},
{0x19EB,So,0x19EB,0x19EB,0x19EB},
{0x19EC,So,0x19EC,0x19EC,0x19EC},
{0x19ED,So,0x19ED,0x19ED,0x19ED},
{0x19EE,So,0x19EE,0x19EE,0x19EE},
{0x19EF,So,0x19EF,0x19EF,0x19EF},
{0x19F0,So,0x19F0,0x19F0,0x19F0},
{0x19F1,So,0x19F1,0x19F1,0x19F1},
{0x19F2,So,0x19F2,0x19F2,0x19F2},
{0x19F3,So,0x19F3,0x19F3,0x19F3},
{0x19F4,So,0x19F4,0x19F4,0x19F4},
{0x19F5,So,0x19F5,0x19F5,0x19F5},
{0x19F6,So,0x19F6,0x19F6,0x19F6},
{0x19F7,So,0x19F7,0x19F7,0x19F7},
{0x19F8,So,0x19F8,0x19F8,0x19F8},
{0x19F9,So,0x19F9,0x19F9,0x19F9},
{0x19FA,So,0x19FA,0x19FA,0x19FA},
{0x19FB,So,0x19FB,0x19FB,0x19FB},
{0x19FC,So,0x19FC,0x19FC,0x19FC},
{0x19FD,So,0x19FD,0x19FD,0x19FD},
{0x19FE,So,0x19FE,0x19FE,0x19FE},
{0x19FF,So,0x19FF,0x19FF,0x19FF},
{0x1A00,Lo,0x1A00,0x1A00,0x1A00},
{0x1A01,Lo,0x1A01,0x1A01,0x1A01},
{0x1A02,Lo,0x1A02,0x1A02,0x1A02},
{0x1A03,Lo,0x1A03,0x1A03,0x1A03},
{0x1A04,Lo,0x1A04,0x1A04,0x1A04},
{0x1A05,Lo,0x1A05,0x1A05,0x1A05},
{0x1A06,Lo,0x1A06,0x1A06,0x1A06},
{0x1A07,Lo,0x1A07,0x1A07,0x1A07},
{0x1A08,Lo,0x1A08,0x1A08,0x1A08},
{0x1A09,Lo,0x1A09,0x1A09,0x1A09},
{0x1A0A,Lo,0x1A0A,0x1A0A,0x1A0A},
{0x1A0B,Lo,0x1A0B,0x1A0B,0x1A0B},
{0x1A0C,Lo,0x1A0C,0x1A0C,0x1A0C},
{0x1A0D,Lo,0x1A0D,0x1A0D,0x1A0D},
{0x1A0E,Lo,0x1A0E,0x1A0E,0x1A0E},
{0x1A0F,Lo,0x1A0F,0x1A0F,0x1A0F},
{0x1A10,Lo,0x1A10,0x1A10,0x1A10},
{0x1A11,Lo,0x1A11,0x1A11,0x1A11},
{0x1A12,Lo,0x1A12,0x1A12,0x1A12},
{0x1A13,Lo,0x1A13,0x1A13,0x1A13},
{0x1A14,Lo,0x1A14,0x1A14,0x1A14},
{0x1A15,Lo,0x1A15,0x1A15,0x1A15},
{0x1A16,Lo,0x1A16,0x1A16,0x1A16},
{0x1A17,Mn,0x1A17,0x1A17,0x1A17},
{0x1A18,Mn,0x1A18,0x1A18,0x1A18},
{0x1A19,Mc,0x1A19,0x1A19,0x1A19},
{0x1A1A,Mc,0x1A1A,0x1A1A,0x1A1A},
{0x1A1B,Mc,0x1A1B,0x1A1B,0x1A1B},
{0x1A1E,Po,0x1A1E,0x1A1E,0x1A1E},
{0x1A1F,Po,0x1A1F,0x1A1F,0x1A1F},
{0x1B00,Mn,0x1B00,0x1B00,0x1B00},
{0x1B01,Mn,0x1B01,0x1B01,0x1B01},
{0x1B02,Mn,0x1B02,0x1B02,0x1B02},
{0x1B03,Mn,0x1B03,0x1B03,0x1B03},
{0x1B04,Mc,0x1B04,0x1B04,0x1B04},
{0x1B05,Lo,0x1B05,0x1B05,0x1B05},
{0x1B06,Lo,0x1B06,0x1B06,0x1B06},
{0x1B07,Lo,0x1B07,0x1B07,0x1B07},
{0x1B08,Lo,0x1B08,0x1B08,0x1B08},
{0x1B09,Lo,0x1B09,0x1B09,0x1B09},
{0x1B0A,Lo,0x1B0A,0x1B0A,0x1B0A},
{0x1B0B,Lo,0x1B0B,0x1B0B,0x1B0B},
{0x1B0C,Lo,0x1B0C,0x1B0C,0x1B0C},
{0x1B0D,Lo,0x1B0D,0x1B0D,0x1B0D},
{0x1B0E,Lo,0x1B0E,0x1B0E,0x1B0E},
{0x1B0F,Lo,0x1B0F,0x1B0F,0x1B0F},
{0x1B10,Lo,0x1B10,0x1B10,0x1B10},
{0x1B11,Lo,0x1B11,0x1B11,0x1B11},
{0x1B12,Lo,0x1B12,0x1B12,0x1B12},
{0x1B13,Lo,0x1B13,0x1B13,0x1B13},
{0x1B14,Lo,0x1B14,0x1B14,0x1B14},
{0x1B15,Lo,0x1B15,0x1B15,0x1B15},
{0x1B16,Lo,0x1B16,0x1B16,0x1B16},
{0x1B17,Lo,0x1B17,0x1B17,0x1B17},
{0x1B18,Lo,0x1B18,0x1B18,0x1B18},
{0x1B19,Lo,0x1B19,0x1B19,0x1B19},
{0x1B1A,Lo,0x1B1A,0x1B1A,0x1B1A},
{0x1B1B,Lo,0x1B1B,0x1B1B,0x1B1B},
{0x1B1C,Lo,0x1B1C,0x1B1C,0x1B1C},
{0x1B1D,Lo,0x1B1D,0x1B1D,0x1B1D},
{0x1B1E,Lo,0x1B1E,0x1B1E,0x1B1E},
{0x1B1F,Lo,0x1B1F,0x1B1F,0x1B1F},
{0x1B20,Lo,0x1B20,0x1B20,0x1B20},
{0x1B21,Lo,0x1B21,0x1B21,0x1B21},
{0x1B22,Lo,0x1B22,0x1B22,0x1B22},
{0x1B23,Lo,0x1B23,0x1B23,0x1B23},
{0x1B24,Lo,0x1B24,0x1B24,0x1B24},
{0x1B25,Lo,0x1B25,0x1B25,0x1B25},
{0x1B26,Lo,0x1B26,0x1B26,0x1B26},
{0x1B27,Lo,0x1B27,0x1B27,0x1B27},
{0x1B28,Lo,0x1B28,0x1B28,0x1B28},
{0x1B29,Lo,0x1B29,0x1B29,0x1B29},
{0x1B2A,Lo,0x1B2A,0x1B2A,0x1B2A},
{0x1B2B,Lo,0x1B2B,0x1B2B,0x1B2B},
{0x1B2C,Lo,0x1B2C,0x1B2C,0x1B2C},
{0x1B2D,Lo,0x1B2D,0x1B2D,0x1B2D},
{0x1B2E,Lo,0x1B2E,0x1B2E,0x1B2E},
{0x1B2F,Lo,0x1B2F,0x1B2F,0x1B2F},
{0x1B30,Lo,0x1B30,0x1B30,0x1B30},
{0x1B31,Lo,0x1B31,0x1B31,0x1B31},
{0x1B32,Lo,0x1B32,0x1B32,0x1B32},
{0x1B33,Lo,0x1B33,0x1B33,0x1B33},
{0x1B34,Mn,0x1B34,0x1B34,0x1B34},
{0x1B35,Mc,0x1B35,0x1B35,0x1B35},
{0x1B36,Mn,0x1B36,0x1B36,0x1B36},
{0x1B37,Mn,0x1B37,0x1B37,0x1B37},
{0x1B38,Mn,0x1B38,0x1B38,0x1B38},
{0x1B39,Mn,0x1B39,0x1B39,0x1B39},
{0x1B3A,Mn,0x1B3A,0x1B3A,0x1B3A},
{0x1B3B,Mc,0x1B3B,0x1B3B,0x1B3B},
{0x1B3C,Mn,0x1B3C,0x1B3C,0x1B3C},
{0x1B3D,Mc,0x1B3D,0x1B3D,0x1B3D},
{0x1B3E,Mc,0x1B3E,0x1B3E,0x1B3E},
{0x1B3F,Mc,0x1B3F,0x1B3F,0x1B3F},
{0x1B40,Mc,0x1B40,0x1B40,0x1B40},
{0x1B41,Mc,0x1B41,0x1B41,0x1B41},
{0x1B42,Mn,0x1B42,0x1B42,0x1B42},
{0x1B43,Mc,0x1B43,0x1B43,0x1B43},
{0x1B44,Mc,0x1B44,0x1B44,0x1B44},
{0x1B45,Lo,0x1B45,0x1B45,0x1B45},
{0x1B46,Lo,0x1B46,0x1B46,0x1B46},
{0x1B47,Lo,0x1B47,0x1B47,0x1B47},
{0x1B48,Lo,0x1B48,0x1B48,0x1B48},
{0x1B49,Lo,0x1B49,0x1B49,0x1B49},
{0x1B4A,Lo,0x1B4A,0x1B4A,0x1B4A},
{0x1B4B,Lo,0x1B4B,0x1B4B,0x1B4B},
{0x1B50,Nd,0x1B50,0x1B50,0x1B50},
{0x1B51,Nd,0x1B51,0x1B51,0x1B51},
{0x1B52,Nd,0x1B52,0x1B52,0x1B52},
{0x1B53,Nd,0x1B53,0x1B53,0x1B53},
{0x1B54,Nd,0x1B54,0x1B54,0x1B54},
{0x1B55,Nd,0x1B55,0x1B55,0x1B55},
{0x1B56,Nd,0x1B56,0x1B56,0x1B56},
{0x1B57,Nd,0x1B57,0x1B57,0x1B57},
{0x1B58,Nd,0x1B58,0x1B58,0x1B58},
{0x1B59,Nd,0x1B59,0x1B59,0x1B59},
{0x1B5A,Po,0x1B5A,0x1B5A,0x1B5A},
{0x1B5B,Po,0x1B5B,0x1B5B,0x1B5B},
{0x1B5C,Po,0x1B5C,0x1B5C,0x1B5C},
{0x1B5D,Po,0x1B5D,0x1B5D,0x1B5D},
{0x1B5E,Po,0x1B5E,0x1B5E,0x1B5E},
{0x1B5F,Po,0x1B5F,0x1B5F,0x1B5F},
{0x1B60,Po,0x1B60,0x1B60,0x1B60},
{0x1B61,So,0x1B61,0x1B61,0x1B61},
{0x1B62,So,0x1B62,0x1B62,0x1B62},
{0x1B63,So,0x1B63,0x1B63,0x1B63},
{0x1B64,So,0x1B64,0x1B64,0x1B64},
{0x1B65,So,0x1B65,0x1B65,0x1B65},
{0x1B66,So,0x1B66,0x1B66,0x1B66},
{0x1B67,So,0x1B67,0x1B67,0x1B67},
{0x1B68,So,0x1B68,0x1B68,0x1B68},
{0x1B69,So,0x1B69,0x1B69,0x1B69},
{0x1B6A,So,0x1B6A,0x1B6A,0x1B6A},
{0x1B6B,Mn,0x1B6B,0x1B6B,0x1B6B},
{0x1B6C,Mn,0x1B6C,0x1B6C,0x1B6C},
{0x1B6D,Mn,0x1B6D,0x1B6D,0x1B6D},
{0x1B6E,Mn,0x1B6E,0x1B6E,0x1B6E},
{0x1B6F,Mn,0x1B6F,0x1B6F,0x1B6F},
{0x1B70,Mn,0x1B70,0x1B70,0x1B70},
{0x1B71,Mn,0x1B71,0x1B71,0x1B71},
{0x1B72,Mn,0x1B72,0x1B72,0x1B72},
{0x1B73,Mn,0x1B73,0x1B73,0x1B73},
{0x1B74,So,0x1B74,0x1B74,0x1B74},
{0x1B75,So,0x1B75,0x1B75,0x1B75},
{0x1B76,So,0x1B76,0x1B76,0x1B76},
{0x1B77,So,0x1B77,0x1B77,0x1B77},
{0x1B78,So,0x1B78,0x1B78,0x1B78},
{0x1B79,So,0x1B79,0x1B79,0x1B79},
{0x1B7A,So,0x1B7A,0x1B7A,0x1B7A},
{0x1B7B,So,0x1B7B,0x1B7B,0x1B7B},
{0x1B7C,So,0x1B7C,0x1B7C,0x1B7C},
{0x1D00,Ll,0x1D00,0x1D00,0x1D00},
{0x1D01,Ll,0x1D01,0x1D01,0x1D01},
{0x1D02,Ll,0x1D02,0x1D02,0x1D02},
{0x1D03,Ll,0x1D03,0x1D03,0x1D03},
{0x1D04,Ll,0x1D04,0x1D04,0x1D04},
{0x1D05,Ll,0x1D05,0x1D05,0x1D05},
{0x1D06,Ll,0x1D06,0x1D06,0x1D06},
{0x1D07,Ll,0x1D07,0x1D07,0x1D07},
{0x1D08,Ll,0x1D08,0x1D08,0x1D08},
{0x1D09,Ll,0x1D09,0x1D09,0x1D09},
{0x1D0A,Ll,0x1D0A,0x1D0A,0x1D0A},
{0x1D0B,Ll,0x1D0B,0x1D0B,0x1D0B},
{0x1D0C,Ll,0x1D0C,0x1D0C,0x1D0C},
{0x1D0D,Ll,0x1D0D,0x1D0D,0x1D0D},
{0x1D0E,Ll,0x1D0E,0x1D0E,0x1D0E},
{0x1D0F,Ll,0x1D0F,0x1D0F,0x1D0F},
{0x1D10,Ll,0x1D10,0x1D10,0x1D10},
{0x1D11,Ll,0x1D11,0x1D11,0x1D11},
{0x1D12,Ll,0x1D12,0x1D12,0x1D12},
{0x1D13,Ll,0x1D13,0x1D13,0x1D13},
{0x1D14,Ll,0x1D14,0x1D14,0x1D14},
{0x1D15,Ll,0x1D15,0x1D15,0x1D15},
{0x1D16,Ll,0x1D16,0x1D16,0x1D16},
{0x1D17,Ll,0x1D17,0x1D17,0x1D17},
{0x1D18,Ll,0x1D18,0x1D18,0x1D18},
{0x1D19,Ll,0x1D19,0x1D19,0x1D19},
{0x1D1A,Ll,0x1D1A,0x1D1A,0x1D1A},
{0x1D1B,Ll,0x1D1B,0x1D1B,0x1D1B},
{0x1D1C,Ll,0x1D1C,0x1D1C,0x1D1C},
{0x1D1D,Ll,0x1D1D,0x1D1D,0x1D1D},
{0x1D1E,Ll,0x1D1E,0x1D1E,0x1D1E},
{0x1D1F,Ll,0x1D1F,0x1D1F,0x1D1F},
{0x1D20,Ll,0x1D20,0x1D20,0x1D20},
{0x1D21,Ll,0x1D21,0x1D21,0x1D21},
{0x1D22,Ll,0x1D22,0x1D22,0x1D22},
{0x1D23,Ll,0x1D23,0x1D23,0x1D23},
{0x1D24,Ll,0x1D24,0x1D24,0x1D24},
{0x1D25,Ll,0x1D25,0x1D25,0x1D25},
{0x1D26,Ll,0x1D26,0x1D26,0x1D26},
{0x1D27,Ll,0x1D27,0x1D27,0x1D27},
{0x1D28,Ll,0x1D28,0x1D28,0x1D28},
{0x1D29,Ll,0x1D29,0x1D29,0x1D29},
{0x1D2A,Ll,0x1D2A,0x1D2A,0x1D2A},
{0x1D2B,Ll,0x1D2B,0x1D2B,0x1D2B},
{0x1D2C,Lm,0x1D2C,0x1D2C,0x1D2C},
{0x1D2D,Lm,0x1D2D,0x1D2D,0x1D2D},
{0x1D2E,Lm,0x1D2E,0x1D2E,0x1D2E},
{0x1D2F,Lm,0x1D2F,0x1D2F,0x1D2F},
{0x1D30,Lm,0x1D30,0x1D30,0x1D30},
{0x1D31,Lm,0x1D31,0x1D31,0x1D31},
{0x1D32,Lm,0x1D32,0x1D32,0x1D32},
{0x1D33,Lm,0x1D33,0x1D33,0x1D33},
{0x1D34,Lm,0x1D34,0x1D34,0x1D34},
{0x1D35,Lm,0x1D35,0x1D35,0x1D35},
{0x1D36,Lm,0x1D36,0x1D36,0x1D36},
{0x1D37,Lm,0x1D37,0x1D37,0x1D37},
{0x1D38,Lm,0x1D38,0x1D38,0x1D38},
{0x1D39,Lm,0x1D39,0x1D39,0x1D39},
{0x1D3A,Lm,0x1D3A,0x1D3A,0x1D3A},
{0x1D3B,Lm,0x1D3B,0x1D3B,0x1D3B},
{0x1D3C,Lm,0x1D3C,0x1D3C,0x1D3C},
{0x1D3D,Lm,0x1D3D,0x1D3D,0x1D3D},
{0x1D3E,Lm,0x1D3E,0x1D3E,0x1D3E},
{0x1D3F,Lm,0x1D3F,0x1D3F,0x1D3F},
{0x1D40,Lm,0x1D40,0x1D40,0x1D40},
{0x1D41,Lm,0x1D41,0x1D41,0x1D41},
{0x1D42,Lm,0x1D42,0x1D42,0x1D42},
{0x1D43,Lm,0x1D43,0x1D43,0x1D43},
{0x1D44,Lm,0x1D44,0x1D44,0x1D44},
{0x1D45,Lm,0x1D45,0x1D45,0x1D45},
{0x1D46,Lm,0x1D46,0x1D46,0x1D46},
{0x1D47,Lm,0x1D47,0x1D47,0x1D47},
{0x1D48,Lm,0x1D48,0x1D48,0x1D48},
{0x1D49,Lm,0x1D49,0x1D49,0x1D49},
{0x1D4A,Lm,0x1D4A,0x1D4A,0x1D4A},
{0x1D4B,Lm,0x1D4B,0x1D4B,0x1D4B},
{0x1D4C,Lm,0x1D4C,0x1D4C,0x1D4C},
{0x1D4D,Lm,0x1D4D,0x1D4D,0x1D4D},
{0x1D4E,Lm,0x1D4E,0x1D4E,0x1D4E},
{0x1D4F,Lm,0x1D4F,0x1D4F,0x1D4F},
{0x1D50,Lm,0x1D50,0x1D50,0x1D50},
{0x1D51,Lm,0x1D51,0x1D51,0x1D51},
{0x1D52,Lm,0x1D52,0x1D52,0x1D52},
{0x1D53,Lm,0x1D53,0x1D53,0x1D53},
{0x1D54,Lm,0x1D54,0x1D54,0x1D54},
{0x1D55,Lm,0x1D55,0x1D55,0x1D55},
{0x1D56,Lm,0x1D56,0x1D56,0x1D56},
{0x1D57,Lm,0x1D57,0x1D57,0x1D57},
{0x1D58,Lm,0x1D58,0x1D58,0x1D58},
{0x1D59,Lm,0x1D59,0x1D59,0x1D59},
{0x1D5A,Lm,0x1D5A,0x1D5A,0x1D5A},
{0x1D5B,Lm,0x1D5B,0x1D5B,0x1D5B},
{0x1D5C,Lm,0x1D5C,0x1D5C,0x1D5C},
{0x1D5D,Lm,0x1D5D,0x1D5D,0x1D5D},
{0x1D5E,Lm,0x1D5E,0x1D5E,0x1D5E},
{0x1D5F,Lm,0x1D5F,0x1D5F,0x1D5F},
{0x1D60,Lm,0x1D60,0x1D60,0x1D60},
{0x1D61,Lm,0x1D61,0x1D61,0x1D61},
{0x1D62,Ll,0x1D62,0x1D62,0x1D62},
{0x1D63,Ll,0x1D63,0x1D63,0x1D63},
{0x1D64,Ll,0x1D64,0x1D64,0x1D64},
{0x1D65,Ll,0x1D65,0x1D65,0x1D65},
{0x1D66,Ll,0x1D66,0x1D66,0x1D66},
{0x1D67,Ll,0x1D67,0x1D67,0x1D67},
{0x1D68,Ll,0x1D68,0x1D68,0x1D68},
{0x1D69,Ll,0x1D69,0x1D69,0x1D69},
{0x1D6A,Ll,0x1D6A,0x1D6A,0x1D6A},
{0x1D6B,Ll,0x1D6B,0x1D6B,0x1D6B},
{0x1D6C,Ll,0x1D6C,0x1D6C,0x1D6C},
{0x1D6D,Ll,0x1D6D,0x1D6D,0x1D6D},
{0x1D6E,Ll,0x1D6E,0x1D6E,0x1D6E},
{0x1D6F,Ll,0x1D6F,0x1D6F,0x1D6F},
{0x1D70,Ll,0x1D70,0x1D70,0x1D70},
{0x1D71,Ll,0x1D71,0x1D71,0x1D71},
{0x1D72,Ll,0x1D72,0x1D72,0x1D72},
{0x1D73,Ll,0x1D73,0x1D73,0x1D73},
{0x1D74,Ll,0x1D74,0x1D74,0x1D74},
{0x1D75,Ll,0x1D75,0x1D75,0x1D75},
{0x1D76,Ll,0x1D76,0x1D76,0x1D76},
{0x1D77,Ll,0x1D77,0x1D77,0x1D77},
{0x1D78,Lm,0x1D78,0x1D78,0x1D78},
{0x1D79,Ll,0x1D79,0x1D79,0x1D79},
{0x1D7A,Ll,0x1D7A,0x1D7A,0x1D7A},
{0x1D7B,Ll,0x1D7B,0x1D7B,0x1D7B},
{0x1D7C,Ll,0x1D7C,0x1D7C,0x1D7C},
{0x1D7D,Ll,0x2C63,0x1D7D,0x2C63},
{0x1D7E,Ll,0x1D7E,0x1D7E,0x1D7E},
{0x1D7F,Ll,0x1D7F,0x1D7F,0x1D7F},
{0x1D80,Ll,0x1D80,0x1D80,0x1D80},
{0x1D81,Ll,0x1D81,0x1D81,0x1D81},
{0x1D82,Ll,0x1D82,0x1D82,0x1D82},
{0x1D83,Ll,0x1D83,0x1D83,0x1D83},
{0x1D84,Ll,0x1D84,0x1D84,0x1D84},
{0x1D85,Ll,0x1D85,0x1D85,0x1D85},
{0x1D86,Ll,0x1D86,0x1D86,0x1D86},
{0x1D87,Ll,0x1D87,0x1D87,0x1D87},
{0x1D88,Ll,0x1D88,0x1D88,0x1D88},
{0x1D89,Ll,0x1D89,0x1D89,0x1D89},
{0x1D8A,Ll,0x1D8A,0x1D8A,0x1D8A},
{0x1D8B,Ll,0x1D8B,0x1D8B,0x1D8B},
{0x1D8C,Ll,0x1D8C,0x1D8C,0x1D8C},
{0x1D8D,Ll,0x1D8D,0x1D8D,0x1D8D},
{0x1D8E,Ll,0x1D8E,0x1D8E,0x1D8E},
{0x1D8F,Ll,0x1D8F,0x1D8F,0x1D8F},
{0x1D90,Ll,0x1D90,0x1D90,0x1D90},
{0x1D91,Ll,0x1D91,0x1D91,0x1D91},
{0x1D92,Ll,0x1D92,0x1D92,0x1D92},
{0x1D93,Ll,0x1D93,0x1D93,0x1D93},
{0x1D94,Ll,0x1D94,0x1D94,0x1D94},
{0x1D95,Ll,0x1D95,0x1D95,0x1D95},
{0x1D96,Ll,0x1D96,0x1D96,0x1D96},
{0x1D97,Ll,0x1D97,0x1D97,0x1D97},
{0x1D98,Ll,0x1D98,0x1D98,0x1D98},
{0x1D99,Ll,0x1D99,0x1D99,0x1D99},
{0x1D9A,Ll,0x1D9A,0x1D9A,0x1D9A},
{0x1D9B,Lm,0x1D9B,0x1D9B,0x1D9B},
{0x1D9C,Lm,0x1D9C,0x1D9C,0x1D9C},
{0x1D9D,Lm,0x1D9D,0x1D9D,0x1D9D},
{0x1D9E,Lm,0x1D9E,0x1D9E,0x1D9E},
{0x1D9F,Lm,0x1D9F,0x1D9F,0x1D9F},
{0x1DA0,Lm,0x1DA0,0x1DA0,0x1DA0},
{0x1DA1,Lm,0x1DA1,0x1DA1,0x1DA1},
{0x1DA2,Lm,0x1DA2,0x1DA2,0x1DA2},
{0x1DA3,Lm,0x1DA3,0x1DA3,0x1DA3},
{0x1DA4,Lm,0x1DA4,0x1DA4,0x1DA4},
{0x1DA5,Lm,0x1DA5,0x1DA5,0x1DA5},
{0x1DA6,Lm,0x1DA6,0x1DA6,0x1DA6},
{0x1DA7,Lm,0x1DA7,0x1DA7,0x1DA7},
{0x1DA8,Lm,0x1DA8,0x1DA8,0x1DA8},
{0x1DA9,Lm,0x1DA9,0x1DA9,0x1DA9},
{0x1DAA,Lm,0x1DAA,0x1DAA,0x1DAA},
{0x1DAB,Lm,0x1DAB,0x1DAB,0x1DAB},
{0x1DAC,Lm,0x1DAC,0x1DAC,0x1DAC},
{0x1DAD,Lm,0x1DAD,0x1DAD,0x1DAD},
{0x1DAE,Lm,0x1DAE,0x1DAE,0x1DAE},
{0x1DAF,Lm,0x1DAF,0x1DAF,0x1DAF},
{0x1DB0,Lm,0x1DB0,0x1DB0,0x1DB0},
{0x1DB1,Lm,0x1DB1,0x1DB1,0x1DB1},
{0x1DB2,Lm,0x1DB2,0x1DB2,0x1DB2},
{0x1DB3,Lm,0x1DB3,0x1DB3,0x1DB3},
{0x1DB4,Lm,0x1DB4,0x1DB4,0x1DB4},
{0x1DB5,Lm,0x1DB5,0x1DB5,0x1DB5},
{0x1DB6,Lm,0x1DB6,0x1DB6,0x1DB6},
{0x1DB7,Lm,0x1DB7,0x1DB7,0x1DB7},
{0x1DB8,Lm,0x1DB8,0x1DB8,0x1DB8},
{0x1DB9,Lm,0x1DB9,0x1DB9,0x1DB9},
{0x1DBA,Lm,0x1DBA,0x1DBA,0x1DBA},
{0x1DBB,Lm,0x1DBB,0x1DBB,0x1DBB},
{0x1DBC,Lm,0x1DBC,0x1DBC,0x1DBC},
{0x1DBD,Lm,0x1DBD,0x1DBD,0x1DBD},
{0x1DBE,Lm,0x1DBE,0x1DBE,0x1DBE},
{0x1DBF,Lm,0x1DBF,0x1DBF,0x1DBF},
{0x1DC0,Mn,0x1DC0,0x1DC0,0x1DC0},
{0x1DC1,Mn,0x1DC1,0x1DC1,0x1DC1},
{0x1DC2,Mn,0x1DC2,0x1DC2,0x1DC2},
{0x1DC3,Mn,0x1DC3,0x1DC3,0x1DC3},
{0x1DC4,Mn,0x1DC4,0x1DC4,0x1DC4},
{0x1DC5,Mn,0x1DC5,0x1DC5,0x1DC5},
{0x1DC6,Mn,0x1DC6,0x1DC6,0x1DC6},
{0x1DC7,Mn,0x1DC7,0x1DC7,0x1DC7},
{0x1DC8,Mn,0x1DC8,0x1DC8,0x1DC8},
{0x1DC9,Mn,0x1DC9,0x1DC9,0x1DC9},
{0x1DCA,Mn,0x1DCA,0x1DCA,0x1DCA},
{0x1DFE,Mn,0x1DFE,0x1DFE,0x1DFE},
{0x1DFF,Mn,0x1DFF,0x1DFF,0x1DFF},
{0x1E00,Lu,0x1E00,0x1E01,0x1E00},
{0x1E01,Ll,0x1E00,0x1E01,0x1E00},
{0x1E02,Lu,0x1E02,0x1E03,0x1E02},
{0x1E03,Ll,0x1E02,0x1E03,0x1E02},
{0x1E04,Lu,0x1E04,0x1E05,0x1E04},
{0x1E05,Ll,0x1E04,0x1E05,0x1E04},
{0x1E06,Lu,0x1E06,0x1E07,0x1E06},
{0x1E07,Ll,0x1E06,0x1E07,0x1E06},
{0x1E08,Lu,0x1E08,0x1E09,0x1E08},
{0x1E09,Ll,0x1E08,0x1E09,0x1E08},
{0x1E0A,Lu,0x1E0A,0x1E0B,0x1E0A},
{0x1E0B,Ll,0x1E0A,0x1E0B,0x1E0A},
{0x1E0C,Lu,0x1E0C,0x1E0D,0x1E0C},
{0x1E0D,Ll,0x1E0C,0x1E0D,0x1E0C},
{0x1E0E,Lu,0x1E0E,0x1E0F,0x1E0E},
{0x1E0F,Ll,0x1E0E,0x1E0F,0x1E0E},
{0x1E10,Lu,0x1E10,0x1E11,0x1E10},
{0x1E11,Ll,0x1E10,0x1E11,0x1E10},
{0x1E12,Lu,0x1E12,0x1E13,0x1E12},
{0x1E13,Ll,0x1E12,0x1E13,0x1E12},
{0x1E14,Lu,0x1E14,0x1E15,0x1E14},
{0x1E15,Ll,0x1E14,0x1E15,0x1E14},
{0x1E16,Lu,0x1E16,0x1E17,0x1E16},
{0x1E17,Ll,0x1E16,0x1E17,0x1E16},
{0x1E18,Lu,0x1E18,0x1E19,0x1E18},
{0x1E19,Ll,0x1E18,0x1E19,0x1E18},
{0x1E1A,Lu,0x1E1A,0x1E1B,0x1E1A},
{0x1E1B,Ll,0x1E1A,0x1E1B,0x1E1A},
{0x1E1C,Lu,0x1E1C,0x1E1D,0x1E1C},
{0x1E1D,Ll,0x1E1C,0x1E1D,0x1E1C},
{0x1E1E,Lu,0x1E1E,0x1E1F,0x1E1E},
{0x1E1F,Ll,0x1E1E,0x1E1F,0x1E1E},
{0x1E20,Lu,0x1E20,0x1E21,0x1E20},
{0x1E21,Ll,0x1E20,0x1E21,0x1E20},
{0x1E22,Lu,0x1E22,0x1E23,0x1E22},
{0x1E23,Ll,0x1E22,0x1E23,0x1E22},
{0x1E24,Lu,0x1E24,0x1E25,0x1E24},
{0x1E25,Ll,0x1E24,0x1E25,0x1E24},
{0x1E26,Lu,0x1E26,0x1E27,0x1E26},
{0x1E27,Ll,0x1E26,0x1E27,0x1E26},
{0x1E28,Lu,0x1E28,0x1E29,0x1E28},
{0x1E29,Ll,0x1E28,0x1E29,0x1E28},
{0x1E2A,Lu,0x1E2A,0x1E2B,0x1E2A},
{0x1E2B,Ll,0x1E2A,0x1E2B,0x1E2A},
{0x1E2C,Lu,0x1E2C,0x1E2D,0x1E2C},
{0x1E2D,Ll,0x1E2C,0x1E2D,0x1E2C},
{0x1E2E,Lu,0x1E2E,0x1E2F,0x1E2E},
{0x1E2F,Ll,0x1E2E,0x1E2F,0x1E2E},
{0x1E30,Lu,0x1E30,0x1E31,0x1E30},
{0x1E31,Ll,0x1E30,0x1E31,0x1E30},
{0x1E32,Lu,0x1E32,0x1E33,0x1E32},
{0x1E33,Ll,0x1E32,0x1E33,0x1E32},
{0x1E34,Lu,0x1E34,0x1E35,0x1E34},
{0x1E35,Ll,0x1E34,0x1E35,0x1E34},
{0x1E36,Lu,0x1E36,0x1E37,0x1E36},
{0x1E37,Ll,0x1E36,0x1E37,0x1E36},
{0x1E38,Lu,0x1E38,0x1E39,0x1E38},
{0x1E39,Ll,0x1E38,0x1E39,0x1E38},
{0x1E3A,Lu,0x1E3A,0x1E3B,0x1E3A},
{0x1E3B,Ll,0x1E3A,0x1E3B,0x1E3A},
{0x1E3C,Lu,0x1E3C,0x1E3D,0x1E3C},
{0x1E3D,Ll,0x1E3C,0x1E3D,0x1E3C},
{0x1E3E,Lu,0x1E3E,0x1E3F,0x1E3E},
{0x1E3F,Ll,0x1E3E,0x1E3F,0x1E3E},
{0x1E40,Lu,0x1E40,0x1E41,0x1E40},
{0x1E41,Ll,0x1E40,0x1E41,0x1E40},
{0x1E42,Lu,0x1E42,0x1E43,0x1E42},
{0x1E43,Ll,0x1E42,0x1E43,0x1E42},
{0x1E44,Lu,0x1E44,0x1E45,0x1E44},
{0x1E45,Ll,0x1E44,0x1E45,0x1E44},
{0x1E46,Lu,0x1E46,0x1E47,0x1E46},
{0x1E47,Ll,0x1E46,0x1E47,0x1E46},
{0x1E48,Lu,0x1E48,0x1E49,0x1E48},
{0x1E49,Ll,0x1E48,0x1E49,0x1E48},
{0x1E4A,Lu,0x1E4A,0x1E4B,0x1E4A},
{0x1E4B,Ll,0x1E4A,0x1E4B,0x1E4A},
{0x1E4C,Lu,0x1E4C,0x1E4D,0x1E4C},
{0x1E4D,Ll,0x1E4C,0x1E4D,0x1E4C},
{0x1E4E,Lu,0x1E4E,0x1E4F,0x1E4E},
{0x1E4F,Ll,0x1E4E,0x1E4F,0x1E4E},
{0x1E50,Lu,0x1E50,0x1E51,0x1E50},
{0x1E51,Ll,0x1E50,0x1E51,0x1E50},
{0x1E52,Lu,0x1E52,0x1E53,0x1E52},
{0x1E53,Ll,0x1E52,0x1E53,0x1E52},
{0x1E54,Lu,0x1E54,0x1E55,0x1E54},
{0x1E55,Ll,0x1E54,0x1E55,0x1E54},
{0x1E56,Lu,0x1E56,0x1E57,0x1E56},
{0x1E57,Ll,0x1E56,0x1E57,0x1E56},
{0x1E58,Lu,0x1E58,0x1E59,0x1E58},
{0x1E59,Ll,0x1E58,0x1E59,0x1E58},
{0x1E5A,Lu,0x1E5A,0x1E5B,0x1E5A},
{0x1E5B,Ll,0x1E5A,0x1E5B,0x1E5A},
{0x1E5C,Lu,0x1E5C,0x1E5D,0x1E5C},
{0x1E5D,Ll,0x1E5C,0x1E5D,0x1E5C},
{0x1E5E,Lu,0x1E5E,0x1E5F,0x1E5E},
{0x1E5F,Ll,0x1E5E,0x1E5F,0x1E5E},
{0x1E60,Lu,0x1E60,0x1E61,0x1E60},
{0x1E61,Ll,0x1E60,0x1E61,0x1E60},
{0x1E62,Lu,0x1E62,0x1E63,0x1E62},
{0x1E63,Ll,0x1E62,0x1E63,0x1E62},
{0x1E64,Lu,0x1E64,0x1E65,0x1E64},
{0x1E65,Ll,0x1E64,0x1E65,0x1E64},
{0x1E66,Lu,0x1E66,0x1E67,0x1E66},
{0x1E67,Ll,0x1E66,0x1E67,0x1E66},
{0x1E68,Lu,0x1E68,0x1E69,0x1E68},
{0x1E69,Ll,0x1E68,0x1E69,0x1E68},
{0x1E6A,Lu,0x1E6A,0x1E6B,0x1E6A},
{0x1E6B,Ll,0x1E6A,0x1E6B,0x1E6A},
{0x1E6C,Lu,0x1E6C,0x1E6D,0x1E6C},
{0x1E6D,Ll,0x1E6C,0x1E6D,0x1E6C},
{0x1E6E,Lu,0x1E6E,0x1E6F,0x1E6E},
{0x1E6F,Ll,0x1E6E,0x1E6F,0x1E6E},
{0x1E70,Lu,0x1E70,0x1E71,0x1E70},
{0x1E71,Ll,0x1E70,0x1E71,0x1E70},
{0x1E72,Lu,0x1E72,0x1E73,0x1E72},
{0x1E73,Ll,0x1E72,0x1E73,0x1E72},
{0x1E74,Lu,0x1E74,0x1E75,0x1E74},
{0x1E75,Ll,0x1E74,0x1E75,0x1E74},
{0x1E76,Lu,0x1E76,0x1E77,0x1E76},
{0x1E77,Ll,0x1E76,0x1E77,0x1E76},
{0x1E78,Lu,0x1E78,0x1E79,0x1E78},
{0x1E79,Ll,0x1E78,0x1E79,0x1E78},
{0x1E7A,Lu,0x1E7A,0x1E7B,0x1E7A},
{0x1E7B,Ll,0x1E7A,0x1E7B,0x1E7A},
{0x1E7C,Lu,0x1E7C,0x1E7D,0x1E7C},
{0x1E7D,Ll,0x1E7C,0x1E7D,0x1E7C},
{0x1E7E,Lu,0x1E7E,0x1E7F,0x1E7E},
{0x1E7F,Ll,0x1E7E,0x1E7F,0x1E7E},
{0x1E80,Lu,0x1E80,0x1E81,0x1E80},
{0x1E81,Ll,0x1E80,0x1E81,0x1E80},
{0x1E82,Lu,0x1E82,0x1E83,0x1E82},
{0x1E83,Ll,0x1E82,0x1E83,0x1E82},
{0x1E84,Lu,0x1E84,0x1E85,0x1E84},
{0x1E85,Ll,0x1E84,0x1E85,0x1E84},
{0x1E86,Lu,0x1E86,0x1E87,0x1E86},
{0x1E87,Ll,0x1E86,0x1E87,0x1E86},
{0x1E88,Lu,0x1E88,0x1E89,0x1E88},
{0x1E89,Ll,0x1E88,0x1E89,0x1E88},
{0x1E8A,Lu,0x1E8A,0x1E8B,0x1E8A},
{0x1E8B,Ll,0x1E8A,0x1E8B,0x1E8A},
{0x1E8C,Lu,0x1E8C,0x1E8D,0x1E8C},
{0x1E8D,Ll,0x1E8C,0x1E8D,0x1E8C},
{0x1E8E,Lu,0x1E8E,0x1E8F,0x1E8E},
{0x1E8F,Ll,0x1E8E,0x1E8F,0x1E8E},
{0x1E90,Lu,0x1E90,0x1E91,0x1E90},
{0x1E91,Ll,0x1E90,0x1E91,0x1E90},
{0x1E92,Lu,0x1E92,0x1E93,0x1E92},
{0x1E93,Ll,0x1E92,0x1E93,0x1E92},
{0x1E94,Lu,0x1E94,0x1E95,0x1E94},
{0x1E95,Ll,0x1E94,0x1E95,0x1E94},
{0x1E96,Ll | SM,0x1E96,0x1E96,0x1E96},
{0x1E97,Ll | SM,0x1E97,0x1E97,0x1E97},
{0x1E98,Ll | SM,0x1E98,0x1E98,0x1E98},
{0x1E99,Ll | SM,0x1E99,0x1E99,0x1E99},
{0x1E9A,Ll | SM,0x1E9A,0x1E9A,0x1E9A},
{0x1E9B,Ll,0x1E60,0x1E9B,0x1E60},
{0x1EA0,Lu,0x1EA0,0x1EA1,0x1EA0},
{0x1EA1,Ll,0x1EA0,0x1EA1,0x1EA0},
{0x1EA2,Lu,0x1EA2,0x1EA3,0x1EA2},
{0x1EA3,Ll,0x1EA2,0x1EA3,0x1EA2},
{0x1EA4,Lu,0x1EA4,0x1EA5,0x1EA4},
{0x1EA5,Ll,0x1EA4,0x1EA5,0x1EA4},
{0x1EA6,Lu,0x1EA6,0x1EA7,0x1EA6},
{0x1EA7,Ll,0x1EA6,0x1EA7,0x1EA6},
{0x1EA8,Lu,0x1EA8,0x1EA9,0x1EA8},
{0x1EA9,Ll,0x1EA8,0x1EA9,0x1EA8},
{0x1EAA,Lu,0x1EAA,0x1EAB,0x1EAA},
{0x1EAB,Ll,0x1EAA,0x1EAB,0x1EAA},
{0x1EAC,Lu,0x1EAC,0x1EAD,0x1EAC},
{0x1EAD,Ll,0x1EAC,0x1EAD,0x1EAC},
{0x1EAE,Lu,0x1EAE,0x1EAF,0x1EAE},
{0x1EAF,Ll,0x1EAE,0x1EAF,0x1EAE},
{0x1EB0,Lu,0x1EB0,0x1EB1,0x1EB0},
{0x1EB1,Ll,0x1EB0,0x1EB1,0x1EB0},
{0x1EB2,Lu,0x1EB2,0x1EB3,0x1EB2},
{0x1EB3,Ll,0x1EB2,0x1EB3,0x1EB2},
{0x1EB4,Lu,0x1EB4,0x1EB5,0x1EB4},
{0x1EB5,Ll,0x1EB4,0x1EB5,0x1EB4},
{0x1EB6,Lu,0x1EB6,0x1EB7,0x1EB6},
{0x1EB7,Ll,0x1EB6,0x1EB7,0x1EB6},
{0x1EB8,Lu,0x1EB8,0x1EB9,0x1EB8},
{0x1EB9,Ll,0x1EB8,0x1EB9,0x1EB8},
{0x1EBA,Lu,0x1EBA,0x1EBB,0x1EBA},
{0x1EBB,Ll,0x1EBA,0x1EBB,0x1EBA},
{0x1EBC,Lu,0x1EBC,0x1EBD,0x1EBC},
{0x1EBD,Ll,0x1EBC,0x1EBD,0x1EBC},
{0x1EBE,Lu,0x1EBE,0x1EBF,0x1EBE},
{0x1EBF,Ll,0x1EBE,0x1EBF,0x1EBE},
{0x1EC0,Lu,0x1EC0,0x1EC1,0x1EC0},
{0x1EC1,Ll,0x1EC0,0x1EC1,0x1EC0},
{0x1EC2,Lu,0x1EC2,0x1EC3,0x1EC2},
{0x1EC3,Ll,0x1EC2,0x1EC3,0x1EC2},
{0x1EC4,Lu,0x1EC4,0x1EC5,0x1EC4},
{0x1EC5,Ll,0x1EC4,0x1EC5,0x1EC4},
{0x1EC6,Lu,0x1EC6,0x1EC7,0x1EC6},
{0x1EC7,Ll,0x1EC6,0x1EC7,0x1EC6},
{0x1EC8,Lu,0x1EC8,0x1EC9,0x1EC8},
{0x1EC9,Ll,0x1EC8,0x1EC9,0x1EC8},
{0x1ECA,Lu,0x1ECA,0x1ECB,0x1ECA},
{0x1ECB,Ll,0x1ECA,0x1ECB,0x1ECA},
{0x1ECC,Lu,0x1ECC,0x1ECD,0x1ECC},
{0x1ECD,Ll,0x1ECC,0x1ECD,0x1ECC},
{0x1ECE,Lu,0x1ECE,0x1ECF,0x1ECE},
{0x1ECF,Ll,0x1ECE,0x1ECF,0x1ECE},
{0x1ED0,Lu,0x1ED0,0x1ED1,0x1ED0},
{0x1ED1,Ll,0x1ED0,0x1ED1,0x1ED0},
{0x1ED2,Lu,0x1ED2,0x1ED3,0x1ED2},
{0x1ED3,Ll,0x1ED2,0x1ED3,0x1ED2},
{0x1ED4,Lu,0x1ED4,0x1ED5,0x1ED4},
{0x1ED5,Ll,0x1ED4,0x1ED5,0x1ED4},
{0x1ED6,Lu,0x1ED6,0x1ED7,0x1ED6},
{0x1ED7,Ll,0x1ED6,0x1ED7,0x1ED6},
{0x1ED8,Lu,0x1ED8,0x1ED9,0x1ED8},
{0x1ED9,Ll,0x1ED8,0x1ED9,0x1ED8},
{0x1EDA,Lu,0x1EDA,0x1EDB,0x1EDA},
{0x1EDB,Ll,0x1EDA,0x1EDB,0x1EDA},
{0x1EDC,Lu,0x1EDC,0x1EDD,0x1EDC},
{0x1EDD,Ll,0x1EDC,0x1EDD,0x1EDC},
{0x1EDE,Lu,0x1EDE,0x1EDF,0x1EDE},
{0x1EDF,Ll,0x1EDE,0x1EDF,0x1EDE},
{0x1EE0,Lu,0x1EE0,0x1EE1,0x1EE0},
{0x1EE1,Ll,0x1EE0,0x1EE1,0x1EE0},
{0x1EE2,Lu,0x1EE2,0x1EE3,0x1EE2},
{0x1EE3,Ll,0x1EE2,0x1EE3,0x1EE2},
{0x1EE4,Lu,0x1EE4,0x1EE5,0x1EE4},
{0x1EE5,Ll,0x1EE4,0x1EE5,0x1EE4},
{0x1EE6,Lu,0x1EE6,0x1EE7,0x1EE6},
{0x1EE7,Ll,0x1EE6,0x1EE7,0x1EE6},
{0x1EE8,Lu,0x1EE8,0x1EE9,0x1EE8},
{0x1EE9,Ll,0x1EE8,0x1EE9,0x1EE8},
{0x1EEA,Lu,0x1EEA,0x1EEB,0x1EEA},
{0x1EEB,Ll,0x1EEA,0x1EEB,0x1EEA},
{0x1EEC,Lu,0x1EEC,0x1EED,0x1EEC},
{0x1EED,Ll,0x1EEC,0x1EED,0x1EEC},
{0x1EEE,Lu,0x1EEE,0x1EEF,0x1EEE},
{0x1EEF,Ll,0x1EEE,0x1EEF,0x1EEE},
{0x1EF0,Lu,0x1EF0,0x1EF1,0x1EF0},
{0x1EF1,Ll,0x1EF0,0x1EF1,0x1EF0},
{0x1EF2,Lu,0x1EF2,0x1EF3,0x1EF2},
{0x1EF3,Ll,0x1EF2,0x1EF3,0x1EF2},
{0x1EF4,Lu,0x1EF4,0x1EF5,0x1EF4},
{0x1EF5,Ll,0x1EF4,0x1EF5,0x1EF4},
{0x1EF6,Lu,0x1EF6,0x1EF7,0x1EF6},
{0x1EF7,Ll,0x1EF6,0x1EF7,0x1EF6},
{0x1EF8,Lu,0x1EF8,0x1EF9,0x1EF8},
{0x1EF9,Ll,0x1EF8,0x1EF9,0x1EF8},
{0x1F00,Ll,0x1F08,0x1F00,0x1F08},
{0x1F01,Ll,0x1F09,0x1F01,0x1F09},
{0x1F02,Ll,0x1F0A,0x1F02,0x1F0A},
{0x1F03,Ll,0x1F0B,0x1F03,0x1F0B},
{0x1F04,Ll,0x1F0C,0x1F04,0x1F0C},
{0x1F05,Ll,0x1F0D,0x1F05,0x1F0D},
{0x1F06,Ll,0x1F0E,0x1F06,0x1F0E},
{0x1F07,Ll,0x1F0F,0x1F07,0x1F0F},
{0x1F08,Lu,0x1F08,0x1F00,0x1F08},
{0x1F09,Lu,0x1F09,0x1F01,0x1F09},
{0x1F0A,Lu,0x1F0A,0x1F02,0x1F0A},
{0x1F0B,Lu,0x1F0B,0x1F03,0x1F0B},
{0x1F0C,Lu,0x1F0C,0x1F04,0x1F0C},
{0x1F0D,Lu,0x1F0D,0x1F05,0x1F0D},
{0x1F0E,Lu,0x1F0E,0x1F06,0x1F0E},
{0x1F0F,Lu,0x1F0F,0x1F07,0x1F0F},
{0x1F10,Ll,0x1F18,0x1F10,0x1F18},
{0x1F11,Ll,0x1F19,0x1F11,0x1F19},
{0x1F12,Ll,0x1F1A,0x1F12,0x1F1A},
{0x1F13,Ll,0x1F1B,0x1F13,0x1F1B},
{0x1F14,Ll,0x1F1C,0x1F14,0x1F1C},
{0x1F15,Ll,0x1F1D,0x1F15,0x1F1D},
{0x1F18,Lu,0x1F18,0x1F10,0x1F18},
{0x1F19,Lu,0x1F19,0x1F11,0x1F19},
{0x1F1A,Lu,0x1F1A,0x1F12,0x1F1A},
{0x1F1B,Lu,0x1F1B,0x1F13,0x1F1B},
{0x1F1C,Lu,0x1F1C,0x1F14,0x1F1C},
{0x1F1D,Lu,0x1F1D,0x1F15,0x1F1D},
{0x1F20,Ll,0x1F28,0x1F20,0x1F28},
{0x1F21,Ll,0x1F29,0x1F21,0x1F29},
{0x1F22,Ll,0x1F2A,0x1F22,0x1F2A},
{0x1F23,Ll,0x1F2B,0x1F23,0x1F2B},
{0x1F24,Ll,0x1F2C,0x1F24,0x1F2C},
{0x1F25,Ll,0x1F2D,0x1F25,0x1F2D},
{0x1F26,Ll,0x1F2E,0x1F26,0x1F2E},
{0x1F27,Ll,0x1F2F,0x1F27,0x1F2F},
{0x1F28,Lu,0x1F28,0x1F20,0x1F28},
{0x1F29,Lu,0x1F29,0x1F21,0x1F29},
{0x1F2A,Lu,0x1F2A,0x1F22,0x1F2A},
{0x1F2B,Lu,0x1F2B,0x1F23,0x1F2B},
{0x1F2C,Lu,0x1F2C,0x1F24,0x1F2C},
{0x1F2D,Lu,0x1F2D,0x1F25,0x1F2D},
{0x1F2E,Lu,0x1F2E,0x1F26,0x1F2E},
{0x1F2F,Lu,0x1F2F,0x1F27,0x1F2F},
{0x1F30,Ll,0x1F38,0x1F30,0x1F38},
{0x1F31,Ll,0x1F39,0x1F31,0x1F39},
{0x1F32,Ll,0x1F3A,0x1F32,0x1F3A},
{0x1F33,Ll,0x1F3B,0x1F33,0x1F3B},
{0x1F34,Ll,0x1F3C,0x1F34,0x1F3C},
{0x1F35,Ll,0x1F3D,0x1F35,0x1F3D},
{0x1F36,Ll,0x1F3E,0x1F36,0x1F3E},
{0x1F37,Ll,0x1F3F,0x1F37,0x1F3F},
{0x1F38,Lu,0x1F38,0x1F30,0x1F38},
{0x1F39,Lu,0x1F39,0x1F31,0x1F39},
{0x1F3A,Lu,0x1F3A,0x1F32,0x1F3A},
{0x1F3B,Lu,0x1F3B,0x1F33,0x1F3B},
{0x1F3C,Lu,0x1F3C,0x1F34,0x1F3C},
{0x1F3D,Lu,0x1F3D,0x1F35,0x1F3D},
{0x1F3E,Lu,0x1F3E,0x1F36,0x1F3E},
{0x1F3F,Lu,0x1F3F,0x1F37,0x1F3F},
{0x1F40,Ll,0x1F48,0x1F40,0x1F48},
{0x1F41,Ll,0x1F49,0x1F41,0x1F49},
{0x1F42,Ll,0x1F4A,0x1F42,0x1F4A},
{0x1F43,Ll,0x1F4B,0x1F43,0x1F4B},
{0x1F44,Ll,0x1F4C,0x1F44,0x1F4C},
{0x1F45,Ll,0x1F4D,0x1F45,0x1F4D},
{0x1F48,Lu,0x1F48,0x1F40,0x1F48},
{0x1F49,Lu,0x1F49,0x1F41,0x1F49},
{0x1F4A,Lu,0x1F4A,0x1F42,0x1F4A},
{0x1F4B,Lu,0x1F4B,0x1F43,0x1F4B},
{0x1F4C,Lu,0x1F4C,0x1F44,0x1F4C},
{0x1F4D,Lu,0x1F4D,0x1F45,0x1F4D},
{0x1F50,Ll | SM,0x1F50,0x1F50,0x1F50},
{0x1F51,Ll,0x1F59,0x1F51,0x1F59},
{0x1F52,Ll | SM,0x1F52,0x1F52,0x1F52},
{0x1F53,Ll,0x1F5B,0x1F53,0x1F5B},
{0x1F54,Ll | SM,0x1F54,0x1F54,0x1F54},
{0x1F55,Ll,0x1F5D,0x1F55,0x1F5D},
{0x1F56,Ll | SM,0x1F56,0x1F56,0x1F56},
{0x1F57,Ll,0x1F5F,0x1F57,0x1F5F},
{0x1F59,Lu,0x1F59,0x1F51,0x1F59},
{0x1F5B,Lu,0x1F5B,0x1F53,0x1F5B},
{0x1F5D,Lu,0x1F5D,0x1F55,0x1F5D},
{0x1F5F,Lu,0x1F5F,0x1F57,0x1F5F},
{0x1F60,Ll,0x1F68,0x1F60,0x1F68},
{0x1F61,Ll,0x1F69,0x1F61,0x1F69},
{0x1F62,Ll,0x1F6A,0x1F62,0x1F6A},
{0x1F63,Ll,0x1F6B,0x1F63,0x1F6B},
{0x1F64,Ll,0x1F6C,0x1F64,0x1F6C},
{0x1F65,Ll,0x1F6D,0x1F65,0x1F6D},
{0x1F66,Ll,0x1F6E,0x1F66,0x1F6E},
{0x1F67,Ll,0x1F6F,0x1F67,0x1F6F},
{0x1F68,Lu,0x1F68,0x1F60,0x1F68},
{0x1F69,Lu,0x1F69,0x1F61,0x1F69},
{0x1F6A,Lu,0x1F6A,0x1F62,0x1F6A},
{0x1F6B,Lu,0x1F6B,0x1F63,0x1F6B},
{0x1F6C,Lu,0x1F6C,0x1F64,0x1F6C},
{0x1F6D,Lu,0x1F6D,0x1F65,0x1F6D},
{0x1F6E,Lu,0x1F6E,0x1F66,0x1F6E},
{0x1F6F,Lu,0x1F6F,0x1F67,0x1F6F},
{0x1F70,Ll,0x1FBA,0x1F70,0x1FBA},
{0x1F71,Ll,0x1FBB,0x1F71,0x1FBB},
{0x1F72,Ll,0x1FC8,0x1F72,0x1FC8},
{0x1F73,Ll,0x1FC9,0x1F73,0x1FC9},
{0x1F74,Ll,0x1FCA,0x1F74,0x1FCA},
{0x1F75,Ll,0x1FCB,0x1F75,0x1FCB},
{0x1F76,Ll,0x1FDA,0x1F76,0x1FDA},
{0x1F77,Ll,0x1FDB,0x1F77,0x1FDB},
{0x1F78,Ll,0x1FF8,0x1F78,0x1FF8},
{0x1F79,Ll,0x1FF9,0x1F79,0x1FF9},
{0x1F7A,Ll,0x1FEA,0x1F7A,0x1FEA},
{0x1F7B,Ll,0x1FEB,0x1F7B,0x1FEB},
{0x1F7C,Ll,0x1FFA,0x1F7C,0x1FFA},
{0x1F7D,Ll,0x1FFB,0x1F7D,0x1FFB},
{0x1F80,Ll | SM,0x1F88,0x1F80,0x1F88},
{0x1F81,Ll | SM,0x1F89,0x1F81,0x1F89},
{0x1F82,Ll | SM,0x1F8A,0x1F82,0x1F8A},
{0x1F83,Ll | SM,0x1F8B,0x1F83,0x1F8B},
{0x1F84,Ll | SM,0x1F8C,0x1F84,0x1F8C},
{0x1F85,Ll | SM,0x1F8D,0x1F85,0x1F8D},
{0x1F86,Ll | SM,0x1F8E,0x1F86,0x1F8E},
{0x1F87,Ll | SM,0x1F8F,0x1F87,0x1F8F},
{0x1F88,Lt | SM,0x1F88,0x1F80,0x1F88},
{0x1F89,Lt | SM,0x1F89,0x1F81,0x1F89},
{0x1F8A,Lt | SM,0x1F8A,0x1F82,0x1F8A},
{0x1F8B,Lt | SM,0x1F8B,0x1F83,0x1F8B},
{0x1F8C,Lt | SM,0x1F8C,0x1F84,0x1F8C},
{0x1F8D,Lt | SM,0x1F8D,0x1F85,0x1F8D},
{0x1F8E,Lt | SM,0x1F8E,0x1F86,0x1F8E},
{0x1F8F,Lt | SM,0x1F8F,0x1F87,0x1F8F},
{0x1F90,Ll | SM,0x1F98,0x1F90,0x1F98},
{0x1F91,Ll | SM,0x1F99,0x1F91,0x1F99},
{0x1F92,Ll | SM,0x1F9A,0x1F92,0x1F9A},
{0x1F93,Ll | SM,0x1F9B,0x1F93,0x1F9B},
{0x1F94,Ll | SM,0x1F9C,0x1F94,0x1F9C},
{0x1F95,Ll | SM,0x1F9D,0x1F95,0x1F9D},
{0x1F96,Ll | SM,0x1F9E,0x1F96,0x1F9E},
{0x1F97,Ll | SM,0x1F9F,0x1F97,0x1F9F},
{0x1F98,Lt | SM,0x1F98,0x1F90,0x1F98},
{0x1F99,Lt | SM,0x1F99,0x1F91,0x1F99},
{0x1F9A,Lt | SM,0x1F9A,0x1F92,0x1F9A},
{0x1F9B,Lt | SM,0x1F9B,0x1F93,0x1F9B},
{0x1F9C,Lt | SM,0x1F9C,0x1F94,0x1F9C},
{0x1F9D,Lt | SM,0x1F9D,0x1F95,0x1F9D},
{0x1F9E,Lt | SM,0x1F9E,0x1F96,0x1F9E},
{0x1F9F,Lt | SM,0x1F9F,0x1F97,0x1F9F},
{0x1FA0,Ll | SM,0x1FA8,0x1FA0,0x1FA8},
{0x1FA1,Ll | SM,0x1FA9,0x1FA1,0x1FA9},
{0x1FA2,Ll | SM,0x1FAA,0x1FA2,0x1FAA},
{0x1FA3,Ll | SM,0x1FAB,0x1FA3,0x1FAB},
{0x1FA4,Ll | SM,0x1FAC,0x1FA4,0x1FAC},
{0x1FA5,Ll | SM,0x1FAD,0x1FA5,0x1FAD},
{0x1FA6,Ll | SM,0x1FAE,0x1FA6,0x1FAE},
{0x1FA7,Ll | SM,0x1FAF,0x1FA7,0x1FAF},
{0x1FA8,Lt | SM,0x1FA8,0x1FA0,0x1FA8},
{0x1FA9,Lt | SM,0x1FA9,0x1FA1,0x1FA9},
{0x1FAA,Lt | SM,0x1FAA,0x1FA2,0x1FAA},
{0x1FAB,Lt | SM,0x1FAB,0x1FA3,0x1FAB},
{0x1FAC,Lt | SM,0x1FAC,0x1FA4,0x1FAC},
{0x1FAD,Lt | SM,0x1FAD,0x1FA5,0x1FAD},
{0x1FAE,Lt | SM,0x1FAE,0x1FA6,0x1FAE},
{0x1FAF,Lt | SM,0x1FAF,0x1FA7,0x1FAF},
{0x1FB0,Ll,0x1FB8,0x1FB0,0x1FB8},
{0x1FB1,Ll,0x1FB9,0x1FB1,0x1FB9},
{0x1FB2,Ll | SM,0x1FB2,0x1FB2,0x1FB2},
{0x1FB3,Ll | SM,0x1FBC,0x1FB3,0x1FBC},
{0x1FB4,Ll | SM,0x1FB4,0x1FB4,0x1FB4},
{0x1FB6,Ll | SM,0x1FB6,0x1FB6,0x1FB6},
{0x1FB7,Ll | SM,0x1FB7,0x1FB7,0x1FB7},
{0x1FB8,Lu,0x1FB8,0x1FB0,0x1FB8},
{0x1FB9,Lu,0x1FB9,0x1FB1,0x1FB9},
{0x1FBA,Lu,0x1FBA,0x1F70,0x1FBA},
{0x1FBB,Lu,0x1FBB,0x1F71,0x1FBB},
{0x1FBC,Lt | SM,0x1FBC,0x1FB3,0x1FBC},
{0x1FBD,Sk,0x1FBD,0x1FBD,0x1FBD},
{0x1FBE,Ll,0x399,0x1FBE,0x399},
{0x1FBF,Sk,0x1FBF,0x1FBF,0x1FBF},
{0x1FC0,Sk,0x1FC0,0x1FC0,0x1FC0},
{0x1FC1,Sk,0x1FC1,0x1FC1,0x1FC1},
{0x1FC2,Ll | SM,0x1FC2,0x1FC2,0x1FC2},
{0x1FC3,Ll | SM,0x1FCC,0x1FC3,0x1FCC},
{0x1FC4,Ll | SM,0x1FC4,0x1FC4,0x1FC4},
{0x1FC6,Ll | SM,0x1FC6,0x1FC6,0x1FC6},
{0x1FC7,Ll | SM,0x1FC7,0x1FC7,0x1FC7},
{0x1FC8,Lu,0x1FC8,0x1F72,0x1FC8},
{0x1FC9,Lu,0x1FC9,0x1F73,0x1FC9},
{0x1FCA,Lu,0x1FCA,0x1F74,0x1FCA},
{0x1FCB,Lu,0x1FCB,0x1F75,0x1FCB},
{0x1FCC,Lt | SM,0x1FCC,0x1FC3,0x1FCC},
{0x1FCD,Sk,0x1FCD,0x1FCD,0x1FCD},
{0x1FCE,Sk,0x1FCE,0x1FCE,0x1FCE},
{0x1FCF,Sk,0x1FCF,0x1FCF,0x1FCF},
{0x1FD0,Ll,0x1FD8,0x1FD0,0x1FD8},
{0x1FD1,Ll,0x1FD9,0x1FD1,0x1FD9},
{0x1FD2,Ll | SM,0x1FD2,0x1FD2,0x1FD2},
{0x1FD3,Ll | SM,0x1FD3,0x1FD3,0x1FD3},
{0x1FD6,Ll | SM,0x1FD6,0x1FD6,0x1FD6},
{0x1FD7,Ll | SM,0x1FD7,0x1FD7,0x1FD7},
{0x1FD8,Lu,0x1FD8,0x1FD0,0x1FD8},
{0x1FD9,Lu,0x1FD9,0x1FD1,0x1FD9},
{0x1FDA,Lu,0x1FDA,0x1F76,0x1FDA},
{0x1FDB,Lu,0x1FDB,0x1F77,0x1FDB},
{0x1FDD,Sk,0x1FDD,0x1FDD,0x1FDD},
{0x1FDE,Sk,0x1FDE,0x1FDE,0x1FDE},
{0x1FDF,Sk,0x1FDF,0x1FDF,0x1FDF},
{0x1FE0,Ll,0x1FE8,0x1FE0,0x1FE8},
{0x1FE1,Ll,0x1FE9,0x1FE1,0x1FE9},
{0x1FE2,Ll | SM,0x1FE2,0x1FE2,0x1FE2},
{0x1FE3,Ll | SM,0x1FE3,0x1FE3,0x1FE3},
{0x1FE4,Ll | SM,0x1FE4,0x1FE4,0x1FE4},
{0x1FE5,Ll,0x1FEC,0x1FE5,0x1FEC},
{0x1FE6,Ll | SM,0x1FE6,0x1FE6,0x1FE6},
{0x1FE7,Ll | SM,0x1FE7,0x1FE7,0x1FE7},
{0x1FE8,Lu,0x1FE8,0x1FE0,0x1FE8},
{0x1FE9,Lu,0x1FE9,0x1FE1,0x1FE9},
{0x1FEA,Lu,0x1FEA,0x1F7A,0x1FEA},
{0x1FEB,Lu,0x1FEB,0x1F7B,0x1FEB},
{0x1FEC,Lu,0x1FEC,0x1FE5,0x1FEC},
{0x1FED,Sk,0x1FED,0x1FED,0x1FED},
{0x1FEE,Sk,0x1FEE,0x1FEE,0x1FEE},
{0x1FEF,Sk,0x1FEF,0x1FEF,0x1FEF},
{0x1FF2,Ll | SM,0x1FF2,0x1FF2,0x1FF2},
{0x1FF3,Ll | SM,0x1FFC,0x1FF3,0x1FFC},
{0x1FF4,Ll | SM,0x1FF4,0x1FF4,0x1FF4},
{0x1FF6,Ll | SM,0x1FF6,0x1FF6,0x1FF6},
{0x1FF7,Ll | SM,0x1FF7,0x1FF7,0x1FF7},
{0x1FF8,Lu,0x1FF8,0x1F78,0x1FF8},
{0x1FF9,Lu,0x1FF9,0x1F79,0x1FF9},
{0x1FFA,Lu,0x1FFA,0x1F7C,0x1FFA},
{0x1FFB,Lu,0x1FFB,0x1F7D,0x1FFB},
{0x1FFC,Lt | SM,0x1FFC,0x1FF3,0x1FFC},
{0x1FFD,Sk,0x1FFD,0x1FFD,0x1FFD},
{0x1FFE,Sk,0x1FFE,0x1FFE,0x1FFE},
{0x2000,Zs,0x2000,0x2000,0x2000},
{0x2001,Zs,0x2001,0x2001,0x2001},
{0x2002,Zs,0x2002,0x2002,0x2002},
{0x2003,Zs,0x2003,0x2003,0x2003},
{0x2004,Zs,0x2004,0x2004,0x2004},
{0x2005,Zs,0x2005,0x2005,0x2005},
{0x2006,Zs,0x2006,0x2006,0x2006},
{0x2007,Zs,0x2007,0x2007,0x2007},
{0x2008,Zs,0x2008,0x2008,0x2008},
{0x2009,Zs,0x2009,0x2009,0x2009},
{0x200A,Zs,0x200A,0x200A,0x200A},
{0x200B,Cf,0x200B,0x200B,0x200B},
{0x200C,Cf,0x200C,0x200C,0x200C},
{0x200D,Cf,0x200D,0x200D,0x200D},
{0x200E,Cf,0x200E,0x200E,0x200E},
{0x200F,Cf,0x200F,0x200F,0x200F},
{0x2010,Pd,0x2010,0x2010,0x2010},
{0x2011,Pd,0x2011,0x2011,0x2011},
{0x2012,Pd,0x2012,0x2012,0x2012},
{0x2013,Pd,0x2013,0x2013,0x2013},
{0x2014,Pd,0x2014,0x2014,0x2014},
{0x2015,Pd,0x2015,0x2015,0x2015},
{0x2016,Po,0x2016,0x2016,0x2016},
{0x2017,Po,0x2017,0x2017,0x2017},
{0x2018,Pi,0x2018,0x2018,0x2018},
{0x2019,Pf,0x2019,0x2019,0x2019},
{0x201A,Ps,0x201A,0x201A,0x201A},
{0x201B,Pi,0x201B,0x201B,0x201B},
{0x201C,Pi,0x201C,0x201C,0x201C},
{0x201D,Pf,0x201D,0x201D,0x201D},
{0x201E,Ps,0x201E,0x201E,0x201E},
{0x201F,Pi,0x201F,0x201F,0x201F},
{0x2020,Po,0x2020,0x2020,0x2020},
{0x2021,Po,0x2021,0x2021,0x2021},
{0x2022,Po,0x2022,0x2022,0x2022},
{0x2023,Po,0x2023,0x2023,0x2023},
{0x2024,Po,0x2024,0x2024,0x2024},
{0x2025,Po,0x2025,0x2025,0x2025},
{0x2026,Po,0x2026,0x2026,0x2026},
{0x2027,Po,0x2027,0x2027,0x2027},
{0x2028,Zl,0x2028,0x2028,0x2028},
{0x2029,Zp,0x2029,0x2029,0x2029},
{0x202A,Cf,0x202A,0x202A,0x202A},
{0x202B,Cf,0x202B,0x202B,0x202B},
{0x202C,Cf,0x202C,0x202C,0x202C},
{0x202D,Cf,0x202D,0x202D,0x202D},
{0x202E,Cf,0x202E,0x202E,0x202E},
{0x202F,Zs,0x202F,0x202F,0x202F},
{0x2030,Po,0x2030,0x2030,0x2030},
{0x2031,Po,0x2031,0x2031,0x2031},
{0x2032,Po,0x2032,0x2032,0x2032},
{0x2033,Po,0x2033,0x2033,0x2033},
{0x2034,Po,0x2034,0x2034,0x2034},
{0x2035,Po,0x2035,0x2035,0x2035},
{0x2036,Po,0x2036,0x2036,0x2036},
{0x2037,Po,0x2037,0x2037,0x2037},
{0x2038,Po,0x2038,0x2038,0x2038},
{0x2039,Pi,0x2039,0x2039,0x2039},
{0x203A,Pf,0x203A,0x203A,0x203A},
{0x203B,Po,0x203B,0x203B,0x203B},
{0x203C,Po,0x203C,0x203C,0x203C},
{0x203D,Po,0x203D,0x203D,0x203D},
{0x203E,Po,0x203E,0x203E,0x203E},
{0x203F,Pc,0x203F,0x203F,0x203F},
{0x2040,Pc,0x2040,0x2040,0x2040},
{0x2041,Po,0x2041,0x2041,0x2041},
{0x2042,Po,0x2042,0x2042,0x2042},
{0x2043,Po,0x2043,0x2043,0x2043},
{0x2044,Sm,0x2044,0x2044,0x2044},
{0x2045,Ps,0x2045,0x2045,0x2045},
{0x2046,Pe,0x2046,0x2046,0x2046},
{0x2047,Po,0x2047,0x2047,0x2047},
{0x2048,Po,0x2048,0x2048,0x2048},
{0x2049,Po,0x2049,0x2049,0x2049},
{0x204A,Po,0x204A,0x204A,0x204A},
{0x204B,Po,0x204B,0x204B,0x204B},
{0x204C,Po,0x204C,0x204C,0x204C},
{0x204D,Po,0x204D,0x204D,0x204D},
{0x204E,Po,0x204E,0x204E,0x204E},
{0x204F,Po,0x204F,0x204F,0x204F},
{0x2050,Po,0x2050,0x2050,0x2050},
{0x2051,Po,0x2051,0x2051,0x2051},
{0x2052,Sm,0x2052,0x2052,0x2052},
{0x2053,Po,0x2053,0x2053,0x2053},
{0x2054,Pc,0x2054,0x2054,0x2054},
{0x2055,Po,0x2055,0x2055,0x2055},
{0x2056,Po,0x2056,0x2056,0x2056},
{0x2057,Po,0x2057,0x2057,0x2057},
{0x2058,Po,0x2058,0x2058,0x2058},
{0x2059,Po,0x2059,0x2059,0x2059},
{0x205A,Po,0x205A,0x205A,0x205A},
{0x205B,Po,0x205B,0x205B,0x205B},
{0x205C,Po,0x205C,0x205C,0x205C},
{0x205D,Po,0x205D,0x205D,0x205D},
{0x205E,Po,0x205E,0x205E,0x205E},
{0x205F,Zs,0x205F,0x205F,0x205F},
{0x2060,Cf,0x2060,0x2060,0x2060},
{0x2061,Cf,0x2061,0x2061,0x2061},
{0x2062,Cf,0x2062,0x2062,0x2062},
{0x2063,Cf,0x2063,0x2063,0x2063},
{0x206A,Cf,0x206A,0x206A,0x206A},
{0x206B,Cf,0x206B,0x206B,0x206B},
{0x206C,Cf,0x206C,0x206C,0x206C},
{0x206D,Cf,0x206D,0x206D,0x206D},
{0x206E,Cf,0x206E,0x206E,0x206E},
{0x206F,Cf,0x206F,0x206F,0x206F},
{0x2070,No,0x2070,0x2070,0x2070},
{0x2071,Ll,0x2071,0x2071,0x2071},
{0x2074,No,0x2074,0x2074,0x2074},
{0x2075,No,0x2075,0x2075,0x2075},
{0x2076,No,0x2076,0x2076,0x2076},
{0x2077,No,0x2077,0x2077,0x2077},
{0x2078,No,0x2078,0x2078,0x2078},
{0x2079,No,0x2079,0x2079,0x2079},
{0x207A,Sm,0x207A,0x207A,0x207A},
{0x207B,Sm,0x207B,0x207B,0x207B},
{0x207C,Sm,0x207C,0x207C,0x207C},
{0x207D,Ps,0x207D,0x207D,0x207D},
{0x207E,Pe,0x207E,0x207E,0x207E},
{0x207F,Ll,0x207F,0x207F,0x207F},
{0x2080,No,0x2080,0x2080,0x2080},
{0x2081,No,0x2081,0x2081,0x2081},
{0x2082,No,0x2082,0x2082,0x2082},
{0x2083,No,0x2083,0x2083,0x2083},
{0x2084,No,0x2084,0x2084,0x2084},
{0x2085,No,0x2085,0x2085,0x2085},
{0x2086,No,0x2086,0x2086,0x2086},
{0x2087,No,0x2087,0x2087,0x2087},
{0x2088,No,0x2088,0x2088,0x2088},
{0x2089,No,0x2089,0x2089,0x2089},
{0x208A,Sm,0x208A,0x208A,0x208A},
{0x208B,Sm,0x208B,0x208B,0x208B},
{0x208C,Sm,0x208C,0x208C,0x208C},
{0x208D,Ps,0x208D,0x208D,0x208D},
{0x208E,Pe,0x208E,0x208E,0x208E},
{0x2090,Lm,0x2090,0x2090,0x2090},
{0x2091,Lm,0x2091,0x2091,0x2091},
{0x2092,Lm,0x2092,0x2092,0x2092},
{0x2093,Lm,0x2093,0x2093,0x2093},
{0x2094,Lm,0x2094,0x2094,0x2094},
{0x20A0,Sc,0x20A0,0x20A0,0x20A0},
{0x20A1,Sc,0x20A1,0x20A1,0x20A1},
{0x20A2,Sc,0x20A2,0x20A2,0x20A2},
{0x20A3,Sc,0x20A3,0x20A3,0x20A3},
{0x20A4,Sc,0x20A4,0x20A4,0x20A4},
{0x20A5,Sc,0x20A5,0x20A5,0x20A5},
{0x20A6,Sc,0x20A6,0x20A6,0x20A6},
{0x20A7,Sc,0x20A7,0x20A7,0x20A7},
{0x20A8,Sc,0x20A8,0x20A8,0x20A8},
{0x20A9,Sc,0x20A9,0x20A9,0x20A9},
{0x20AA,Sc,0x20AA,0x20AA,0x20AA},
{0x20AB,Sc,0x20AB,0x20AB,0x20AB},
{0x20AC,Sc,0x20AC,0x20AC,0x20AC},
{0x20AD,Sc,0x20AD,0x20AD,0x20AD},
{0x20AE,Sc,0x20AE,0x20AE,0x20AE},
{0x20AF,Sc,0x20AF,0x20AF,0x20AF},
{0x20B0,Sc,0x20B0,0x20B0,0x20B0},
{0x20B1,Sc,0x20B1,0x20B1,0x20B1},
{0x20B2,Sc,0x20B2,0x20B2,0x20B2},
{0x20B3,Sc,0x20B3,0x20B3,0x20B3},
{0x20B4,Sc,0x20B4,0x20B4,0x20B4},
{0x20B5,Sc,0x20B5,0x20B5,0x20B5},
{0x20D0,Mn,0x20D0,0x20D0,0x20D0},
{0x20D1,Mn,0x20D1,0x20D1,0x20D1},
{0x20D2,Mn,0x20D2,0x20D2,0x20D2},
{0x20D3,Mn,0x20D3,0x20D3,0x20D3},
{0x20D4,Mn,0x20D4,0x20D4,0x20D4},
{0x20D5,Mn,0x20D5,0x20D5,0x20D5},
{0x20D6,Mn,0x20D6,0x20D6,0x20D6},
{0x20D7,Mn,0x20D7,0x20D7,0x20D7},
{0x20D8,Mn,0x20D8,0x20D8,0x20D8},
{0x20D9,Mn,0x20D9,0x20D9,0x20D9},
{0x20DA,Mn,0x20DA,0x20DA,0x20DA},
{0x20DB,Mn,0x20DB,0x20DB,0x20DB},
{0x20DC,Mn,0x20DC,0x20DC,0x20DC},
{0x20DD,Me,0x20DD,0x20DD,0x20DD},
{0x20DE,Me,0x20DE,0x20DE,0x20DE},
{0x20DF,Me,0x20DF,0x20DF,0x20DF},
{0x20E0,Me,0x20E0,0x20E0,0x20E0},
{0x20E1,Mn,0x20E1,0x20E1,0x20E1},
{0x20E2,Me,0x20E2,0x20E2,0x20E2},
{0x20E3,Me,0x20E3,0x20E3,0x20E3},
{0x20E4,Me,0x20E4,0x20E4,0x20E4},
{0x20E5,Mn,0x20E5,0x20E5,0x20E5},
{0x20E6,Mn,0x20E6,0x20E6,0x20E6},
{0x20E7,Mn,0x20E7,0x20E7,0x20E7},
{0x20E8,Mn,0x20E8,0x20E8,0x20E8},
{0x20E9,Mn,0x20E9,0x20E9,0x20E9},
{0x20EA,Mn,0x20EA,0x20EA,0x20EA},
{0x20EB,Mn,0x20EB,0x20EB,0x20EB},
{0x20EC,Mn,0x20EC,0x20EC,0x20EC},
{0x20ED,Mn,0x20ED,0x20ED,0x20ED},
{0x20EE,Mn,0x20EE,0x20EE,0x20EE},
{0x20EF,Mn,0x20EF,0x20EF,0x20EF},
{0x2100,So,0x2100,0x2100,0x2100},
{0x2101,So,0x2101,0x2101,0x2101},
{0x2102,Lu,0x2102,0x2102,0x2102},
{0x2103,So,0x2103,0x2103,0x2103},
{0x2104,So,0x2104,0x2104,0x2104},
{0x2105,So,0x2105,0x2105,0x2105},
{0x2106,So,0x2106,0x2106,0x2106},
{0x2107,Lu,0x2107,0x2107,0x2107},
{0x2108,So,0x2108,0x2108,0x2108},
{0x2109,So,0x2109,0x2109,0x2109},
{0x210A,Ll,0x210A,0x210A,0x210A},
{0x210B,Lu,0x210B,0x210B,0x210B},
{0x210C,Lu,0x210C,0x210C,0x210C},
{0x210D,Lu,0x210D,0x210D,0x210D},
{0x210E,Ll,0x210E,0x210E,0x210E},
{0x210F,Ll,0x210F,0x210F,0x210F},
{0x2110,Lu,0x2110,0x2110,0x2110},
{0x2111,Lu,0x2111,0x2111,0x2111},
{0x2112,Lu,0x2112,0x2112,0x2112},
{0x2113,Ll,0x2113,0x2113,0x2113},
{0x2114,So,0x2114,0x2114,0x2114},
{0x2115,Lu,0x2115,0x2115,0x2115},
{0x2116,So,0x2116,0x2116,0x2116},
{0x2117,So,0x2117,0x2117,0x2117},
{0x2118,So,0x2118,0x2118,0x2118},
{0x2119,Lu,0x2119,0x2119,0x2119},
{0x211A,Lu,0x211A,0x211A,0x211A},
{0x211B,Lu,0x211B,0x211B,0x211B},
{0x211C,Lu,0x211C,0x211C,0x211C},
{0x211D,Lu,0x211D,0x211D,0x211D},
{0x211E,So,0x211E,0x211E,0x211E},
{0x211F,So,0x211F,0x211F,0x211F},
{0x2120,So,0x2120,0x2120,0x2120},
{0x2121,So,0x2121,0x2121,0x2121},
{0x2122,So,0x2122,0x2122,0x2122},
{0x2123,So,0x2123,0x2123,0x2123},
{0x2124,Lu,0x2124,0x2124,0x2124},
{0x2125,So,0x2125,0x2125,0x2125},
{0x2126,Lu,0x2126,0x3C9,0x2126},
{0x2127,So,0x2127,0x2127,0x2127},
{0x2128,Lu,0x2128,0x2128,0x2128},
{0x2129,So,0x2129,0x2129,0x2129},
{0x212A,Lu,0x212A,0x6B,0x212A},
{0x212B,Lu,0x212B,0xE5,0x212B},
{0x212C,Lu,0x212C,0x212C,0x212C},
{0x212D,Lu,0x212D,0x212D,0x212D},
{0x212E,So,0x212E,0x212E,0x212E},
{0x212F,Ll,0x212F,0x212F,0x212F},
{0x2130,Lu,0x2130,0x2130,0x2130},
{0x2131,Lu,0x2131,0x2131,0x2131},
{0x2132,Lu,0x2132,0x214E,0x2132},
{0x2133,Lu,0x2133,0x2133,0x2133},
{0x2134,Ll,0x2134,0x2134,0x2134},
{0x2135,Lo,0x2135,0x2135,0x2135},
{0x2136,Lo,0x2136,0x2136,0x2136},
{0x2137,Lo,0x2137,0x2137,0x2137},
{0x2138,Lo,0x2138,0x2138,0x2138},
{0x2139,Ll,0x2139,0x2139,0x2139},
{0x213A,So,0x213A,0x213A,0x213A},
{0x213B,So,0x213B,0x213B,0x213B},
{0x213C,Ll,0x213C,0x213C,0x213C},
{0x213D,Ll,0x213D,0x213D,0x213D},
{0x213E,Lu,0x213E,0x213E,0x213E},
{0x213F,Lu,0x213F,0x213F,0x213F},
{0x2140,Sm,0x2140,0x2140,0x2140},
{0x2141,Sm,0x2141,0x2141,0x2141},
{0x2142,Sm,0x2142,0x2142,0x2142},
{0x2143,Sm,0x2143,0x2143,0x2143},
{0x2144,Sm,0x2144,0x2144,0x2144},
{0x2145,Lu,0x2145,0x2145,0x2145},
{0x2146,Ll,0x2146,0x2146,0x2146},
{0x2147,Ll,0x2147,0x2147,0x2147},
{0x2148,Ll,0x2148,0x2148,0x2148},
{0x2149,Ll,0x2149,0x2149,0x2149},
{0x214A,So,0x214A,0x214A,0x214A},
{0x214B,Sm,0x214B,0x214B,0x214B},
{0x214C,So,0x214C,0x214C,0x214C},
{0x214D,So,0x214D,0x214D,0x214D},
{0x214E,Ll,0x2132,0x214E,0x2132},
{0x2153,No,0x2153,0x2153,0x2153},
{0x2154,No,0x2154,0x2154,0x2154},
{0x2155,No,0x2155,0x2155,0x2155},
{0x2156,No,0x2156,0x2156,0x2156},
{0x2157,No,0x2157,0x2157,0x2157},
{0x2158,No,0x2158,0x2158,0x2158},
{0x2159,No,0x2159,0x2159,0x2159},
{0x215A,No,0x215A,0x215A,0x215A},
{0x215B,No,0x215B,0x215B,0x215B},
{0x215C,No,0x215C,0x215C,0x215C},
{0x215D,No,0x215D,0x215D,0x215D},
{0x215E,No,0x215E,0x215E,0x215E},
{0x215F,No,0x215F,0x215F,0x215F},
{0x2160,Nl,0x2160,0x2170,0x2160},
{0x2161,Nl,0x2161,0x2171,0x2161},
{0x2162,Nl,0x2162,0x2172,0x2162},
{0x2163,Nl,0x2163,0x2173,0x2163},
{0x2164,Nl,0x2164,0x2174,0x2164},
{0x2165,Nl,0x2165,0x2175,0x2165},
{0x2166,Nl,0x2166,0x2176,0x2166},
{0x2167,Nl,0x2167,0x2177,0x2167},
{0x2168,Nl,0x2168,0x2178,0x2168},
{0x2169,Nl,0x2169,0x2179,0x2169},
{0x216A,Nl,0x216A,0x217A,0x216A},
{0x216B,Nl,0x216B,0x217B,0x216B},
{0x216C,Nl,0x216C,0x217C,0x216C},
{0x216D,Nl,0x216D,0x217D,0x216D},
{0x216E,Nl,0x216E,0x217E,0x216E},
{0x216F,Nl,0x216F,0x217F,0x216F},
{0x2170,Nl,0x2160,0x2170,0x2160},
{0x2171,Nl,0x2161,0x2171,0x2161},
{0x2172,Nl,0x2162,0x2172,0x2162},
{0x2173,Nl,0x2163,0x2173,0x2163},
{0x2174,Nl,0x2164,0x2174,0x2164},
{0x2175,Nl,0x2165,0x2175,0x2165},
{0x2176,Nl,0x2166,0x2176,0x2166},
{0x2177,Nl,0x2167,0x2177,0x2167},
{0x2178,Nl,0x2168,0x2178,0x2168},
{0x2179,Nl,0x2169,0x2179,0x2169},
{0x217A,Nl,0x216A,0x217A,0x216A},
{0x217B,Nl,0x216B,0x217B,0x216B},
{0x217C,Nl,0x216C,0x217C,0x216C},
{0x217D,Nl,0x216D,0x217D,0x216D},
{0x217E,Nl,0x216E,0x217E,0x216E},
{0x217F,Nl,0x216F,0x217F,0x216F},
{0x2180,Nl,0x2180,0x2180,0x2180},
{0x2181,Nl,0x2181,0x2181,0x2181},
{0x2182,Nl,0x2182,0x2182,0x2182},
{0x2183,Lu,0x2183,0x2184,0x2183},
{0x2184,Ll,0x2183,0x2184,0x2183},
{0x2190,Sm,0x2190,0x2190,0x2190},
{0x2191,Sm,0x2191,0x2191,0x2191},
{0x2192,Sm,0x2192,0x2192,0x2192},
{0x2193,Sm,0x2193,0x2193,0x2193},
{0x2194,Sm,0x2194,0x2194,0x2194},
{0x2195,So,0x2195,0x2195,0x2195},
{0x2196,So,0x2196,0x2196,0x2196},
{0x2197,So,0x2197,0x2197,0x2197},
{0x2198,So,0x2198,0x2198,0x2198},
{0x2199,So,0x2199,0x2199,0x2199},
{0x219A,Sm,0x219A,0x219A,0x219A},
{0x219B,Sm,0x219B,0x219B,0x219B},
{0x219C,So,0x219C,0x219C,0x219C},
{0x219D,So,0x219D,0x219D,0x219D},
{0x219E,So,0x219E,0x219E,0x219E},
{0x219F,So,0x219F,0x219F,0x219F},
{0x21A0,Sm,0x21A0,0x21A0,0x21A0},
{0x21A1,So,0x21A1,0x21A1,0x21A1},
{0x21A2,So,0x21A2,0x21A2,0x21A2},
{0x21A3,Sm,0x21A3,0x21A3,0x21A3},
{0x21A4,So,0x21A4,0x21A4,0x21A4},
{0x21A5,So,0x21A5,0x21A5,0x21A5},
{0x21A6,Sm,0x21A6,0x21A6,0x21A6},
{0x21A7,So,0x21A7,0x21A7,0x21A7},
{0x21A8,So,0x21A8,0x21A8,0x21A8},
{0x21A9,So,0x21A9,0x21A9,0x21A9},
{0x21AA,So,0x21AA,0x21AA,0x21AA},
{0x21AB,So,0x21AB,0x21AB,0x21AB},
{0x21AC,So,0x21AC,0x21AC,0x21AC},
{0x21AD,So,0x21AD,0x21AD,0x21AD},
{0x21AE,Sm,0x21AE,0x21AE,0x21AE},
{0x21AF,So,0x21AF,0x21AF,0x21AF},
{0x21B0,So,0x21B0,0x21B0,0x21B0},
{0x21B1,So,0x21B1,0x21B1,0x21B1},
{0x21B2,So,0x21B2,0x21B2,0x21B2},
{0x21B3,So,0x21B3,0x21B3,0x21B3},
{0x21B4,So,0x21B4,0x21B4,0x21B4},
{0x21B5,So,0x21B5,0x21B5,0x21B5},
{0x21B6,So,0x21B6,0x21B6,0x21B6},
{0x21B7,So,0x21B7,0x21B7,0x21B7},
{0x21B8,So,0x21B8,0x21B8,0x21B8},
{0x21B9,So,0x21B9,0x21B9,0x21B9},
{0x21BA,So,0x21BA,0x21BA,0x21BA},
{0x21BB,So,0x21BB,0x21BB,0x21BB},
{0x21BC,So,0x21BC,0x21BC,0x21BC},
{0x21BD,So,0x21BD,0x21BD,0x21BD},
{0x21BE,So,0x21BE,0x21BE,0x21BE},
{0x21BF,So,0x21BF,0x21BF,0x21BF},
{0x21C0,So,0x21C0,0x21C0,0x21C0},
{0x21C1,So,0x21C1,0x21C1,0x21C1},
{0x21C2,So,0x21C2,0x21C2,0x21C2},
{0x21C3,So,0x21C3,0x21C3,0x21C3},
{0x21C4,So,0x21C4,0x21C4,0x21C4},
{0x21C5,So,0x21C5,0x21C5,0x21C5},
{0x21C6,So,0x21C6,0x21C6,0x21C6},
{0x21C7,So,0x21C7,0x21C7,0x21C7},
{0x21C8,So,0x21C8,0x21C8,0x21C8},
{0x21C9,So,0x21C9,0x21C9,0x21C9},
{0x21CA,So,0x21CA,0x21CA,0x21CA},
{0x21CB,So,0x21CB,0x21CB,0x21CB},
{0x21CC,So,0x21CC,0x21CC,0x21CC},
{0x21CD,So,0x21CD,0x21CD,0x21CD},
{0x21CE,Sm,0x21CE,0x21CE,0x21CE},
{0x21CF,Sm,0x21CF,0x21CF,0x21CF},
{0x21D0,So,0x21D0,0x21D0,0x21D0},
{0x21D1,So,0x21D1,0x21D1,0x21D1},
{0x21D2,Sm,0x21D2,0x21D2,0x21D2},
{0x21D3,So,0x21D3,0x21D3,0x21D3},
{0x21D4,Sm,0x21D4,0x21D4,0x21D4},
{0x21D5,So,0x21D5,0x21D5,0x21D5},
{0x21D6,So,0x21D6,0x21D6,0x21D6},
{0x21D7,So,0x21D7,0x21D7,0x21D7},
{0x21D8,So,0x21D8,0x21D8,0x21D8},
{0x21D9,So,0x21D9,0x21D9,0x21D9},
{0x21DA,So,0x21DA,0x21DA,0x21DA},
{0x21DB,So,0x21DB,0x21DB,0x21DB},
{0x21DC,So,0x21DC,0x21DC,0x21DC},
{0x21DD,So,0x21DD,0x21DD,0x21DD},
{0x21DE,So,0x21DE,0x21DE,0x21DE},
{0x21DF,So,0x21DF,0x21DF,0x21DF},
{0x21E0,So,0x21E0,0x21E0,0x21E0},
{0x21E1,So,0x21E1,0x21E1,0x21E1},
{0x21E2,So,0x21E2,0x21E2,0x21E2},
{0x21E3,So,0x21E3,0x21E3,0x21E3},
{0x21E4,So,0x21E4,0x21E4,0x21E4},
{0x21E5,So,0x21E5,0x21E5,0x21E5},
{0x21E6,So,0x21E6,0x21E6,0x21E6},
{0x21E7,So,0x21E7,0x21E7,0x21E7},
{0x21E8,So,0x21E8,0x21E8,0x21E8},
{0x21E9,So,0x21E9,0x21E9,0x21E9},
{0x21EA,So,0x21EA,0x21EA,0x21EA},
{0x21EB,So,0x21EB,0x21EB,0x21EB},
{0x21EC,So,0x21EC,0x21EC,0x21EC},
{0x21ED,So,0x21ED,0x21ED,0x21ED},
{0x21EE,So,0x21EE,0x21EE,0x21EE},
{0x21EF,So,0x21EF,0x21EF,0x21EF},
{0x21F0,So,0x21F0,0x21F0,0x21F0},
{0x21F1,So,0x21F1,0x21F1,0x21F1},
{0x21F2,So,0x21F2,0x21F2,0x21F2},
{0x21F3,So,0x21F3,0x21F3,0x21F3},
{0x21F4,Sm,0x21F4,0x21F4,0x21F4},
{0x21F5,Sm,0x21F5,0x21F5,0x21F5},
{0x21F6,Sm,0x21F6,0x21F6,0x21F6},
{0x21F7,Sm,0x21F7,0x21F7,0x21F7},
{0x21F8,Sm,0x21F8,0x21F8,0x21F8},
{0x21F9,Sm,0x21F9,0x21F9,0x21F9},
{0x21FA,Sm,0x21FA,0x21FA,0x21FA},
{0x21FB,Sm,0x21FB,0x21FB,0x21FB},
{0x21FC,Sm,0x21FC,0x21FC,0x21FC},
{0x21FD,Sm,0x21FD,0x21FD,0x21FD},
{0x21FE,Sm,0x21FE,0x21FE,0x21FE},
{0x21FF,Sm,0x21FF,0x21FF,0x21FF},
{0x2200,Sm,0x2200,0x2200,0x2200},
{0x2201,Sm,0x2201,0x2201,0x2201},
{0x2202,Sm,0x2202,0x2202,0x2202},
{0x2203,Sm,0x2203,0x2203,0x2203},
{0x2204,Sm,0x2204,0x2204,0x2204},
{0x2205,Sm,0x2205,0x2205,0x2205},
{0x2206,Sm,0x2206,0x2206,0x2206},
{0x2207,Sm,0x2207,0x2207,0x2207},
{0x2208,Sm,0x2208,0x2208,0x2208},
{0x2209,Sm,0x2209,0x2209,0x2209},
{0x220A,Sm,0x220A,0x220A,0x220A},
{0x220B,Sm,0x220B,0x220B,0x220B},
{0x220C,Sm,0x220C,0x220C,0x220C},
{0x220D,Sm,0x220D,0x220D,0x220D},
{0x220E,Sm,0x220E,0x220E,0x220E},
{0x220F,Sm,0x220F,0x220F,0x220F},
{0x2210,Sm,0x2210,0x2210,0x2210},
{0x2211,Sm,0x2211,0x2211,0x2211},
{0x2212,Sm,0x2212,0x2212,0x2212},
{0x2213,Sm,0x2213,0x2213,0x2213},
{0x2214,Sm,0x2214,0x2214,0x2214},
{0x2215,Sm,0x2215,0x2215,0x2215},
{0x2216,Sm,0x2216,0x2216,0x2216},
{0x2217,Sm,0x2217,0x2217,0x2217},
{0x2218,Sm,0x2218,0x2218,0x2218},
{0x2219,Sm,0x2219,0x2219,0x2219},
{0x221A,Sm,0x221A,0x221A,0x221A},
{0x221B,Sm,0x221B,0x221B,0x221B},
{0x221C,Sm,0x221C,0x221C,0x221C},
{0x221D,Sm,0x221D,0x221D,0x221D},
{0x221E,Sm,0x221E,0x221E,0x221E},
{0x221F,Sm,0x221F,0x221F,0x221F},
{0x2220,Sm,0x2220,0x2220,0x2220},
{0x2221,Sm,0x2221,0x2221,0x2221},
{0x2222,Sm,0x2222,0x2222,0x2222},
{0x2223,Sm,0x2223,0x2223,0x2223},
{0x2224,Sm,0x2224,0x2224,0x2224},
{0x2225,Sm,0x2225,0x2225,0x2225},
{0x2226,Sm,0x2226,0x2226,0x2226},
{0x2227,Sm,0x2227,0x2227,0x2227},
{0x2228,Sm,0x2228,0x2228,0x2228},
{0x2229,Sm,0x2229,0x2229,0x2229},
{0x222A,Sm,0x222A,0x222A,0x222A},
{0x222B,Sm,0x222B,0x222B,0x222B},
{0x222C,Sm,0x222C,0x222C,0x222C},
{0x222D,Sm,0x222D,0x222D,0x222D},
{0x222E,Sm,0x222E,0x222E,0x222E},
{0x222F,Sm,0x222F,0x222F,0x222F},
{0x2230,Sm,0x2230,0x2230,0x2230},
{0x2231,Sm,0x2231,0x2231,0x2231},
{0x2232,Sm,0x2232,0x2232,0x2232},
{0x2233,Sm,0x2233,0x2233,0x2233},
{0x2234,Sm,0x2234,0x2234,0x2234},
{0x2235,Sm,0x2235,0x2235,0x2235},
{0x2236,Sm,0x2236,0x2236,0x2236},
{0x2237,Sm,0x2237,0x2237,0x2237},
{0x2238,Sm,0x2238,0x2238,0x2238},
{0x2239,Sm,0x2239,0x2239,0x2239},
{0x223A,Sm,0x223A,0x223A,0x223A},
{0x223B,Sm,0x223B,0x223B,0x223B},
{0x223C,Sm,0x223C,0x223C,0x223C},
{0x223D,Sm,0x223D,0x223D,0x223D},
{0x223E,Sm,0x223E,0x223E,0x223E},
{0x223F,Sm,0x223F,0x223F,0x223F},
{0x2240,Sm,0x2240,0x2240,0x2240},
{0x2241,Sm,0x2241,0x2241,0x2241},
{0x2242,Sm,0x2242,0x2242,0x2242},
{0x2243,Sm,0x2243,0x2243,0x2243},
{0x2244,Sm,0x2244,0x2244,0x2244},
{0x2245,Sm,0x2245,0x2245,0x2245},
{0x2246,Sm,0x2246,0x2246,0x2246},
{0x2247,Sm,0x2247,0x2247,0x2247},
{0x2248,Sm,0x2248,0x2248,0x2248},
{0x2249,Sm,0x2249,0x2249,0x2249},
{0x224A,Sm,0x224A,0x224A,0x224A},
{0x224B,Sm,0x224B,0x224B,0x224B},
{0x224C,Sm,0x224C,0x224C,0x224C},
{0x224D,Sm,0x224D,0x224D,0x224D},
{0x224E,Sm,0x224E,0x224E,0x224E},
{0x224F,Sm,0x224F,0x224F,0x224F},
{0x2250,Sm,0x2250,0x2250,0x2250},
{0x2251,Sm,0x2251,0x2251,0x2251},
{0x2252,Sm,0x2252,0x2252,0x2252},
{0x2253,Sm,0x2253,0x2253,0x2253},
{0x2254,Sm,0x2254,0x2254,0x2254},
{0x2255,Sm,0x2255,0x2255,0x2255},
{0x2256,Sm,0x2256,0x2256,0x2256},
{0x2257,Sm,0x2257,0x2257,0x2257},
{0x2258,Sm,0x2258,0x2258,0x2258},
{0x2259,Sm,0x2259,0x2259,0x2259},
{0x225A,Sm,0x225A,0x225A,0x225A},
{0x225B,Sm,0x225B,0x225B,0x225B},
{0x225C,Sm,0x225C,0x225C,0x225C},
{0x225D,Sm,0x225D,0x225D,0x225D},
{0x225E,Sm,0x225E,0x225E,0x225E},
{0x225F,Sm,0x225F,0x225F,0x225F},
{0x2260,Sm,0x2260,0x2260,0x2260},
{0x2261,Sm,0x2261,0x2261,0x2261},
{0x2262,Sm,0x2262,0x2262,0x2262},
{0x2263,Sm,0x2263,0x2263,0x2263},
{0x2264,Sm,0x2264,0x2264,0x2264},
{0x2265,Sm,0x2265,0x2265,0x2265},
{0x2266,Sm,0x2266,0x2266,0x2266},
{0x2267,Sm,0x2267,0x2267,0x2267},
{0x2268,Sm,0x2268,0x2268,0x2268},
{0x2269,Sm,0x2269,0x2269,0x2269},
{0x226A,Sm,0x226A,0x226A,0x226A},
{0x226B,Sm,0x226B,0x226B,0x226B},
{0x226C,Sm,0x226C,0x226C,0x226C},
{0x226D,Sm,0x226D,0x226D,0x226D},
{0x226E,Sm,0x226E,0x226E,0x226E},
{0x226F,Sm,0x226F,0x226F,0x226F},
{0x2270,Sm,0x2270,0x2270,0x2270},
{0x2271,Sm,0x2271,0x2271,0x2271},
{0x2272,Sm,0x2272,0x2272,0x2272},
{0x2273,Sm,0x2273,0x2273,0x2273},
{0x2274,Sm,0x2274,0x2274,0x2274},
{0x2275,Sm,0x2275,0x2275,0x2275},
{0x2276,Sm,0x2276,0x2276,0x2276},
{0x2277,Sm,0x2277,0x2277,0x2277},
{0x2278,Sm,0x2278,0x2278,0x2278},
{0x2279,Sm,0x2279,0x2279,0x2279},
{0x227A,Sm,0x227A,0x227A,0x227A},
{0x227B,Sm,0x227B,0x227B,0x227B},
{0x227C,Sm,0x227C,0x227C,0x227C},
{0x227D,Sm,0x227D,0x227D,0x227D},
{0x227E,Sm,0x227E,0x227E,0x227E},
{0x227F,Sm,0x227F,0x227F,0x227F},
{0x2280,Sm,0x2280,0x2280,0x2280},
{0x2281,Sm,0x2281,0x2281,0x2281},
{0x2282,Sm,0x2282,0x2282,0x2282},
{0x2283,Sm,0x2283,0x2283,0x2283},
{0x2284,Sm,0x2284,0x2284,0x2284},
{0x2285,Sm,0x2285,0x2285,0x2285},
{0x2286,Sm,0x2286,0x2286,0x2286},
{0x2287,Sm,0x2287,0x2287,0x2287},
{0x2288,Sm,0x2288,0x2288,0x2288},
{0x2289,Sm,0x2289,0x2289,0x2289},
{0x228A,Sm,0x228A,0x228A,0x228A},
{0x228B,Sm,0x228B,0x228B,0x228B},
{0x228C,Sm,0x228C,0x228C,0x228C},
{0x228D,Sm,0x228D,0x228D,0x228D},
{0x228E,Sm,0x228E,0x228E,0x228E},
{0x228F,Sm,0x228F,0x228F,0x228F},
{0x2290,Sm,0x2290,0x2290,0x2290},
{0x2291,Sm,0x2291,0x2291,0x2291},
{0x2292,Sm,0x2292,0x2292,0x2292},
{0x2293,Sm,0x2293,0x2293,0x2293},
{0x2294,Sm,0x2294,0x2294,0x2294},
{0x2295,Sm,0x2295,0x2295,0x2295},
{0x2296,Sm,0x2296,0x2296,0x2296},
{0x2297,Sm,0x2297,0x2297,0x2297},
{0x2298,Sm,0x2298,0x2298,0x2298},
{0x2299,Sm,0x2299,0x2299,0x2299},
{0x229A,Sm,0x229A,0x229A,0x229A},
{0x229B,Sm,0x229B,0x229B,0x229B},
{0x229C,Sm,0x229C,0x229C,0x229C},
{0x229D,Sm,0x229D,0x229D,0x229D},
{0x229E,Sm,0x229E,0x229E,0x229E},
{0x229F,Sm,0x229F,0x229F,0x229F},
{0x22A0,Sm,0x22A0,0x22A0,0x22A0},
{0x22A1,Sm,0x22A1,0x22A1,0x22A1},
{0x22A2,Sm,0x22A2,0x22A2,0x22A2},
{0x22A3,Sm,0x22A3,0x22A3,0x22A3},
{0x22A4,Sm,0x22A4,0x22A4,0x22A4},
{0x22A5,Sm,0x22A5,0x22A5,0x22A5},
{0x22A6,Sm,0x22A6,0x22A6,0x22A6},
{0x22A7,Sm,0x22A7,0x22A7,0x22A7},
{0x22A8,Sm,0x22A8,0x22A8,0x22A8},
{0x22A9,Sm,0x22A9,0x22A9,0x22A9},
{0x22AA,Sm,0x22AA,0x22AA,0x22AA},
{0x22AB,Sm,0x22AB,0x22AB,0x22AB},
{0x22AC,Sm,0x22AC,0x22AC,0x22AC},
{0x22AD,Sm,0x22AD,0x22AD,0x22AD},
{0x22AE,Sm,0x22AE,0x22AE,0x22AE},
{0x22AF,Sm,0x22AF,0x22AF,0x22AF},
{0x22B0,Sm,0x22B0,0x22B0,0x22B0},
{0x22B1,Sm,0x22B1,0x22B1,0x22B1},
{0x22B2,Sm,0x22B2,0x22B2,0x22B2},
{0x22B3,Sm,0x22B3,0x22B3,0x22B3},
{0x22B4,Sm,0x22B4,0x22B4,0x22B4},
{0x22B5,Sm,0x22B5,0x22B5,0x22B5},
{0x22B6,Sm,0x22B6,0x22B6,0x22B6},
{0x22B7,Sm,0x22B7,0x22B7,0x22B7},
{0x22B8,Sm,0x22B8,0x22B8,0x22B8},
{0x22B9,Sm,0x22B9,0x22B9,0x22B9},
{0x22BA,Sm,0x22BA,0x22BA,0x22BA},
{0x22BB,Sm,0x22BB,0x22BB,0x22BB},
{0x22BC,Sm,0x22BC,0x22BC,0x22BC},
{0x22BD,Sm,0x22BD,0x22BD,0x22BD},
{0x22BE,Sm,0x22BE,0x22BE,0x22BE},
{0x22BF,Sm,0x22BF,0x22BF,0x22BF},
{0x22C0,Sm,0x22C0,0x22C0,0x22C0},
{0x22C1,Sm,0x22C1,0x22C1,0x22C1},
{0x22C2,Sm,0x22C2,0x22C2,0x22C2},
{0x22C3,Sm,0x22C3,0x22C3,0x22C3},
{0x22C4,Sm,0x22C4,0x22C4,0x22C4},
{0x22C5,Sm,0x22C5,0x22C5,0x22C5},
{0x22C6,Sm,0x22C6,0x22C6,0x22C6},
{0x22C7,Sm,0x22C7,0x22C7,0x22C7},
{0x22C8,Sm,0x22C8,0x22C8,0x22C8},
{0x22C9,Sm,0x22C9,0x22C9,0x22C9},
{0x22CA,Sm,0x22CA,0x22CA,0x22CA},
{0x22CB,Sm,0x22CB,0x22CB,0x22CB},
{0x22CC,Sm,0x22CC,0x22CC,0x22CC},
{0x22CD,Sm,0x22CD,0x22CD,0x22CD},
{0x22CE,Sm,0x22CE,0x22CE,0x22CE},
{0x22CF,Sm,0x22CF,0x22CF,0x22CF},
{0x22D0,Sm,0x22D0,0x22D0,0x22D0},
{0x22D1,Sm,0x22D1,0x22D1,0x22D1},
{0x22D2,Sm,0x22D2,0x22D2,0x22D2},
{0x22D3,Sm,0x22D3,0x22D3,0x22D3},
{0x22D4,Sm,0x22D4,0x22D4,0x22D4},
{0x22D5,Sm,0x22D5,0x22D5,0x22D5},
{0x22D6,Sm,0x22D6,0x22D6,0x22D6},
{0x22D7,Sm,0x22D7,0x22D7,0x22D7},
{0x22D8,Sm,0x22D8,0x22D8,0x22D8},
{0x22D9,Sm,0x22D9,0x22D9,0x22D9},
{0x22DA,Sm,0x22DA,0x22DA,0x22DA},
{0x22DB,Sm,0x22DB,0x22DB,0x22DB},
{0x22DC,Sm,0x22DC,0x22DC,0x22DC},
{0x22DD,Sm,0x22DD,0x22DD,0x22DD},
{0x22DE,Sm,0x22DE,0x22DE,0x22DE},
{0x22DF,Sm,0x22DF,0x22DF,0x22DF},
{0x22E0,Sm,0x22E0,0x22E0,0x22E0},
{0x22E1,Sm,0x22E1,0x22E1,0x22E1},
{0x22E2,Sm,0x22E2,0x22E2,0x22E2},
{0x22E3,Sm,0x22E3,0x22E3,0x22E3},
{0x22E4,Sm,0x22E4,0x22E4,0x22E4},
{0x22E5,Sm,0x22E5,0x22E5,0x22E5},
{0x22E6,Sm,0x22E6,0x22E6,0x22E6},
{0x22E7,Sm,0x22E7,0x22E7,0x22E7},
{0x22E8,Sm,0x22E8,0x22E8,0x22E8},
{0x22E9,Sm,0x22E9,0x22E9,0x22E9},
{0x22EA,Sm,0x22EA,0x22EA,0x22EA},
{0x22EB,Sm,0x22EB,0x22EB,0x22EB},
{0x22EC,Sm,0x22EC,0x22EC,0x22EC},
{0x22ED,Sm,0x22ED,0x22ED,0x22ED},
{0x22EE,Sm,0x22EE,0x22EE,0x22EE},
{0x22EF,Sm,0x22EF,0x22EF,0x22EF},
{0x22F0,Sm,0x22F0,0x22F0,0x22F0},
{0x22F1,Sm,0x22F1,0x22F1,0x22F1},
{0x22F2,Sm,0x22F2,0x22F2,0x22F2},
{0x22F3,Sm,0x22F3,0x22F3,0x22F3},
{0x22F4,Sm,0x22F4,0x22F4,0x22F4},
{0x22F5,Sm,0x22F5,0x22F5,0x22F5},
{0x22F6,Sm,0x22F6,0x22F6,0x22F6},
{0x22F7,Sm,0x22F7,0x22F7,0x22F7},
{0x22F8,Sm,0x22F8,0x22F8,0x22F8},
{0x22F9,Sm,0x22F9,0x22F9,0x22F9},
{0x22FA,Sm,0x22FA,0x22FA,0x22FA},
{0x22FB,Sm,0x22FB,0x22FB,0x22FB},
{0x22FC,Sm,0x22FC,0x22FC,0x22FC},
{0x22FD,Sm,0x22FD,0x22FD,0x22FD},
{0x22FE,Sm,0x22FE,0x22FE,0x22FE},
{0x22FF,Sm,0x22FF,0x22FF,0x22FF},
{0x2300,So,0x2300,0x2300,0x2300},
{0x2301,So,0x2301,0x2301,0x2301},
{0x2302,So,0x2302,0x2302,0x2302},
{0x2303,So,0x2303,0x2303,0x2303},
{0x2304,So,0x2304,0x2304,0x2304},
{0x2305,So,0x2305,0x2305,0x2305},
{0x2306,So,0x2306,0x2306,0x2306},
{0x2307,So,0x2307,0x2307,0x2307},
{0x2308,Sm,0x2308,0x2308,0x2308},
{0x2309,Sm,0x2309,0x2309,0x2309},
{0x230A,Sm,0x230A,0x230A,0x230A},
{0x230B,Sm,0x230B,0x230B,0x230B},
{0x230C,So,0x230C,0x230C,0x230C},
{0x230D,So,0x230D,0x230D,0x230D},
{0x230E,So,0x230E,0x230E,0x230E},
{0x230F,So,0x230F,0x230F,0x230F},
{0x2310,So,0x2310,0x2310,0x2310},
{0x2311,So,0x2311,0x2311,0x2311},
{0x2312,So,0x2312,0x2312,0x2312},
{0x2313,So,0x2313,0x2313,0x2313},
{0x2314,So,0x2314,0x2314,0x2314},
{0x2315,So,0x2315,0x2315,0x2315},
{0x2316,So,0x2316,0x2316,0x2316},
{0x2317,So,0x2317,0x2317,0x2317},
{0x2318,So,0x2318,0x2318,0x2318},
{0x2319,So,0x2319,0x2319,0x2319},
{0x231A,So,0x231A,0x231A,0x231A},
{0x231B,So,0x231B,0x231B,0x231B},
{0x231C,So,0x231C,0x231C,0x231C},
{0x231D,So,0x231D,0x231D,0x231D},
{0x231E,So,0x231E,0x231E,0x231E},
{0x231F,So,0x231F,0x231F,0x231F},
{0x2320,Sm,0x2320,0x2320,0x2320},
{0x2321,Sm,0x2321,0x2321,0x2321},
{0x2322,So,0x2322,0x2322,0x2322},
{0x2323,So,0x2323,0x2323,0x2323},
{0x2324,So,0x2324,0x2324,0x2324},
{0x2325,So,0x2325,0x2325,0x2325},
{0x2326,So,0x2326,0x2326,0x2326},
{0x2327,So,0x2327,0x2327,0x2327},
{0x2328,So,0x2328,0x2328,0x2328},
{0x2329,Ps,0x2329,0x2329,0x2329},
{0x232A,Pe,0x232A,0x232A,0x232A},
{0x232B,So,0x232B,0x232B,0x232B},
{0x232C,So,0x232C,0x232C,0x232C},
{0x232D,So,0x232D,0x232D,0x232D},
{0x232E,So,0x232E,0x232E,0x232E},
{0x232F,So,0x232F,0x232F,0x232F},
{0x2330,So,0x2330,0x2330,0x2330},
{0x2331,So,0x2331,0x2331,0x2331},
{0x2332,So,0x2332,0x2332,0x2332},
{0x2333,So,0x2333,0x2333,0x2333},
{0x2334,So,0x2334,0x2334,0x2334},
{0x2335,So,0x2335,0x2335,0x2335},
{0x2336,So,0x2336,0x2336,0x2336},
{0x2337,So,0x2337,0x2337,0x2337},
{0x2338,So,0x2338,0x2338,0x2338},
{0x2339,So,0x2339,0x2339,0x2339},
{0x233A,So,0x233A,0x233A,0x233A},
{0x233B,So,0x233B,0x233B,0x233B},
{0x233C,So,0x233C,0x233C,0x233C},
{0x233D,So,0x233D,0x233D,0x233D},
{0x233E,So,0x233E,0x233E,0x233E},
{0x233F,So,0x233F,0x233F,0x233F},
{0x2340,So,0x2340,0x2340,0x2340},
{0x2341,So,0x2341,0x2341,0x2341},
{0x2342,So,0x2342,0x2342,0x2342},
{0x2343,So,0x2343,0x2343,0x2343},
{0x2344,So,0x2344,0x2344,0x2344},
{0x2345,So,0x2345,0x2345,0x2345},
{0x2346,So,0x2346,0x2346,0x2346},
{0x2347,So,0x2347,0x2347,0x2347},
{0x2348,So,0x2348,0x2348,0x2348},
{0x2349,So,0x2349,0x2349,0x2349},
{0x234A,So,0x234A,0x234A,0x234A},
{0x234B,So,0x234B,0x234B,0x234B},
{0x234C,So,0x234C,0x234C,0x234C},
{0x234D,So,0x234D,0x234D,0x234D},
{0x234E,So,0x234E,0x234E,0x234E},
{0x234F,So,0x234F,0x234F,0x234F},
{0x2350,So,0x2350,0x2350,0x2350},
{0x2351,So,0x2351,0x2351,0x2351},
{0x2352,So,0x2352,0x2352,0x2352},
{0x2353,So,0x2353,0x2353,0x2353},
{0x2354,So,0x2354,0x2354,0x2354},
{0x2355,So,0x2355,0x2355,0x2355},
{0x2356,So,0x2356,0x2356,0x2356},
{0x2357,So,0x2357,0x2357,0x2357},
{0x2358,So,0x2358,0x2358,0x2358},
{0x2359,So,0x2359,0x2359,0x2359},
{0x235A,So,0x235A,0x235A,0x235A},
{0x235B,So,0x235B,0x235B,0x235B},
{0x235C,So,0x235C,0x235C,0x235C},
{0x235D,So,0x235D,0x235D,0x235D},
{0x235E,So,0x235E,0x235E,0x235E},
{0x235F,So,0x235F,0x235F,0x235F},
{0x2360,So,0x2360,0x2360,0x2360},
{0x2361,So,0x2361,0x2361,0x2361},
{0x2362,So,0x2362,0x2362,0x2362},
{0x2363,So,0x2363,0x2363,0x2363},
{0x2364,So,0x2364,0x2364,0x2364},
{0x2365,So,0x2365,0x2365,0x2365},
{0x2366,So,0x2366,0x2366,0x2366},
{0x2367,So,0x2367,0x2367,0x2367},
{0x2368,So,0x2368,0x2368,0x2368},
{0x2369,So,0x2369,0x2369,0x2369},
{0x236A,So,0x236A,0x236A,0x236A},
{0x236B,So,0x236B,0x236B,0x236B},
{0x236C,So,0x236C,0x236C,0x236C},
{0x236D,So,0x236D,0x236D,0x236D},
{0x236E,So,0x236E,0x236E,0x236E},
{0x236F,So,0x236F,0x236F,0x236F},
{0x2370,So,0x2370,0x2370,0x2370},
{0x2371,So,0x2371,0x2371,0x2371},
{0x2372,So,0x2372,0x2372,0x2372},
{0x2373,So,0x2373,0x2373,0x2373},
{0x2374,So,0x2374,0x2374,0x2374},
{0x2375,So,0x2375,0x2375,0x2375},
{0x2376,So,0x2376,0x2376,0x2376},
{0x2377,So,0x2377,0x2377,0x2377},
{0x2378,So,0x2378,0x2378,0x2378},
{0x2379,So,0x2379,0x2379,0x2379},
{0x237A,So,0x237A,0x237A,0x237A},
{0x237B,So,0x237B,0x237B,0x237B},
{0x237C,Sm,0x237C,0x237C,0x237C},
{0x237D,So,0x237D,0x237D,0x237D},
{0x237E,So,0x237E,0x237E,0x237E},
{0x237F,So,0x237F,0x237F,0x237F},
{0x2380,So,0x2380,0x2380,0x2380},
{0x2381,So,0x2381,0x2381,0x2381},
{0x2382,So,0x2382,0x2382,0x2382},
{0x2383,So,0x2383,0x2383,0x2383},
{0x2384,So,0x2384,0x2384,0x2384},
{0x2385,So,0x2385,0x2385,0x2385},
{0x2386,So,0x2386,0x2386,0x2386},
{0x2387,So,0x2387,0x2387,0x2387},
{0x2388,So,0x2388,0x2388,0x2388},
{0x2389,So,0x2389,0x2389,0x2389},
{0x238A,So,0x238A,0x238A,0x238A},
{0x238B,So,0x238B,0x238B,0x238B},
{0x238C,So,0x238C,0x238C,0x238C},
{0x238D,So,0x238D,0x238D,0x238D},
{0x238E,So,0x238E,0x238E,0x238E},
{0x238F,So,0x238F,0x238F,0x238F},
{0x2390,So,0x2390,0x2390,0x2390},
{0x2391,So,0x2391,0x2391,0x2391},
{0x2392,So,0x2392,0x2392,0x2392},
{0x2393,So,0x2393,0x2393,0x2393},
{0x2394,So,0x2394,0x2394,0x2394},
{0x2395,So,0x2395,0x2395,0x2395},
{0x2396,So,0x2396,0x2396,0x2396},
{0x2397,So,0x2397,0x2397,0x2397},
{0x2398,So,0x2398,0x2398,0x2398},
{0x2399,So,0x2399,0x2399,0x2399},
{0x239A,So,0x239A,0x239A,0x239A},
{0x239B,Sm,0x239B,0x239B,0x239B},
{0x239C,Sm,0x239C,0x239C,0x239C},
{0x239D,Sm,0x239D,0x239D,0x239D},
{0x239E,Sm,0x239E,0x239E,0x239E},
{0x239F,Sm,0x239F,0x239F,0x239F},
{0x23A0,Sm,0x23A0,0x23A0,0x23A0},
{0x23A1,Sm,0x23A1,0x23A1,0x23A1},
{0x23A2,Sm,0x23A2,0x23A2,0x23A2},
{0x23A3,Sm,0x23A3,0x23A3,0x23A3},
{0x23A4,Sm,0x23A4,0x23A4,0x23A4},
{0x23A5,Sm,0x23A5,0x23A5,0x23A5},
{0x23A6,Sm,0x23A6,0x23A6,0x23A6},
{0x23A7,Sm,0x23A7,0x23A7,0x23A7},
{0x23A8,Sm,0x23A8,0x23A8,0x23A8},
{0x23A9,Sm,0x23A9,0x23A9,0x23A9},
{0x23AA,Sm,0x23AA,0x23AA,0x23AA},
{0x23AB,Sm,0x23AB,0x23AB,0x23AB},
{0x23AC,Sm,0x23AC,0x23AC,0x23AC},
{0x23AD,Sm,0x23AD,0x23AD,0x23AD},
{0x23AE,Sm,0x23AE,0x23AE,0x23AE},
{0x23AF,Sm,0x23AF,0x23AF,0x23AF},
{0x23B0,Sm,0x23B0,0x23B0,0x23B0},
{0x23B1,Sm,0x23B1,0x23B1,0x23B1},
{0x23B2,Sm,0x23B2,0x23B2,0x23B2},
{0x23B3,Sm,0x23B3,0x23B3,0x23B3},
{0x23B4,So,0x23B4,0x23B4,0x23B4},
{0x23B5,So,0x23B5,0x23B5,0x23B5},
{0x23B6,So,0x23B6,0x23B6,0x23B6},
{0x23B7,So,0x23B7,0x23B7,0x23B7},
{0x23B8,So,0x23B8,0x23B8,0x23B8},
{0x23B9,So,0x23B9,0x23B9,0x23B9},
{0x23BA,So,0x23BA,0x23BA,0x23BA},
{0x23BB,So,0x23BB,0x23BB,0x23BB},
{0x23BC,So,0x23BC,0x23BC,0x23BC},
{0x23BD,So,0x23BD,0x23BD,0x23BD},
{0x23BE,So,0x23BE,0x23BE,0x23BE},
{0x23BF,So,0x23BF,0x23BF,0x23BF},
{0x23C0,So,0x23C0,0x23C0,0x23C0},
{0x23C1,So,0x23C1,0x23C1,0x23C1},
{0x23C2,So,0x23C2,0x23C2,0x23C2},
{0x23C3,So,0x23C3,0x23C3,0x23C3},
{0x23C4,So,0x23C4,0x23C4,0x23C4},
{0x23C5,So,0x23C5,0x23C5,0x23C5},
{0x23C6,So,0x23C6,0x23C6,0x23C6},
{0x23C7,So,0x23C7,0x23C7,0x23C7},
{0x23C8,So,0x23C8,0x23C8,0x23C8},
{0x23C9,So,0x23C9,0x23C9,0x23C9},
{0x23CA,So,0x23CA,0x23CA,0x23CA},
{0x23CB,So,0x23CB,0x23CB,0x23CB},
{0x23CC,So,0x23CC,0x23CC,0x23CC},
{0x23CD,So,0x23CD,0x23CD,0x23CD},
{0x23CE,So,0x23CE,0x23CE,0x23CE},
{0x23CF,So,0x23CF,0x23CF,0x23CF},
{0x23D0,So,0x23D0,0x23D0,0x23D0},
{0x23D1,So,0x23D1,0x23D1,0x23D1},
{0x23D2,So,0x23D2,0x23D2,0x23D2},
{0x23D3,So,0x23D3,0x23D3,0x23D3},
{0x23D4,So,0x23D4,0x23D4,0x23D4},
{0x23D5,So,0x23D5,0x23D5,0x23D5},
{0x23D6,So,0x23D6,0x23D6,0x23D6},
{0x23D7,So,0x23D7,0x23D7,0x23D7},
{0x23D8,So,0x23D8,0x23D8,0x23D8},
{0x23D9,So,0x23D9,0x23D9,0x23D9},
{0x23DA,So,0x23DA,0x23DA,0x23DA},
{0x23DB,So,0x23DB,0x23DB,0x23DB},
{0x23DC,Sm,0x23DC,0x23DC,0x23DC},
{0x23DD,Sm,0x23DD,0x23DD,0x23DD},
{0x23DE,Sm,0x23DE,0x23DE,0x23DE},
{0x23DF,Sm,0x23DF,0x23DF,0x23DF},
{0x23E0,Sm,0x23E0,0x23E0,0x23E0},
{0x23E1,Sm,0x23E1,0x23E1,0x23E1},
{0x23E2,So,0x23E2,0x23E2,0x23E2},
{0x23E3,So,0x23E3,0x23E3,0x23E3},
{0x23E4,So,0x23E4,0x23E4,0x23E4},
{0x23E5,So,0x23E5,0x23E5,0x23E5},
{0x23E6,So,0x23E6,0x23E6,0x23E6},
{0x23E7,So,0x23E7,0x23E7,0x23E7},
{0x2400,So,0x2400,0x2400,0x2400},
{0x2401,So,0x2401,0x2401,0x2401},
{0x2402,So,0x2402,0x2402,0x2402},
{0x2403,So,0x2403,0x2403,0x2403},
{0x2404,So,0x2404,0x2404,0x2404},
{0x2405,So,0x2405,0x2405,0x2405},
{0x2406,So,0x2406,0x2406,0x2406},
{0x2407,So,0x2407,0x2407,0x2407},
{0x2408,So,0x2408,0x2408,0x2408},
{0x2409,So,0x2409,0x2409,0x2409},
{0x240A,So,0x240A,0x240A,0x240A},
{0x240B,So,0x240B,0x240B,0x240B},
{0x240C,So,0x240C,0x240C,0x240C},
{0x240D,So,0x240D,0x240D,0x240D},
{0x240E,So,0x240E,0x240E,0x240E},
{0x240F,So,0x240F,0x240F,0x240F},
{0x2410,So,0x2410,0x2410,0x2410},
{0x2411,So,0x2411,0x2411,0x2411},
{0x2412,So,0x2412,0x2412,0x2412},
{0x2413,So,0x2413,0x2413,0x2413},
{0x2414,So,0x2414,0x2414,0x2414},
{0x2415,So,0x2415,0x2415,0x2415},
{0x2416,So,0x2416,0x2416,0x2416},
{0x2417,So,0x2417,0x2417,0x2417},
{0x2418,So,0x2418,0x2418,0x2418},
{0x2419,So,0x2419,0x2419,0x2419},
{0x241A,So,0x241A,0x241A,0x241A},
{0x241B,So,0x241B,0x241B,0x241B},
{0x241C,So,0x241C,0x241C,0x241C},
{0x241D,So,0x241D,0x241D,0x241D},
{0x241E,So,0x241E,0x241E,0x241E},
{0x241F,So,0x241F,0x241F,0x241F},
{0x2420,So,0x2420,0x2420,0x2420},
{0x2421,So,0x2421,0x2421,0x2421},
{0x2422,So,0x2422,0x2422,0x2422},
{0x2423,So,0x2423,0x2423,0x2423},
{0x2424,So,0x2424,0x2424,0x2424},
{0x2425,So,0x2425,0x2425,0x2425},
{0x2426,So,0x2426,0x2426,0x2426},
{0x2440,So,0x2440,0x2440,0x2440},
{0x2441,So,0x2441,0x2441,0x2441},
{0x2442,So,0x2442,0x2442,0x2442},
{0x2443,So,0x2443,0x2443,0x2443},
{0x2444,So,0x2444,0x2444,0x2444},
{0x2445,So,0x2445,0x2445,0x2445},
{0x2446,So,0x2446,0x2446,0x2446},
{0x2447,So,0x2447,0x2447,0x2447},
{0x2448,So,0x2448,0x2448,0x2448},
{0x2449,So,0x2449,0x2449,0x2449},
{0x244A,So,0x244A,0x244A,0x244A},
{0x2460,No,0x2460,0x2460,0x2460},
{0x2461,No,0x2461,0x2461,0x2461},
{0x2462,No,0x2462,0x2462,0x2462},
{0x2463,No,0x2463,0x2463,0x2463},
{0x2464,No,0x2464,0x2464,0x2464},
{0x2465,No,0x2465,0x2465,0x2465},
{0x2466,No,0x2466,0x2466,0x2466},
{0x2467,No,0x2467,0x2467,0x2467},
{0x2468,No,0x2468,0x2468,0x2468},
{0x2469,No,0x2469,0x2469,0x2469},
{0x246A,No,0x246A,0x246A,0x246A},
{0x246B,No,0x246B,0x246B,0x246B},
{0x246C,No,0x246C,0x246C,0x246C},
{0x246D,No,0x246D,0x246D,0x246D},
{0x246E,No,0x246E,0x246E,0x246E},
{0x246F,No,0x246F,0x246F,0x246F},
{0x2470,No,0x2470,0x2470,0x2470},
{0x2471,No,0x2471,0x2471,0x2471},
{0x2472,No,0x2472,0x2472,0x2472},
{0x2473,No,0x2473,0x2473,0x2473},
{0x2474,No,0x2474,0x2474,0x2474},
{0x2475,No,0x2475,0x2475,0x2475},
{0x2476,No,0x2476,0x2476,0x2476},
{0x2477,No,0x2477,0x2477,0x2477},
{0x2478,No,0x2478,0x2478,0x2478},
{0x2479,No,0x2479,0x2479,0x2479},
{0x247A,No,0x247A,0x247A,0x247A},
{0x247B,No,0x247B,0x247B,0x247B},
{0x247C,No,0x247C,0x247C,0x247C},
{0x247D,No,0x247D,0x247D,0x247D},
{0x247E,No,0x247E,0x247E,0x247E},
{0x247F,No,0x247F,0x247F,0x247F},
{0x2480,No,0x2480,0x2480,0x2480},
{0x2481,No,0x2481,0x2481,0x2481},
{0x2482,No,0x2482,0x2482,0x2482},
{0x2483,No,0x2483,0x2483,0x2483},
{0x2484,No,0x2484,0x2484,0x2484},
{0x2485,No,0x2485,0x2485,0x2485},
{0x2486,No,0x2486,0x2486,0x2486},
{0x2487,No,0x2487,0x2487,0x2487},
{0x2488,No,0x2488,0x2488,0x2488},
{0x2489,No,0x2489,0x2489,0x2489},
{0x248A,No,0x248A,0x248A,0x248A},
{0x248B,No,0x248B,0x248B,0x248B},
{0x248C,No,0x248C,0x248C,0x248C},
{0x248D,No,0x248D,0x248D,0x248D},
{0x248E,No,0x248E,0x248E,0x248E},
{0x248F,No,0x248F,0x248F,0x248F},
{0x2490,No,0x2490,0x2490,0x2490},
{0x2491,No,0x2491,0x2491,0x2491},
{0x2492,No,0x2492,0x2492,0x2492},
{0x2493,No,0x2493,0x2493,0x2493},
{0x2494,No,0x2494,0x2494,0x2494},
{0x2495,No,0x2495,0x2495,0x2495},
{0x2496,No,0x2496,0x2496,0x2496},
{0x2497,No,0x2497,0x2497,0x2497},
{0x2498,No,0x2498,0x2498,0x2498},
{0x2499,No,0x2499,0x2499,0x2499},
{0x249A,No,0x249A,0x249A,0x249A},
{0x249B,No,0x249B,0x249B,0x249B},
{0x249C,So,0x249C,0x249C,0x249C},
{0x249D,So,0x249D,0x249D,0x249D},
{0x249E,So,0x249E,0x249E,0x249E},
{0x249F,So,0x249F,0x249F,0x249F},
{0x24A0,So,0x24A0,0x24A0,0x24A0},
{0x24A1,So,0x24A1,0x24A1,0x24A1},
{0x24A2,So,0x24A2,0x24A2,0x24A2},
{0x24A3,So,0x24A3,0x24A3,0x24A3},
{0x24A4,So,0x24A4,0x24A4,0x24A4},
{0x24A5,So,0x24A5,0x24A5,0x24A5},
{0x24A6,So,0x24A6,0x24A6,0x24A6},
{0x24A7,So,0x24A7,0x24A7,0x24A7},
{0x24A8,So,0x24A8,0x24A8,0x24A8},
{0x24A9,So,0x24A9,0x24A9,0x24A9},
{0x24AA,So,0x24AA,0x24AA,0x24AA},
{0x24AB,So,0x24AB,0x24AB,0x24AB},
{0x24AC,So,0x24AC,0x24AC,0x24AC},
{0x24AD,So,0x24AD,0x24AD,0x24AD},
{0x24AE,So,0x24AE,0x24AE,0x24AE},
{0x24AF,So,0x24AF,0x24AF,0x24AF},
{0x24B0,So,0x24B0,0x24B0,0x24B0},
{0x24B1,So,0x24B1,0x24B1,0x24B1},
{0x24B2,So,0x24B2,0x24B2,0x24B2},
{0x24B3,So,0x24B3,0x24B3,0x24B3},
{0x24B4,So,0x24B4,0x24B4,0x24B4},
{0x24B5,So,0x24B5,0x24B5,0x24B5},
{0x24B6,So,0x24B6,0x24D0,0x24B6},
{0x24B7,So,0x24B7,0x24D1,0x24B7},
{0x24B8,So,0x24B8,0x24D2,0x24B8},
{0x24B9,So,0x24B9,0x24D3,0x24B9},
{0x24BA,So,0x24BA,0x24D4,0x24BA},
{0x24BB,So,0x24BB,0x24D5,0x24BB},
{0x24BC,So,0x24BC,0x24D6,0x24BC},
{0x24BD,So,0x24BD,0x24D7,0x24BD},
{0x24BE,So,0x24BE,0x24D8,0x24BE},
{0x24BF,So,0x24BF,0x24D9,0x24BF},
{0x24C0,So,0x24C0,0x24DA,0x24C0},
{0x24C1,So,0x24C1,0x24DB,0x24C1},
{0x24C2,So,0x24C2,0x24DC,0x24C2},
{0x24C3,So,0x24C3,0x24DD,0x24C3},
{0x24C4,So,0x24C4,0x24DE,0x24C4},
{0x24C5,So,0x24C5,0x24DF,0x24C5},
{0x24C6,So,0x24C6,0x24E0,0x24C6},
{0x24C7,So,0x24C7,0x24E1,0x24C7},
{0x24C8,So,0x24C8,0x24E2,0x24C8},
{0x24C9,So,0x24C9,0x24E3,0x24C9},
{0x24CA,So,0x24CA,0x24E4,0x24CA},
{0x24CB,So,0x24CB,0x24E5,0x24CB},
{0x24CC,So,0x24CC,0x24E6,0x24CC},
{0x24CD,So,0x24CD,0x24E7,0x24CD},
{0x24CE,So,0x24CE,0x24E8,0x24CE},
{0x24CF,So,0x24CF,0x24E9,0x24CF},
{0x24D0,So,0x24B6,0x24D0,0x24B6},
{0x24D1,So,0x24B7,0x24D1,0x24B7},
{0x24D2,So,0x24B8,0x24D2,0x24B8},
{0x24D3,So,0x24B9,0x24D3,0x24B9},
{0x24D4,So,0x24BA,0x24D4,0x24BA},
{0x24D5,So,0x24BB,0x24D5,0x24BB},
{0x24D6,So,0x24BC,0x24D6,0x24BC},
{0x24D7,So,0x24BD,0x24D7,0x24BD},
{0x24D8,So,0x24BE,0x24D8,0x24BE},
{0x24D9,So,0x24BF,0x24D9,0x24BF},
{0x24DA,So,0x24C0,0x24DA,0x24C0},
{0x24DB,So,0x24C1,0x24DB,0x24C1},
{0x24DC,So,0x24C2,0x24DC,0x24C2},
{0x24DD,So,0x24C3,0x24DD,0x24C3},
{0x24DE,So,0x24C4,0x24DE,0x24C4},
{0x24DF,So,0x24C5,0x24DF,0x24C5},
{0x24E0,So,0x24C6,0x24E0,0x24C6},
{0x24E1,So,0x24C7,0x24E1,0x24C7},
{0x24E2,So,0x24C8,0x24E2,0x24C8},
{0x24E3,So,0x24C9,0x24E3,0x24C9},
{0x24E4,So,0x24CA,0x24E4,0x24CA},
{0x24E5,So,0x24CB,0x24E5,0x24CB},
{0x24E6,So,0x24CC,0x24E6,0x24CC},
{0x24E7,So,0x24CD,0x24E7,0x24CD},
{0x24E8,So,0x24CE,0x24E8,0x24CE},
{0x24E9,So,0x24CF,0x24E9,0x24CF},
{0x24EA,No,0x24EA,0x24EA,0x24EA},
{0x24EB,No,0x24EB,0x24EB,0x24EB},
{0x24EC,No,0x24EC,0x24EC,0x24EC},
{0x24ED,No,0x24ED,0x24ED,0x24ED},
{0x24EE,No,0x24EE,0x24EE,0x24EE},
{0x24EF,No,0x24EF,0x24EF,0x24EF},
{0x24F0,No,0x24F0,0x24F0,0x24F0},
{0x24F1,No,0x24F1,0x24F1,0x24F1},
{0x24F2,No,0x24F2,0x24F2,0x24F2},
{0x24F3,No,0x24F3,0x24F3,0x24F3},
{0x24F4,No,0x24F4,0x24F4,0x24F4},
{0x24F5,No,0x24F5,0x24F5,0x24F5},
{0x24F6,No,0x24F6,0x24F6,0x24F6},
{0x24F7,No,0x24F7,0x24F7,0x24F7},
{0x24F8,No,0x24F8,0x24F8,0x24F8},
{0x24F9,No,0x24F9,0x24F9,0x24F9},
{0x24FA,No,0x24FA,0x24FA,0x24FA},
{0x24FB,No,0x24FB,0x24FB,0x24FB},
{0x24FC,No,0x24FC,0x24FC,0x24FC},
{0x24FD,No,0x24FD,0x24FD,0x24FD},
{0x24FE,No,0x24FE,0x24FE,0x24FE},
{0x24FF,No,0x24FF,0x24FF,0x24FF},
{0x2500,So,0x2500,0x2500,0x2500},
{0x2501,So,0x2501,0x2501,0x2501},
{0x2502,So,0x2502,0x2502,0x2502},
{0x2503,So,0x2503,0x2503,0x2503},
{0x2504,So,0x2504,0x2504,0x2504},
{0x2505,So,0x2505,0x2505,0x2505},
{0x2506,So,0x2506,0x2506,0x2506},
{0x2507,So,0x2507,0x2507,0x2507},
{0x2508,So,0x2508,0x2508,0x2508},
{0x2509,So,0x2509,0x2509,0x2509},
{0x250A,So,0x250A,0x250A,0x250A},
{0x250B,So,0x250B,0x250B,0x250B},
{0x250C,So,0x250C,0x250C,0x250C},
{0x250D,So,0x250D,0x250D,0x250D},
{0x250E,So,0x250E,0x250E,0x250E},
{0x250F,So,0x250F,0x250F,0x250F},
{0x2510,So,0x2510,0x2510,0x2510},
{0x2511,So,0x2511,0x2511,0x2511},
{0x2512,So,0x2512,0x2512,0x2512},
{0x2513,So,0x2513,0x2513,0x2513},
{0x2514,So,0x2514,0x2514,0x2514},
{0x2515,So,0x2515,0x2515,0x2515},
{0x2516,So,0x2516,0x2516,0x2516},
{0x2517,So,0x2517,0x2517,0x2517},
{0x2518,So,0x2518,0x2518,0x2518},
{0x2519,So,0x2519,0x2519,0x2519},
{0x251A,So,0x251A,0x251A,0x251A},
{0x251B,So,0x251B,0x251B,0x251B},
{0x251C,So,0x251C,0x251C,0x251C},
{0x251D,So,0x251D,0x251D,0x251D},
{0x251E,So,0x251E,0x251E,0x251E},
{0x251F,So,0x251F,0x251F,0x251F},
{0x2520,So,0x2520,0x2520,0x2520},
{0x2521,So,0x2521,0x2521,0x2521},
{0x2522,So,0x2522,0x2522,0x2522},
{0x2523,So,0x2523,0x2523,0x2523},
{0x2524,So,0x2524,0x2524,0x2524},
{0x2525,So,0x2525,0x2525,0x2525},
{0x2526,So,0x2526,0x2526,0x2526},
{0x2527,So,0x2527,0x2527,0x2527},
{0x2528,So,0x2528,0x2528,0x2528},
{0x2529,So,0x2529,0x2529,0x2529},
{0x252A,So,0x252A,0x252A,0x252A},
{0x252B,So,0x252B,0x252B,0x252B},
{0x252C,So,0x252C,0x252C,0x252C},
{0x252D,So,0x252D,0x252D,0x252D},
{0x252E,So,0x252E,0x252E,0x252E},
{0x252F,So,0x252F,0x252F,0x252F},
{0x2530,So,0x2530,0x2530,0x2530},
{0x2531,So,0x2531,0x2531,0x2531},
{0x2532,So,0x2532,0x2532,0x2532},
{0x2533,So,0x2533,0x2533,0x2533},
{0x2534,So,0x2534,0x2534,0x2534},
{0x2535,So,0x2535,0x2535,0x2535},
{0x2536,So,0x2536,0x2536,0x2536},
{0x2537,So,0x2537,0x2537,0x2537},
{0x2538,So,0x2538,0x2538,0x2538},
{0x2539,So,0x2539,0x2539,0x2539},
{0x253A,So,0x253A,0x253A,0x253A},
{0x253B,So,0x253B,0x253B,0x253B},
{0x253C,So,0x253C,0x253C,0x253C},
{0x253D,So,0x253D,0x253D,0x253D},
{0x253E,So,0x253E,0x253E,0x253E},
{0x253F,So,0x253F,0x253F,0x253F},
{0x2540,So,0x2540,0x2540,0x2540},
{0x2541,So,0x2541,0x2541,0x2541},
{0x2542,So,0x2542,0x2542,0x2542},
{0x2543,So,0x2543,0x2543,0x2543},
{0x2544,So,0x2544,0x2544,0x2544},
{0x2545,So,0x2545,0x2545,0x2545},
{0x2546,So,0x2546,0x2546,0x2546},
{0x2547,So,0x2547,0x2547,0x2547},
{0x2548,So,0x2548,0x2548,0x2548},
{0x2549,So,0x2549,0x2549,0x2549},
{0x254A,So,0x254A,0x254A,0x254A},
{0x254B,So,0x254B,0x254B,0x254B},
{0x254C,So,0x254C,0x254C,0x254C},
{0x254D,So,0x254D,0x254D,0x254D},
{0x254E,So,0x254E,0x254E,0x254E},
{0x254F,So,0x254F,0x254F,0x254F},
{0x2550,So,0x2550,0x2550,0x2550},
{0x2551,So,0x2551,0x2551,0x2551},
{0x2552,So,0x2552,0x2552,0x2552},
{0x2553,So,0x2553,0x2553,0x2553},
{0x2554,So,0x2554,0x2554,0x2554},
{0x2555,So,0x2555,0x2555,0x2555},
{0x2556,So,0x2556,0x2556,0x2556},
{0x2557,So,0x2557,0x2557,0x2557},
{0x2558,So,0x2558,0x2558,0x2558},
{0x2559,So,0x2559,0x2559,0x2559},
{0x255A,So,0x255A,0x255A,0x255A},
{0x255B,So,0x255B,0x255B,0x255B},
{0x255C,So,0x255C,0x255C,0x255C},
{0x255D,So,0x255D,0x255D,0x255D},
{0x255E,So,0x255E,0x255E,0x255E},
{0x255F,So,0x255F,0x255F,0x255F},
{0x2560,So,0x2560,0x2560,0x2560},
{0x2561,So,0x2561,0x2561,0x2561},
{0x2562,So,0x2562,0x2562,0x2562},
{0x2563,So,0x2563,0x2563,0x2563},
{0x2564,So,0x2564,0x2564,0x2564},
{0x2565,So,0x2565,0x2565,0x2565},
{0x2566,So,0x2566,0x2566,0x2566},
{0x2567,So,0x2567,0x2567,0x2567},
{0x2568,So,0x2568,0x2568,0x2568},
{0x2569,So,0x2569,0x2569,0x2569},
{0x256A,So,0x256A,0x256A,0x256A},
{0x256B,So,0x256B,0x256B,0x256B},
{0x256C,So,0x256C,0x256C,0x256C},
{0x256D,So,0x256D,0x256D,0x256D},
{0x256E,So,0x256E,0x256E,0x256E},
{0x256F,So,0x256F,0x256F,0x256F},
{0x2570,So,0x2570,0x2570,0x2570},
{0x2571,So,0x2571,0x2571,0x2571},
{0x2572,So,0x2572,0x2572,0x2572},
{0x2573,So,0x2573,0x2573,0x2573},
{0x2574,So,0x2574,0x2574,0x2574},
{0x2575,So,0x2575,0x2575,0x2575},
{0x2576,So,0x2576,0x2576,0x2576},
{0x2577,So,0x2577,0x2577,0x2577},
{0x2578,So,0x2578,0x2578,0x2578},
{0x2579,So,0x2579,0x2579,0x2579},
{0x257A,So,0x257A,0x257A,0x257A},
{0x257B,So,0x257B,0x257B,0x257B},
{0x257C,So,0x257C,0x257C,0x257C},
{0x257D,So,0x257D,0x257D,0x257D},
{0x257E,So,0x257E,0x257E,0x257E},
{0x257F,So,0x257F,0x257F,0x257F},
{0x2580,So,0x2580,0x2580,0x2580},
{0x2581,So,0x2581,0x2581,0x2581},
{0x2582,So,0x2582,0x2582,0x2582},
{0x2583,So,0x2583,0x2583,0x2583},
{0x2584,So,0x2584,0x2584,0x2584},
{0x2585,So,0x2585,0x2585,0x2585},
{0x2586,So,0x2586,0x2586,0x2586},
{0x2587,So,0x2587,0x2587,0x2587},
{0x2588,So,0x2588,0x2588,0x2588},
{0x2589,So,0x2589,0x2589,0x2589},
{0x258A,So,0x258A,0x258A,0x258A},
{0x258B,So,0x258B,0x258B,0x258B},
{0x258C,So,0x258C,0x258C,0x258C},
{0x258D,So,0x258D,0x258D,0x258D},
{0x258E,So,0x258E,0x258E,0x258E},
{0x258F,So,0x258F,0x258F,0x258F},
{0x2590,So,0x2590,0x2590,0x2590},
{0x2591,So,0x2591,0x2591,0x2591},
{0x2592,So,0x2592,0x2592,0x2592},
{0x2593,So,0x2593,0x2593,0x2593},
{0x2594,So,0x2594,0x2594,0x2594},
{0x2595,So,0x2595,0x2595,0x2595},
{0x2596,So,0x2596,0x2596,0x2596},
{0x2597,So,0x2597,0x2597,0x2597},
{0x2598,So,0x2598,0x2598,0x2598},
{0x2599,So,0x2599,0x2599,0x2599},
{0x259A,So,0x259A,0x259A,0x259A},
{0x259B,So,0x259B,0x259B,0x259B},
{0x259C,So,0x259C,0x259C,0x259C},
{0x259D,So,0x259D,0x259D,0x259D},
{0x259E,So,0x259E,0x259E,0x259E},
{0x259F,So,0x259F,0x259F,0x259F},
{0x25A0,So,0x25A0,0x25A0,0x25A0},
{0x25A1,So,0x25A1,0x25A1,0x25A1},
{0x25A2,So,0x25A2,0x25A2,0x25A2},
{0x25A3,So,0x25A3,0x25A3,0x25A3},
{0x25A4,So,0x25A4,0x25A4,0x25A4},
{0x25A5,So,0x25A5,0x25A5,0x25A5},
{0x25A6,So,0x25A6,0x25A6,0x25A6},
{0x25A7,So,0x25A7,0x25A7,0x25A7},
{0x25A8,So,0x25A8,0x25A8,0x25A8},
{0x25A9,So,0x25A9,0x25A9,0x25A9},
{0x25AA,So,0x25AA,0x25AA,0x25AA},
{0x25AB,So,0x25AB,0x25AB,0x25AB},
{0x25AC,So,0x25AC,0x25AC,0x25AC},
{0x25AD,So,0x25AD,0x25AD,0x25AD},
{0x25AE,So,0x25AE,0x25AE,0x25AE},
{0x25AF,So,0x25AF,0x25AF,0x25AF},
{0x25B0,So,0x25B0,0x25B0,0x25B0},
{0x25B1,So,0x25B1,0x25B1,0x25B1},
{0x25B2,So,0x25B2,0x25B2,0x25B2},
{0x25B3,So,0x25B3,0x25B3,0x25B3},
{0x25B4,So,0x25B4,0x25B4,0x25B4},
{0x25B5,So,0x25B5,0x25B5,0x25B5},
{0x25B6,So,0x25B6,0x25B6,0x25B6},
{0x25B7,Sm,0x25B7,0x25B7,0x25B7},
{0x25B8,So,0x25B8,0x25B8,0x25B8},
{0x25B9,So,0x25B9,0x25B9,0x25B9},
{0x25BA,So,0x25BA,0x25BA,0x25BA},
{0x25BB,So,0x25BB,0x25BB,0x25BB},
{0x25BC,So,0x25BC,0x25BC,0x25BC},
{0x25BD,So,0x25BD,0x25BD,0x25BD},
{0x25BE,So,0x25BE,0x25BE,0x25BE},
{0x25BF,So,0x25BF,0x25BF,0x25BF},
{0x25C0,So,0x25C0,0x25C0,0x25C0},
{0x25C1,Sm,0x25C1,0x25C1,0x25C1},
{0x25C2,So,0x25C2,0x25C2,0x25C2},
{0x25C3,So,0x25C3,0x25C3,0x25C3},
{0x25C4,So,0x25C4,0x25C4,0x25C4},
{0x25C5,So,0x25C5,0x25C5,0x25C5},
{0x25C6,So,0x25C6,0x25C6,0x25C6},
{0x25C7,So,0x25C7,0x25C7,0x25C7},
{0x25C8,So,0x25C8,0x25C8,0x25C8},
{0x25C9,So,0x25C9,0x25C9,0x25C9},
{0x25CA,So,0x25CA,0x25CA,0x25CA},
{0x25CB,So,0x25CB,0x25CB,0x25CB},
{0x25CC,So,0x25CC,0x25CC,0x25CC},
{0x25CD,So,0x25CD,0x25CD,0x25CD},
{0x25CE,So,0x25CE,0x25CE,0x25CE},
{0x25CF,So,0x25CF,0x25CF,0x25CF},
{0x25D0,So,0x25D0,0x25D0,0x25D0},
{0x25D1,So,0x25D1,0x25D1,0x25D1},
{0x25D2,So,0x25D2,0x25D2,0x25D2},
{0x25D3,So,0x25D3,0x25D3,0x25D3},
{0x25D4,So,0x25D4,0x25D4,0x25D4},
{0x25D5,So,0x25D5,0x25D5,0x25D5},
{0x25D6,So,0x25D6,0x25D6,0x25D6},
{0x25D7,So,0x25D7,0x25D7,0x25D7},
{0x25D8,So,0x25D8,0x25D8,0x25D8},
{0x25D9,So,0x25D9,0x25D9,0x25D9},
{0x25DA,So,0x25DA,0x25DA,0x25DA},
{0x25DB,So,0x25DB,0x25DB,0x25DB},
{0x25DC,So,0x25DC,0x25DC,0x25DC},
{0x25DD,So,0x25DD,0x25DD,0x25DD},
{0x25DE,So,0x25DE,0x25DE,0x25DE},
{0x25DF,So,0x25DF,0x25DF,0x25DF},
{0x25E0,So,0x25E0,0x25E0,0x25E0},
{0x25E1,So,0x25E1,0x25E1,0x25E1},
{0x25E2,So,0x25E2,0x25E2,0x25E2},
{0x25E3,So,0x25E3,0x25E3,0x25E3},
{0x25E4,So,0x25E4,0x25E4,0x25E4},
{0x25E5,So,0x25E5,0x25E5,0x25E5},
{0x25E6,So,0x25E6,0x25E6,0x25E6},
{0x25E7,So,0x25E7,0x25E7,0x25E7},
{0x25E8,So,0x25E8,0x25E8,0x25E8},
{0x25E9,So,0x25E9,0x25E9,0x25E9},
{0x25EA,So,0x25EA,0x25EA,0x25EA},
{0x25EB,So,0x25EB,0x25EB,0x25EB},
{0x25EC,So,0x25EC,0x25EC,0x25EC},
{0x25ED,So,0x25ED,0x25ED,0x25ED},
{0x25EE,So,0x25EE,0x25EE,0x25EE},
{0x25EF,So,0x25EF,0x25EF,0x25EF},
{0x25F0,So,0x25F0,0x25F0,0x25F0},
{0x25F1,So,0x25F1,0x25F1,0x25F1},
{0x25F2,So,0x25F2,0x25F2,0x25F2},
{0x25F3,So,0x25F3,0x25F3,0x25F3},
{0x25F4,So,0x25F4,0x25F4,0x25F4},
{0x25F5,So,0x25F5,0x25F5,0x25F5},
{0x25F6,So,0x25F6,0x25F6,0x25F6},
{0x25F7,So,0x25F7,0x25F7,0x25F7},
{0x25F8,Sm,0x25F8,0x25F8,0x25F8},
{0x25F9,Sm,0x25F9,0x25F9,0x25F9},
{0x25FA,Sm,0x25FA,0x25FA,0x25FA},
{0x25FB,Sm,0x25FB,0x25FB,0x25FB},
{0x25FC,Sm,0x25FC,0x25FC,0x25FC},
{0x25FD,Sm,0x25FD,0x25FD,0x25FD},
{0x25FE,Sm,0x25FE,0x25FE,0x25FE},
{0x25FF,Sm,0x25FF,0x25FF,0x25FF},
{0x2600,So,0x2600,0x2600,0x2600},
{0x2601,So,0x2601,0x2601,0x2601},
{0x2602,So,0x2602,0x2602,0x2602},
{0x2603,So,0x2603,0x2603,0x2603},
{0x2604,So,0x2604,0x2604,0x2604},
{0x2605,So,0x2605,0x2605,0x2605},
{0x2606,So,0x2606,0x2606,0x2606},
{0x2607,So,0x2607,0x2607,0x2607},
{0x2608,So,0x2608,0x2608,0x2608},
{0x2609,So,0x2609,0x2609,0x2609},
{0x260A,So,0x260A,0x260A,0x260A},
{0x260B,So,0x260B,0x260B,0x260B},
{0x260C,So,0x260C,0x260C,0x260C},
{0x260D,So,0x260D,0x260D,0x260D},
{0x260E,So,0x260E,0x260E,0x260E},
{0x260F,So,0x260F,0x260F,0x260F},
{0x2610,So,0x2610,0x2610,0x2610},
{0x2611,So,0x2611,0x2611,0x2611},
{0x2612,So,0x2612,0x2612,0x2612},
{0x2613,So,0x2613,0x2613,0x2613},
{0x2614,So,0x2614,0x2614,0x2614},
{0x2615,So,0x2615,0x2615,0x2615},
{0x2616,So,0x2616,0x2616,0x2616},
{0x2617,So,0x2617,0x2617,0x2617},
{0x2618,So,0x2618,0x2618,0x2618},
{0x2619,So,0x2619,0x2619,0x2619},
{0x261A,So,0x261A,0x261A,0x261A},
{0x261B,So,0x261B,0x261B,0x261B},
{0x261C,So,0x261C,0x261C,0x261C},
{0x261D,So,0x261D,0x261D,0x261D},
{0x261E,So,0x261E,0x261E,0x261E},
{0x261F,So,0x261F,0x261F,0x261F},
{0x2620,So,0x2620,0x2620,0x2620},
{0x2621,So,0x2621,0x2621,0x2621},
{0x2622,So,0x2622,0x2622,0x2622},
{0x2623,So,0x2623,0x2623,0x2623},
{0x2624,So,0x2624,0x2624,0x2624},
{0x2625,So,0x2625,0x2625,0x2625},
{0x2626,So,0x2626,0x2626,0x2626},
{0x2627,So,0x2627,0x2627,0x2627},
{0x2628,So,0x2628,0x2628,0x2628},
{0x2629,So,0x2629,0x2629,0x2629},
{0x262A,So,0x262A,0x262A,0x262A},
{0x262B,So,0x262B,0x262B,0x262B},
{0x262C,So,0x262C,0x262C,0x262C},
{0x262D,So,0x262D,0x262D,0x262D},
{0x262E,So,0x262E,0x262E,0x262E},
{0x262F,So,0x262F,0x262F,0x262F},
{0x2630,So,0x2630,0x2630,0x2630},
{0x2631,So,0x2631,0x2631,0x2631},
{0x2632,So,0x2632,0x2632,0x2632},
{0x2633,So,0x2633,0x2633,0x2633},
{0x2634,So,0x2634,0x2634,0x2634},
{0x2635,So,0x2635,0x2635,0x2635},
{0x2636,So,0x2636,0x2636,0x2636},
{0x2637,So,0x2637,0x2637,0x2637},
{0x2638,So,0x2638,0x2638,0x2638},
{0x2639,So,0x2639,0x2639,0x2639},
{0x263A,So,0x263A,0x263A,0x263A},
{0x263B,So,0x263B,0x263B,0x263B},
{0x263C,So,0x263C,0x263C,0x263C},
{0x263D,So,0x263D,0x263D,0x263D},
{0x263E,So,0x263E,0x263E,0x263E},
{0x263F,So,0x263F,0x263F,0x263F},
{0x2640,So,0x2640,0x2640,0x2640},
{0x2641,So,0x2641,0x2641,0x2641},
{0x2642,So,0x2642,0x2642,0x2642},
{0x2643,So,0x2643,0x2643,0x2643},
{0x2644,So,0x2644,0x2644,0x2644},
{0x2645,So,0x2645,0x2645,0x2645},
{0x2646,So,0x2646,0x2646,0x2646},
{0x2647,So,0x2647,0x2647,0x2647},
{0x2648,So,0x2648,0x2648,0x2648},
{0x2649,So,0x2649,0x2649,0x2649},
{0x264A,So,0x264A,0x264A,0x264A},
{0x264B,So,0x264B,0x264B,0x264B},
{0x264C,So,0x264C,0x264C,0x264C},
{0x264D,So,0x264D,0x264D,0x264D},
{0x264E,So,0x264E,0x264E,0x264E},
{0x264F,So,0x264F,0x264F,0x264F},
{0x2650,So,0x2650,0x2650,0x2650},
{0x2651,So,0x2651,0x2651,0x2651},
{0x2652,So,0x2652,0x2652,0x2652},
{0x2653,So,0x2653,0x2653,0x2653},
{0x2654,So,0x2654,0x2654,0x2654},
{0x2655,So,0x2655,0x2655,0x2655},
{0x2656,So,0x2656,0x2656,0x2656},
{0x2657,So,0x2657,0x2657,0x2657},
{0x2658,So,0x2658,0x2658,0x2658},
{0x2659,So,0x2659,0x2659,0x2659},
{0x265A,So,0x265A,0x265A,0x265A},
{0x265B,So,0x265B,0x265B,0x265B},
{0x265C,So,0x265C,0x265C,0x265C},
{0x265D,So,0x265D,0x265D,0x265D},
{0x265E,So,0x265E,0x265E,0x265E},
{0x265F,So,0x265F,0x265F,0x265F},
{0x2660,So,0x2660,0x2660,0x2660},
{0x2661,So,0x2661,0x2661,0x2661},
{0x2662,So,0x2662,0x2662,0x2662},
{0x2663,So,0x2663,0x2663,0x2663},
{0x2664,So,0x2664,0x2664,0x2664},
{0x2665,So,0x2665,0x2665,0x2665},
{0x2666,So,0x2666,0x2666,0x2666},
{0x2667,So,0x2667,0x2667,0x2667},
{0x2668,So,0x2668,0x2668,0x2668},
{0x2669,So,0x2669,0x2669,0x2669},
{0x266A,So,0x266A,0x266A,0x266A},
{0x266B,So,0x266B,0x266B,0x266B},
{0x266C,So,0x266C,0x266C,0x266C},
{0x266D,So,0x266D,0x266D,0x266D},
{0x266E,So,0x266E,0x266E,0x266E},
{0x266F,Sm,0x266F,0x266F,0x266F},
{0x2670,So,0x2670,0x2670,0x2670},
{0x2671,So,0x2671,0x2671,0x2671},
{0x2672,So,0x2672,0x2672,0x2672},
{0x2673,So,0x2673,0x2673,0x2673},
{0x2674,So,0x2674,0x2674,0x2674},
{0x2675,So,0x2675,0x2675,0x2675},
{0x2676,So,0x2676,0x2676,0x2676},
{0x2677,So,0x2677,0x2677,0x2677},
{0x2678,So,0x2678,0x2678,0x2678},
{0x2679,So,0x2679,0x2679,0x2679},
{0x267A,So,0x267A,0x267A,0x267A},
{0x267B,So,0x267B,0x267B,0x267B},
{0x267C,So,0x267C,0x267C,0x267C},
{0x267D,So,0x267D,0x267D,0x267D},
{0x267E,So,0x267E,0x267E,0x267E},
{0x267F,So,0x267F,0x267F,0x267F},
{0x2680,So,0x2680,0x2680,0x2680},
{0x2681,So,0x2681,0x2681,0x2681},
{0x2682,So,0x2682,0x2682,0x2682},
{0x2683,So,0x2683,0x2683,0x2683},
{0x2684,So,0x2684,0x2684,0x2684},
{0x2685,So,0x2685,0x2685,0x2685},
{0x2686,So,0x2686,0x2686,0x2686},
{0x2687,So,0x2687,0x2687,0x2687},
{0x2688,So,0x2688,0x2688,0x2688},
{0x2689,So,0x2689,0x2689,0x2689},
{0x268A,So,0x268A,0x268A,0x268A},
{0x268B,So,0x268B,0x268B,0x268B},
{0x268C,So,0x268C,0x268C,0x268C},
{0x268D,So,0x268D,0x268D,0x268D},
{0x268E,So,0x268E,0x268E,0x268E},
{0x268F,So,0x268F,0x268F,0x268F},
{0x2690,So,0x2690,0x2690,0x2690},
{0x2691,So,0x2691,0x2691,0x2691},
{0x2692,So,0x2692,0x2692,0x2692},
{0x2693,So,0x2693,0x2693,0x2693},
{0x2694,So,0x2694,0x2694,0x2694},
{0x2695,So,0x2695,0x2695,0x2695},
{0x2696,So,0x2696,0x2696,0x2696},
{0x2697,So,0x2697,0x2697,0x2697},
{0x2698,So,0x2698,0x2698,0x2698},
{0x2699,So,0x2699,0x2699,0x2699},
{0x269A,So,0x269A,0x269A,0x269A},
{0x269B,So,0x269B,0x269B,0x269B},
{0x269C,So,0x269C,0x269C,0x269C},
{0x26A0,So,0x26A0,0x26A0,0x26A0},
{0x26A1,So,0x26A1,0x26A1,0x26A1},
{0x26A2,So,0x26A2,0x26A2,0x26A2},
{0x26A3,So,0x26A3,0x26A3,0x26A3},
{0x26A4,So,0x26A4,0x26A4,0x26A4},
{0x26A5,So,0x26A5,0x26A5,0x26A5},
{0x26A6,So,0x26A6,0x26A6,0x26A6},
{0x26A7,So,0x26A7,0x26A7,0x26A7},
{0x26A8,So,0x26A8,0x26A8,0x26A8},
{0x26A9,So,0x26A9,0x26A9,0x26A9},
{0x26AA,So,0x26AA,0x26AA,0x26AA},
{0x26AB,So,0x26AB,0x26AB,0x26AB},
{0x26AC,So,0x26AC,0x26AC,0x26AC},
{0x26AD,So,0x26AD,0x26AD,0x26AD},
{0x26AE,So,0x26AE,0x26AE,0x26AE},
{0x26AF,So,0x26AF,0x26AF,0x26AF},
{0x26B0,So,0x26B0,0x26B0,0x26B0},
{0x26B1,So,0x26B1,0x26B1,0x26B1},
{0x26B2,So,0x26B2,0x26B2,0x26B2},
{0x2701,So,0x2701,0x2701,0x2701},
{0x2702,So,0x2702,0x2702,0x2702},
{0x2703,So,0x2703,0x2703,0x2703},
{0x2704,So,0x2704,0x2704,0x2704},
{0x2706,So,0x2706,0x2706,0x2706},
{0x2707,So,0x2707,0x2707,0x2707},
{0x2708,So,0x2708,0x2708,0x2708},
{0x2709,So,0x2709,0x2709,0x2709},
{0x270C,So,0x270C,0x270C,0x270C},
{0x270D,So,0x270D,0x270D,0x270D},
{0x270E,So,0x270E,0x270E,0x270E},
{0x270F,So,0x270F,0x270F,0x270F},
{0x2710,So,0x2710,0x2710,0x2710},
{0x2711,So,0x2711,0x2711,0x2711},
{0x2712,So,0x2712,0x2712,0x2712},
{0x2713,So,0x2713,0x2713,0x2713},
{0x2714,So,0x2714,0x2714,0x2714},
{0x2715,So,0x2715,0x2715,0x2715},
{0x2716,So,0x2716,0x2716,0x2716},
{0x2717,So,0x2717,0x2717,0x2717},
{0x2718,So,0x2718,0x2718,0x2718},
{0x2719,So,0x2719,0x2719,0x2719},
{0x271A,So,0x271A,0x271A,0x271A},
{0x271B,So,0x271B,0x271B,0x271B},
{0x271C,So,0x271C,0x271C,0x271C},
{0x271D,So,0x271D,0x271D,0x271D},
{0x271E,So,0x271E,0x271E,0x271E},
{0x271F,So,0x271F,0x271F,0x271F},
{0x2720,So,0x2720,0x2720,0x2720},
{0x2721,So,0x2721,0x2721,0x2721},
{0x2722,So,0x2722,0x2722,0x2722},
{0x2723,So,0x2723,0x2723,0x2723},
{0x2724,So,0x2724,0x2724,0x2724},
{0x2725,So,0x2725,0x2725,0x2725},
{0x2726,So,0x2726,0x2726,0x2726},
{0x2727,So,0x2727,0x2727,0x2727},
{0x2729,So,0x2729,0x2729,0x2729},
{0x272A,So,0x272A,0x272A,0x272A},
{0x272B,So,0x272B,0x272B,0x272B},
{0x272C,So,0x272C,0x272C,0x272C},
{0x272D,So,0x272D,0x272D,0x272D},
{0x272E,So,0x272E,0x272E,0x272E},
{0x272F,So,0x272F,0x272F,0x272F},
{0x2730,So,0x2730,0x2730,0x2730},
{0x2731,So,0x2731,0x2731,0x2731},
{0x2732,So,0x2732,0x2732,0x2732},
{0x2733,So,0x2733,0x2733,0x2733},
{0x2734,So,0x2734,0x2734,0x2734},
{0x2735,So,0x2735,0x2735,0x2735},
{0x2736,So,0x2736,0x2736,0x2736},
{0x2737,So,0x2737,0x2737,0x2737},
{0x2738,So,0x2738,0x2738,0x2738},
{0x2739,So,0x2739,0x2739,0x2739},
{0x273A,So,0x273A,0x273A,0x273A},
{0x273B,So,0x273B,0x273B,0x273B},
{0x273C,So,0x273C,0x273C,0x273C},
{0x273D,So,0x273D,0x273D,0x273D},
{0x273E,So,0x273E,0x273E,0x273E},
{0x273F,So,0x273F,0x273F,0x273F},
{0x2740,So,0x2740,0x2740,0x2740},
{0x2741,So,0x2741,0x2741,0x2741},
{0x2742,So,0x2742,0x2742,0x2742},
{0x2743,So,0x2743,0x2743,0x2743},
{0x2744,So,0x2744,0x2744,0x2744},
{0x2745,So,0x2745,0x2745,0x2745},
{0x2746,So,0x2746,0x2746,0x2746},
{0x2747,So,0x2747,0x2747,0x2747},
{0x2748,So,0x2748,0x2748,0x2748},
{0x2749,So,0x2749,0x2749,0x2749},
{0x274A,So,0x274A,0x274A,0x274A},
{0x274B,So,0x274B,0x274B,0x274B},
{0x274D,So,0x274D,0x274D,0x274D},
{0x274F,So,0x274F,0x274F,0x274F},
{0x2750,So,0x2750,0x2750,0x2750},
{0x2751,So,0x2751,0x2751,0x2751},
{0x2752,So,0x2752,0x2752,0x2752},
{0x2756,So,0x2756,0x2756,0x2756},
{0x2758,So,0x2758,0x2758,0x2758},
{0x2759,So,0x2759,0x2759,0x2759},
{0x275A,So,0x275A,0x275A,0x275A},
{0x275B,So,0x275B,0x275B,0x275B},
{0x275C,So,0x275C,0x275C,0x275C},
{0x275D,So,0x275D,0x275D,0x275D},
{0x275E,So,0x275E,0x275E,0x275E},
{0x2761,So,0x2761,0x2761,0x2761},
{0x2762,So,0x2762,0x2762,0x2762},
{0x2763,So,0x2763,0x2763,0x2763},
{0x2764,So,0x2764,0x2764,0x2764},
{0x2765,So,0x2765,0x2765,0x2765},
{0x2766,So,0x2766,0x2766,0x2766},
{0x2767,So,0x2767,0x2767,0x2767},
{0x2768,Ps,0x2768,0x2768,0x2768},
{0x2769,Pe,0x2769,0x2769,0x2769},
{0x276A,Ps,0x276A,0x276A,0x276A},
{0x276B,Pe,0x276B,0x276B,0x276B},
{0x276C,Ps,0x276C,0x276C,0x276C},
{0x276D,Pe,0x276D,0x276D,0x276D},
{0x276E,Ps,0x276E,0x276E,0x276E},
{0x276F,Pe,0x276F,0x276F,0x276F},
{0x2770,Ps,0x2770,0x2770,0x2770},
{0x2771,Pe,0x2771,0x2771,0x2771},
{0x2772,Ps,0x2772,0x2772,0x2772},
{0x2773,Pe,0x2773,0x2773,0x2773},
{0x2774,Ps,0x2774,0x2774,0x2774},
{0x2775,Pe,0x2775,0x2775,0x2775},
{0x2776,No,0x2776,0x2776,0x2776},
{0x2777,No,0x2777,0x2777,0x2777},
{0x2778,No,0x2778,0x2778,0x2778},
{0x2779,No,0x2779,0x2779,0x2779},
{0x277A,No,0x277A,0x277A,0x277A},
{0x277B,No,0x277B,0x277B,0x277B},
{0x277C,No,0x277C,0x277C,0x277C},
{0x277D,No,0x277D,0x277D,0x277D},
{0x277E,No,0x277E,0x277E,0x277E},
{0x277F,No,0x277F,0x277F,0x277F},
{0x2780,No,0x2780,0x2780,0x2780},
{0x2781,No,0x2781,0x2781,0x2781},
{0x2782,No,0x2782,0x2782,0x2782},
{0x2783,No,0x2783,0x2783,0x2783},
{0x2784,No,0x2784,0x2784,0x2784},
{0x2785,No,0x2785,0x2785,0x2785},
{0x2786,No,0x2786,0x2786,0x2786},
{0x2787,No,0x2787,0x2787,0x2787},
{0x2788,No,0x2788,0x2788,0x2788},
{0x2789,No,0x2789,0x2789,0x2789},
{0x278A,No,0x278A,0x278A,0x278A},
{0x278B,No,0x278B,0x278B,0x278B},
{0x278C,No,0x278C,0x278C,0x278C},
{0x278D,No,0x278D,0x278D,0x278D},
{0x278E,No,0x278E,0x278E,0x278E},
{0x278F,No,0x278F,0x278F,0x278F},
{0x2790,No,0x2790,0x2790,0x2790},
{0x2791,No,0x2791,0x2791,0x2791},
{0x2792,No,0x2792,0x2792,0x2792},
{0x2793,No,0x2793,0x2793,0x2793},
{0x2794,So,0x2794,0x2794,0x2794},
{0x2798,So,0x2798,0x2798,0x2798},
{0x2799,So,0x2799,0x2799,0x2799},
{0x279A,So,0x279A,0x279A,0x279A},
{0x279B,So,0x279B,0x279B,0x279B},
{0x279C,So,0x279C,0x279C,0x279C},
{0x279D,So,0x279D,0x279D,0x279D},
{0x279E,So,0x279E,0x279E,0x279E},
{0x279F,So,0x279F,0x279F,0x279F},
{0x27A0,So,0x27A0,0x27A0,0x27A0},
{0x27A1,So,0x27A1,0x27A1,0x27A1},
{0x27A2,So,0x27A2,0x27A2,0x27A2},
{0x27A3,So,0x27A3,0x27A3,0x27A3},
{0x27A4,So,0x27A4,0x27A4,0x27A4},
{0x27A5,So,0x27A5,0x27A5,0x27A5},
{0x27A6,So,0x27A6,0x27A6,0x27A6},
{0x27A7,So,0x27A7,0x27A7,0x27A7},
{0x27A8,So,0x27A8,0x27A8,0x27A8},
{0x27A9,So,0x27A9,0x27A9,0x27A9},
{0x27AA,So,0x27AA,0x27AA,0x27AA},
{0x27AB,So,0x27AB,0x27AB,0x27AB},
{0x27AC,So,0x27AC,0x27AC,0x27AC},
{0x27AD,So,0x27AD,0x27AD,0x27AD},
{0x27AE,So,0x27AE,0x27AE,0x27AE},
{0x27AF,So,0x27AF,0x27AF,0x27AF},
{0x27B1,So,0x27B1,0x27B1,0x27B1},
{0x27B2,So,0x27B2,0x27B2,0x27B2},
{0x27B3,So,0x27B3,0x27B3,0x27B3},
{0x27B4,So,0x27B4,0x27B4,0x27B4},
{0x27B5,So,0x27B5,0x27B5,0x27B5},
{0x27B6,So,0x27B6,0x27B6,0x27B6},
{0x27B7,So,0x27B7,0x27B7,0x27B7},
{0x27B8,So,0x27B8,0x27B8,0x27B8},
{0x27B9,So,0x27B9,0x27B9,0x27B9},
{0x27BA,So,0x27BA,0x27BA,0x27BA},
{0x27BB,So,0x27BB,0x27BB,0x27BB},
{0x27BC,So,0x27BC,0x27BC,0x27BC},
{0x27BD,So,0x27BD,0x27BD,0x27BD},
{0x27BE,So,0x27BE,0x27BE,0x27BE},
{0x27C0,Sm,0x27C0,0x27C0,0x27C0},
{0x27C1,Sm,0x27C1,0x27C1,0x27C1},
{0x27C2,Sm,0x27C2,0x27C2,0x27C2},
{0x27C3,Sm,0x27C3,0x27C3,0x27C3},
{0x27C4,Sm,0x27C4,0x27C4,0x27C4},
{0x27C5,Ps,0x27C5,0x27C5,0x27C5},
{0x27C6,Pe,0x27C6,0x27C6,0x27C6},
{0x27C7,Sm,0x27C7,0x27C7,0x27C7},
{0x27C8,Sm,0x27C8,0x27C8,0x27C8},
{0x27C9,Sm,0x27C9,0x27C9,0x27C9},
{0x27CA,Sm,0x27CA,0x27CA,0x27CA},
{0x27D0,Sm,0x27D0,0x27D0,0x27D0},
{0x27D1,Sm,0x27D1,0x27D1,0x27D1},
{0x27D2,Sm,0x27D2,0x27D2,0x27D2},
{0x27D3,Sm,0x27D3,0x27D3,0x27D3},
{0x27D4,Sm,0x27D4,0x27D4,0x27D4},
{0x27D5,Sm,0x27D5,0x27D5,0x27D5},
{0x27D6,Sm,0x27D6,0x27D6,0x27D6},
{0x27D7,Sm,0x27D7,0x27D7,0x27D7},
{0x27D8,Sm,0x27D8,0x27D8,0x27D8},
{0x27D9,Sm,0x27D9,0x27D9,0x27D9},
{0x27DA,Sm,0x27DA,0x27DA,0x27DA},
{0x27DB,Sm,0x27DB,0x27DB,0x27DB},
{0x27DC,Sm,0x27DC,0x27DC,0x27DC},
{0x27DD,Sm,0x27DD,0x27DD,0x27DD},
{0x27DE,Sm,0x27DE,0x27DE,0x27DE},
{0x27DF,Sm,0x27DF,0x27DF,0x27DF},
{0x27E0,Sm,0x27E0,0x27E0,0x27E0},
{0x27E1,Sm,0x27E1,0x27E1,0x27E1},
{0x27E2,Sm,0x27E2,0x27E2,0x27E2},
{0x27E3,Sm,0x27E3,0x27E3,0x27E3},
{0x27E4,Sm,0x27E4,0x27E4,0x27E4},
{0x27E5,Sm,0x27E5,0x27E5,0x27E5},
{0x27E6,Ps,0x27E6,0x27E6,0x27E6},
{0x27E7,Pe,0x27E7,0x27E7,0x27E7},
{0x27E8,Ps,0x27E8,0x27E8,0x27E8},
{0x27E9,Pe,0x27E9,0x27E9,0x27E9},
{0x27EA,Ps,0x27EA,0x27EA,0x27EA},
{0x27EB,Pe,0x27EB,0x27EB,0x27EB},
{0x27F0,Sm,0x27F0,0x27F0,0x27F0},
{0x27F1,Sm,0x27F1,0x27F1,0x27F1},
{0x27F2,Sm,0x27F2,0x27F2,0x27F2},
{0x27F3,Sm,0x27F3,0x27F3,0x27F3},
{0x27F4,Sm,0x27F4,0x27F4,0x27F4},
{0x27F5,Sm,0x27F5,0x27F5,0x27F5},
{0x27F6,Sm,0x27F6,0x27F6,0x27F6},
{0x27F7,Sm,0x27F7,0x27F7,0x27F7},
{0x27F8,Sm,0x27F8,0x27F8,0x27F8},
{0x27F9,Sm,0x27F9,0x27F9,0x27F9},
{0x27FA,Sm,0x27FA,0x27FA,0x27FA},
{0x27FB,Sm,0x27FB,0x27FB,0x27FB},
{0x27FC,Sm,0x27FC,0x27FC,0x27FC},
{0x27FD,Sm,0x27FD,0x27FD,0x27FD},
{0x27FE,Sm,0x27FE,0x27FE,0x27FE},
{0x27FF,Sm,0x27FF,0x27FF,0x27FF},
{0x2800,So,0x2800,0x2800,0x2800},
{0x2801,So,0x2801,0x2801,0x2801},
{0x2802,So,0x2802,0x2802,0x2802},
{0x2803,So,0x2803,0x2803,0x2803},
{0x2804,So,0x2804,0x2804,0x2804},
{0x2805,So,0x2805,0x2805,0x2805},
{0x2806,So,0x2806,0x2806,0x2806},
{0x2807,So,0x2807,0x2807,0x2807},
{0x2808,So,0x2808,0x2808,0x2808},
{0x2809,So,0x2809,0x2809,0x2809},
{0x280A,So,0x280A,0x280A,0x280A},
{0x280B,So,0x280B,0x280B,0x280B},
{0x280C,So,0x280C,0x280C,0x280C},
{0x280D,So,0x280D,0x280D,0x280D},
{0x280E,So,0x280E,0x280E,0x280E},
{0x280F,So,0x280F,0x280F,0x280F},
{0x2810,So,0x2810,0x2810,0x2810},
{0x2811,So,0x2811,0x2811,0x2811},
{0x2812,So,0x2812,0x2812,0x2812},
{0x2813,So,0x2813,0x2813,0x2813},
{0x2814,So,0x2814,0x2814,0x2814},
{0x2815,So,0x2815,0x2815,0x2815},
{0x2816,So,0x2816,0x2816,0x2816},
{0x2817,So,0x2817,0x2817,0x2817},
{0x2818,So,0x2818,0x2818,0x2818},
{0x2819,So,0x2819,0x2819,0x2819},
{0x281A,So,0x281A,0x281A,0x281A},
{0x281B,So,0x281B,0x281B,0x281B},
{0x281C,So,0x281C,0x281C,0x281C},
{0x281D,So,0x281D,0x281D,0x281D},
{0x281E,So,0x281E,0x281E,0x281E},
{0x281F,So,0x281F,0x281F,0x281F},
{0x2820,So,0x2820,0x2820,0x2820},
{0x2821,So,0x2821,0x2821,0x2821},
{0x2822,So,0x2822,0x2822,0x2822},
{0x2823,So,0x2823,0x2823,0x2823},
{0x2824,So,0x2824,0x2824,0x2824},
{0x2825,So,0x2825,0x2825,0x2825},
{0x2826,So,0x2826,0x2826,0x2826},
{0x2827,So,0x2827,0x2827,0x2827},
{0x2828,So,0x2828,0x2828,0x2828},
{0x2829,So,0x2829,0x2829,0x2829},
{0x282A,So,0x282A,0x282A,0x282A},
{0x282B,So,0x282B,0x282B,0x282B},
{0x282C,So,0x282C,0x282C,0x282C},
{0x282D,So,0x282D,0x282D,0x282D},
{0x282E,So,0x282E,0x282E,0x282E},
{0x282F,So,0x282F,0x282F,0x282F},
{0x2830,So,0x2830,0x2830,0x2830},
{0x2831,So,0x2831,0x2831,0x2831},
{0x2832,So,0x2832,0x2832,0x2832},
{0x2833,So,0x2833,0x2833,0x2833},
{0x2834,So,0x2834,0x2834,0x2834},
{0x2835,So,0x2835,0x2835,0x2835},
{0x2836,So,0x2836,0x2836,0x2836},
{0x2837,So,0x2837,0x2837,0x2837},
{0x2838,So,0x2838,0x2838,0x2838},
{0x2839,So,0x2839,0x2839,0x2839},
{0x283A,So,0x283A,0x283A,0x283A},
{0x283B,So,0x283B,0x283B,0x283B},
{0x283C,So,0x283C,0x283C,0x283C},
{0x283D,So,0x283D,0x283D,0x283D},
{0x283E,So,0x283E,0x283E,0x283E},
{0x283F,So,0x283F,0x283F,0x283F},
{0x2840,So,0x2840,0x2840,0x2840},
{0x2841,So,0x2841,0x2841,0x2841},
{0x2842,So,0x2842,0x2842,0x2842},
{0x2843,So,0x2843,0x2843,0x2843},
{0x2844,So,0x2844,0x2844,0x2844},
{0x2845,So,0x2845,0x2845,0x2845},
{0x2846,So,0x2846,0x2846,0x2846},
{0x2847,So,0x2847,0x2847,0x2847},
{0x2848,So,0x2848,0x2848,0x2848},
{0x2849,So,0x2849,0x2849,0x2849},
{0x284A,So,0x284A,0x284A,0x284A},
{0x284B,So,0x284B,0x284B,0x284B},
{0x284C,So,0x284C,0x284C,0x284C},
{0x284D,So,0x284D,0x284D,0x284D},
{0x284E,So,0x284E,0x284E,0x284E},
{0x284F,So,0x284F,0x284F,0x284F},
{0x2850,So,0x2850,0x2850,0x2850},
{0x2851,So,0x2851,0x2851,0x2851},
{0x2852,So,0x2852,0x2852,0x2852},
{0x2853,So,0x2853,0x2853,0x2853},
{0x2854,So,0x2854,0x2854,0x2854},
{0x2855,So,0x2855,0x2855,0x2855},
{0x2856,So,0x2856,0x2856,0x2856},
{0x2857,So,0x2857,0x2857,0x2857},
{0x2858,So,0x2858,0x2858,0x2858},
{0x2859,So,0x2859,0x2859,0x2859},
{0x285A,So,0x285A,0x285A,0x285A},
{0x285B,So,0x285B,0x285B,0x285B},
{0x285C,So,0x285C,0x285C,0x285C},
{0x285D,So,0x285D,0x285D,0x285D},
{0x285E,So,0x285E,0x285E,0x285E},
{0x285F,So,0x285F,0x285F,0x285F},
{0x2860,So,0x2860,0x2860,0x2860},
{0x2861,So,0x2861,0x2861,0x2861},
{0x2862,So,0x2862,0x2862,0x2862},
{0x2863,So,0x2863,0x2863,0x2863},
{0x2864,So,0x2864,0x2864,0x2864},
{0x2865,So,0x2865,0x2865,0x2865},
{0x2866,So,0x2866,0x2866,0x2866},
{0x2867,So,0x2867,0x2867,0x2867},
{0x2868,So,0x2868,0x2868,0x2868},
{0x2869,So,0x2869,0x2869,0x2869},
{0x286A,So,0x286A,0x286A,0x286A},
{0x286B,So,0x286B,0x286B,0x286B},
{0x286C,So,0x286C,0x286C,0x286C},
{0x286D,So,0x286D,0x286D,0x286D},
{0x286E,So,0x286E,0x286E,0x286E},
{0x286F,So,0x286F,0x286F,0x286F},
{0x2870,So,0x2870,0x2870,0x2870},
{0x2871,So,0x2871,0x2871,0x2871},
{0x2872,So,0x2872,0x2872,0x2872},
{0x2873,So,0x2873,0x2873,0x2873},
{0x2874,So,0x2874,0x2874,0x2874},
{0x2875,So,0x2875,0x2875,0x2875},
{0x2876,So,0x2876,0x2876,0x2876},
{0x2877,So,0x2877,0x2877,0x2877},
{0x2878,So,0x2878,0x2878,0x2878},
{0x2879,So,0x2879,0x2879,0x2879},
{0x287A,So,0x287A,0x287A,0x287A},
{0x287B,So,0x287B,0x287B,0x287B},
{0x287C,So,0x287C,0x287C,0x287C},
{0x287D,So,0x287D,0x287D,0x287D},
{0x287E,So,0x287E,0x287E,0x287E},
{0x287F,So,0x287F,0x287F,0x287F},
{0x2880,So,0x2880,0x2880,0x2880},
{0x2881,So,0x2881,0x2881,0x2881},
{0x2882,So,0x2882,0x2882,0x2882},
{0x2883,So,0x2883,0x2883,0x2883},
{0x2884,So,0x2884,0x2884,0x2884},
{0x2885,So,0x2885,0x2885,0x2885},
{0x2886,So,0x2886,0x2886,0x2886},
{0x2887,So,0x2887,0x2887,0x2887},
{0x2888,So,0x2888,0x2888,0x2888},
{0x2889,So,0x2889,0x2889,0x2889},
{0x288A,So,0x288A,0x288A,0x288A},
{0x288B,So,0x288B,0x288B,0x288B},
{0x288C,So,0x288C,0x288C,0x288C},
{0x288D,So,0x288D,0x288D,0x288D},
{0x288E,So,0x288E,0x288E,0x288E},
{0x288F,So,0x288F,0x288F,0x288F},
{0x2890,So,0x2890,0x2890,0x2890},
{0x2891,So,0x2891,0x2891,0x2891},
{0x2892,So,0x2892,0x2892,0x2892},
{0x2893,So,0x2893,0x2893,0x2893},
{0x2894,So,0x2894,0x2894,0x2894},
{0x2895,So,0x2895,0x2895,0x2895},
{0x2896,So,0x2896,0x2896,0x2896},
{0x2897,So,0x2897,0x2897,0x2897},
{0x2898,So,0x2898,0x2898,0x2898},
{0x2899,So,0x2899,0x2899,0x2899},
{0x289A,So,0x289A,0x289A,0x289A},
{0x289B,So,0x289B,0x289B,0x289B},
{0x289C,So,0x289C,0x289C,0x289C},
{0x289D,So,0x289D,0x289D,0x289D},
{0x289E,So,0x289E,0x289E,0x289E},
{0x289F,So,0x289F,0x289F,0x289F},
{0x28A0,So,0x28A0,0x28A0,0x28A0},
{0x28A1,So,0x28A1,0x28A1,0x28A1},
{0x28A2,So,0x28A2,0x28A2,0x28A2},
{0x28A3,So,0x28A3,0x28A3,0x28A3},
{0x28A4,So,0x28A4,0x28A4,0x28A4},
{0x28A5,So,0x28A5,0x28A5,0x28A5},
{0x28A6,So,0x28A6,0x28A6,0x28A6},
{0x28A7,So,0x28A7,0x28A7,0x28A7},
{0x28A8,So,0x28A8,0x28A8,0x28A8},
{0x28A9,So,0x28A9,0x28A9,0x28A9},
{0x28AA,So,0x28AA,0x28AA,0x28AA},
{0x28AB,So,0x28AB,0x28AB,0x28AB},
{0x28AC,So,0x28AC,0x28AC,0x28AC},
{0x28AD,So,0x28AD,0x28AD,0x28AD},
{0x28AE,So,0x28AE,0x28AE,0x28AE},
{0x28AF,So,0x28AF,0x28AF,0x28AF},
{0x28B0,So,0x28B0,0x28B0,0x28B0},
{0x28B1,So,0x28B1,0x28B1,0x28B1},
{0x28B2,So,0x28B2,0x28B2,0x28B2},
{0x28B3,So,0x28B3,0x28B3,0x28B3},
{0x28B4,So,0x28B4,0x28B4,0x28B4},
{0x28B5,So,0x28B5,0x28B5,0x28B5},
{0x28B6,So,0x28B6,0x28B6,0x28B6},
{0x28B7,So,0x28B7,0x28B7,0x28B7},
{0x28B8,So,0x28B8,0x28B8,0x28B8},
{0x28B9,So,0x28B9,0x28B9,0x28B9},
{0x28BA,So,0x28BA,0x28BA,0x28BA},
{0x28BB,So,0x28BB,0x28BB,0x28BB},
{0x28BC,So,0x28BC,0x28BC,0x28BC},
{0x28BD,So,0x28BD,0x28BD,0x28BD},
{0x28BE,So,0x28BE,0x28BE,0x28BE},
{0x28BF,So,0x28BF,0x28BF,0x28BF},
{0x28C0,So,0x28C0,0x28C0,0x28C0},
{0x28C1,So,0x28C1,0x28C1,0x28C1},
{0x28C2,So,0x28C2,0x28C2,0x28C2},
{0x28C3,So,0x28C3,0x28C3,0x28C3},
{0x28C4,So,0x28C4,0x28C4,0x28C4},
{0x28C5,So,0x28C5,0x28C5,0x28C5},
{0x28C6,So,0x28C6,0x28C6,0x28C6},
{0x28C7,So,0x28C7,0x28C7,0x28C7},
{0x28C8,So,0x28C8,0x28C8,0x28C8},
{0x28C9,So,0x28C9,0x28C9,0x28C9},
{0x28CA,So,0x28CA,0x28CA,0x28CA},
{0x28CB,So,0x28CB,0x28CB,0x28CB},
{0x28CC,So,0x28CC,0x28CC,0x28CC},
{0x28CD,So,0x28CD,0x28CD,0x28CD},
{0x28CE,So,0x28CE,0x28CE,0x28CE},
{0x28CF,So,0x28CF,0x28CF,0x28CF},
{0x28D0,So,0x28D0,0x28D0,0x28D0},
{0x28D1,So,0x28D1,0x28D1,0x28D1},
{0x28D2,So,0x28D2,0x28D2,0x28D2},
{0x28D3,So,0x28D3,0x28D3,0x28D3},
{0x28D4,So,0x28D4,0x28D4,0x28D4},
{0x28D5,So,0x28D5,0x28D5,0x28D5},
{0x28D6,So,0x28D6,0x28D6,0x28D6},
{0x28D7,So,0x28D7,0x28D7,0x28D7},
{0x28D8,So,0x28D8,0x28D8,0x28D8},
{0x28D9,So,0x28D9,0x28D9,0x28D9},
{0x28DA,So,0x28DA,0x28DA,0x28DA},
{0x28DB,So,0x28DB,0x28DB,0x28DB},
{0x28DC,So,0x28DC,0x28DC,0x28DC},
{0x28DD,So,0x28DD,0x28DD,0x28DD},
{0x28DE,So,0x28DE,0x28DE,0x28DE},
{0x28DF,So,0x28DF,0x28DF,0x28DF},
{0x28E0,So,0x28E0,0x28E0,0x28E0},
{0x28E1,So,0x28E1,0x28E1,0x28E1},
{0x28E2,So,0x28E2,0x28E2,0x28E2},
{0x28E3,So,0x28E3,0x28E3,0x28E3},
{0x28E4,So,0x28E4,0x28E4,0x28E4},
{0x28E5,So,0x28E5,0x28E5,0x28E5},
{0x28E6,So,0x28E6,0x28E6,0x28E6},
{0x28E7,So,0x28E7,0x28E7,0x28E7},
{0x28E8,So,0x28E8,0x28E8,0x28E8},
{0x28E9,So,0x28E9,0x28E9,0x28E9},
{0x28EA,So,0x28EA,0x28EA,0x28EA},
{0x28EB,So,0x28EB,0x28EB,0x28EB},
{0x28EC,So,0x28EC,0x28EC,0x28EC},
{0x28ED,So,0x28ED,0x28ED,0x28ED},
{0x28EE,So,0x28EE,0x28EE,0x28EE},
{0x28EF,So,0x28EF,0x28EF,0x28EF},
{0x28F0,So,0x28F0,0x28F0,0x28F0},
{0x28F1,So,0x28F1,0x28F1,0x28F1},
{0x28F2,So,0x28F2,0x28F2,0x28F2},
{0x28F3,So,0x28F3,0x28F3,0x28F3},
{0x28F4,So,0x28F4,0x28F4,0x28F4},
{0x28F5,So,0x28F5,0x28F5,0x28F5},
{0x28F6,So,0x28F6,0x28F6,0x28F6},
{0x28F7,So,0x28F7,0x28F7,0x28F7},
{0x28F8,So,0x28F8,0x28F8,0x28F8},
{0x28F9,So,0x28F9,0x28F9,0x28F9},
{0x28FA,So,0x28FA,0x28FA,0x28FA},
{0x28FB,So,0x28FB,0x28FB,0x28FB},
{0x28FC,So,0x28FC,0x28FC,0x28FC},
{0x28FD,So,0x28FD,0x28FD,0x28FD},
{0x28FE,So,0x28FE,0x28FE,0x28FE},
{0x28FF,So,0x28FF,0x28FF,0x28FF},
{0x2900,Sm,0x2900,0x2900,0x2900},
{0x2901,Sm,0x2901,0x2901,0x2901},
{0x2902,Sm,0x2902,0x2902,0x2902},
{0x2903,Sm,0x2903,0x2903,0x2903},
{0x2904,Sm,0x2904,0x2904,0x2904},
{0x2905,Sm,0x2905,0x2905,0x2905},
{0x2906,Sm,0x2906,0x2906,0x2906},
{0x2907,Sm,0x2907,0x2907,0x2907},
{0x2908,Sm,0x2908,0x2908,0x2908},
{0x2909,Sm,0x2909,0x2909,0x2909},
{0x290A,Sm,0x290A,0x290A,0x290A},
{0x290B,Sm,0x290B,0x290B,0x290B},
{0x290C,Sm,0x290C,0x290C,0x290C},
{0x290D,Sm,0x290D,0x290D,0x290D},
{0x290E,Sm,0x290E,0x290E,0x290E},
{0x290F,Sm,0x290F,0x290F,0x290F},
{0x2910,Sm,0x2910,0x2910,0x2910},
{0x2911,Sm,0x2911,0x2911,0x2911},
{0x2912,Sm,0x2912,0x2912,0x2912},
{0x2913,Sm,0x2913,0x2913,0x2913},
{0x2914,Sm,0x2914,0x2914,0x2914},
{0x2915,Sm,0x2915,0x2915,0x2915},
{0x2916,Sm,0x2916,0x2916,0x2916},
{0x2917,Sm,0x2917,0x2917,0x2917},
{0x2918,Sm,0x2918,0x2918,0x2918},
{0x2919,Sm,0x2919,0x2919,0x2919},
{0x291A,Sm,0x291A,0x291A,0x291A},
{0x291B,Sm,0x291B,0x291B,0x291B},
{0x291C,Sm,0x291C,0x291C,0x291C},
{0x291D,Sm,0x291D,0x291D,0x291D},
{0x291E,Sm,0x291E,0x291E,0x291E},
{0x291F,Sm,0x291F,0x291F,0x291F},
{0x2920,Sm,0x2920,0x2920,0x2920},
{0x2921,Sm,0x2921,0x2921,0x2921},
{0x2922,Sm,0x2922,0x2922,0x2922},
{0x2923,Sm,0x2923,0x2923,0x2923},
{0x2924,Sm,0x2924,0x2924,0x2924},
{0x2925,Sm,0x2925,0x2925,0x2925},
{0x2926,Sm,0x2926,0x2926,0x2926},
{0x2927,Sm,0x2927,0x2927,0x2927},
{0x2928,Sm,0x2928,0x2928,0x2928},
{0x2929,Sm,0x2929,0x2929,0x2929},
{0x292A,Sm,0x292A,0x292A,0x292A},
{0x292B,Sm,0x292B,0x292B,0x292B},
{0x292C,Sm,0x292C,0x292C,0x292C},
{0x292D,Sm,0x292D,0x292D,0x292D},
{0x292E,Sm,0x292E,0x292E,0x292E},
{0x292F,Sm,0x292F,0x292F,0x292F},
{0x2930,Sm,0x2930,0x2930,0x2930},
{0x2931,Sm,0x2931,0x2931,0x2931},
{0x2932,Sm,0x2932,0x2932,0x2932},
{0x2933,Sm,0x2933,0x2933,0x2933},
{0x2934,Sm,0x2934,0x2934,0x2934},
{0x2935,Sm,0x2935,0x2935,0x2935},
{0x2936,Sm,0x2936,0x2936,0x2936},
{0x2937,Sm,0x2937,0x2937,0x2937},
{0x2938,Sm,0x2938,0x2938,0x2938},
{0x2939,Sm,0x2939,0x2939,0x2939},
{0x293A,Sm,0x293A,0x293A,0x293A},
{0x293B,Sm,0x293B,0x293B,0x293B},
{0x293C,Sm,0x293C,0x293C,0x293C},
{0x293D,Sm,0x293D,0x293D,0x293D},
{0x293E,Sm,0x293E,0x293E,0x293E},
{0x293F,Sm,0x293F,0x293F,0x293F},
{0x2940,Sm,0x2940,0x2940,0x2940},
{0x2941,Sm,0x2941,0x2941,0x2941},
{0x2942,Sm,0x2942,0x2942,0x2942},
{0x2943,Sm,0x2943,0x2943,0x2943},
{0x2944,Sm,0x2944,0x2944,0x2944},
{0x2945,Sm,0x2945,0x2945,0x2945},
{0x2946,Sm,0x2946,0x2946,0x2946},
{0x2947,Sm,0x2947,0x2947,0x2947},
{0x2948,Sm,0x2948,0x2948,0x2948},
{0x2949,Sm,0x2949,0x2949,0x2949},
{0x294A,Sm,0x294A,0x294A,0x294A},
{0x294B,Sm,0x294B,0x294B,0x294B},
{0x294C,Sm,0x294C,0x294C,0x294C},
{0x294D,Sm,0x294D,0x294D,0x294D},
{0x294E,Sm,0x294E,0x294E,0x294E},
{0x294F,Sm,0x294F,0x294F,0x294F},
{0x2950,Sm,0x2950,0x2950,0x2950},
{0x2951,Sm,0x2951,0x2951,0x2951},
{0x2952,Sm,0x2952,0x2952,0x2952},
{0x2953,Sm,0x2953,0x2953,0x2953},
{0x2954,Sm,0x2954,0x2954,0x2954},
{0x2955,Sm,0x2955,0x2955,0x2955},
{0x2956,Sm,0x2956,0x2956,0x2956},
{0x2957,Sm,0x2957,0x2957,0x2957},
{0x2958,Sm,0x2958,0x2958,0x2958},
{0x2959,Sm,0x2959,0x2959,0x2959},
{0x295A,Sm,0x295A,0x295A,0x295A},
{0x295B,Sm,0x295B,0x295B,0x295B},
{0x295C,Sm,0x295C,0x295C,0x295C},
{0x295D,Sm,0x295D,0x295D,0x295D},
{0x295E,Sm,0x295E,0x295E,0x295E},
{0x295F,Sm,0x295F,0x295F,0x295F},
{0x2960,Sm,0x2960,0x2960,0x2960},
{0x2961,Sm,0x2961,0x2961,0x2961},
{0x2962,Sm,0x2962,0x2962,0x2962},
{0x2963,Sm,0x2963,0x2963,0x2963},
{0x2964,Sm,0x2964,0x2964,0x2964},
{0x2965,Sm,0x2965,0x2965,0x2965},
{0x2966,Sm,0x2966,0x2966,0x2966},
{0x2967,Sm,0x2967,0x2967,0x2967},
{0x2968,Sm,0x2968,0x2968,0x2968},
{0x2969,Sm,0x2969,0x2969,0x2969},
{0x296A,Sm,0x296A,0x296A,0x296A},
{0x296B,Sm,0x296B,0x296B,0x296B},
{0x296C,Sm,0x296C,0x296C,0x296C},
{0x296D,Sm,0x296D,0x296D,0x296D},
{0x296E,Sm,0x296E,0x296E,0x296E},
{0x296F,Sm,0x296F,0x296F,0x296F},
{0x2970,Sm,0x2970,0x2970,0x2970},
{0x2971,Sm,0x2971,0x2971,0x2971},
{0x2972,Sm,0x2972,0x2972,0x2972},
{0x2973,Sm,0x2973,0x2973,0x2973},
{0x2974,Sm,0x2974,0x2974,0x2974},
{0x2975,Sm,0x2975,0x2975,0x2975},
{0x2976,Sm,0x2976,0x2976,0x2976},
{0x2977,Sm,0x2977,0x2977,0x2977},
{0x2978,Sm,0x2978,0x2978,0x2978},
{0x2979,Sm,0x2979,0x2979,0x2979},
{0x297A,Sm,0x297A,0x297A,0x297A},
{0x297B,Sm,0x297B,0x297B,0x297B},
{0x297C,Sm,0x297C,0x297C,0x297C},
{0x297D,Sm,0x297D,0x297D,0x297D},
{0x297E,Sm,0x297E,0x297E,0x297E},
{0x297F,Sm,0x297F,0x297F,0x297F},
{0x2980,Sm,0x2980,0x2980,0x2980},
{0x2981,Sm,0x2981,0x2981,0x2981},
{0x2982,Sm,0x2982,0x2982,0x2982},
{0x2983,Ps,0x2983,0x2983,0x2983},
{0x2984,Pe,0x2984,0x2984,0x2984},
{0x2985,Ps,0x2985,0x2985,0x2985},
{0x2986,Pe,0x2986,0x2986,0x2986},
{0x2987,Ps,0x2987,0x2987,0x2987},
{0x2988,Pe,0x2988,0x2988,0x2988},
{0x2989,Ps,0x2989,0x2989,0x2989},
{0x298A,Pe,0x298A,0x298A,0x298A},
{0x298B,Ps,0x298B,0x298B,0x298B},
{0x298C,Pe,0x298C,0x298C,0x298C},
{0x298D,Ps,0x298D,0x298D,0x298D},
{0x298E,Pe,0x298E,0x298E,0x298E},
{0x298F,Ps,0x298F,0x298F,0x298F},
{0x2990,Pe,0x2990,0x2990,0x2990},
{0x2991,Ps,0x2991,0x2991,0x2991},
{0x2992,Pe,0x2992,0x2992,0x2992},
{0x2993,Ps,0x2993,0x2993,0x2993},
{0x2994,Pe,0x2994,0x2994,0x2994},
{0x2995,Ps,0x2995,0x2995,0x2995},
{0x2996,Pe,0x2996,0x2996,0x2996},
{0x2997,Ps,0x2997,0x2997,0x2997},
{0x2998,Pe,0x2998,0x2998,0x2998},
{0x2999,Sm,0x2999,0x2999,0x2999},
{0x299A,Sm,0x299A,0x299A,0x299A},
{0x299B,Sm,0x299B,0x299B,0x299B},
{0x299C,Sm,0x299C,0x299C,0x299C},
{0x299D,Sm,0x299D,0x299D,0x299D},
{0x299E,Sm,0x299E,0x299E,0x299E},
{0x299F,Sm,0x299F,0x299F,0x299F},
{0x29A0,Sm,0x29A0,0x29A0,0x29A0},
{0x29A1,Sm,0x29A1,0x29A1,0x29A1},
{0x29A2,Sm,0x29A2,0x29A2,0x29A2},
{0x29A3,Sm,0x29A3,0x29A3,0x29A3},
{0x29A4,Sm,0x29A4,0x29A4,0x29A4},
{0x29A5,Sm,0x29A5,0x29A5,0x29A5},
{0x29A6,Sm,0x29A6,0x29A6,0x29A6},
{0x29A7,Sm,0x29A7,0x29A7,0x29A7},
{0x29A8,Sm,0x29A8,0x29A8,0x29A8},
{0x29A9,Sm,0x29A9,0x29A9,0x29A9},
{0x29AA,Sm,0x29AA,0x29AA,0x29AA},
{0x29AB,Sm,0x29AB,0x29AB,0x29AB},
{0x29AC,Sm,0x29AC,0x29AC,0x29AC},
{0x29AD,Sm,0x29AD,0x29AD,0x29AD},
{0x29AE,Sm,0x29AE,0x29AE,0x29AE},
{0x29AF,Sm,0x29AF,0x29AF,0x29AF},
{0x29B0,Sm,0x29B0,0x29B0,0x29B0},
{0x29B1,Sm,0x29B1,0x29B1,0x29B1},
{0x29B2,Sm,0x29B2,0x29B2,0x29B2},
{0x29B3,Sm,0x29B3,0x29B3,0x29B3},
{0x29B4,Sm,0x29B4,0x29B4,0x29B4},
{0x29B5,Sm,0x29B5,0x29B5,0x29B5},
{0x29B6,Sm,0x29B6,0x29B6,0x29B6},
{0x29B7,Sm,0x29B7,0x29B7,0x29B7},
{0x29B8,Sm,0x29B8,0x29B8,0x29B8},
{0x29B9,Sm,0x29B9,0x29B9,0x29B9},
{0x29BA,Sm,0x29BA,0x29BA,0x29BA},
{0x29BB,Sm,0x29BB,0x29BB,0x29BB},
{0x29BC,Sm,0x29BC,0x29BC,0x29BC},
{0x29BD,Sm,0x29BD,0x29BD,0x29BD},
{0x29BE,Sm,0x29BE,0x29BE,0x29BE},
{0x29BF,Sm,0x29BF,0x29BF,0x29BF},
{0x29C0,Sm,0x29C0,0x29C0,0x29C0},
{0x29C1,Sm,0x29C1,0x29C1,0x29C1},
{0x29C2,Sm,0x29C2,0x29C2,0x29C2},
{0x29C3,Sm,0x29C3,0x29C3,0x29C3},
{0x29C4,Sm,0x29C4,0x29C4,0x29C4},
{0x29C5,Sm,0x29C5,0x29C5,0x29C5},
{0x29C6,Sm,0x29C6,0x29C6,0x29C6},
{0x29C7,Sm,0x29C7,0x29C7,0x29C7},
{0x29C8,Sm,0x29C8,0x29C8,0x29C8},
{0x29C9,Sm,0x29C9,0x29C9,0x29C9},
{0x29CA,Sm,0x29CA,0x29CA,0x29CA},
{0x29CB,Sm,0x29CB,0x29CB,0x29CB},
{0x29CC,Sm,0x29CC,0x29CC,0x29CC},
{0x29CD,Sm,0x29CD,0x29CD,0x29CD},
{0x29CE,Sm,0x29CE,0x29CE,0x29CE},
{0x29CF,Sm,0x29CF,0x29CF,0x29CF},
{0x29D0,Sm,0x29D0,0x29D0,0x29D0},
{0x29D1,Sm,0x29D1,0x29D1,0x29D1},
{0x29D2,Sm,0x29D2,0x29D2,0x29D2},
{0x29D3,Sm,0x29D3,0x29D3,0x29D3},
{0x29D4,Sm,0x29D4,0x29D4,0x29D4},
{0x29D5,Sm,0x29D5,0x29D5,0x29D5},
{0x29D6,Sm,0x29D6,0x29D6,0x29D6},
{0x29D7,Sm,0x29D7,0x29D7,0x29D7},
{0x29D8,Ps,0x29D8,0x29D8,0x29D8},
{0x29D9,Pe,0x29D9,0x29D9,0x29D9},
{0x29DA,Ps,0x29DA,0x29DA,0x29DA},
{0x29DB,Pe,0x29DB,0x29DB,0x29DB},
{0x29DC,Sm,0x29DC,0x29DC,0x29DC},
{0x29DD,Sm,0x29DD,0x29DD,0x29DD},
{0x29DE,Sm,0x29DE,0x29DE,0x29DE},
{0x29DF,Sm,0x29DF,0x29DF,0x29DF},
{0x29E0,Sm,0x29E0,0x29E0,0x29E0},
{0x29E1,Sm,0x29E1,0x29E1,0x29E1},
{0x29E2,Sm,0x29E2,0x29E2,0x29E2},
{0x29E3,Sm,0x29E3,0x29E3,0x29E3},
{0x29E4,Sm,0x29E4,0x29E4,0x29E4},
{0x29E5,Sm,0x29E5,0x29E5,0x29E5},
{0x29E6,Sm,0x29E6,0x29E6,0x29E6},
{0x29E7,Sm,0x29E7,0x29E7,0x29E7},
{0x29E8,Sm,0x29E8,0x29E8,0x29E8},
{0x29E9,Sm,0x29E9,0x29E9,0x29E9},
{0x29EA,Sm,0x29EA,0x29EA,0x29EA},
{0x29EB,Sm,0x29EB,0x29EB,0x29EB},
{0x29EC,Sm,0x29EC,0x29EC,0x29EC},
{0x29ED,Sm,0x29ED,0x29ED,0x29ED},
{0x29EE,Sm,0x29EE,0x29EE,0x29EE},
{0x29EF,Sm,0x29EF,0x29EF,0x29EF},
{0x29F0,Sm,0x29F0,0x29F0,0x29F0},
{0x29F1,Sm,0x29F1,0x29F1,0x29F1},
{0x29F2,Sm,0x29F2,0x29F2,0x29F2},
{0x29F3,Sm,0x29F3,0x29F3,0x29F3},
{0x29F4,Sm,0x29F4,0x29F4,0x29F4},
{0x29F5,Sm,0x29F5,0x29F5,0x29F5},
{0x29F6,Sm,0x29F6,0x29F6,0x29F6},
{0x29F7,Sm,0x29F7,0x29F7,0x29F7},
{0x29F8,Sm,0x29F8,0x29F8,0x29F8},
{0x29F9,Sm,0x29F9,0x29F9,0x29F9},
{0x29FA,Sm,0x29FA,0x29FA,0x29FA},
{0x29FB,Sm,0x29FB,0x29FB,0x29FB},
{0x29FC,Ps,0x29FC,0x29FC,0x29FC},
{0x29FD,Pe,0x29FD,0x29FD,0x29FD},
{0x29FE,Sm,0x29FE,0x29FE,0x29FE},
{0x29FF,Sm,0x29FF,0x29FF,0x29FF},
{0x2A00,Sm,0x2A00,0x2A00,0x2A00},
{0x2A01,Sm,0x2A01,0x2A01,0x2A01},
{0x2A02,Sm,0x2A02,0x2A02,0x2A02},
{0x2A03,Sm,0x2A03,0x2A03,0x2A03},
{0x2A04,Sm,0x2A04,0x2A04,0x2A04},
{0x2A05,Sm,0x2A05,0x2A05,0x2A05},
{0x2A06,Sm,0x2A06,0x2A06,0x2A06},
{0x2A07,Sm,0x2A07,0x2A07,0x2A07},
{0x2A08,Sm,0x2A08,0x2A08,0x2A08},
{0x2A09,Sm,0x2A09,0x2A09,0x2A09},
{0x2A0A,Sm,0x2A0A,0x2A0A,0x2A0A},
{0x2A0B,Sm,0x2A0B,0x2A0B,0x2A0B},
{0x2A0C,Sm,0x2A0C,0x2A0C,0x2A0C},
{0x2A0D,Sm,0x2A0D,0x2A0D,0x2A0D},
{0x2A0E,Sm,0x2A0E,0x2A0E,0x2A0E},
{0x2A0F,Sm,0x2A0F,0x2A0F,0x2A0F},
{0x2A10,Sm,0x2A10,0x2A10,0x2A10},
{0x2A11,Sm,0x2A11,0x2A11,0x2A11},
{0x2A12,Sm,0x2A12,0x2A12,0x2A12},
{0x2A13,Sm,0x2A13,0x2A13,0x2A13},
{0x2A14,Sm,0x2A14,0x2A14,0x2A14},
{0x2A15,Sm,0x2A15,0x2A15,0x2A15},
{0x2A16,Sm,0x2A16,0x2A16,0x2A16},
{0x2A17,Sm,0x2A17,0x2A17,0x2A17},
{0x2A18,Sm,0x2A18,0x2A18,0x2A18},
{0x2A19,Sm,0x2A19,0x2A19,0x2A19},
{0x2A1A,Sm,0x2A1A,0x2A1A,0x2A1A},
{0x2A1B,Sm,0x2A1B,0x2A1B,0x2A1B},
{0x2A1C,Sm,0x2A1C,0x2A1C,0x2A1C},
{0x2A1D,Sm,0x2A1D,0x2A1D,0x2A1D},
{0x2A1E,Sm,0x2A1E,0x2A1E,0x2A1E},
{0x2A1F,Sm,0x2A1F,0x2A1F,0x2A1F},
{0x2A20,Sm,0x2A20,0x2A20,0x2A20},
{0x2A21,Sm,0x2A21,0x2A21,0x2A21},
{0x2A22,Sm,0x2A22,0x2A22,0x2A22},
{0x2A23,Sm,0x2A23,0x2A23,0x2A23},
{0x2A24,Sm,0x2A24,0x2A24,0x2A24},
{0x2A25,Sm,0x2A25,0x2A25,0x2A25},
{0x2A26,Sm,0x2A26,0x2A26,0x2A26},
{0x2A27,Sm,0x2A27,0x2A27,0x2A27},
{0x2A28,Sm,0x2A28,0x2A28,0x2A28},
{0x2A29,Sm,0x2A29,0x2A29,0x2A29},
{0x2A2A,Sm,0x2A2A,0x2A2A,0x2A2A},
{0x2A2B,Sm,0x2A2B,0x2A2B,0x2A2B},
{0x2A2C,Sm,0x2A2C,0x2A2C,0x2A2C},
{0x2A2D,Sm,0x2A2D,0x2A2D,0x2A2D},
{0x2A2E,Sm,0x2A2E,0x2A2E,0x2A2E},
{0x2A2F,Sm,0x2A2F,0x2A2F,0x2A2F},
{0x2A30,Sm,0x2A30,0x2A30,0x2A30},
{0x2A31,Sm,0x2A31,0x2A31,0x2A31},
{0x2A32,Sm,0x2A32,0x2A32,0x2A32},
{0x2A33,Sm,0x2A33,0x2A33,0x2A33},
{0x2A34,Sm,0x2A34,0x2A34,0x2A34},
{0x2A35,Sm,0x2A35,0x2A35,0x2A35},
{0x2A36,Sm,0x2A36,0x2A36,0x2A36},
{0x2A37,Sm,0x2A37,0x2A37,0x2A37},
{0x2A38,Sm,0x2A38,0x2A38,0x2A38},
{0x2A39,Sm,0x2A39,0x2A39,0x2A39},
{0x2A3A,Sm,0x2A3A,0x2A3A,0x2A3A},
{0x2A3B,Sm,0x2A3B,0x2A3B,0x2A3B},
{0x2A3C,Sm,0x2A3C,0x2A3C,0x2A3C},
{0x2A3D,Sm,0x2A3D,0x2A3D,0x2A3D},
{0x2A3E,Sm,0x2A3E,0x2A3E,0x2A3E},
{0x2A3F,Sm,0x2A3F,0x2A3F,0x2A3F},
{0x2A40,Sm,0x2A40,0x2A40,0x2A40},
{0x2A41,Sm,0x2A41,0x2A41,0x2A41},
{0x2A42,Sm,0x2A42,0x2A42,0x2A42},
{0x2A43,Sm,0x2A43,0x2A43,0x2A43},
{0x2A44,Sm,0x2A44,0x2A44,0x2A44},
{0x2A45,Sm,0x2A45,0x2A45,0x2A45},
{0x2A46,Sm,0x2A46,0x2A46,0x2A46},
{0x2A47,Sm,0x2A47,0x2A47,0x2A47},
{0x2A48,Sm,0x2A48,0x2A48,0x2A48},
{0x2A49,Sm,0x2A49,0x2A49,0x2A49},
{0x2A4A,Sm,0x2A4A,0x2A4A,0x2A4A},
{0x2A4B,Sm,0x2A4B,0x2A4B,0x2A4B},
{0x2A4C,Sm,0x2A4C,0x2A4C,0x2A4C},
{0x2A4D,Sm,0x2A4D,0x2A4D,0x2A4D},
{0x2A4E,Sm,0x2A4E,0x2A4E,0x2A4E},
{0x2A4F,Sm,0x2A4F,0x2A4F,0x2A4F},
{0x2A50,Sm,0x2A50,0x2A50,0x2A50},
{0x2A51,Sm,0x2A51,0x2A51,0x2A51},
{0x2A52,Sm,0x2A52,0x2A52,0x2A52},
{0x2A53,Sm,0x2A53,0x2A53,0x2A53},
{0x2A54,Sm,0x2A54,0x2A54,0x2A54},
{0x2A55,Sm,0x2A55,0x2A55,0x2A55},
{0x2A56,Sm,0x2A56,0x2A56,0x2A56},
{0x2A57,Sm,0x2A57,0x2A57,0x2A57},
{0x2A58,Sm,0x2A58,0x2A58,0x2A58},
{0x2A59,Sm,0x2A59,0x2A59,0x2A59},
{0x2A5A,Sm,0x2A5A,0x2A5A,0x2A5A},
{0x2A5B,Sm,0x2A5B,0x2A5B,0x2A5B},
{0x2A5C,Sm,0x2A5C,0x2A5C,0x2A5C},
{0x2A5D,Sm,0x2A5D,0x2A5D,0x2A5D},
{0x2A5E,Sm,0x2A5E,0x2A5E,0x2A5E},
{0x2A5F,Sm,0x2A5F,0x2A5F,0x2A5F},
{0x2A60,Sm,0x2A60,0x2A60,0x2A60},
{0x2A61,Sm,0x2A61,0x2A61,0x2A61},
{0x2A62,Sm,0x2A62,0x2A62,0x2A62},
{0x2A63,Sm,0x2A63,0x2A63,0x2A63},
{0x2A64,Sm,0x2A64,0x2A64,0x2A64},
{0x2A65,Sm,0x2A65,0x2A65,0x2A65},
{0x2A66,Sm,0x2A66,0x2A66,0x2A66},
{0x2A67,Sm,0x2A67,0x2A67,0x2A67},
{0x2A68,Sm,0x2A68,0x2A68,0x2A68},
{0x2A69,Sm,0x2A69,0x2A69,0x2A69},
{0x2A6A,Sm,0x2A6A,0x2A6A,0x2A6A},
{0x2A6B,Sm,0x2A6B,0x2A6B,0x2A6B},
{0x2A6C,Sm,0x2A6C,0x2A6C,0x2A6C},
{0x2A6D,Sm,0x2A6D,0x2A6D,0x2A6D},
{0x2A6E,Sm,0x2A6E,0x2A6E,0x2A6E},
{0x2A6F,Sm,0x2A6F,0x2A6F,0x2A6F},
{0x2A70,Sm,0x2A70,0x2A70,0x2A70},
{0x2A71,Sm,0x2A71,0x2A71,0x2A71},
{0x2A72,Sm,0x2A72,0x2A72,0x2A72},
{0x2A73,Sm,0x2A73,0x2A73,0x2A73},
{0x2A74,Sm,0x2A74,0x2A74,0x2A74},
{0x2A75,Sm,0x2A75,0x2A75,0x2A75},
{0x2A76,Sm,0x2A76,0x2A76,0x2A76},
{0x2A77,Sm,0x2A77,0x2A77,0x2A77},
{0x2A78,Sm,0x2A78,0x2A78,0x2A78},
{0x2A79,Sm,0x2A79,0x2A79,0x2A79},
{0x2A7A,Sm,0x2A7A,0x2A7A,0x2A7A},
{0x2A7B,Sm,0x2A7B,0x2A7B,0x2A7B},
{0x2A7C,Sm,0x2A7C,0x2A7C,0x2A7C},
{0x2A7D,Sm,0x2A7D,0x2A7D,0x2A7D},
{0x2A7E,Sm,0x2A7E,0x2A7E,0x2A7E},
{0x2A7F,Sm,0x2A7F,0x2A7F,0x2A7F},
{0x2A80,Sm,0x2A80,0x2A80,0x2A80},
{0x2A81,Sm,0x2A81,0x2A81,0x2A81},
{0x2A82,Sm,0x2A82,0x2A82,0x2A82},
{0x2A83,Sm,0x2A83,0x2A83,0x2A83},
{0x2A84,Sm,0x2A84,0x2A84,0x2A84},
{0x2A85,Sm,0x2A85,0x2A85,0x2A85},
{0x2A86,Sm,0x2A86,0x2A86,0x2A86},
{0x2A87,Sm,0x2A87,0x2A87,0x2A87},
{0x2A88,Sm,0x2A88,0x2A88,0x2A88},
{0x2A89,Sm,0x2A89,0x2A89,0x2A89},
{0x2A8A,Sm,0x2A8A,0x2A8A,0x2A8A},
{0x2A8B,Sm,0x2A8B,0x2A8B,0x2A8B},
{0x2A8C,Sm,0x2A8C,0x2A8C,0x2A8C},
{0x2A8D,Sm,0x2A8D,0x2A8D,0x2A8D},
{0x2A8E,Sm,0x2A8E,0x2A8E,0x2A8E},
{0x2A8F,Sm,0x2A8F,0x2A8F,0x2A8F},
{0x2A90,Sm,0x2A90,0x2A90,0x2A90},
{0x2A91,Sm,0x2A91,0x2A91,0x2A91},
{0x2A92,Sm,0x2A92,0x2A92,0x2A92},
{0x2A93,Sm,0x2A93,0x2A93,0x2A93},
{0x2A94,Sm,0x2A94,0x2A94,0x2A94},
{0x2A95,Sm,0x2A95,0x2A95,0x2A95},
{0x2A96,Sm,0x2A96,0x2A96,0x2A96},
{0x2A97,Sm,0x2A97,0x2A97,0x2A97},
{0x2A98,Sm,0x2A98,0x2A98,0x2A98},
{0x2A99,Sm,0x2A99,0x2A99,0x2A99},
{0x2A9A,Sm,0x2A9A,0x2A9A,0x2A9A},
{0x2A9B,Sm,0x2A9B,0x2A9B,0x2A9B},
{0x2A9C,Sm,0x2A9C,0x2A9C,0x2A9C},
{0x2A9D,Sm,0x2A9D,0x2A9D,0x2A9D},
{0x2A9E,Sm,0x2A9E,0x2A9E,0x2A9E},
{0x2A9F,Sm,0x2A9F,0x2A9F,0x2A9F},
{0x2AA0,Sm,0x2AA0,0x2AA0,0x2AA0},
{0x2AA1,Sm,0x2AA1,0x2AA1,0x2AA1},
{0x2AA2,Sm,0x2AA2,0x2AA2,0x2AA2},
{0x2AA3,Sm,0x2AA3,0x2AA3,0x2AA3},
{0x2AA4,Sm,0x2AA4,0x2AA4,0x2AA4},
{0x2AA5,Sm,0x2AA5,0x2AA5,0x2AA5},
{0x2AA6,Sm,0x2AA6,0x2AA6,0x2AA6},
{0x2AA7,Sm,0x2AA7,0x2AA7,0x2AA7},
{0x2AA8,Sm,0x2AA8,0x2AA8,0x2AA8},
{0x2AA9,Sm,0x2AA9,0x2AA9,0x2AA9},
{0x2AAA,Sm,0x2AAA,0x2AAA,0x2AAA},
{0x2AAB,Sm,0x2AAB,0x2AAB,0x2AAB},
{0x2AAC,Sm,0x2AAC,0x2AAC,0x2AAC},
{0x2AAD,Sm,0x2AAD,0x2AAD,0x2AAD},
{0x2AAE,Sm,0x2AAE,0x2AAE,0x2AAE},
{0x2AAF,Sm,0x2AAF,0x2AAF,0x2AAF},
{0x2AB0,Sm,0x2AB0,0x2AB0,0x2AB0},
{0x2AB1,Sm,0x2AB1,0x2AB1,0x2AB1},
{0x2AB2,Sm,0x2AB2,0x2AB2,0x2AB2},
{0x2AB3,Sm,0x2AB3,0x2AB3,0x2AB3},
{0x2AB4,Sm,0x2AB4,0x2AB4,0x2AB4},
{0x2AB5,Sm,0x2AB5,0x2AB5,0x2AB5},
{0x2AB6,Sm,0x2AB6,0x2AB6,0x2AB6},
{0x2AB7,Sm,0x2AB7,0x2AB7,0x2AB7},
{0x2AB8,Sm,0x2AB8,0x2AB8,0x2AB8},
{0x2AB9,Sm,0x2AB9,0x2AB9,0x2AB9},
{0x2ABA,Sm,0x2ABA,0x2ABA,0x2ABA},
{0x2ABB,Sm,0x2ABB,0x2ABB,0x2ABB},
{0x2ABC,Sm,0x2ABC,0x2ABC,0x2ABC},
{0x2ABD,Sm,0x2ABD,0x2ABD,0x2ABD},
{0x2ABE,Sm,0x2ABE,0x2ABE,0x2ABE},
{0x2ABF,Sm,0x2ABF,0x2ABF,0x2ABF},
{0x2AC0,Sm,0x2AC0,0x2AC0,0x2AC0},
{0x2AC1,Sm,0x2AC1,0x2AC1,0x2AC1},
{0x2AC2,Sm,0x2AC2,0x2AC2,0x2AC2},
{0x2AC3,Sm,0x2AC3,0x2AC3,0x2AC3},
{0x2AC4,Sm,0x2AC4,0x2AC4,0x2AC4},
{0x2AC5,Sm,0x2AC5,0x2AC5,0x2AC5},
{0x2AC6,Sm,0x2AC6,0x2AC6,0x2AC6},
{0x2AC7,Sm,0x2AC7,0x2AC7,0x2AC7},
{0x2AC8,Sm,0x2AC8,0x2AC8,0x2AC8},
{0x2AC9,Sm,0x2AC9,0x2AC9,0x2AC9},
{0x2ACA,Sm,0x2ACA,0x2ACA,0x2ACA},
{0x2ACB,Sm,0x2ACB,0x2ACB,0x2ACB},
{0x2ACC,Sm,0x2ACC,0x2ACC,0x2ACC},
{0x2ACD,Sm,0x2ACD,0x2ACD,0x2ACD},
{0x2ACE,Sm,0x2ACE,0x2ACE,0x2ACE},
{0x2ACF,Sm,0x2ACF,0x2ACF,0x2ACF},
{0x2AD0,Sm,0x2AD0,0x2AD0,0x2AD0},
{0x2AD1,Sm,0x2AD1,0x2AD1,0x2AD1},
{0x2AD2,Sm,0x2AD2,0x2AD2,0x2AD2},
{0x2AD3,Sm,0x2AD3,0x2AD3,0x2AD3},
{0x2AD4,Sm,0x2AD4,0x2AD4,0x2AD4},
{0x2AD5,Sm,0x2AD5,0x2AD5,0x2AD5},
{0x2AD6,Sm,0x2AD6,0x2AD6,0x2AD6},
{0x2AD7,Sm,0x2AD7,0x2AD7,0x2AD7},
{0x2AD8,Sm,0x2AD8,0x2AD8,0x2AD8},
{0x2AD9,Sm,0x2AD9,0x2AD9,0x2AD9},
{0x2ADA,Sm,0x2ADA,0x2ADA,0x2ADA},
{0x2ADB,Sm,0x2ADB,0x2ADB,0x2ADB},
{0x2ADC,Sm,0x2ADC,0x2ADC,0x2ADC},
{0x2ADD,Sm,0x2ADD,0x2ADD,0x2ADD},
{0x2ADE,Sm,0x2ADE,0x2ADE,0x2ADE},
{0x2ADF,Sm,0x2ADF,0x2ADF,0x2ADF},
{0x2AE0,Sm,0x2AE0,0x2AE0,0x2AE0},
{0x2AE1,Sm,0x2AE1,0x2AE1,0x2AE1},
{0x2AE2,Sm,0x2AE2,0x2AE2,0x2AE2},
{0x2AE3,Sm,0x2AE3,0x2AE3,0x2AE3},
{0x2AE4,Sm,0x2AE4,0x2AE4,0x2AE4},
{0x2AE5,Sm,0x2AE5,0x2AE5,0x2AE5},
{0x2AE6,Sm,0x2AE6,0x2AE6,0x2AE6},
{0x2AE7,Sm,0x2AE7,0x2AE7,0x2AE7},
{0x2AE8,Sm,0x2AE8,0x2AE8,0x2AE8},
{0x2AE9,Sm,0x2AE9,0x2AE9,0x2AE9},
{0x2AEA,Sm,0x2AEA,0x2AEA,0x2AEA},
{0x2AEB,Sm,0x2AEB,0x2AEB,0x2AEB},
{0x2AEC,Sm,0x2AEC,0x2AEC,0x2AEC},
{0x2AED,Sm,0x2AED,0x2AED,0x2AED},
{0x2AEE,Sm,0x2AEE,0x2AEE,0x2AEE},
{0x2AEF,Sm,0x2AEF,0x2AEF,0x2AEF},
{0x2AF0,Sm,0x2AF0,0x2AF0,0x2AF0},
{0x2AF1,Sm,0x2AF1,0x2AF1,0x2AF1},
{0x2AF2,Sm,0x2AF2,0x2AF2,0x2AF2},
{0x2AF3,Sm,0x2AF3,0x2AF3,0x2AF3},
{0x2AF4,Sm,0x2AF4,0x2AF4,0x2AF4},
{0x2AF5,Sm,0x2AF5,0x2AF5,0x2AF5},
{0x2AF6,Sm,0x2AF6,0x2AF6,0x2AF6},
{0x2AF7,Sm,0x2AF7,0x2AF7,0x2AF7},
{0x2AF8,Sm,0x2AF8,0x2AF8,0x2AF8},
{0x2AF9,Sm,0x2AF9,0x2AF9,0x2AF9},
{0x2AFA,Sm,0x2AFA,0x2AFA,0x2AFA},
{0x2AFB,Sm,0x2AFB,0x2AFB,0x2AFB},
{0x2AFC,Sm,0x2AFC,0x2AFC,0x2AFC},
{0x2AFD,Sm,0x2AFD,0x2AFD,0x2AFD},
{0x2AFE,Sm,0x2AFE,0x2AFE,0x2AFE},
{0x2AFF,Sm,0x2AFF,0x2AFF,0x2AFF},
{0x2B00,So,0x2B00,0x2B00,0x2B00},
{0x2B01,So,0x2B01,0x2B01,0x2B01},
{0x2B02,So,0x2B02,0x2B02,0x2B02},
{0x2B03,So,0x2B03,0x2B03,0x2B03},
{0x2B04,So,0x2B04,0x2B04,0x2B04},
{0x2B05,So,0x2B05,0x2B05,0x2B05},
{0x2B06,So,0x2B06,0x2B06,0x2B06},
{0x2B07,So,0x2B07,0x2B07,0x2B07},
{0x2B08,So,0x2B08,0x2B08,0x2B08},
{0x2B09,So,0x2B09,0x2B09,0x2B09},
{0x2B0A,So,0x2B0A,0x2B0A,0x2B0A},
{0x2B0B,So,0x2B0B,0x2B0B,0x2B0B},
{0x2B0C,So,0x2B0C,0x2B0C,0x2B0C},
{0x2B0D,So,0x2B0D,0x2B0D,0x2B0D},
{0x2B0E,So,0x2B0E,0x2B0E,0x2B0E},
{0x2B0F,So,0x2B0F,0x2B0F,0x2B0F},
{0x2B10,So,0x2B10,0x2B10,0x2B10},
{0x2B11,So,0x2B11,0x2B11,0x2B11},
{0x2B12,So,0x2B12,0x2B12,0x2B12},
{0x2B13,So,0x2B13,0x2B13,0x2B13},
{0x2B14,So,0x2B14,0x2B14,0x2B14},
{0x2B15,So,0x2B15,0x2B15,0x2B15},
{0x2B16,So,0x2B16,0x2B16,0x2B16},
{0x2B17,So,0x2B17,0x2B17,0x2B17},
{0x2B18,So,0x2B18,0x2B18,0x2B18},
{0x2B19,So,0x2B19,0x2B19,0x2B19},
{0x2B1A,So,0x2B1A,0x2B1A,0x2B1A},
{0x2B20,So,0x2B20,0x2B20,0x2B20},
{0x2B21,So,0x2B21,0x2B21,0x2B21},
{0x2B22,So,0x2B22,0x2B22,0x2B22},
{0x2B23,So,0x2B23,0x2B23,0x2B23},
{0x2C00,Lu,0x2C00,0x2C30,0x2C00},
{0x2C01,Lu,0x2C01,0x2C31,0x2C01},
{0x2C02,Lu,0x2C02,0x2C32,0x2C02},
{0x2C03,Lu,0x2C03,0x2C33,0x2C03},
{0x2C04,Lu,0x2C04,0x2C34,0x2C04},
{0x2C05,Lu,0x2C05,0x2C35,0x2C05},
{0x2C06,Lu,0x2C06,0x2C36,0x2C06},
{0x2C07,Lu,0x2C07,0x2C37,0x2C07},
{0x2C08,Lu,0x2C08,0x2C38,0x2C08},
{0x2C09,Lu,0x2C09,0x2C39,0x2C09},
{0x2C0A,Lu,0x2C0A,0x2C3A,0x2C0A},
{0x2C0B,Lu,0x2C0B,0x2C3B,0x2C0B},
{0x2C0C,Lu,0x2C0C,0x2C3C,0x2C0C},
{0x2C0D,Lu,0x2C0D,0x2C3D,0x2C0D},
{0x2C0E,Lu,0x2C0E,0x2C3E,0x2C0E},
{0x2C0F,Lu,0x2C0F,0x2C3F,0x2C0F},
{0x2C10,Lu,0x2C10,0x2C40,0x2C10},
{0x2C11,Lu,0x2C11,0x2C41,0x2C11},
{0x2C12,Lu,0x2C12,0x2C42,0x2C12},
{0x2C13,Lu,0x2C13,0x2C43,0x2C13},
{0x2C14,Lu,0x2C14,0x2C44,0x2C14},
{0x2C15,Lu,0x2C15,0x2C45,0x2C15},
{0x2C16,Lu,0x2C16,0x2C46,0x2C16},
{0x2C17,Lu,0x2C17,0x2C47,0x2C17},
{0x2C18,Lu,0x2C18,0x2C48,0x2C18},
{0x2C19,Lu,0x2C19,0x2C49,0x2C19},
{0x2C1A,Lu,0x2C1A,0x2C4A,0x2C1A},
{0x2C1B,Lu,0x2C1B,0x2C4B,0x2C1B},
{0x2C1C,Lu,0x2C1C,0x2C4C,0x2C1C},
{0x2C1D,Lu,0x2C1D,0x2C4D,0x2C1D},
{0x2C1E,Lu,0x2C1E,0x2C4E,0x2C1E},
{0x2C1F,Lu,0x2C1F,0x2C4F,0x2C1F},
{0x2C20,Lu,0x2C20,0x2C50,0x2C20},
{0x2C21,Lu,0x2C21,0x2C51,0x2C21},
{0x2C22,Lu,0x2C22,0x2C52,0x2C22},
{0x2C23,Lu,0x2C23,0x2C53,0x2C23},
{0x2C24,Lu,0x2C24,0x2C54,0x2C24},
{0x2C25,Lu,0x2C25,0x2C55,0x2C25},
{0x2C26,Lu,0x2C26,0x2C56,0x2C26},
{0x2C27,Lu,0x2C27,0x2C57,0x2C27},
{0x2C28,Lu,0x2C28,0x2C58,0x2C28},
{0x2C29,Lu,0x2C29,0x2C59,0x2C29},
{0x2C2A,Lu,0x2C2A,0x2C5A,0x2C2A},
{0x2C2B,Lu,0x2C2B,0x2C5B,0x2C2B},
{0x2C2C,Lu,0x2C2C,0x2C5C,0x2C2C},
{0x2C2D,Lu,0x2C2D,0x2C5D,0x2C2D},
{0x2C2E,Lu,0x2C2E,0x2C5E,0x2C2E},
{0x2C30,Ll,0x2C00,0x2C30,0x2C00},
{0x2C31,Ll,0x2C01,0x2C31,0x2C01},
{0x2C32,Ll,0x2C02,0x2C32,0x2C02},
{0x2C33,Ll,0x2C03,0x2C33,0x2C03},
{0x2C34,Ll,0x2C04,0x2C34,0x2C04},
{0x2C35,Ll,0x2C05,0x2C35,0x2C05},
{0x2C36,Ll,0x2C06,0x2C36,0x2C06},
{0x2C37,Ll,0x2C07,0x2C37,0x2C07},
{0x2C38,Ll,0x2C08,0x2C38,0x2C08},
{0x2C39,Ll,0x2C09,0x2C39,0x2C09},
{0x2C3A,Ll,0x2C0A,0x2C3A,0x2C0A},
{0x2C3B,Ll,0x2C0B,0x2C3B,0x2C0B},
{0x2C3C,Ll,0x2C0C,0x2C3C,0x2C0C},
{0x2C3D,Ll,0x2C0D,0x2C3D,0x2C0D},
{0x2C3E,Ll,0x2C0E,0x2C3E,0x2C0E},
{0x2C3F,Ll,0x2C0F,0x2C3F,0x2C0F},
{0x2C40,Ll,0x2C10,0x2C40,0x2C10},
{0x2C41,Ll,0x2C11,0x2C41,0x2C11},
{0x2C42,Ll,0x2C12,0x2C42,0x2C12},
{0x2C43,Ll,0x2C13,0x2C43,0x2C13},
{0x2C44,Ll,0x2C14,0x2C44,0x2C14},
{0x2C45,Ll,0x2C15,0x2C45,0x2C15},
{0x2C46,Ll,0x2C16,0x2C46,0x2C16},
{0x2C47,Ll,0x2C17,0x2C47,0x2C17},
{0x2C48,Ll,0x2C18,0x2C48,0x2C18},
{0x2C49,Ll,0x2C19,0x2C49,0x2C19},
{0x2C4A,Ll,0x2C1A,0x2C4A,0x2C1A},
{0x2C4B,Ll,0x2C1B,0x2C4B,0x2C1B},
{0x2C4C,Ll,0x2C1C,0x2C4C,0x2C1C},
{0x2C4D,Ll,0x2C1D,0x2C4D,0x2C1D},
{0x2C4E,Ll,0x2C1E,0x2C4E,0x2C1E},
{0x2C4F,Ll,0x2C1F,0x2C4F,0x2C1F},
{0x2C50,Ll,0x2C20,0x2C50,0x2C20},
{0x2C51,Ll,0x2C21,0x2C51,0x2C21},
{0x2C52,Ll,0x2C22,0x2C52,0x2C22},
{0x2C53,Ll,0x2C23,0x2C53,0x2C23},
{0x2C54,Ll,0x2C24,0x2C54,0x2C24},
{0x2C55,Ll,0x2C25,0x2C55,0x2C25},
{0x2C56,Ll,0x2C26,0x2C56,0x2C26},
{0x2C57,Ll,0x2C27,0x2C57,0x2C27},
{0x2C58,Ll,0x2C28,0x2C58,0x2C28},
{0x2C59,Ll,0x2C29,0x2C59,0x2C29},
{0x2C5A,Ll,0x2C2A,0x2C5A,0x2C2A},
{0x2C5B,Ll,0x2C2B,0x2C5B,0x2C2B},
{0x2C5C,Ll,0x2C2C,0x2C5C,0x2C2C},
{0x2C5D,Ll,0x2C2D,0x2C5D,0x2C2D},
{0x2C5E,Ll,0x2C2E,0x2C5E,0x2C2E},
{0x2C60,Lu,0x2C60,0x2C61,0x2C60},
{0x2C61,Ll,0x2C60,0x2C61,0x2C60},
{0x2C62,Lu,0x2C62,0x26B,0x2C62},
{0x2C63,Lu,0x2C63,0x1D7D,0x2C63},
{0x2C64,Lu,0x2C64,0x27D,0x2C64},
{0x2C65,Ll,0x23A,0x2C65,0x23A},
{0x2C66,Ll,0x23E,0x2C66,0x23E},
{0x2C67,Lu,0x2C67,0x2C68,0x2C67},
{0x2C68,Ll,0x2C67,0x2C68,0x2C67},
{0x2C69,Lu,0x2C69,0x2C6A,0x2C69},
{0x2C6A,Ll,0x2C69,0x2C6A,0x2C69},
{0x2C6B,Lu,0x2C6B,0x2C6C,0x2C6B},
{0x2C6C,Ll,0x2C6B,0x2C6C,0x2C6B},
{0x2C74,Ll,0x2C74,0x2C74,0x2C74},
{0x2C75,Lu,0x2C75,0x2C76,0x2C75},
{0x2C76,Ll,0x2C75,0x2C76,0x2C75},
{0x2C77,Ll,0x2C77,0x2C77,0x2C77},
{0x2C80,Lu,0x2C80,0x2C81,0x2C80},
{0x2C81,Ll,0x2C80,0x2C81,0x2C80},
{0x2C82,Lu,0x2C82,0x2C83,0x2C82},
{0x2C83,Ll,0x2C82,0x2C83,0x2C82},
{0x2C84,Lu,0x2C84,0x2C85,0x2C84},
{0x2C85,Ll,0x2C84,0x2C85,0x2C84},
{0x2C86,Lu,0x2C86,0x2C87,0x2C86},
{0x2C87,Ll,0x2C86,0x2C87,0x2C86},
{0x2C88,Lu,0x2C88,0x2C89,0x2C88},
{0x2C89,Ll,0x2C88,0x2C89,0x2C88},
{0x2C8A,Lu,0x2C8A,0x2C8B,0x2C8A},
{0x2C8B,Ll,0x2C8A,0x2C8B,0x2C8A},
{0x2C8C,Lu,0x2C8C,0x2C8D,0x2C8C},
{0x2C8D,Ll,0x2C8C,0x2C8D,0x2C8C},
{0x2C8E,Lu,0x2C8E,0x2C8F,0x2C8E},
{0x2C8F,Ll,0x2C8E,0x2C8F,0x2C8E},
{0x2C90,Lu,0x2C90,0x2C91,0x2C90},
{0x2C91,Ll,0x2C90,0x2C91,0x2C90},
{0x2C92,Lu,0x2C92,0x2C93,0x2C92},
{0x2C93,Ll,0x2C92,0x2C93,0x2C92},
{0x2C94,Lu,0x2C94,0x2C95,0x2C94},
{0x2C95,Ll,0x2C94,0x2C95,0x2C94},
{0x2C96,Lu,0x2C96,0x2C97,0x2C96},
{0x2C97,Ll,0x2C96,0x2C97,0x2C96},
{0x2C98,Lu,0x2C98,0x2C99,0x2C98},
{0x2C99,Ll,0x2C98,0x2C99,0x2C98},
{0x2C9A,Lu,0x2C9A,0x2C9B,0x2C9A},
{0x2C9B,Ll,0x2C9A,0x2C9B,0x2C9A},
{0x2C9C,Lu,0x2C9C,0x2C9D,0x2C9C},
{0x2C9D,Ll,0x2C9C,0x2C9D,0x2C9C},
{0x2C9E,Lu,0x2C9E,0x2C9F,0x2C9E},
{0x2C9F,Ll,0x2C9E,0x2C9F,0x2C9E},
{0x2CA0,Lu,0x2CA0,0x2CA1,0x2CA0},
{0x2CA1,Ll,0x2CA0,0x2CA1,0x2CA0},
{0x2CA2,Lu,0x2CA2,0x2CA3,0x2CA2},
{0x2CA3,Ll,0x2CA2,0x2CA3,0x2CA2},
{0x2CA4,Lu,0x2CA4,0x2CA5,0x2CA4},
{0x2CA5,Ll,0x2CA4,0x2CA5,0x2CA4},
{0x2CA6,Lu,0x2CA6,0x2CA7,0x2CA6},
{0x2CA7,Ll,0x2CA6,0x2CA7,0x2CA6},
{0x2CA8,Lu,0x2CA8,0x2CA9,0x2CA8},
{0x2CA9,Ll,0x2CA8,0x2CA9,0x2CA8},
{0x2CAA,Lu,0x2CAA,0x2CAB,0x2CAA},
{0x2CAB,Ll,0x2CAA,0x2CAB,0x2CAA},
{0x2CAC,Lu,0x2CAC,0x2CAD,0x2CAC},
{0x2CAD,Ll,0x2CAC,0x2CAD,0x2CAC},
{0x2CAE,Lu,0x2CAE,0x2CAF,0x2CAE},
{0x2CAF,Ll,0x2CAE,0x2CAF,0x2CAE},
{0x2CB0,Lu,0x2CB0,0x2CB1,0x2CB0},
{0x2CB1,Ll,0x2CB0,0x2CB1,0x2CB0},
{0x2CB2,Lu,0x2CB2,0x2CB3,0x2CB2},
{0x2CB3,Ll,0x2CB2,0x2CB3,0x2CB2},
{0x2CB4,Lu,0x2CB4,0x2CB5,0x2CB4},
{0x2CB5,Ll,0x2CB4,0x2CB5,0x2CB4},
{0x2CB6,Lu,0x2CB6,0x2CB7,0x2CB6},
{0x2CB7,Ll,0x2CB6,0x2CB7,0x2CB6},
{0x2CB8,Lu,0x2CB8,0x2CB9,0x2CB8},
{0x2CB9,Ll,0x2CB8,0x2CB9,0x2CB8},
{0x2CBA,Lu,0x2CBA,0x2CBB,0x2CBA},
{0x2CBB,Ll,0x2CBA,0x2CBB,0x2CBA},
{0x2CBC,Lu,0x2CBC,0x2CBD,0x2CBC},
{0x2CBD,Ll,0x2CBC,0x2CBD,0x2CBC},
{0x2CBE,Lu,0x2CBE,0x2CBF,0x2CBE},
{0x2CBF,Ll,0x2CBE,0x2CBF,0x2CBE},
{0x2CC0,Lu,0x2CC0,0x2CC1,0x2CC0},
{0x2CC1,Ll,0x2CC0,0x2CC1,0x2CC0},
{0x2CC2,Lu,0x2CC2,0x2CC3,0x2CC2},
{0x2CC3,Ll,0x2CC2,0x2CC3,0x2CC2},
{0x2CC4,Lu,0x2CC4,0x2CC5,0x2CC4},
{0x2CC5,Ll,0x2CC4,0x2CC5,0x2CC4},
{0x2CC6,Lu,0x2CC6,0x2CC7,0x2CC6},
{0x2CC7,Ll,0x2CC6,0x2CC7,0x2CC6},
{0x2CC8,Lu,0x2CC8,0x2CC9,0x2CC8},
{0x2CC9,Ll,0x2CC8,0x2CC9,0x2CC8},
{0x2CCA,Lu,0x2CCA,0x2CCB,0x2CCA},
{0x2CCB,Ll,0x2CCA,0x2CCB,0x2CCA},
{0x2CCC,Lu,0x2CCC,0x2CCD,0x2CCC},
{0x2CCD,Ll,0x2CCC,0x2CCD,0x2CCC},
{0x2CCE,Lu,0x2CCE,0x2CCF,0x2CCE},
{0x2CCF,Ll,0x2CCE,0x2CCF,0x2CCE},
{0x2CD0,Lu,0x2CD0,0x2CD1,0x2CD0},
{0x2CD1,Ll,0x2CD0,0x2CD1,0x2CD0},
{0x2CD2,Lu,0x2CD2,0x2CD3,0x2CD2},
{0x2CD3,Ll,0x2CD2,0x2CD3,0x2CD2},
{0x2CD4,Lu,0x2CD4,0x2CD5,0x2CD4},
{0x2CD5,Ll,0x2CD4,0x2CD5,0x2CD4},
{0x2CD6,Lu,0x2CD6,0x2CD7,0x2CD6},
{0x2CD7,Ll,0x2CD6,0x2CD7,0x2CD6},
{0x2CD8,Lu,0x2CD8,0x2CD9,0x2CD8},
{0x2CD9,Ll,0x2CD8,0x2CD9,0x2CD8},
{0x2CDA,Lu,0x2CDA,0x2CDB,0x2CDA},
{0x2CDB,Ll,0x2CDA,0x2CDB,0x2CDA},
{0x2CDC,Lu,0x2CDC,0x2CDD,0x2CDC},
{0x2CDD,Ll,0x2CDC,0x2CDD,0x2CDC},
{0x2CDE,Lu,0x2CDE,0x2CDF,0x2CDE},
{0x2CDF,Ll,0x2CDE,0x2CDF,0x2CDE},
{0x2CE0,Lu,0x2CE0,0x2CE1,0x2CE0},
{0x2CE1,Ll,0x2CE0,0x2CE1,0x2CE0},
{0x2CE2,Lu,0x2CE2,0x2CE3,0x2CE2},
{0x2CE3,Ll,0x2CE2,0x2CE3,0x2CE2},
{0x2CE4,Ll,0x2CE4,0x2CE4,0x2CE4},
{0x2CE5,So,0x2CE5,0x2CE5,0x2CE5},
{0x2CE6,So,0x2CE6,0x2CE6,0x2CE6},
{0x2CE7,So,0x2CE7,0x2CE7,0x2CE7},
{0x2CE8,So,0x2CE8,0x2CE8,0x2CE8},
{0x2CE9,So,0x2CE9,0x2CE9,0x2CE9},
{0x2CEA,So,0x2CEA,0x2CEA,0x2CEA},
{0x2CF9,Po,0x2CF9,0x2CF9,0x2CF9},
{0x2CFA,Po,0x2CFA,0x2CFA,0x2CFA},
{0x2CFB,Po,0x2CFB,0x2CFB,0x2CFB},
{0x2CFC,Po,0x2CFC,0x2CFC,0x2CFC},
{0x2CFD,No,0x2CFD,0x2CFD,0x2CFD},
{0x2CFE,Po,0x2CFE,0x2CFE,0x2CFE},
{0x2CFF,Po,0x2CFF,0x2CFF,0x2CFF},
{0x2D00,Ll,0x10A0,0x2D00,0x10A0},
{0x2D01,Ll,0x10A1,0x2D01,0x10A1},
{0x2D02,Ll,0x10A2,0x2D02,0x10A2},
{0x2D03,Ll,0x10A3,0x2D03,0x10A3},
{0x2D04,Ll,0x10A4,0x2D04,0x10A4},
{0x2D05,Ll,0x10A5,0x2D05,0x10A5},
{0x2D06,Ll,0x10A6,0x2D06,0x10A6},
{0x2D07,Ll,0x10A7,0x2D07,0x10A7},
{0x2D08,Ll,0x10A8,0x2D08,0x10A8},
{0x2D09,Ll,0x10A9,0x2D09,0x10A9},
{0x2D0A,Ll,0x10AA,0x2D0A,0x10AA},
{0x2D0B,Ll,0x10AB,0x2D0B,0x10AB},
{0x2D0C,Ll,0x10AC,0x2D0C,0x10AC},
{0x2D0D,Ll,0x10AD,0x2D0D,0x10AD},
{0x2D0E,Ll,0x10AE,0x2D0E,0x10AE},
{0x2D0F,Ll,0x10AF,0x2D0F,0x10AF},
{0x2D10,Ll,0x10B0,0x2D10,0x10B0},
{0x2D11,Ll,0x10B1,0x2D11,0x10B1},
{0x2D12,Ll,0x10B2,0x2D12,0x10B2},
{0x2D13,Ll,0x10B3,0x2D13,0x10B3},
{0x2D14,Ll,0x10B4,0x2D14,0x10B4},
{0x2D15,Ll,0x10B5,0x2D15,0x10B5},
{0x2D16,Ll,0x10B6,0x2D16,0x10B6},
{0x2D17,Ll,0x10B7,0x2D17,0x10B7},
{0x2D18,Ll,0x10B8,0x2D18,0x10B8},
{0x2D19,Ll,0x10B9,0x2D19,0x10B9},
{0x2D1A,Ll,0x10BA,0x2D1A,0x10BA},
{0x2D1B,Ll,0x10BB,0x2D1B,0x10BB},
{0x2D1C,Ll,0x10BC,0x2D1C,0x10BC},
{0x2D1D,Ll,0x10BD,0x2D1D,0x10BD},
{0x2D1E,Ll,0x10BE,0x2D1E,0x10BE},
{0x2D1F,Ll,0x10BF,0x2D1F,0x10BF},
{0x2D20,Ll,0x10C0,0x2D20,0x10C0},
{0x2D21,Ll,0x10C1,0x2D21,0x10C1},
{0x2D22,Ll,0x10C2,0x2D22,0x10C2},
{0x2D23,Ll,0x10C3,0x2D23,0x10C3},
{0x2D24,Ll,0x10C4,0x2D24,0x10C4},
{0x2D25,Ll,0x10C5,0x2D25,0x10C5},
{0x2D30,Lo,0x2D30,0x2D30,0x2D30},
{0x2D31,Lo,0x2D31,0x2D31,0x2D31},
{0x2D32,Lo,0x2D32,0x2D32,0x2D32},
{0x2D33,Lo,0x2D33,0x2D33,0x2D33},
{0x2D34,Lo,0x2D34,0x2D34,0x2D34},
{0x2D35,Lo,0x2D35,0x2D35,0x2D35},
{0x2D36,Lo,0x2D36,0x2D36,0x2D36},
{0x2D37,Lo,0x2D37,0x2D37,0x2D37},
{0x2D38,Lo,0x2D38,0x2D38,0x2D38},
{0x2D39,Lo,0x2D39,0x2D39,0x2D39},
{0x2D3A,Lo,0x2D3A,0x2D3A,0x2D3A},
{0x2D3B,Lo,0x2D3B,0x2D3B,0x2D3B},
{0x2D3C,Lo,0x2D3C,0x2D3C,0x2D3C},
{0x2D3D,Lo,0x2D3D,0x2D3D,0x2D3D},
{0x2D3E,Lo,0x2D3E,0x2D3E,0x2D3E},
{0x2D3F,Lo,0x2D3F,0x2D3F,0x2D3F},
{0x2D40,Lo,0x2D40,0x2D40,0x2D40},
{0x2D41,Lo,0x2D41,0x2D41,0x2D41},
{0x2D42,Lo,0x2D42,0x2D42,0x2D42},
{0x2D43,Lo,0x2D43,0x2D43,0x2D43},
{0x2D44,Lo,0x2D44,0x2D44,0x2D44},
{0x2D45,Lo,0x2D45,0x2D45,0x2D45},
{0x2D46,Lo,0x2D46,0x2D46,0x2D46},
{0x2D47,Lo,0x2D47,0x2D47,0x2D47},
{0x2D48,Lo,0x2D48,0x2D48,0x2D48},
{0x2D49,Lo,0x2D49,0x2D49,0x2D49},
{0x2D4A,Lo,0x2D4A,0x2D4A,0x2D4A},
{0x2D4B,Lo,0x2D4B,0x2D4B,0x2D4B},
{0x2D4C,Lo,0x2D4C,0x2D4C,0x2D4C},
{0x2D4D,Lo,0x2D4D,0x2D4D,0x2D4D},
{0x2D4E,Lo,0x2D4E,0x2D4E,0x2D4E},
{0x2D4F,Lo,0x2D4F,0x2D4F,0x2D4F},
{0x2D50,Lo,0x2D50,0x2D50,0x2D50},
{0x2D51,Lo,0x2D51,0x2D51,0x2D51},
{0x2D52,Lo,0x2D52,0x2D52,0x2D52},
{0x2D53,Lo,0x2D53,0x2D53,0x2D53},
{0x2D54,Lo,0x2D54,0x2D54,0x2D54},
{0x2D55,Lo,0x2D55,0x2D55,0x2D55},
{0x2D56,Lo,0x2D56,0x2D56,0x2D56},
{0x2D57,Lo,0x2D57,0x2D57,0x2D57},
{0x2D58,Lo,0x2D58,0x2D58,0x2D58},
{0x2D59,Lo,0x2D59,0x2D59,0x2D59},
{0x2D5A,Lo,0x2D5A,0x2D5A,0x2D5A},
{0x2D5B,Lo,0x2D5B,0x2D5B,0x2D5B},
{0x2D5C,Lo,0x2D5C,0x2D5C,0x2D5C},
{0x2D5D,Lo,0x2D5D,0x2D5D,0x2D5D},
{0x2D5E,Lo,0x2D5E,0x2D5E,0x2D5E},
{0x2D5F,Lo,0x2D5F,0x2D5F,0x2D5F},
{0x2D60,Lo,0x2D60,0x2D60,0x2D60},
{0x2D61,Lo,0x2D61,0x2D61,0x2D61},
{0x2D62,Lo,0x2D62,0x2D62,0x2D62},
{0x2D63,Lo,0x2D63,0x2D63,0x2D63},
{0x2D64,Lo,0x2D64,0x2D64,0x2D64},
{0x2D65,Lo,0x2D65,0x2D65,0x2D65},
{0x2D6F,Lm,0x2D6F,0x2D6F,0x2D6F},
{0x2D80,Lo,0x2D80,0x2D80,0x2D80},
{0x2D81,Lo,0x2D81,0x2D81,0x2D81},
{0x2D82,Lo,0x2D82,0x2D82,0x2D82},
{0x2D83,Lo,0x2D83,0x2D83,0x2D83},
{0x2D84,Lo,0x2D84,0x2D84,0x2D84},
{0x2D85,Lo,0x2D85,0x2D85,0x2D85},
{0x2D86,Lo,0x2D86,0x2D86,0x2D86},
{0x2D87,Lo,0x2D87,0x2D87,0x2D87},
{0x2D88,Lo,0x2D88,0x2D88,0x2D88},
{0x2D89,Lo,0x2D89,0x2D89,0x2D89},
{0x2D8A,Lo,0x2D8A,0x2D8A,0x2D8A},
{0x2D8B,Lo,0x2D8B,0x2D8B,0x2D8B},
{0x2D8C,Lo,0x2D8C,0x2D8C,0x2D8C},
{0x2D8D,Lo,0x2D8D,0x2D8D,0x2D8D},
{0x2D8E,Lo,0x2D8E,0x2D8E,0x2D8E},
{0x2D8F,Lo,0x2D8F,0x2D8F,0x2D8F},
{0x2D90,Lo,0x2D90,0x2D90,0x2D90},
{0x2D91,Lo,0x2D91,0x2D91,0x2D91},
{0x2D92,Lo,0x2D92,0x2D92,0x2D92},
{0x2D93,Lo,0x2D93,0x2D93,0x2D93},
{0x2D94,Lo,0x2D94,0x2D94,0x2D94},
{0x2D95,Lo,0x2D95,0x2D95,0x2D95},
{0x2D96,Lo,0x2D96,0x2D96,0x2D96},
{0x2DA0,Lo,0x2DA0,0x2DA0,0x2DA0},
{0x2DA1,Lo,0x2DA1,0x2DA1,0x2DA1},
{0x2DA2,Lo,0x2DA2,0x2DA2,0x2DA2},
{0x2DA3,Lo,0x2DA3,0x2DA3,0x2DA3},
{0x2DA4,Lo,0x2DA4,0x2DA4,0x2DA4},
{0x2DA5,Lo,0x2DA5,0x2DA5,0x2DA5},
{0x2DA6,Lo,0x2DA6,0x2DA6,0x2DA6},
{0x2DA8,Lo,0x2DA8,0x2DA8,0x2DA8},
{0x2DA9,Lo,0x2DA9,0x2DA9,0x2DA9},
{0x2DAA,Lo,0x2DAA,0x2DAA,0x2DAA},
{0x2DAB,Lo,0x2DAB,0x2DAB,0x2DAB},
{0x2DAC,Lo,0x2DAC,0x2DAC,0x2DAC},
{0x2DAD,Lo,0x2DAD,0x2DAD,0x2DAD},
{0x2DAE,Lo,0x2DAE,0x2DAE,0x2DAE},
{0x2DB0,Lo,0x2DB0,0x2DB0,0x2DB0},
{0x2DB1,Lo,0x2DB1,0x2DB1,0x2DB1},
{0x2DB2,Lo,0x2DB2,0x2DB2,0x2DB2},
{0x2DB3,Lo,0x2DB3,0x2DB3,0x2DB3},
{0x2DB4,Lo,0x2DB4,0x2DB4,0x2DB4},
{0x2DB5,Lo,0x2DB5,0x2DB5,0x2DB5},
{0x2DB6,Lo,0x2DB6,0x2DB6,0x2DB6},
{0x2DB8,Lo,0x2DB8,0x2DB8,0x2DB8},
{0x2DB9,Lo,0x2DB9,0x2DB9,0x2DB9},
{0x2DBA,Lo,0x2DBA,0x2DBA,0x2DBA},
{0x2DBB,Lo,0x2DBB,0x2DBB,0x2DBB},
{0x2DBC,Lo,0x2DBC,0x2DBC,0x2DBC},
{0x2DBD,Lo,0x2DBD,0x2DBD,0x2DBD},
{0x2DBE,Lo,0x2DBE,0x2DBE,0x2DBE},
{0x2DC0,Lo,0x2DC0,0x2DC0,0x2DC0},
{0x2DC1,Lo,0x2DC1,0x2DC1,0x2DC1},
{0x2DC2,Lo,0x2DC2,0x2DC2,0x2DC2},
{0x2DC3,Lo,0x2DC3,0x2DC3,0x2DC3},
{0x2DC4,Lo,0x2DC4,0x2DC4,0x2DC4},
{0x2DC5,Lo,0x2DC5,0x2DC5,0x2DC5},
{0x2DC6,Lo,0x2DC6,0x2DC6,0x2DC6},
{0x2DC8,Lo,0x2DC8,0x2DC8,0x2DC8},
{0x2DC9,Lo,0x2DC9,0x2DC9,0x2DC9},
{0x2DCA,Lo,0x2DCA,0x2DCA,0x2DCA},
{0x2DCB,Lo,0x2DCB,0x2DCB,0x2DCB},
{0x2DCC,Lo,0x2DCC,0x2DCC,0x2DCC},
{0x2DCD,Lo,0x2DCD,0x2DCD,0x2DCD},
{0x2DCE,Lo,0x2DCE,0x2DCE,0x2DCE},
{0x2DD0,Lo,0x2DD0,0x2DD0,0x2DD0},
{0x2DD1,Lo,0x2DD1,0x2DD1,0x2DD1},
{0x2DD2,Lo,0x2DD2,0x2DD2,0x2DD2},
{0x2DD3,Lo,0x2DD3,0x2DD3,0x2DD3},
{0x2DD4,Lo,0x2DD4,0x2DD4,0x2DD4},
{0x2DD5,Lo,0x2DD5,0x2DD5,0x2DD5},
{0x2DD6,Lo,0x2DD6,0x2DD6,0x2DD6},
{0x2DD8,Lo,0x2DD8,0x2DD8,0x2DD8},
{0x2DD9,Lo,0x2DD9,0x2DD9,0x2DD9},
{0x2DDA,Lo,0x2DDA,0x2DDA,0x2DDA},
{0x2DDB,Lo,0x2DDB,0x2DDB,0x2DDB},
{0x2DDC,Lo,0x2DDC,0x2DDC,0x2DDC},
{0x2DDD,Lo,0x2DDD,0x2DDD,0x2DDD},
{0x2DDE,Lo,0x2DDE,0x2DDE,0x2DDE},
{0x2E00,Po,0x2E00,0x2E00,0x2E00},
{0x2E01,Po,0x2E01,0x2E01,0x2E01},
{0x2E02,Pi,0x2E02,0x2E02,0x2E02},
{0x2E03,Pf,0x2E03,0x2E03,0x2E03},
{0x2E04,Pi,0x2E04,0x2E04,0x2E04},
{0x2E05,Pf,0x2E05,0x2E05,0x2E05},
{0x2E06,Po,0x2E06,0x2E06,0x2E06},
{0x2E07,Po,0x2E07,0x2E07,0x2E07},
{0x2E08,Po,0x2E08,0x2E08,0x2E08},
{0x2E09,Pi,0x2E09,0x2E09,0x2E09},
{0x2E0A,Pf,0x2E0A,0x2E0A,0x2E0A},
{0x2E0B,Po,0x2E0B,0x2E0B,0x2E0B},
{0x2E0C,Pi,0x2E0C,0x2E0C,0x2E0C},
{0x2E0D,Pf,0x2E0D,0x2E0D,0x2E0D},
{0x2E0E,Po,0x2E0E,0x2E0E,0x2E0E},
{0x2E0F,Po,0x2E0F,0x2E0F,0x2E0F},
{0x2E10,Po,0x2E10,0x2E10,0x2E10},
{0x2E11,Po,0x2E11,0x2E11,0x2E11},
{0x2E12,Po,0x2E12,0x2E12,0x2E12},
{0x2E13,Po,0x2E13,0x2E13,0x2E13},
{0x2E14,Po,0x2E14,0x2E14,0x2E14},
{0x2E15,Po,0x2E15,0x2E15,0x2E15},
{0x2E16,Po,0x2E16,0x2E16,0x2E16},
{0x2E17,Pd,0x2E17,0x2E17,0x2E17},
{0x2E1C,Pi,0x2E1C,0x2E1C,0x2E1C},
{0x2E1D,Pf,0x2E1D,0x2E1D,0x2E1D},
{0x2E80,So,0x2E80,0x2E80,0x2E80},
{0x2E81,So,0x2E81,0x2E81,0x2E81},
{0x2E82,So,0x2E82,0x2E82,0x2E82},
{0x2E83,So,0x2E83,0x2E83,0x2E83},
{0x2E84,So,0x2E84,0x2E84,0x2E84},
{0x2E85,So,0x2E85,0x2E85,0x2E85},
{0x2E86,So,0x2E86,0x2E86,0x2E86},
{0x2E87,So,0x2E87,0x2E87,0x2E87},
{0x2E88,So,0x2E88,0x2E88,0x2E88},
{0x2E89,So,0x2E89,0x2E89,0x2E89},
{0x2E8A,So,0x2E8A,0x2E8A,0x2E8A},
{0x2E8B,So,0x2E8B,0x2E8B,0x2E8B},
{0x2E8C,So,0x2E8C,0x2E8C,0x2E8C},
{0x2E8D,So,0x2E8D,0x2E8D,0x2E8D},
{0x2E8E,So,0x2E8E,0x2E8E,0x2E8E},
{0x2E8F,So,0x2E8F,0x2E8F,0x2E8F},
{0x2E90,So,0x2E90,0x2E90,0x2E90},
{0x2E91,So,0x2E91,0x2E91,0x2E91},
{0x2E92,So,0x2E92,0x2E92,0x2E92},
{0x2E93,So,0x2E93,0x2E93,0x2E93},
{0x2E94,So,0x2E94,0x2E94,0x2E94},
{0x2E95,So,0x2E95,0x2E95,0x2E95},
{0x2E96,So,0x2E96,0x2E96,0x2E96},
{0x2E97,So,0x2E97,0x2E97,0x2E97},
{0x2E98,So,0x2E98,0x2E98,0x2E98},
{0x2E99,So,0x2E99,0x2E99,0x2E99},
{0x2E9B,So,0x2E9B,0x2E9B,0x2E9B},
{0x2E9C,So,0x2E9C,0x2E9C,0x2E9C},
{0x2E9D,So,0x2E9D,0x2E9D,0x2E9D},
{0x2E9E,So,0x2E9E,0x2E9E,0x2E9E},
{0x2E9F,So,0x2E9F,0x2E9F,0x2E9F},
{0x2EA0,So,0x2EA0,0x2EA0,0x2EA0},
{0x2EA1,So,0x2EA1,0x2EA1,0x2EA1},
{0x2EA2,So,0x2EA2,0x2EA2,0x2EA2},
{0x2EA3,So,0x2EA3,0x2EA3,0x2EA3},
{0x2EA4,So,0x2EA4,0x2EA4,0x2EA4},
{0x2EA5,So,0x2EA5,0x2EA5,0x2EA5},
{0x2EA6,So,0x2EA6,0x2EA6,0x2EA6},
{0x2EA7,So,0x2EA7,0x2EA7,0x2EA7},
{0x2EA8,So,0x2EA8,0x2EA8,0x2EA8},
{0x2EA9,So,0x2EA9,0x2EA9,0x2EA9},
{0x2EAA,So,0x2EAA,0x2EAA,0x2EAA},
{0x2EAB,So,0x2EAB,0x2EAB,0x2EAB},
{0x2EAC,So,0x2EAC,0x2EAC,0x2EAC},
{0x2EAD,So,0x2EAD,0x2EAD,0x2EAD},
{0x2EAE,So,0x2EAE,0x2EAE,0x2EAE},
{0x2EAF,So,0x2EAF,0x2EAF,0x2EAF},
{0x2EB0,So,0x2EB0,0x2EB0,0x2EB0},
{0x2EB1,So,0x2EB1,0x2EB1,0x2EB1},
{0x2EB2,So,0x2EB2,0x2EB2,0x2EB2},
{0x2EB3,So,0x2EB3,0x2EB3,0x2EB3},
{0x2EB4,So,0x2EB4,0x2EB4,0x2EB4},
{0x2EB5,So,0x2EB5,0x2EB5,0x2EB5},
{0x2EB6,So,0x2EB6,0x2EB6,0x2EB6},
{0x2EB7,So,0x2EB7,0x2EB7,0x2EB7},
{0x2EB8,So,0x2EB8,0x2EB8,0x2EB8},
{0x2EB9,So,0x2EB9,0x2EB9,0x2EB9},
{0x2EBA,So,0x2EBA,0x2EBA,0x2EBA},
{0x2EBB,So,0x2EBB,0x2EBB,0x2EBB},
{0x2EBC,So,0x2EBC,0x2EBC,0x2EBC},
{0x2EBD,So,0x2EBD,0x2EBD,0x2EBD},
{0x2EBE,So,0x2EBE,0x2EBE,0x2EBE},
{0x2EBF,So,0x2EBF,0x2EBF,0x2EBF},
{0x2EC0,So,0x2EC0,0x2EC0,0x2EC0},
{0x2EC1,So,0x2EC1,0x2EC1,0x2EC1},
{0x2EC2,So,0x2EC2,0x2EC2,0x2EC2},
{0x2EC3,So,0x2EC3,0x2EC3,0x2EC3},
{0x2EC4,So,0x2EC4,0x2EC4,0x2EC4},
{0x2EC5,So,0x2EC5,0x2EC5,0x2EC5},
{0x2EC6,So,0x2EC6,0x2EC6,0x2EC6},
{0x2EC7,So,0x2EC7,0x2EC7,0x2EC7},
{0x2EC8,So,0x2EC8,0x2EC8,0x2EC8},
{0x2EC9,So,0x2EC9,0x2EC9,0x2EC9},
{0x2ECA,So,0x2ECA,0x2ECA,0x2ECA},
{0x2ECB,So,0x2ECB,0x2ECB,0x2ECB},
{0x2ECC,So,0x2ECC,0x2ECC,0x2ECC},
{0x2ECD,So,0x2ECD,0x2ECD,0x2ECD},
{0x2ECE,So,0x2ECE,0x2ECE,0x2ECE},
{0x2ECF,So,0x2ECF,0x2ECF,0x2ECF},
{0x2ED0,So,0x2ED0,0x2ED0,0x2ED0},
{0x2ED1,So,0x2ED1,0x2ED1,0x2ED1},
{0x2ED2,So,0x2ED2,0x2ED2,0x2ED2},
{0x2ED3,So,0x2ED3,0x2ED3,0x2ED3},
{0x2ED4,So,0x2ED4,0x2ED4,0x2ED4},
{0x2ED5,So,0x2ED5,0x2ED5,0x2ED5},
{0x2ED6,So,0x2ED6,0x2ED6,0x2ED6},
{0x2ED7,So,0x2ED7,0x2ED7,0x2ED7},
{0x2ED8,So,0x2ED8,0x2ED8,0x2ED8},
{0x2ED9,So,0x2ED9,0x2ED9,0x2ED9},
{0x2EDA,So,0x2EDA,0x2EDA,0x2EDA},
{0x2EDB,So,0x2EDB,0x2EDB,0x2EDB},
{0x2EDC,So,0x2EDC,0x2EDC,0x2EDC},
{0x2EDD,So,0x2EDD,0x2EDD,0x2EDD},
{0x2EDE,So,0x2EDE,0x2EDE,0x2EDE},
{0x2EDF,So,0x2EDF,0x2EDF,0x2EDF},
{0x2EE0,So,0x2EE0,0x2EE0,0x2EE0},
{0x2EE1,So,0x2EE1,0x2EE1,0x2EE1},
{0x2EE2,So,0x2EE2,0x2EE2,0x2EE2},
{0x2EE3,So,0x2EE3,0x2EE3,0x2EE3},
{0x2EE4,So,0x2EE4,0x2EE4,0x2EE4},
{0x2EE5,So,0x2EE5,0x2EE5,0x2EE5},
{0x2EE6,So,0x2EE6,0x2EE6,0x2EE6},
{0x2EE7,So,0x2EE7,0x2EE7,0x2EE7},
{0x2EE8,So,0x2EE8,0x2EE8,0x2EE8},
{0x2EE9,So,0x2EE9,0x2EE9,0x2EE9},
{0x2EEA,So,0x2EEA,0x2EEA,0x2EEA},
{0x2EEB,So,0x2EEB,0x2EEB,0x2EEB},
{0x2EEC,So,0x2EEC,0x2EEC,0x2EEC},
{0x2EED,So,0x2EED,0x2EED,0x2EED},
{0x2EEE,So,0x2EEE,0x2EEE,0x2EEE},
{0x2EEF,So,0x2EEF,0x2EEF,0x2EEF},
{0x2EF0,So,0x2EF0,0x2EF0,0x2EF0},
{0x2EF1,So,0x2EF1,0x2EF1,0x2EF1},
{0x2EF2,So,0x2EF2,0x2EF2,0x2EF2},
{0x2EF3,So,0x2EF3,0x2EF3,0x2EF3},
{0x2F00,So,0x2F00,0x2F00,0x2F00},
{0x2F01,So,0x2F01,0x2F01,0x2F01},
{0x2F02,So,0x2F02,0x2F02,0x2F02},
{0x2F03,So,0x2F03,0x2F03,0x2F03},
{0x2F04,So,0x2F04,0x2F04,0x2F04},
{0x2F05,So,0x2F05,0x2F05,0x2F05},
{0x2F06,So,0x2F06,0x2F06,0x2F06},
{0x2F07,So,0x2F07,0x2F07,0x2F07},
{0x2F08,So,0x2F08,0x2F08,0x2F08},
{0x2F09,So,0x2F09,0x2F09,0x2F09},
{0x2F0A,So,0x2F0A,0x2F0A,0x2F0A},
{0x2F0B,So,0x2F0B,0x2F0B,0x2F0B},
{0x2F0C,So,0x2F0C,0x2F0C,0x2F0C},
{0x2F0D,So,0x2F0D,0x2F0D,0x2F0D},
{0x2F0E,So,0x2F0E,0x2F0E,0x2F0E},
{0x2F0F,So,0x2F0F,0x2F0F,0x2F0F},
{0x2F10,So,0x2F10,0x2F10,0x2F10},
{0x2F11,So,0x2F11,0x2F11,0x2F11},
{0x2F12,So,0x2F12,0x2F12,0x2F12},
{0x2F13,So,0x2F13,0x2F13,0x2F13},
{0x2F14,So,0x2F14,0x2F14,0x2F14},
{0x2F15,So,0x2F15,0x2F15,0x2F15},
{0x2F16,So,0x2F16,0x2F16,0x2F16},
{0x2F17,So,0x2F17,0x2F17,0x2F17},
{0x2F18,So,0x2F18,0x2F18,0x2F18},
{0x2F19,So,0x2F19,0x2F19,0x2F19},
{0x2F1A,So,0x2F1A,0x2F1A,0x2F1A},
{0x2F1B,So,0x2F1B,0x2F1B,0x2F1B},
{0x2F1C,So,0x2F1C,0x2F1C,0x2F1C},
{0x2F1D,So,0x2F1D,0x2F1D,0x2F1D},
{0x2F1E,So,0x2F1E,0x2F1E,0x2F1E},
{0x2F1F,So,0x2F1F,0x2F1F,0x2F1F},
{0x2F20,So,0x2F20,0x2F20,0x2F20},
{0x2F21,So,0x2F21,0x2F21,0x2F21},
{0x2F22,So,0x2F22,0x2F22,0x2F22},
{0x2F23,So,0x2F23,0x2F23,0x2F23},
{0x2F24,So,0x2F24,0x2F24,0x2F24},
{0x2F25,So,0x2F25,0x2F25,0x2F25},
{0x2F26,So,0x2F26,0x2F26,0x2F26},
{0x2F27,So,0x2F27,0x2F27,0x2F27},
{0x2F28,So,0x2F28,0x2F28,0x2F28},
{0x2F29,So,0x2F29,0x2F29,0x2F29},
{0x2F2A,So,0x2F2A,0x2F2A,0x2F2A},
{0x2F2B,So,0x2F2B,0x2F2B,0x2F2B},
{0x2F2C,So,0x2F2C,0x2F2C,0x2F2C},
{0x2F2D,So,0x2F2D,0x2F2D,0x2F2D},
{0x2F2E,So,0x2F2E,0x2F2E,0x2F2E},
{0x2F2F,So,0x2F2F,0x2F2F,0x2F2F},
{0x2F30,So,0x2F30,0x2F30,0x2F30},
{0x2F31,So,0x2F31,0x2F31,0x2F31},
{0x2F32,So,0x2F32,0x2F32,0x2F32},
{0x2F33,So,0x2F33,0x2F33,0x2F33},
{0x2F34,So,0x2F34,0x2F34,0x2F34},
{0x2F35,So,0x2F35,0x2F35,0x2F35},
{0x2F36,So,0x2F36,0x2F36,0x2F36},
{0x2F37,So,0x2F37,0x2F37,0x2F37},
{0x2F38,So,0x2F38,0x2F38,0x2F38},
{0x2F39,So,0x2F39,0x2F39,0x2F39},
{0x2F3A,So,0x2F3A,0x2F3A,0x2F3A},
{0x2F3B,So,0x2F3B,0x2F3B,0x2F3B},
{0x2F3C,So,0x2F3C,0x2F3C,0x2F3C},
{0x2F3D,So,0x2F3D,0x2F3D,0x2F3D},
{0x2F3E,So,0x2F3E,0x2F3E,0x2F3E},
{0x2F3F,So,0x2F3F,0x2F3F,0x2F3F},
{0x2F40,So,0x2F40,0x2F40,0x2F40},
{0x2F41,So,0x2F41,0x2F41,0x2F41},
{0x2F42,So,0x2F42,0x2F42,0x2F42},
{0x2F43,So,0x2F43,0x2F43,0x2F43},
{0x2F44,So,0x2F44,0x2F44,0x2F44},
{0x2F45,So,0x2F45,0x2F45,0x2F45},
{0x2F46,So,0x2F46,0x2F46,0x2F46},
{0x2F47,So,0x2F47,0x2F47,0x2F47},
{0x2F48,So,0x2F48,0x2F48,0x2F48},
{0x2F49,So,0x2F49,0x2F49,0x2F49},
{0x2F4A,So,0x2F4A,0x2F4A,0x2F4A},
{0x2F4B,So,0x2F4B,0x2F4B,0x2F4B},
{0x2F4C,So,0x2F4C,0x2F4C,0x2F4C},
{0x2F4D,So,0x2F4D,0x2F4D,0x2F4D},
{0x2F4E,So,0x2F4E,0x2F4E,0x2F4E},
{0x2F4F,So,0x2F4F,0x2F4F,0x2F4F},
{0x2F50,So,0x2F50,0x2F50,0x2F50},
{0x2F51,So,0x2F51,0x2F51,0x2F51},
{0x2F52,So,0x2F52,0x2F52,0x2F52},
{0x2F53,So,0x2F53,0x2F53,0x2F53},
{0x2F54,So,0x2F54,0x2F54,0x2F54},
{0x2F55,So,0x2F55,0x2F55,0x2F55},
{0x2F56,So,0x2F56,0x2F56,0x2F56},
{0x2F57,So,0x2F57,0x2F57,0x2F57},
{0x2F58,So,0x2F58,0x2F58,0x2F58},
{0x2F59,So,0x2F59,0x2F59,0x2F59},
{0x2F5A,So,0x2F5A,0x2F5A,0x2F5A},
{0x2F5B,So,0x2F5B,0x2F5B,0x2F5B},
{0x2F5C,So,0x2F5C,0x2F5C,0x2F5C},
{0x2F5D,So,0x2F5D,0x2F5D,0x2F5D},
{0x2F5E,So,0x2F5E,0x2F5E,0x2F5E},
{0x2F5F,So,0x2F5F,0x2F5F,0x2F5F},
{0x2F60,So,0x2F60,0x2F60,0x2F60},
{0x2F61,So,0x2F61,0x2F61,0x2F61},
{0x2F62,So,0x2F62,0x2F62,0x2F62},
{0x2F63,So,0x2F63,0x2F63,0x2F63},
{0x2F64,So,0x2F64,0x2F64,0x2F64},
{0x2F65,So,0x2F65,0x2F65,0x2F65},
{0x2F66,So,0x2F66,0x2F66,0x2F66},
{0x2F67,So,0x2F67,0x2F67,0x2F67},
{0x2F68,So,0x2F68,0x2F68,0x2F68},
{0x2F69,So,0x2F69,0x2F69,0x2F69},
{0x2F6A,So,0x2F6A,0x2F6A,0x2F6A},
{0x2F6B,So,0x2F6B,0x2F6B,0x2F6B},
{0x2F6C,So,0x2F6C,0x2F6C,0x2F6C},
{0x2F6D,So,0x2F6D,0x2F6D,0x2F6D},
{0x2F6E,So,0x2F6E,0x2F6E,0x2F6E},
{0x2F6F,So,0x2F6F,0x2F6F,0x2F6F},
{0x2F70,So,0x2F70,0x2F70,0x2F70},
{0x2F71,So,0x2F71,0x2F71,0x2F71},
{0x2F72,So,0x2F72,0x2F72,0x2F72},
{0x2F73,So,0x2F73,0x2F73,0x2F73},
{0x2F74,So,0x2F74,0x2F74,0x2F74},
{0x2F75,So,0x2F75,0x2F75,0x2F75},
{0x2F76,So,0x2F76,0x2F76,0x2F76},
{0x2F77,So,0x2F77,0x2F77,0x2F77},
{0x2F78,So,0x2F78,0x2F78,0x2F78},
{0x2F79,So,0x2F79,0x2F79,0x2F79},
{0x2F7A,So,0x2F7A,0x2F7A,0x2F7A},
{0x2F7B,So,0x2F7B,0x2F7B,0x2F7B},
{0x2F7C,So,0x2F7C,0x2F7C,0x2F7C},
{0x2F7D,So,0x2F7D,0x2F7D,0x2F7D},
{0x2F7E,So,0x2F7E,0x2F7E,0x2F7E},
{0x2F7F,So,0x2F7F,0x2F7F,0x2F7F},
{0x2F80,So,0x2F80,0x2F80,0x2F80},
{0x2F81,So,0x2F81,0x2F81,0x2F81},
{0x2F82,So,0x2F82,0x2F82,0x2F82},
{0x2F83,So,0x2F83,0x2F83,0x2F83},
{0x2F84,So,0x2F84,0x2F84,0x2F84},
{0x2F85,So,0x2F85,0x2F85,0x2F85},
{0x2F86,So,0x2F86,0x2F86,0x2F86},
{0x2F87,So,0x2F87,0x2F87,0x2F87},
{0x2F88,So,0x2F88,0x2F88,0x2F88},
{0x2F89,So,0x2F89,0x2F89,0x2F89},
{0x2F8A,So,0x2F8A,0x2F8A,0x2F8A},
{0x2F8B,So,0x2F8B,0x2F8B,0x2F8B},
{0x2F8C,So,0x2F8C,0x2F8C,0x2F8C},
{0x2F8D,So,0x2F8D,0x2F8D,0x2F8D},
{0x2F8E,So,0x2F8E,0x2F8E,0x2F8E},
{0x2F8F,So,0x2F8F,0x2F8F,0x2F8F},
{0x2F90,So,0x2F90,0x2F90,0x2F90},
{0x2F91,So,0x2F91,0x2F91,0x2F91},
{0x2F92,So,0x2F92,0x2F92,0x2F92},
{0x2F93,So,0x2F93,0x2F93,0x2F93},
{0x2F94,So,0x2F94,0x2F94,0x2F94},
{0x2F95,So,0x2F95,0x2F95,0x2F95},
{0x2F96,So,0x2F96,0x2F96,0x2F96},
{0x2F97,So,0x2F97,0x2F97,0x2F97},
{0x2F98,So,0x2F98,0x2F98,0x2F98},
{0x2F99,So,0x2F99,0x2F99,0x2F99},
{0x2F9A,So,0x2F9A,0x2F9A,0x2F9A},
{0x2F9B,So,0x2F9B,0x2F9B,0x2F9B},
{0x2F9C,So,0x2F9C,0x2F9C,0x2F9C},
{0x2F9D,So,0x2F9D,0x2F9D,0x2F9D},
{0x2F9E,So,0x2F9E,0x2F9E,0x2F9E},
{0x2F9F,So,0x2F9F,0x2F9F,0x2F9F},
{0x2FA0,So,0x2FA0,0x2FA0,0x2FA0},
{0x2FA1,So,0x2FA1,0x2FA1,0x2FA1},
{0x2FA2,So,0x2FA2,0x2FA2,0x2FA2},
{0x2FA3,So,0x2FA3,0x2FA3,0x2FA3},
{0x2FA4,So,0x2FA4,0x2FA4,0x2FA4},
{0x2FA5,So,0x2FA5,0x2FA5,0x2FA5},
{0x2FA6,So,0x2FA6,0x2FA6,0x2FA6},
{0x2FA7,So,0x2FA7,0x2FA7,0x2FA7},
{0x2FA8,So,0x2FA8,0x2FA8,0x2FA8},
{0x2FA9,So,0x2FA9,0x2FA9,0x2FA9},
{0x2FAA,So,0x2FAA,0x2FAA,0x2FAA},
{0x2FAB,So,0x2FAB,0x2FAB,0x2FAB},
{0x2FAC,So,0x2FAC,0x2FAC,0x2FAC},
{0x2FAD,So,0x2FAD,0x2FAD,0x2FAD},
{0x2FAE,So,0x2FAE,0x2FAE,0x2FAE},
{0x2FAF,So,0x2FAF,0x2FAF,0x2FAF},
{0x2FB0,So,0x2FB0,0x2FB0,0x2FB0},
{0x2FB1,So,0x2FB1,0x2FB1,0x2FB1},
{0x2FB2,So,0x2FB2,0x2FB2,0x2FB2},
{0x2FB3,So,0x2FB3,0x2FB3,0x2FB3},
{0x2FB4,So,0x2FB4,0x2FB4,0x2FB4},
{0x2FB5,So,0x2FB5,0x2FB5,0x2FB5},
{0x2FB6,So,0x2FB6,0x2FB6,0x2FB6},
{0x2FB7,So,0x2FB7,0x2FB7,0x2FB7},
{0x2FB8,So,0x2FB8,0x2FB8,0x2FB8},
{0x2FB9,So,0x2FB9,0x2FB9,0x2FB9},
{0x2FBA,So,0x2FBA,0x2FBA,0x2FBA},
{0x2FBB,So,0x2FBB,0x2FBB,0x2FBB},
{0x2FBC,So,0x2FBC,0x2FBC,0x2FBC},
{0x2FBD,So,0x2FBD,0x2FBD,0x2FBD},
{0x2FBE,So,0x2FBE,0x2FBE,0x2FBE},
{0x2FBF,So,0x2FBF,0x2FBF,0x2FBF},
{0x2FC0,So,0x2FC0,0x2FC0,0x2FC0},
{0x2FC1,So,0x2FC1,0x2FC1,0x2FC1},
{0x2FC2,So,0x2FC2,0x2FC2,0x2FC2},
{0x2FC3,So,0x2FC3,0x2FC3,0x2FC3},
{0x2FC4,So,0x2FC4,0x2FC4,0x2FC4},
{0x2FC5,So,0x2FC5,0x2FC5,0x2FC5},
{0x2FC6,So,0x2FC6,0x2FC6,0x2FC6},
{0x2FC7,So,0x2FC7,0x2FC7,0x2FC7},
{0x2FC8,So,0x2FC8,0x2FC8,0x2FC8},
{0x2FC9,So,0x2FC9,0x2FC9,0x2FC9},
{0x2FCA,So,0x2FCA,0x2FCA,0x2FCA},
{0x2FCB,So,0x2FCB,0x2FCB,0x2FCB},
{0x2FCC,So,0x2FCC,0x2FCC,0x2FCC},
{0x2FCD,So,0x2FCD,0x2FCD,0x2FCD},
{0x2FCE,So,0x2FCE,0x2FCE,0x2FCE},
{0x2FCF,So,0x2FCF,0x2FCF,0x2FCF},
{0x2FD0,So,0x2FD0,0x2FD0,0x2FD0},
{0x2FD1,So,0x2FD1,0x2FD1,0x2FD1},
{0x2FD2,So,0x2FD2,0x2FD2,0x2FD2},
{0x2FD3,So,0x2FD3,0x2FD3,0x2FD3},
{0x2FD4,So,0x2FD4,0x2FD4,0x2FD4},
{0x2FD5,So,0x2FD5,0x2FD5,0x2FD5},
{0x2FF0,So,0x2FF0,0x2FF0,0x2FF0},
{0x2FF1,So,0x2FF1,0x2FF1,0x2FF1},
{0x2FF2,So,0x2FF2,0x2FF2,0x2FF2},
{0x2FF3,So,0x2FF3,0x2FF3,0x2FF3},
{0x2FF4,So,0x2FF4,0x2FF4,0x2FF4},
{0x2FF5,So,0x2FF5,0x2FF5,0x2FF5},
{0x2FF6,So,0x2FF6,0x2FF6,0x2FF6},
{0x2FF7,So,0x2FF7,0x2FF7,0x2FF7},
{0x2FF8,So,0x2FF8,0x2FF8,0x2FF8},
{0x2FF9,So,0x2FF9,0x2FF9,0x2FF9},
{0x2FFA,So,0x2FFA,0x2FFA,0x2FFA},
{0x2FFB,So,0x2FFB,0x2FFB,0x2FFB},
{0x3000,Zs,0x3000,0x3000,0x3000},
{0x3001,Po,0x3001,0x3001,0x3001},
{0x3002,Po,0x3002,0x3002,0x3002},
{0x3003,Po,0x3003,0x3003,0x3003},
{0x3004,So,0x3004,0x3004,0x3004},
{0x3005,Lm,0x3005,0x3005,0x3005},
{0x3006,Lo,0x3006,0x3006,0x3006},
{0x3007,Nl,0x3007,0x3007,0x3007},
{0x3008,Ps,0x3008,0x3008,0x3008},
{0x3009,Pe,0x3009,0x3009,0x3009},
{0x300A,Ps,0x300A,0x300A,0x300A},
{0x300B,Pe,0x300B,0x300B,0x300B},
{0x300C,Ps,0x300C,0x300C,0x300C},
{0x300D,Pe,0x300D,0x300D,0x300D},
{0x300E,Ps,0x300E,0x300E,0x300E},
{0x300F,Pe,0x300F,0x300F,0x300F},
{0x3010,Ps,0x3010,0x3010,0x3010},
{0x3011,Pe,0x3011,0x3011,0x3011},
{0x3012,So,0x3012,0x3012,0x3012},
{0x3013,So,0x3013,0x3013,0x3013},
{0x3014,Ps,0x3014,0x3014,0x3014},
{0x3015,Pe,0x3015,0x3015,0x3015},
{0x3016,Ps,0x3016,0x3016,0x3016},
{0x3017,Pe,0x3017,0x3017,0x3017},
{0x3018,Ps,0x3018,0x3018,0x3018},
{0x3019,Pe,0x3019,0x3019,0x3019},
{0x301A,Ps,0x301A,0x301A,0x301A},
{0x301B,Pe,0x301B,0x301B,0x301B},
{0x301C,Pd,0x301C,0x301C,0x301C},
{0x301D,Ps,0x301D,0x301D,0x301D},
{0x301E,Pe,0x301E,0x301E,0x301E},
{0x301F,Pe,0x301F,0x301F,0x301F},
{0x3020,So,0x3020,0x3020,0x3020},
{0x3021,Nl,0x3021,0x3021,0x3021},
{0x3022,Nl,0x3022,0x3022,0x3022},
{0x3023,Nl,0x3023,0x3023,0x3023},
{0x3024,Nl,0x3024,0x3024,0x3024},
{0x3025,Nl,0x3025,0x3025,0x3025},
{0x3026,Nl,0x3026,0x3026,0x3026},
{0x3027,Nl,0x3027,0x3027,0x3027},
{0x3028,Nl,0x3028,0x3028,0x3028},
{0x3029,Nl,0x3029,0x3029,0x3029},
{0x302A,Mn,0x302A,0x302A,0x302A},
{0x302B,Mn,0x302B,0x302B,0x302B},
{0x302C,Mn,0x302C,0x302C,0x302C},
{0x302D,Mn,0x302D,0x302D,0x302D},
{0x302E,Mn,0x302E,0x302E,0x302E},
{0x302F,Mn,0x302F,0x302F,0x302F},
{0x3030,Pd,0x3030,0x3030,0x3030},
{0x3031,Lm,0x3031,0x3031,0x3031},
{0x3032,Lm,0x3032,0x3032,0x3032},
{0x3033,Lm,0x3033,0x3033,0x3033},
{0x3034,Lm,0x3034,0x3034,0x3034},
{0x3035,Lm,0x3035,0x3035,0x3035},
{0x3036,So,0x3036,0x3036,0x3036},
{0x3037,So,0x3037,0x3037,0x3037},
{0x3038,Nl,0x3038,0x3038,0x3038},
{0x3039,Nl,0x3039,0x3039,0x3039},
{0x303A,Nl,0x303A,0x303A,0x303A},
{0x303B,Lm,0x303B,0x303B,0x303B},
{0x303C,Lo,0x303C,0x303C,0x303C},
{0x303D,Po,0x303D,0x303D,0x303D},
{0x303E,So,0x303E,0x303E,0x303E},
{0x303F,So,0x303F,0x303F,0x303F},
{0x3041,Lo,0x3041,0x3041,0x3041},
{0x3042,Lo,0x3042,0x3042,0x3042},
{0x3043,Lo,0x3043,0x3043,0x3043},
{0x3044,Lo,0x3044,0x3044,0x3044},
{0x3045,Lo,0x3045,0x3045,0x3045},
{0x3046,Lo,0x3046,0x3046,0x3046},
{0x3047,Lo,0x3047,0x3047,0x3047},
{0x3048,Lo,0x3048,0x3048,0x3048},
{0x3049,Lo,0x3049,0x3049,0x3049},
{0x304A,Lo,0x304A,0x304A,0x304A},
{0x304B,Lo,0x304B,0x304B,0x304B},
{0x304C,Lo,0x304C,0x304C,0x304C},
{0x304D,Lo,0x304D,0x304D,0x304D},
{0x304E,Lo,0x304E,0x304E,0x304E},
{0x304F,Lo,0x304F,0x304F,0x304F},
{0x3050,Lo,0x3050,0x3050,0x3050},
{0x3051,Lo,0x3051,0x3051,0x3051},
{0x3052,Lo,0x3052,0x3052,0x3052},
{0x3053,Lo,0x3053,0x3053,0x3053},
{0x3054,Lo,0x3054,0x3054,0x3054},
{0x3055,Lo,0x3055,0x3055,0x3055},
{0x3056,Lo,0x3056,0x3056,0x3056},
{0x3057,Lo,0x3057,0x3057,0x3057},
{0x3058,Lo,0x3058,0x3058,0x3058},
{0x3059,Lo,0x3059,0x3059,0x3059},
{0x305A,Lo,0x305A,0x305A,0x305A},
{0x305B,Lo,0x305B,0x305B,0x305B},
{0x305C,Lo,0x305C,0x305C,0x305C},
{0x305D,Lo,0x305D,0x305D,0x305D},
{0x305E,Lo,0x305E,0x305E,0x305E},
{0x305F,Lo,0x305F,0x305F,0x305F},
{0x3060,Lo,0x3060,0x3060,0x3060},
{0x3061,Lo,0x3061,0x3061,0x3061},
{0x3062,Lo,0x3062,0x3062,0x3062},
{0x3063,Lo,0x3063,0x3063,0x3063},
{0x3064,Lo,0x3064,0x3064,0x3064},
{0x3065,Lo,0x3065,0x3065,0x3065},
{0x3066,Lo,0x3066,0x3066,0x3066},
{0x3067,Lo,0x3067,0x3067,0x3067},
{0x3068,Lo,0x3068,0x3068,0x3068},
{0x3069,Lo,0x3069,0x3069,0x3069},
{0x306A,Lo,0x306A,0x306A,0x306A},
{0x306B,Lo,0x306B,0x306B,0x306B},
{0x306C,Lo,0x306C,0x306C,0x306C},
{0x306D,Lo,0x306D,0x306D,0x306D},
{0x306E,Lo,0x306E,0x306E,0x306E},
{0x306F,Lo,0x306F,0x306F,0x306F},
{0x3070,Lo,0x3070,0x3070,0x3070},
{0x3071,Lo,0x3071,0x3071,0x3071},
{0x3072,Lo,0x3072,0x3072,0x3072},
{0x3073,Lo,0x3073,0x3073,0x3073},
{0x3074,Lo,0x3074,0x3074,0x3074},
{0x3075,Lo,0x3075,0x3075,0x3075},
{0x3076,Lo,0x3076,0x3076,0x3076},
{0x3077,Lo,0x3077,0x3077,0x3077},
{0x3078,Lo,0x3078,0x3078,0x3078},
{0x3079,Lo,0x3079,0x3079,0x3079},
{0x307A,Lo,0x307A,0x307A,0x307A},
{0x307B,Lo,0x307B,0x307B,0x307B},
{0x307C,Lo,0x307C,0x307C,0x307C},
{0x307D,Lo,0x307D,0x307D,0x307D},
{0x307E,Lo,0x307E,0x307E,0x307E},
{0x307F,Lo,0x307F,0x307F,0x307F},
{0x3080,Lo,0x3080,0x3080,0x3080},
{0x3081,Lo,0x3081,0x3081,0x3081},
{0x3082,Lo,0x3082,0x3082,0x3082},
{0x3083,Lo,0x3083,0x3083,0x3083},
{0x3084,Lo,0x3084,0x3084,0x3084},
{0x3085,Lo,0x3085,0x3085,0x3085},
{0x3086,Lo,0x3086,0x3086,0x3086},
{0x3087,Lo,0x3087,0x3087,0x3087},
{0x3088,Lo,0x3088,0x3088,0x3088},
{0x3089,Lo,0x3089,0x3089,0x3089},
{0x308A,Lo,0x308A,0x308A,0x308A},
{0x308B,Lo,0x308B,0x308B,0x308B},
{0x308C,Lo,0x308C,0x308C,0x308C},
{0x308D,Lo,0x308D,0x308D,0x308D},
{0x308E,Lo,0x308E,0x308E,0x308E},
{0x308F,Lo,0x308F,0x308F,0x308F},
{0x3090,Lo,0x3090,0x3090,0x3090},
{0x3091,Lo,0x3091,0x3091,0x3091},
{0x3092,Lo,0x3092,0x3092,0x3092},
{0x3093,Lo,0x3093,0x3093,0x3093},
{0x3094,Lo,0x3094,0x3094,0x3094},
{0x3095,Lo,0x3095,0x3095,0x3095},
{0x3096,Lo,0x3096,0x3096,0x3096},
{0x3099,Mn,0x3099,0x3099,0x3099},
{0x309A,Mn,0x309A,0x309A,0x309A},
{0x309B,Sk,0x309B,0x309B,0x309B},
{0x309C,Sk,0x309C,0x309C,0x309C},
{0x309D,Lm,0x309D,0x309D,0x309D},
{0x309E,Lm,0x309E,0x309E,0x309E},
{0x309F,Lo,0x309F,0x309F,0x309F},
{0x30A0,Pd,0x30A0,0x30A0,0x30A0},
{0x30A1,Lo,0x30A1,0x30A1,0x30A1},
{0x30A2,Lo,0x30A2,0x30A2,0x30A2},
{0x30A3,Lo,0x30A3,0x30A3,0x30A3},
{0x30A4,Lo,0x30A4,0x30A4,0x30A4},
{0x30A5,Lo,0x30A5,0x30A5,0x30A5},
{0x30A6,Lo,0x30A6,0x30A6,0x30A6},
{0x30A7,Lo,0x30A7,0x30A7,0x30A7},
{0x30A8,Lo,0x30A8,0x30A8,0x30A8},
{0x30A9,Lo,0x30A9,0x30A9,0x30A9},
{0x30AA,Lo,0x30AA,0x30AA,0x30AA},
{0x30AB,Lo,0x30AB,0x30AB,0x30AB},
{0x30AC,Lo,0x30AC,0x30AC,0x30AC},
{0x30AD,Lo,0x30AD,0x30AD,0x30AD},
{0x30AE,Lo,0x30AE,0x30AE,0x30AE},
{0x30AF,Lo,0x30AF,0x30AF,0x30AF},
{0x30B0,Lo,0x30B0,0x30B0,0x30B0},
{0x30B1,Lo,0x30B1,0x30B1,0x30B1},
{0x30B2,Lo,0x30B2,0x30B2,0x30B2},
{0x30B3,Lo,0x30B3,0x30B3,0x30B3},
{0x30B4,Lo,0x30B4,0x30B4,0x30B4},
{0x30B5,Lo,0x30B5,0x30B5,0x30B5},
{0x30B6,Lo,0x30B6,0x30B6,0x30B6},
{0x30B7,Lo,0x30B7,0x30B7,0x30B7},
{0x30B8,Lo,0x30B8,0x30B8,0x30B8},
{0x30B9,Lo,0x30B9,0x30B9,0x30B9},
{0x30BA,Lo,0x30BA,0x30BA,0x30BA},
{0x30BB,Lo,0x30BB,0x30BB,0x30BB},
{0x30BC,Lo,0x30BC,0x30BC,0x30BC},
{0x30BD,Lo,0x30BD,0x30BD,0x30BD},
{0x30BE,Lo,0x30BE,0x30BE,0x30BE},
{0x30BF,Lo,0x30BF,0x30BF,0x30BF},
{0x30C0,Lo,0x30C0,0x30C0,0x30C0},
{0x30C1,Lo,0x30C1,0x30C1,0x30C1},
{0x30C2,Lo,0x30C2,0x30C2,0x30C2},
{0x30C3,Lo,0x30C3,0x30C3,0x30C3},
{0x30C4,Lo,0x30C4,0x30C4,0x30C4},
{0x30C5,Lo,0x30C5,0x30C5,0x30C5},
{0x30C6,Lo,0x30C6,0x30C6,0x30C6},
{0x30C7,Lo,0x30C7,0x30C7,0x30C7},
{0x30C8,Lo,0x30C8,0x30C8,0x30C8},
{0x30C9,Lo,0x30C9,0x30C9,0x30C9},
{0x30CA,Lo,0x30CA,0x30CA,0x30CA},
{0x30CB,Lo,0x30CB,0x30CB,0x30CB},
{0x30CC,Lo,0x30CC,0x30CC,0x30CC},
{0x30CD,Lo,0x30CD,0x30CD,0x30CD},
{0x30CE,Lo,0x30CE,0x30CE,0x30CE},
{0x30CF,Lo,0x30CF,0x30CF,0x30CF},
{0x30D0,Lo,0x30D0,0x30D0,0x30D0},
{0x30D1,Lo,0x30D1,0x30D1,0x30D1},
{0x30D2,Lo,0x30D2,0x30D2,0x30D2},
{0x30D3,Lo,0x30D3,0x30D3,0x30D3},
{0x30D4,Lo,0x30D4,0x30D4,0x30D4},
{0x30D5,Lo,0x30D5,0x30D5,0x30D5},
{0x30D6,Lo,0x30D6,0x30D6,0x30D6},
{0x30D7,Lo,0x30D7,0x30D7,0x30D7},
{0x30D8,Lo,0x30D8,0x30D8,0x30D8},
{0x30D9,Lo,0x30D9,0x30D9,0x30D9},
{0x30DA,Lo,0x30DA,0x30DA,0x30DA},
{0x30DB,Lo,0x30DB,0x30DB,0x30DB},
{0x30DC,Lo,0x30DC,0x30DC,0x30DC},
{0x30DD,Lo,0x30DD,0x30DD,0x30DD},
{0x30DE,Lo,0x30DE,0x30DE,0x30DE},
{0x30DF,Lo,0x30DF,0x30DF,0x30DF},
{0x30E0,Lo,0x30E0,0x30E0,0x30E0},
{0x30E1,Lo,0x30E1,0x30E1,0x30E1},
{0x30E2,Lo,0x30E2,0x30E2,0x30E2},
{0x30E3,Lo,0x30E3,0x30E3,0x30E3},
{0x30E4,Lo,0x30E4,0x30E4,0x30E4},
{0x30E5,Lo,0x30E5,0x30E5,0x30E5},
{0x30E6,Lo,0x30E6,0x30E6,0x30E6},
{0x30E7,Lo,0x30E7,0x30E7,0x30E7},
{0x30E8,Lo,0x30E8,0x30E8,0x30E8},
{0x30E9,Lo,0x30E9,0x30E9,0x30E9},
{0x30EA,Lo,0x30EA,0x30EA,0x30EA},
{0x30EB,Lo,0x30EB,0x30EB,0x30EB},
{0x30EC,Lo,0x30EC,0x30EC,0x30EC},
{0x30ED,Lo,0x30ED,0x30ED,0x30ED},
{0x30EE,Lo,0x30EE,0x30EE,0x30EE},
{0x30EF,Lo,0x30EF,0x30EF,0x30EF},
{0x30F0,Lo,0x30F0,0x30F0,0x30F0},
{0x30F1,Lo,0x30F1,0x30F1,0x30F1},
{0x30F2,Lo,0x30F2,0x30F2,0x30F2},
{0x30F3,Lo,0x30F3,0x30F3,0x30F3},
{0x30F4,Lo,0x30F4,0x30F4,0x30F4},
{0x30F5,Lo,0x30F5,0x30F5,0x30F5},
{0x30F6,Lo,0x30F6,0x30F6,0x30F6},
{0x30F7,Lo,0x30F7,0x30F7,0x30F7},
{0x30F8,Lo,0x30F8,0x30F8,0x30F8},
{0x30F9,Lo,0x30F9,0x30F9,0x30F9},
{0x30FA,Lo,0x30FA,0x30FA,0x30FA},
{0x30FB,Po,0x30FB,0x30FB,0x30FB},
{0x30FC,Lm,0x30FC,0x30FC,0x30FC},
{0x30FD,Lm,0x30FD,0x30FD,0x30FD},
{0x30FE,Lm,0x30FE,0x30FE,0x30FE},
{0x30FF,Lo,0x30FF,0x30FF,0x30FF},
{0x3105,Lo,0x3105,0x3105,0x3105},
{0x3106,Lo,0x3106,0x3106,0x3106},
{0x3107,Lo,0x3107,0x3107,0x3107},
{0x3108,Lo,0x3108,0x3108,0x3108},
{0x3109,Lo,0x3109,0x3109,0x3109},
{0x310A,Lo,0x310A,0x310A,0x310A},
{0x310B,Lo,0x310B,0x310B,0x310B},
{0x310C,Lo,0x310C,0x310C,0x310C},
{0x310D,Lo,0x310D,0x310D,0x310D},
{0x310E,Lo,0x310E,0x310E,0x310E},
{0x310F,Lo,0x310F,0x310F,0x310F},
{0x3110,Lo,0x3110,0x3110,0x3110},
{0x3111,Lo,0x3111,0x3111,0x3111},
{0x3112,Lo,0x3112,0x3112,0x3112},
{0x3113,Lo,0x3113,0x3113,0x3113},
{0x3114,Lo,0x3114,0x3114,0x3114},
{0x3115,Lo,0x3115,0x3115,0x3115},
{0x3116,Lo,0x3116,0x3116,0x3116},
{0x3117,Lo,0x3117,0x3117,0x3117},
{0x3118,Lo,0x3118,0x3118,0x3118},
{0x3119,Lo,0x3119,0x3119,0x3119},
{0x311A,Lo,0x311A,0x311A,0x311A},
{0x311B,Lo,0x311B,0x311B,0x311B},
{0x311C,Lo,0x311C,0x311C,0x311C},
{0x311D,Lo,0x311D,0x311D,0x311D},
{0x311E,Lo,0x311E,0x311E,0x311E},
{0x311F,Lo,0x311F,0x311F,0x311F},
{0x3120,Lo,0x3120,0x3120,0x3120},
{0x3121,Lo,0x3121,0x3121,0x3121},
{0x3122,Lo,0x3122,0x3122,0x3122},
{0x3123,Lo,0x3123,0x3123,0x3123},
{0x3124,Lo,0x3124,0x3124,0x3124},
{0x3125,Lo,0x3125,0x3125,0x3125},
{0x3126,Lo,0x3126,0x3126,0x3126},
{0x3127,Lo,0x3127,0x3127,0x3127},
{0x3128,Lo,0x3128,0x3128,0x3128},
{0x3129,Lo,0x3129,0x3129,0x3129},
{0x312A,Lo,0x312A,0x312A,0x312A},
{0x312B,Lo,0x312B,0x312B,0x312B},
{0x312C,Lo,0x312C,0x312C,0x312C},
{0x3131,Lo,0x3131,0x3131,0x3131},
{0x3132,Lo,0x3132,0x3132,0x3132},
{0x3133,Lo,0x3133,0x3133,0x3133},
{0x3134,Lo,0x3134,0x3134,0x3134},
{0x3135,Lo,0x3135,0x3135,0x3135},
{0x3136,Lo,0x3136,0x3136,0x3136},
{0x3137,Lo,0x3137,0x3137,0x3137},
{0x3138,Lo,0x3138,0x3138,0x3138},
{0x3139,Lo,0x3139,0x3139,0x3139},
{0x313A,Lo,0x313A,0x313A,0x313A},
{0x313B,Lo,0x313B,0x313B,0x313B},
{0x313C,Lo,0x313C,0x313C,0x313C},
{0x313D,Lo,0x313D,0x313D,0x313D},
{0x313E,Lo,0x313E,0x313E,0x313E},
{0x313F,Lo,0x313F,0x313F,0x313F},
{0x3140,Lo,0x3140,0x3140,0x3140},
{0x3141,Lo,0x3141,0x3141,0x3141},
{0x3142,Lo,0x3142,0x3142,0x3142},
{0x3143,Lo,0x3143,0x3143,0x3143},
{0x3144,Lo,0x3144,0x3144,0x3144},
{0x3145,Lo,0x3145,0x3145,0x3145},
{0x3146,Lo,0x3146,0x3146,0x3146},
{0x3147,Lo,0x3147,0x3147,0x3147},
{0x3148,Lo,0x3148,0x3148,0x3148},
{0x3149,Lo,0x3149,0x3149,0x3149},
{0x314A,Lo,0x314A,0x314A,0x314A},
{0x314B,Lo,0x314B,0x314B,0x314B},
{0x314C,Lo,0x314C,0x314C,0x314C},
{0x314D,Lo,0x314D,0x314D,0x314D},
{0x314E,Lo,0x314E,0x314E,0x314E},
{0x314F,Lo,0x314F,0x314F,0x314F},
{0x3150,Lo,0x3150,0x3150,0x3150},
{0x3151,Lo,0x3151,0x3151,0x3151},
{0x3152,Lo,0x3152,0x3152,0x3152},
{0x3153,Lo,0x3153,0x3153,0x3153},
{0x3154,Lo,0x3154,0x3154,0x3154},
{0x3155,Lo,0x3155,0x3155,0x3155},
{0x3156,Lo,0x3156,0x3156,0x3156},
{0x3157,Lo,0x3157,0x3157,0x3157},
{0x3158,Lo,0x3158,0x3158,0x3158},
{0x3159,Lo,0x3159,0x3159,0x3159},
{0x315A,Lo,0x315A,0x315A,0x315A},
{0x315B,Lo,0x315B,0x315B,0x315B},
{0x315C,Lo,0x315C,0x315C,0x315C},
{0x315D,Lo,0x315D,0x315D,0x315D},
{0x315E,Lo,0x315E,0x315E,0x315E},
{0x315F,Lo,0x315F,0x315F,0x315F},
{0x3160,Lo,0x3160,0x3160,0x3160},
{0x3161,Lo,0x3161,0x3161,0x3161},
{0x3162,Lo,0x3162,0x3162,0x3162},
{0x3163,Lo,0x3163,0x3163,0x3163},
{0x3164,Lo,0x3164,0x3164,0x3164},
{0x3165,Lo,0x3165,0x3165,0x3165},
{0x3166,Lo,0x3166,0x3166,0x3166},
{0x3167,Lo,0x3167,0x3167,0x3167},
{0x3168,Lo,0x3168,0x3168,0x3168},
{0x3169,Lo,0x3169,0x3169,0x3169},
{0x316A,Lo,0x316A,0x316A,0x316A},
{0x316B,Lo,0x316B,0x316B,0x316B},
{0x316C,Lo,0x316C,0x316C,0x316C},
{0x316D,Lo,0x316D,0x316D,0x316D},
{0x316E,Lo,0x316E,0x316E,0x316E},
{0x316F,Lo,0x316F,0x316F,0x316F},
{0x3170,Lo,0x3170,0x3170,0x3170},
{0x3171,Lo,0x3171,0x3171,0x3171},
{0x3172,Lo,0x3172,0x3172,0x3172},
{0x3173,Lo,0x3173,0x3173,0x3173},
{0x3174,Lo,0x3174,0x3174,0x3174},
{0x3175,Lo,0x3175,0x3175,0x3175},
{0x3176,Lo,0x3176,0x3176,0x3176},
{0x3177,Lo,0x3177,0x3177,0x3177},
{0x3178,Lo,0x3178,0x3178,0x3178},
{0x3179,Lo,0x3179,0x3179,0x3179},
{0x317A,Lo,0x317A,0x317A,0x317A},
{0x317B,Lo,0x317B,0x317B,0x317B},
{0x317C,Lo,0x317C,0x317C,0x317C},
{0x317D,Lo,0x317D,0x317D,0x317D},
{0x317E,Lo,0x317E,0x317E,0x317E},
{0x317F,Lo,0x317F,0x317F,0x317F},
{0x3180,Lo,0x3180,0x3180,0x3180},
{0x3181,Lo,0x3181,0x3181,0x3181},
{0x3182,Lo,0x3182,0x3182,0x3182},
{0x3183,Lo,0x3183,0x3183,0x3183},
{0x3184,Lo,0x3184,0x3184,0x3184},
{0x3185,Lo,0x3185,0x3185,0x3185},
{0x3186,Lo,0x3186,0x3186,0x3186},
{0x3187,Lo,0x3187,0x3187,0x3187},
{0x3188,Lo,0x3188,0x3188,0x3188},
{0x3189,Lo,0x3189,0x3189,0x3189},
{0x318A,Lo,0x318A,0x318A,0x318A},
{0x318B,Lo,0x318B,0x318B,0x318B},
{0x318C,Lo,0x318C,0x318C,0x318C},
{0x318D,Lo,0x318D,0x318D,0x318D},
{0x318E,Lo,0x318E,0x318E,0x318E},
{0x3190,So,0x3190,0x3190,0x3190},
{0x3191,So,0x3191,0x3191,0x3191},
{0x3192,No,0x3192,0x3192,0x3192},
{0x3193,No,0x3193,0x3193,0x3193},
{0x3194,No,0x3194,0x3194,0x3194},
{0x3195,No,0x3195,0x3195,0x3195},
{0x3196,So,0x3196,0x3196,0x3196},
{0x3197,So,0x3197,0x3197,0x3197},
{0x3198,So,0x3198,0x3198,0x3198},
{0x3199,So,0x3199,0x3199,0x3199},
{0x319A,So,0x319A,0x319A,0x319A},
{0x319B,So,0x319B,0x319B,0x319B},
{0x319C,So,0x319C,0x319C,0x319C},
{0x319D,So,0x319D,0x319D,0x319D},
{0x319E,So,0x319E,0x319E,0x319E},
{0x319F,So,0x319F,0x319F,0x319F},
{0x31A0,Lo,0x31A0,0x31A0,0x31A0},
{0x31A1,Lo,0x31A1,0x31A1,0x31A1},
{0x31A2,Lo,0x31A2,0x31A2,0x31A2},
{0x31A3,Lo,0x31A3,0x31A3,0x31A3},
{0x31A4,Lo,0x31A4,0x31A4,0x31A4},
{0x31A5,Lo,0x31A5,0x31A5,0x31A5},
{0x31A6,Lo,0x31A6,0x31A6,0x31A6},
{0x31A7,Lo,0x31A7,0x31A7,0x31A7},
{0x31A8,Lo,0x31A8,0x31A8,0x31A8},
{0x31A9,Lo,0x31A9,0x31A9,0x31A9},
{0x31AA,Lo,0x31AA,0x31AA,0x31AA},
{0x31AB,Lo,0x31AB,0x31AB,0x31AB},
{0x31AC,Lo,0x31AC,0x31AC,0x31AC},
{0x31AD,Lo,0x31AD,0x31AD,0x31AD},
{0x31AE,Lo,0x31AE,0x31AE,0x31AE},
{0x31AF,Lo,0x31AF,0x31AF,0x31AF},
{0x31B0,Lo,0x31B0,0x31B0,0x31B0},
{0x31B1,Lo,0x31B1,0x31B1,0x31B1},
{0x31B2,Lo,0x31B2,0x31B2,0x31B2},
{0x31B3,Lo,0x31B3,0x31B3,0x31B3},
{0x31B4,Lo,0x31B4,0x31B4,0x31B4},
{0x31B5,Lo,0x31B5,0x31B5,0x31B5},
{0x31B6,Lo,0x31B6,0x31B6,0x31B6},
{0x31B7,Lo,0x31B7,0x31B7,0x31B7},
{0x31C0,So,0x31C0,0x31C0,0x31C0},
{0x31C1,So,0x31C1,0x31C1,0x31C1},
{0x31C2,So,0x31C2,0x31C2,0x31C2},
{0x31C3,So,0x31C3,0x31C3,0x31C3},
{0x31C4,So,0x31C4,0x31C4,0x31C4},
{0x31C5,So,0x31C5,0x31C5,0x31C5},
{0x31C6,So,0x31C6,0x31C6,0x31C6},
{0x31C7,So,0x31C7,0x31C7,0x31C7},
{0x31C8,So,0x31C8,0x31C8,0x31C8},
{0x31C9,So,0x31C9,0x31C9,0x31C9},
{0x31CA,So,0x31CA,0x31CA,0x31CA},
{0x31CB,So,0x31CB,0x31CB,0x31CB},
{0x31CC,So,0x31CC,0x31CC,0x31CC},
{0x31CD,So,0x31CD,0x31CD,0x31CD},
{0x31CE,So,0x31CE,0x31CE,0x31CE},
{0x31CF,So,0x31CF,0x31CF,0x31CF},
{0x31F0,Lo,0x31F0,0x31F0,0x31F0},
{0x31F1,Lo,0x31F1,0x31F1,0x31F1},
{0x31F2,Lo,0x31F2,0x31F2,0x31F2},
{0x31F3,Lo,0x31F3,0x31F3,0x31F3},
{0x31F4,Lo,0x31F4,0x31F4,0x31F4},
{0x31F5,Lo,0x31F5,0x31F5,0x31F5},
{0x31F6,Lo,0x31F6,0x31F6,0x31F6},
{0x31F7,Lo,0x31F7,0x31F7,0x31F7},
{0x31F8,Lo,0x31F8,0x31F8,0x31F8},
{0x31F9,Lo,0x31F9,0x31F9,0x31F9},
{0x31FA,Lo,0x31FA,0x31FA,0x31FA},
{0x31FB,Lo,0x31FB,0x31FB,0x31FB},
{0x31FC,Lo,0x31FC,0x31FC,0x31FC},
{0x31FD,Lo,0x31FD,0x31FD,0x31FD},
{0x31FE,Lo,0x31FE,0x31FE,0x31FE},
{0x31FF,Lo,0x31FF,0x31FF,0x31FF},
{0x3200,So,0x3200,0x3200,0x3200},
{0x3201,So,0x3201,0x3201,0x3201},
{0x3202,So,0x3202,0x3202,0x3202},
{0x3203,So,0x3203,0x3203,0x3203},
{0x3204,So,0x3204,0x3204,0x3204},
{0x3205,So,0x3205,0x3205,0x3205},
{0x3206,So,0x3206,0x3206,0x3206},
{0x3207,So,0x3207,0x3207,0x3207},
{0x3208,So,0x3208,0x3208,0x3208},
{0x3209,So,0x3209,0x3209,0x3209},
{0x320A,So,0x320A,0x320A,0x320A},
{0x320B,So,0x320B,0x320B,0x320B},
{0x320C,So,0x320C,0x320C,0x320C},
{0x320D,So,0x320D,0x320D,0x320D},
{0x320E,So,0x320E,0x320E,0x320E},
{0x320F,So,0x320F,0x320F,0x320F},
{0x3210,So,0x3210,0x3210,0x3210},
{0x3211,So,0x3211,0x3211,0x3211},
{0x3212,So,0x3212,0x3212,0x3212},
{0x3213,So,0x3213,0x3213,0x3213},
{0x3214,So,0x3214,0x3214,0x3214},
{0x3215,So,0x3215,0x3215,0x3215},
{0x3216,So,0x3216,0x3216,0x3216},
{0x3217,So,0x3217,0x3217,0x3217},
{0x3218,So,0x3218,0x3218,0x3218},
{0x3219,So,0x3219,0x3219,0x3219},
{0x321A,So,0x321A,0x321A,0x321A},
{0x321B,So,0x321B,0x321B,0x321B},
{0x321C,So,0x321C,0x321C,0x321C},
{0x321D,So,0x321D,0x321D,0x321D},
{0x321E,So,0x321E,0x321E,0x321E},
{0x3220,No,0x3220,0x3220,0x3220},
{0x3221,No,0x3221,0x3221,0x3221},
{0x3222,No,0x3222,0x3222,0x3222},
{0x3223,No,0x3223,0x3223,0x3223},
{0x3224,No,0x3224,0x3224,0x3224},
{0x3225,No,0x3225,0x3225,0x3225},
{0x3226,No,0x3226,0x3226,0x3226},
{0x3227,No,0x3227,0x3227,0x3227},
{0x3228,No,0x3228,0x3228,0x3228},
{0x3229,No,0x3229,0x3229,0x3229},
{0x322A,So,0x322A,0x322A,0x322A},
{0x322B,So,0x322B,0x322B,0x322B},
{0x322C,So,0x322C,0x322C,0x322C},
{0x322D,So,0x322D,0x322D,0x322D},
{0x322E,So,0x322E,0x322E,0x322E},
{0x322F,So,0x322F,0x322F,0x322F},
{0x3230,So,0x3230,0x3230,0x3230},
{0x3231,So,0x3231,0x3231,0x3231},
{0x3232,So,0x3232,0x3232,0x3232},
{0x3233,So,0x3233,0x3233,0x3233},
{0x3234,So,0x3234,0x3234,0x3234},
{0x3235,So,0x3235,0x3235,0x3235},
{0x3236,So,0x3236,0x3236,0x3236},
{0x3237,So,0x3237,0x3237,0x3237},
{0x3238,So,0x3238,0x3238,0x3238},
{0x3239,So,0x3239,0x3239,0x3239},
{0x323A,So,0x323A,0x323A,0x323A},
{0x323B,So,0x323B,0x323B,0x323B},
{0x323C,So,0x323C,0x323C,0x323C},
{0x323D,So,0x323D,0x323D,0x323D},
{0x323E,So,0x323E,0x323E,0x323E},
{0x323F,So,0x323F,0x323F,0x323F},
{0x3240,So,0x3240,0x3240,0x3240},
{0x3241,So,0x3241,0x3241,0x3241},
{0x3242,So,0x3242,0x3242,0x3242},
{0x3243,So,0x3243,0x3243,0x3243},
{0x3250,So,0x3250,0x3250,0x3250},
{0x3251,No,0x3251,0x3251,0x3251},
{0x3252,No,0x3252,0x3252,0x3252},
{0x3253,No,0x3253,0x3253,0x3253},
{0x3254,No,0x3254,0x3254,0x3254},
{0x3255,No,0x3255,0x3255,0x3255},
{0x3256,No,0x3256,0x3256,0x3256},
{0x3257,No,0x3257,0x3257,0x3257},
{0x3258,No,0x3258,0x3258,0x3258},
{0x3259,No,0x3259,0x3259,0x3259},
{0x325A,No,0x325A,0x325A,0x325A},
{0x325B,No,0x325B,0x325B,0x325B},
{0x325C,No,0x325C,0x325C,0x325C},
{0x325D,No,0x325D,0x325D,0x325D},
{0x325E,No,0x325E,0x325E,0x325E},
{0x325F,No,0x325F,0x325F,0x325F},
{0x3260,So,0x3260,0x3260,0x3260},
{0x3261,So,0x3261,0x3261,0x3261},
{0x3262,So,0x3262,0x3262,0x3262},
{0x3263,So,0x3263,0x3263,0x3263},
{0x3264,So,0x3264,0x3264,0x3264},
{0x3265,So,0x3265,0x3265,0x3265},
{0x3266,So,0x3266,0x3266,0x3266},
{0x3267,So,0x3267,0x3267,0x3267},
{0x3268,So,0x3268,0x3268,0x3268},
{0x3269,So,0x3269,0x3269,0x3269},
{0x326A,So,0x326A,0x326A,0x326A},
{0x326B,So,0x326B,0x326B,0x326B},
{0x326C,So,0x326C,0x326C,0x326C},
{0x326D,So,0x326D,0x326D,0x326D},
{0x326E,So,0x326E,0x326E,0x326E},
{0x326F,So,0x326F,0x326F,0x326F},
{0x3270,So,0x3270,0x3270,0x3270},
{0x3271,So,0x3271,0x3271,0x3271},
{0x3272,So,0x3272,0x3272,0x3272},
{0x3273,So,0x3273,0x3273,0x3273},
{0x3274,So,0x3274,0x3274,0x3274},
{0x3275,So,0x3275,0x3275,0x3275},
{0x3276,So,0x3276,0x3276,0x3276},
{0x3277,So,0x3277,0x3277,0x3277},
{0x3278,So,0x3278,0x3278,0x3278},
{0x3279,So,0x3279,0x3279,0x3279},
{0x327A,So,0x327A,0x327A,0x327A},
{0x327B,So,0x327B,0x327B,0x327B},
{0x327C,So,0x327C,0x327C,0x327C},
{0x327D,So,0x327D,0x327D,0x327D},
{0x327E,So,0x327E,0x327E,0x327E},
{0x327F,So,0x327F,0x327F,0x327F},
{0x3280,No,0x3280,0x3280,0x3280},
{0x3281,No,0x3281,0x3281,0x3281},
{0x3282,No,0x3282,0x3282,0x3282},
{0x3283,No,0x3283,0x3283,0x3283},
{0x3284,No,0x3284,0x3284,0x3284},
{0x3285,No,0x3285,0x3285,0x3285},
{0x3286,No,0x3286,0x3286,0x3286},
{0x3287,No,0x3287,0x3287,0x3287},
{0x3288,No,0x3288,0x3288,0x3288},
{0x3289,No,0x3289,0x3289,0x3289},
{0x328A,So,0x328A,0x328A,0x328A},
{0x328B,So,0x328B,0x328B,0x328B},
{0x328C,So,0x328C,0x328C,0x328C},
{0x328D,So,0x328D,0x328D,0x328D},
{0x328E,So,0x328E,0x328E,0x328E},
{0x328F,So,0x328F,0x328F,0x328F},
{0x3290,So,0x3290,0x3290,0x3290},
{0x3291,So,0x3291,0x3291,0x3291},
{0x3292,So,0x3292,0x3292,0x3292},
{0x3293,So,0x3293,0x3293,0x3293},
{0x3294,So,0x3294,0x3294,0x3294},
{0x3295,So,0x3295,0x3295,0x3295},
{0x3296,So,0x3296,0x3296,0x3296},
{0x3297,So,0x3297,0x3297,0x3297},
{0x3298,So,0x3298,0x3298,0x3298},
{0x3299,So,0x3299,0x3299,0x3299},
{0x329A,So,0x329A,0x329A,0x329A},
{0x329B,So,0x329B,0x329B,0x329B},
{0x329C,So,0x329C,0x329C,0x329C},
{0x329D,So,0x329D,0x329D,0x329D},
{0x329E,So,0x329E,0x329E,0x329E},
{0x329F,So,0x329F,0x329F,0x329F},
{0x32A0,So,0x32A0,0x32A0,0x32A0},
{0x32A1,So,0x32A1,0x32A1,0x32A1},
{0x32A2,So,0x32A2,0x32A2,0x32A2},
{0x32A3,So,0x32A3,0x32A3,0x32A3},
{0x32A4,So,0x32A4,0x32A4,0x32A4},
{0x32A5,So,0x32A5,0x32A5,0x32A5},
{0x32A6,So,0x32A6,0x32A6,0x32A6},
{0x32A7,So,0x32A7,0x32A7,0x32A7},
{0x32A8,So,0x32A8,0x32A8,0x32A8},
{0x32A9,So,0x32A9,0x32A9,0x32A9},
{0x32AA,So,0x32AA,0x32AA,0x32AA},
{0x32AB,So,0x32AB,0x32AB,0x32AB},
{0x32AC,So,0x32AC,0x32AC,0x32AC},
{0x32AD,So,0x32AD,0x32AD,0x32AD},
{0x32AE,So,0x32AE,0x32AE,0x32AE},
{0x32AF,So,0x32AF,0x32AF,0x32AF},
{0x32B0,So,0x32B0,0x32B0,0x32B0},
{0x32B1,No,0x32B1,0x32B1,0x32B1},
{0x32B2,No,0x32B2,0x32B2,0x32B2},
{0x32B3,No,0x32B3,0x32B3,0x32B3},
{0x32B4,No,0x32B4,0x32B4,0x32B4},
{0x32B5,No,0x32B5,0x32B5,0x32B5},
{0x32B6,No,0x32B6,0x32B6,0x32B6},
{0x32B7,No,0x32B7,0x32B7,0x32B7},
{0x32B8,No,0x32B8,0x32B8,0x32B8},
{0x32B9,No,0x32B9,0x32B9,0x32B9},
{0x32BA,No,0x32BA,0x32BA,0x32BA},
{0x32BB,No,0x32BB,0x32BB,0x32BB},
{0x32BC,No,0x32BC,0x32BC,0x32BC},
{0x32BD,No,0x32BD,0x32BD,0x32BD},
{0x32BE,No,0x32BE,0x32BE,0x32BE},
{0x32BF,No,0x32BF,0x32BF,0x32BF},
{0x32C0,So,0x32C0,0x32C0,0x32C0},
{0x32C1,So,0x32C1,0x32C1,0x32C1},
{0x32C2,So,0x32C2,0x32C2,0x32C2},
{0x32C3,So,0x32C3,0x32C3,0x32C3},
{0x32C4,So,0x32C4,0x32C4,0x32C4},
{0x32C5,So,0x32C5,0x32C5,0x32C5},
{0x32C6,So,0x32C6,0x32C6,0x32C6},
{0x32C7,So,0x32C7,0x32C7,0x32C7},
{0x32C8,So,0x32C8,0x32C8,0x32C8},
{0x32C9,So,0x32C9,0x32C9,0x32C9},
{0x32CA,So,0x32CA,0x32CA,0x32CA},
{0x32CB,So,0x32CB,0x32CB,0x32CB},
{0x32CC,So,0x32CC,0x32CC,0x32CC},
{0x32CD,So,0x32CD,0x32CD,0x32CD},
{0x32CE,So,0x32CE,0x32CE,0x32CE},
{0x32CF,So,0x32CF,0x32CF,0x32CF},
{0x32D0,So,0x32D0,0x32D0,0x32D0},
{0x32D1,So,0x32D1,0x32D1,0x32D1},
{0x32D2,So,0x32D2,0x32D2,0x32D2},
{0x32D3,So,0x32D3,0x32D3,0x32D3},
{0x32D4,So,0x32D4,0x32D4,0x32D4},
{0x32D5,So,0x32D5,0x32D5,0x32D5},
{0x32D6,So,0x32D6,0x32D6,0x32D6},
{0x32D7,So,0x32D7,0x32D7,0x32D7},
{0x32D8,So,0x32D8,0x32D8,0x32D8},
{0x32D9,So,0x32D9,0x32D9,0x32D9},
{0x32DA,So,0x32DA,0x32DA,0x32DA},
{0x32DB,So,0x32DB,0x32DB,0x32DB},
{0x32DC,So,0x32DC,0x32DC,0x32DC},
{0x32DD,So,0x32DD,0x32DD,0x32DD},
{0x32DE,So,0x32DE,0x32DE,0x32DE},
{0x32DF,So,0x32DF,0x32DF,0x32DF},
{0x32E0,So,0x32E0,0x32E0,0x32E0},
{0x32E1,So,0x32E1,0x32E1,0x32E1},
{0x32E2,So,0x32E2,0x32E2,0x32E2},
{0x32E3,So,0x32E3,0x32E3,0x32E3},
{0x32E4,So,0x32E4,0x32E4,0x32E4},
{0x32E5,So,0x32E5,0x32E5,0x32E5},
{0x32E6,So,0x32E6,0x32E6,0x32E6},
{0x32E7,So,0x32E7,0x32E7,0x32E7},
{0x32E8,So,0x32E8,0x32E8,0x32E8},
{0x32E9,So,0x32E9,0x32E9,0x32E9},
{0x32EA,So,0x32EA,0x32EA,0x32EA},
{0x32EB,So,0x32EB,0x32EB,0x32EB},
{0x32EC,So,0x32EC,0x32EC,0x32EC},
{0x32ED,So,0x32ED,0x32ED,0x32ED},
{0x32EE,So,0x32EE,0x32EE,0x32EE},
{0x32EF,So,0x32EF,0x32EF,0x32EF},
{0x32F0,So,0x32F0,0x32F0,0x32F0},
{0x32F1,So,0x32F1,0x32F1,0x32F1},
{0x32F2,So,0x32F2,0x32F2,0x32F2},
{0x32F3,So,0x32F3,0x32F3,0x32F3},
{0x32F4,So,0x32F4,0x32F4,0x32F4},
{0x32F5,So,0x32F5,0x32F5,0x32F5},
{0x32F6,So,0x32F6,0x32F6,0x32F6},
{0x32F7,So,0x32F7,0x32F7,0x32F7},
{0x32F8,So,0x32F8,0x32F8,0x32F8},
{0x32F9,So,0x32F9,0x32F9,0x32F9},
{0x32FA,So,0x32FA,0x32FA,0x32FA},
{0x32FB,So,0x32FB,0x32FB,0x32FB},
{0x32FC,So,0x32FC,0x32FC,0x32FC},
{0x32FD,So,0x32FD,0x32FD,0x32FD},
{0x32FE,So,0x32FE,0x32FE,0x32FE},
{0x3300,So,0x3300,0x3300,0x3300},
{0x3301,So,0x3301,0x3301,0x3301},
{0x3302,So,0x3302,0x3302,0x3302},
{0x3303,So,0x3303,0x3303,0x3303},
{0x3304,So,0x3304,0x3304,0x3304},
{0x3305,So,0x3305,0x3305,0x3305},
{0x3306,So,0x3306,0x3306,0x3306},
{0x3307,So,0x3307,0x3307,0x3307},
{0x3308,So,0x3308,0x3308,0x3308},
{0x3309,So,0x3309,0x3309,0x3309},
{0x330A,So,0x330A,0x330A,0x330A},
{0x330B,So,0x330B,0x330B,0x330B},
{0x330C,So,0x330C,0x330C,0x330C},
{0x330D,So,0x330D,0x330D,0x330D},
{0x330E,So,0x330E,0x330E,0x330E},
{0x330F,So,0x330F,0x330F,0x330F},
{0x3310,So,0x3310,0x3310,0x3310},
{0x3311,So,0x3311,0x3311,0x3311},
{0x3312,So,0x3312,0x3312,0x3312},
{0x3313,So,0x3313,0x3313,0x3313},
{0x3314,So,0x3314,0x3314,0x3314},
{0x3315,So,0x3315,0x3315,0x3315},
{0x3316,So,0x3316,0x3316,0x3316},
{0x3317,So,0x3317,0x3317,0x3317},
{0x3318,So,0x3318,0x3318,0x3318},
{0x3319,So,0x3319,0x3319,0x3319},
{0x331A,So,0x331A,0x331A,0x331A},
{0x331B,So,0x331B,0x331B,0x331B},
{0x331C,So,0x331C,0x331C,0x331C},
{0x331D,So,0x331D,0x331D,0x331D},
{0x331E,So,0x331E,0x331E,0x331E},
{0x331F,So,0x331F,0x331F,0x331F},
{0x3320,So,0x3320,0x3320,0x3320},
{0x3321,So,0x3321,0x3321,0x3321},
{0x3322,So,0x3322,0x3322,0x3322},
{0x3323,So,0x3323,0x3323,0x3323},
{0x3324,So,0x3324,0x3324,0x3324},
{0x3325,So,0x3325,0x3325,0x3325},
{0x3326,So,0x3326,0x3326,0x3326},
{0x3327,So,0x3327,0x3327,0x3327},
{0x3328,So,0x3328,0x3328,0x3328},
{0x3329,So,0x3329,0x3329,0x3329},
{0x332A,So,0x332A,0x332A,0x332A},
{0x332B,So,0x332B,0x332B,0x332B},
{0x332C,So,0x332C,0x332C,0x332C},
{0x332D,So,0x332D,0x332D,0x332D},
{0x332E,So,0x332E,0x332E,0x332E},
{0x332F,So,0x332F,0x332F,0x332F},
{0x3330,So,0x3330,0x3330,0x3330},
{0x3331,So,0x3331,0x3331,0x3331},
{0x3332,So,0x3332,0x3332,0x3332},
{0x3333,So,0x3333,0x3333,0x3333},
{0x3334,So,0x3334,0x3334,0x3334},
{0x3335,So,0x3335,0x3335,0x3335},
{0x3336,So,0x3336,0x3336,0x3336},
{0x3337,So,0x3337,0x3337,0x3337},
{0x3338,So,0x3338,0x3338,0x3338},
{0x3339,So,0x3339,0x3339,0x3339},
{0x333A,So,0x333A,0x333A,0x333A},
{0x333B,So,0x333B,0x333B,0x333B},
{0x333C,So,0x333C,0x333C,0x333C},
{0x333D,So,0x333D,0x333D,0x333D},
{0x333E,So,0x333E,0x333E,0x333E},
{0x333F,So,0x333F,0x333F,0x333F},
{0x3340,So,0x3340,0x3340,0x3340},
{0x3341,So,0x3341,0x3341,0x3341},
{0x3342,So,0x3342,0x3342,0x3342},
{0x3343,So,0x3343,0x3343,0x3343},
{0x3344,So,0x3344,0x3344,0x3344},
{0x3345,So,0x3345,0x3345,0x3345},
{0x3346,So,0x3346,0x3346,0x3346},
{0x3347,So,0x3347,0x3347,0x3347},
{0x3348,So,0x3348,0x3348,0x3348},
{0x3349,So,0x3349,0x3349,0x3349},
{0x334A,So,0x334A,0x334A,0x334A},
{0x334B,So,0x334B,0x334B,0x334B},
{0x334C,So,0x334C,0x334C,0x334C},
{0x334D,So,0x334D,0x334D,0x334D},
{0x334E,So,0x334E,0x334E,0x334E},
{0x334F,So,0x334F,0x334F,0x334F},
{0x3350,So,0x3350,0x3350,0x3350},
{0x3351,So,0x3351,0x3351,0x3351},
{0x3352,So,0x3352,0x3352,0x3352},
{0x3353,So,0x3353,0x3353,0x3353},
{0x3354,So,0x3354,0x3354,0x3354},
{0x3355,So,0x3355,0x3355,0x3355},
{0x3356,So,0x3356,0x3356,0x3356},
{0x3357,So,0x3357,0x3357,0x3357},
{0x3358,So,0x3358,0x3358,0x3358},
{0x3359,So,0x3359,0x3359,0x3359},
{0x335A,So,0x335A,0x335A,0x335A},
{0x335B,So,0x335B,0x335B,0x335B},
{0x335C,So,0x335C,0x335C,0x335C},
{0x335D,So,0x335D,0x335D,0x335D},
{0x335E,So,0x335E,0x335E,0x335E},
{0x335F,So,0x335F,0x335F,0x335F},
{0x3360,So,0x3360,0x3360,0x3360},
{0x3361,So,0x3361,0x3361,0x3361},
{0x3362,So,0x3362,0x3362,0x3362},
{0x3363,So,0x3363,0x3363,0x3363},
{0x3364,So,0x3364,0x3364,0x3364},
{0x3365,So,0x3365,0x3365,0x3365},
{0x3366,So,0x3366,0x3366,0x3366},
{0x3367,So,0x3367,0x3367,0x3367},
{0x3368,So,0x3368,0x3368,0x3368},
{0x3369,So,0x3369,0x3369,0x3369},
{0x336A,So,0x336A,0x336A,0x336A},
{0x336B,So,0x336B,0x336B,0x336B},
{0x336C,So,0x336C,0x336C,0x336C},
{0x336D,So,0x336D,0x336D,0x336D},
{0x336E,So,0x336E,0x336E,0x336E},
{0x336F,So,0x336F,0x336F,0x336F},
{0x3370,So,0x3370,0x3370,0x3370},
{0x3371,So,0x3371,0x3371,0x3371},
{0x3372,So,0x3372,0x3372,0x3372},
{0x3373,So,0x3373,0x3373,0x3373},
{0x3374,So,0x3374,0x3374,0x3374},
{0x3375,So,0x3375,0x3375,0x3375},
{0x3376,So,0x3376,0x3376,0x3376},
{0x3377,So,0x3377,0x3377,0x3377},
{0x3378,So,0x3378,0x3378,0x3378},
{0x3379,So,0x3379,0x3379,0x3379},
{0x337A,So,0x337A,0x337A,0x337A},
{0x337B,So,0x337B,0x337B,0x337B},
{0x337C,So,0x337C,0x337C,0x337C},
{0x337D,So,0x337D,0x337D,0x337D},
{0x337E,So,0x337E,0x337E,0x337E},
{0x337F,So,0x337F,0x337F,0x337F},
{0x3380,So,0x3380,0x3380,0x3380},
{0x3381,So,0x3381,0x3381,0x3381},
{0x3382,So,0x3382,0x3382,0x3382},
{0x3383,So,0x3383,0x3383,0x3383},
{0x3384,So,0x3384,0x3384,0x3384},
{0x3385,So,0x3385,0x3385,0x3385},
{0x3386,So,0x3386,0x3386,0x3386},
{0x3387,So,0x3387,0x3387,0x3387},
{0x3388,So,0x3388,0x3388,0x3388},
{0x3389,So,0x3389,0x3389,0x3389},
{0x338A,So,0x338A,0x338A,0x338A},
{0x338B,So,0x338B,0x338B,0x338B},
{0x338C,So,0x338C,0x338C,0x338C},
{0x338D,So,0x338D,0x338D,0x338D},
{0x338E,So,0x338E,0x338E,0x338E},
{0x338F,So,0x338F,0x338F,0x338F},
{0x3390,So,0x3390,0x3390,0x3390},
{0x3391,So,0x3391,0x3391,0x3391},
{0x3392,So,0x3392,0x3392,0x3392},
{0x3393,So,0x3393,0x3393,0x3393},
{0x3394,So,0x3394,0x3394,0x3394},
{0x3395,So,0x3395,0x3395,0x3395},
{0x3396,So,0x3396,0x3396,0x3396},
{0x3397,So,0x3397,0x3397,0x3397},
{0x3398,So,0x3398,0x3398,0x3398},
{0x3399,So,0x3399,0x3399,0x3399},
{0x339A,So,0x339A,0x339A,0x339A},
{0x339B,So,0x339B,0x339B,0x339B},
{0x339C,So,0x339C,0x339C,0x339C},
{0x339D,So,0x339D,0x339D,0x339D},
{0x339E,So,0x339E,0x339E,0x339E},
{0x339F,So,0x339F,0x339F,0x339F},
{0x33A0,So,0x33A0,0x33A0,0x33A0},
{0x33A1,So,0x33A1,0x33A1,0x33A1},
{0x33A2,So,0x33A2,0x33A2,0x33A2},
{0x33A3,So,0x33A3,0x33A3,0x33A3},
{0x33A4,So,0x33A4,0x33A4,0x33A4},
{0x33A5,So,0x33A5,0x33A5,0x33A5},
{0x33A6,So,0x33A6,0x33A6,0x33A6},
{0x33A7,So,0x33A7,0x33A7,0x33A7},
{0x33A8,So,0x33A8,0x33A8,0x33A8},
{0x33A9,So,0x33A9,0x33A9,0x33A9},
{0x33AA,So,0x33AA,0x33AA,0x33AA},
{0x33AB,So,0x33AB,0x33AB,0x33AB},
{0x33AC,So,0x33AC,0x33AC,0x33AC},
{0x33AD,So,0x33AD,0x33AD,0x33AD},
{0x33AE,So,0x33AE,0x33AE,0x33AE},
{0x33AF,So,0x33AF,0x33AF,0x33AF},
{0x33B0,So,0x33B0,0x33B0,0x33B0},
{0x33B1,So,0x33B1,0x33B1,0x33B1},
{0x33B2,So,0x33B2,0x33B2,0x33B2},
{0x33B3,So,0x33B3,0x33B3,0x33B3},
{0x33B4,So,0x33B4,0x33B4,0x33B4},
{0x33B5,So,0x33B5,0x33B5,0x33B5},
{0x33B6,So,0x33B6,0x33B6,0x33B6},
{0x33B7,So,0x33B7,0x33B7,0x33B7},
{0x33B8,So,0x33B8,0x33B8,0x33B8},
{0x33B9,So,0x33B9,0x33B9,0x33B9},
{0x33BA,So,0x33BA,0x33BA,0x33BA},
{0x33BB,So,0x33BB,0x33BB,0x33BB},
{0x33BC,So,0x33BC,0x33BC,0x33BC},
{0x33BD,So,0x33BD,0x33BD,0x33BD},
{0x33BE,So,0x33BE,0x33BE,0x33BE},
{0x33BF,So,0x33BF,0x33BF,0x33BF},
{0x33C0,So,0x33C0,0x33C0,0x33C0},
{0x33C1,So,0x33C1,0x33C1,0x33C1},
{0x33C2,So,0x33C2,0x33C2,0x33C2},
{0x33C3,So,0x33C3,0x33C3,0x33C3},
{0x33C4,So,0x33C4,0x33C4,0x33C4},
{0x33C5,So,0x33C5,0x33C5,0x33C5},
{0x33C6,So,0x33C6,0x33C6,0x33C6},
{0x33C7,So,0x33C7,0x33C7,0x33C7},
{0x33C8,So,0x33C8,0x33C8,0x33C8},
{0x33C9,So,0x33C9,0x33C9,0x33C9},
{0x33CA,So,0x33CA,0x33CA,0x33CA},
{0x33CB,So,0x33CB,0x33CB,0x33CB},
{0x33CC,So,0x33CC,0x33CC,0x33CC},
{0x33CD,So,0x33CD,0x33CD,0x33CD},
{0x33CE,So,0x33CE,0x33CE,0x33CE},
{0x33CF,So,0x33CF,0x33CF,0x33CF},
{0x33D0,So,0x33D0,0x33D0,0x33D0},
{0x33D1,So,0x33D1,0x33D1,0x33D1},
{0x33D2,So,0x33D2,0x33D2,0x33D2},
{0x33D3,So,0x33D3,0x33D3,0x33D3},
{0x33D4,So,0x33D4,0x33D4,0x33D4},
{0x33D5,So,0x33D5,0x33D5,0x33D5},
{0x33D6,So,0x33D6,0x33D6,0x33D6},
{0x33D7,So,0x33D7,0x33D7,0x33D7},
{0x33D8,So,0x33D8,0x33D8,0x33D8},
{0x33D9,So,0x33D9,0x33D9,0x33D9},
{0x33DA,So,0x33DA,0x33DA,0x33DA},
{0x33DB,So,0x33DB,0x33DB,0x33DB},
{0x33DC,So,0x33DC,0x33DC,0x33DC},
{0x33DD,So,0x33DD,0x33DD,0x33DD},
{0x33DE,So,0x33DE,0x33DE,0x33DE},
{0x33DF,So,0x33DF,0x33DF,0x33DF},
{0x33E0,So,0x33E0,0x33E0,0x33E0},
{0x33E1,So,0x33E1,0x33E1,0x33E1},
{0x33E2,So,0x33E2,0x33E2,0x33E2},
{0x33E3,So,0x33E3,0x33E3,0x33E3},
{0x33E4,So,0x33E4,0x33E4,0x33E4},
{0x33E5,So,0x33E5,0x33E5,0x33E5},
{0x33E6,So,0x33E6,0x33E6,0x33E6},
{0x33E7,So,0x33E7,0x33E7,0x33E7},
{0x33E8,So,0x33E8,0x33E8,0x33E8},
{0x33E9,So,0x33E9,0x33E9,0x33E9},
{0x33EA,So,0x33EA,0x33EA,0x33EA},
{0x33EB,So,0x33EB,0x33EB,0x33EB},
{0x33EC,So,0x33EC,0x33EC,0x33EC},
{0x33ED,So,0x33ED,0x33ED,0x33ED},
{0x33EE,So,0x33EE,0x33EE,0x33EE},
{0x33EF,So,0x33EF,0x33EF,0x33EF},
{0x33F0,So,0x33F0,0x33F0,0x33F0},
{0x33F1,So,0x33F1,0x33F1,0x33F1},
{0x33F2,So,0x33F2,0x33F2,0x33F2},
{0x33F3,So,0x33F3,0x33F3,0x33F3},
{0x33F4,So,0x33F4,0x33F4,0x33F4},
{0x33F5,So,0x33F5,0x33F5,0x33F5},
{0x33F6,So,0x33F6,0x33F6,0x33F6},
{0x33F7,So,0x33F7,0x33F7,0x33F7},
{0x33F8,So,0x33F8,0x33F8,0x33F8},
{0x33F9,So,0x33F9,0x33F9,0x33F9},
{0x33FA,So,0x33FA,0x33FA,0x33FA},
{0x33FB,So,0x33FB,0x33FB,0x33FB},
{0x33FC,So,0x33FC,0x33FC,0x33FC},
{0x33FD,So,0x33FD,0x33FD,0x33FD},
{0x33FE,So,0x33FE,0x33FE,0x33FE},
{0x33FF,So,0x33FF,0x33FF,0x33FF},
{0x3400,Lo,0x3400,0x3400,0x3400},
{0x4DB5,Lo,0x4DB5,0x4DB5,0x4DB5},
{0x4DC0,So,0x4DC0,0x4DC0,0x4DC0},
{0x4DC1,So,0x4DC1,0x4DC1,0x4DC1},
{0x4DC2,So,0x4DC2,0x4DC2,0x4DC2},
{0x4DC3,So,0x4DC3,0x4DC3,0x4DC3},
{0x4DC4,So,0x4DC4,0x4DC4,0x4DC4},
{0x4DC5,So,0x4DC5,0x4DC5,0x4DC5},
{0x4DC6,So,0x4DC6,0x4DC6,0x4DC6},
{0x4DC7,So,0x4DC7,0x4DC7,0x4DC7},
{0x4DC8,So,0x4DC8,0x4DC8,0x4DC8},
{0x4DC9,So,0x4DC9,0x4DC9,0x4DC9},
{0x4DCA,So,0x4DCA,0x4DCA,0x4DCA},
{0x4DCB,So,0x4DCB,0x4DCB,0x4DCB},
{0x4DCC,So,0x4DCC,0x4DCC,0x4DCC},
{0x4DCD,So,0x4DCD,0x4DCD,0x4DCD},
{0x4DCE,So,0x4DCE,0x4DCE,0x4DCE},
{0x4DCF,So,0x4DCF,0x4DCF,0x4DCF},
{0x4DD0,So,0x4DD0,0x4DD0,0x4DD0},
{0x4DD1,So,0x4DD1,0x4DD1,0x4DD1},
{0x4DD2,So,0x4DD2,0x4DD2,0x4DD2},
{0x4DD3,So,0x4DD3,0x4DD3,0x4DD3},
{0x4DD4,So,0x4DD4,0x4DD4,0x4DD4},
{0x4DD5,So,0x4DD5,0x4DD5,0x4DD5},
{0x4DD6,So,0x4DD6,0x4DD6,0x4DD6},
{0x4DD7,So,0x4DD7,0x4DD7,0x4DD7},
{0x4DD8,So,0x4DD8,0x4DD8,0x4DD8},
{0x4DD9,So,0x4DD9,0x4DD9,0x4DD9},
{0x4DDA,So,0x4DDA,0x4DDA,0x4DDA},
{0x4DDB,So,0x4DDB,0x4DDB,0x4DDB},
{0x4DDC,So,0x4DDC,0x4DDC,0x4DDC},
{0x4DDD,So,0x4DDD,0x4DDD,0x4DDD},
{0x4DDE,So,0x4DDE,0x4DDE,0x4DDE},
{0x4DDF,So,0x4DDF,0x4DDF,0x4DDF},
{0x4DE0,So,0x4DE0,0x4DE0,0x4DE0},
{0x4DE1,So,0x4DE1,0x4DE1,0x4DE1},
{0x4DE2,So,0x4DE2,0x4DE2,0x4DE2},
{0x4DE3,So,0x4DE3,0x4DE3,0x4DE3},
{0x4DE4,So,0x4DE4,0x4DE4,0x4DE4},
{0x4DE5,So,0x4DE5,0x4DE5,0x4DE5},
{0x4DE6,So,0x4DE6,0x4DE6,0x4DE6},
{0x4DE7,So,0x4DE7,0x4DE7,0x4DE7},
{0x4DE8,So,0x4DE8,0x4DE8,0x4DE8},
{0x4DE9,So,0x4DE9,0x4DE9,0x4DE9},
{0x4DEA,So,0x4DEA,0x4DEA,0x4DEA},
{0x4DEB,So,0x4DEB,0x4DEB,0x4DEB},
{0x4DEC,So,0x4DEC,0x4DEC,0x4DEC},
{0x4DED,So,0x4DED,0x4DED,0x4DED},
{0x4DEE,So,0x4DEE,0x4DEE,0x4DEE},
{0x4DEF,So,0x4DEF,0x4DEF,0x4DEF},
{0x4DF0,So,0x4DF0,0x4DF0,0x4DF0},
{0x4DF1,So,0x4DF1,0x4DF1,0x4DF1},
{0x4DF2,So,0x4DF2,0x4DF2,0x4DF2},
{0x4DF3,So,0x4DF3,0x4DF3,0x4DF3},
{0x4DF4,So,0x4DF4,0x4DF4,0x4DF4},
{0x4DF5,So,0x4DF5,0x4DF5,0x4DF5},
{0x4DF6,So,0x4DF6,0x4DF6,0x4DF6},
{0x4DF7,So,0x4DF7,0x4DF7,0x4DF7},
{0x4DF8,So,0x4DF8,0x4DF8,0x4DF8},
{0x4DF9,So,0x4DF9,0x4DF9,0x4DF9},
{0x4DFA,So,0x4DFA,0x4DFA,0x4DFA},
{0x4DFB,So,0x4DFB,0x4DFB,0x4DFB},
{0x4DFC,So,0x4DFC,0x4DFC,0x4DFC},
{0x4DFD,So,0x4DFD,0x4DFD,0x4DFD},
{0x4DFE,So,0x4DFE,0x4DFE,0x4DFE},
{0x4DFF,So,0x4DFF,0x4DFF,0x4DFF},
{0x4E00,Lo,0x4E00,0x4E00,0x4E00},
{0x9FBB,Lo,0x9FBB,0x9FBB,0x9FBB},
{0xA000,Lo,0xA000,0xA000,0xA000},
{0xA001,Lo,0xA001,0xA001,0xA001},
{0xA002,Lo,0xA002,0xA002,0xA002},
{0xA003,Lo,0xA003,0xA003,0xA003},
{0xA004,Lo,0xA004,0xA004,0xA004},
{0xA005,Lo,0xA005,0xA005,0xA005},
{0xA006,Lo,0xA006,0xA006,0xA006},
{0xA007,Lo,0xA007,0xA007,0xA007},
{0xA008,Lo,0xA008,0xA008,0xA008},
{0xA009,Lo,0xA009,0xA009,0xA009},
{0xA00A,Lo,0xA00A,0xA00A,0xA00A},
{0xA00B,Lo,0xA00B,0xA00B,0xA00B},
{0xA00C,Lo,0xA00C,0xA00C,0xA00C},
{0xA00D,Lo,0xA00D,0xA00D,0xA00D},
{0xA00E,Lo,0xA00E,0xA00E,0xA00E},
{0xA00F,Lo,0xA00F,0xA00F,0xA00F},
{0xA010,Lo,0xA010,0xA010,0xA010},
{0xA011,Lo,0xA011,0xA011,0xA011},
{0xA012,Lo,0xA012,0xA012,0xA012},
{0xA013,Lo,0xA013,0xA013,0xA013},
{0xA014,Lo,0xA014,0xA014,0xA014},
{0xA015,Lm,0xA015,0xA015,0xA015},
{0xA016,Lo,0xA016,0xA016,0xA016},
{0xA017,Lo,0xA017,0xA017,0xA017},
{0xA018,Lo,0xA018,0xA018,0xA018},
{0xA019,Lo,0xA019,0xA019,0xA019},
{0xA01A,Lo,0xA01A,0xA01A,0xA01A},
{0xA01B,Lo,0xA01B,0xA01B,0xA01B},
{0xA01C,Lo,0xA01C,0xA01C,0xA01C},
{0xA01D,Lo,0xA01D,0xA01D,0xA01D},
{0xA01E,Lo,0xA01E,0xA01E,0xA01E},
{0xA01F,Lo,0xA01F,0xA01F,0xA01F},
{0xA020,Lo,0xA020,0xA020,0xA020},
{0xA021,Lo,0xA021,0xA021,0xA021},
{0xA022,Lo,0xA022,0xA022,0xA022},
{0xA023,Lo,0xA023,0xA023,0xA023},
{0xA024,Lo,0xA024,0xA024,0xA024},
{0xA025,Lo,0xA025,0xA025,0xA025},
{0xA026,Lo,0xA026,0xA026,0xA026},
{0xA027,Lo,0xA027,0xA027,0xA027},
{0xA028,Lo,0xA028,0xA028,0xA028},
{0xA029,Lo,0xA029,0xA029,0xA029},
{0xA02A,Lo,0xA02A,0xA02A,0xA02A},
{0xA02B,Lo,0xA02B,0xA02B,0xA02B},
{0xA02C,Lo,0xA02C,0xA02C,0xA02C},
{0xA02D,Lo,0xA02D,0xA02D,0xA02D},
{0xA02E,Lo,0xA02E,0xA02E,0xA02E},
{0xA02F,Lo,0xA02F,0xA02F,0xA02F},
{0xA030,Lo,0xA030,0xA030,0xA030},
{0xA031,Lo,0xA031,0xA031,0xA031},
{0xA032,Lo,0xA032,0xA032,0xA032},
{0xA033,Lo,0xA033,0xA033,0xA033},
{0xA034,Lo,0xA034,0xA034,0xA034},
{0xA035,Lo,0xA035,0xA035,0xA035},
{0xA036,Lo,0xA036,0xA036,0xA036},
{0xA037,Lo,0xA037,0xA037,0xA037},
{0xA038,Lo,0xA038,0xA038,0xA038},
{0xA039,Lo,0xA039,0xA039,0xA039},
{0xA03A,Lo,0xA03A,0xA03A,0xA03A},
{0xA03B,Lo,0xA03B,0xA03B,0xA03B},
{0xA03C,Lo,0xA03C,0xA03C,0xA03C},
{0xA03D,Lo,0xA03D,0xA03D,0xA03D},
{0xA03E,Lo,0xA03E,0xA03E,0xA03E},
{0xA03F,Lo,0xA03F,0xA03F,0xA03F},
{0xA040,Lo,0xA040,0xA040,0xA040},
{0xA041,Lo,0xA041,0xA041,0xA041},
{0xA042,Lo,0xA042,0xA042,0xA042},
{0xA043,Lo,0xA043,0xA043,0xA043},
{0xA044,Lo,0xA044,0xA044,0xA044},
{0xA045,Lo,0xA045,0xA045,0xA045},
{0xA046,Lo,0xA046,0xA046,0xA046},
{0xA047,Lo,0xA047,0xA047,0xA047},
{0xA048,Lo,0xA048,0xA048,0xA048},
{0xA049,Lo,0xA049,0xA049,0xA049},
{0xA04A,Lo,0xA04A,0xA04A,0xA04A},
{0xA04B,Lo,0xA04B,0xA04B,0xA04B},
{0xA04C,Lo,0xA04C,0xA04C,0xA04C},
{0xA04D,Lo,0xA04D,0xA04D,0xA04D},
{0xA04E,Lo,0xA04E,0xA04E,0xA04E},
{0xA04F,Lo,0xA04F,0xA04F,0xA04F},
{0xA050,Lo,0xA050,0xA050,0xA050},
{0xA051,Lo,0xA051,0xA051,0xA051},
{0xA052,Lo,0xA052,0xA052,0xA052},
{0xA053,Lo,0xA053,0xA053,0xA053},
{0xA054,Lo,0xA054,0xA054,0xA054},
{0xA055,Lo,0xA055,0xA055,0xA055},
{0xA056,Lo,0xA056,0xA056,0xA056},
{0xA057,Lo,0xA057,0xA057,0xA057},
{0xA058,Lo,0xA058,0xA058,0xA058},
{0xA059,Lo,0xA059,0xA059,0xA059},
{0xA05A,Lo,0xA05A,0xA05A,0xA05A},
{0xA05B,Lo,0xA05B,0xA05B,0xA05B},
{0xA05C,Lo,0xA05C,0xA05C,0xA05C},
{0xA05D,Lo,0xA05D,0xA05D,0xA05D},
{0xA05E,Lo,0xA05E,0xA05E,0xA05E},
{0xA05F,Lo,0xA05F,0xA05F,0xA05F},
{0xA060,Lo,0xA060,0xA060,0xA060},
{0xA061,Lo,0xA061,0xA061,0xA061},
{0xA062,Lo,0xA062,0xA062,0xA062},
{0xA063,Lo,0xA063,0xA063,0xA063},
{0xA064,Lo,0xA064,0xA064,0xA064},
{0xA065,Lo,0xA065,0xA065,0xA065},
{0xA066,Lo,0xA066,0xA066,0xA066},
{0xA067,Lo,0xA067,0xA067,0xA067},
{0xA068,Lo,0xA068,0xA068,0xA068},
{0xA069,Lo,0xA069,0xA069,0xA069},
{0xA06A,Lo,0xA06A,0xA06A,0xA06A},
{0xA06B,Lo,0xA06B,0xA06B,0xA06B},
{0xA06C,Lo,0xA06C,0xA06C,0xA06C},
{0xA06D,Lo,0xA06D,0xA06D,0xA06D},
{0xA06E,Lo,0xA06E,0xA06E,0xA06E},
{0xA06F,Lo,0xA06F,0xA06F,0xA06F},
{0xA070,Lo,0xA070,0xA070,0xA070},
{0xA071,Lo,0xA071,0xA071,0xA071},
{0xA072,Lo,0xA072,0xA072,0xA072},
{0xA073,Lo,0xA073,0xA073,0xA073},
{0xA074,Lo,0xA074,0xA074,0xA074},
{0xA075,Lo,0xA075,0xA075,0xA075},
{0xA076,Lo,0xA076,0xA076,0xA076},
{0xA077,Lo,0xA077,0xA077,0xA077},
{0xA078,Lo,0xA078,0xA078,0xA078},
{0xA079,Lo,0xA079,0xA079,0xA079},
{0xA07A,Lo,0xA07A,0xA07A,0xA07A},
{0xA07B,Lo,0xA07B,0xA07B,0xA07B},
{0xA07C,Lo,0xA07C,0xA07C,0xA07C},
{0xA07D,Lo,0xA07D,0xA07D,0xA07D},
{0xA07E,Lo,0xA07E,0xA07E,0xA07E},
{0xA07F,Lo,0xA07F,0xA07F,0xA07F},
{0xA080,Lo,0xA080,0xA080,0xA080},
{0xA081,Lo,0xA081,0xA081,0xA081},
{0xA082,Lo,0xA082,0xA082,0xA082},
{0xA083,Lo,0xA083,0xA083,0xA083},
{0xA084,Lo,0xA084,0xA084,0xA084},
{0xA085,Lo,0xA085,0xA085,0xA085},
{0xA086,Lo,0xA086,0xA086,0xA086},
{0xA087,Lo,0xA087,0xA087,0xA087},
{0xA088,Lo,0xA088,0xA088,0xA088},
{0xA089,Lo,0xA089,0xA089,0xA089},
{0xA08A,Lo,0xA08A,0xA08A,0xA08A},
{0xA08B,Lo,0xA08B,0xA08B,0xA08B},
{0xA08C,Lo,0xA08C,0xA08C,0xA08C},
{0xA08D,Lo,0xA08D,0xA08D,0xA08D},
{0xA08E,Lo,0xA08E,0xA08E,0xA08E},
{0xA08F,Lo,0xA08F,0xA08F,0xA08F},
{0xA090,Lo,0xA090,0xA090,0xA090},
{0xA091,Lo,0xA091,0xA091,0xA091},
{0xA092,Lo,0xA092,0xA092,0xA092},
{0xA093,Lo,0xA093,0xA093,0xA093},
{0xA094,Lo,0xA094,0xA094,0xA094},
{0xA095,Lo,0xA095,0xA095,0xA095},
{0xA096,Lo,0xA096,0xA096,0xA096},
{0xA097,Lo,0xA097,0xA097,0xA097},
{0xA098,Lo,0xA098,0xA098,0xA098},
{0xA099,Lo,0xA099,0xA099,0xA099},
{0xA09A,Lo,0xA09A,0xA09A,0xA09A},
{0xA09B,Lo,0xA09B,0xA09B,0xA09B},
{0xA09C,Lo,0xA09C,0xA09C,0xA09C},
{0xA09D,Lo,0xA09D,0xA09D,0xA09D},
{0xA09E,Lo,0xA09E,0xA09E,0xA09E},
{0xA09F,Lo,0xA09F,0xA09F,0xA09F},
{0xA0A0,Lo,0xA0A0,0xA0A0,0xA0A0},
{0xA0A1,Lo,0xA0A1,0xA0A1,0xA0A1},
{0xA0A2,Lo,0xA0A2,0xA0A2,0xA0A2},
{0xA0A3,Lo,0xA0A3,0xA0A3,0xA0A3},
{0xA0A4,Lo,0xA0A4,0xA0A4,0xA0A4},
{0xA0A5,Lo,0xA0A5,0xA0A5,0xA0A5},
{0xA0A6,Lo,0xA0A6,0xA0A6,0xA0A6},
{0xA0A7,Lo,0xA0A7,0xA0A7,0xA0A7},
{0xA0A8,Lo,0xA0A8,0xA0A8,0xA0A8},
{0xA0A9,Lo,0xA0A9,0xA0A9,0xA0A9},
{0xA0AA,Lo,0xA0AA,0xA0AA,0xA0AA},
{0xA0AB,Lo,0xA0AB,0xA0AB,0xA0AB},
{0xA0AC,Lo,0xA0AC,0xA0AC,0xA0AC},
{0xA0AD,Lo,0xA0AD,0xA0AD,0xA0AD},
{0xA0AE,Lo,0xA0AE,0xA0AE,0xA0AE},
{0xA0AF,Lo,0xA0AF,0xA0AF,0xA0AF},
{0xA0B0,Lo,0xA0B0,0xA0B0,0xA0B0},
{0xA0B1,Lo,0xA0B1,0xA0B1,0xA0B1},
{0xA0B2,Lo,0xA0B2,0xA0B2,0xA0B2},
{0xA0B3,Lo,0xA0B3,0xA0B3,0xA0B3},
{0xA0B4,Lo,0xA0B4,0xA0B4,0xA0B4},
{0xA0B5,Lo,0xA0B5,0xA0B5,0xA0B5},
{0xA0B6,Lo,0xA0B6,0xA0B6,0xA0B6},
{0xA0B7,Lo,0xA0B7,0xA0B7,0xA0B7},
{0xA0B8,Lo,0xA0B8,0xA0B8,0xA0B8},
{0xA0B9,Lo,0xA0B9,0xA0B9,0xA0B9},
{0xA0BA,Lo,0xA0BA,0xA0BA,0xA0BA},
{0xA0BB,Lo,0xA0BB,0xA0BB,0xA0BB},
{0xA0BC,Lo,0xA0BC,0xA0BC,0xA0BC},
{0xA0BD,Lo,0xA0BD,0xA0BD,0xA0BD},
{0xA0BE,Lo,0xA0BE,0xA0BE,0xA0BE},
{0xA0BF,Lo,0xA0BF,0xA0BF,0xA0BF},
{0xA0C0,Lo,0xA0C0,0xA0C0,0xA0C0},
{0xA0C1,Lo,0xA0C1,0xA0C1,0xA0C1},
{0xA0C2,Lo,0xA0C2,0xA0C2,0xA0C2},
{0xA0C3,Lo,0xA0C3,0xA0C3,0xA0C3},
{0xA0C4,Lo,0xA0C4,0xA0C4,0xA0C4},
{0xA0C5,Lo,0xA0C5,0xA0C5,0xA0C5},
{0xA0C6,Lo,0xA0C6,0xA0C6,0xA0C6},
{0xA0C7,Lo,0xA0C7,0xA0C7,0xA0C7},
{0xA0C8,Lo,0xA0C8,0xA0C8,0xA0C8},
{0xA0C9,Lo,0xA0C9,0xA0C9,0xA0C9},
{0xA0CA,Lo,0xA0CA,0xA0CA,0xA0CA},
{0xA0CB,Lo,0xA0CB,0xA0CB,0xA0CB},
{0xA0CC,Lo,0xA0CC,0xA0CC,0xA0CC},
{0xA0CD,Lo,0xA0CD,0xA0CD,0xA0CD},
{0xA0CE,Lo,0xA0CE,0xA0CE,0xA0CE},
{0xA0CF,Lo,0xA0CF,0xA0CF,0xA0CF},
{0xA0D0,Lo,0xA0D0,0xA0D0,0xA0D0},
{0xA0D1,Lo,0xA0D1,0xA0D1,0xA0D1},
{0xA0D2,Lo,0xA0D2,0xA0D2,0xA0D2},
{0xA0D3,Lo,0xA0D3,0xA0D3,0xA0D3},
{0xA0D4,Lo,0xA0D4,0xA0D4,0xA0D4},
{0xA0D5,Lo,0xA0D5,0xA0D5,0xA0D5},
{0xA0D6,Lo,0xA0D6,0xA0D6,0xA0D6},
{0xA0D7,Lo,0xA0D7,0xA0D7,0xA0D7},
{0xA0D8,Lo,0xA0D8,0xA0D8,0xA0D8},
{0xA0D9,Lo,0xA0D9,0xA0D9,0xA0D9},
{0xA0DA,Lo,0xA0DA,0xA0DA,0xA0DA},
{0xA0DB,Lo,0xA0DB,0xA0DB,0xA0DB},
{0xA0DC,Lo,0xA0DC,0xA0DC,0xA0DC},
{0xA0DD,Lo,0xA0DD,0xA0DD,0xA0DD},
{0xA0DE,Lo,0xA0DE,0xA0DE,0xA0DE},
{0xA0DF,Lo,0xA0DF,0xA0DF,0xA0DF},
{0xA0E0,Lo,0xA0E0,0xA0E0,0xA0E0},
{0xA0E1,Lo,0xA0E1,0xA0E1,0xA0E1},
{0xA0E2,Lo,0xA0E2,0xA0E2,0xA0E2},
{0xA0E3,Lo,0xA0E3,0xA0E3,0xA0E3},
{0xA0E4,Lo,0xA0E4,0xA0E4,0xA0E4},
{0xA0E5,Lo,0xA0E5,0xA0E5,0xA0E5},
{0xA0E6,Lo,0xA0E6,0xA0E6,0xA0E6},
{0xA0E7,Lo,0xA0E7,0xA0E7,0xA0E7},
{0xA0E8,Lo,0xA0E8,0xA0E8,0xA0E8},
{0xA0E9,Lo,0xA0E9,0xA0E9,0xA0E9},
{0xA0EA,Lo,0xA0EA,0xA0EA,0xA0EA},
{0xA0EB,Lo,0xA0EB,0xA0EB,0xA0EB},
{0xA0EC,Lo,0xA0EC,0xA0EC,0xA0EC},
{0xA0ED,Lo,0xA0ED,0xA0ED,0xA0ED},
{0xA0EE,Lo,0xA0EE,0xA0EE,0xA0EE},
{0xA0EF,Lo,0xA0EF,0xA0EF,0xA0EF},
{0xA0F0,Lo,0xA0F0,0xA0F0,0xA0F0},
{0xA0F1,Lo,0xA0F1,0xA0F1,0xA0F1},
{0xA0F2,Lo,0xA0F2,0xA0F2,0xA0F2},
{0xA0F3,Lo,0xA0F3,0xA0F3,0xA0F3},
{0xA0F4,Lo,0xA0F4,0xA0F4,0xA0F4},
{0xA0F5,Lo,0xA0F5,0xA0F5,0xA0F5},
{0xA0F6,Lo,0xA0F6,0xA0F6,0xA0F6},
{0xA0F7,Lo,0xA0F7,0xA0F7,0xA0F7},
{0xA0F8,Lo,0xA0F8,0xA0F8,0xA0F8},
{0xA0F9,Lo,0xA0F9,0xA0F9,0xA0F9},
{0xA0FA,Lo,0xA0FA,0xA0FA,0xA0FA},
{0xA0FB,Lo,0xA0FB,0xA0FB,0xA0FB},
{0xA0FC,Lo,0xA0FC,0xA0FC,0xA0FC},
{0xA0FD,Lo,0xA0FD,0xA0FD,0xA0FD},
{0xA0FE,Lo,0xA0FE,0xA0FE,0xA0FE},
{0xA0FF,Lo,0xA0FF,0xA0FF,0xA0FF},
{0xA100,Lo,0xA100,0xA100,0xA100},
{0xA101,Lo,0xA101,0xA101,0xA101},
{0xA102,Lo,0xA102,0xA102,0xA102},
{0xA103,Lo,0xA103,0xA103,0xA103},
{0xA104,Lo,0xA104,0xA104,0xA104},
{0xA105,Lo,0xA105,0xA105,0xA105},
{0xA106,Lo,0xA106,0xA106,0xA106},
{0xA107,Lo,0xA107,0xA107,0xA107},
{0xA108,Lo,0xA108,0xA108,0xA108},
{0xA109,Lo,0xA109,0xA109,0xA109},
{0xA10A,Lo,0xA10A,0xA10A,0xA10A},
{0xA10B,Lo,0xA10B,0xA10B,0xA10B},
{0xA10C,Lo,0xA10C,0xA10C,0xA10C},
{0xA10D,Lo,0xA10D,0xA10D,0xA10D},
{0xA10E,Lo,0xA10E,0xA10E,0xA10E},
{0xA10F,Lo,0xA10F,0xA10F,0xA10F},
{0xA110,Lo,0xA110,0xA110,0xA110},
{0xA111,Lo,0xA111,0xA111,0xA111},
{0xA112,Lo,0xA112,0xA112,0xA112},
{0xA113,Lo,0xA113,0xA113,0xA113},
{0xA114,Lo,0xA114,0xA114,0xA114},
{0xA115,Lo,0xA115,0xA115,0xA115},
{0xA116,Lo,0xA116,0xA116,0xA116},
{0xA117,Lo,0xA117,0xA117,0xA117},
{0xA118,Lo,0xA118,0xA118,0xA118},
{0xA119,Lo,0xA119,0xA119,0xA119},
{0xA11A,Lo,0xA11A,0xA11A,0xA11A},
{0xA11B,Lo,0xA11B,0xA11B,0xA11B},
{0xA11C,Lo,0xA11C,0xA11C,0xA11C},
{0xA11D,Lo,0xA11D,0xA11D,0xA11D},
{0xA11E,Lo,0xA11E,0xA11E,0xA11E},
{0xA11F,Lo,0xA11F,0xA11F,0xA11F},
{0xA120,Lo,0xA120,0xA120,0xA120},
{0xA121,Lo,0xA121,0xA121,0xA121},
{0xA122,Lo,0xA122,0xA122,0xA122},
{0xA123,Lo,0xA123,0xA123,0xA123},
{0xA124,Lo,0xA124,0xA124,0xA124},
{0xA125,Lo,0xA125,0xA125,0xA125},
{0xA126,Lo,0xA126,0xA126,0xA126},
{0xA127,Lo,0xA127,0xA127,0xA127},
{0xA128,Lo,0xA128,0xA128,0xA128},
{0xA129,Lo,0xA129,0xA129,0xA129},
{0xA12A,Lo,0xA12A,0xA12A,0xA12A},
{0xA12B,Lo,0xA12B,0xA12B,0xA12B},
{0xA12C,Lo,0xA12C,0xA12C,0xA12C},
{0xA12D,Lo,0xA12D,0xA12D,0xA12D},
{0xA12E,Lo,0xA12E,0xA12E,0xA12E},
{0xA12F,Lo,0xA12F,0xA12F,0xA12F},
{0xA130,Lo,0xA130,0xA130,0xA130},
{0xA131,Lo,0xA131,0xA131,0xA131},
{0xA132,Lo,0xA132,0xA132,0xA132},
{0xA133,Lo,0xA133,0xA133,0xA133},
{0xA134,Lo,0xA134,0xA134,0xA134},
{0xA135,Lo,0xA135,0xA135,0xA135},
{0xA136,Lo,0xA136,0xA136,0xA136},
{0xA137,Lo,0xA137,0xA137,0xA137},
{0xA138,Lo,0xA138,0xA138,0xA138},
{0xA139,Lo,0xA139,0xA139,0xA139},
{0xA13A,Lo,0xA13A,0xA13A,0xA13A},
{0xA13B,Lo,0xA13B,0xA13B,0xA13B},
{0xA13C,Lo,0xA13C,0xA13C,0xA13C},
{0xA13D,Lo,0xA13D,0xA13D,0xA13D},
{0xA13E,Lo,0xA13E,0xA13E,0xA13E},
{0xA13F,Lo,0xA13F,0xA13F,0xA13F},
{0xA140,Lo,0xA140,0xA140,0xA140},
{0xA141,Lo,0xA141,0xA141,0xA141},
{0xA142,Lo,0xA142,0xA142,0xA142},
{0xA143,Lo,0xA143,0xA143,0xA143},
{0xA144,Lo,0xA144,0xA144,0xA144},
{0xA145,Lo,0xA145,0xA145,0xA145},
{0xA146,Lo,0xA146,0xA146,0xA146},
{0xA147,Lo,0xA147,0xA147,0xA147},
{0xA148,Lo,0xA148,0xA148,0xA148},
{0xA149,Lo,0xA149,0xA149,0xA149},
{0xA14A,Lo,0xA14A,0xA14A,0xA14A},
{0xA14B,Lo,0xA14B,0xA14B,0xA14B},
{0xA14C,Lo,0xA14C,0xA14C,0xA14C},
{0xA14D,Lo,0xA14D,0xA14D,0xA14D},
{0xA14E,Lo,0xA14E,0xA14E,0xA14E},
{0xA14F,Lo,0xA14F,0xA14F,0xA14F},
{0xA150,Lo,0xA150,0xA150,0xA150},
{0xA151,Lo,0xA151,0xA151,0xA151},
{0xA152,Lo,0xA152,0xA152,0xA152},
{0xA153,Lo,0xA153,0xA153,0xA153},
{0xA154,Lo,0xA154,0xA154,0xA154},
{0xA155,Lo,0xA155,0xA155,0xA155},
{0xA156,Lo,0xA156,0xA156,0xA156},
{0xA157,Lo,0xA157,0xA157,0xA157},
{0xA158,Lo,0xA158,0xA158,0xA158},
{0xA159,Lo,0xA159,0xA159,0xA159},
{0xA15A,Lo,0xA15A,0xA15A,0xA15A},
{0xA15B,Lo,0xA15B,0xA15B,0xA15B},
{0xA15C,Lo,0xA15C,0xA15C,0xA15C},
{0xA15D,Lo,0xA15D,0xA15D,0xA15D},
{0xA15E,Lo,0xA15E,0xA15E,0xA15E},
{0xA15F,Lo,0xA15F,0xA15F,0xA15F},
{0xA160,Lo,0xA160,0xA160,0xA160},
{0xA161,Lo,0xA161,0xA161,0xA161},
{0xA162,Lo,0xA162,0xA162,0xA162},
{0xA163,Lo,0xA163,0xA163,0xA163},
{0xA164,Lo,0xA164,0xA164,0xA164},
{0xA165,Lo,0xA165,0xA165,0xA165},
{0xA166,Lo,0xA166,0xA166,0xA166},
{0xA167,Lo,0xA167,0xA167,0xA167},
{0xA168,Lo,0xA168,0xA168,0xA168},
{0xA169,Lo,0xA169,0xA169,0xA169},
{0xA16A,Lo,0xA16A,0xA16A,0xA16A},
{0xA16B,Lo,0xA16B,0xA16B,0xA16B},
{0xA16C,Lo,0xA16C,0xA16C,0xA16C},
{0xA16D,Lo,0xA16D,0xA16D,0xA16D},
{0xA16E,Lo,0xA16E,0xA16E,0xA16E},
{0xA16F,Lo,0xA16F,0xA16F,0xA16F},
{0xA170,Lo,0xA170,0xA170,0xA170},
{0xA171,Lo,0xA171,0xA171,0xA171},
{0xA172,Lo,0xA172,0xA172,0xA172},
{0xA173,Lo,0xA173,0xA173,0xA173},
{0xA174,Lo,0xA174,0xA174,0xA174},
{0xA175,Lo,0xA175,0xA175,0xA175},
{0xA176,Lo,0xA176,0xA176,0xA176},
{0xA177,Lo,0xA177,0xA177,0xA177},
{0xA178,Lo,0xA178,0xA178,0xA178},
{0xA179,Lo,0xA179,0xA179,0xA179},
{0xA17A,Lo,0xA17A,0xA17A,0xA17A},
{0xA17B,Lo,0xA17B,0xA17B,0xA17B},
{0xA17C,Lo,0xA17C,0xA17C,0xA17C},
{0xA17D,Lo,0xA17D,0xA17D,0xA17D},
{0xA17E,Lo,0xA17E,0xA17E,0xA17E},
{0xA17F,Lo,0xA17F,0xA17F,0xA17F},
{0xA180,Lo,0xA180,0xA180,0xA180},
{0xA181,Lo,0xA181,0xA181,0xA181},
{0xA182,Lo,0xA182,0xA182,0xA182},
{0xA183,Lo,0xA183,0xA183,0xA183},
{0xA184,Lo,0xA184,0xA184,0xA184},
{0xA185,Lo,0xA185,0xA185,0xA185},
{0xA186,Lo,0xA186,0xA186,0xA186},
{0xA187,Lo,0xA187,0xA187,0xA187},
{0xA188,Lo,0xA188,0xA188,0xA188},
{0xA189,Lo,0xA189,0xA189,0xA189},
{0xA18A,Lo,0xA18A,0xA18A,0xA18A},
{0xA18B,Lo,0xA18B,0xA18B,0xA18B},
{0xA18C,Lo,0xA18C,0xA18C,0xA18C},
{0xA18D,Lo,0xA18D,0xA18D,0xA18D},
{0xA18E,Lo,0xA18E,0xA18E,0xA18E},
{0xA18F,Lo,0xA18F,0xA18F,0xA18F},
{0xA190,Lo,0xA190,0xA190,0xA190},
{0xA191,Lo,0xA191,0xA191,0xA191},
{0xA192,Lo,0xA192,0xA192,0xA192},
{0xA193,Lo,0xA193,0xA193,0xA193},
{0xA194,Lo,0xA194,0xA194,0xA194},
{0xA195,Lo,0xA195,0xA195,0xA195},
{0xA196,Lo,0xA196,0xA196,0xA196},
{0xA197,Lo,0xA197,0xA197,0xA197},
{0xA198,Lo,0xA198,0xA198,0xA198},
{0xA199,Lo,0xA199,0xA199,0xA199},
{0xA19A,Lo,0xA19A,0xA19A,0xA19A},
{0xA19B,Lo,0xA19B,0xA19B,0xA19B},
{0xA19C,Lo,0xA19C,0xA19C,0xA19C},
{0xA19D,Lo,0xA19D,0xA19D,0xA19D},
{0xA19E,Lo,0xA19E,0xA19E,0xA19E},
{0xA19F,Lo,0xA19F,0xA19F,0xA19F},
{0xA1A0,Lo,0xA1A0,0xA1A0,0xA1A0},
{0xA1A1,Lo,0xA1A1,0xA1A1,0xA1A1},
{0xA1A2,Lo,0xA1A2,0xA1A2,0xA1A2},
{0xA1A3,Lo,0xA1A3,0xA1A3,0xA1A3},
{0xA1A4,Lo,0xA1A4,0xA1A4,0xA1A4},
{0xA1A5,Lo,0xA1A5,0xA1A5,0xA1A5},
{0xA1A6,Lo,0xA1A6,0xA1A6,0xA1A6},
{0xA1A7,Lo,0xA1A7,0xA1A7,0xA1A7},
{0xA1A8,Lo,0xA1A8,0xA1A8,0xA1A8},
{0xA1A9,Lo,0xA1A9,0xA1A9,0xA1A9},
{0xA1AA,Lo,0xA1AA,0xA1AA,0xA1AA},
{0xA1AB,Lo,0xA1AB,0xA1AB,0xA1AB},
{0xA1AC,Lo,0xA1AC,0xA1AC,0xA1AC},
{0xA1AD,Lo,0xA1AD,0xA1AD,0xA1AD},
{0xA1AE,Lo,0xA1AE,0xA1AE,0xA1AE},
{0xA1AF,Lo,0xA1AF,0xA1AF,0xA1AF},
{0xA1B0,Lo,0xA1B0,0xA1B0,0xA1B0},
{0xA1B1,Lo,0xA1B1,0xA1B1,0xA1B1},
{0xA1B2,Lo,0xA1B2,0xA1B2,0xA1B2},
{0xA1B3,Lo,0xA1B3,0xA1B3,0xA1B3},
{0xA1B4,Lo,0xA1B4,0xA1B4,0xA1B4},
{0xA1B5,Lo,0xA1B5,0xA1B5,0xA1B5},
{0xA1B6,Lo,0xA1B6,0xA1B6,0xA1B6},
{0xA1B7,Lo,0xA1B7,0xA1B7,0xA1B7},
{0xA1B8,Lo,0xA1B8,0xA1B8,0xA1B8},
{0xA1B9,Lo,0xA1B9,0xA1B9,0xA1B9},
{0xA1BA,Lo,0xA1BA,0xA1BA,0xA1BA},
{0xA1BB,Lo,0xA1BB,0xA1BB,0xA1BB},
{0xA1BC,Lo,0xA1BC,0xA1BC,0xA1BC},
{0xA1BD,Lo,0xA1BD,0xA1BD,0xA1BD},
{0xA1BE,Lo,0xA1BE,0xA1BE,0xA1BE},
{0xA1BF,Lo,0xA1BF,0xA1BF,0xA1BF},
{0xA1C0,Lo,0xA1C0,0xA1C0,0xA1C0},
{0xA1C1,Lo,0xA1C1,0xA1C1,0xA1C1},
{0xA1C2,Lo,0xA1C2,0xA1C2,0xA1C2},
{0xA1C3,Lo,0xA1C3,0xA1C3,0xA1C3},
{0xA1C4,Lo,0xA1C4,0xA1C4,0xA1C4},
{0xA1C5,Lo,0xA1C5,0xA1C5,0xA1C5},
{0xA1C6,Lo,0xA1C6,0xA1C6,0xA1C6},
{0xA1C7,Lo,0xA1C7,0xA1C7,0xA1C7},
{0xA1C8,Lo,0xA1C8,0xA1C8,0xA1C8},
{0xA1C9,Lo,0xA1C9,0xA1C9,0xA1C9},
{0xA1CA,Lo,0xA1CA,0xA1CA,0xA1CA},
{0xA1CB,Lo,0xA1CB,0xA1CB,0xA1CB},
{0xA1CC,Lo,0xA1CC,0xA1CC,0xA1CC},
{0xA1CD,Lo,0xA1CD,0xA1CD,0xA1CD},
{0xA1CE,Lo,0xA1CE,0xA1CE,0xA1CE},
{0xA1CF,Lo,0xA1CF,0xA1CF,0xA1CF},
{0xA1D0,Lo,0xA1D0,0xA1D0,0xA1D0},
{0xA1D1,Lo,0xA1D1,0xA1D1,0xA1D1},
{0xA1D2,Lo,0xA1D2,0xA1D2,0xA1D2},
{0xA1D3,Lo,0xA1D3,0xA1D3,0xA1D3},
{0xA1D4,Lo,0xA1D4,0xA1D4,0xA1D4},
{0xA1D5,Lo,0xA1D5,0xA1D5,0xA1D5},
{0xA1D6,Lo,0xA1D6,0xA1D6,0xA1D6},
{0xA1D7,Lo,0xA1D7,0xA1D7,0xA1D7},
{0xA1D8,Lo,0xA1D8,0xA1D8,0xA1D8},
{0xA1D9,Lo,0xA1D9,0xA1D9,0xA1D9},
{0xA1DA,Lo,0xA1DA,0xA1DA,0xA1DA},
{0xA1DB,Lo,0xA1DB,0xA1DB,0xA1DB},
{0xA1DC,Lo,0xA1DC,0xA1DC,0xA1DC},
{0xA1DD,Lo,0xA1DD,0xA1DD,0xA1DD},
{0xA1DE,Lo,0xA1DE,0xA1DE,0xA1DE},
{0xA1DF,Lo,0xA1DF,0xA1DF,0xA1DF},
{0xA1E0,Lo,0xA1E0,0xA1E0,0xA1E0},
{0xA1E1,Lo,0xA1E1,0xA1E1,0xA1E1},
{0xA1E2,Lo,0xA1E2,0xA1E2,0xA1E2},
{0xA1E3,Lo,0xA1E3,0xA1E3,0xA1E3},
{0xA1E4,Lo,0xA1E4,0xA1E4,0xA1E4},
{0xA1E5,Lo,0xA1E5,0xA1E5,0xA1E5},
{0xA1E6,Lo,0xA1E6,0xA1E6,0xA1E6},
{0xA1E7,Lo,0xA1E7,0xA1E7,0xA1E7},
{0xA1E8,Lo,0xA1E8,0xA1E8,0xA1E8},
{0xA1E9,Lo,0xA1E9,0xA1E9,0xA1E9},
{0xA1EA,Lo,0xA1EA,0xA1EA,0xA1EA},
{0xA1EB,Lo,0xA1EB,0xA1EB,0xA1EB},
{0xA1EC,Lo,0xA1EC,0xA1EC,0xA1EC},
{0xA1ED,Lo,0xA1ED,0xA1ED,0xA1ED},
{0xA1EE,Lo,0xA1EE,0xA1EE,0xA1EE},
{0xA1EF,Lo,0xA1EF,0xA1EF,0xA1EF},
{0xA1F0,Lo,0xA1F0,0xA1F0,0xA1F0},
{0xA1F1,Lo,0xA1F1,0xA1F1,0xA1F1},
{0xA1F2,Lo,0xA1F2,0xA1F2,0xA1F2},
{0xA1F3,Lo,0xA1F3,0xA1F3,0xA1F3},
{0xA1F4,Lo,0xA1F4,0xA1F4,0xA1F4},
{0xA1F5,Lo,0xA1F5,0xA1F5,0xA1F5},
{0xA1F6,Lo,0xA1F6,0xA1F6,0xA1F6},
{0xA1F7,Lo,0xA1F7,0xA1F7,0xA1F7},
{0xA1F8,Lo,0xA1F8,0xA1F8,0xA1F8},
{0xA1F9,Lo,0xA1F9,0xA1F9,0xA1F9},
{0xA1FA,Lo,0xA1FA,0xA1FA,0xA1FA},
{0xA1FB,Lo,0xA1FB,0xA1FB,0xA1FB},
{0xA1FC,Lo,0xA1FC,0xA1FC,0xA1FC},
{0xA1FD,Lo,0xA1FD,0xA1FD,0xA1FD},
{0xA1FE,Lo,0xA1FE,0xA1FE,0xA1FE},
{0xA1FF,Lo,0xA1FF,0xA1FF,0xA1FF},
{0xA200,Lo,0xA200,0xA200,0xA200},
{0xA201,Lo,0xA201,0xA201,0xA201},
{0xA202,Lo,0xA202,0xA202,0xA202},
{0xA203,Lo,0xA203,0xA203,0xA203},
{0xA204,Lo,0xA204,0xA204,0xA204},
{0xA205,Lo,0xA205,0xA205,0xA205},
{0xA206,Lo,0xA206,0xA206,0xA206},
{0xA207,Lo,0xA207,0xA207,0xA207},
{0xA208,Lo,0xA208,0xA208,0xA208},
{0xA209,Lo,0xA209,0xA209,0xA209},
{0xA20A,Lo,0xA20A,0xA20A,0xA20A},
{0xA20B,Lo,0xA20B,0xA20B,0xA20B},
{0xA20C,Lo,0xA20C,0xA20C,0xA20C},
{0xA20D,Lo,0xA20D,0xA20D,0xA20D},
{0xA20E,Lo,0xA20E,0xA20E,0xA20E},
{0xA20F,Lo,0xA20F,0xA20F,0xA20F},
{0xA210,Lo,0xA210,0xA210,0xA210},
{0xA211,Lo,0xA211,0xA211,0xA211},
{0xA212,Lo,0xA212,0xA212,0xA212},
{0xA213,Lo,0xA213,0xA213,0xA213},
{0xA214,Lo,0xA214,0xA214,0xA214},
{0xA215,Lo,0xA215,0xA215,0xA215},
{0xA216,Lo,0xA216,0xA216,0xA216},
{0xA217,Lo,0xA217,0xA217,0xA217},
{0xA218,Lo,0xA218,0xA218,0xA218},
{0xA219,Lo,0xA219,0xA219,0xA219},
{0xA21A,Lo,0xA21A,0xA21A,0xA21A},
{0xA21B,Lo,0xA21B,0xA21B,0xA21B},
{0xA21C,Lo,0xA21C,0xA21C,0xA21C},
{0xA21D,Lo,0xA21D,0xA21D,0xA21D},
{0xA21E,Lo,0xA21E,0xA21E,0xA21E},
{0xA21F,Lo,0xA21F,0xA21F,0xA21F},
{0xA220,Lo,0xA220,0xA220,0xA220},
{0xA221,Lo,0xA221,0xA221,0xA221},
{0xA222,Lo,0xA222,0xA222,0xA222},
{0xA223,Lo,0xA223,0xA223,0xA223},
{0xA224,Lo,0xA224,0xA224,0xA224},
{0xA225,Lo,0xA225,0xA225,0xA225},
{0xA226,Lo,0xA226,0xA226,0xA226},
{0xA227,Lo,0xA227,0xA227,0xA227},
{0xA228,Lo,0xA228,0xA228,0xA228},
{0xA229,Lo,0xA229,0xA229,0xA229},
{0xA22A,Lo,0xA22A,0xA22A,0xA22A},
{0xA22B,Lo,0xA22B,0xA22B,0xA22B},
{0xA22C,Lo,0xA22C,0xA22C,0xA22C},
{0xA22D,Lo,0xA22D,0xA22D,0xA22D},
{0xA22E,Lo,0xA22E,0xA22E,0xA22E},
{0xA22F,Lo,0xA22F,0xA22F,0xA22F},
{0xA230,Lo,0xA230,0xA230,0xA230},
{0xA231,Lo,0xA231,0xA231,0xA231},
{0xA232,Lo,0xA232,0xA232,0xA232},
{0xA233,Lo,0xA233,0xA233,0xA233},
{0xA234,Lo,0xA234,0xA234,0xA234},
{0xA235,Lo,0xA235,0xA235,0xA235},
{0xA236,Lo,0xA236,0xA236,0xA236},
{0xA237,Lo,0xA237,0xA237,0xA237},
{0xA238,Lo,0xA238,0xA238,0xA238},
{0xA239,Lo,0xA239,0xA239,0xA239},
{0xA23A,Lo,0xA23A,0xA23A,0xA23A},
{0xA23B,Lo,0xA23B,0xA23B,0xA23B},
{0xA23C,Lo,0xA23C,0xA23C,0xA23C},
{0xA23D,Lo,0xA23D,0xA23D,0xA23D},
{0xA23E,Lo,0xA23E,0xA23E,0xA23E},
{0xA23F,Lo,0xA23F,0xA23F,0xA23F},
{0xA240,Lo,0xA240,0xA240,0xA240},
{0xA241,Lo,0xA241,0xA241,0xA241},
{0xA242,Lo,0xA242,0xA242,0xA242},
{0xA243,Lo,0xA243,0xA243,0xA243},
{0xA244,Lo,0xA244,0xA244,0xA244},
{0xA245,Lo,0xA245,0xA245,0xA245},
{0xA246,Lo,0xA246,0xA246,0xA246},
{0xA247,Lo,0xA247,0xA247,0xA247},
{0xA248,Lo,0xA248,0xA248,0xA248},
{0xA249,Lo,0xA249,0xA249,0xA249},
{0xA24A,Lo,0xA24A,0xA24A,0xA24A},
{0xA24B,Lo,0xA24B,0xA24B,0xA24B},
{0xA24C,Lo,0xA24C,0xA24C,0xA24C},
{0xA24D,Lo,0xA24D,0xA24D,0xA24D},
{0xA24E,Lo,0xA24E,0xA24E,0xA24E},
{0xA24F,Lo,0xA24F,0xA24F,0xA24F},
{0xA250,Lo,0xA250,0xA250,0xA250},
{0xA251,Lo,0xA251,0xA251,0xA251},
{0xA252,Lo,0xA252,0xA252,0xA252},
{0xA253,Lo,0xA253,0xA253,0xA253},
{0xA254,Lo,0xA254,0xA254,0xA254},
{0xA255,Lo,0xA255,0xA255,0xA255},
{0xA256,Lo,0xA256,0xA256,0xA256},
{0xA257,Lo,0xA257,0xA257,0xA257},
{0xA258,Lo,0xA258,0xA258,0xA258},
{0xA259,Lo,0xA259,0xA259,0xA259},
{0xA25A,Lo,0xA25A,0xA25A,0xA25A},
{0xA25B,Lo,0xA25B,0xA25B,0xA25B},
{0xA25C,Lo,0xA25C,0xA25C,0xA25C},
{0xA25D,Lo,0xA25D,0xA25D,0xA25D},
{0xA25E,Lo,0xA25E,0xA25E,0xA25E},
{0xA25F,Lo,0xA25F,0xA25F,0xA25F},
{0xA260,Lo,0xA260,0xA260,0xA260},
{0xA261,Lo,0xA261,0xA261,0xA261},
{0xA262,Lo,0xA262,0xA262,0xA262},
{0xA263,Lo,0xA263,0xA263,0xA263},
{0xA264,Lo,0xA264,0xA264,0xA264},
{0xA265,Lo,0xA265,0xA265,0xA265},
{0xA266,Lo,0xA266,0xA266,0xA266},
{0xA267,Lo,0xA267,0xA267,0xA267},
{0xA268,Lo,0xA268,0xA268,0xA268},
{0xA269,Lo,0xA269,0xA269,0xA269},
{0xA26A,Lo,0xA26A,0xA26A,0xA26A},
{0xA26B,Lo,0xA26B,0xA26B,0xA26B},
{0xA26C,Lo,0xA26C,0xA26C,0xA26C},
{0xA26D,Lo,0xA26D,0xA26D,0xA26D},
{0xA26E,Lo,0xA26E,0xA26E,0xA26E},
{0xA26F,Lo,0xA26F,0xA26F,0xA26F},
{0xA270,Lo,0xA270,0xA270,0xA270},
{0xA271,Lo,0xA271,0xA271,0xA271},
{0xA272,Lo,0xA272,0xA272,0xA272},
{0xA273,Lo,0xA273,0xA273,0xA273},
{0xA274,Lo,0xA274,0xA274,0xA274},
{0xA275,Lo,0xA275,0xA275,0xA275},
{0xA276,Lo,0xA276,0xA276,0xA276},
{0xA277,Lo,0xA277,0xA277,0xA277},
{0xA278,Lo,0xA278,0xA278,0xA278},
{0xA279,Lo,0xA279,0xA279,0xA279},
{0xA27A,Lo,0xA27A,0xA27A,0xA27A},
{0xA27B,Lo,0xA27B,0xA27B,0xA27B},
{0xA27C,Lo,0xA27C,0xA27C,0xA27C},
{0xA27D,Lo,0xA27D,0xA27D,0xA27D},
{0xA27E,Lo,0xA27E,0xA27E,0xA27E},
{0xA27F,Lo,0xA27F,0xA27F,0xA27F},
{0xA280,Lo,0xA280,0xA280,0xA280},
{0xA281,Lo,0xA281,0xA281,0xA281},
{0xA282,Lo,0xA282,0xA282,0xA282},
{0xA283,Lo,0xA283,0xA283,0xA283},
{0xA284,Lo,0xA284,0xA284,0xA284},
{0xA285,Lo,0xA285,0xA285,0xA285},
{0xA286,Lo,0xA286,0xA286,0xA286},
{0xA287,Lo,0xA287,0xA287,0xA287},
{0xA288,Lo,0xA288,0xA288,0xA288},
{0xA289,Lo,0xA289,0xA289,0xA289},
{0xA28A,Lo,0xA28A,0xA28A,0xA28A},
{0xA28B,Lo,0xA28B,0xA28B,0xA28B},
{0xA28C,Lo,0xA28C,0xA28C,0xA28C},
{0xA28D,Lo,0xA28D,0xA28D,0xA28D},
{0xA28E,Lo,0xA28E,0xA28E,0xA28E},
{0xA28F,Lo,0xA28F,0xA28F,0xA28F},
{0xA290,Lo,0xA290,0xA290,0xA290},
{0xA291,Lo,0xA291,0xA291,0xA291},
{0xA292,Lo,0xA292,0xA292,0xA292},
{0xA293,Lo,0xA293,0xA293,0xA293},
{0xA294,Lo,0xA294,0xA294,0xA294},
{0xA295,Lo,0xA295,0xA295,0xA295},
{0xA296,Lo,0xA296,0xA296,0xA296},
{0xA297,Lo,0xA297,0xA297,0xA297},
{0xA298,Lo,0xA298,0xA298,0xA298},
{0xA299,Lo,0xA299,0xA299,0xA299},
{0xA29A,Lo,0xA29A,0xA29A,0xA29A},
{0xA29B,Lo,0xA29B,0xA29B,0xA29B},
{0xA29C,Lo,0xA29C,0xA29C,0xA29C},
{0xA29D,Lo,0xA29D,0xA29D,0xA29D},
{0xA29E,Lo,0xA29E,0xA29E,0xA29E},
{0xA29F,Lo,0xA29F,0xA29F,0xA29F},
{0xA2A0,Lo,0xA2A0,0xA2A0,0xA2A0},
{0xA2A1,Lo,0xA2A1,0xA2A1,0xA2A1},
{0xA2A2,Lo,0xA2A2,0xA2A2,0xA2A2},
{0xA2A3,Lo,0xA2A3,0xA2A3,0xA2A3},
{0xA2A4,Lo,0xA2A4,0xA2A4,0xA2A4},
{0xA2A5,Lo,0xA2A5,0xA2A5,0xA2A5},
{0xA2A6,Lo,0xA2A6,0xA2A6,0xA2A6},
{0xA2A7,Lo,0xA2A7,0xA2A7,0xA2A7},
{0xA2A8,Lo,0xA2A8,0xA2A8,0xA2A8},
{0xA2A9,Lo,0xA2A9,0xA2A9,0xA2A9},
{0xA2AA,Lo,0xA2AA,0xA2AA,0xA2AA},
{0xA2AB,Lo,0xA2AB,0xA2AB,0xA2AB},
{0xA2AC,Lo,0xA2AC,0xA2AC,0xA2AC},
{0xA2AD,Lo,0xA2AD,0xA2AD,0xA2AD},
{0xA2AE,Lo,0xA2AE,0xA2AE,0xA2AE},
{0xA2AF,Lo,0xA2AF,0xA2AF,0xA2AF},
{0xA2B0,Lo,0xA2B0,0xA2B0,0xA2B0},
{0xA2B1,Lo,0xA2B1,0xA2B1,0xA2B1},
{0xA2B2,Lo,0xA2B2,0xA2B2,0xA2B2},
{0xA2B3,Lo,0xA2B3,0xA2B3,0xA2B3},
{0xA2B4,Lo,0xA2B4,0xA2B4,0xA2B4},
{0xA2B5,Lo,0xA2B5,0xA2B5,0xA2B5},
{0xA2B6,Lo,0xA2B6,0xA2B6,0xA2B6},
{0xA2B7,Lo,0xA2B7,0xA2B7,0xA2B7},
{0xA2B8,Lo,0xA2B8,0xA2B8,0xA2B8},
{0xA2B9,Lo,0xA2B9,0xA2B9,0xA2B9},
{0xA2BA,Lo,0xA2BA,0xA2BA,0xA2BA},
{0xA2BB,Lo,0xA2BB,0xA2BB,0xA2BB},
{0xA2BC,Lo,0xA2BC,0xA2BC,0xA2BC},
{0xA2BD,Lo,0xA2BD,0xA2BD,0xA2BD},
{0xA2BE,Lo,0xA2BE,0xA2BE,0xA2BE},
{0xA2BF,Lo,0xA2BF,0xA2BF,0xA2BF},
{0xA2C0,Lo,0xA2C0,0xA2C0,0xA2C0},
{0xA2C1,Lo,0xA2C1,0xA2C1,0xA2C1},
{0xA2C2,Lo,0xA2C2,0xA2C2,0xA2C2},
{0xA2C3,Lo,0xA2C3,0xA2C3,0xA2C3},
{0xA2C4,Lo,0xA2C4,0xA2C4,0xA2C4},
{0xA2C5,Lo,0xA2C5,0xA2C5,0xA2C5},
{0xA2C6,Lo,0xA2C6,0xA2C6,0xA2C6},
{0xA2C7,Lo,0xA2C7,0xA2C7,0xA2C7},
{0xA2C8,Lo,0xA2C8,0xA2C8,0xA2C8},
{0xA2C9,Lo,0xA2C9,0xA2C9,0xA2C9},
{0xA2CA,Lo,0xA2CA,0xA2CA,0xA2CA},
{0xA2CB,Lo,0xA2CB,0xA2CB,0xA2CB},
{0xA2CC,Lo,0xA2CC,0xA2CC,0xA2CC},
{0xA2CD,Lo,0xA2CD,0xA2CD,0xA2CD},
{0xA2CE,Lo,0xA2CE,0xA2CE,0xA2CE},
{0xA2CF,Lo,0xA2CF,0xA2CF,0xA2CF},
{0xA2D0,Lo,0xA2D0,0xA2D0,0xA2D0},
{0xA2D1,Lo,0xA2D1,0xA2D1,0xA2D1},
{0xA2D2,Lo,0xA2D2,0xA2D2,0xA2D2},
{0xA2D3,Lo,0xA2D3,0xA2D3,0xA2D3},
{0xA2D4,Lo,0xA2D4,0xA2D4,0xA2D4},
{0xA2D5,Lo,0xA2D5,0xA2D5,0xA2D5},
{0xA2D6,Lo,0xA2D6,0xA2D6,0xA2D6},
{0xA2D7,Lo,0xA2D7,0xA2D7,0xA2D7},
{0xA2D8,Lo,0xA2D8,0xA2D8,0xA2D8},
{0xA2D9,Lo,0xA2D9,0xA2D9,0xA2D9},
{0xA2DA,Lo,0xA2DA,0xA2DA,0xA2DA},
{0xA2DB,Lo,0xA2DB,0xA2DB,0xA2DB},
{0xA2DC,Lo,0xA2DC,0xA2DC,0xA2DC},
{0xA2DD,Lo,0xA2DD,0xA2DD,0xA2DD},
{0xA2DE,Lo,0xA2DE,0xA2DE,0xA2DE},
{0xA2DF,Lo,0xA2DF,0xA2DF,0xA2DF},
{0xA2E0,Lo,0xA2E0,0xA2E0,0xA2E0},
{0xA2E1,Lo,0xA2E1,0xA2E1,0xA2E1},
{0xA2E2,Lo,0xA2E2,0xA2E2,0xA2E2},
{0xA2E3,Lo,0xA2E3,0xA2E3,0xA2E3},
{0xA2E4,Lo,0xA2E4,0xA2E4,0xA2E4},
{0xA2E5,Lo,0xA2E5,0xA2E5,0xA2E5},
{0xA2E6,Lo,0xA2E6,0xA2E6,0xA2E6},
{0xA2E7,Lo,0xA2E7,0xA2E7,0xA2E7},
{0xA2E8,Lo,0xA2E8,0xA2E8,0xA2E8},
{0xA2E9,Lo,0xA2E9,0xA2E9,0xA2E9},
{0xA2EA,Lo,0xA2EA,0xA2EA,0xA2EA},
{0xA2EB,Lo,0xA2EB,0xA2EB,0xA2EB},
{0xA2EC,Lo,0xA2EC,0xA2EC,0xA2EC},
{0xA2ED,Lo,0xA2ED,0xA2ED,0xA2ED},
{0xA2EE,Lo,0xA2EE,0xA2EE,0xA2EE},
{0xA2EF,Lo,0xA2EF,0xA2EF,0xA2EF},
{0xA2F0,Lo,0xA2F0,0xA2F0,0xA2F0},
{0xA2F1,Lo,0xA2F1,0xA2F1,0xA2F1},
{0xA2F2,Lo,0xA2F2,0xA2F2,0xA2F2},
{0xA2F3,Lo,0xA2F3,0xA2F3,0xA2F3},
{0xA2F4,Lo,0xA2F4,0xA2F4,0xA2F4},
{0xA2F5,Lo,0xA2F5,0xA2F5,0xA2F5},
{0xA2F6,Lo,0xA2F6,0xA2F6,0xA2F6},
{0xA2F7,Lo,0xA2F7,0xA2F7,0xA2F7},
{0xA2F8,Lo,0xA2F8,0xA2F8,0xA2F8},
{0xA2F9,Lo,0xA2F9,0xA2F9,0xA2F9},
{0xA2FA,Lo,0xA2FA,0xA2FA,0xA2FA},
{0xA2FB,Lo,0xA2FB,0xA2FB,0xA2FB},
{0xA2FC,Lo,0xA2FC,0xA2FC,0xA2FC},
{0xA2FD,Lo,0xA2FD,0xA2FD,0xA2FD},
{0xA2FE,Lo,0xA2FE,0xA2FE,0xA2FE},
{0xA2FF,Lo,0xA2FF,0xA2FF,0xA2FF},
{0xA300,Lo,0xA300,0xA300,0xA300},
{0xA301,Lo,0xA301,0xA301,0xA301},
{0xA302,Lo,0xA302,0xA302,0xA302},
{0xA303,Lo,0xA303,0xA303,0xA303},
{0xA304,Lo,0xA304,0xA304,0xA304},
{0xA305,Lo,0xA305,0xA305,0xA305},
{0xA306,Lo,0xA306,0xA306,0xA306},
{0xA307,Lo,0xA307,0xA307,0xA307},
{0xA308,Lo,0xA308,0xA308,0xA308},
{0xA309,Lo,0xA309,0xA309,0xA309},
{0xA30A,Lo,0xA30A,0xA30A,0xA30A},
{0xA30B,Lo,0xA30B,0xA30B,0xA30B},
{0xA30C,Lo,0xA30C,0xA30C,0xA30C},
{0xA30D,Lo,0xA30D,0xA30D,0xA30D},
{0xA30E,Lo,0xA30E,0xA30E,0xA30E},
{0xA30F,Lo,0xA30F,0xA30F,0xA30F},
{0xA310,Lo,0xA310,0xA310,0xA310},
{0xA311,Lo,0xA311,0xA311,0xA311},
{0xA312,Lo,0xA312,0xA312,0xA312},
{0xA313,Lo,0xA313,0xA313,0xA313},
{0xA314,Lo,0xA314,0xA314,0xA314},
{0xA315,Lo,0xA315,0xA315,0xA315},
{0xA316,Lo,0xA316,0xA316,0xA316},
{0xA317,Lo,0xA317,0xA317,0xA317},
{0xA318,Lo,0xA318,0xA318,0xA318},
{0xA319,Lo,0xA319,0xA319,0xA319},
{0xA31A,Lo,0xA31A,0xA31A,0xA31A},
{0xA31B,Lo,0xA31B,0xA31B,0xA31B},
{0xA31C,Lo,0xA31C,0xA31C,0xA31C},
{0xA31D,Lo,0xA31D,0xA31D,0xA31D},
{0xA31E,Lo,0xA31E,0xA31E,0xA31E},
{0xA31F,Lo,0xA31F,0xA31F,0xA31F},
{0xA320,Lo,0xA320,0xA320,0xA320},
{0xA321,Lo,0xA321,0xA321,0xA321},
{0xA322,Lo,0xA322,0xA322,0xA322},
{0xA323,Lo,0xA323,0xA323,0xA323},
{0xA324,Lo,0xA324,0xA324,0xA324},
{0xA325,Lo,0xA325,0xA325,0xA325},
{0xA326,Lo,0xA326,0xA326,0xA326},
{0xA327,Lo,0xA327,0xA327,0xA327},
{0xA328,Lo,0xA328,0xA328,0xA328},
{0xA329,Lo,0xA329,0xA329,0xA329},
{0xA32A,Lo,0xA32A,0xA32A,0xA32A},
{0xA32B,Lo,0xA32B,0xA32B,0xA32B},
{0xA32C,Lo,0xA32C,0xA32C,0xA32C},
{0xA32D,Lo,0xA32D,0xA32D,0xA32D},
{0xA32E,Lo,0xA32E,0xA32E,0xA32E},
{0xA32F,Lo,0xA32F,0xA32F,0xA32F},
{0xA330,Lo,0xA330,0xA330,0xA330},
{0xA331,Lo,0xA331,0xA331,0xA331},
{0xA332,Lo,0xA332,0xA332,0xA332},
{0xA333,Lo,0xA333,0xA333,0xA333},
{0xA334,Lo,0xA334,0xA334,0xA334},
{0xA335,Lo,0xA335,0xA335,0xA335},
{0xA336,Lo,0xA336,0xA336,0xA336},
{0xA337,Lo,0xA337,0xA337,0xA337},
{0xA338,Lo,0xA338,0xA338,0xA338},
{0xA339,Lo,0xA339,0xA339,0xA339},
{0xA33A,Lo,0xA33A,0xA33A,0xA33A},
{0xA33B,Lo,0xA33B,0xA33B,0xA33B},
{0xA33C,Lo,0xA33C,0xA33C,0xA33C},
{0xA33D,Lo,0xA33D,0xA33D,0xA33D},
{0xA33E,Lo,0xA33E,0xA33E,0xA33E},
{0xA33F,Lo,0xA33F,0xA33F,0xA33F},
{0xA340,Lo,0xA340,0xA340,0xA340},
{0xA341,Lo,0xA341,0xA341,0xA341},
{0xA342,Lo,0xA342,0xA342,0xA342},
{0xA343,Lo,0xA343,0xA343,0xA343},
{0xA344,Lo,0xA344,0xA344,0xA344},
{0xA345,Lo,0xA345,0xA345,0xA345},
{0xA346,Lo,0xA346,0xA346,0xA346},
{0xA347,Lo,0xA347,0xA347,0xA347},
{0xA348,Lo,0xA348,0xA348,0xA348},
{0xA349,Lo,0xA349,0xA349,0xA349},
{0xA34A,Lo,0xA34A,0xA34A,0xA34A},
{0xA34B,Lo,0xA34B,0xA34B,0xA34B},
{0xA34C,Lo,0xA34C,0xA34C,0xA34C},
{0xA34D,Lo,0xA34D,0xA34D,0xA34D},
{0xA34E,Lo,0xA34E,0xA34E,0xA34E},
{0xA34F,Lo,0xA34F,0xA34F,0xA34F},
{0xA350,Lo,0xA350,0xA350,0xA350},
{0xA351,Lo,0xA351,0xA351,0xA351},
{0xA352,Lo,0xA352,0xA352,0xA352},
{0xA353,Lo,0xA353,0xA353,0xA353},
{0xA354,Lo,0xA354,0xA354,0xA354},
{0xA355,Lo,0xA355,0xA355,0xA355},
{0xA356,Lo,0xA356,0xA356,0xA356},
{0xA357,Lo,0xA357,0xA357,0xA357},
{0xA358,Lo,0xA358,0xA358,0xA358},
{0xA359,Lo,0xA359,0xA359,0xA359},
{0xA35A,Lo,0xA35A,0xA35A,0xA35A},
{0xA35B,Lo,0xA35B,0xA35B,0xA35B},
{0xA35C,Lo,0xA35C,0xA35C,0xA35C},
{0xA35D,Lo,0xA35D,0xA35D,0xA35D},
{0xA35E,Lo,0xA35E,0xA35E,0xA35E},
{0xA35F,Lo,0xA35F,0xA35F,0xA35F},
{0xA360,Lo,0xA360,0xA360,0xA360},
{0xA361,Lo,0xA361,0xA361,0xA361},
{0xA362,Lo,0xA362,0xA362,0xA362},
{0xA363,Lo,0xA363,0xA363,0xA363},
{0xA364,Lo,0xA364,0xA364,0xA364},
{0xA365,Lo,0xA365,0xA365,0xA365},
{0xA366,Lo,0xA366,0xA366,0xA366},
{0xA367,Lo,0xA367,0xA367,0xA367},
{0xA368,Lo,0xA368,0xA368,0xA368},
{0xA369,Lo,0xA369,0xA369,0xA369},
{0xA36A,Lo,0xA36A,0xA36A,0xA36A},
{0xA36B,Lo,0xA36B,0xA36B,0xA36B},
{0xA36C,Lo,0xA36C,0xA36C,0xA36C},
{0xA36D,Lo,0xA36D,0xA36D,0xA36D},
{0xA36E,Lo,0xA36E,0xA36E,0xA36E},
{0xA36F,Lo,0xA36F,0xA36F,0xA36F},
{0xA370,Lo,0xA370,0xA370,0xA370},
{0xA371,Lo,0xA371,0xA371,0xA371},
{0xA372,Lo,0xA372,0xA372,0xA372},
{0xA373,Lo,0xA373,0xA373,0xA373},
{0xA374,Lo,0xA374,0xA374,0xA374},
{0xA375,Lo,0xA375,0xA375,0xA375},
{0xA376,Lo,0xA376,0xA376,0xA376},
{0xA377,Lo,0xA377,0xA377,0xA377},
{0xA378,Lo,0xA378,0xA378,0xA378},
{0xA379,Lo,0xA379,0xA379,0xA379},
{0xA37A,Lo,0xA37A,0xA37A,0xA37A},
{0xA37B,Lo,0xA37B,0xA37B,0xA37B},
{0xA37C,Lo,0xA37C,0xA37C,0xA37C},
{0xA37D,Lo,0xA37D,0xA37D,0xA37D},
{0xA37E,Lo,0xA37E,0xA37E,0xA37E},
{0xA37F,Lo,0xA37F,0xA37F,0xA37F},
{0xA380,Lo,0xA380,0xA380,0xA380},
{0xA381,Lo,0xA381,0xA381,0xA381},
{0xA382,Lo,0xA382,0xA382,0xA382},
{0xA383,Lo,0xA383,0xA383,0xA383},
{0xA384,Lo,0xA384,0xA384,0xA384},
{0xA385,Lo,0xA385,0xA385,0xA385},
{0xA386,Lo,0xA386,0xA386,0xA386},
{0xA387,Lo,0xA387,0xA387,0xA387},
{0xA388,Lo,0xA388,0xA388,0xA388},
{0xA389,Lo,0xA389,0xA389,0xA389},
{0xA38A,Lo,0xA38A,0xA38A,0xA38A},
{0xA38B,Lo,0xA38B,0xA38B,0xA38B},
{0xA38C,Lo,0xA38C,0xA38C,0xA38C},
{0xA38D,Lo,0xA38D,0xA38D,0xA38D},
{0xA38E,Lo,0xA38E,0xA38E,0xA38E},
{0xA38F,Lo,0xA38F,0xA38F,0xA38F},
{0xA390,Lo,0xA390,0xA390,0xA390},
{0xA391,Lo,0xA391,0xA391,0xA391},
{0xA392,Lo,0xA392,0xA392,0xA392},
{0xA393,Lo,0xA393,0xA393,0xA393},
{0xA394,Lo,0xA394,0xA394,0xA394},
{0xA395,Lo,0xA395,0xA395,0xA395},
{0xA396,Lo,0xA396,0xA396,0xA396},
{0xA397,Lo,0xA397,0xA397,0xA397},
{0xA398,Lo,0xA398,0xA398,0xA398},
{0xA399,Lo,0xA399,0xA399,0xA399},
{0xA39A,Lo,0xA39A,0xA39A,0xA39A},
{0xA39B,Lo,0xA39B,0xA39B,0xA39B},
{0xA39C,Lo,0xA39C,0xA39C,0xA39C},
{0xA39D,Lo,0xA39D,0xA39D,0xA39D},
{0xA39E,Lo,0xA39E,0xA39E,0xA39E},
{0xA39F,Lo,0xA39F,0xA39F,0xA39F},
{0xA3A0,Lo,0xA3A0,0xA3A0,0xA3A0},
{0xA3A1,Lo,0xA3A1,0xA3A1,0xA3A1},
{0xA3A2,Lo,0xA3A2,0xA3A2,0xA3A2},
{0xA3A3,Lo,0xA3A3,0xA3A3,0xA3A3},
{0xA3A4,Lo,0xA3A4,0xA3A4,0xA3A4},
{0xA3A5,Lo,0xA3A5,0xA3A5,0xA3A5},
{0xA3A6,Lo,0xA3A6,0xA3A6,0xA3A6},
{0xA3A7,Lo,0xA3A7,0xA3A7,0xA3A7},
{0xA3A8,Lo,0xA3A8,0xA3A8,0xA3A8},
{0xA3A9,Lo,0xA3A9,0xA3A9,0xA3A9},
{0xA3AA,Lo,0xA3AA,0xA3AA,0xA3AA},
{0xA3AB,Lo,0xA3AB,0xA3AB,0xA3AB},
{0xA3AC,Lo,0xA3AC,0xA3AC,0xA3AC},
{0xA3AD,Lo,0xA3AD,0xA3AD,0xA3AD},
{0xA3AE,Lo,0xA3AE,0xA3AE,0xA3AE},
{0xA3AF,Lo,0xA3AF,0xA3AF,0xA3AF},
{0xA3B0,Lo,0xA3B0,0xA3B0,0xA3B0},
{0xA3B1,Lo,0xA3B1,0xA3B1,0xA3B1},
{0xA3B2,Lo,0xA3B2,0xA3B2,0xA3B2},
{0xA3B3,Lo,0xA3B3,0xA3B3,0xA3B3},
{0xA3B4,Lo,0xA3B4,0xA3B4,0xA3B4},
{0xA3B5,Lo,0xA3B5,0xA3B5,0xA3B5},
{0xA3B6,Lo,0xA3B6,0xA3B6,0xA3B6},
{0xA3B7,Lo,0xA3B7,0xA3B7,0xA3B7},
{0xA3B8,Lo,0xA3B8,0xA3B8,0xA3B8},
{0xA3B9,Lo,0xA3B9,0xA3B9,0xA3B9},
{0xA3BA,Lo,0xA3BA,0xA3BA,0xA3BA},
{0xA3BB,Lo,0xA3BB,0xA3BB,0xA3BB},
{0xA3BC,Lo,0xA3BC,0xA3BC,0xA3BC},
{0xA3BD,Lo,0xA3BD,0xA3BD,0xA3BD},
{0xA3BE,Lo,0xA3BE,0xA3BE,0xA3BE},
{0xA3BF,Lo,0xA3BF,0xA3BF,0xA3BF},
{0xA3C0,Lo,0xA3C0,0xA3C0,0xA3C0},
{0xA3C1,Lo,0xA3C1,0xA3C1,0xA3C1},
{0xA3C2,Lo,0xA3C2,0xA3C2,0xA3C2},
{0xA3C3,Lo,0xA3C3,0xA3C3,0xA3C3},
{0xA3C4,Lo,0xA3C4,0xA3C4,0xA3C4},
{0xA3C5,Lo,0xA3C5,0xA3C5,0xA3C5},
{0xA3C6,Lo,0xA3C6,0xA3C6,0xA3C6},
{0xA3C7,Lo,0xA3C7,0xA3C7,0xA3C7},
{0xA3C8,Lo,0xA3C8,0xA3C8,0xA3C8},
{0xA3C9,Lo,0xA3C9,0xA3C9,0xA3C9},
{0xA3CA,Lo,0xA3CA,0xA3CA,0xA3CA},
{0xA3CB,Lo,0xA3CB,0xA3CB,0xA3CB},
{0xA3CC,Lo,0xA3CC,0xA3CC,0xA3CC},
{0xA3CD,Lo,0xA3CD,0xA3CD,0xA3CD},
{0xA3CE,Lo,0xA3CE,0xA3CE,0xA3CE},
{0xA3CF,Lo,0xA3CF,0xA3CF,0xA3CF},
{0xA3D0,Lo,0xA3D0,0xA3D0,0xA3D0},
{0xA3D1,Lo,0xA3D1,0xA3D1,0xA3D1},
{0xA3D2,Lo,0xA3D2,0xA3D2,0xA3D2},
{0xA3D3,Lo,0xA3D3,0xA3D3,0xA3D3},
{0xA3D4,Lo,0xA3D4,0xA3D4,0xA3D4},
{0xA3D5,Lo,0xA3D5,0xA3D5,0xA3D5},
{0xA3D6,Lo,0xA3D6,0xA3D6,0xA3D6},
{0xA3D7,Lo,0xA3D7,0xA3D7,0xA3D7},
{0xA3D8,Lo,0xA3D8,0xA3D8,0xA3D8},
{0xA3D9,Lo,0xA3D9,0xA3D9,0xA3D9},
{0xA3DA,Lo,0xA3DA,0xA3DA,0xA3DA},
{0xA3DB,Lo,0xA3DB,0xA3DB,0xA3DB},
{0xA3DC,Lo,0xA3DC,0xA3DC,0xA3DC},
{0xA3DD,Lo,0xA3DD,0xA3DD,0xA3DD},
{0xA3DE,Lo,0xA3DE,0xA3DE,0xA3DE},
{0xA3DF,Lo,0xA3DF,0xA3DF,0xA3DF},
{0xA3E0,Lo,0xA3E0,0xA3E0,0xA3E0},
{0xA3E1,Lo,0xA3E1,0xA3E1,0xA3E1},
{0xA3E2,Lo,0xA3E2,0xA3E2,0xA3E2},
{0xA3E3,Lo,0xA3E3,0xA3E3,0xA3E3},
{0xA3E4,Lo,0xA3E4,0xA3E4,0xA3E4},
{0xA3E5,Lo,0xA3E5,0xA3E5,0xA3E5},
{0xA3E6,Lo,0xA3E6,0xA3E6,0xA3E6},
{0xA3E7,Lo,0xA3E7,0xA3E7,0xA3E7},
{0xA3E8,Lo,0xA3E8,0xA3E8,0xA3E8},
{0xA3E9,Lo,0xA3E9,0xA3E9,0xA3E9},
{0xA3EA,Lo,0xA3EA,0xA3EA,0xA3EA},
{0xA3EB,Lo,0xA3EB,0xA3EB,0xA3EB},
{0xA3EC,Lo,0xA3EC,0xA3EC,0xA3EC},
{0xA3ED,Lo,0xA3ED,0xA3ED,0xA3ED},
{0xA3EE,Lo,0xA3EE,0xA3EE,0xA3EE},
{0xA3EF,Lo,0xA3EF,0xA3EF,0xA3EF},
{0xA3F0,Lo,0xA3F0,0xA3F0,0xA3F0},
{0xA3F1,Lo,0xA3F1,0xA3F1,0xA3F1},
{0xA3F2,Lo,0xA3F2,0xA3F2,0xA3F2},
{0xA3F3,Lo,0xA3F3,0xA3F3,0xA3F3},
{0xA3F4,Lo,0xA3F4,0xA3F4,0xA3F4},
{0xA3F5,Lo,0xA3F5,0xA3F5,0xA3F5},
{0xA3F6,Lo,0xA3F6,0xA3F6,0xA3F6},
{0xA3F7,Lo,0xA3F7,0xA3F7,0xA3F7},
{0xA3F8,Lo,0xA3F8,0xA3F8,0xA3F8},
{0xA3F9,Lo,0xA3F9,0xA3F9,0xA3F9},
{0xA3FA,Lo,0xA3FA,0xA3FA,0xA3FA},
{0xA3FB,Lo,0xA3FB,0xA3FB,0xA3FB},
{0xA3FC,Lo,0xA3FC,0xA3FC,0xA3FC},
{0xA3FD,Lo,0xA3FD,0xA3FD,0xA3FD},
{0xA3FE,Lo,0xA3FE,0xA3FE,0xA3FE},
{0xA3FF,Lo,0xA3FF,0xA3FF,0xA3FF},
{0xA400,Lo,0xA400,0xA400,0xA400},
{0xA401,Lo,0xA401,0xA401,0xA401},
{0xA402,Lo,0xA402,0xA402,0xA402},
{0xA403,Lo,0xA403,0xA403,0xA403},
{0xA404,Lo,0xA404,0xA404,0xA404},
{0xA405,Lo,0xA405,0xA405,0xA405},
{0xA406,Lo,0xA406,0xA406,0xA406},
{0xA407,Lo,0xA407,0xA407,0xA407},
{0xA408,Lo,0xA408,0xA408,0xA408},
{0xA409,Lo,0xA409,0xA409,0xA409},
{0xA40A,Lo,0xA40A,0xA40A,0xA40A},
{0xA40B,Lo,0xA40B,0xA40B,0xA40B},
{0xA40C,Lo,0xA40C,0xA40C,0xA40C},
{0xA40D,Lo,0xA40D,0xA40D,0xA40D},
{0xA40E,Lo,0xA40E,0xA40E,0xA40E},
{0xA40F,Lo,0xA40F,0xA40F,0xA40F},
{0xA410,Lo,0xA410,0xA410,0xA410},
{0xA411,Lo,0xA411,0xA411,0xA411},
{0xA412,Lo,0xA412,0xA412,0xA412},
{0xA413,Lo,0xA413,0xA413,0xA413},
{0xA414,Lo,0xA414,0xA414,0xA414},
{0xA415,Lo,0xA415,0xA415,0xA415},
{0xA416,Lo,0xA416,0xA416,0xA416},
{0xA417,Lo,0xA417,0xA417,0xA417},
{0xA418,Lo,0xA418,0xA418,0xA418},
{0xA419,Lo,0xA419,0xA419,0xA419},
{0xA41A,Lo,0xA41A,0xA41A,0xA41A},
{0xA41B,Lo,0xA41B,0xA41B,0xA41B},
{0xA41C,Lo,0xA41C,0xA41C,0xA41C},
{0xA41D,Lo,0xA41D,0xA41D,0xA41D},
{0xA41E,Lo,0xA41E,0xA41E,0xA41E},
{0xA41F,Lo,0xA41F,0xA41F,0xA41F},
{0xA420,Lo,0xA420,0xA420,0xA420},
{0xA421,Lo,0xA421,0xA421,0xA421},
{0xA422,Lo,0xA422,0xA422,0xA422},
{0xA423,Lo,0xA423,0xA423,0xA423},
{0xA424,Lo,0xA424,0xA424,0xA424},
{0xA425,Lo,0xA425,0xA425,0xA425},
{0xA426,Lo,0xA426,0xA426,0xA426},
{0xA427,Lo,0xA427,0xA427,0xA427},
{0xA428,Lo,0xA428,0xA428,0xA428},
{0xA429,Lo,0xA429,0xA429,0xA429},
{0xA42A,Lo,0xA42A,0xA42A,0xA42A},
{0xA42B,Lo,0xA42B,0xA42B,0xA42B},
{0xA42C,Lo,0xA42C,0xA42C,0xA42C},
{0xA42D,Lo,0xA42D,0xA42D,0xA42D},
{0xA42E,Lo,0xA42E,0xA42E,0xA42E},
{0xA42F,Lo,0xA42F,0xA42F,0xA42F},
{0xA430,Lo,0xA430,0xA430,0xA430},
{0xA431,Lo,0xA431,0xA431,0xA431},
{0xA432,Lo,0xA432,0xA432,0xA432},
{0xA433,Lo,0xA433,0xA433,0xA433},
{0xA434,Lo,0xA434,0xA434,0xA434},
{0xA435,Lo,0xA435,0xA435,0xA435},
{0xA436,Lo,0xA436,0xA436,0xA436},
{0xA437,Lo,0xA437,0xA437,0xA437},
{0xA438,Lo,0xA438,0xA438,0xA438},
{0xA439,Lo,0xA439,0xA439,0xA439},
{0xA43A,Lo,0xA43A,0xA43A,0xA43A},
{0xA43B,Lo,0xA43B,0xA43B,0xA43B},
{0xA43C,Lo,0xA43C,0xA43C,0xA43C},
{0xA43D,Lo,0xA43D,0xA43D,0xA43D},
{0xA43E,Lo,0xA43E,0xA43E,0xA43E},
{0xA43F,Lo,0xA43F,0xA43F,0xA43F},
{0xA440,Lo,0xA440,0xA440,0xA440},
{0xA441,Lo,0xA441,0xA441,0xA441},
{0xA442,Lo,0xA442,0xA442,0xA442},
{0xA443,Lo,0xA443,0xA443,0xA443},
{0xA444,Lo,0xA444,0xA444,0xA444},
{0xA445,Lo,0xA445,0xA445,0xA445},
{0xA446,Lo,0xA446,0xA446,0xA446},
{0xA447,Lo,0xA447,0xA447,0xA447},
{0xA448,Lo,0xA448,0xA448,0xA448},
{0xA449,Lo,0xA449,0xA449,0xA449},
{0xA44A,Lo,0xA44A,0xA44A,0xA44A},
{0xA44B,Lo,0xA44B,0xA44B,0xA44B},
{0xA44C,Lo,0xA44C,0xA44C,0xA44C},
{0xA44D,Lo,0xA44D,0xA44D,0xA44D},
{0xA44E,Lo,0xA44E,0xA44E,0xA44E},
{0xA44F,Lo,0xA44F,0xA44F,0xA44F},
{0xA450,Lo,0xA450,0xA450,0xA450},
{0xA451,Lo,0xA451,0xA451,0xA451},
{0xA452,Lo,0xA452,0xA452,0xA452},
{0xA453,Lo,0xA453,0xA453,0xA453},
{0xA454,Lo,0xA454,0xA454,0xA454},
{0xA455,Lo,0xA455,0xA455,0xA455},
{0xA456,Lo,0xA456,0xA456,0xA456},
{0xA457,Lo,0xA457,0xA457,0xA457},
{0xA458,Lo,0xA458,0xA458,0xA458},
{0xA459,Lo,0xA459,0xA459,0xA459},
{0xA45A,Lo,0xA45A,0xA45A,0xA45A},
{0xA45B,Lo,0xA45B,0xA45B,0xA45B},
{0xA45C,Lo,0xA45C,0xA45C,0xA45C},
{0xA45D,Lo,0xA45D,0xA45D,0xA45D},
{0xA45E,Lo,0xA45E,0xA45E,0xA45E},
{0xA45F,Lo,0xA45F,0xA45F,0xA45F},
{0xA460,Lo,0xA460,0xA460,0xA460},
{0xA461,Lo,0xA461,0xA461,0xA461},
{0xA462,Lo,0xA462,0xA462,0xA462},
{0xA463,Lo,0xA463,0xA463,0xA463},
{0xA464,Lo,0xA464,0xA464,0xA464},
{0xA465,Lo,0xA465,0xA465,0xA465},
{0xA466,Lo,0xA466,0xA466,0xA466},
{0xA467,Lo,0xA467,0xA467,0xA467},
{0xA468,Lo,0xA468,0xA468,0xA468},
{0xA469,Lo,0xA469,0xA469,0xA469},
{0xA46A,Lo,0xA46A,0xA46A,0xA46A},
{0xA46B,Lo,0xA46B,0xA46B,0xA46B},
{0xA46C,Lo,0xA46C,0xA46C,0xA46C},
{0xA46D,Lo,0xA46D,0xA46D,0xA46D},
{0xA46E,Lo,0xA46E,0xA46E,0xA46E},
{0xA46F,Lo,0xA46F,0xA46F,0xA46F},
{0xA470,Lo,0xA470,0xA470,0xA470},
{0xA471,Lo,0xA471,0xA471,0xA471},
{0xA472,Lo,0xA472,0xA472,0xA472},
{0xA473,Lo,0xA473,0xA473,0xA473},
{0xA474,Lo,0xA474,0xA474,0xA474},
{0xA475,Lo,0xA475,0xA475,0xA475},
{0xA476,Lo,0xA476,0xA476,0xA476},
{0xA477,Lo,0xA477,0xA477,0xA477},
{0xA478,Lo,0xA478,0xA478,0xA478},
{0xA479,Lo,0xA479,0xA479,0xA479},
{0xA47A,Lo,0xA47A,0xA47A,0xA47A},
{0xA47B,Lo,0xA47B,0xA47B,0xA47B},
{0xA47C,Lo,0xA47C,0xA47C,0xA47C},
{0xA47D,Lo,0xA47D,0xA47D,0xA47D},
{0xA47E,Lo,0xA47E,0xA47E,0xA47E},
{0xA47F,Lo,0xA47F,0xA47F,0xA47F},
{0xA480,Lo,0xA480,0xA480,0xA480},
{0xA481,Lo,0xA481,0xA481,0xA481},
{0xA482,Lo,0xA482,0xA482,0xA482},
{0xA483,Lo,0xA483,0xA483,0xA483},
{0xA484,Lo,0xA484,0xA484,0xA484},
{0xA485,Lo,0xA485,0xA485,0xA485},
{0xA486,Lo,0xA486,0xA486,0xA486},
{0xA487,Lo,0xA487,0xA487,0xA487},
{0xA488,Lo,0xA488,0xA488,0xA488},
{0xA489,Lo,0xA489,0xA489,0xA489},
{0xA48A,Lo,0xA48A,0xA48A,0xA48A},
{0xA48B,Lo,0xA48B,0xA48B,0xA48B},
{0xA48C,Lo,0xA48C,0xA48C,0xA48C},
{0xA490,So,0xA490,0xA490,0xA490},
{0xA491,So,0xA491,0xA491,0xA491},
{0xA492,So,0xA492,0xA492,0xA492},
{0xA493,So,0xA493,0xA493,0xA493},
{0xA494,So,0xA494,0xA494,0xA494},
{0xA495,So,0xA495,0xA495,0xA495},
{0xA496,So,0xA496,0xA496,0xA496},
{0xA497,So,0xA497,0xA497,0xA497},
{0xA498,So,0xA498,0xA498,0xA498},
{0xA499,So,0xA499,0xA499,0xA499},
{0xA49A,So,0xA49A,0xA49A,0xA49A},
{0xA49B,So,0xA49B,0xA49B,0xA49B},
{0xA49C,So,0xA49C,0xA49C,0xA49C},
{0xA49D,So,0xA49D,0xA49D,0xA49D},
{0xA49E,So,0xA49E,0xA49E,0xA49E},
{0xA49F,So,0xA49F,0xA49F,0xA49F},
{0xA4A0,So,0xA4A0,0xA4A0,0xA4A0},
{0xA4A1,So,0xA4A1,0xA4A1,0xA4A1},
{0xA4A2,So,0xA4A2,0xA4A2,0xA4A2},
{0xA4A3,So,0xA4A3,0xA4A3,0xA4A3},
{0xA4A4,So,0xA4A4,0xA4A4,0xA4A4},
{0xA4A5,So,0xA4A5,0xA4A5,0xA4A5},
{0xA4A6,So,0xA4A6,0xA4A6,0xA4A6},
{0xA4A7,So,0xA4A7,0xA4A7,0xA4A7},
{0xA4A8,So,0xA4A8,0xA4A8,0xA4A8},
{0xA4A9,So,0xA4A9,0xA4A9,0xA4A9},
{0xA4AA,So,0xA4AA,0xA4AA,0xA4AA},
{0xA4AB,So,0xA4AB,0xA4AB,0xA4AB},
{0xA4AC,So,0xA4AC,0xA4AC,0xA4AC},
{0xA4AD,So,0xA4AD,0xA4AD,0xA4AD},
{0xA4AE,So,0xA4AE,0xA4AE,0xA4AE},
{0xA4AF,So,0xA4AF,0xA4AF,0xA4AF},
{0xA4B0,So,0xA4B0,0xA4B0,0xA4B0},
{0xA4B1,So,0xA4B1,0xA4B1,0xA4B1},
{0xA4B2,So,0xA4B2,0xA4B2,0xA4B2},
{0xA4B3,So,0xA4B3,0xA4B3,0xA4B3},
{0xA4B4,So,0xA4B4,0xA4B4,0xA4B4},
{0xA4B5,So,0xA4B5,0xA4B5,0xA4B5},
{0xA4B6,So,0xA4B6,0xA4B6,0xA4B6},
{0xA4B7,So,0xA4B7,0xA4B7,0xA4B7},
{0xA4B8,So,0xA4B8,0xA4B8,0xA4B8},
{0xA4B9,So,0xA4B9,0xA4B9,0xA4B9},
{0xA4BA,So,0xA4BA,0xA4BA,0xA4BA},
{0xA4BB,So,0xA4BB,0xA4BB,0xA4BB},
{0xA4BC,So,0xA4BC,0xA4BC,0xA4BC},
{0xA4BD,So,0xA4BD,0xA4BD,0xA4BD},
{0xA4BE,So,0xA4BE,0xA4BE,0xA4BE},
{0xA4BF,So,0xA4BF,0xA4BF,0xA4BF},
{0xA4C0,So,0xA4C0,0xA4C0,0xA4C0},
{0xA4C1,So,0xA4C1,0xA4C1,0xA4C1},
{0xA4C2,So,0xA4C2,0xA4C2,0xA4C2},
{0xA4C3,So,0xA4C3,0xA4C3,0xA4C3},
{0xA4C4,So,0xA4C4,0xA4C4,0xA4C4},
{0xA4C5,So,0xA4C5,0xA4C5,0xA4C5},
{0xA4C6,So,0xA4C6,0xA4C6,0xA4C6},
{0xA700,Sk,0xA700,0xA700,0xA700},
{0xA701,Sk,0xA701,0xA701,0xA701},
{0xA702,Sk,0xA702,0xA702,0xA702},
{0xA703,Sk,0xA703,0xA703,0xA703},
{0xA704,Sk,0xA704,0xA704,0xA704},
{0xA705,Sk,0xA705,0xA705,0xA705},
{0xA706,Sk,0xA706,0xA706,0xA706},
{0xA707,Sk,0xA707,0xA707,0xA707},
{0xA708,Sk,0xA708,0xA708,0xA708},
{0xA709,Sk,0xA709,0xA709,0xA709},
{0xA70A,Sk,0xA70A,0xA70A,0xA70A},
{0xA70B,Sk,0xA70B,0xA70B,0xA70B},
{0xA70C,Sk,0xA70C,0xA70C,0xA70C},
{0xA70D,Sk,0xA70D,0xA70D,0xA70D},
{0xA70E,Sk,0xA70E,0xA70E,0xA70E},
{0xA70F,Sk,0xA70F,0xA70F,0xA70F},
{0xA710,Sk,0xA710,0xA710,0xA710},
{0xA711,Sk,0xA711,0xA711,0xA711},
{0xA712,Sk,0xA712,0xA712,0xA712},
{0xA713,Sk,0xA713,0xA713,0xA713},
{0xA714,Sk,0xA714,0xA714,0xA714},
{0xA715,Sk,0xA715,0xA715,0xA715},
{0xA716,Sk,0xA716,0xA716,0xA716},
{0xA717,Lm,0xA717,0xA717,0xA717},
{0xA718,Lm,0xA718,0xA718,0xA718},
{0xA719,Lm,0xA719,0xA719,0xA719},
{0xA71A,Lm,0xA71A,0xA71A,0xA71A},
{0xA720,Sk,0xA720,0xA720,0xA720},
{0xA721,Sk,0xA721,0xA721,0xA721},
{0xA800,Lo,0xA800,0xA800,0xA800},
{0xA801,Lo,0xA801,0xA801,0xA801},
{0xA802,Mc,0xA802,0xA802,0xA802},
{0xA803,Lo,0xA803,0xA803,0xA803},
{0xA804,Lo,0xA804,0xA804,0xA804},
{0xA805,Lo,0xA805,0xA805,0xA805},
{0xA806,Mn,0xA806,0xA806,0xA806},
{0xA807,Lo,0xA807,0xA807,0xA807},
{0xA808,Lo,0xA808,0xA808,0xA808},
{0xA809,Lo,0xA809,0xA809,0xA809},
{0xA80A,Lo,0xA80A,0xA80A,0xA80A},
{0xA80B,Mn,0xA80B,0xA80B,0xA80B},
{0xA80C,Lo,0xA80C,0xA80C,0xA80C},
{0xA80D,Lo,0xA80D,0xA80D,0xA80D},
{0xA80E,Lo,0xA80E,0xA80E,0xA80E},
{0xA80F,Lo,0xA80F,0xA80F,0xA80F},
{0xA810,Lo,0xA810,0xA810,0xA810},
{0xA811,Lo,0xA811,0xA811,0xA811},
{0xA812,Lo,0xA812,0xA812,0xA812},
{0xA813,Lo,0xA813,0xA813,0xA813},
{0xA814,Lo,0xA814,0xA814,0xA814},
{0xA815,Lo,0xA815,0xA815,0xA815},
{0xA816,Lo,0xA816,0xA816,0xA816},
{0xA817,Lo,0xA817,0xA817,0xA817},
{0xA818,Lo,0xA818,0xA818,0xA818},
{0xA819,Lo,0xA819,0xA819,0xA819},
{0xA81A,Lo,0xA81A,0xA81A,0xA81A},
{0xA81B,Lo,0xA81B,0xA81B,0xA81B},
{0xA81C,Lo,0xA81C,0xA81C,0xA81C},
{0xA81D,Lo,0xA81D,0xA81D,0xA81D},
{0xA81E,Lo,0xA81E,0xA81E,0xA81E},
{0xA81F,Lo,0xA81F,0xA81F,0xA81F},
{0xA820,Lo,0xA820,0xA820,0xA820},
{0xA821,Lo,0xA821,0xA821,0xA821},
{0xA822,Lo,0xA822,0xA822,0xA822},
{0xA823,Mc,0xA823,0xA823,0xA823},
{0xA824,Mc,0xA824,0xA824,0xA824},
{0xA825,Mn,0xA825,0xA825,0xA825},
{0xA826,Mn,0xA826,0xA826,0xA826},
{0xA827,Mc,0xA827,0xA827,0xA827},
{0xA828,So,0xA828,0xA828,0xA828},
{0xA829,So,0xA829,0xA829,0xA829},
{0xA82A,So,0xA82A,0xA82A,0xA82A},
{0xA82B,So,0xA82B,0xA82B,0xA82B},
{0xA840,Lo,0xA840,0xA840,0xA840},
{0xA841,Lo,0xA841,0xA841,0xA841},
{0xA842,Lo,0xA842,0xA842,0xA842},
{0xA843,Lo,0xA843,0xA843,0xA843},
{0xA844,Lo,0xA844,0xA844,0xA844},
{0xA845,Lo,0xA845,0xA845,0xA845},
{0xA846,Lo,0xA846,0xA846,0xA846},
{0xA847,Lo,0xA847,0xA847,0xA847},
{0xA848,Lo,0xA848,0xA848,0xA848},
{0xA849,Lo,0xA849,0xA849,0xA849},
{0xA84A,Lo,0xA84A,0xA84A,0xA84A},
{0xA84B,Lo,0xA84B,0xA84B,0xA84B},
{0xA84C,Lo,0xA84C,0xA84C,0xA84C},
{0xA84D,Lo,0xA84D,0xA84D,0xA84D},
{0xA84E,Lo,0xA84E,0xA84E,0xA84E},
{0xA84F,Lo,0xA84F,0xA84F,0xA84F},
{0xA850,Lo,0xA850,0xA850,0xA850},
{0xA851,Lo,0xA851,0xA851,0xA851},
{0xA852,Lo,0xA852,0xA852,0xA852},
{0xA853,Lo,0xA853,0xA853,0xA853},
{0xA854,Lo,0xA854,0xA854,0xA854},
{0xA855,Lo,0xA855,0xA855,0xA855},
{0xA856,Lo,0xA856,0xA856,0xA856},
{0xA857,Lo,0xA857,0xA857,0xA857},
{0xA858,Lo,0xA858,0xA858,0xA858},
{0xA859,Lo,0xA859,0xA859,0xA859},
{0xA85A,Lo,0xA85A,0xA85A,0xA85A},
{0xA85B,Lo,0xA85B,0xA85B,0xA85B},
{0xA85C,Lo,0xA85C,0xA85C,0xA85C},
{0xA85D,Lo,0xA85D,0xA85D,0xA85D},
{0xA85E,Lo,0xA85E,0xA85E,0xA85E},
{0xA85F,Lo,0xA85F,0xA85F,0xA85F},
{0xA860,Lo,0xA860,0xA860,0xA860},
{0xA861,Lo,0xA861,0xA861,0xA861},
{0xA862,Lo,0xA862,0xA862,0xA862},
{0xA863,Lo,0xA863,0xA863,0xA863},
{0xA864,Lo,0xA864,0xA864,0xA864},
{0xA865,Lo,0xA865,0xA865,0xA865},
{0xA866,Lo,0xA866,0xA866,0xA866},
{0xA867,Lo,0xA867,0xA867,0xA867},
{0xA868,Lo,0xA868,0xA868,0xA868},
{0xA869,Lo,0xA869,0xA869,0xA869},
{0xA86A,Lo,0xA86A,0xA86A,0xA86A},
{0xA86B,Lo,0xA86B,0xA86B,0xA86B},
{0xA86C,Lo,0xA86C,0xA86C,0xA86C},
{0xA86D,Lo,0xA86D,0xA86D,0xA86D},
{0xA86E,Lo,0xA86E,0xA86E,0xA86E},
{0xA86F,Lo,0xA86F,0xA86F,0xA86F},
{0xA870,Lo,0xA870,0xA870,0xA870},
{0xA871,Lo,0xA871,0xA871,0xA871},
{0xA872,Lo,0xA872,0xA872,0xA872},
{0xA873,Lo,0xA873,0xA873,0xA873},
{0xA874,Po,0xA874,0xA874,0xA874},
{0xA875,Po,0xA875,0xA875,0xA875},
{0xA876,Po,0xA876,0xA876,0xA876},
{0xA877,Po,0xA877,0xA877,0xA877},
{0xAC00,Lo,0xAC00,0xAC00,0xAC00},
{0xD7A3,Lo,0xD7A3,0xD7A3,0xD7A3},
{0xD800,Cs,0xD800,0xD800,0xD800},
{0xDB7F,Cs,0xDB7F,0xDB7F,0xDB7F},
{0xDB80,Cs,0xDB80,0xDB80,0xDB80},
{0xDBFF,Cs,0xDBFF,0xDBFF,0xDBFF},
{0xDC00,Cs,0xDC00,0xDC00,0xDC00},
{0xDFFF,Cs,0xDFFF,0xDFFF,0xDFFF},
{0xE000,Co,0xE000,0xE000,0xE000},
{0xF8FF,Co,0xF8FF,0xF8FF,0xF8FF},
{0xF900,Lo,0xF900,0xF900,0xF900},
{0xF901,Lo,0xF901,0xF901,0xF901},
{0xF902,Lo,0xF902,0xF902,0xF902},
{0xF903,Lo,0xF903,0xF903,0xF903},
{0xF904,Lo,0xF904,0xF904,0xF904},
{0xF905,Lo,0xF905,0xF905,0xF905},
{0xF906,Lo,0xF906,0xF906,0xF906},
{0xF907,Lo,0xF907,0xF907,0xF907},
{0xF908,Lo,0xF908,0xF908,0xF908},
{0xF909,Lo,0xF909,0xF909,0xF909},
{0xF90A,Lo,0xF90A,0xF90A,0xF90A},
{0xF90B,Lo,0xF90B,0xF90B,0xF90B},
{0xF90C,Lo,0xF90C,0xF90C,0xF90C},
{0xF90D,Lo,0xF90D,0xF90D,0xF90D},
{0xF90E,Lo,0xF90E,0xF90E,0xF90E},
{0xF90F,Lo,0xF90F,0xF90F,0xF90F},
{0xF910,Lo,0xF910,0xF910,0xF910},
{0xF911,Lo,0xF911,0xF911,0xF911},
{0xF912,Lo,0xF912,0xF912,0xF912},
{0xF913,Lo,0xF913,0xF913,0xF913},
{0xF914,Lo,0xF914,0xF914,0xF914},
{0xF915,Lo,0xF915,0xF915,0xF915},
{0xF916,Lo,0xF916,0xF916,0xF916},
{0xF917,Lo,0xF917,0xF917,0xF917},
{0xF918,Lo,0xF918,0xF918,0xF918},
{0xF919,Lo,0xF919,0xF919,0xF919},
{0xF91A,Lo,0xF91A,0xF91A,0xF91A},
{0xF91B,Lo,0xF91B,0xF91B,0xF91B},
{0xF91C,Lo,0xF91C,0xF91C,0xF91C},
{0xF91D,Lo,0xF91D,0xF91D,0xF91D},
{0xF91E,Lo,0xF91E,0xF91E,0xF91E},
{0xF91F,Lo,0xF91F,0xF91F,0xF91F},
{0xF920,Lo,0xF920,0xF920,0xF920},
{0xF921,Lo,0xF921,0xF921,0xF921},
{0xF922,Lo,0xF922,0xF922,0xF922},
{0xF923,Lo,0xF923,0xF923,0xF923},
{0xF924,Lo,0xF924,0xF924,0xF924},
{0xF925,Lo,0xF925,0xF925,0xF925},
{0xF926,Lo,0xF926,0xF926,0xF926},
{0xF927,Lo,0xF927,0xF927,0xF927},
{0xF928,Lo,0xF928,0xF928,0xF928},
{0xF929,Lo,0xF929,0xF929,0xF929},
{0xF92A,Lo,0xF92A,0xF92A,0xF92A},
{0xF92B,Lo,0xF92B,0xF92B,0xF92B},
{0xF92C,Lo,0xF92C,0xF92C,0xF92C},
{0xF92D,Lo,0xF92D,0xF92D,0xF92D},
{0xF92E,Lo,0xF92E,0xF92E,0xF92E},
{0xF92F,Lo,0xF92F,0xF92F,0xF92F},
{0xF930,Lo,0xF930,0xF930,0xF930},
{0xF931,Lo,0xF931,0xF931,0xF931},
{0xF932,Lo,0xF932,0xF932,0xF932},
{0xF933,Lo,0xF933,0xF933,0xF933},
{0xF934,Lo,0xF934,0xF934,0xF934},
{0xF935,Lo,0xF935,0xF935,0xF935},
{0xF936,Lo,0xF936,0xF936,0xF936},
{0xF937,Lo,0xF937,0xF937,0xF937},
{0xF938,Lo,0xF938,0xF938,0xF938},
{0xF939,Lo,0xF939,0xF939,0xF939},
{0xF93A,Lo,0xF93A,0xF93A,0xF93A},
{0xF93B,Lo,0xF93B,0xF93B,0xF93B},
{0xF93C,Lo,0xF93C,0xF93C,0xF93C},
{0xF93D,Lo,0xF93D,0xF93D,0xF93D},
{0xF93E,Lo,0xF93E,0xF93E,0xF93E},
{0xF93F,Lo,0xF93F,0xF93F,0xF93F},
{0xF940,Lo,0xF940,0xF940,0xF940},
{0xF941,Lo,0xF941,0xF941,0xF941},
{0xF942,Lo,0xF942,0xF942,0xF942},
{0xF943,Lo,0xF943,0xF943,0xF943},
{0xF944,Lo,0xF944,0xF944,0xF944},
{0xF945,Lo,0xF945,0xF945,0xF945},
{0xF946,Lo,0xF946,0xF946,0xF946},
{0xF947,Lo,0xF947,0xF947,0xF947},
{0xF948,Lo,0xF948,0xF948,0xF948},
{0xF949,Lo,0xF949,0xF949,0xF949},
{0xF94A,Lo,0xF94A,0xF94A,0xF94A},
{0xF94B,Lo,0xF94B,0xF94B,0xF94B},
{0xF94C,Lo,0xF94C,0xF94C,0xF94C},
{0xF94D,Lo,0xF94D,0xF94D,0xF94D},
{0xF94E,Lo,0xF94E,0xF94E,0xF94E},
{0xF94F,Lo,0xF94F,0xF94F,0xF94F},
{0xF950,Lo,0xF950,0xF950,0xF950},
{0xF951,Lo,0xF951,0xF951,0xF951},
{0xF952,Lo,0xF952,0xF952,0xF952},
{0xF953,Lo,0xF953,0xF953,0xF953},
{0xF954,Lo,0xF954,0xF954,0xF954},
{0xF955,Lo,0xF955,0xF955,0xF955},
{0xF956,Lo,0xF956,0xF956,0xF956},
{0xF957,Lo,0xF957,0xF957,0xF957},
{0xF958,Lo,0xF958,0xF958,0xF958},
{0xF959,Lo,0xF959,0xF959,0xF959},
{0xF95A,Lo,0xF95A,0xF95A,0xF95A},
{0xF95B,Lo,0xF95B,0xF95B,0xF95B},
{0xF95C,Lo,0xF95C,0xF95C,0xF95C},
{0xF95D,Lo,0xF95D,0xF95D,0xF95D},
{0xF95E,Lo,0xF95E,0xF95E,0xF95E},
{0xF95F,Lo,0xF95F,0xF95F,0xF95F},
{0xF960,Lo,0xF960,0xF960,0xF960},
{0xF961,Lo,0xF961,0xF961,0xF961},
{0xF962,Lo,0xF962,0xF962,0xF962},
{0xF963,Lo,0xF963,0xF963,0xF963},
{0xF964,Lo,0xF964,0xF964,0xF964},
{0xF965,Lo,0xF965,0xF965,0xF965},
{0xF966,Lo,0xF966,0xF966,0xF966},
{0xF967,Lo,0xF967,0xF967,0xF967},
{0xF968,Lo,0xF968,0xF968,0xF968},
{0xF969,Lo,0xF969,0xF969,0xF969},
{0xF96A,Lo,0xF96A,0xF96A,0xF96A},
{0xF96B,Lo,0xF96B,0xF96B,0xF96B},
{0xF96C,Lo,0xF96C,0xF96C,0xF96C},
{0xF96D,Lo,0xF96D,0xF96D,0xF96D},
{0xF96E,Lo,0xF96E,0xF96E,0xF96E},
{0xF96F,Lo,0xF96F,0xF96F,0xF96F},
{0xF970,Lo,0xF970,0xF970,0xF970},
{0xF971,Lo,0xF971,0xF971,0xF971},
{0xF972,Lo,0xF972,0xF972,0xF972},
{0xF973,Lo,0xF973,0xF973,0xF973},
{0xF974,Lo,0xF974,0xF974,0xF974},
{0xF975,Lo,0xF975,0xF975,0xF975},
{0xF976,Lo,0xF976,0xF976,0xF976},
{0xF977,Lo,0xF977,0xF977,0xF977},
{0xF978,Lo,0xF978,0xF978,0xF978},
{0xF979,Lo,0xF979,0xF979,0xF979},
{0xF97A,Lo,0xF97A,0xF97A,0xF97A},
{0xF97B,Lo,0xF97B,0xF97B,0xF97B},
{0xF97C,Lo,0xF97C,0xF97C,0xF97C},
{0xF97D,Lo,0xF97D,0xF97D,0xF97D},
{0xF97E,Lo,0xF97E,0xF97E,0xF97E},
{0xF97F,Lo,0xF97F,0xF97F,0xF97F},
{0xF980,Lo,0xF980,0xF980,0xF980},
{0xF981,Lo,0xF981,0xF981,0xF981},
{0xF982,Lo,0xF982,0xF982,0xF982},
{0xF983,Lo,0xF983,0xF983,0xF983},
{0xF984,Lo,0xF984,0xF984,0xF984},
{0xF985,Lo,0xF985,0xF985,0xF985},
{0xF986,Lo,0xF986,0xF986,0xF986},
{0xF987,Lo,0xF987,0xF987,0xF987},
{0xF988,Lo,0xF988,0xF988,0xF988},
{0xF989,Lo,0xF989,0xF989,0xF989},
{0xF98A,Lo,0xF98A,0xF98A,0xF98A},
{0xF98B,Lo,0xF98B,0xF98B,0xF98B},
{0xF98C,Lo,0xF98C,0xF98C,0xF98C},
{0xF98D,Lo,0xF98D,0xF98D,0xF98D},
{0xF98E,Lo,0xF98E,0xF98E,0xF98E},
{0xF98F,Lo,0xF98F,0xF98F,0xF98F},
{0xF990,Lo,0xF990,0xF990,0xF990},
{0xF991,Lo,0xF991,0xF991,0xF991},
{0xF992,Lo,0xF992,0xF992,0xF992},
{0xF993,Lo,0xF993,0xF993,0xF993},
{0xF994,Lo,0xF994,0xF994,0xF994},
{0xF995,Lo,0xF995,0xF995,0xF995},
{0xF996,Lo,0xF996,0xF996,0xF996},
{0xF997,Lo,0xF997,0xF997,0xF997},
{0xF998,Lo,0xF998,0xF998,0xF998},
{0xF999,Lo,0xF999,0xF999,0xF999},
{0xF99A,Lo,0xF99A,0xF99A,0xF99A},
{0xF99B,Lo,0xF99B,0xF99B,0xF99B},
{0xF99C,Lo,0xF99C,0xF99C,0xF99C},
{0xF99D,Lo,0xF99D,0xF99D,0xF99D},
{0xF99E,Lo,0xF99E,0xF99E,0xF99E},
{0xF99F,Lo,0xF99F,0xF99F,0xF99F},
{0xF9A0,Lo,0xF9A0,0xF9A0,0xF9A0},
{0xF9A1,Lo,0xF9A1,0xF9A1,0xF9A1},
{0xF9A2,Lo,0xF9A2,0xF9A2,0xF9A2},
{0xF9A3,Lo,0xF9A3,0xF9A3,0xF9A3},
{0xF9A4,Lo,0xF9A4,0xF9A4,0xF9A4},
{0xF9A5,Lo,0xF9A5,0xF9A5,0xF9A5},
{0xF9A6,Lo,0xF9A6,0xF9A6,0xF9A6},
{0xF9A7,Lo,0xF9A7,0xF9A7,0xF9A7},
{0xF9A8,Lo,0xF9A8,0xF9A8,0xF9A8},
{0xF9A9,Lo,0xF9A9,0xF9A9,0xF9A9},
{0xF9AA,Lo,0xF9AA,0xF9AA,0xF9AA},
{0xF9AB,Lo,0xF9AB,0xF9AB,0xF9AB},
{0xF9AC,Lo,0xF9AC,0xF9AC,0xF9AC},
{0xF9AD,Lo,0xF9AD,0xF9AD,0xF9AD},
{0xF9AE,Lo,0xF9AE,0xF9AE,0xF9AE},
{0xF9AF,Lo,0xF9AF,0xF9AF,0xF9AF},
{0xF9B0,Lo,0xF9B0,0xF9B0,0xF9B0},
{0xF9B1,Lo,0xF9B1,0xF9B1,0xF9B1},
{0xF9B2,Lo,0xF9B2,0xF9B2,0xF9B2},
{0xF9B3,Lo,0xF9B3,0xF9B3,0xF9B3},
{0xF9B4,Lo,0xF9B4,0xF9B4,0xF9B4},
{0xF9B5,Lo,0xF9B5,0xF9B5,0xF9B5},
{0xF9B6,Lo,0xF9B6,0xF9B6,0xF9B6},
{0xF9B7,Lo,0xF9B7,0xF9B7,0xF9B7},
{0xF9B8,Lo,0xF9B8,0xF9B8,0xF9B8},
{0xF9B9,Lo,0xF9B9,0xF9B9,0xF9B9},
{0xF9BA,Lo,0xF9BA,0xF9BA,0xF9BA},
{0xF9BB,Lo,0xF9BB,0xF9BB,0xF9BB},
{0xF9BC,Lo,0xF9BC,0xF9BC,0xF9BC},
{0xF9BD,Lo,0xF9BD,0xF9BD,0xF9BD},
{0xF9BE,Lo,0xF9BE,0xF9BE,0xF9BE},
{0xF9BF,Lo,0xF9BF,0xF9BF,0xF9BF},
{0xF9C0,Lo,0xF9C0,0xF9C0,0xF9C0},
{0xF9C1,Lo,0xF9C1,0xF9C1,0xF9C1},
{0xF9C2,Lo,0xF9C2,0xF9C2,0xF9C2},
{0xF9C3,Lo,0xF9C3,0xF9C3,0xF9C3},
{0xF9C4,Lo,0xF9C4,0xF9C4,0xF9C4},
{0xF9C5,Lo,0xF9C5,0xF9C5,0xF9C5},
{0xF9C6,Lo,0xF9C6,0xF9C6,0xF9C6},
{0xF9C7,Lo,0xF9C7,0xF9C7,0xF9C7},
{0xF9C8,Lo,0xF9C8,0xF9C8,0xF9C8},
{0xF9C9,Lo,0xF9C9,0xF9C9,0xF9C9},
{0xF9CA,Lo,0xF9CA,0xF9CA,0xF9CA},
{0xF9CB,Lo,0xF9CB,0xF9CB,0xF9CB},
{0xF9CC,Lo,0xF9CC,0xF9CC,0xF9CC},
{0xF9CD,Lo,0xF9CD,0xF9CD,0xF9CD},
{0xF9CE,Lo,0xF9CE,0xF9CE,0xF9CE},
{0xF9CF,Lo,0xF9CF,0xF9CF,0xF9CF},
{0xF9D0,Lo,0xF9D0,0xF9D0,0xF9D0},
{0xF9D1,Lo,0xF9D1,0xF9D1,0xF9D1},
{0xF9D2,Lo,0xF9D2,0xF9D2,0xF9D2},
{0xF9D3,Lo,0xF9D3,0xF9D3,0xF9D3},
{0xF9D4,Lo,0xF9D4,0xF9D4,0xF9D4},
{0xF9D5,Lo,0xF9D5,0xF9D5,0xF9D5},
{0xF9D6,Lo,0xF9D6,0xF9D6,0xF9D6},
{0xF9D7,Lo,0xF9D7,0xF9D7,0xF9D7},
{0xF9D8,Lo,0xF9D8,0xF9D8,0xF9D8},
{0xF9D9,Lo,0xF9D9,0xF9D9,0xF9D9},
{0xF9DA,Lo,0xF9DA,0xF9DA,0xF9DA},
{0xF9DB,Lo,0xF9DB,0xF9DB,0xF9DB},
{0xF9DC,Lo,0xF9DC,0xF9DC,0xF9DC},
{0xF9DD,Lo,0xF9DD,0xF9DD,0xF9DD},
{0xF9DE,Lo,0xF9DE,0xF9DE,0xF9DE},
{0xF9DF,Lo,0xF9DF,0xF9DF,0xF9DF},
{0xF9E0,Lo,0xF9E0,0xF9E0,0xF9E0},
{0xF9E1,Lo,0xF9E1,0xF9E1,0xF9E1},
{0xF9E2,Lo,0xF9E2,0xF9E2,0xF9E2},
{0xF9E3,Lo,0xF9E3,0xF9E3,0xF9E3},
{0xF9E4,Lo,0xF9E4,0xF9E4,0xF9E4},
{0xF9E5,Lo,0xF9E5,0xF9E5,0xF9E5},
{0xF9E6,Lo,0xF9E6,0xF9E6,0xF9E6},
{0xF9E7,Lo,0xF9E7,0xF9E7,0xF9E7},
{0xF9E8,Lo,0xF9E8,0xF9E8,0xF9E8},
{0xF9E9,Lo,0xF9E9,0xF9E9,0xF9E9},
{0xF9EA,Lo,0xF9EA,0xF9EA,0xF9EA},
{0xF9EB,Lo,0xF9EB,0xF9EB,0xF9EB},
{0xF9EC,Lo,0xF9EC,0xF9EC,0xF9EC},
{0xF9ED,Lo,0xF9ED,0xF9ED,0xF9ED},
{0xF9EE,Lo,0xF9EE,0xF9EE,0xF9EE},
{0xF9EF,Lo,0xF9EF,0xF9EF,0xF9EF},
{0xF9F0,Lo,0xF9F0,0xF9F0,0xF9F0},
{0xF9F1,Lo,0xF9F1,0xF9F1,0xF9F1},
{0xF9F2,Lo,0xF9F2,0xF9F2,0xF9F2},
{0xF9F3,Lo,0xF9F3,0xF9F3,0xF9F3},
{0xF9F4,Lo,0xF9F4,0xF9F4,0xF9F4},
{0xF9F5,Lo,0xF9F5,0xF9F5,0xF9F5},
{0xF9F6,Lo,0xF9F6,0xF9F6,0xF9F6},
{0xF9F7,Lo,0xF9F7,0xF9F7,0xF9F7},
{0xF9F8,Lo,0xF9F8,0xF9F8,0xF9F8},
{0xF9F9,Lo,0xF9F9,0xF9F9,0xF9F9},
{0xF9FA,Lo,0xF9FA,0xF9FA,0xF9FA},
{0xF9FB,Lo,0xF9FB,0xF9FB,0xF9FB},
{0xF9FC,Lo,0xF9FC,0xF9FC,0xF9FC},
{0xF9FD,Lo,0xF9FD,0xF9FD,0xF9FD},
{0xF9FE,Lo,0xF9FE,0xF9FE,0xF9FE},
{0xF9FF,Lo,0xF9FF,0xF9FF,0xF9FF},
{0xFA00,Lo,0xFA00,0xFA00,0xFA00},
{0xFA01,Lo,0xFA01,0xFA01,0xFA01},
{0xFA02,Lo,0xFA02,0xFA02,0xFA02},
{0xFA03,Lo,0xFA03,0xFA03,0xFA03},
{0xFA04,Lo,0xFA04,0xFA04,0xFA04},
{0xFA05,Lo,0xFA05,0xFA05,0xFA05},
{0xFA06,Lo,0xFA06,0xFA06,0xFA06},
{0xFA07,Lo,0xFA07,0xFA07,0xFA07},
{0xFA08,Lo,0xFA08,0xFA08,0xFA08},
{0xFA09,Lo,0xFA09,0xFA09,0xFA09},
{0xFA0A,Lo,0xFA0A,0xFA0A,0xFA0A},
{0xFA0B,Lo,0xFA0B,0xFA0B,0xFA0B},
{0xFA0C,Lo,0xFA0C,0xFA0C,0xFA0C},
{0xFA0D,Lo,0xFA0D,0xFA0D,0xFA0D},
{0xFA0E,Lo,0xFA0E,0xFA0E,0xFA0E},
{0xFA0F,Lo,0xFA0F,0xFA0F,0xFA0F},
{0xFA10,Lo,0xFA10,0xFA10,0xFA10},
{0xFA11,Lo,0xFA11,0xFA11,0xFA11},
{0xFA12,Lo,0xFA12,0xFA12,0xFA12},
{0xFA13,Lo,0xFA13,0xFA13,0xFA13},
{0xFA14,Lo,0xFA14,0xFA14,0xFA14},
{0xFA15,Lo,0xFA15,0xFA15,0xFA15},
{0xFA16,Lo,0xFA16,0xFA16,0xFA16},
{0xFA17,Lo,0xFA17,0xFA17,0xFA17},
{0xFA18,Lo,0xFA18,0xFA18,0xFA18},
{0xFA19,Lo,0xFA19,0xFA19,0xFA19},
{0xFA1A,Lo,0xFA1A,0xFA1A,0xFA1A},
{0xFA1B,Lo,0xFA1B,0xFA1B,0xFA1B},
{0xFA1C,Lo,0xFA1C,0xFA1C,0xFA1C},
{0xFA1D,Lo,0xFA1D,0xFA1D,0xFA1D},
{0xFA1E,Lo,0xFA1E,0xFA1E,0xFA1E},
{0xFA1F,Lo,0xFA1F,0xFA1F,0xFA1F},
{0xFA20,Lo,0xFA20,0xFA20,0xFA20},
{0xFA21,Lo,0xFA21,0xFA21,0xFA21},
{0xFA22,Lo,0xFA22,0xFA22,0xFA22},
{0xFA23,Lo,0xFA23,0xFA23,0xFA23},
{0xFA24,Lo,0xFA24,0xFA24,0xFA24},
{0xFA25,Lo,0xFA25,0xFA25,0xFA25},
{0xFA26,Lo,0xFA26,0xFA26,0xFA26},
{0xFA27,Lo,0xFA27,0xFA27,0xFA27},
{0xFA28,Lo,0xFA28,0xFA28,0xFA28},
{0xFA29,Lo,0xFA29,0xFA29,0xFA29},
{0xFA2A,Lo,0xFA2A,0xFA2A,0xFA2A},
{0xFA2B,Lo,0xFA2B,0xFA2B,0xFA2B},
{0xFA2C,Lo,0xFA2C,0xFA2C,0xFA2C},
{0xFA2D,Lo,0xFA2D,0xFA2D,0xFA2D},
{0xFA30,Lo,0xFA30,0xFA30,0xFA30},
{0xFA31,Lo,0xFA31,0xFA31,0xFA31},
{0xFA32,Lo,0xFA32,0xFA32,0xFA32},
{0xFA33,Lo,0xFA33,0xFA33,0xFA33},
{0xFA34,Lo,0xFA34,0xFA34,0xFA34},
{0xFA35,Lo,0xFA35,0xFA35,0xFA35},
{0xFA36,Lo,0xFA36,0xFA36,0xFA36},
{0xFA37,Lo,0xFA37,0xFA37,0xFA37},
{0xFA38,Lo,0xFA38,0xFA38,0xFA38},
{0xFA39,Lo,0xFA39,0xFA39,0xFA39},
{0xFA3A,Lo,0xFA3A,0xFA3A,0xFA3A},
{0xFA3B,Lo,0xFA3B,0xFA3B,0xFA3B},
{0xFA3C,Lo,0xFA3C,0xFA3C,0xFA3C},
{0xFA3D,Lo,0xFA3D,0xFA3D,0xFA3D},
{0xFA3E,Lo,0xFA3E,0xFA3E,0xFA3E},
{0xFA3F,Lo,0xFA3F,0xFA3F,0xFA3F},
{0xFA40,Lo,0xFA40,0xFA40,0xFA40},
{0xFA41,Lo,0xFA41,0xFA41,0xFA41},
{0xFA42,Lo,0xFA42,0xFA42,0xFA42},
{0xFA43,Lo,0xFA43,0xFA43,0xFA43},
{0xFA44,Lo,0xFA44,0xFA44,0xFA44},
{0xFA45,Lo,0xFA45,0xFA45,0xFA45},
{0xFA46,Lo,0xFA46,0xFA46,0xFA46},
{0xFA47,Lo,0xFA47,0xFA47,0xFA47},
{0xFA48,Lo,0xFA48,0xFA48,0xFA48},
{0xFA49,Lo,0xFA49,0xFA49,0xFA49},
{0xFA4A,Lo,0xFA4A,0xFA4A,0xFA4A},
{0xFA4B,Lo,0xFA4B,0xFA4B,0xFA4B},
{0xFA4C,Lo,0xFA4C,0xFA4C,0xFA4C},
{0xFA4D,Lo,0xFA4D,0xFA4D,0xFA4D},
{0xFA4E,Lo,0xFA4E,0xFA4E,0xFA4E},
{0xFA4F,Lo,0xFA4F,0xFA4F,0xFA4F},
{0xFA50,Lo,0xFA50,0xFA50,0xFA50},
{0xFA51,Lo,0xFA51,0xFA51,0xFA51},
{0xFA52,Lo,0xFA52,0xFA52,0xFA52},
{0xFA53,Lo,0xFA53,0xFA53,0xFA53},
{0xFA54,Lo,0xFA54,0xFA54,0xFA54},
{0xFA55,Lo,0xFA55,0xFA55,0xFA55},
{0xFA56,Lo,0xFA56,0xFA56,0xFA56},
{0xFA57,Lo,0xFA57,0xFA57,0xFA57},
{0xFA58,Lo,0xFA58,0xFA58,0xFA58},
{0xFA59,Lo,0xFA59,0xFA59,0xFA59},
{0xFA5A,Lo,0xFA5A,0xFA5A,0xFA5A},
{0xFA5B,Lo,0xFA5B,0xFA5B,0xFA5B},
{0xFA5C,Lo,0xFA5C,0xFA5C,0xFA5C},
{0xFA5D,Lo,0xFA5D,0xFA5D,0xFA5D},
{0xFA5E,Lo,0xFA5E,0xFA5E,0xFA5E},
{0xFA5F,Lo,0xFA5F,0xFA5F,0xFA5F},
{0xFA60,Lo,0xFA60,0xFA60,0xFA60},
{0xFA61,Lo,0xFA61,0xFA61,0xFA61},
{0xFA62,Lo,0xFA62,0xFA62,0xFA62},
{0xFA63,Lo,0xFA63,0xFA63,0xFA63},
{0xFA64,Lo,0xFA64,0xFA64,0xFA64},
{0xFA65,Lo,0xFA65,0xFA65,0xFA65},
{0xFA66,Lo,0xFA66,0xFA66,0xFA66},
{0xFA67,Lo,0xFA67,0xFA67,0xFA67},
{0xFA68,Lo,0xFA68,0xFA68,0xFA68},
{0xFA69,Lo,0xFA69,0xFA69,0xFA69},
{0xFA6A,Lo,0xFA6A,0xFA6A,0xFA6A},
{0xFA70,Lo,0xFA70,0xFA70,0xFA70},
{0xFA71,Lo,0xFA71,0xFA71,0xFA71},
{0xFA72,Lo,0xFA72,0xFA72,0xFA72},
{0xFA73,Lo,0xFA73,0xFA73,0xFA73},
{0xFA74,Lo,0xFA74,0xFA74,0xFA74},
{0xFA75,Lo,0xFA75,0xFA75,0xFA75},
{0xFA76,Lo,0xFA76,0xFA76,0xFA76},
{0xFA77,Lo,0xFA77,0xFA77,0xFA77},
{0xFA78,Lo,0xFA78,0xFA78,0xFA78},
{0xFA79,Lo,0xFA79,0xFA79,0xFA79},
{0xFA7A,Lo,0xFA7A,0xFA7A,0xFA7A},
{0xFA7B,Lo,0xFA7B,0xFA7B,0xFA7B},
{0xFA7C,Lo,0xFA7C,0xFA7C,0xFA7C},
{0xFA7D,Lo,0xFA7D,0xFA7D,0xFA7D},
{0xFA7E,Lo,0xFA7E,0xFA7E,0xFA7E},
{0xFA7F,Lo,0xFA7F,0xFA7F,0xFA7F},
{0xFA80,Lo,0xFA80,0xFA80,0xFA80},
{0xFA81,Lo,0xFA81,0xFA81,0xFA81},
{0xFA82,Lo,0xFA82,0xFA82,0xFA82},
{0xFA83,Lo,0xFA83,0xFA83,0xFA83},
{0xFA84,Lo,0xFA84,0xFA84,0xFA84},
{0xFA85,Lo,0xFA85,0xFA85,0xFA85},
{0xFA86,Lo,0xFA86,0xFA86,0xFA86},
{0xFA87,Lo,0xFA87,0xFA87,0xFA87},
{0xFA88,Lo,0xFA88,0xFA88,0xFA88},
{0xFA89,Lo,0xFA89,0xFA89,0xFA89},
{0xFA8A,Lo,0xFA8A,0xFA8A,0xFA8A},
{0xFA8B,Lo,0xFA8B,0xFA8B,0xFA8B},
{0xFA8C,Lo,0xFA8C,0xFA8C,0xFA8C},
{0xFA8D,Lo,0xFA8D,0xFA8D,0xFA8D},
{0xFA8E,Lo,0xFA8E,0xFA8E,0xFA8E},
{0xFA8F,Lo,0xFA8F,0xFA8F,0xFA8F},
{0xFA90,Lo,0xFA90,0xFA90,0xFA90},
{0xFA91,Lo,0xFA91,0xFA91,0xFA91},
{0xFA92,Lo,0xFA92,0xFA92,0xFA92},
{0xFA93,Lo,0xFA93,0xFA93,0xFA93},
{0xFA94,Lo,0xFA94,0xFA94,0xFA94},
{0xFA95,Lo,0xFA95,0xFA95,0xFA95},
{0xFA96,Lo,0xFA96,0xFA96,0xFA96},
{0xFA97,Lo,0xFA97,0xFA97,0xFA97},
{0xFA98,Lo,0xFA98,0xFA98,0xFA98},
{0xFA99,Lo,0xFA99,0xFA99,0xFA99},
{0xFA9A,Lo,0xFA9A,0xFA9A,0xFA9A},
{0xFA9B,Lo,0xFA9B,0xFA9B,0xFA9B},
{0xFA9C,Lo,0xFA9C,0xFA9C,0xFA9C},
{0xFA9D,Lo,0xFA9D,0xFA9D,0xFA9D},
{0xFA9E,Lo,0xFA9E,0xFA9E,0xFA9E},
{0xFA9F,Lo,0xFA9F,0xFA9F,0xFA9F},
{0xFAA0,Lo,0xFAA0,0xFAA0,0xFAA0},
{0xFAA1,Lo,0xFAA1,0xFAA1,0xFAA1},
{0xFAA2,Lo,0xFAA2,0xFAA2,0xFAA2},
{0xFAA3,Lo,0xFAA3,0xFAA3,0xFAA3},
{0xFAA4,Lo,0xFAA4,0xFAA4,0xFAA4},
{0xFAA5,Lo,0xFAA5,0xFAA5,0xFAA5},
{0xFAA6,Lo,0xFAA6,0xFAA6,0xFAA6},
{0xFAA7,Lo,0xFAA7,0xFAA7,0xFAA7},
{0xFAA8,Lo,0xFAA8,0xFAA8,0xFAA8},
{0xFAA9,Lo,0xFAA9,0xFAA9,0xFAA9},
{0xFAAA,Lo,0xFAAA,0xFAAA,0xFAAA},
{0xFAAB,Lo,0xFAAB,0xFAAB,0xFAAB},
{0xFAAC,Lo,0xFAAC,0xFAAC,0xFAAC},
{0xFAAD,Lo,0xFAAD,0xFAAD,0xFAAD},
{0xFAAE,Lo,0xFAAE,0xFAAE,0xFAAE},
{0xFAAF,Lo,0xFAAF,0xFAAF,0xFAAF},
{0xFAB0,Lo,0xFAB0,0xFAB0,0xFAB0},
{0xFAB1,Lo,0xFAB1,0xFAB1,0xFAB1},
{0xFAB2,Lo,0xFAB2,0xFAB2,0xFAB2},
{0xFAB3,Lo,0xFAB3,0xFAB3,0xFAB3},
{0xFAB4,Lo,0xFAB4,0xFAB4,0xFAB4},
{0xFAB5,Lo,0xFAB5,0xFAB5,0xFAB5},
{0xFAB6,Lo,0xFAB6,0xFAB6,0xFAB6},
{0xFAB7,Lo,0xFAB7,0xFAB7,0xFAB7},
{0xFAB8,Lo,0xFAB8,0xFAB8,0xFAB8},
{0xFAB9,Lo,0xFAB9,0xFAB9,0xFAB9},
{0xFABA,Lo,0xFABA,0xFABA,0xFABA},
{0xFABB,Lo,0xFABB,0xFABB,0xFABB},
{0xFABC,Lo,0xFABC,0xFABC,0xFABC},
{0xFABD,Lo,0xFABD,0xFABD,0xFABD},
{0xFABE,Lo,0xFABE,0xFABE,0xFABE},
{0xFABF,Lo,0xFABF,0xFABF,0xFABF},
{0xFAC0,Lo,0xFAC0,0xFAC0,0xFAC0},
{0xFAC1,Lo,0xFAC1,0xFAC1,0xFAC1},
{0xFAC2,Lo,0xFAC2,0xFAC2,0xFAC2},
{0xFAC3,Lo,0xFAC3,0xFAC3,0xFAC3},
{0xFAC4,Lo,0xFAC4,0xFAC4,0xFAC4},
{0xFAC5,Lo,0xFAC5,0xFAC5,0xFAC5},
{0xFAC6,Lo,0xFAC6,0xFAC6,0xFAC6},
{0xFAC7,Lo,0xFAC7,0xFAC7,0xFAC7},
{0xFAC8,Lo,0xFAC8,0xFAC8,0xFAC8},
{0xFAC9,Lo,0xFAC9,0xFAC9,0xFAC9},
{0xFACA,Lo,0xFACA,0xFACA,0xFACA},
{0xFACB,Lo,0xFACB,0xFACB,0xFACB},
{0xFACC,Lo,0xFACC,0xFACC,0xFACC},
{0xFACD,Lo,0xFACD,0xFACD,0xFACD},
{0xFACE,Lo,0xFACE,0xFACE,0xFACE},
{0xFACF,Lo,0xFACF,0xFACF,0xFACF},
{0xFAD0,Lo,0xFAD0,0xFAD0,0xFAD0},
{0xFAD1,Lo,0xFAD1,0xFAD1,0xFAD1},
{0xFAD2,Lo,0xFAD2,0xFAD2,0xFAD2},
{0xFAD3,Lo,0xFAD3,0xFAD3,0xFAD3},
{0xFAD4,Lo,0xFAD4,0xFAD4,0xFAD4},
{0xFAD5,Lo,0xFAD5,0xFAD5,0xFAD5},
{0xFAD6,Lo,0xFAD6,0xFAD6,0xFAD6},
{0xFAD7,Lo,0xFAD7,0xFAD7,0xFAD7},
{0xFAD8,Lo,0xFAD8,0xFAD8,0xFAD8},
{0xFAD9,Lo,0xFAD9,0xFAD9,0xFAD9},
{0xFB00,Ll | SM,0xFB00,0xFB00,0xFB00},
{0xFB01,Ll | SM,0xFB01,0xFB01,0xFB01},
{0xFB02,Ll | SM,0xFB02,0xFB02,0xFB02},
{0xFB03,Ll | SM,0xFB03,0xFB03,0xFB03},
{0xFB04,Ll | SM,0xFB04,0xFB04,0xFB04},
{0xFB05,Ll | SM,0xFB05,0xFB05,0xFB05},
{0xFB06,Ll | SM,0xFB06,0xFB06,0xFB06},
{0xFB13,Ll | SM,0xFB13,0xFB13,0xFB13},
{0xFB14,Ll | SM,0xFB14,0xFB14,0xFB14},
{0xFB15,Ll | SM,0xFB15,0xFB15,0xFB15},
{0xFB16,Ll | SM,0xFB16,0xFB16,0xFB16},
{0xFB17,Ll | SM,0xFB17,0xFB17,0xFB17},
{0xFB1D,Lo,0xFB1D,0xFB1D,0xFB1D},
{0xFB1E,Mn,0xFB1E,0xFB1E,0xFB1E},
{0xFB1F,Lo,0xFB1F,0xFB1F,0xFB1F},
{0xFB20,Lo,0xFB20,0xFB20,0xFB20},
{0xFB21,Lo,0xFB21,0xFB21,0xFB21},
{0xFB22,Lo,0xFB22,0xFB22,0xFB22},
{0xFB23,Lo,0xFB23,0xFB23,0xFB23},
{0xFB24,Lo,0xFB24,0xFB24,0xFB24},
{0xFB25,Lo,0xFB25,0xFB25,0xFB25},
{0xFB26,Lo,0xFB26,0xFB26,0xFB26},
{0xFB27,Lo,0xFB27,0xFB27,0xFB27},
{0xFB28,Lo,0xFB28,0xFB28,0xFB28},
{0xFB29,Sm,0xFB29,0xFB29,0xFB29},
{0xFB2A,Lo,0xFB2A,0xFB2A,0xFB2A},
{0xFB2B,Lo,0xFB2B,0xFB2B,0xFB2B},
{0xFB2C,Lo,0xFB2C,0xFB2C,0xFB2C},
{0xFB2D,Lo,0xFB2D,0xFB2D,0xFB2D},
{0xFB2E,Lo,0xFB2E,0xFB2E,0xFB2E},
{0xFB2F,Lo,0xFB2F,0xFB2F,0xFB2F},
{0xFB30,Lo,0xFB30,0xFB30,0xFB30},
{0xFB31,Lo,0xFB31,0xFB31,0xFB31},
{0xFB32,Lo,0xFB32,0xFB32,0xFB32},
{0xFB33,Lo,0xFB33,0xFB33,0xFB33},
{0xFB34,Lo,0xFB34,0xFB34,0xFB34},
{0xFB35,Lo,0xFB35,0xFB35,0xFB35},
{0xFB36,Lo,0xFB36,0xFB36,0xFB36},
{0xFB38,Lo,0xFB38,0xFB38,0xFB38},
{0xFB39,Lo,0xFB39,0xFB39,0xFB39},
{0xFB3A,Lo,0xFB3A,0xFB3A,0xFB3A},
{0xFB3B,Lo,0xFB3B,0xFB3B,0xFB3B},
{0xFB3C,Lo,0xFB3C,0xFB3C,0xFB3C},
{0xFB3E,Lo,0xFB3E,0xFB3E,0xFB3E},
{0xFB40,Lo,0xFB40,0xFB40,0xFB40},
{0xFB41,Lo,0xFB41,0xFB41,0xFB41},
{0xFB43,Lo,0xFB43,0xFB43,0xFB43},
{0xFB44,Lo,0xFB44,0xFB44,0xFB44},
{0xFB46,Lo,0xFB46,0xFB46,0xFB46},
{0xFB47,Lo,0xFB47,0xFB47,0xFB47},
{0xFB48,Lo,0xFB48,0xFB48,0xFB48},
{0xFB49,Lo,0xFB49,0xFB49,0xFB49},
{0xFB4A,Lo,0xFB4A,0xFB4A,0xFB4A},
{0xFB4B,Lo,0xFB4B,0xFB4B,0xFB4B},
{0xFB4C,Lo,0xFB4C,0xFB4C,0xFB4C},
{0xFB4D,Lo,0xFB4D,0xFB4D,0xFB4D},
{0xFB4E,Lo,0xFB4E,0xFB4E,0xFB4E},
{0xFB4F,Lo,0xFB4F,0xFB4F,0xFB4F},
{0xFB50,Lo,0xFB50,0xFB50,0xFB50},
{0xFB51,Lo,0xFB51,0xFB51,0xFB51},
{0xFB52,Lo,0xFB52,0xFB52,0xFB52},
{0xFB53,Lo,0xFB53,0xFB53,0xFB53},
{0xFB54,Lo,0xFB54,0xFB54,0xFB54},
{0xFB55,Lo,0xFB55,0xFB55,0xFB55},
{0xFB56,Lo,0xFB56,0xFB56,0xFB56},
{0xFB57,Lo,0xFB57,0xFB57,0xFB57},
{0xFB58,Lo,0xFB58,0xFB58,0xFB58},
{0xFB59,Lo,0xFB59,0xFB59,0xFB59},
{0xFB5A,Lo,0xFB5A,0xFB5A,0xFB5A},
{0xFB5B,Lo,0xFB5B,0xFB5B,0xFB5B},
{0xFB5C,Lo,0xFB5C,0xFB5C,0xFB5C},
{0xFB5D,Lo,0xFB5D,0xFB5D,0xFB5D},
{0xFB5E,Lo,0xFB5E,0xFB5E,0xFB5E},
{0xFB5F,Lo,0xFB5F,0xFB5F,0xFB5F},
{0xFB60,Lo,0xFB60,0xFB60,0xFB60},
{0xFB61,Lo,0xFB61,0xFB61,0xFB61},
{0xFB62,Lo,0xFB62,0xFB62,0xFB62},
{0xFB63,Lo,0xFB63,0xFB63,0xFB63},
{0xFB64,Lo,0xFB64,0xFB64,0xFB64},
{0xFB65,Lo,0xFB65,0xFB65,0xFB65},
{0xFB66,Lo,0xFB66,0xFB66,0xFB66},
{0xFB67,Lo,0xFB67,0xFB67,0xFB67},
{0xFB68,Lo,0xFB68,0xFB68,0xFB68},
{0xFB69,Lo,0xFB69,0xFB69,0xFB69},
{0xFB6A,Lo,0xFB6A,0xFB6A,0xFB6A},
{0xFB6B,Lo,0xFB6B,0xFB6B,0xFB6B},
{0xFB6C,Lo,0xFB6C,0xFB6C,0xFB6C},
{0xFB6D,Lo,0xFB6D,0xFB6D,0xFB6D},
{0xFB6E,Lo,0xFB6E,0xFB6E,0xFB6E},
{0xFB6F,Lo,0xFB6F,0xFB6F,0xFB6F},
{0xFB70,Lo,0xFB70,0xFB70,0xFB70},
{0xFB71,Lo,0xFB71,0xFB71,0xFB71},
{0xFB72,Lo,0xFB72,0xFB72,0xFB72},
{0xFB73,Lo,0xFB73,0xFB73,0xFB73},
{0xFB74,Lo,0xFB74,0xFB74,0xFB74},
{0xFB75,Lo,0xFB75,0xFB75,0xFB75},
{0xFB76,Lo,0xFB76,0xFB76,0xFB76},
{0xFB77,Lo,0xFB77,0xFB77,0xFB77},
{0xFB78,Lo,0xFB78,0xFB78,0xFB78},
{0xFB79,Lo,0xFB79,0xFB79,0xFB79},
{0xFB7A,Lo,0xFB7A,0xFB7A,0xFB7A},
{0xFB7B,Lo,0xFB7B,0xFB7B,0xFB7B},
{0xFB7C,Lo,0xFB7C,0xFB7C,0xFB7C},
{0xFB7D,Lo,0xFB7D,0xFB7D,0xFB7D},
{0xFB7E,Lo,0xFB7E,0xFB7E,0xFB7E},
{0xFB7F,Lo,0xFB7F,0xFB7F,0xFB7F},
{0xFB80,Lo,0xFB80,0xFB80,0xFB80},
{0xFB81,Lo,0xFB81,0xFB81,0xFB81},
{0xFB82,Lo,0xFB82,0xFB82,0xFB82},
{0xFB83,Lo,0xFB83,0xFB83,0xFB83},
{0xFB84,Lo,0xFB84,0xFB84,0xFB84},
{0xFB85,Lo,0xFB85,0xFB85,0xFB85},
{0xFB86,Lo,0xFB86,0xFB86,0xFB86},
{0xFB87,Lo,0xFB87,0xFB87,0xFB87},
{0xFB88,Lo,0xFB88,0xFB88,0xFB88},
{0xFB89,Lo,0xFB89,0xFB89,0xFB89},
{0xFB8A,Lo,0xFB8A,0xFB8A,0xFB8A},
{0xFB8B,Lo,0xFB8B,0xFB8B,0xFB8B},
{0xFB8C,Lo,0xFB8C,0xFB8C,0xFB8C},
{0xFB8D,Lo,0xFB8D,0xFB8D,0xFB8D},
{0xFB8E,Lo,0xFB8E,0xFB8E,0xFB8E},
{0xFB8F,Lo,0xFB8F,0xFB8F,0xFB8F},
{0xFB90,Lo,0xFB90,0xFB90,0xFB90},
{0xFB91,Lo,0xFB91,0xFB91,0xFB91},
{0xFB92,Lo,0xFB92,0xFB92,0xFB92},
{0xFB93,Lo,0xFB93,0xFB93,0xFB93},
{0xFB94,Lo,0xFB94,0xFB94,0xFB94},
{0xFB95,Lo,0xFB95,0xFB95,0xFB95},
{0xFB96,Lo,0xFB96,0xFB96,0xFB96},
{0xFB97,Lo,0xFB97,0xFB97,0xFB97},
{0xFB98,Lo,0xFB98,0xFB98,0xFB98},
{0xFB99,Lo,0xFB99,0xFB99,0xFB99},
{0xFB9A,Lo,0xFB9A,0xFB9A,0xFB9A},
{0xFB9B,Lo,0xFB9B,0xFB9B,0xFB9B},
{0xFB9C,Lo,0xFB9C,0xFB9C,0xFB9C},
{0xFB9D,Lo,0xFB9D,0xFB9D,0xFB9D},
{0xFB9E,Lo,0xFB9E,0xFB9E,0xFB9E},
{0xFB9F,Lo,0xFB9F,0xFB9F,0xFB9F},
{0xFBA0,Lo,0xFBA0,0xFBA0,0xFBA0},
{0xFBA1,Lo,0xFBA1,0xFBA1,0xFBA1},
{0xFBA2,Lo,0xFBA2,0xFBA2,0xFBA2},
{0xFBA3,Lo,0xFBA3,0xFBA3,0xFBA3},
{0xFBA4,Lo,0xFBA4,0xFBA4,0xFBA4},
{0xFBA5,Lo,0xFBA5,0xFBA5,0xFBA5},
{0xFBA6,Lo,0xFBA6,0xFBA6,0xFBA6},
{0xFBA7,Lo,0xFBA7,0xFBA7,0xFBA7},
{0xFBA8,Lo,0xFBA8,0xFBA8,0xFBA8},
{0xFBA9,Lo,0xFBA9,0xFBA9,0xFBA9},
{0xFBAA,Lo,0xFBAA,0xFBAA,0xFBAA},
{0xFBAB,Lo,0xFBAB,0xFBAB,0xFBAB},
{0xFBAC,Lo,0xFBAC,0xFBAC,0xFBAC},
{0xFBAD,Lo,0xFBAD,0xFBAD,0xFBAD},
{0xFBAE,Lo,0xFBAE,0xFBAE,0xFBAE},
{0xFBAF,Lo,0xFBAF,0xFBAF,0xFBAF},
{0xFBB0,Lo,0xFBB0,0xFBB0,0xFBB0},
{0xFBB1,Lo,0xFBB1,0xFBB1,0xFBB1},
{0xFBD3,Lo,0xFBD3,0xFBD3,0xFBD3},
{0xFBD4,Lo,0xFBD4,0xFBD4,0xFBD4},
{0xFBD5,Lo,0xFBD5,0xFBD5,0xFBD5},
{0xFBD6,Lo,0xFBD6,0xFBD6,0xFBD6},
{0xFBD7,Lo,0xFBD7,0xFBD7,0xFBD7},
{0xFBD8,Lo,0xFBD8,0xFBD8,0xFBD8},
{0xFBD9,Lo,0xFBD9,0xFBD9,0xFBD9},
{0xFBDA,Lo,0xFBDA,0xFBDA,0xFBDA},
{0xFBDB,Lo,0xFBDB,0xFBDB,0xFBDB},
{0xFBDC,Lo,0xFBDC,0xFBDC,0xFBDC},
{0xFBDD,Lo,0xFBDD,0xFBDD,0xFBDD},
{0xFBDE,Lo,0xFBDE,0xFBDE,0xFBDE},
{0xFBDF,Lo,0xFBDF,0xFBDF,0xFBDF},
{0xFBE0,Lo,0xFBE0,0xFBE0,0xFBE0},
{0xFBE1,Lo,0xFBE1,0xFBE1,0xFBE1},
{0xFBE2,Lo,0xFBE2,0xFBE2,0xFBE2},
{0xFBE3,Lo,0xFBE3,0xFBE3,0xFBE3},
{0xFBE4,Lo,0xFBE4,0xFBE4,0xFBE4},
{0xFBE5,Lo,0xFBE5,0xFBE5,0xFBE5},
{0xFBE6,Lo,0xFBE6,0xFBE6,0xFBE6},
{0xFBE7,Lo,0xFBE7,0xFBE7,0xFBE7},
{0xFBE8,Lo,0xFBE8,0xFBE8,0xFBE8},
{0xFBE9,Lo,0xFBE9,0xFBE9,0xFBE9},
{0xFBEA,Lo,0xFBEA,0xFBEA,0xFBEA},
{0xFBEB,Lo,0xFBEB,0xFBEB,0xFBEB},
{0xFBEC,Lo,0xFBEC,0xFBEC,0xFBEC},
{0xFBED,Lo,0xFBED,0xFBED,0xFBED},
{0xFBEE,Lo,0xFBEE,0xFBEE,0xFBEE},
{0xFBEF,Lo,0xFBEF,0xFBEF,0xFBEF},
{0xFBF0,Lo,0xFBF0,0xFBF0,0xFBF0},
{0xFBF1,Lo,0xFBF1,0xFBF1,0xFBF1},
{0xFBF2,Lo,0xFBF2,0xFBF2,0xFBF2},
{0xFBF3,Lo,0xFBF3,0xFBF3,0xFBF3},
{0xFBF4,Lo,0xFBF4,0xFBF4,0xFBF4},
{0xFBF5,Lo,0xFBF5,0xFBF5,0xFBF5},
{0xFBF6,Lo,0xFBF6,0xFBF6,0xFBF6},
{0xFBF7,Lo,0xFBF7,0xFBF7,0xFBF7},
{0xFBF8,Lo,0xFBF8,0xFBF8,0xFBF8},
{0xFBF9,Lo,0xFBF9,0xFBF9,0xFBF9},
{0xFBFA,Lo,0xFBFA,0xFBFA,0xFBFA},
{0xFBFB,Lo,0xFBFB,0xFBFB,0xFBFB},
{0xFBFC,Lo,0xFBFC,0xFBFC,0xFBFC},
{0xFBFD,Lo,0xFBFD,0xFBFD,0xFBFD},
{0xFBFE,Lo,0xFBFE,0xFBFE,0xFBFE},
{0xFBFF,Lo,0xFBFF,0xFBFF,0xFBFF},
{0xFC00,Lo,0xFC00,0xFC00,0xFC00},
{0xFC01,Lo,0xFC01,0xFC01,0xFC01},
{0xFC02,Lo,0xFC02,0xFC02,0xFC02},
{0xFC03,Lo,0xFC03,0xFC03,0xFC03},
{0xFC04,Lo,0xFC04,0xFC04,0xFC04},
{0xFC05,Lo,0xFC05,0xFC05,0xFC05},
{0xFC06,Lo,0xFC06,0xFC06,0xFC06},
{0xFC07,Lo,0xFC07,0xFC07,0xFC07},
{0xFC08,Lo,0xFC08,0xFC08,0xFC08},
{0xFC09,Lo,0xFC09,0xFC09,0xFC09},
{0xFC0A,Lo,0xFC0A,0xFC0A,0xFC0A},
{0xFC0B,Lo,0xFC0B,0xFC0B,0xFC0B},
{0xFC0C,Lo,0xFC0C,0xFC0C,0xFC0C},
{0xFC0D,Lo,0xFC0D,0xFC0D,0xFC0D},
{0xFC0E,Lo,0xFC0E,0xFC0E,0xFC0E},
{0xFC0F,Lo,0xFC0F,0xFC0F,0xFC0F},
{0xFC10,Lo,0xFC10,0xFC10,0xFC10},
{0xFC11,Lo,0xFC11,0xFC11,0xFC11},
{0xFC12,Lo,0xFC12,0xFC12,0xFC12},
{0xFC13,Lo,0xFC13,0xFC13,0xFC13},
{0xFC14,Lo,0xFC14,0xFC14,0xFC14},
{0xFC15,Lo,0xFC15,0xFC15,0xFC15},
{0xFC16,Lo,0xFC16,0xFC16,0xFC16},
{0xFC17,Lo,0xFC17,0xFC17,0xFC17},
{0xFC18,Lo,0xFC18,0xFC18,0xFC18},
{0xFC19,Lo,0xFC19,0xFC19,0xFC19},
{0xFC1A,Lo,0xFC1A,0xFC1A,0xFC1A},
{0xFC1B,Lo,0xFC1B,0xFC1B,0xFC1B},
{0xFC1C,Lo,0xFC1C,0xFC1C,0xFC1C},
{0xFC1D,Lo,0xFC1D,0xFC1D,0xFC1D},
{0xFC1E,Lo,0xFC1E,0xFC1E,0xFC1E},
{0xFC1F,Lo,0xFC1F,0xFC1F,0xFC1F},
{0xFC20,Lo,0xFC20,0xFC20,0xFC20},
{0xFC21,Lo,0xFC21,0xFC21,0xFC21},
{0xFC22,Lo,0xFC22,0xFC22,0xFC22},
{0xFC23,Lo,0xFC23,0xFC23,0xFC23},
{0xFC24,Lo,0xFC24,0xFC24,0xFC24},
{0xFC25,Lo,0xFC25,0xFC25,0xFC25},
{0xFC26,Lo,0xFC26,0xFC26,0xFC26},
{0xFC27,Lo,0xFC27,0xFC27,0xFC27},
{0xFC28,Lo,0xFC28,0xFC28,0xFC28},
{0xFC29,Lo,0xFC29,0xFC29,0xFC29},
{0xFC2A,Lo,0xFC2A,0xFC2A,0xFC2A},
{0xFC2B,Lo,0xFC2B,0xFC2B,0xFC2B},
{0xFC2C,Lo,0xFC2C,0xFC2C,0xFC2C},
{0xFC2D,Lo,0xFC2D,0xFC2D,0xFC2D},
{0xFC2E,Lo,0xFC2E,0xFC2E,0xFC2E},
{0xFC2F,Lo,0xFC2F,0xFC2F,0xFC2F},
{0xFC30,Lo,0xFC30,0xFC30,0xFC30},
{0xFC31,Lo,0xFC31,0xFC31,0xFC31},
{0xFC32,Lo,0xFC32,0xFC32,0xFC32},
{0xFC33,Lo,0xFC33,0xFC33,0xFC33},
{0xFC34,Lo,0xFC34,0xFC34,0xFC34},
{0xFC35,Lo,0xFC35,0xFC35,0xFC35},
{0xFC36,Lo,0xFC36,0xFC36,0xFC36},
{0xFC37,Lo,0xFC37,0xFC37,0xFC37},
{0xFC38,Lo,0xFC38,0xFC38,0xFC38},
{0xFC39,Lo,0xFC39,0xFC39,0xFC39},
{0xFC3A,Lo,0xFC3A,0xFC3A,0xFC3A},
{0xFC3B,Lo,0xFC3B,0xFC3B,0xFC3B},
{0xFC3C,Lo,0xFC3C,0xFC3C,0xFC3C},
{0xFC3D,Lo,0xFC3D,0xFC3D,0xFC3D},
{0xFC3E,Lo,0xFC3E,0xFC3E,0xFC3E},
{0xFC3F,Lo,0xFC3F,0xFC3F,0xFC3F},
{0xFC40,Lo,0xFC40,0xFC40,0xFC40},
{0xFC41,Lo,0xFC41,0xFC41,0xFC41},
{0xFC42,Lo,0xFC42,0xFC42,0xFC42},
{0xFC43,Lo,0xFC43,0xFC43,0xFC43},
{0xFC44,Lo,0xFC44,0xFC44,0xFC44},
{0xFC45,Lo,0xFC45,0xFC45,0xFC45},
{0xFC46,Lo,0xFC46,0xFC46,0xFC46},
{0xFC47,Lo,0xFC47,0xFC47,0xFC47},
{0xFC48,Lo,0xFC48,0xFC48,0xFC48},
{0xFC49,Lo,0xFC49,0xFC49,0xFC49},
{0xFC4A,Lo,0xFC4A,0xFC4A,0xFC4A},
{0xFC4B,Lo,0xFC4B,0xFC4B,0xFC4B},
{0xFC4C,Lo,0xFC4C,0xFC4C,0xFC4C},
{0xFC4D,Lo,0xFC4D,0xFC4D,0xFC4D},
{0xFC4E,Lo,0xFC4E,0xFC4E,0xFC4E},
{0xFC4F,Lo,0xFC4F,0xFC4F,0xFC4F},
{0xFC50,Lo,0xFC50,0xFC50,0xFC50},
{0xFC51,Lo,0xFC51,0xFC51,0xFC51},
{0xFC52,Lo,0xFC52,0xFC52,0xFC52},
{0xFC53,Lo,0xFC53,0xFC53,0xFC53},
{0xFC54,Lo,0xFC54,0xFC54,0xFC54},
{0xFC55,Lo,0xFC55,0xFC55,0xFC55},
{0xFC56,Lo,0xFC56,0xFC56,0xFC56},
{0xFC57,Lo,0xFC57,0xFC57,0xFC57},
{0xFC58,Lo,0xFC58,0xFC58,0xFC58},
{0xFC59,Lo,0xFC59,0xFC59,0xFC59},
{0xFC5A,Lo,0xFC5A,0xFC5A,0xFC5A},
{0xFC5B,Lo,0xFC5B,0xFC5B,0xFC5B},
{0xFC5C,Lo,0xFC5C,0xFC5C,0xFC5C},
{0xFC5D,Lo,0xFC5D,0xFC5D,0xFC5D},
{0xFC5E,Lo,0xFC5E,0xFC5E,0xFC5E},
{0xFC5F,Lo,0xFC5F,0xFC5F,0xFC5F},
{0xFC60,Lo,0xFC60,0xFC60,0xFC60},
{0xFC61,Lo,0xFC61,0xFC61,0xFC61},
{0xFC62,Lo,0xFC62,0xFC62,0xFC62},
{0xFC63,Lo,0xFC63,0xFC63,0xFC63},
{0xFC64,Lo,0xFC64,0xFC64,0xFC64},
{0xFC65,Lo,0xFC65,0xFC65,0xFC65},
{0xFC66,Lo,0xFC66,0xFC66,0xFC66},
{0xFC67,Lo,0xFC67,0xFC67,0xFC67},
{0xFC68,Lo,0xFC68,0xFC68,0xFC68},
{0xFC69,Lo,0xFC69,0xFC69,0xFC69},
{0xFC6A,Lo,0xFC6A,0xFC6A,0xFC6A},
{0xFC6B,Lo,0xFC6B,0xFC6B,0xFC6B},
{0xFC6C,Lo,0xFC6C,0xFC6C,0xFC6C},
{0xFC6D,Lo,0xFC6D,0xFC6D,0xFC6D},
{0xFC6E,Lo,0xFC6E,0xFC6E,0xFC6E},
{0xFC6F,Lo,0xFC6F,0xFC6F,0xFC6F},
{0xFC70,Lo,0xFC70,0xFC70,0xFC70},
{0xFC71,Lo,0xFC71,0xFC71,0xFC71},
{0xFC72,Lo,0xFC72,0xFC72,0xFC72},
{0xFC73,Lo,0xFC73,0xFC73,0xFC73},
{0xFC74,Lo,0xFC74,0xFC74,0xFC74},
{0xFC75,Lo,0xFC75,0xFC75,0xFC75},
{0xFC76,Lo,0xFC76,0xFC76,0xFC76},
{0xFC77,Lo,0xFC77,0xFC77,0xFC77},
{0xFC78,Lo,0xFC78,0xFC78,0xFC78},
{0xFC79,Lo,0xFC79,0xFC79,0xFC79},
{0xFC7A,Lo,0xFC7A,0xFC7A,0xFC7A},
{0xFC7B,Lo,0xFC7B,0xFC7B,0xFC7B},
{0xFC7C,Lo,0xFC7C,0xFC7C,0xFC7C},
{0xFC7D,Lo,0xFC7D,0xFC7D,0xFC7D},
{0xFC7E,Lo,0xFC7E,0xFC7E,0xFC7E},
{0xFC7F,Lo,0xFC7F,0xFC7F,0xFC7F},
{0xFC80,Lo,0xFC80,0xFC80,0xFC80},
{0xFC81,Lo,0xFC81,0xFC81,0xFC81},
{0xFC82,Lo,0xFC82,0xFC82,0xFC82},
{0xFC83,Lo,0xFC83,0xFC83,0xFC83},
{0xFC84,Lo,0xFC84,0xFC84,0xFC84},
{0xFC85,Lo,0xFC85,0xFC85,0xFC85},
{0xFC86,Lo,0xFC86,0xFC86,0xFC86},
{0xFC87,Lo,0xFC87,0xFC87,0xFC87},
{0xFC88,Lo,0xFC88,0xFC88,0xFC88},
{0xFC89,Lo,0xFC89,0xFC89,0xFC89},
{0xFC8A,Lo,0xFC8A,0xFC8A,0xFC8A},
{0xFC8B,Lo,0xFC8B,0xFC8B,0xFC8B},
{0xFC8C,Lo,0xFC8C,0xFC8C,0xFC8C},
{0xFC8D,Lo,0xFC8D,0xFC8D,0xFC8D},
{0xFC8E,Lo,0xFC8E,0xFC8E,0xFC8E},
{0xFC8F,Lo,0xFC8F,0xFC8F,0xFC8F},
{0xFC90,Lo,0xFC90,0xFC90,0xFC90},
{0xFC91,Lo,0xFC91,0xFC91,0xFC91},
{0xFC92,Lo,0xFC92,0xFC92,0xFC92},
{0xFC93,Lo,0xFC93,0xFC93,0xFC93},
{0xFC94,Lo,0xFC94,0xFC94,0xFC94},
{0xFC95,Lo,0xFC95,0xFC95,0xFC95},
{0xFC96,Lo,0xFC96,0xFC96,0xFC96},
{0xFC97,Lo,0xFC97,0xFC97,0xFC97},
{0xFC98,Lo,0xFC98,0xFC98,0xFC98},
{0xFC99,Lo,0xFC99,0xFC99,0xFC99},
{0xFC9A,Lo,0xFC9A,0xFC9A,0xFC9A},
{0xFC9B,Lo,0xFC9B,0xFC9B,0xFC9B},
{0xFC9C,Lo,0xFC9C,0xFC9C,0xFC9C},
{0xFC9D,Lo,0xFC9D,0xFC9D,0xFC9D},
{0xFC9E,Lo,0xFC9E,0xFC9E,0xFC9E},
{0xFC9F,Lo,0xFC9F,0xFC9F,0xFC9F},
{0xFCA0,Lo,0xFCA0,0xFCA0,0xFCA0},
{0xFCA1,Lo,0xFCA1,0xFCA1,0xFCA1},
{0xFCA2,Lo,0xFCA2,0xFCA2,0xFCA2},
{0xFCA3,Lo,0xFCA3,0xFCA3,0xFCA3},
{0xFCA4,Lo,0xFCA4,0xFCA4,0xFCA4},
{0xFCA5,Lo,0xFCA5,0xFCA5,0xFCA5},
{0xFCA6,Lo,0xFCA6,0xFCA6,0xFCA6},
{0xFCA7,Lo,0xFCA7,0xFCA7,0xFCA7},
{0xFCA8,Lo,0xFCA8,0xFCA8,0xFCA8},
{0xFCA9,Lo,0xFCA9,0xFCA9,0xFCA9},
{0xFCAA,Lo,0xFCAA,0xFCAA,0xFCAA},
{0xFCAB,Lo,0xFCAB,0xFCAB,0xFCAB},
{0xFCAC,Lo,0xFCAC,0xFCAC,0xFCAC},
{0xFCAD,Lo,0xFCAD,0xFCAD,0xFCAD},
{0xFCAE,Lo,0xFCAE,0xFCAE,0xFCAE},
{0xFCAF,Lo,0xFCAF,0xFCAF,0xFCAF},
{0xFCB0,Lo,0xFCB0,0xFCB0,0xFCB0},
{0xFCB1,Lo,0xFCB1,0xFCB1,0xFCB1},
{0xFCB2,Lo,0xFCB2,0xFCB2,0xFCB2},
{0xFCB3,Lo,0xFCB3,0xFCB3,0xFCB3},
{0xFCB4,Lo,0xFCB4,0xFCB4,0xFCB4},
{0xFCB5,Lo,0xFCB5,0xFCB5,0xFCB5},
{0xFCB6,Lo,0xFCB6,0xFCB6,0xFCB6},
{0xFCB7,Lo,0xFCB7,0xFCB7,0xFCB7},
{0xFCB8,Lo,0xFCB8,0xFCB8,0xFCB8},
{0xFCB9,Lo,0xFCB9,0xFCB9,0xFCB9},
{0xFCBA,Lo,0xFCBA,0xFCBA,0xFCBA},
{0xFCBB,Lo,0xFCBB,0xFCBB,0xFCBB},
{0xFCBC,Lo,0xFCBC,0xFCBC,0xFCBC},
{0xFCBD,Lo,0xFCBD,0xFCBD,0xFCBD},
{0xFCBE,Lo,0xFCBE,0xFCBE,0xFCBE},
{0xFCBF,Lo,0xFCBF,0xFCBF,0xFCBF},
{0xFCC0,Lo,0xFCC0,0xFCC0,0xFCC0},
{0xFCC1,Lo,0xFCC1,0xFCC1,0xFCC1},
{0xFCC2,Lo,0xFCC2,0xFCC2,0xFCC2},
{0xFCC3,Lo,0xFCC3,0xFCC3,0xFCC3},
{0xFCC4,Lo,0xFCC4,0xFCC4,0xFCC4},
{0xFCC5,Lo,0xFCC5,0xFCC5,0xFCC5},
{0xFCC6,Lo,0xFCC6,0xFCC6,0xFCC6},
{0xFCC7,Lo,0xFCC7,0xFCC7,0xFCC7},
{0xFCC8,Lo,0xFCC8,0xFCC8,0xFCC8},
{0xFCC9,Lo,0xFCC9,0xFCC9,0xFCC9},
{0xFCCA,Lo,0xFCCA,0xFCCA,0xFCCA},
{0xFCCB,Lo,0xFCCB,0xFCCB,0xFCCB},
{0xFCCC,Lo,0xFCCC,0xFCCC,0xFCCC},
{0xFCCD,Lo,0xFCCD,0xFCCD,0xFCCD},
{0xFCCE,Lo,0xFCCE,0xFCCE,0xFCCE},
{0xFCCF,Lo,0xFCCF,0xFCCF,0xFCCF},
{0xFCD0,Lo,0xFCD0,0xFCD0,0xFCD0},
{0xFCD1,Lo,0xFCD1,0xFCD1,0xFCD1},
{0xFCD2,Lo,0xFCD2,0xFCD2,0xFCD2},
{0xFCD3,Lo,0xFCD3,0xFCD3,0xFCD3},
{0xFCD4,Lo,0xFCD4,0xFCD4,0xFCD4},
{0xFCD5,Lo,0xFCD5,0xFCD5,0xFCD5},
{0xFCD6,Lo,0xFCD6,0xFCD6,0xFCD6},
{0xFCD7,Lo,0xFCD7,0xFCD7,0xFCD7},
{0xFCD8,Lo,0xFCD8,0xFCD8,0xFCD8},
{0xFCD9,Lo,0xFCD9,0xFCD9,0xFCD9},
{0xFCDA,Lo,0xFCDA,0xFCDA,0xFCDA},
{0xFCDB,Lo,0xFCDB,0xFCDB,0xFCDB},
{0xFCDC,Lo,0xFCDC,0xFCDC,0xFCDC},
{0xFCDD,Lo,0xFCDD,0xFCDD,0xFCDD},
{0xFCDE,Lo,0xFCDE,0xFCDE,0xFCDE},
{0xFCDF,Lo,0xFCDF,0xFCDF,0xFCDF},
{0xFCE0,Lo,0xFCE0,0xFCE0,0xFCE0},
{0xFCE1,Lo,0xFCE1,0xFCE1,0xFCE1},
{0xFCE2,Lo,0xFCE2,0xFCE2,0xFCE2},
{0xFCE3,Lo,0xFCE3,0xFCE3,0xFCE3},
{0xFCE4,Lo,0xFCE4,0xFCE4,0xFCE4},
{0xFCE5,Lo,0xFCE5,0xFCE5,0xFCE5},
{0xFCE6,Lo,0xFCE6,0xFCE6,0xFCE6},
{0xFCE7,Lo,0xFCE7,0xFCE7,0xFCE7},
{0xFCE8,Lo,0xFCE8,0xFCE8,0xFCE8},
{0xFCE9,Lo,0xFCE9,0xFCE9,0xFCE9},
{0xFCEA,Lo,0xFCEA,0xFCEA,0xFCEA},
{0xFCEB,Lo,0xFCEB,0xFCEB,0xFCEB},
{0xFCEC,Lo,0xFCEC,0xFCEC,0xFCEC},
{0xFCED,Lo,0xFCED,0xFCED,0xFCED},
{0xFCEE,Lo,0xFCEE,0xFCEE,0xFCEE},
{0xFCEF,Lo,0xFCEF,0xFCEF,0xFCEF},
{0xFCF0,Lo,0xFCF0,0xFCF0,0xFCF0},
{0xFCF1,Lo,0xFCF1,0xFCF1,0xFCF1},
{0xFCF2,Lo,0xFCF2,0xFCF2,0xFCF2},
{0xFCF3,Lo,0xFCF3,0xFCF3,0xFCF3},
{0xFCF4,Lo,0xFCF4,0xFCF4,0xFCF4},
{0xFCF5,Lo,0xFCF5,0xFCF5,0xFCF5},
{0xFCF6,Lo,0xFCF6,0xFCF6,0xFCF6},
{0xFCF7,Lo,0xFCF7,0xFCF7,0xFCF7},
{0xFCF8,Lo,0xFCF8,0xFCF8,0xFCF8},
{0xFCF9,Lo,0xFCF9,0xFCF9,0xFCF9},
{0xFCFA,Lo,0xFCFA,0xFCFA,0xFCFA},
{0xFCFB,Lo,0xFCFB,0xFCFB,0xFCFB},
{0xFCFC,Lo,0xFCFC,0xFCFC,0xFCFC},
{0xFCFD,Lo,0xFCFD,0xFCFD,0xFCFD},
{0xFCFE,Lo,0xFCFE,0xFCFE,0xFCFE},
{0xFCFF,Lo,0xFCFF,0xFCFF,0xFCFF},
{0xFD00,Lo,0xFD00,0xFD00,0xFD00},
{0xFD01,Lo,0xFD01,0xFD01,0xFD01},
{0xFD02,Lo,0xFD02,0xFD02,0xFD02},
{0xFD03,Lo,0xFD03,0xFD03,0xFD03},
{0xFD04,Lo,0xFD04,0xFD04,0xFD04},
{0xFD05,Lo,0xFD05,0xFD05,0xFD05},
{0xFD06,Lo,0xFD06,0xFD06,0xFD06},
{0xFD07,Lo,0xFD07,0xFD07,0xFD07},
{0xFD08,Lo,0xFD08,0xFD08,0xFD08},
{0xFD09,Lo,0xFD09,0xFD09,0xFD09},
{0xFD0A,Lo,0xFD0A,0xFD0A,0xFD0A},
{0xFD0B,Lo,0xFD0B,0xFD0B,0xFD0B},
{0xFD0C,Lo,0xFD0C,0xFD0C,0xFD0C},
{0xFD0D,Lo,0xFD0D,0xFD0D,0xFD0D},
{0xFD0E,Lo,0xFD0E,0xFD0E,0xFD0E},
{0xFD0F,Lo,0xFD0F,0xFD0F,0xFD0F},
{0xFD10,Lo,0xFD10,0xFD10,0xFD10},
{0xFD11,Lo,0xFD11,0xFD11,0xFD11},
{0xFD12,Lo,0xFD12,0xFD12,0xFD12},
{0xFD13,Lo,0xFD13,0xFD13,0xFD13},
{0xFD14,Lo,0xFD14,0xFD14,0xFD14},
{0xFD15,Lo,0xFD15,0xFD15,0xFD15},
{0xFD16,Lo,0xFD16,0xFD16,0xFD16},
{0xFD17,Lo,0xFD17,0xFD17,0xFD17},
{0xFD18,Lo,0xFD18,0xFD18,0xFD18},
{0xFD19,Lo,0xFD19,0xFD19,0xFD19},
{0xFD1A,Lo,0xFD1A,0xFD1A,0xFD1A},
{0xFD1B,Lo,0xFD1B,0xFD1B,0xFD1B},
{0xFD1C,Lo,0xFD1C,0xFD1C,0xFD1C},
{0xFD1D,Lo,0xFD1D,0xFD1D,0xFD1D},
{0xFD1E,Lo,0xFD1E,0xFD1E,0xFD1E},
{0xFD1F,Lo,0xFD1F,0xFD1F,0xFD1F},
{0xFD20,Lo,0xFD20,0xFD20,0xFD20},
{0xFD21,Lo,0xFD21,0xFD21,0xFD21},
{0xFD22,Lo,0xFD22,0xFD22,0xFD22},
{0xFD23,Lo,0xFD23,0xFD23,0xFD23},
{0xFD24,Lo,0xFD24,0xFD24,0xFD24},
{0xFD25,Lo,0xFD25,0xFD25,0xFD25},
{0xFD26,Lo,0xFD26,0xFD26,0xFD26},
{0xFD27,Lo,0xFD27,0xFD27,0xFD27},
{0xFD28,Lo,0xFD28,0xFD28,0xFD28},
{0xFD29,Lo,0xFD29,0xFD29,0xFD29},
{0xFD2A,Lo,0xFD2A,0xFD2A,0xFD2A},
{0xFD2B,Lo,0xFD2B,0xFD2B,0xFD2B},
{0xFD2C,Lo,0xFD2C,0xFD2C,0xFD2C},
{0xFD2D,Lo,0xFD2D,0xFD2D,0xFD2D},
{0xFD2E,Lo,0xFD2E,0xFD2E,0xFD2E},
{0xFD2F,Lo,0xFD2F,0xFD2F,0xFD2F},
{0xFD30,Lo,0xFD30,0xFD30,0xFD30},
{0xFD31,Lo,0xFD31,0xFD31,0xFD31},
{0xFD32,Lo,0xFD32,0xFD32,0xFD32},
{0xFD33,Lo,0xFD33,0xFD33,0xFD33},
{0xFD34,Lo,0xFD34,0xFD34,0xFD34},
{0xFD35,Lo,0xFD35,0xFD35,0xFD35},
{0xFD36,Lo,0xFD36,0xFD36,0xFD36},
{0xFD37,Lo,0xFD37,0xFD37,0xFD37},
{0xFD38,Lo,0xFD38,0xFD38,0xFD38},
{0xFD39,Lo,0xFD39,0xFD39,0xFD39},
{0xFD3A,Lo,0xFD3A,0xFD3A,0xFD3A},
{0xFD3B,Lo,0xFD3B,0xFD3B,0xFD3B},
{0xFD3C,Lo,0xFD3C,0xFD3C,0xFD3C},
{0xFD3D,Lo,0xFD3D,0xFD3D,0xFD3D},
{0xFD3E,Ps,0xFD3E,0xFD3E,0xFD3E},
{0xFD3F,Pe,0xFD3F,0xFD3F,0xFD3F},
{0xFD50,Lo,0xFD50,0xFD50,0xFD50},
{0xFD51,Lo,0xFD51,0xFD51,0xFD51},
{0xFD52,Lo,0xFD52,0xFD52,0xFD52},
{0xFD53,Lo,0xFD53,0xFD53,0xFD53},
{0xFD54,Lo,0xFD54,0xFD54,0xFD54},
{0xFD55,Lo,0xFD55,0xFD55,0xFD55},
{0xFD56,Lo,0xFD56,0xFD56,0xFD56},
{0xFD57,Lo,0xFD57,0xFD57,0xFD57},
{0xFD58,Lo,0xFD58,0xFD58,0xFD58},
{0xFD59,Lo,0xFD59,0xFD59,0xFD59},
{0xFD5A,Lo,0xFD5A,0xFD5A,0xFD5A},
{0xFD5B,Lo,0xFD5B,0xFD5B,0xFD5B},
{0xFD5C,Lo,0xFD5C,0xFD5C,0xFD5C},
{0xFD5D,Lo,0xFD5D,0xFD5D,0xFD5D},
{0xFD5E,Lo,0xFD5E,0xFD5E,0xFD5E},
{0xFD5F,Lo,0xFD5F,0xFD5F,0xFD5F},
{0xFD60,Lo,0xFD60,0xFD60,0xFD60},
{0xFD61,Lo,0xFD61,0xFD61,0xFD61},
{0xFD62,Lo,0xFD62,0xFD62,0xFD62},
{0xFD63,Lo,0xFD63,0xFD63,0xFD63},
{0xFD64,Lo,0xFD64,0xFD64,0xFD64},
{0xFD65,Lo,0xFD65,0xFD65,0xFD65},
{0xFD66,Lo,0xFD66,0xFD66,0xFD66},
{0xFD67,Lo,0xFD67,0xFD67,0xFD67},
{0xFD68,Lo,0xFD68,0xFD68,0xFD68},
{0xFD69,Lo,0xFD69,0xFD69,0xFD69},
{0xFD6A,Lo,0xFD6A,0xFD6A,0xFD6A},
{0xFD6B,Lo,0xFD6B,0xFD6B,0xFD6B},
{0xFD6C,Lo,0xFD6C,0xFD6C,0xFD6C},
{0xFD6D,Lo,0xFD6D,0xFD6D,0xFD6D},
{0xFD6E,Lo,0xFD6E,0xFD6E,0xFD6E},
{0xFD6F,Lo,0xFD6F,0xFD6F,0xFD6F},
{0xFD70,Lo,0xFD70,0xFD70,0xFD70},
{0xFD71,Lo,0xFD71,0xFD71,0xFD71},
{0xFD72,Lo,0xFD72,0xFD72,0xFD72},
{0xFD73,Lo,0xFD73,0xFD73,0xFD73},
{0xFD74,Lo,0xFD74,0xFD74,0xFD74},
{0xFD75,Lo,0xFD75,0xFD75,0xFD75},
{0xFD76,Lo,0xFD76,0xFD76,0xFD76},
{0xFD77,Lo,0xFD77,0xFD77,0xFD77},
{0xFD78,Lo,0xFD78,0xFD78,0xFD78},
{0xFD79,Lo,0xFD79,0xFD79,0xFD79},
{0xFD7A,Lo,0xFD7A,0xFD7A,0xFD7A},
{0xFD7B,Lo,0xFD7B,0xFD7B,0xFD7B},
{0xFD7C,Lo,0xFD7C,0xFD7C,0xFD7C},
{0xFD7D,Lo,0xFD7D,0xFD7D,0xFD7D},
{0xFD7E,Lo,0xFD7E,0xFD7E,0xFD7E},
{0xFD7F,Lo,0xFD7F,0xFD7F,0xFD7F},
{0xFD80,Lo,0xFD80,0xFD80,0xFD80},
{0xFD81,Lo,0xFD81,0xFD81,0xFD81},
{0xFD82,Lo,0xFD82,0xFD82,0xFD82},
{0xFD83,Lo,0xFD83,0xFD83,0xFD83},
{0xFD84,Lo,0xFD84,0xFD84,0xFD84},
{0xFD85,Lo,0xFD85,0xFD85,0xFD85},
{0xFD86,Lo,0xFD86,0xFD86,0xFD86},
{0xFD87,Lo,0xFD87,0xFD87,0xFD87},
{0xFD88,Lo,0xFD88,0xFD88,0xFD88},
{0xFD89,Lo,0xFD89,0xFD89,0xFD89},
{0xFD8A,Lo,0xFD8A,0xFD8A,0xFD8A},
{0xFD8B,Lo,0xFD8B,0xFD8B,0xFD8B},
{0xFD8C,Lo,0xFD8C,0xFD8C,0xFD8C},
{0xFD8D,Lo,0xFD8D,0xFD8D,0xFD8D},
{0xFD8E,Lo,0xFD8E,0xFD8E,0xFD8E},
{0xFD8F,Lo,0xFD8F,0xFD8F,0xFD8F},
{0xFD92,Lo,0xFD92,0xFD92,0xFD92},
{0xFD93,Lo,0xFD93,0xFD93,0xFD93},
{0xFD94,Lo,0xFD94,0xFD94,0xFD94},
{0xFD95,Lo,0xFD95,0xFD95,0xFD95},
{0xFD96,Lo,0xFD96,0xFD96,0xFD96},
{0xFD97,Lo,0xFD97,0xFD97,0xFD97},
{0xFD98,Lo,0xFD98,0xFD98,0xFD98},
{0xFD99,Lo,0xFD99,0xFD99,0xFD99},
{0xFD9A,Lo,0xFD9A,0xFD9A,0xFD9A},
{0xFD9B,Lo,0xFD9B,0xFD9B,0xFD9B},
{0xFD9C,Lo,0xFD9C,0xFD9C,0xFD9C},
{0xFD9D,Lo,0xFD9D,0xFD9D,0xFD9D},
{0xFD9E,Lo,0xFD9E,0xFD9E,0xFD9E},
{0xFD9F,Lo,0xFD9F,0xFD9F,0xFD9F},
{0xFDA0,Lo,0xFDA0,0xFDA0,0xFDA0},
{0xFDA1,Lo,0xFDA1,0xFDA1,0xFDA1},
{0xFDA2,Lo,0xFDA2,0xFDA2,0xFDA2},
{0xFDA3,Lo,0xFDA3,0xFDA3,0xFDA3},
{0xFDA4,Lo,0xFDA4,0xFDA4,0xFDA4},
{0xFDA5,Lo,0xFDA5,0xFDA5,0xFDA5},
{0xFDA6,Lo,0xFDA6,0xFDA6,0xFDA6},
{0xFDA7,Lo,0xFDA7,0xFDA7,0xFDA7},
{0xFDA8,Lo,0xFDA8,0xFDA8,0xFDA8},
{0xFDA9,Lo,0xFDA9,0xFDA9,0xFDA9},
{0xFDAA,Lo,0xFDAA,0xFDAA,0xFDAA},
{0xFDAB,Lo,0xFDAB,0xFDAB,0xFDAB},
{0xFDAC,Lo,0xFDAC,0xFDAC,0xFDAC},
{0xFDAD,Lo,0xFDAD,0xFDAD,0xFDAD},
{0xFDAE,Lo,0xFDAE,0xFDAE,0xFDAE},
{0xFDAF,Lo,0xFDAF,0xFDAF,0xFDAF},
{0xFDB0,Lo,0xFDB0,0xFDB0,0xFDB0},
{0xFDB1,Lo,0xFDB1,0xFDB1,0xFDB1},
{0xFDB2,Lo,0xFDB2,0xFDB2,0xFDB2},
{0xFDB3,Lo,0xFDB3,0xFDB3,0xFDB3},
{0xFDB4,Lo,0xFDB4,0xFDB4,0xFDB4},
{0xFDB5,Lo,0xFDB5,0xFDB5,0xFDB5},
{0xFDB6,Lo,0xFDB6,0xFDB6,0xFDB6},
{0xFDB7,Lo,0xFDB7,0xFDB7,0xFDB7},
{0xFDB8,Lo,0xFDB8,0xFDB8,0xFDB8},
{0xFDB9,Lo,0xFDB9,0xFDB9,0xFDB9},
{0xFDBA,Lo,0xFDBA,0xFDBA,0xFDBA},
{0xFDBB,Lo,0xFDBB,0xFDBB,0xFDBB},
{0xFDBC,Lo,0xFDBC,0xFDBC,0xFDBC},
{0xFDBD,Lo,0xFDBD,0xFDBD,0xFDBD},
{0xFDBE,Lo,0xFDBE,0xFDBE,0xFDBE},
{0xFDBF,Lo,0xFDBF,0xFDBF,0xFDBF},
{0xFDC0,Lo,0xFDC0,0xFDC0,0xFDC0},
{0xFDC1,Lo,0xFDC1,0xFDC1,0xFDC1},
{0xFDC2,Lo,0xFDC2,0xFDC2,0xFDC2},
{0xFDC3,Lo,0xFDC3,0xFDC3,0xFDC3},
{0xFDC4,Lo,0xFDC4,0xFDC4,0xFDC4},
{0xFDC5,Lo,0xFDC5,0xFDC5,0xFDC5},
{0xFDC6,Lo,0xFDC6,0xFDC6,0xFDC6},
{0xFDC7,Lo,0xFDC7,0xFDC7,0xFDC7},
{0xFDF0,Lo,0xFDF0,0xFDF0,0xFDF0},
{0xFDF1,Lo,0xFDF1,0xFDF1,0xFDF1},
{0xFDF2,Lo,0xFDF2,0xFDF2,0xFDF2},
{0xFDF3,Lo,0xFDF3,0xFDF3,0xFDF3},
{0xFDF4,Lo,0xFDF4,0xFDF4,0xFDF4},
{0xFDF5,Lo,0xFDF5,0xFDF5,0xFDF5},
{0xFDF6,Lo,0xFDF6,0xFDF6,0xFDF6},
{0xFDF7,Lo,0xFDF7,0xFDF7,0xFDF7},
{0xFDF8,Lo,0xFDF8,0xFDF8,0xFDF8},
{0xFDF9,Lo,0xFDF9,0xFDF9,0xFDF9},
{0xFDFA,Lo,0xFDFA,0xFDFA,0xFDFA},
{0xFDFB,Lo,0xFDFB,0xFDFB,0xFDFB},
{0xFDFC,Sc,0xFDFC,0xFDFC,0xFDFC},
{0xFDFD,So,0xFDFD,0xFDFD,0xFDFD},
{0xFE00,Mn,0xFE00,0xFE00,0xFE00},
{0xFE01,Mn,0xFE01,0xFE01,0xFE01},
{0xFE02,Mn,0xFE02,0xFE02,0xFE02},
{0xFE03,Mn,0xFE03,0xFE03,0xFE03},
{0xFE04,Mn,0xFE04,0xFE04,0xFE04},
{0xFE05,Mn,0xFE05,0xFE05,0xFE05},
{0xFE06,Mn,0xFE06,0xFE06,0xFE06},
{0xFE07,Mn,0xFE07,0xFE07,0xFE07},
{0xFE08,Mn,0xFE08,0xFE08,0xFE08},
{0xFE09,Mn,0xFE09,0xFE09,0xFE09},
{0xFE0A,Mn,0xFE0A,0xFE0A,0xFE0A},
{0xFE0B,Mn,0xFE0B,0xFE0B,0xFE0B},
{0xFE0C,Mn,0xFE0C,0xFE0C,0xFE0C},
{0xFE0D,Mn,0xFE0D,0xFE0D,0xFE0D},
{0xFE0E,Mn,0xFE0E,0xFE0E,0xFE0E},
{0xFE0F,Mn,0xFE0F,0xFE0F,0xFE0F},
{0xFE10,Po,0xFE10,0xFE10,0xFE10},
{0xFE11,Po,0xFE11,0xFE11,0xFE11},
{0xFE12,Po,0xFE12,0xFE12,0xFE12},
{0xFE13,Po,0xFE13,0xFE13,0xFE13},
{0xFE14,Po,0xFE14,0xFE14,0xFE14},
{0xFE15,Po,0xFE15,0xFE15,0xFE15},
{0xFE16,Po,0xFE16,0xFE16,0xFE16},
{0xFE17,Ps,0xFE17,0xFE17,0xFE17},
{0xFE18,Pe,0xFE18,0xFE18,0xFE18},
{0xFE19,Po,0xFE19,0xFE19,0xFE19},
{0xFE20,Mn,0xFE20,0xFE20,0xFE20},
{0xFE21,Mn,0xFE21,0xFE21,0xFE21},
{0xFE22,Mn,0xFE22,0xFE22,0xFE22},
{0xFE23,Mn,0xFE23,0xFE23,0xFE23},
{0xFE30,Po,0xFE30,0xFE30,0xFE30},
{0xFE31,Pd,0xFE31,0xFE31,0xFE31},
{0xFE32,Pd,0xFE32,0xFE32,0xFE32},
{0xFE33,Pc,0xFE33,0xFE33,0xFE33},
{0xFE34,Pc,0xFE34,0xFE34,0xFE34},
{0xFE35,Ps,0xFE35,0xFE35,0xFE35},
{0xFE36,Pe,0xFE36,0xFE36,0xFE36},
{0xFE37,Ps,0xFE37,0xFE37,0xFE37},
{0xFE38,Pe,0xFE38,0xFE38,0xFE38},
{0xFE39,Ps,0xFE39,0xFE39,0xFE39},
{0xFE3A,Pe,0xFE3A,0xFE3A,0xFE3A},
{0xFE3B,Ps,0xFE3B,0xFE3B,0xFE3B},
{0xFE3C,Pe,0xFE3C,0xFE3C,0xFE3C},
{0xFE3D,Ps,0xFE3D,0xFE3D,0xFE3D},
{0xFE3E,Pe,0xFE3E,0xFE3E,0xFE3E},
{0xFE3F,Ps,0xFE3F,0xFE3F,0xFE3F},
{0xFE40,Pe,0xFE40,0xFE40,0xFE40},
{0xFE41,Ps,0xFE41,0xFE41,0xFE41},
{0xFE42,Pe,0xFE42,0xFE42,0xFE42},
{0xFE43,Ps,0xFE43,0xFE43,0xFE43},
{0xFE44,Pe,0xFE44,0xFE44,0xFE44},
{0xFE45,Po,0xFE45,0xFE45,0xFE45},
{0xFE46,Po,0xFE46,0xFE46,0xFE46},
{0xFE47,Ps,0xFE47,0xFE47,0xFE47},
{0xFE48,Pe,0xFE48,0xFE48,0xFE48},
{0xFE49,Po,0xFE49,0xFE49,0xFE49},
{0xFE4A,Po,0xFE4A,0xFE4A,0xFE4A},
{0xFE4B,Po,0xFE4B,0xFE4B,0xFE4B},
{0xFE4C,Po,0xFE4C,0xFE4C,0xFE4C},
{0xFE4D,Pc,0xFE4D,0xFE4D,0xFE4D},
{0xFE4E,Pc,0xFE4E,0xFE4E,0xFE4E},
{0xFE4F,Pc,0xFE4F,0xFE4F,0xFE4F},
{0xFE50,Po,0xFE50,0xFE50,0xFE50},
{0xFE51,Po,0xFE51,0xFE51,0xFE51},
{0xFE52,Po,0xFE52,0xFE52,0xFE52},
{0xFE54,Po,0xFE54,0xFE54,0xFE54},
{0xFE55,Po,0xFE55,0xFE55,0xFE55},
{0xFE56,Po,0xFE56,0xFE56,0xFE56},
{0xFE57,Po,0xFE57,0xFE57,0xFE57},
{0xFE58,Pd,0xFE58,0xFE58,0xFE58},
{0xFE59,Ps,0xFE59,0xFE59,0xFE59},
{0xFE5A,Pe,0xFE5A,0xFE5A,0xFE5A},
{0xFE5B,Ps,0xFE5B,0xFE5B,0xFE5B},
{0xFE5C,Pe,0xFE5C,0xFE5C,0xFE5C},
{0xFE5D,Ps,0xFE5D,0xFE5D,0xFE5D},
{0xFE5E,Pe,0xFE5E,0xFE5E,0xFE5E},
{0xFE5F,Po,0xFE5F,0xFE5F,0xFE5F},
{0xFE60,Po,0xFE60,0xFE60,0xFE60},
{0xFE61,Po,0xFE61,0xFE61,0xFE61},
{0xFE62,Sm,0xFE62,0xFE62,0xFE62},
{0xFE63,Pd,0xFE63,0xFE63,0xFE63},
{0xFE64,Sm,0xFE64,0xFE64,0xFE64},
{0xFE65,Sm,0xFE65,0xFE65,0xFE65},
{0xFE66,Sm,0xFE66,0xFE66,0xFE66},
{0xFE68,Po,0xFE68,0xFE68,0xFE68},
{0xFE69,Sc,0xFE69,0xFE69,0xFE69},
{0xFE6A,Po,0xFE6A,0xFE6A,0xFE6A},
{0xFE6B,Po,0xFE6B,0xFE6B,0xFE6B},
{0xFE70,Lo,0xFE70,0xFE70,0xFE70},
{0xFE71,Lo,0xFE71,0xFE71,0xFE71},
{0xFE72,Lo,0xFE72,0xFE72,0xFE72},
{0xFE73,Lo,0xFE73,0xFE73,0xFE73},
{0xFE74,Lo,0xFE74,0xFE74,0xFE74},
{0xFE76,Lo,0xFE76,0xFE76,0xFE76},
{0xFE77,Lo,0xFE77,0xFE77,0xFE77},
{0xFE78,Lo,0xFE78,0xFE78,0xFE78},
{0xFE79,Lo,0xFE79,0xFE79,0xFE79},
{0xFE7A,Lo,0xFE7A,0xFE7A,0xFE7A},
{0xFE7B,Lo,0xFE7B,0xFE7B,0xFE7B},
{0xFE7C,Lo,0xFE7C,0xFE7C,0xFE7C},
{0xFE7D,Lo,0xFE7D,0xFE7D,0xFE7D},
{0xFE7E,Lo,0xFE7E,0xFE7E,0xFE7E},
{0xFE7F,Lo,0xFE7F,0xFE7F,0xFE7F},
{0xFE80,Lo,0xFE80,0xFE80,0xFE80},
{0xFE81,Lo,0xFE81,0xFE81,0xFE81},
{0xFE82,Lo,0xFE82,0xFE82,0xFE82},
{0xFE83,Lo,0xFE83,0xFE83,0xFE83},
{0xFE84,Lo,0xFE84,0xFE84,0xFE84},
{0xFE85,Lo,0xFE85,0xFE85,0xFE85},
{0xFE86,Lo,0xFE86,0xFE86,0xFE86},
{0xFE87,Lo,0xFE87,0xFE87,0xFE87},
{0xFE88,Lo,0xFE88,0xFE88,0xFE88},
{0xFE89,Lo,0xFE89,0xFE89,0xFE89},
{0xFE8A,Lo,0xFE8A,0xFE8A,0xFE8A},
{0xFE8B,Lo,0xFE8B,0xFE8B,0xFE8B},
{0xFE8C,Lo,0xFE8C,0xFE8C,0xFE8C},
{0xFE8D,Lo,0xFE8D,0xFE8D,0xFE8D},
{0xFE8E,Lo,0xFE8E,0xFE8E,0xFE8E},
{0xFE8F,Lo,0xFE8F,0xFE8F,0xFE8F},
{0xFE90,Lo,0xFE90,0xFE90,0xFE90},
{0xFE91,Lo,0xFE91,0xFE91,0xFE91},
{0xFE92,Lo,0xFE92,0xFE92,0xFE92},
{0xFE93,Lo,0xFE93,0xFE93,0xFE93},
{0xFE94,Lo,0xFE94,0xFE94,0xFE94},
{0xFE95,Lo,0xFE95,0xFE95,0xFE95},
{0xFE96,Lo,0xFE96,0xFE96,0xFE96},
{0xFE97,Lo,0xFE97,0xFE97,0xFE97},
{0xFE98,Lo,0xFE98,0xFE98,0xFE98},
{0xFE99,Lo,0xFE99,0xFE99,0xFE99},
{0xFE9A,Lo,0xFE9A,0xFE9A,0xFE9A},
{0xFE9B,Lo,0xFE9B,0xFE9B,0xFE9B},
{0xFE9C,Lo,0xFE9C,0xFE9C,0xFE9C},
{0xFE9D,Lo,0xFE9D,0xFE9D,0xFE9D},
{0xFE9E,Lo,0xFE9E,0xFE9E,0xFE9E},
{0xFE9F,Lo,0xFE9F,0xFE9F,0xFE9F},
{0xFEA0,Lo,0xFEA0,0xFEA0,0xFEA0},
{0xFEA1,Lo,0xFEA1,0xFEA1,0xFEA1},
{0xFEA2,Lo,0xFEA2,0xFEA2,0xFEA2},
{0xFEA3,Lo,0xFEA3,0xFEA3,0xFEA3},
{0xFEA4,Lo,0xFEA4,0xFEA4,0xFEA4},
{0xFEA5,Lo,0xFEA5,0xFEA5,0xFEA5},
{0xFEA6,Lo,0xFEA6,0xFEA6,0xFEA6},
{0xFEA7,Lo,0xFEA7,0xFEA7,0xFEA7},
{0xFEA8,Lo,0xFEA8,0xFEA8,0xFEA8},
{0xFEA9,Lo,0xFEA9,0xFEA9,0xFEA9},
{0xFEAA,Lo,0xFEAA,0xFEAA,0xFEAA},
{0xFEAB,Lo,0xFEAB,0xFEAB,0xFEAB},
{0xFEAC,Lo,0xFEAC,0xFEAC,0xFEAC},
{0xFEAD,Lo,0xFEAD,0xFEAD,0xFEAD},
{0xFEAE,Lo,0xFEAE,0xFEAE,0xFEAE},
{0xFEAF,Lo,0xFEAF,0xFEAF,0xFEAF},
{0xFEB0,Lo,0xFEB0,0xFEB0,0xFEB0},
{0xFEB1,Lo,0xFEB1,0xFEB1,0xFEB1},
{0xFEB2,Lo,0xFEB2,0xFEB2,0xFEB2},
{0xFEB3,Lo,0xFEB3,0xFEB3,0xFEB3},
{0xFEB4,Lo,0xFEB4,0xFEB4,0xFEB4},
{0xFEB5,Lo,0xFEB5,0xFEB5,0xFEB5},
{0xFEB6,Lo,0xFEB6,0xFEB6,0xFEB6},
{0xFEB7,Lo,0xFEB7,0xFEB7,0xFEB7},
{0xFEB8,Lo,0xFEB8,0xFEB8,0xFEB8},
{0xFEB9,Lo,0xFEB9,0xFEB9,0xFEB9},
{0xFEBA,Lo,0xFEBA,0xFEBA,0xFEBA},
{0xFEBB,Lo,0xFEBB,0xFEBB,0xFEBB},
{0xFEBC,Lo,0xFEBC,0xFEBC,0xFEBC},
{0xFEBD,Lo,0xFEBD,0xFEBD,0xFEBD},
{0xFEBE,Lo,0xFEBE,0xFEBE,0xFEBE},
{0xFEBF,Lo,0xFEBF,0xFEBF,0xFEBF},
{0xFEC0,Lo,0xFEC0,0xFEC0,0xFEC0},
{0xFEC1,Lo,0xFEC1,0xFEC1,0xFEC1},
{0xFEC2,Lo,0xFEC2,0xFEC2,0xFEC2},
{0xFEC3,Lo,0xFEC3,0xFEC3,0xFEC3},
{0xFEC4,Lo,0xFEC4,0xFEC4,0xFEC4},
{0xFEC5,Lo,0xFEC5,0xFEC5,0xFEC5},
{0xFEC6,Lo,0xFEC6,0xFEC6,0xFEC6},
{0xFEC7,Lo,0xFEC7,0xFEC7,0xFEC7},
{0xFEC8,Lo,0xFEC8,0xFEC8,0xFEC8},
{0xFEC9,Lo,0xFEC9,0xFEC9,0xFEC9},
{0xFECA,Lo,0xFECA,0xFECA,0xFECA},
{0xFECB,Lo,0xFECB,0xFECB,0xFECB},
{0xFECC,Lo,0xFECC,0xFECC,0xFECC},
{0xFECD,Lo,0xFECD,0xFECD,0xFECD},
{0xFECE,Lo,0xFECE,0xFECE,0xFECE},
{0xFECF,Lo,0xFECF,0xFECF,0xFECF},
{0xFED0,Lo,0xFED0,0xFED0,0xFED0},
{0xFED1,Lo,0xFED1,0xFED1,0xFED1},
{0xFED2,Lo,0xFED2,0xFED2,0xFED2},
{0xFED3,Lo,0xFED3,0xFED3,0xFED3},
{0xFED4,Lo,0xFED4,0xFED4,0xFED4},
{0xFED5,Lo,0xFED5,0xFED5,0xFED5},
{0xFED6,Lo,0xFED6,0xFED6,0xFED6},
{0xFED7,Lo,0xFED7,0xFED7,0xFED7},
{0xFED8,Lo,0xFED8,0xFED8,0xFED8},
{0xFED9,Lo,0xFED9,0xFED9,0xFED9},
{0xFEDA,Lo,0xFEDA,0xFEDA,0xFEDA},
{0xFEDB,Lo,0xFEDB,0xFEDB,0xFEDB},
{0xFEDC,Lo,0xFEDC,0xFEDC,0xFEDC},
{0xFEDD,Lo,0xFEDD,0xFEDD,0xFEDD},
{0xFEDE,Lo,0xFEDE,0xFEDE,0xFEDE},
{0xFEDF,Lo,0xFEDF,0xFEDF,0xFEDF},
{0xFEE0,Lo,0xFEE0,0xFEE0,0xFEE0},
{0xFEE1,Lo,0xFEE1,0xFEE1,0xFEE1},
{0xFEE2,Lo,0xFEE2,0xFEE2,0xFEE2},
{0xFEE3,Lo,0xFEE3,0xFEE3,0xFEE3},
{0xFEE4,Lo,0xFEE4,0xFEE4,0xFEE4},
{0xFEE5,Lo,0xFEE5,0xFEE5,0xFEE5},
{0xFEE6,Lo,0xFEE6,0xFEE6,0xFEE6},
{0xFEE7,Lo,0xFEE7,0xFEE7,0xFEE7},
{0xFEE8,Lo,0xFEE8,0xFEE8,0xFEE8},
{0xFEE9,Lo,0xFEE9,0xFEE9,0xFEE9},
{0xFEEA,Lo,0xFEEA,0xFEEA,0xFEEA},
{0xFEEB,Lo,0xFEEB,0xFEEB,0xFEEB},
{0xFEEC,Lo,0xFEEC,0xFEEC,0xFEEC},
{0xFEED,Lo,0xFEED,0xFEED,0xFEED},
{0xFEEE,Lo,0xFEEE,0xFEEE,0xFEEE},
{0xFEEF,Lo,0xFEEF,0xFEEF,0xFEEF},
{0xFEF0,Lo,0xFEF0,0xFEF0,0xFEF0},
{0xFEF1,Lo,0xFEF1,0xFEF1,0xFEF1},
{0xFEF2,Lo,0xFEF2,0xFEF2,0xFEF2},
{0xFEF3,Lo,0xFEF3,0xFEF3,0xFEF3},
{0xFEF4,Lo,0xFEF4,0xFEF4,0xFEF4},
{0xFEF5,Lo,0xFEF5,0xFEF5,0xFEF5},
{0xFEF6,Lo,0xFEF6,0xFEF6,0xFEF6},
{0xFEF7,Lo,0xFEF7,0xFEF7,0xFEF7},
{0xFEF8,Lo,0xFEF8,0xFEF8,0xFEF8},
{0xFEF9,Lo,0xFEF9,0xFEF9,0xFEF9},
{0xFEFA,Lo,0xFEFA,0xFEFA,0xFEFA},
{0xFEFB,Lo,0xFEFB,0xFEFB,0xFEFB},
{0xFEFC,Lo,0xFEFC,0xFEFC,0xFEFC},
{0xFEFF,Cf,0xFEFF,0xFEFF,0xFEFF},
{0xFF01,Po,0xFF01,0xFF01,0xFF01},
{0xFF02,Po,0xFF02,0xFF02,0xFF02},
{0xFF03,Po,0xFF03,0xFF03,0xFF03},
{0xFF04,Sc,0xFF04,0xFF04,0xFF04},
{0xFF05,Po,0xFF05,0xFF05,0xFF05},
{0xFF06,Po,0xFF06,0xFF06,0xFF06},
{0xFF07,Po,0xFF07,0xFF07,0xFF07},
{0xFF08,Ps,0xFF08,0xFF08,0xFF08},
{0xFF09,Pe,0xFF09,0xFF09,0xFF09},
{0xFF0A,Po,0xFF0A,0xFF0A,0xFF0A},
{0xFF0B,Sm,0xFF0B,0xFF0B,0xFF0B},
{0xFF0C,Po,0xFF0C,0xFF0C,0xFF0C},
{0xFF0D,Pd,0xFF0D,0xFF0D,0xFF0D},
{0xFF0E,Po,0xFF0E,0xFF0E,0xFF0E},
{0xFF0F,Po,0xFF0F,0xFF0F,0xFF0F},
{0xFF10,Nd,0xFF10,0xFF10,0xFF10},
{0xFF11,Nd,0xFF11,0xFF11,0xFF11},
{0xFF12,Nd,0xFF12,0xFF12,0xFF12},
{0xFF13,Nd,0xFF13,0xFF13,0xFF13},
{0xFF14,Nd,0xFF14,0xFF14,0xFF14},
{0xFF15,Nd,0xFF15,0xFF15,0xFF15},
{0xFF16,Nd,0xFF16,0xFF16,0xFF16},
{0xFF17,Nd,0xFF17,0xFF17,0xFF17},
{0xFF18,Nd,0xFF18,0xFF18,0xFF18},
{0xFF19,Nd,0xFF19,0xFF19,0xFF19},
{0xFF1A,Po,0xFF1A,0xFF1A,0xFF1A},
{0xFF1B,Po,0xFF1B,0xFF1B,0xFF1B},
{0xFF1C,Sm,0xFF1C,0xFF1C,0xFF1C},
{0xFF1D,Sm,0xFF1D,0xFF1D,0xFF1D},
{0xFF1E,Sm,0xFF1E,0xFF1E,0xFF1E},
{0xFF1F,Po,0xFF1F,0xFF1F,0xFF1F},
{0xFF20,Po,0xFF20,0xFF20,0xFF20},
{0xFF21,Lu,0xFF21,0xFF41,0xFF21},
{0xFF22,Lu,0xFF22,0xFF42,0xFF22},
{0xFF23,Lu,0xFF23,0xFF43,0xFF23},
{0xFF24,Lu,0xFF24,0xFF44,0xFF24},
{0xFF25,Lu,0xFF25,0xFF45,0xFF25},
{0xFF26,Lu,0xFF26,0xFF46,0xFF26},
{0xFF27,Lu,0xFF27,0xFF47,0xFF27},
{0xFF28,Lu,0xFF28,0xFF48,0xFF28},
{0xFF29,Lu,0xFF29,0xFF49,0xFF29},
{0xFF2A,Lu,0xFF2A,0xFF4A,0xFF2A},
{0xFF2B,Lu,0xFF2B,0xFF4B,0xFF2B},
{0xFF2C,Lu,0xFF2C,0xFF4C,0xFF2C},
{0xFF2D,Lu,0xFF2D,0xFF4D,0xFF2D},
{0xFF2E,Lu,0xFF2E,0xFF4E,0xFF2E},
{0xFF2F,Lu,0xFF2F,0xFF4F,0xFF2F},
{0xFF30,Lu,0xFF30,0xFF50,0xFF30},
{0xFF31,Lu,0xFF31,0xFF51,0xFF31},
{0xFF32,Lu,0xFF32,0xFF52,0xFF32},
{0xFF33,Lu,0xFF33,0xFF53,0xFF33},
{0xFF34,Lu,0xFF34,0xFF54,0xFF34},
{0xFF35,Lu,0xFF35,0xFF55,0xFF35},
{0xFF36,Lu,0xFF36,0xFF56,0xFF36},
{0xFF37,Lu,0xFF37,0xFF57,0xFF37},
{0xFF38,Lu,0xFF38,0xFF58,0xFF38},
{0xFF39,Lu,0xFF39,0xFF59,0xFF39},
{0xFF3A,Lu,0xFF3A,0xFF5A,0xFF3A},
{0xFF3B,Ps,0xFF3B,0xFF3B,0xFF3B},
{0xFF3C,Po,0xFF3C,0xFF3C,0xFF3C},
{0xFF3D,Pe,0xFF3D,0xFF3D,0xFF3D},
{0xFF3E,Sk,0xFF3E,0xFF3E,0xFF3E},
{0xFF3F,Pc,0xFF3F,0xFF3F,0xFF3F},
{0xFF40,Sk,0xFF40,0xFF40,0xFF40},
{0xFF41,Ll,0xFF21,0xFF41,0xFF21},
{0xFF42,Ll,0xFF22,0xFF42,0xFF22},
{0xFF43,Ll,0xFF23,0xFF43,0xFF23},
{0xFF44,Ll,0xFF24,0xFF44,0xFF24},
{0xFF45,Ll,0xFF25,0xFF45,0xFF25},
{0xFF46,Ll,0xFF26,0xFF46,0xFF26},
{0xFF47,Ll,0xFF27,0xFF47,0xFF27},
{0xFF48,Ll,0xFF28,0xFF48,0xFF28},
{0xFF49,Ll,0xFF29,0xFF49,0xFF29},
{0xFF4A,Ll,0xFF2A,0xFF4A,0xFF2A},
{0xFF4B,Ll,0xFF2B,0xFF4B,0xFF2B},
{0xFF4C,Ll,0xFF2C,0xFF4C,0xFF2C},
{0xFF4D,Ll,0xFF2D,0xFF4D,0xFF2D},
{0xFF4E,Ll,0xFF2E,0xFF4E,0xFF2E},
{0xFF4F,Ll,0xFF2F,0xFF4F,0xFF2F},
{0xFF50,Ll,0xFF30,0xFF50,0xFF30},
{0xFF51,Ll,0xFF31,0xFF51,0xFF31},
{0xFF52,Ll,0xFF32,0xFF52,0xFF32},
{0xFF53,Ll,0xFF33,0xFF53,0xFF33},
{0xFF54,Ll,0xFF34,0xFF54,0xFF34},
{0xFF55,Ll,0xFF35,0xFF55,0xFF35},
{0xFF56,Ll,0xFF36,0xFF56,0xFF36},
{0xFF57,Ll,0xFF37,0xFF57,0xFF37},
{0xFF58,Ll,0xFF38,0xFF58,0xFF38},
{0xFF59,Ll,0xFF39,0xFF59,0xFF39},
{0xFF5A,Ll,0xFF3A,0xFF5A,0xFF3A},
{0xFF5B,Ps,0xFF5B,0xFF5B,0xFF5B},
{0xFF5C,Sm,0xFF5C,0xFF5C,0xFF5C},
{0xFF5D,Pe,0xFF5D,0xFF5D,0xFF5D},
{0xFF5E,Sm,0xFF5E,0xFF5E,0xFF5E},
{0xFF5F,Ps,0xFF5F,0xFF5F,0xFF5F},
{0xFF60,Pe,0xFF60,0xFF60,0xFF60},
{0xFF61,Po,0xFF61,0xFF61,0xFF61},
{0xFF62,Ps,0xFF62,0xFF62,0xFF62},
{0xFF63,Pe,0xFF63,0xFF63,0xFF63},
{0xFF64,Po,0xFF64,0xFF64,0xFF64},
{0xFF65,Po,0xFF65,0xFF65,0xFF65},
{0xFF66,Lo,0xFF66,0xFF66,0xFF66},
{0xFF67,Lo,0xFF67,0xFF67,0xFF67},
{0xFF68,Lo,0xFF68,0xFF68,0xFF68},
{0xFF69,Lo,0xFF69,0xFF69,0xFF69},
{0xFF6A,Lo,0xFF6A,0xFF6A,0xFF6A},
{0xFF6B,Lo,0xFF6B,0xFF6B,0xFF6B},
{0xFF6C,Lo,0xFF6C,0xFF6C,0xFF6C},
{0xFF6D,Lo,0xFF6D,0xFF6D,0xFF6D},
{0xFF6E,Lo,0xFF6E,0xFF6E,0xFF6E},
{0xFF6F,Lo,0xFF6F,0xFF6F,0xFF6F},
{0xFF70,Lm,0xFF70,0xFF70,0xFF70},
{0xFF71,Lo,0xFF71,0xFF71,0xFF71},
{0xFF72,Lo,0xFF72,0xFF72,0xFF72},
{0xFF73,Lo,0xFF73,0xFF73,0xFF73},
{0xFF74,Lo,0xFF74,0xFF74,0xFF74},
{0xFF75,Lo,0xFF75,0xFF75,0xFF75},
{0xFF76,Lo,0xFF76,0xFF76,0xFF76},
{0xFF77,Lo,0xFF77,0xFF77,0xFF77},
{0xFF78,Lo,0xFF78,0xFF78,0xFF78},
{0xFF79,Lo,0xFF79,0xFF79,0xFF79},
{0xFF7A,Lo,0xFF7A,0xFF7A,0xFF7A},
{0xFF7B,Lo,0xFF7B,0xFF7B,0xFF7B},
{0xFF7C,Lo,0xFF7C,0xFF7C,0xFF7C},
{0xFF7D,Lo,0xFF7D,0xFF7D,0xFF7D},
{0xFF7E,Lo,0xFF7E,0xFF7E,0xFF7E},
{0xFF7F,Lo,0xFF7F,0xFF7F,0xFF7F},
{0xFF80,Lo,0xFF80,0xFF80,0xFF80},
{0xFF81,Lo,0xFF81,0xFF81,0xFF81},
{0xFF82,Lo,0xFF82,0xFF82,0xFF82},
{0xFF83,Lo,0xFF83,0xFF83,0xFF83},
{0xFF84,Lo,0xFF84,0xFF84,0xFF84},
{0xFF85,Lo,0xFF85,0xFF85,0xFF85},
{0xFF86,Lo,0xFF86,0xFF86,0xFF86},
{0xFF87,Lo,0xFF87,0xFF87,0xFF87},
{0xFF88,Lo,0xFF88,0xFF88,0xFF88},
{0xFF89,Lo,0xFF89,0xFF89,0xFF89},
{0xFF8A,Lo,0xFF8A,0xFF8A,0xFF8A},
{0xFF8B,Lo,0xFF8B,0xFF8B,0xFF8B},
{0xFF8C,Lo,0xFF8C,0xFF8C,0xFF8C},
{0xFF8D,Lo,0xFF8D,0xFF8D,0xFF8D},
{0xFF8E,Lo,0xFF8E,0xFF8E,0xFF8E},
{0xFF8F,Lo,0xFF8F,0xFF8F,0xFF8F},
{0xFF90,Lo,0xFF90,0xFF90,0xFF90},
{0xFF91,Lo,0xFF91,0xFF91,0xFF91},
{0xFF92,Lo,0xFF92,0xFF92,0xFF92},
{0xFF93,Lo,0xFF93,0xFF93,0xFF93},
{0xFF94,Lo,0xFF94,0xFF94,0xFF94},
{0xFF95,Lo,0xFF95,0xFF95,0xFF95},
{0xFF96,Lo,0xFF96,0xFF96,0xFF96},
{0xFF97,Lo,0xFF97,0xFF97,0xFF97},
{0xFF98,Lo,0xFF98,0xFF98,0xFF98},
{0xFF99,Lo,0xFF99,0xFF99,0xFF99},
{0xFF9A,Lo,0xFF9A,0xFF9A,0xFF9A},
{0xFF9B,Lo,0xFF9B,0xFF9B,0xFF9B},
{0xFF9C,Lo,0xFF9C,0xFF9C,0xFF9C},
{0xFF9D,Lo,0xFF9D,0xFF9D,0xFF9D},
{0xFF9E,Lm,0xFF9E,0xFF9E,0xFF9E},
{0xFF9F,Lm,0xFF9F,0xFF9F,0xFF9F},
{0xFFA0,Lo,0xFFA0,0xFFA0,0xFFA0},
{0xFFA1,Lo,0xFFA1,0xFFA1,0xFFA1},
{0xFFA2,Lo,0xFFA2,0xFFA2,0xFFA2},
{0xFFA3,Lo,0xFFA3,0xFFA3,0xFFA3},
{0xFFA4,Lo,0xFFA4,0xFFA4,0xFFA4},
{0xFFA5,Lo,0xFFA5,0xFFA5,0xFFA5},
{0xFFA6,Lo,0xFFA6,0xFFA6,0xFFA6},
{0xFFA7,Lo,0xFFA7,0xFFA7,0xFFA7},
{0xFFA8,Lo,0xFFA8,0xFFA8,0xFFA8},
{0xFFA9,Lo,0xFFA9,0xFFA9,0xFFA9},
{0xFFAA,Lo,0xFFAA,0xFFAA,0xFFAA},
{0xFFAB,Lo,0xFFAB,0xFFAB,0xFFAB},
{0xFFAC,Lo,0xFFAC,0xFFAC,0xFFAC},
{0xFFAD,Lo,0xFFAD,0xFFAD,0xFFAD},
{0xFFAE,Lo,0xFFAE,0xFFAE,0xFFAE},
{0xFFAF,Lo,0xFFAF,0xFFAF,0xFFAF},
{0xFFB0,Lo,0xFFB0,0xFFB0,0xFFB0},
{0xFFB1,Lo,0xFFB1,0xFFB1,0xFFB1},
{0xFFB2,Lo,0xFFB2,0xFFB2,0xFFB2},
{0xFFB3,Lo,0xFFB3,0xFFB3,0xFFB3},
{0xFFB4,Lo,0xFFB4,0xFFB4,0xFFB4},
{0xFFB5,Lo,0xFFB5,0xFFB5,0xFFB5},
{0xFFB6,Lo,0xFFB6,0xFFB6,0xFFB6},
{0xFFB7,Lo,0xFFB7,0xFFB7,0xFFB7},
{0xFFB8,Lo,0xFFB8,0xFFB8,0xFFB8},
{0xFFB9,Lo,0xFFB9,0xFFB9,0xFFB9},
{0xFFBA,Lo,0xFFBA,0xFFBA,0xFFBA},
{0xFFBB,Lo,0xFFBB,0xFFBB,0xFFBB},
{0xFFBC,Lo,0xFFBC,0xFFBC,0xFFBC},
{0xFFBD,Lo,0xFFBD,0xFFBD,0xFFBD},
{0xFFBE,Lo,0xFFBE,0xFFBE,0xFFBE},
{0xFFC2,Lo,0xFFC2,0xFFC2,0xFFC2},
{0xFFC3,Lo,0xFFC3,0xFFC3,0xFFC3},
{0xFFC4,Lo,0xFFC4,0xFFC4,0xFFC4},
{0xFFC5,Lo,0xFFC5,0xFFC5,0xFFC5},
{0xFFC6,Lo,0xFFC6,0xFFC6,0xFFC6},
{0xFFC7,Lo,0xFFC7,0xFFC7,0xFFC7},
{0xFFCA,Lo,0xFFCA,0xFFCA,0xFFCA},
{0xFFCB,Lo,0xFFCB,0xFFCB,0xFFCB},
{0xFFCC,Lo,0xFFCC,0xFFCC,0xFFCC},
{0xFFCD,Lo,0xFFCD,0xFFCD,0xFFCD},
{0xFFCE,Lo,0xFFCE,0xFFCE,0xFFCE},
{0xFFCF,Lo,0xFFCF,0xFFCF,0xFFCF},
{0xFFD2,Lo,0xFFD2,0xFFD2,0xFFD2},
{0xFFD3,Lo,0xFFD3,0xFFD3,0xFFD3},
{0xFFD4,Lo,0xFFD4,0xFFD4,0xFFD4},
{0xFFD5,Lo,0xFFD5,0xFFD5,0xFFD5},
{0xFFD6,Lo,0xFFD6,0xFFD6,0xFFD6},
{0xFFD7,Lo,0xFFD7,0xFFD7,0xFFD7},
{0xFFDA,Lo,0xFFDA,0xFFDA,0xFFDA},
{0xFFDB,Lo,0xFFDB,0xFFDB,0xFFDB},
{0xFFDC,Lo,0xFFDC,0xFFDC,0xFFDC},
{0xFFE0,Sc,0xFFE0,0xFFE0,0xFFE0},
{0xFFE1,Sc,0xFFE1,0xFFE1,0xFFE1},
{0xFFE2,Sm,0xFFE2,0xFFE2,0xFFE2},
{0xFFE3,Sk,0xFFE3,0xFFE3,0xFFE3},
{0xFFE4,So,0xFFE4,0xFFE4,0xFFE4},
{0xFFE5,Sc,0xFFE5,0xFFE5,0xFFE5},
{0xFFE6,Sc,0xFFE6,0xFFE6,0xFFE6},
{0xFFE8,So,0xFFE8,0xFFE8,0xFFE8},
{0xFFE9,Sm,0xFFE9,0xFFE9,0xFFE9},
{0xFFEA,Sm,0xFFEA,0xFFEA,0xFFEA},
{0xFFEB,Sm,0xFFEB,0xFFEB,0xFFEB},
{0xFFEC,Sm,0xFFEC,0xFFEC,0xFFEC},
{0xFFED,So,0xFFED,0xFFED,0xFFED},
{0xFFEE,So,0xFFEE,0xFFEE,0xFFEE},
{0xFFF9,Cf,0xFFF9,0xFFF9,0xFFF9},
{0xFFFA,Cf,0xFFFA,0xFFFA,0xFFFA},
{0xFFFB,Cf,0xFFFB,0xFFFB,0xFFFB},
{0xFFFC,So,0xFFFC,0xFFFC,0xFFFC},
{0xFFFD,So,0xFFFD,0xFFFD,0xFFFD},
{0x10000,Lo,0x10000,0x10000,0x10000},
{0x10001,Lo,0x10001,0x10001,0x10001},
{0x10002,Lo,0x10002,0x10002,0x10002},
{0x10003,Lo,0x10003,0x10003,0x10003},
{0x10004,Lo,0x10004,0x10004,0x10004},
{0x10005,Lo,0x10005,0x10005,0x10005},
{0x10006,Lo,0x10006,0x10006,0x10006},
{0x10007,Lo,0x10007,0x10007,0x10007},
{0x10008,Lo,0x10008,0x10008,0x10008},
{0x10009,Lo,0x10009,0x10009,0x10009},
{0x1000A,Lo,0x1000A,0x1000A,0x1000A},
{0x1000B,Lo,0x1000B,0x1000B,0x1000B},
{0x1000D,Lo,0x1000D,0x1000D,0x1000D},
{0x1000E,Lo,0x1000E,0x1000E,0x1000E},
{0x1000F,Lo,0x1000F,0x1000F,0x1000F},
{0x10010,Lo,0x10010,0x10010,0x10010},
{0x10011,Lo,0x10011,0x10011,0x10011},
{0x10012,Lo,0x10012,0x10012,0x10012},
{0x10013,Lo,0x10013,0x10013,0x10013},
{0x10014,Lo,0x10014,0x10014,0x10014},
{0x10015,Lo,0x10015,0x10015,0x10015},
{0x10016,Lo,0x10016,0x10016,0x10016},
{0x10017,Lo,0x10017,0x10017,0x10017},
{0x10018,Lo,0x10018,0x10018,0x10018},
{0x10019,Lo,0x10019,0x10019,0x10019},
{0x1001A,Lo,0x1001A,0x1001A,0x1001A},
{0x1001B,Lo,0x1001B,0x1001B,0x1001B},
{0x1001C,Lo,0x1001C,0x1001C,0x1001C},
{0x1001D,Lo,0x1001D,0x1001D,0x1001D},
{0x1001E,Lo,0x1001E,0x1001E,0x1001E},
{0x1001F,Lo,0x1001F,0x1001F,0x1001F},
{0x10020,Lo,0x10020,0x10020,0x10020},
{0x10021,Lo,0x10021,0x10021,0x10021},
{0x10022,Lo,0x10022,0x10022,0x10022},
{0x10023,Lo,0x10023,0x10023,0x10023},
{0x10024,Lo,0x10024,0x10024,0x10024},
{0x10025,Lo,0x10025,0x10025,0x10025},
{0x10026,Lo,0x10026,0x10026,0x10026},
{0x10028,Lo,0x10028,0x10028,0x10028},
{0x10029,Lo,0x10029,0x10029,0x10029},
{0x1002A,Lo,0x1002A,0x1002A,0x1002A},
{0x1002B,Lo,0x1002B,0x1002B,0x1002B},
{0x1002C,Lo,0x1002C,0x1002C,0x1002C},
{0x1002D,Lo,0x1002D,0x1002D,0x1002D},
{0x1002E,Lo,0x1002E,0x1002E,0x1002E},
{0x1002F,Lo,0x1002F,0x1002F,0x1002F},
{0x10030,Lo,0x10030,0x10030,0x10030},
{0x10031,Lo,0x10031,0x10031,0x10031},
{0x10032,Lo,0x10032,0x10032,0x10032},
{0x10033,Lo,0x10033,0x10033,0x10033},
{0x10034,Lo,0x10034,0x10034,0x10034},
{0x10035,Lo,0x10035,0x10035,0x10035},
{0x10036,Lo,0x10036,0x10036,0x10036},
{0x10037,Lo,0x10037,0x10037,0x10037},
{0x10038,Lo,0x10038,0x10038,0x10038},
{0x10039,Lo,0x10039,0x10039,0x10039},
{0x1003A,Lo,0x1003A,0x1003A,0x1003A},
{0x1003C,Lo,0x1003C,0x1003C,0x1003C},
{0x1003D,Lo,0x1003D,0x1003D,0x1003D},
{0x1003F,Lo,0x1003F,0x1003F,0x1003F},
{0x10040,Lo,0x10040,0x10040,0x10040},
{0x10041,Lo,0x10041,0x10041,0x10041},
{0x10042,Lo,0x10042,0x10042,0x10042},
{0x10043,Lo,0x10043,0x10043,0x10043},
{0x10044,Lo,0x10044,0x10044,0x10044},
{0x10045,Lo,0x10045,0x10045,0x10045},
{0x10046,Lo,0x10046,0x10046,0x10046},
{0x10047,Lo,0x10047,0x10047,0x10047},
{0x10048,Lo,0x10048,0x10048,0x10048},
{0x10049,Lo,0x10049,0x10049,0x10049},
{0x1004A,Lo,0x1004A,0x1004A,0x1004A},
{0x1004B,Lo,0x1004B,0x1004B,0x1004B},
{0x1004C,Lo,0x1004C,0x1004C,0x1004C},
{0x1004D,Lo,0x1004D,0x1004D,0x1004D},
{0x10050,Lo,0x10050,0x10050,0x10050},
{0x10051,Lo,0x10051,0x10051,0x10051},
{0x10052,Lo,0x10052,0x10052,0x10052},
{0x10053,Lo,0x10053,0x10053,0x10053},
{0x10054,Lo,0x10054,0x10054,0x10054},
{0x10055,Lo,0x10055,0x10055,0x10055},
{0x10056,Lo,0x10056,0x10056,0x10056},
{0x10057,Lo,0x10057,0x10057,0x10057},
{0x10058,Lo,0x10058,0x10058,0x10058},
{0x10059,Lo,0x10059,0x10059,0x10059},
{0x1005A,Lo,0x1005A,0x1005A,0x1005A},
{0x1005B,Lo,0x1005B,0x1005B,0x1005B},
{0x1005C,Lo,0x1005C,0x1005C,0x1005C},
{0x1005D,Lo,0x1005D,0x1005D,0x1005D},
{0x10080,Lo,0x10080,0x10080,0x10080},
{0x10081,Lo,0x10081,0x10081,0x10081},
{0x10082,Lo,0x10082,0x10082,0x10082},
{0x10083,Lo,0x10083,0x10083,0x10083},
{0x10084,Lo,0x10084,0x10084,0x10084},
{0x10085,Lo,0x10085,0x10085,0x10085},
{0x10086,Lo,0x10086,0x10086,0x10086},
{0x10087,Lo,0x10087,0x10087,0x10087},
{0x10088,Lo,0x10088,0x10088,0x10088},
{0x10089,Lo,0x10089,0x10089,0x10089},
{0x1008A,Lo,0x1008A,0x1008A,0x1008A},
{0x1008B,Lo,0x1008B,0x1008B,0x1008B},
{0x1008C,Lo,0x1008C,0x1008C,0x1008C},
{0x1008D,Lo,0x1008D,0x1008D,0x1008D},
{0x1008E,Lo,0x1008E,0x1008E,0x1008E},
{0x1008F,Lo,0x1008F,0x1008F,0x1008F},
{0x10090,Lo,0x10090,0x10090,0x10090},
{0x10091,Lo,0x10091,0x10091,0x10091},
{0x10092,Lo,0x10092,0x10092,0x10092},
{0x10093,Lo,0x10093,0x10093,0x10093},
{0x10094,Lo,0x10094,0x10094,0x10094},
{0x10095,Lo,0x10095,0x10095,0x10095},
{0x10096,Lo,0x10096,0x10096,0x10096},
{0x10097,Lo,0x10097,0x10097,0x10097},
{0x10098,Lo,0x10098,0x10098,0x10098},
{0x10099,Lo,0x10099,0x10099,0x10099},
{0x1009A,Lo,0x1009A,0x1009A,0x1009A},
{0x1009B,Lo,0x1009B,0x1009B,0x1009B},
{0x1009C,Lo,0x1009C,0x1009C,0x1009C},
{0x1009D,Lo,0x1009D,0x1009D,0x1009D},
{0x1009E,Lo,0x1009E,0x1009E,0x1009E},
{0x1009F,Lo,0x1009F,0x1009F,0x1009F},
{0x100A0,Lo,0x100A0,0x100A0,0x100A0},
{0x100A1,Lo,0x100A1,0x100A1,0x100A1},
{0x100A2,Lo,0x100A2,0x100A2,0x100A2},
{0x100A3,Lo,0x100A3,0x100A3,0x100A3},
{0x100A4,Lo,0x100A4,0x100A4,0x100A4},
{0x100A5,Lo,0x100A5,0x100A5,0x100A5},
{0x100A6,Lo,0x100A6,0x100A6,0x100A6},
{0x100A7,Lo,0x100A7,0x100A7,0x100A7},
{0x100A8,Lo,0x100A8,0x100A8,0x100A8},
{0x100A9,Lo,0x100A9,0x100A9,0x100A9},
{0x100AA,Lo,0x100AA,0x100AA,0x100AA},
{0x100AB,Lo,0x100AB,0x100AB,0x100AB},
{0x100AC,Lo,0x100AC,0x100AC,0x100AC},
{0x100AD,Lo,0x100AD,0x100AD,0x100AD},
{0x100AE,Lo,0x100AE,0x100AE,0x100AE},
{0x100AF,Lo,0x100AF,0x100AF,0x100AF},
{0x100B0,Lo,0x100B0,0x100B0,0x100B0},
{0x100B1,Lo,0x100B1,0x100B1,0x100B1},
{0x100B2,Lo,0x100B2,0x100B2,0x100B2},
{0x100B3,Lo,0x100B3,0x100B3,0x100B3},
{0x100B4,Lo,0x100B4,0x100B4,0x100B4},
{0x100B5,Lo,0x100B5,0x100B5,0x100B5},
{0x100B6,Lo,0x100B6,0x100B6,0x100B6},
{0x100B7,Lo,0x100B7,0x100B7,0x100B7},
{0x100B8,Lo,0x100B8,0x100B8,0x100B8},
{0x100B9,Lo,0x100B9,0x100B9,0x100B9},
{0x100BA,Lo,0x100BA,0x100BA,0x100BA},
{0x100BB,Lo,0x100BB,0x100BB,0x100BB},
{0x100BC,Lo,0x100BC,0x100BC,0x100BC},
{0x100BD,Lo,0x100BD,0x100BD,0x100BD},
{0x100BE,Lo,0x100BE,0x100BE,0x100BE},
{0x100BF,Lo,0x100BF,0x100BF,0x100BF},
{0x100C0,Lo,0x100C0,0x100C0,0x100C0},
{0x100C1,Lo,0x100C1,0x100C1,0x100C1},
{0x100C2,Lo,0x100C2,0x100C2,0x100C2},
{0x100C3,Lo,0x100C3,0x100C3,0x100C3},
{0x100C4,Lo,0x100C4,0x100C4,0x100C4},
{0x100C5,Lo,0x100C5,0x100C5,0x100C5},
{0x100C6,Lo,0x100C6,0x100C6,0x100C6},
{0x100C7,Lo,0x100C7,0x100C7,0x100C7},
{0x100C8,Lo,0x100C8,0x100C8,0x100C8},
{0x100C9,Lo,0x100C9,0x100C9,0x100C9},
{0x100CA,Lo,0x100CA,0x100CA,0x100CA},
{0x100CB,Lo,0x100CB,0x100CB,0x100CB},
{0x100CC,Lo,0x100CC,0x100CC,0x100CC},
{0x100CD,Lo,0x100CD,0x100CD,0x100CD},
{0x100CE,Lo,0x100CE,0x100CE,0x100CE},
{0x100CF,Lo,0x100CF,0x100CF,0x100CF},
{0x100D0,Lo,0x100D0,0x100D0,0x100D0},
{0x100D1,Lo,0x100D1,0x100D1,0x100D1},
{0x100D2,Lo,0x100D2,0x100D2,0x100D2},
{0x100D3,Lo,0x100D3,0x100D3,0x100D3},
{0x100D4,Lo,0x100D4,0x100D4,0x100D4},
{0x100D5,Lo,0x100D5,0x100D5,0x100D5},
{0x100D6,Lo,0x100D6,0x100D6,0x100D6},
{0x100D7,Lo,0x100D7,0x100D7,0x100D7},
{0x100D8,Lo,0x100D8,0x100D8,0x100D8},
{0x100D9,Lo,0x100D9,0x100D9,0x100D9},
{0x100DA,Lo,0x100DA,0x100DA,0x100DA},
{0x100DB,Lo,0x100DB,0x100DB,0x100DB},
{0x100DC,Lo,0x100DC,0x100DC,0x100DC},
{0x100DD,Lo,0x100DD,0x100DD,0x100DD},
{0x100DE,Lo,0x100DE,0x100DE,0x100DE},
{0x100DF,Lo,0x100DF,0x100DF,0x100DF},
{0x100E0,Lo,0x100E0,0x100E0,0x100E0},
{0x100E1,Lo,0x100E1,0x100E1,0x100E1},
{0x100E2,Lo,0x100E2,0x100E2,0x100E2},
{0x100E3,Lo,0x100E3,0x100E3,0x100E3},
{0x100E4,Lo,0x100E4,0x100E4,0x100E4},
{0x100E5,Lo,0x100E5,0x100E5,0x100E5},
{0x100E6,Lo,0x100E6,0x100E6,0x100E6},
{0x100E7,Lo,0x100E7,0x100E7,0x100E7},
{0x100E8,Lo,0x100E8,0x100E8,0x100E8},
{0x100E9,Lo,0x100E9,0x100E9,0x100E9},
{0x100EA,Lo,0x100EA,0x100EA,0x100EA},
{0x100EB,Lo,0x100EB,0x100EB,0x100EB},
{0x100EC,Lo,0x100EC,0x100EC,0x100EC},
{0x100ED,Lo,0x100ED,0x100ED,0x100ED},
{0x100EE,Lo,0x100EE,0x100EE,0x100EE},
{0x100EF,Lo,0x100EF,0x100EF,0x100EF},
{0x100F0,Lo,0x100F0,0x100F0,0x100F0},
{0x100F1,Lo,0x100F1,0x100F1,0x100F1},
{0x100F2,Lo,0x100F2,0x100F2,0x100F2},
{0x100F3,Lo,0x100F3,0x100F3,0x100F3},
{0x100F4,Lo,0x100F4,0x100F4,0x100F4},
{0x100F5,Lo,0x100F5,0x100F5,0x100F5},
{0x100F6,Lo,0x100F6,0x100F6,0x100F6},
{0x100F7,Lo,0x100F7,0x100F7,0x100F7},
{0x100F8,Lo,0x100F8,0x100F8,0x100F8},
{0x100F9,Lo,0x100F9,0x100F9,0x100F9},
{0x100FA,Lo,0x100FA,0x100FA,0x100FA},
{0x10100,Po,0x10100,0x10100,0x10100},
{0x10101,Po,0x10101,0x10101,0x10101},
{0x10102,So,0x10102,0x10102,0x10102},
{0x10107,No,0x10107,0x10107,0x10107},
{0x10108,No,0x10108,0x10108,0x10108},
{0x10109,No,0x10109,0x10109,0x10109},
{0x1010A,No,0x1010A,0x1010A,0x1010A},
{0x1010B,No,0x1010B,0x1010B,0x1010B},
{0x1010C,No,0x1010C,0x1010C,0x1010C},
{0x1010D,No,0x1010D,0x1010D,0x1010D},
{0x1010E,No,0x1010E,0x1010E,0x1010E},
{0x1010F,No,0x1010F,0x1010F,0x1010F},
{0x10110,No,0x10110,0x10110,0x10110},
{0x10111,No,0x10111,0x10111,0x10111},
{0x10112,No,0x10112,0x10112,0x10112},
{0x10113,No,0x10113,0x10113,0x10113},
{0x10114,No,0x10114,0x10114,0x10114},
{0x10115,No,0x10115,0x10115,0x10115},
{0x10116,No,0x10116,0x10116,0x10116},
{0x10117,No,0x10117,0x10117,0x10117},
{0x10118,No,0x10118,0x10118,0x10118},
{0x10119,No,0x10119,0x10119,0x10119},
{0x1011A,No,0x1011A,0x1011A,0x1011A},
{0x1011B,No,0x1011B,0x1011B,0x1011B},
{0x1011C,No,0x1011C,0x1011C,0x1011C},
{0x1011D,No,0x1011D,0x1011D,0x1011D},
{0x1011E,No,0x1011E,0x1011E,0x1011E},
{0x1011F,No,0x1011F,0x1011F,0x1011F},
{0x10120,No,0x10120,0x10120,0x10120},
{0x10121,No,0x10121,0x10121,0x10121},
{0x10122,No,0x10122,0x10122,0x10122},
{0x10123,No,0x10123,0x10123,0x10123},
{0x10124,No,0x10124,0x10124,0x10124},
{0x10125,No,0x10125,0x10125,0x10125},
{0x10126,No,0x10126,0x10126,0x10126},
{0x10127,No,0x10127,0x10127,0x10127},
{0x10128,No,0x10128,0x10128,0x10128},
{0x10129,No,0x10129,0x10129,0x10129},
{0x1012A,No,0x1012A,0x1012A,0x1012A},
{0x1012B,No,0x1012B,0x1012B,0x1012B},
{0x1012C,No,0x1012C,0x1012C,0x1012C},
{0x1012D,No,0x1012D,0x1012D,0x1012D},
{0x1012E,No,0x1012E,0x1012E,0x1012E},
{0x1012F,No,0x1012F,0x1012F,0x1012F},
{0x10130,No,0x10130,0x10130,0x10130},
{0x10131,No,0x10131,0x10131,0x10131},
{0x10132,No,0x10132,0x10132,0x10132},
{0x10133,No,0x10133,0x10133,0x10133},
{0x10137,So,0x10137,0x10137,0x10137},
{0x10138,So,0x10138,0x10138,0x10138},
{0x10139,So,0x10139,0x10139,0x10139},
{0x1013A,So,0x1013A,0x1013A,0x1013A},
{0x1013B,So,0x1013B,0x1013B,0x1013B},
{0x1013C,So,0x1013C,0x1013C,0x1013C},
{0x1013D,So,0x1013D,0x1013D,0x1013D},
{0x1013E,So,0x1013E,0x1013E,0x1013E},
{0x1013F,So,0x1013F,0x1013F,0x1013F},
{0x10140,Nl,0x10140,0x10140,0x10140},
{0x10141,Nl,0x10141,0x10141,0x10141},
{0x10142,Nl,0x10142,0x10142,0x10142},
{0x10143,Nl,0x10143,0x10143,0x10143},
{0x10144,Nl,0x10144,0x10144,0x10144},
{0x10145,Nl,0x10145,0x10145,0x10145},
{0x10146,Nl,0x10146,0x10146,0x10146},
{0x10147,Nl,0x10147,0x10147,0x10147},
{0x10148,Nl,0x10148,0x10148,0x10148},
{0x10149,Nl,0x10149,0x10149,0x10149},
{0x1014A,Nl,0x1014A,0x1014A,0x1014A},
{0x1014B,Nl,0x1014B,0x1014B,0x1014B},
{0x1014C,Nl,0x1014C,0x1014C,0x1014C},
{0x1014D,Nl,0x1014D,0x1014D,0x1014D},
{0x1014E,Nl,0x1014E,0x1014E,0x1014E},
{0x1014F,Nl,0x1014F,0x1014F,0x1014F},
{0x10150,Nl,0x10150,0x10150,0x10150},
{0x10151,Nl,0x10151,0x10151,0x10151},
{0x10152,Nl,0x10152,0x10152,0x10152},
{0x10153,Nl,0x10153,0x10153,0x10153},
{0x10154,Nl,0x10154,0x10154,0x10154},
{0x10155,Nl,0x10155,0x10155,0x10155},
{0x10156,Nl,0x10156,0x10156,0x10156},
{0x10157,Nl,0x10157,0x10157,0x10157},
{0x10158,Nl,0x10158,0x10158,0x10158},
{0x10159,Nl,0x10159,0x10159,0x10159},
{0x1015A,Nl,0x1015A,0x1015A,0x1015A},
{0x1015B,Nl,0x1015B,0x1015B,0x1015B},
{0x1015C,Nl,0x1015C,0x1015C,0x1015C},
{0x1015D,Nl,0x1015D,0x1015D,0x1015D},
{0x1015E,Nl,0x1015E,0x1015E,0x1015E},
{0x1015F,Nl,0x1015F,0x1015F,0x1015F},
{0x10160,Nl,0x10160,0x10160,0x10160},
{0x10161,Nl,0x10161,0x10161,0x10161},
{0x10162,Nl,0x10162,0x10162,0x10162},
{0x10163,Nl,0x10163,0x10163,0x10163},
{0x10164,Nl,0x10164,0x10164,0x10164},
{0x10165,Nl,0x10165,0x10165,0x10165},
{0x10166,Nl,0x10166,0x10166,0x10166},
{0x10167,Nl,0x10167,0x10167,0x10167},
{0x10168,Nl,0x10168,0x10168,0x10168},
{0x10169,Nl,0x10169,0x10169,0x10169},
{0x1016A,Nl,0x1016A,0x1016A,0x1016A},
{0x1016B,Nl,0x1016B,0x1016B,0x1016B},
{0x1016C,Nl,0x1016C,0x1016C,0x1016C},
{0x1016D,Nl,0x1016D,0x1016D,0x1016D},
{0x1016E,Nl,0x1016E,0x1016E,0x1016E},
{0x1016F,Nl,0x1016F,0x1016F,0x1016F},
{0x10170,Nl,0x10170,0x10170,0x10170},
{0x10171,Nl,0x10171,0x10171,0x10171},
{0x10172,Nl,0x10172,0x10172,0x10172},
{0x10173,Nl,0x10173,0x10173,0x10173},
{0x10174,Nl,0x10174,0x10174,0x10174},
{0x10175,No,0x10175,0x10175,0x10175},
{0x10176,No,0x10176,0x10176,0x10176},
{0x10177,No,0x10177,0x10177,0x10177},
{0x10178,No,0x10178,0x10178,0x10178},
{0x10179,So,0x10179,0x10179,0x10179},
{0x1017A,So,0x1017A,0x1017A,0x1017A},
{0x1017B,So,0x1017B,0x1017B,0x1017B},
{0x1017C,So,0x1017C,0x1017C,0x1017C},
{0x1017D,So,0x1017D,0x1017D,0x1017D},
{0x1017E,So,0x1017E,0x1017E,0x1017E},
{0x1017F,So,0x1017F,0x1017F,0x1017F},
{0x10180,So,0x10180,0x10180,0x10180},
{0x10181,So,0x10181,0x10181,0x10181},
{0x10182,So,0x10182,0x10182,0x10182},
{0x10183,So,0x10183,0x10183,0x10183},
{0x10184,So,0x10184,0x10184,0x10184},
{0x10185,So,0x10185,0x10185,0x10185},
{0x10186,So,0x10186,0x10186,0x10186},
{0x10187,So,0x10187,0x10187,0x10187},
{0x10188,So,0x10188,0x10188,0x10188},
{0x10189,So,0x10189,0x10189,0x10189},
{0x1018A,No,0x1018A,0x1018A,0x1018A},
{0x10300,Lo,0x10300,0x10300,0x10300},
{0x10301,Lo,0x10301,0x10301,0x10301},
{0x10302,Lo,0x10302,0x10302,0x10302},
{0x10303,Lo,0x10303,0x10303,0x10303},
{0x10304,Lo,0x10304,0x10304,0x10304},
{0x10305,Lo,0x10305,0x10305,0x10305},
{0x10306,Lo,0x10306,0x10306,0x10306},
{0x10307,Lo,0x10307,0x10307,0x10307},
{0x10308,Lo,0x10308,0x10308,0x10308},
{0x10309,Lo,0x10309,0x10309,0x10309},
{0x1030A,Lo,0x1030A,0x1030A,0x1030A},
{0x1030B,Lo,0x1030B,0x1030B,0x1030B},
{0x1030C,Lo,0x1030C,0x1030C,0x1030C},
{0x1030D,Lo,0x1030D,0x1030D,0x1030D},
{0x1030E,Lo,0x1030E,0x1030E,0x1030E},
{0x1030F,Lo,0x1030F,0x1030F,0x1030F},
{0x10310,Lo,0x10310,0x10310,0x10310},
{0x10311,Lo,0x10311,0x10311,0x10311},
{0x10312,Lo,0x10312,0x10312,0x10312},
{0x10313,Lo,0x10313,0x10313,0x10313},
{0x10314,Lo,0x10314,0x10314,0x10314},
{0x10315,Lo,0x10315,0x10315,0x10315},
{0x10316,Lo,0x10316,0x10316,0x10316},
{0x10317,Lo,0x10317,0x10317,0x10317},
{0x10318,Lo,0x10318,0x10318,0x10318},
{0x10319,Lo,0x10319,0x10319,0x10319},
{0x1031A,Lo,0x1031A,0x1031A,0x1031A},
{0x1031B,Lo,0x1031B,0x1031B,0x1031B},
{0x1031C,Lo,0x1031C,0x1031C,0x1031C},
{0x1031D,Lo,0x1031D,0x1031D,0x1031D},
{0x1031E,Lo,0x1031E,0x1031E,0x1031E},
{0x10320,No,0x10320,0x10320,0x10320},
{0x10321,No,0x10321,0x10321,0x10321},
{0x10322,No,0x10322,0x10322,0x10322},
{0x10323,No,0x10323,0x10323,0x10323},
{0x10330,Lo,0x10330,0x10330,0x10330},
{0x10331,Lo,0x10331,0x10331,0x10331},
{0x10332,Lo,0x10332,0x10332,0x10332},
{0x10333,Lo,0x10333,0x10333,0x10333},
{0x10334,Lo,0x10334,0x10334,0x10334},
{0x10335,Lo,0x10335,0x10335,0x10335},
{0x10336,Lo,0x10336,0x10336,0x10336},
{0x10337,Lo,0x10337,0x10337,0x10337},
{0x10338,Lo,0x10338,0x10338,0x10338},
{0x10339,Lo,0x10339,0x10339,0x10339},
{0x1033A,Lo,0x1033A,0x1033A,0x1033A},
{0x1033B,Lo,0x1033B,0x1033B,0x1033B},
{0x1033C,Lo,0x1033C,0x1033C,0x1033C},
{0x1033D,Lo,0x1033D,0x1033D,0x1033D},
{0x1033E,Lo,0x1033E,0x1033E,0x1033E},
{0x1033F,Lo,0x1033F,0x1033F,0x1033F},
{0x10340,Lo,0x10340,0x10340,0x10340},
{0x10341,Nl,0x10341,0x10341,0x10341},
{0x10342,Lo,0x10342,0x10342,0x10342},
{0x10343,Lo,0x10343,0x10343,0x10343},
{0x10344,Lo,0x10344,0x10344,0x10344},
{0x10345,Lo,0x10345,0x10345,0x10345},
{0x10346,Lo,0x10346,0x10346,0x10346},
{0x10347,Lo,0x10347,0x10347,0x10347},
{0x10348,Lo,0x10348,0x10348,0x10348},
{0x10349,Lo,0x10349,0x10349,0x10349},
{0x1034A,Nl,0x1034A,0x1034A,0x1034A},
{0x10380,Lo,0x10380,0x10380,0x10380},
{0x10381,Lo,0x10381,0x10381,0x10381},
{0x10382,Lo,0x10382,0x10382,0x10382},
{0x10383,Lo,0x10383,0x10383,0x10383},
{0x10384,Lo,0x10384,0x10384,0x10384},
{0x10385,Lo,0x10385,0x10385,0x10385},
{0x10386,Lo,0x10386,0x10386,0x10386},
{0x10387,Lo,0x10387,0x10387,0x10387},
{0x10388,Lo,0x10388,0x10388,0x10388},
{0x10389,Lo,0x10389,0x10389,0x10389},
{0x1038A,Lo,0x1038A,0x1038A,0x1038A},
{0x1038B,Lo,0x1038B,0x1038B,0x1038B},
{0x1038C,Lo,0x1038C,0x1038C,0x1038C},
{0x1038D,Lo,0x1038D,0x1038D,0x1038D},
{0x1038E,Lo,0x1038E,0x1038E,0x1038E},
{0x1038F,Lo,0x1038F,0x1038F,0x1038F},
{0x10390,Lo,0x10390,0x10390,0x10390},
{0x10391,Lo,0x10391,0x10391,0x10391},
{0x10392,Lo,0x10392,0x10392,0x10392},
{0x10393,Lo,0x10393,0x10393,0x10393},
{0x10394,Lo,0x10394,0x10394,0x10394},
{0x10395,Lo,0x10395,0x10395,0x10395},
{0x10396,Lo,0x10396,0x10396,0x10396},
{0x10397,Lo,0x10397,0x10397,0x10397},
{0x10398,Lo,0x10398,0x10398,0x10398},
{0x10399,Lo,0x10399,0x10399,0x10399},
{0x1039A,Lo,0x1039A,0x1039A,0x1039A},
{0x1039B,Lo,0x1039B,0x1039B,0x1039B},
{0x1039C,Lo,0x1039C,0x1039C,0x1039C},
{0x1039D,Lo,0x1039D,0x1039D,0x1039D},
{0x1039F,Po,0x1039F,0x1039F,0x1039F},
{0x103A0,Lo,0x103A0,0x103A0,0x103A0},
{0x103A1,Lo,0x103A1,0x103A1,0x103A1},
{0x103A2,Lo,0x103A2,0x103A2,0x103A2},
{0x103A3,Lo,0x103A3,0x103A3,0x103A3},
{0x103A4,Lo,0x103A4,0x103A4,0x103A4},
{0x103A5,Lo,0x103A5,0x103A5,0x103A5},
{0x103A6,Lo,0x103A6,0x103A6,0x103A6},
{0x103A7,Lo,0x103A7,0x103A7,0x103A7},
{0x103A8,Lo,0x103A8,0x103A8,0x103A8},
{0x103A9,Lo,0x103A9,0x103A9,0x103A9},
{0x103AA,Lo,0x103AA,0x103AA,0x103AA},
{0x103AB,Lo,0x103AB,0x103AB,0x103AB},
{0x103AC,Lo,0x103AC,0x103AC,0x103AC},
{0x103AD,Lo,0x103AD,0x103AD,0x103AD},
{0x103AE,Lo,0x103AE,0x103AE,0x103AE},
{0x103AF,Lo,0x103AF,0x103AF,0x103AF},
{0x103B0,Lo,0x103B0,0x103B0,0x103B0},
{0x103B1,Lo,0x103B1,0x103B1,0x103B1},
{0x103B2,Lo,0x103B2,0x103B2,0x103B2},
{0x103B3,Lo,0x103B3,0x103B3,0x103B3},
{0x103B4,Lo,0x103B4,0x103B4,0x103B4},
{0x103B5,Lo,0x103B5,0x103B5,0x103B5},
{0x103B6,Lo,0x103B6,0x103B6,0x103B6},
{0x103B7,Lo,0x103B7,0x103B7,0x103B7},
{0x103B8,Lo,0x103B8,0x103B8,0x103B8},
{0x103B9,Lo,0x103B9,0x103B9,0x103B9},
{0x103BA,Lo,0x103BA,0x103BA,0x103BA},
{0x103BB,Lo,0x103BB,0x103BB,0x103BB},
{0x103BC,Lo,0x103BC,0x103BC,0x103BC},
{0x103BD,Lo,0x103BD,0x103BD,0x103BD},
{0x103BE,Lo,0x103BE,0x103BE,0x103BE},
{0x103BF,Lo,0x103BF,0x103BF,0x103BF},
{0x103C0,Lo,0x103C0,0x103C0,0x103C0},
{0x103C1,Lo,0x103C1,0x103C1,0x103C1},
{0x103C2,Lo,0x103C2,0x103C2,0x103C2},
{0x103C3,Lo,0x103C3,0x103C3,0x103C3},
{0x103C8,Lo,0x103C8,0x103C8,0x103C8},
{0x103C9,Lo,0x103C9,0x103C9,0x103C9},
{0x103CA,Lo,0x103CA,0x103CA,0x103CA},
{0x103CB,Lo,0x103CB,0x103CB,0x103CB},
{0x103CC,Lo,0x103CC,0x103CC,0x103CC},
{0x103CD,Lo,0x103CD,0x103CD,0x103CD},
{0x103CE,Lo,0x103CE,0x103CE,0x103CE},
{0x103CF,Lo,0x103CF,0x103CF,0x103CF},
{0x103D0,Po,0x103D0,0x103D0,0x103D0},
{0x103D1,Nl,0x103D1,0x103D1,0x103D1},
{0x103D2,Nl,0x103D2,0x103D2,0x103D2},
{0x103D3,Nl,0x103D3,0x103D3,0x103D3},
{0x103D4,Nl,0x103D4,0x103D4,0x103D4},
{0x103D5,Nl,0x103D5,0x103D5,0x103D5},
{0x10400,Lu,0x10400,0x10428,0x10400},
{0x10401,Lu,0x10401,0x10429,0x10401},
{0x10402,Lu,0x10402,0x1042A,0x10402},
{0x10403,Lu,0x10403,0x1042B,0x10403},
{0x10404,Lu,0x10404,0x1042C,0x10404},
{0x10405,Lu,0x10405,0x1042D,0x10405},
{0x10406,Lu,0x10406,0x1042E,0x10406},
{0x10407,Lu,0x10407,0x1042F,0x10407},
{0x10408,Lu,0x10408,0x10430,0x10408},
{0x10409,Lu,0x10409,0x10431,0x10409},
{0x1040A,Lu,0x1040A,0x10432,0x1040A},
{0x1040B,Lu,0x1040B,0x10433,0x1040B},
{0x1040C,Lu,0x1040C,0x10434,0x1040C},
{0x1040D,Lu,0x1040D,0x10435,0x1040D},
{0x1040E,Lu,0x1040E,0x10436,0x1040E},
{0x1040F,Lu,0x1040F,0x10437,0x1040F},
{0x10410,Lu,0x10410,0x10438,0x10410},
{0x10411,Lu,0x10411,0x10439,0x10411},
{0x10412,Lu,0x10412,0x1043A,0x10412},
{0x10413,Lu,0x10413,0x1043B,0x10413},
{0x10414,Lu,0x10414,0x1043C,0x10414},
{0x10415,Lu,0x10415,0x1043D,0x10415},
{0x10416,Lu,0x10416,0x1043E,0x10416},
{0x10417,Lu,0x10417,0x1043F,0x10417},
{0x10418,Lu,0x10418,0x10440,0x10418},
{0x10419,Lu,0x10419,0x10441,0x10419},
{0x1041A,Lu,0x1041A,0x10442,0x1041A},
{0x1041B,Lu,0x1041B,0x10443,0x1041B},
{0x1041C,Lu,0x1041C,0x10444,0x1041C},
{0x1041D,Lu,0x1041D,0x10445,0x1041D},
{0x1041E,Lu,0x1041E,0x10446,0x1041E},
{0x1041F,Lu,0x1041F,0x10447,0x1041F},
{0x10420,Lu,0x10420,0x10448,0x10420},
{0x10421,Lu,0x10421,0x10449,0x10421},
{0x10422,Lu,0x10422,0x1044A,0x10422},
{0x10423,Lu,0x10423,0x1044B,0x10423},
{0x10424,Lu,0x10424,0x1044C,0x10424},
{0x10425,Lu,0x10425,0x1044D,0x10425},
{0x10426,Lu,0x10426,0x1044E,0x10426},
{0x10427,Lu,0x10427,0x1044F,0x10427},
{0x10428,Ll,0x10400,0x10428,0x10400},
{0x10429,Ll,0x10401,0x10429,0x10401},
{0x1042A,Ll,0x10402,0x1042A,0x10402},
{0x1042B,Ll,0x10403,0x1042B,0x10403},
{0x1042C,Ll,0x10404,0x1042C,0x10404},
{0x1042D,Ll,0x10405,0x1042D,0x10405},
{0x1042E,Ll,0x10406,0x1042E,0x10406},
{0x1042F,Ll,0x10407,0x1042F,0x10407},
{0x10430,Ll,0x10408,0x10430,0x10408},
{0x10431,Ll,0x10409,0x10431,0x10409},
{0x10432,Ll,0x1040A,0x10432,0x1040A},
{0x10433,Ll,0x1040B,0x10433,0x1040B},
{0x10434,Ll,0x1040C,0x10434,0x1040C},
{0x10435,Ll,0x1040D,0x10435,0x1040D},
{0x10436,Ll,0x1040E,0x10436,0x1040E},
{0x10437,Ll,0x1040F,0x10437,0x1040F},
{0x10438,Ll,0x10410,0x10438,0x10410},
{0x10439,Ll,0x10411,0x10439,0x10411},
{0x1043A,Ll,0x10412,0x1043A,0x10412},
{0x1043B,Ll,0x10413,0x1043B,0x10413},
{0x1043C,Ll,0x10414,0x1043C,0x10414},
{0x1043D,Ll,0x10415,0x1043D,0x10415},
{0x1043E,Ll,0x10416,0x1043E,0x10416},
{0x1043F,Ll,0x10417,0x1043F,0x10417},
{0x10440,Ll,0x10418,0x10440,0x10418},
{0x10441,Ll,0x10419,0x10441,0x10419},
{0x10442,Ll,0x1041A,0x10442,0x1041A},
{0x10443,Ll,0x1041B,0x10443,0x1041B},
{0x10444,Ll,0x1041C,0x10444,0x1041C},
{0x10445,Ll,0x1041D,0x10445,0x1041D},
{0x10446,Ll,0x1041E,0x10446,0x1041E},
{0x10447,Ll,0x1041F,0x10447,0x1041F},
{0x10448,Ll,0x10420,0x10448,0x10420},
{0x10449,Ll,0x10421,0x10449,0x10421},
{0x1044A,Ll,0x10422,0x1044A,0x10422},
{0x1044B,Ll,0x10423,0x1044B,0x10423},
{0x1044C,Ll,0x10424,0x1044C,0x10424},
{0x1044D,Ll,0x10425,0x1044D,0x10425},
{0x1044E,Ll,0x10426,0x1044E,0x10426},
{0x1044F,Ll,0x10427,0x1044F,0x10427},
{0x10450,Lo,0x10450,0x10450,0x10450},
{0x10451,Lo,0x10451,0x10451,0x10451},
{0x10452,Lo,0x10452,0x10452,0x10452},
{0x10453,Lo,0x10453,0x10453,0x10453},
{0x10454,Lo,0x10454,0x10454,0x10454},
{0x10455,Lo,0x10455,0x10455,0x10455},
{0x10456,Lo,0x10456,0x10456,0x10456},
{0x10457,Lo,0x10457,0x10457,0x10457},
{0x10458,Lo,0x10458,0x10458,0x10458},
{0x10459,Lo,0x10459,0x10459,0x10459},
{0x1045A,Lo,0x1045A,0x1045A,0x1045A},
{0x1045B,Lo,0x1045B,0x1045B,0x1045B},
{0x1045C,Lo,0x1045C,0x1045C,0x1045C},
{0x1045D,Lo,0x1045D,0x1045D,0x1045D},
{0x1045E,Lo,0x1045E,0x1045E,0x1045E},
{0x1045F,Lo,0x1045F,0x1045F,0x1045F},
{0x10460,Lo,0x10460,0x10460,0x10460},
{0x10461,Lo,0x10461,0x10461,0x10461},
{0x10462,Lo,0x10462,0x10462,0x10462},
{0x10463,Lo,0x10463,0x10463,0x10463},
{0x10464,Lo,0x10464,0x10464,0x10464},
{0x10465,Lo,0x10465,0x10465,0x10465},
{0x10466,Lo,0x10466,0x10466,0x10466},
{0x10467,Lo,0x10467,0x10467,0x10467},
{0x10468,Lo,0x10468,0x10468,0x10468},
{0x10469,Lo,0x10469,0x10469,0x10469},
{0x1046A,Lo,0x1046A,0x1046A,0x1046A},
{0x1046B,Lo,0x1046B,0x1046B,0x1046B},
{0x1046C,Lo,0x1046C,0x1046C,0x1046C},
{0x1046D,Lo,0x1046D,0x1046D,0x1046D},
{0x1046E,Lo,0x1046E,0x1046E,0x1046E},
{0x1046F,Lo,0x1046F,0x1046F,0x1046F},
{0x10470,Lo,0x10470,0x10470,0x10470},
{0x10471,Lo,0x10471,0x10471,0x10471},
{0x10472,Lo,0x10472,0x10472,0x10472},
{0x10473,Lo,0x10473,0x10473,0x10473},
{0x10474,Lo,0x10474,0x10474,0x10474},
{0x10475,Lo,0x10475,0x10475,0x10475},
{0x10476,Lo,0x10476,0x10476,0x10476},
{0x10477,Lo,0x10477,0x10477,0x10477},
{0x10478,Lo,0x10478,0x10478,0x10478},
{0x10479,Lo,0x10479,0x10479,0x10479},
{0x1047A,Lo,0x1047A,0x1047A,0x1047A},
{0x1047B,Lo,0x1047B,0x1047B,0x1047B},
{0x1047C,Lo,0x1047C,0x1047C,0x1047C},
{0x1047D,Lo,0x1047D,0x1047D,0x1047D},
{0x1047E,Lo,0x1047E,0x1047E,0x1047E},
{0x1047F,Lo,0x1047F,0x1047F,0x1047F},
{0x10480,Lo,0x10480,0x10480,0x10480},
{0x10481,Lo,0x10481,0x10481,0x10481},
{0x10482,Lo,0x10482,0x10482,0x10482},
{0x10483,Lo,0x10483,0x10483,0x10483},
{0x10484,Lo,0x10484,0x10484,0x10484},
{0x10485,Lo,0x10485,0x10485,0x10485},
{0x10486,Lo,0x10486,0x10486,0x10486},
{0x10487,Lo,0x10487,0x10487,0x10487},
{0x10488,Lo,0x10488,0x10488,0x10488},
{0x10489,Lo,0x10489,0x10489,0x10489},
{0x1048A,Lo,0x1048A,0x1048A,0x1048A},
{0x1048B,Lo,0x1048B,0x1048B,0x1048B},
{0x1048C,Lo,0x1048C,0x1048C,0x1048C},
{0x1048D,Lo,0x1048D,0x1048D,0x1048D},
{0x1048E,Lo,0x1048E,0x1048E,0x1048E},
{0x1048F,Lo,0x1048F,0x1048F,0x1048F},
{0x10490,Lo,0x10490,0x10490,0x10490},
{0x10491,Lo,0x10491,0x10491,0x10491},
{0x10492,Lo,0x10492,0x10492,0x10492},
{0x10493,Lo,0x10493,0x10493,0x10493},
{0x10494,Lo,0x10494,0x10494,0x10494},
{0x10495,Lo,0x10495,0x10495,0x10495},
{0x10496,Lo,0x10496,0x10496,0x10496},
{0x10497,Lo,0x10497,0x10497,0x10497},
{0x10498,Lo,0x10498,0x10498,0x10498},
{0x10499,Lo,0x10499,0x10499,0x10499},
{0x1049A,Lo,0x1049A,0x1049A,0x1049A},
{0x1049B,Lo,0x1049B,0x1049B,0x1049B},
{0x1049C,Lo,0x1049C,0x1049C,0x1049C},
{0x1049D,Lo,0x1049D,0x1049D,0x1049D},
{0x104A0,Nd,0x104A0,0x104A0,0x104A0},
{0x104A1,Nd,0x104A1,0x104A1,0x104A1},
{0x104A2,Nd,0x104A2,0x104A2,0x104A2},
{0x104A3,Nd,0x104A3,0x104A3,0x104A3},
{0x104A4,Nd,0x104A4,0x104A4,0x104A4},
{0x104A5,Nd,0x104A5,0x104A5,0x104A5},
{0x104A6,Nd,0x104A6,0x104A6,0x104A6},
{0x104A7,Nd,0x104A7,0x104A7,0x104A7},
{0x104A8,Nd,0x104A8,0x104A8,0x104A8},
{0x104A9,Nd,0x104A9,0x104A9,0x104A9},
{0x10800,Lo,0x10800,0x10800,0x10800},
{0x10801,Lo,0x10801,0x10801,0x10801},
{0x10802,Lo,0x10802,0x10802,0x10802},
{0x10803,Lo,0x10803,0x10803,0x10803},
{0x10804,Lo,0x10804,0x10804,0x10804},
{0x10805,Lo,0x10805,0x10805,0x10805},
{0x10808,Lo,0x10808,0x10808,0x10808},
{0x1080A,Lo,0x1080A,0x1080A,0x1080A},
{0x1080B,Lo,0x1080B,0x1080B,0x1080B},
{0x1080C,Lo,0x1080C,0x1080C,0x1080C},
{0x1080D,Lo,0x1080D,0x1080D,0x1080D},
{0x1080E,Lo,0x1080E,0x1080E,0x1080E},
{0x1080F,Lo,0x1080F,0x1080F,0x1080F},
{0x10810,Lo,0x10810,0x10810,0x10810},
{0x10811,Lo,0x10811,0x10811,0x10811},
{0x10812,Lo,0x10812,0x10812,0x10812},
{0x10813,Lo,0x10813,0x10813,0x10813},
{0x10814,Lo,0x10814,0x10814,0x10814},
{0x10815,Lo,0x10815,0x10815,0x10815},
{0x10816,Lo,0x10816,0x10816,0x10816},
{0x10817,Lo,0x10817,0x10817,0x10817},
{0x10818,Lo,0x10818,0x10818,0x10818},
{0x10819,Lo,0x10819,0x10819,0x10819},
{0x1081A,Lo,0x1081A,0x1081A,0x1081A},
{0x1081B,Lo,0x1081B,0x1081B,0x1081B},
{0x1081C,Lo,0x1081C,0x1081C,0x1081C},
{0x1081D,Lo,0x1081D,0x1081D,0x1081D},
{0x1081E,Lo,0x1081E,0x1081E,0x1081E},
{0x1081F,Lo,0x1081F,0x1081F,0x1081F},
{0x10820,Lo,0x10820,0x10820,0x10820},
{0x10821,Lo,0x10821,0x10821,0x10821},
{0x10822,Lo,0x10822,0x10822,0x10822},
{0x10823,Lo,0x10823,0x10823,0x10823},
{0x10824,Lo,0x10824,0x10824,0x10824},
{0x10825,Lo,0x10825,0x10825,0x10825},
{0x10826,Lo,0x10826,0x10826,0x10826},
{0x10827,Lo,0x10827,0x10827,0x10827},
{0x10828,Lo,0x10828,0x10828,0x10828},
{0x10829,Lo,0x10829,0x10829,0x10829},
{0x1082A,Lo,0x1082A,0x1082A,0x1082A},
{0x1082B,Lo,0x1082B,0x1082B,0x1082B},
{0x1082C,Lo,0x1082C,0x1082C,0x1082C},
{0x1082D,Lo,0x1082D,0x1082D,0x1082D},
{0x1082E,Lo,0x1082E,0x1082E,0x1082E},
{0x1082F,Lo,0x1082F,0x1082F,0x1082F},
{0x10830,Lo,0x10830,0x10830,0x10830},
{0x10831,Lo,0x10831,0x10831,0x10831},
{0x10832,Lo,0x10832,0x10832,0x10832},
{0x10833,Lo,0x10833,0x10833,0x10833},
{0x10834,Lo,0x10834,0x10834,0x10834},
{0x10835,Lo,0x10835,0x10835,0x10835},
{0x10837,Lo,0x10837,0x10837,0x10837},
{0x10838,Lo,0x10838,0x10838,0x10838},
{0x1083C,Lo,0x1083C,0x1083C,0x1083C},
{0x1083F,Lo,0x1083F,0x1083F,0x1083F},
{0x10900,Lo,0x10900,0x10900,0x10900},
{0x10901,Lo,0x10901,0x10901,0x10901},
{0x10902,Lo,0x10902,0x10902,0x10902},
{0x10903,Lo,0x10903,0x10903,0x10903},
{0x10904,Lo,0x10904,0x10904,0x10904},
{0x10905,Lo,0x10905,0x10905,0x10905},
{0x10906,Lo,0x10906,0x10906,0x10906},
{0x10907,Lo,0x10907,0x10907,0x10907},
{0x10908,Lo,0x10908,0x10908,0x10908},
{0x10909,Lo,0x10909,0x10909,0x10909},
{0x1090A,Lo,0x1090A,0x1090A,0x1090A},
{0x1090B,Lo,0x1090B,0x1090B,0x1090B},
{0x1090C,Lo,0x1090C,0x1090C,0x1090C},
{0x1090D,Lo,0x1090D,0x1090D,0x1090D},
{0x1090E,Lo,0x1090E,0x1090E,0x1090E},
{0x1090F,Lo,0x1090F,0x1090F,0x1090F},
{0x10910,Lo,0x10910,0x10910,0x10910},
{0x10911,Lo,0x10911,0x10911,0x10911},
{0x10912,Lo,0x10912,0x10912,0x10912},
{0x10913,Lo,0x10913,0x10913,0x10913},
{0x10914,Lo,0x10914,0x10914,0x10914},
{0x10915,Lo,0x10915,0x10915,0x10915},
{0x10916,No,0x10916,0x10916,0x10916},
{0x10917,No,0x10917,0x10917,0x10917},
{0x10918,No,0x10918,0x10918,0x10918},
{0x10919,No,0x10919,0x10919,0x10919},
{0x1091F,Po,0x1091F,0x1091F,0x1091F},
{0x10A00,Lo,0x10A00,0x10A00,0x10A00},
{0x10A01,Mn,0x10A01,0x10A01,0x10A01},
{0x10A02,Mn,0x10A02,0x10A02,0x10A02},
{0x10A03,Mn,0x10A03,0x10A03,0x10A03},
{0x10A05,Mn,0x10A05,0x10A05,0x10A05},
{0x10A06,Mn,0x10A06,0x10A06,0x10A06},
{0x10A0C,Mn,0x10A0C,0x10A0C,0x10A0C},
{0x10A0D,Mn,0x10A0D,0x10A0D,0x10A0D},
{0x10A0E,Mn,0x10A0E,0x10A0E,0x10A0E},
{0x10A0F,Mn,0x10A0F,0x10A0F,0x10A0F},
{0x10A10,Lo,0x10A10,0x10A10,0x10A10},
{0x10A11,Lo,0x10A11,0x10A11,0x10A11},
{0x10A12,Lo,0x10A12,0x10A12,0x10A12},
{0x10A13,Lo,0x10A13,0x10A13,0x10A13},
{0x10A15,Lo,0x10A15,0x10A15,0x10A15},
{0x10A16,Lo,0x10A16,0x10A16,0x10A16},
{0x10A17,Lo,0x10A17,0x10A17,0x10A17},
{0x10A19,Lo,0x10A19,0x10A19,0x10A19},
{0x10A1A,Lo,0x10A1A,0x10A1A,0x10A1A},
{0x10A1B,Lo,0x10A1B,0x10A1B,0x10A1B},
{0x10A1C,Lo,0x10A1C,0x10A1C,0x10A1C},
{0x10A1D,Lo,0x10A1D,0x10A1D,0x10A1D},
{0x10A1E,Lo,0x10A1E,0x10A1E,0x10A1E},
{0x10A1F,Lo,0x10A1F,0x10A1F,0x10A1F},
{0x10A20,Lo,0x10A20,0x10A20,0x10A20},
{0x10A21,Lo,0x10A21,0x10A21,0x10A21},
{0x10A22,Lo,0x10A22,0x10A22,0x10A22},
{0x10A23,Lo,0x10A23,0x10A23,0x10A23},
{0x10A24,Lo,0x10A24,0x10A24,0x10A24},
{0x10A25,Lo,0x10A25,0x10A25,0x10A25},
{0x10A26,Lo,0x10A26,0x10A26,0x10A26},
{0x10A27,Lo,0x10A27,0x10A27,0x10A27},
{0x10A28,Lo,0x10A28,0x10A28,0x10A28},
{0x10A29,Lo,0x10A29,0x10A29,0x10A29},
{0x10A2A,Lo,0x10A2A,0x10A2A,0x10A2A},
{0x10A2B,Lo,0x10A2B,0x10A2B,0x10A2B},
{0x10A2C,Lo,0x10A2C,0x10A2C,0x10A2C},
{0x10A2D,Lo,0x10A2D,0x10A2D,0x10A2D},
{0x10A2E,Lo,0x10A2E,0x10A2E,0x10A2E},
{0x10A2F,Lo,0x10A2F,0x10A2F,0x10A2F},
{0x10A30,Lo,0x10A30,0x10A30,0x10A30},
{0x10A31,Lo,0x10A31,0x10A31,0x10A31},
{0x10A32,Lo,0x10A32,0x10A32,0x10A32},
{0x10A33,Lo,0x10A33,0x10A33,0x10A33},
{0x10A38,Mn,0x10A38,0x10A38,0x10A38},
{0x10A39,Mn,0x10A39,0x10A39,0x10A39},
{0x10A3A,Mn,0x10A3A,0x10A3A,0x10A3A},
{0x10A3F,Mn,0x10A3F,0x10A3F,0x10A3F},
{0x10A40,No,0x10A40,0x10A40,0x10A40},
{0x10A41,No,0x10A41,0x10A41,0x10A41},
{0x10A42,No,0x10A42,0x10A42,0x10A42},
{0x10A43,No,0x10A43,0x10A43,0x10A43},
{0x10A44,No,0x10A44,0x10A44,0x10A44},
{0x10A45,No,0x10A45,0x10A45,0x10A45},
{0x10A46,No,0x10A46,0x10A46,0x10A46},
{0x10A47,No,0x10A47,0x10A47,0x10A47},
{0x10A50,Po,0x10A50,0x10A50,0x10A50},
{0x10A51,Po,0x10A51,0x10A51,0x10A51},
{0x10A52,Po,0x10A52,0x10A52,0x10A52},
{0x10A53,Po,0x10A53,0x10A53,0x10A53},
{0x10A54,Po,0x10A54,0x10A54,0x10A54},
{0x10A55,Po,0x10A55,0x10A55,0x10A55},
{0x10A56,Po,0x10A56,0x10A56,0x10A56},
{0x10A57,Po,0x10A57,0x10A57,0x10A57},
{0x10A58,Po,0x10A58,0x10A58,0x10A58},
{0x12000,Lo,0x12000,0x12000,0x12000},
{0x12001,Lo,0x12001,0x12001,0x12001},
{0x12002,Lo,0x12002,0x12002,0x12002},
{0x12003,Lo,0x12003,0x12003,0x12003},
{0x12004,Lo,0x12004,0x12004,0x12004},
{0x12005,Lo,0x12005,0x12005,0x12005},
{0x12006,Lo,0x12006,0x12006,0x12006},
{0x12007,Lo,0x12007,0x12007,0x12007},
{0x12008,Lo,0x12008,0x12008,0x12008},
{0x12009,Lo,0x12009,0x12009,0x12009},
{0x1200A,Lo,0x1200A,0x1200A,0x1200A},
{0x1200B,Lo,0x1200B,0x1200B,0x1200B},
{0x1200C,Lo,0x1200C,0x1200C,0x1200C},
{0x1200D,Lo,0x1200D,0x1200D,0x1200D},
{0x1200E,Lo,0x1200E,0x1200E,0x1200E},
{0x1200F,Lo,0x1200F,0x1200F,0x1200F},
{0x12010,Lo,0x12010,0x12010,0x12010},
{0x12011,Lo,0x12011,0x12011,0x12011},
{0x12012,Lo,0x12012,0x12012,0x12012},
{0x12013,Lo,0x12013,0x12013,0x12013},
{0x12014,Lo,0x12014,0x12014,0x12014},
{0x12015,Lo,0x12015,0x12015,0x12015},
{0x12016,Lo,0x12016,0x12016,0x12016},
{0x12017,Lo,0x12017,0x12017,0x12017},
{0x12018,Lo,0x12018,0x12018,0x12018},
{0x12019,Lo,0x12019,0x12019,0x12019},
{0x1201A,Lo,0x1201A,0x1201A,0x1201A},
{0x1201B,Lo,0x1201B,0x1201B,0x1201B},
{0x1201C,Lo,0x1201C,0x1201C,0x1201C},
{0x1201D,Lo,0x1201D,0x1201D,0x1201D},
{0x1201E,Lo,0x1201E,0x1201E,0x1201E},
{0x1201F,Lo,0x1201F,0x1201F,0x1201F},
{0x12020,Lo,0x12020,0x12020,0x12020},
{0x12021,Lo,0x12021,0x12021,0x12021},
{0x12022,Lo,0x12022,0x12022,0x12022},
{0x12023,Lo,0x12023,0x12023,0x12023},
{0x12024,Lo,0x12024,0x12024,0x12024},
{0x12025,Lo,0x12025,0x12025,0x12025},
{0x12026,Lo,0x12026,0x12026,0x12026},
{0x12027,Lo,0x12027,0x12027,0x12027},
{0x12028,Lo,0x12028,0x12028,0x12028},
{0x12029,Lo,0x12029,0x12029,0x12029},
{0x1202A,Lo,0x1202A,0x1202A,0x1202A},
{0x1202B,Lo,0x1202B,0x1202B,0x1202B},
{0x1202C,Lo,0x1202C,0x1202C,0x1202C},
{0x1202D,Lo,0x1202D,0x1202D,0x1202D},
{0x1202E,Lo,0x1202E,0x1202E,0x1202E},
{0x1202F,Lo,0x1202F,0x1202F,0x1202F},
{0x12030,Lo,0x12030,0x12030,0x12030},
{0x12031,Lo,0x12031,0x12031,0x12031},
{0x12032,Lo,0x12032,0x12032,0x12032},
{0x12033,Lo,0x12033,0x12033,0x12033},
{0x12034,Lo,0x12034,0x12034,0x12034},
{0x12035,Lo,0x12035,0x12035,0x12035},
{0x12036,Lo,0x12036,0x12036,0x12036},
{0x12037,Lo,0x12037,0x12037,0x12037},
{0x12038,Lo,0x12038,0x12038,0x12038},
{0x12039,Lo,0x12039,0x12039,0x12039},
{0x1203A,Lo,0x1203A,0x1203A,0x1203A},
{0x1203B,Lo,0x1203B,0x1203B,0x1203B},
{0x1203C,Lo,0x1203C,0x1203C,0x1203C},
{0x1203D,Lo,0x1203D,0x1203D,0x1203D},
{0x1203E,Lo,0x1203E,0x1203E,0x1203E},
{0x1203F,Lo,0x1203F,0x1203F,0x1203F},
{0x12040,Lo,0x12040,0x12040,0x12040},
{0x12041,Lo,0x12041,0x12041,0x12041},
{0x12042,Lo,0x12042,0x12042,0x12042},
{0x12043,Lo,0x12043,0x12043,0x12043},
{0x12044,Lo,0x12044,0x12044,0x12044},
{0x12045,Lo,0x12045,0x12045,0x12045},
{0x12046,Lo,0x12046,0x12046,0x12046},
{0x12047,Lo,0x12047,0x12047,0x12047},
{0x12048,Lo,0x12048,0x12048,0x12048},
{0x12049,Lo,0x12049,0x12049,0x12049},
{0x1204A,Lo,0x1204A,0x1204A,0x1204A},
{0x1204B,Lo,0x1204B,0x1204B,0x1204B},
{0x1204C,Lo,0x1204C,0x1204C,0x1204C},
{0x1204D,Lo,0x1204D,0x1204D,0x1204D},
{0x1204E,Lo,0x1204E,0x1204E,0x1204E},
{0x1204F,Lo,0x1204F,0x1204F,0x1204F},
{0x12050,Lo,0x12050,0x12050,0x12050},
{0x12051,Lo,0x12051,0x12051,0x12051},
{0x12052,Lo,0x12052,0x12052,0x12052},
{0x12053,Lo,0x12053,0x12053,0x12053},
{0x12054,Lo,0x12054,0x12054,0x12054},
{0x12055,Lo,0x12055,0x12055,0x12055},
{0x12056,Lo,0x12056,0x12056,0x12056},
{0x12057,Lo,0x12057,0x12057,0x12057},
{0x12058,Lo,0x12058,0x12058,0x12058},
{0x12059,Lo,0x12059,0x12059,0x12059},
{0x1205A,Lo,0x1205A,0x1205A,0x1205A},
{0x1205B,Lo,0x1205B,0x1205B,0x1205B},
{0x1205C,Lo,0x1205C,0x1205C,0x1205C},
{0x1205D,Lo,0x1205D,0x1205D,0x1205D},
{0x1205E,Lo,0x1205E,0x1205E,0x1205E},
{0x1205F,Lo,0x1205F,0x1205F,0x1205F},
{0x12060,Lo,0x12060,0x12060,0x12060},
{0x12061,Lo,0x12061,0x12061,0x12061},
{0x12062,Lo,0x12062,0x12062,0x12062},
{0x12063,Lo,0x12063,0x12063,0x12063},
{0x12064,Lo,0x12064,0x12064,0x12064},
{0x12065,Lo,0x12065,0x12065,0x12065},
{0x12066,Lo,0x12066,0x12066,0x12066},
{0x12067,Lo,0x12067,0x12067,0x12067},
{0x12068,Lo,0x12068,0x12068,0x12068},
{0x12069,Lo,0x12069,0x12069,0x12069},
{0x1206A,Lo,0x1206A,0x1206A,0x1206A},
{0x1206B,Lo,0x1206B,0x1206B,0x1206B},
{0x1206C,Lo,0x1206C,0x1206C,0x1206C},
{0x1206D,Lo,0x1206D,0x1206D,0x1206D},
{0x1206E,Lo,0x1206E,0x1206E,0x1206E},
{0x1206F,Lo,0x1206F,0x1206F,0x1206F},
{0x12070,Lo,0x12070,0x12070,0x12070},
{0x12071,Lo,0x12071,0x12071,0x12071},
{0x12072,Lo,0x12072,0x12072,0x12072},
{0x12073,Lo,0x12073,0x12073,0x12073},
{0x12074,Lo,0x12074,0x12074,0x12074},
{0x12075,Lo,0x12075,0x12075,0x12075},
{0x12076,Lo,0x12076,0x12076,0x12076},
{0x12077,Lo,0x12077,0x12077,0x12077},
{0x12078,Lo,0x12078,0x12078,0x12078},
{0x12079,Lo,0x12079,0x12079,0x12079},
{0x1207A,Lo,0x1207A,0x1207A,0x1207A},
{0x1207B,Lo,0x1207B,0x1207B,0x1207B},
{0x1207C,Lo,0x1207C,0x1207C,0x1207C},
{0x1207D,Lo,0x1207D,0x1207D,0x1207D},
{0x1207E,Lo,0x1207E,0x1207E,0x1207E},
{0x1207F,Lo,0x1207F,0x1207F,0x1207F},
{0x12080,Lo,0x12080,0x12080,0x12080},
{0x12081,Lo,0x12081,0x12081,0x12081},
{0x12082,Lo,0x12082,0x12082,0x12082},
{0x12083,Lo,0x12083,0x12083,0x12083},
{0x12084,Lo,0x12084,0x12084,0x12084},
{0x12085,Lo,0x12085,0x12085,0x12085},
{0x12086,Lo,0x12086,0x12086,0x12086},
{0x12087,Lo,0x12087,0x12087,0x12087},
{0x12088,Lo,0x12088,0x12088,0x12088},
{0x12089,Lo,0x12089,0x12089,0x12089},
{0x1208A,Lo,0x1208A,0x1208A,0x1208A},
{0x1208B,Lo,0x1208B,0x1208B,0x1208B},
{0x1208C,Lo,0x1208C,0x1208C,0x1208C},
{0x1208D,Lo,0x1208D,0x1208D,0x1208D},
{0x1208E,Lo,0x1208E,0x1208E,0x1208E},
{0x1208F,Lo,0x1208F,0x1208F,0x1208F},
{0x12090,Lo,0x12090,0x12090,0x12090},
{0x12091,Lo,0x12091,0x12091,0x12091},
{0x12092,Lo,0x12092,0x12092,0x12092},
{0x12093,Lo,0x12093,0x12093,0x12093},
{0x12094,Lo,0x12094,0x12094,0x12094},
{0x12095,Lo,0x12095,0x12095,0x12095},
{0x12096,Lo,0x12096,0x12096,0x12096},
{0x12097,Lo,0x12097,0x12097,0x12097},
{0x12098,Lo,0x12098,0x12098,0x12098},
{0x12099,Lo,0x12099,0x12099,0x12099},
{0x1209A,Lo,0x1209A,0x1209A,0x1209A},
{0x1209B,Lo,0x1209B,0x1209B,0x1209B},
{0x1209C,Lo,0x1209C,0x1209C,0x1209C},
{0x1209D,Lo,0x1209D,0x1209D,0x1209D},
{0x1209E,Lo,0x1209E,0x1209E,0x1209E},
{0x1209F,Lo,0x1209F,0x1209F,0x1209F},
{0x120A0,Lo,0x120A0,0x120A0,0x120A0},
{0x120A1,Lo,0x120A1,0x120A1,0x120A1},
{0x120A2,Lo,0x120A2,0x120A2,0x120A2},
{0x120A3,Lo,0x120A3,0x120A3,0x120A3},
{0x120A4,Lo,0x120A4,0x120A4,0x120A4},
{0x120A5,Lo,0x120A5,0x120A5,0x120A5},
{0x120A6,Lo,0x120A6,0x120A6,0x120A6},
{0x120A7,Lo,0x120A7,0x120A7,0x120A7},
{0x120A8,Lo,0x120A8,0x120A8,0x120A8},
{0x120A9,Lo,0x120A9,0x120A9,0x120A9},
{0x120AA,Lo,0x120AA,0x120AA,0x120AA},
{0x120AB,Lo,0x120AB,0x120AB,0x120AB},
{0x120AC,Lo,0x120AC,0x120AC,0x120AC},
{0x120AD,Lo,0x120AD,0x120AD,0x120AD},
{0x120AE,Lo,0x120AE,0x120AE,0x120AE},
{0x120AF,Lo,0x120AF,0x120AF,0x120AF},
{0x120B0,Lo,0x120B0,0x120B0,0x120B0},
{0x120B1,Lo,0x120B1,0x120B1,0x120B1},
{0x120B2,Lo,0x120B2,0x120B2,0x120B2},
{0x120B3,Lo,0x120B3,0x120B3,0x120B3},
{0x120B4,Lo,0x120B4,0x120B4,0x120B4},
{0x120B5,Lo,0x120B5,0x120B5,0x120B5},
{0x120B6,Lo,0x120B6,0x120B6,0x120B6},
{0x120B7,Lo,0x120B7,0x120B7,0x120B7},
{0x120B8,Lo,0x120B8,0x120B8,0x120B8},
{0x120B9,Lo,0x120B9,0x120B9,0x120B9},
{0x120BA,Lo,0x120BA,0x120BA,0x120BA},
{0x120BB,Lo,0x120BB,0x120BB,0x120BB},
{0x120BC,Lo,0x120BC,0x120BC,0x120BC},
{0x120BD,Lo,0x120BD,0x120BD,0x120BD},
{0x120BE,Lo,0x120BE,0x120BE,0x120BE},
{0x120BF,Lo,0x120BF,0x120BF,0x120BF},
{0x120C0,Lo,0x120C0,0x120C0,0x120C0},
{0x120C1,Lo,0x120C1,0x120C1,0x120C1},
{0x120C2,Lo,0x120C2,0x120C2,0x120C2},
{0x120C3,Lo,0x120C3,0x120C3,0x120C3},
{0x120C4,Lo,0x120C4,0x120C4,0x120C4},
{0x120C5,Lo,0x120C5,0x120C5,0x120C5},
{0x120C6,Lo,0x120C6,0x120C6,0x120C6},
{0x120C7,Lo,0x120C7,0x120C7,0x120C7},
{0x120C8,Lo,0x120C8,0x120C8,0x120C8},
{0x120C9,Lo,0x120C9,0x120C9,0x120C9},
{0x120CA,Lo,0x120CA,0x120CA,0x120CA},
{0x120CB,Lo,0x120CB,0x120CB,0x120CB},
{0x120CC,Lo,0x120CC,0x120CC,0x120CC},
{0x120CD,Lo,0x120CD,0x120CD,0x120CD},
{0x120CE,Lo,0x120CE,0x120CE,0x120CE},
{0x120CF,Lo,0x120CF,0x120CF,0x120CF},
{0x120D0,Lo,0x120D0,0x120D0,0x120D0},
{0x120D1,Lo,0x120D1,0x120D1,0x120D1},
{0x120D2,Lo,0x120D2,0x120D2,0x120D2},
{0x120D3,Lo,0x120D3,0x120D3,0x120D3},
{0x120D4,Lo,0x120D4,0x120D4,0x120D4},
{0x120D5,Lo,0x120D5,0x120D5,0x120D5},
{0x120D6,Lo,0x120D6,0x120D6,0x120D6},
{0x120D7,Lo,0x120D7,0x120D7,0x120D7},
{0x120D8,Lo,0x120D8,0x120D8,0x120D8},
{0x120D9,Lo,0x120D9,0x120D9,0x120D9},
{0x120DA,Lo,0x120DA,0x120DA,0x120DA},
{0x120DB,Lo,0x120DB,0x120DB,0x120DB},
{0x120DC,Lo,0x120DC,0x120DC,0x120DC},
{0x120DD,Lo,0x120DD,0x120DD,0x120DD},
{0x120DE,Lo,0x120DE,0x120DE,0x120DE},
{0x120DF,Lo,0x120DF,0x120DF,0x120DF},
{0x120E0,Lo,0x120E0,0x120E0,0x120E0},
{0x120E1,Lo,0x120E1,0x120E1,0x120E1},
{0x120E2,Lo,0x120E2,0x120E2,0x120E2},
{0x120E3,Lo,0x120E3,0x120E3,0x120E3},
{0x120E4,Lo,0x120E4,0x120E4,0x120E4},
{0x120E5,Lo,0x120E5,0x120E5,0x120E5},
{0x120E6,Lo,0x120E6,0x120E6,0x120E6},
{0x120E7,Lo,0x120E7,0x120E7,0x120E7},
{0x120E8,Lo,0x120E8,0x120E8,0x120E8},
{0x120E9,Lo,0x120E9,0x120E9,0x120E9},
{0x120EA,Lo,0x120EA,0x120EA,0x120EA},
{0x120EB,Lo,0x120EB,0x120EB,0x120EB},
{0x120EC,Lo,0x120EC,0x120EC,0x120EC},
{0x120ED,Lo,0x120ED,0x120ED,0x120ED},
{0x120EE,Lo,0x120EE,0x120EE,0x120EE},
{0x120EF,Lo,0x120EF,0x120EF,0x120EF},
{0x120F0,Lo,0x120F0,0x120F0,0x120F0},
{0x120F1,Lo,0x120F1,0x120F1,0x120F1},
{0x120F2,Lo,0x120F2,0x120F2,0x120F2},
{0x120F3,Lo,0x120F3,0x120F3,0x120F3},
{0x120F4,Lo,0x120F4,0x120F4,0x120F4},
{0x120F5,Lo,0x120F5,0x120F5,0x120F5},
{0x120F6,Lo,0x120F6,0x120F6,0x120F6},
{0x120F7,Lo,0x120F7,0x120F7,0x120F7},
{0x120F8,Lo,0x120F8,0x120F8,0x120F8},
{0x120F9,Lo,0x120F9,0x120F9,0x120F9},
{0x120FA,Lo,0x120FA,0x120FA,0x120FA},
{0x120FB,Lo,0x120FB,0x120FB,0x120FB},
{0x120FC,Lo,0x120FC,0x120FC,0x120FC},
{0x120FD,Lo,0x120FD,0x120FD,0x120FD},
{0x120FE,Lo,0x120FE,0x120FE,0x120FE},
{0x120FF,Lo,0x120FF,0x120FF,0x120FF},
{0x12100,Lo,0x12100,0x12100,0x12100},
{0x12101,Lo,0x12101,0x12101,0x12101},
{0x12102,Lo,0x12102,0x12102,0x12102},
{0x12103,Lo,0x12103,0x12103,0x12103},
{0x12104,Lo,0x12104,0x12104,0x12104},
{0x12105,Lo,0x12105,0x12105,0x12105},
{0x12106,Lo,0x12106,0x12106,0x12106},
{0x12107,Lo,0x12107,0x12107,0x12107},
{0x12108,Lo,0x12108,0x12108,0x12108},
{0x12109,Lo,0x12109,0x12109,0x12109},
{0x1210A,Lo,0x1210A,0x1210A,0x1210A},
{0x1210B,Lo,0x1210B,0x1210B,0x1210B},
{0x1210C,Lo,0x1210C,0x1210C,0x1210C},
{0x1210D,Lo,0x1210D,0x1210D,0x1210D},
{0x1210E,Lo,0x1210E,0x1210E,0x1210E},
{0x1210F,Lo,0x1210F,0x1210F,0x1210F},
{0x12110,Lo,0x12110,0x12110,0x12110},
{0x12111,Lo,0x12111,0x12111,0x12111},
{0x12112,Lo,0x12112,0x12112,0x12112},
{0x12113,Lo,0x12113,0x12113,0x12113},
{0x12114,Lo,0x12114,0x12114,0x12114},
{0x12115,Lo,0x12115,0x12115,0x12115},
{0x12116,Lo,0x12116,0x12116,0x12116},
{0x12117,Lo,0x12117,0x12117,0x12117},
{0x12118,Lo,0x12118,0x12118,0x12118},
{0x12119,Lo,0x12119,0x12119,0x12119},
{0x1211A,Lo,0x1211A,0x1211A,0x1211A},
{0x1211B,Lo,0x1211B,0x1211B,0x1211B},
{0x1211C,Lo,0x1211C,0x1211C,0x1211C},
{0x1211D,Lo,0x1211D,0x1211D,0x1211D},
{0x1211E,Lo,0x1211E,0x1211E,0x1211E},
{0x1211F,Lo,0x1211F,0x1211F,0x1211F},
{0x12120,Lo,0x12120,0x12120,0x12120},
{0x12121,Lo,0x12121,0x12121,0x12121},
{0x12122,Lo,0x12122,0x12122,0x12122},
{0x12123,Lo,0x12123,0x12123,0x12123},
{0x12124,Lo,0x12124,0x12124,0x12124},
{0x12125,Lo,0x12125,0x12125,0x12125},
{0x12126,Lo,0x12126,0x12126,0x12126},
{0x12127,Lo,0x12127,0x12127,0x12127},
{0x12128,Lo,0x12128,0x12128,0x12128},
{0x12129,Lo,0x12129,0x12129,0x12129},
{0x1212A,Lo,0x1212A,0x1212A,0x1212A},
{0x1212B,Lo,0x1212B,0x1212B,0x1212B},
{0x1212C,Lo,0x1212C,0x1212C,0x1212C},
{0x1212D,Lo,0x1212D,0x1212D,0x1212D},
{0x1212E,Lo,0x1212E,0x1212E,0x1212E},
{0x1212F,Lo,0x1212F,0x1212F,0x1212F},
{0x12130,Lo,0x12130,0x12130,0x12130},
{0x12131,Lo,0x12131,0x12131,0x12131},
{0x12132,Lo,0x12132,0x12132,0x12132},
{0x12133,Lo,0x12133,0x12133,0x12133},
{0x12134,Lo,0x12134,0x12134,0x12134},
{0x12135,Lo,0x12135,0x12135,0x12135},
{0x12136,Lo,0x12136,0x12136,0x12136},
{0x12137,Lo,0x12137,0x12137,0x12137},
{0x12138,Lo,0x12138,0x12138,0x12138},
{0x12139,Lo,0x12139,0x12139,0x12139},
{0x1213A,Lo,0x1213A,0x1213A,0x1213A},
{0x1213B,Lo,0x1213B,0x1213B,0x1213B},
{0x1213C,Lo,0x1213C,0x1213C,0x1213C},
{0x1213D,Lo,0x1213D,0x1213D,0x1213D},
{0x1213E,Lo,0x1213E,0x1213E,0x1213E},
{0x1213F,Lo,0x1213F,0x1213F,0x1213F},
{0x12140,Lo,0x12140,0x12140,0x12140},
{0x12141,Lo,0x12141,0x12141,0x12141},
{0x12142,Lo,0x12142,0x12142,0x12142},
{0x12143,Lo,0x12143,0x12143,0x12143},
{0x12144,Lo,0x12144,0x12144,0x12144},
{0x12145,Lo,0x12145,0x12145,0x12145},
{0x12146,Lo,0x12146,0x12146,0x12146},
{0x12147,Lo,0x12147,0x12147,0x12147},
{0x12148,Lo,0x12148,0x12148,0x12148},
{0x12149,Lo,0x12149,0x12149,0x12149},
{0x1214A,Lo,0x1214A,0x1214A,0x1214A},
{0x1214B,Lo,0x1214B,0x1214B,0x1214B},
{0x1214C,Lo,0x1214C,0x1214C,0x1214C},
{0x1214D,Lo,0x1214D,0x1214D,0x1214D},
{0x1214E,Lo,0x1214E,0x1214E,0x1214E},
{0x1214F,Lo,0x1214F,0x1214F,0x1214F},
{0x12150,Lo,0x12150,0x12150,0x12150},
{0x12151,Lo,0x12151,0x12151,0x12151},
{0x12152,Lo,0x12152,0x12152,0x12152},
{0x12153,Lo,0x12153,0x12153,0x12153},
{0x12154,Lo,0x12154,0x12154,0x12154},
{0x12155,Lo,0x12155,0x12155,0x12155},
{0x12156,Lo,0x12156,0x12156,0x12156},
{0x12157,Lo,0x12157,0x12157,0x12157},
{0x12158,Lo,0x12158,0x12158,0x12158},
{0x12159,Lo,0x12159,0x12159,0x12159},
{0x1215A,Lo,0x1215A,0x1215A,0x1215A},
{0x1215B,Lo,0x1215B,0x1215B,0x1215B},
{0x1215C,Lo,0x1215C,0x1215C,0x1215C},
{0x1215D,Lo,0x1215D,0x1215D,0x1215D},
{0x1215E,Lo,0x1215E,0x1215E,0x1215E},
{0x1215F,Lo,0x1215F,0x1215F,0x1215F},
{0x12160,Lo,0x12160,0x12160,0x12160},
{0x12161,Lo,0x12161,0x12161,0x12161},
{0x12162,Lo,0x12162,0x12162,0x12162},
{0x12163,Lo,0x12163,0x12163,0x12163},
{0x12164,Lo,0x12164,0x12164,0x12164},
{0x12165,Lo,0x12165,0x12165,0x12165},
{0x12166,Lo,0x12166,0x12166,0x12166},
{0x12167,Lo,0x12167,0x12167,0x12167},
{0x12168,Lo,0x12168,0x12168,0x12168},
{0x12169,Lo,0x12169,0x12169,0x12169},
{0x1216A,Lo,0x1216A,0x1216A,0x1216A},
{0x1216B,Lo,0x1216B,0x1216B,0x1216B},
{0x1216C,Lo,0x1216C,0x1216C,0x1216C},
{0x1216D,Lo,0x1216D,0x1216D,0x1216D},
{0x1216E,Lo,0x1216E,0x1216E,0x1216E},
{0x1216F,Lo,0x1216F,0x1216F,0x1216F},
{0x12170,Lo,0x12170,0x12170,0x12170},
{0x12171,Lo,0x12171,0x12171,0x12171},
{0x12172,Lo,0x12172,0x12172,0x12172},
{0x12173,Lo,0x12173,0x12173,0x12173},
{0x12174,Lo,0x12174,0x12174,0x12174},
{0x12175,Lo,0x12175,0x12175,0x12175},
{0x12176,Lo,0x12176,0x12176,0x12176},
{0x12177,Lo,0x12177,0x12177,0x12177},
{0x12178,Lo,0x12178,0x12178,0x12178},
{0x12179,Lo,0x12179,0x12179,0x12179},
{0x1217A,Lo,0x1217A,0x1217A,0x1217A},
{0x1217B,Lo,0x1217B,0x1217B,0x1217B},
{0x1217C,Lo,0x1217C,0x1217C,0x1217C},
{0x1217D,Lo,0x1217D,0x1217D,0x1217D},
{0x1217E,Lo,0x1217E,0x1217E,0x1217E},
{0x1217F,Lo,0x1217F,0x1217F,0x1217F},
{0x12180,Lo,0x12180,0x12180,0x12180},
{0x12181,Lo,0x12181,0x12181,0x12181},
{0x12182,Lo,0x12182,0x12182,0x12182},
{0x12183,Lo,0x12183,0x12183,0x12183},
{0x12184,Lo,0x12184,0x12184,0x12184},
{0x12185,Lo,0x12185,0x12185,0x12185},
{0x12186,Lo,0x12186,0x12186,0x12186},
{0x12187,Lo,0x12187,0x12187,0x12187},
{0x12188,Lo,0x12188,0x12188,0x12188},
{0x12189,Lo,0x12189,0x12189,0x12189},
{0x1218A,Lo,0x1218A,0x1218A,0x1218A},
{0x1218B,Lo,0x1218B,0x1218B,0x1218B},
{0x1218C,Lo,0x1218C,0x1218C,0x1218C},
{0x1218D,Lo,0x1218D,0x1218D,0x1218D},
{0x1218E,Lo,0x1218E,0x1218E,0x1218E},
{0x1218F,Lo,0x1218F,0x1218F,0x1218F},
{0x12190,Lo,0x12190,0x12190,0x12190},
{0x12191,Lo,0x12191,0x12191,0x12191},
{0x12192,Lo,0x12192,0x12192,0x12192},
{0x12193,Lo,0x12193,0x12193,0x12193},
{0x12194,Lo,0x12194,0x12194,0x12194},
{0x12195,Lo,0x12195,0x12195,0x12195},
{0x12196,Lo,0x12196,0x12196,0x12196},
{0x12197,Lo,0x12197,0x12197,0x12197},
{0x12198,Lo,0x12198,0x12198,0x12198},
{0x12199,Lo,0x12199,0x12199,0x12199},
{0x1219A,Lo,0x1219A,0x1219A,0x1219A},
{0x1219B,Lo,0x1219B,0x1219B,0x1219B},
{0x1219C,Lo,0x1219C,0x1219C,0x1219C},
{0x1219D,Lo,0x1219D,0x1219D,0x1219D},
{0x1219E,Lo,0x1219E,0x1219E,0x1219E},
{0x1219F,Lo,0x1219F,0x1219F,0x1219F},
{0x121A0,Lo,0x121A0,0x121A0,0x121A0},
{0x121A1,Lo,0x121A1,0x121A1,0x121A1},
{0x121A2,Lo,0x121A2,0x121A2,0x121A2},
{0x121A3,Lo,0x121A3,0x121A3,0x121A3},
{0x121A4,Lo,0x121A4,0x121A4,0x121A4},
{0x121A5,Lo,0x121A5,0x121A5,0x121A5},
{0x121A6,Lo,0x121A6,0x121A6,0x121A6},
{0x121A7,Lo,0x121A7,0x121A7,0x121A7},
{0x121A8,Lo,0x121A8,0x121A8,0x121A8},
{0x121A9,Lo,0x121A9,0x121A9,0x121A9},
{0x121AA,Lo,0x121AA,0x121AA,0x121AA},
{0x121AB,Lo,0x121AB,0x121AB,0x121AB},
{0x121AC,Lo,0x121AC,0x121AC,0x121AC},
{0x121AD,Lo,0x121AD,0x121AD,0x121AD},
{0x121AE,Lo,0x121AE,0x121AE,0x121AE},
{0x121AF,Lo,0x121AF,0x121AF,0x121AF},
{0x121B0,Lo,0x121B0,0x121B0,0x121B0},
{0x121B1,Lo,0x121B1,0x121B1,0x121B1},
{0x121B2,Lo,0x121B2,0x121B2,0x121B2},
{0x121B3,Lo,0x121B3,0x121B3,0x121B3},
{0x121B4,Lo,0x121B4,0x121B4,0x121B4},
{0x121B5,Lo,0x121B5,0x121B5,0x121B5},
{0x121B6,Lo,0x121B6,0x121B6,0x121B6},
{0x121B7,Lo,0x121B7,0x121B7,0x121B7},
{0x121B8,Lo,0x121B8,0x121B8,0x121B8},
{0x121B9,Lo,0x121B9,0x121B9,0x121B9},
{0x121BA,Lo,0x121BA,0x121BA,0x121BA},
{0x121BB,Lo,0x121BB,0x121BB,0x121BB},
{0x121BC,Lo,0x121BC,0x121BC,0x121BC},
{0x121BD,Lo,0x121BD,0x121BD,0x121BD},
{0x121BE,Lo,0x121BE,0x121BE,0x121BE},
{0x121BF,Lo,0x121BF,0x121BF,0x121BF},
{0x121C0,Lo,0x121C0,0x121C0,0x121C0},
{0x121C1,Lo,0x121C1,0x121C1,0x121C1},
{0x121C2,Lo,0x121C2,0x121C2,0x121C2},
{0x121C3,Lo,0x121C3,0x121C3,0x121C3},
{0x121C4,Lo,0x121C4,0x121C4,0x121C4},
{0x121C5,Lo,0x121C5,0x121C5,0x121C5},
{0x121C6,Lo,0x121C6,0x121C6,0x121C6},
{0x121C7,Lo,0x121C7,0x121C7,0x121C7},
{0x121C8,Lo,0x121C8,0x121C8,0x121C8},
{0x121C9,Lo,0x121C9,0x121C9,0x121C9},
{0x121CA,Lo,0x121CA,0x121CA,0x121CA},
{0x121CB,Lo,0x121CB,0x121CB,0x121CB},
{0x121CC,Lo,0x121CC,0x121CC,0x121CC},
{0x121CD,Lo,0x121CD,0x121CD,0x121CD},
{0x121CE,Lo,0x121CE,0x121CE,0x121CE},
{0x121CF,Lo,0x121CF,0x121CF,0x121CF},
{0x121D0,Lo,0x121D0,0x121D0,0x121D0},
{0x121D1,Lo,0x121D1,0x121D1,0x121D1},
{0x121D2,Lo,0x121D2,0x121D2,0x121D2},
{0x121D3,Lo,0x121D3,0x121D3,0x121D3},
{0x121D4,Lo,0x121D4,0x121D4,0x121D4},
{0x121D5,Lo,0x121D5,0x121D5,0x121D5},
{0x121D6,Lo,0x121D6,0x121D6,0x121D6},
{0x121D7,Lo,0x121D7,0x121D7,0x121D7},
{0x121D8,Lo,0x121D8,0x121D8,0x121D8},
{0x121D9,Lo,0x121D9,0x121D9,0x121D9},
{0x121DA,Lo,0x121DA,0x121DA,0x121DA},
{0x121DB,Lo,0x121DB,0x121DB,0x121DB},
{0x121DC,Lo,0x121DC,0x121DC,0x121DC},
{0x121DD,Lo,0x121DD,0x121DD,0x121DD},
{0x121DE,Lo,0x121DE,0x121DE,0x121DE},
{0x121DF,Lo,0x121DF,0x121DF,0x121DF},
{0x121E0,Lo,0x121E0,0x121E0,0x121E0},
{0x121E1,Lo,0x121E1,0x121E1,0x121E1},
{0x121E2,Lo,0x121E2,0x121E2,0x121E2},
{0x121E3,Lo,0x121E3,0x121E3,0x121E3},
{0x121E4,Lo,0x121E4,0x121E4,0x121E4},
{0x121E5,Lo,0x121E5,0x121E5,0x121E5},
{0x121E6,Lo,0x121E6,0x121E6,0x121E6},
{0x121E7,Lo,0x121E7,0x121E7,0x121E7},
{0x121E8,Lo,0x121E8,0x121E8,0x121E8},
{0x121E9,Lo,0x121E9,0x121E9,0x121E9},
{0x121EA,Lo,0x121EA,0x121EA,0x121EA},
{0x121EB,Lo,0x121EB,0x121EB,0x121EB},
{0x121EC,Lo,0x121EC,0x121EC,0x121EC},
{0x121ED,Lo,0x121ED,0x121ED,0x121ED},
{0x121EE,Lo,0x121EE,0x121EE,0x121EE},
{0x121EF,Lo,0x121EF,0x121EF,0x121EF},
{0x121F0,Lo,0x121F0,0x121F0,0x121F0},
{0x121F1,Lo,0x121F1,0x121F1,0x121F1},
{0x121F2,Lo,0x121F2,0x121F2,0x121F2},
{0x121F3,Lo,0x121F3,0x121F3,0x121F3},
{0x121F4,Lo,0x121F4,0x121F4,0x121F4},
{0x121F5,Lo,0x121F5,0x121F5,0x121F5},
{0x121F6,Lo,0x121F6,0x121F6,0x121F6},
{0x121F7,Lo,0x121F7,0x121F7,0x121F7},
{0x121F8,Lo,0x121F8,0x121F8,0x121F8},
{0x121F9,Lo,0x121F9,0x121F9,0x121F9},
{0x121FA,Lo,0x121FA,0x121FA,0x121FA},
{0x121FB,Lo,0x121FB,0x121FB,0x121FB},
{0x121FC,Lo,0x121FC,0x121FC,0x121FC},
{0x121FD,Lo,0x121FD,0x121FD,0x121FD},
{0x121FE,Lo,0x121FE,0x121FE,0x121FE},
{0x121FF,Lo,0x121FF,0x121FF,0x121FF},
{0x12200,Lo,0x12200,0x12200,0x12200},
{0x12201,Lo,0x12201,0x12201,0x12201},
{0x12202,Lo,0x12202,0x12202,0x12202},
{0x12203,Lo,0x12203,0x12203,0x12203},
{0x12204,Lo,0x12204,0x12204,0x12204},
{0x12205,Lo,0x12205,0x12205,0x12205},
{0x12206,Lo,0x12206,0x12206,0x12206},
{0x12207,Lo,0x12207,0x12207,0x12207},
{0x12208,Lo,0x12208,0x12208,0x12208},
{0x12209,Lo,0x12209,0x12209,0x12209},
{0x1220A,Lo,0x1220A,0x1220A,0x1220A},
{0x1220B,Lo,0x1220B,0x1220B,0x1220B},
{0x1220C,Lo,0x1220C,0x1220C,0x1220C},
{0x1220D,Lo,0x1220D,0x1220D,0x1220D},
{0x1220E,Lo,0x1220E,0x1220E,0x1220E},
{0x1220F,Lo,0x1220F,0x1220F,0x1220F},
{0x12210,Lo,0x12210,0x12210,0x12210},
{0x12211,Lo,0x12211,0x12211,0x12211},
{0x12212,Lo,0x12212,0x12212,0x12212},
{0x12213,Lo,0x12213,0x12213,0x12213},
{0x12214,Lo,0x12214,0x12214,0x12214},
{0x12215,Lo,0x12215,0x12215,0x12215},
{0x12216,Lo,0x12216,0x12216,0x12216},
{0x12217,Lo,0x12217,0x12217,0x12217},
{0x12218,Lo,0x12218,0x12218,0x12218},
{0x12219,Lo,0x12219,0x12219,0x12219},
{0x1221A,Lo,0x1221A,0x1221A,0x1221A},
{0x1221B,Lo,0x1221B,0x1221B,0x1221B},
{0x1221C,Lo,0x1221C,0x1221C,0x1221C},
{0x1221D,Lo,0x1221D,0x1221D,0x1221D},
{0x1221E,Lo,0x1221E,0x1221E,0x1221E},
{0x1221F,Lo,0x1221F,0x1221F,0x1221F},
{0x12220,Lo,0x12220,0x12220,0x12220},
{0x12221,Lo,0x12221,0x12221,0x12221},
{0x12222,Lo,0x12222,0x12222,0x12222},
{0x12223,Lo,0x12223,0x12223,0x12223},
{0x12224,Lo,0x12224,0x12224,0x12224},
{0x12225,Lo,0x12225,0x12225,0x12225},
{0x12226,Lo,0x12226,0x12226,0x12226},
{0x12227,Lo,0x12227,0x12227,0x12227},
{0x12228,Lo,0x12228,0x12228,0x12228},
{0x12229,Lo,0x12229,0x12229,0x12229},
{0x1222A,Lo,0x1222A,0x1222A,0x1222A},
{0x1222B,Lo,0x1222B,0x1222B,0x1222B},
{0x1222C,Lo,0x1222C,0x1222C,0x1222C},
{0x1222D,Lo,0x1222D,0x1222D,0x1222D},
{0x1222E,Lo,0x1222E,0x1222E,0x1222E},
{0x1222F,Lo,0x1222F,0x1222F,0x1222F},
{0x12230,Lo,0x12230,0x12230,0x12230},
{0x12231,Lo,0x12231,0x12231,0x12231},
{0x12232,Lo,0x12232,0x12232,0x12232},
{0x12233,Lo,0x12233,0x12233,0x12233},
{0x12234,Lo,0x12234,0x12234,0x12234},
{0x12235,Lo,0x12235,0x12235,0x12235},
{0x12236,Lo,0x12236,0x12236,0x12236},
{0x12237,Lo,0x12237,0x12237,0x12237},
{0x12238,Lo,0x12238,0x12238,0x12238},
{0x12239,Lo,0x12239,0x12239,0x12239},
{0x1223A,Lo,0x1223A,0x1223A,0x1223A},
{0x1223B,Lo,0x1223B,0x1223B,0x1223B},
{0x1223C,Lo,0x1223C,0x1223C,0x1223C},
{0x1223D,Lo,0x1223D,0x1223D,0x1223D},
{0x1223E,Lo,0x1223E,0x1223E,0x1223E},
{0x1223F,Lo,0x1223F,0x1223F,0x1223F},
{0x12240,Lo,0x12240,0x12240,0x12240},
{0x12241,Lo,0x12241,0x12241,0x12241},
{0x12242,Lo,0x12242,0x12242,0x12242},
{0x12243,Lo,0x12243,0x12243,0x12243},
{0x12244,Lo,0x12244,0x12244,0x12244},
{0x12245,Lo,0x12245,0x12245,0x12245},
{0x12246,Lo,0x12246,0x12246,0x12246},
{0x12247,Lo,0x12247,0x12247,0x12247},
{0x12248,Lo,0x12248,0x12248,0x12248},
{0x12249,Lo,0x12249,0x12249,0x12249},
{0x1224A,Lo,0x1224A,0x1224A,0x1224A},
{0x1224B,Lo,0x1224B,0x1224B,0x1224B},
{0x1224C,Lo,0x1224C,0x1224C,0x1224C},
{0x1224D,Lo,0x1224D,0x1224D,0x1224D},
{0x1224E,Lo,0x1224E,0x1224E,0x1224E},
{0x1224F,Lo,0x1224F,0x1224F,0x1224F},
{0x12250,Lo,0x12250,0x12250,0x12250},
{0x12251,Lo,0x12251,0x12251,0x12251},
{0x12252,Lo,0x12252,0x12252,0x12252},
{0x12253,Lo,0x12253,0x12253,0x12253},
{0x12254,Lo,0x12254,0x12254,0x12254},
{0x12255,Lo,0x12255,0x12255,0x12255},
{0x12256,Lo,0x12256,0x12256,0x12256},
{0x12257,Lo,0x12257,0x12257,0x12257},
{0x12258,Lo,0x12258,0x12258,0x12258},
{0x12259,Lo,0x12259,0x12259,0x12259},
{0x1225A,Lo,0x1225A,0x1225A,0x1225A},
{0x1225B,Lo,0x1225B,0x1225B,0x1225B},
{0x1225C,Lo,0x1225C,0x1225C,0x1225C},
{0x1225D,Lo,0x1225D,0x1225D,0x1225D},
{0x1225E,Lo,0x1225E,0x1225E,0x1225E},
{0x1225F,Lo,0x1225F,0x1225F,0x1225F},
{0x12260,Lo,0x12260,0x12260,0x12260},
{0x12261,Lo,0x12261,0x12261,0x12261},
{0x12262,Lo,0x12262,0x12262,0x12262},
{0x12263,Lo,0x12263,0x12263,0x12263},
{0x12264,Lo,0x12264,0x12264,0x12264},
{0x12265,Lo,0x12265,0x12265,0x12265},
{0x12266,Lo,0x12266,0x12266,0x12266},
{0x12267,Lo,0x12267,0x12267,0x12267},
{0x12268,Lo,0x12268,0x12268,0x12268},
{0x12269,Lo,0x12269,0x12269,0x12269},
{0x1226A,Lo,0x1226A,0x1226A,0x1226A},
{0x1226B,Lo,0x1226B,0x1226B,0x1226B},
{0x1226C,Lo,0x1226C,0x1226C,0x1226C},
{0x1226D,Lo,0x1226D,0x1226D,0x1226D},
{0x1226E,Lo,0x1226E,0x1226E,0x1226E},
{0x1226F,Lo,0x1226F,0x1226F,0x1226F},
{0x12270,Lo,0x12270,0x12270,0x12270},
{0x12271,Lo,0x12271,0x12271,0x12271},
{0x12272,Lo,0x12272,0x12272,0x12272},
{0x12273,Lo,0x12273,0x12273,0x12273},
{0x12274,Lo,0x12274,0x12274,0x12274},
{0x12275,Lo,0x12275,0x12275,0x12275},
{0x12276,Lo,0x12276,0x12276,0x12276},
{0x12277,Lo,0x12277,0x12277,0x12277},
{0x12278,Lo,0x12278,0x12278,0x12278},
{0x12279,Lo,0x12279,0x12279,0x12279},
{0x1227A,Lo,0x1227A,0x1227A,0x1227A},
{0x1227B,Lo,0x1227B,0x1227B,0x1227B},
{0x1227C,Lo,0x1227C,0x1227C,0x1227C},
{0x1227D,Lo,0x1227D,0x1227D,0x1227D},
{0x1227E,Lo,0x1227E,0x1227E,0x1227E},
{0x1227F,Lo,0x1227F,0x1227F,0x1227F},
{0x12280,Lo,0x12280,0x12280,0x12280},
{0x12281,Lo,0x12281,0x12281,0x12281},
{0x12282,Lo,0x12282,0x12282,0x12282},
{0x12283,Lo,0x12283,0x12283,0x12283},
{0x12284,Lo,0x12284,0x12284,0x12284},
{0x12285,Lo,0x12285,0x12285,0x12285},
{0x12286,Lo,0x12286,0x12286,0x12286},
{0x12287,Lo,0x12287,0x12287,0x12287},
{0x12288,Lo,0x12288,0x12288,0x12288},
{0x12289,Lo,0x12289,0x12289,0x12289},
{0x1228A,Lo,0x1228A,0x1228A,0x1228A},
{0x1228B,Lo,0x1228B,0x1228B,0x1228B},
{0x1228C,Lo,0x1228C,0x1228C,0x1228C},
{0x1228D,Lo,0x1228D,0x1228D,0x1228D},
{0x1228E,Lo,0x1228E,0x1228E,0x1228E},
{0x1228F,Lo,0x1228F,0x1228F,0x1228F},
{0x12290,Lo,0x12290,0x12290,0x12290},
{0x12291,Lo,0x12291,0x12291,0x12291},
{0x12292,Lo,0x12292,0x12292,0x12292},
{0x12293,Lo,0x12293,0x12293,0x12293},
{0x12294,Lo,0x12294,0x12294,0x12294},
{0x12295,Lo,0x12295,0x12295,0x12295},
{0x12296,Lo,0x12296,0x12296,0x12296},
{0x12297,Lo,0x12297,0x12297,0x12297},
{0x12298,Lo,0x12298,0x12298,0x12298},
{0x12299,Lo,0x12299,0x12299,0x12299},
{0x1229A,Lo,0x1229A,0x1229A,0x1229A},
{0x1229B,Lo,0x1229B,0x1229B,0x1229B},
{0x1229C,Lo,0x1229C,0x1229C,0x1229C},
{0x1229D,Lo,0x1229D,0x1229D,0x1229D},
{0x1229E,Lo,0x1229E,0x1229E,0x1229E},
{0x1229F,Lo,0x1229F,0x1229F,0x1229F},
{0x122A0,Lo,0x122A0,0x122A0,0x122A0},
{0x122A1,Lo,0x122A1,0x122A1,0x122A1},
{0x122A2,Lo,0x122A2,0x122A2,0x122A2},
{0x122A3,Lo,0x122A3,0x122A3,0x122A3},
{0x122A4,Lo,0x122A4,0x122A4,0x122A4},
{0x122A5,Lo,0x122A5,0x122A5,0x122A5},
{0x122A6,Lo,0x122A6,0x122A6,0x122A6},
{0x122A7,Lo,0x122A7,0x122A7,0x122A7},
{0x122A8,Lo,0x122A8,0x122A8,0x122A8},
{0x122A9,Lo,0x122A9,0x122A9,0x122A9},
{0x122AA,Lo,0x122AA,0x122AA,0x122AA},
{0x122AB,Lo,0x122AB,0x122AB,0x122AB},
{0x122AC,Lo,0x122AC,0x122AC,0x122AC},
{0x122AD,Lo,0x122AD,0x122AD,0x122AD},
{0x122AE,Lo,0x122AE,0x122AE,0x122AE},
{0x122AF,Lo,0x122AF,0x122AF,0x122AF},
{0x122B0,Lo,0x122B0,0x122B0,0x122B0},
{0x122B1,Lo,0x122B1,0x122B1,0x122B1},
{0x122B2,Lo,0x122B2,0x122B2,0x122B2},
{0x122B3,Lo,0x122B3,0x122B3,0x122B3},
{0x122B4,Lo,0x122B4,0x122B4,0x122B4},
{0x122B5,Lo,0x122B5,0x122B5,0x122B5},
{0x122B6,Lo,0x122B6,0x122B6,0x122B6},
{0x122B7,Lo,0x122B7,0x122B7,0x122B7},
{0x122B8,Lo,0x122B8,0x122B8,0x122B8},
{0x122B9,Lo,0x122B9,0x122B9,0x122B9},
{0x122BA,Lo,0x122BA,0x122BA,0x122BA},
{0x122BB,Lo,0x122BB,0x122BB,0x122BB},
{0x122BC,Lo,0x122BC,0x122BC,0x122BC},
{0x122BD,Lo,0x122BD,0x122BD,0x122BD},
{0x122BE,Lo,0x122BE,0x122BE,0x122BE},
{0x122BF,Lo,0x122BF,0x122BF,0x122BF},
{0x122C0,Lo,0x122C0,0x122C0,0x122C0},
{0x122C1,Lo,0x122C1,0x122C1,0x122C1},
{0x122C2,Lo,0x122C2,0x122C2,0x122C2},
{0x122C3,Lo,0x122C3,0x122C3,0x122C3},
{0x122C4,Lo,0x122C4,0x122C4,0x122C4},
{0x122C5,Lo,0x122C5,0x122C5,0x122C5},
{0x122C6,Lo,0x122C6,0x122C6,0x122C6},
{0x122C7,Lo,0x122C7,0x122C7,0x122C7},
{0x122C8,Lo,0x122C8,0x122C8,0x122C8},
{0x122C9,Lo,0x122C9,0x122C9,0x122C9},
{0x122CA,Lo,0x122CA,0x122CA,0x122CA},
{0x122CB,Lo,0x122CB,0x122CB,0x122CB},
{0x122CC,Lo,0x122CC,0x122CC,0x122CC},
{0x122CD,Lo,0x122CD,0x122CD,0x122CD},
{0x122CE,Lo,0x122CE,0x122CE,0x122CE},
{0x122CF,Lo,0x122CF,0x122CF,0x122CF},
{0x122D0,Lo,0x122D0,0x122D0,0x122D0},
{0x122D1,Lo,0x122D1,0x122D1,0x122D1},
{0x122D2,Lo,0x122D2,0x122D2,0x122D2},
{0x122D3,Lo,0x122D3,0x122D3,0x122D3},
{0x122D4,Lo,0x122D4,0x122D4,0x122D4},
{0x122D5,Lo,0x122D5,0x122D5,0x122D5},
{0x122D6,Lo,0x122D6,0x122D6,0x122D6},
{0x122D7,Lo,0x122D7,0x122D7,0x122D7},
{0x122D8,Lo,0x122D8,0x122D8,0x122D8},
{0x122D9,Lo,0x122D9,0x122D9,0x122D9},
{0x122DA,Lo,0x122DA,0x122DA,0x122DA},
{0x122DB,Lo,0x122DB,0x122DB,0x122DB},
{0x122DC,Lo,0x122DC,0x122DC,0x122DC},
{0x122DD,Lo,0x122DD,0x122DD,0x122DD},
{0x122DE,Lo,0x122DE,0x122DE,0x122DE},
{0x122DF,Lo,0x122DF,0x122DF,0x122DF},
{0x122E0,Lo,0x122E0,0x122E0,0x122E0},
{0x122E1,Lo,0x122E1,0x122E1,0x122E1},
{0x122E2,Lo,0x122E2,0x122E2,0x122E2},
{0x122E3,Lo,0x122E3,0x122E3,0x122E3},
{0x122E4,Lo,0x122E4,0x122E4,0x122E4},
{0x122E5,Lo,0x122E5,0x122E5,0x122E5},
{0x122E6,Lo,0x122E6,0x122E6,0x122E6},
{0x122E7,Lo,0x122E7,0x122E7,0x122E7},
{0x122E8,Lo,0x122E8,0x122E8,0x122E8},
{0x122E9,Lo,0x122E9,0x122E9,0x122E9},
{0x122EA,Lo,0x122EA,0x122EA,0x122EA},
{0x122EB,Lo,0x122EB,0x122EB,0x122EB},
{0x122EC,Lo,0x122EC,0x122EC,0x122EC},
{0x122ED,Lo,0x122ED,0x122ED,0x122ED},
{0x122EE,Lo,0x122EE,0x122EE,0x122EE},
{0x122EF,Lo,0x122EF,0x122EF,0x122EF},
{0x122F0,Lo,0x122F0,0x122F0,0x122F0},
{0x122F1,Lo,0x122F1,0x122F1,0x122F1},
{0x122F2,Lo,0x122F2,0x122F2,0x122F2},
{0x122F3,Lo,0x122F3,0x122F3,0x122F3},
{0x122F4,Lo,0x122F4,0x122F4,0x122F4},
{0x122F5,Lo,0x122F5,0x122F5,0x122F5},
{0x122F6,Lo,0x122F6,0x122F6,0x122F6},
{0x122F7,Lo,0x122F7,0x122F7,0x122F7},
{0x122F8,Lo,0x122F8,0x122F8,0x122F8},
{0x122F9,Lo,0x122F9,0x122F9,0x122F9},
{0x122FA,Lo,0x122FA,0x122FA,0x122FA},
{0x122FB,Lo,0x122FB,0x122FB,0x122FB},
{0x122FC,Lo,0x122FC,0x122FC,0x122FC},
{0x122FD,Lo,0x122FD,0x122FD,0x122FD},
{0x122FE,Lo,0x122FE,0x122FE,0x122FE},
{0x122FF,Lo,0x122FF,0x122FF,0x122FF},
{0x12300,Lo,0x12300,0x12300,0x12300},
{0x12301,Lo,0x12301,0x12301,0x12301},
{0x12302,Lo,0x12302,0x12302,0x12302},
{0x12303,Lo,0x12303,0x12303,0x12303},
{0x12304,Lo,0x12304,0x12304,0x12304},
{0x12305,Lo,0x12305,0x12305,0x12305},
{0x12306,Lo,0x12306,0x12306,0x12306},
{0x12307,Lo,0x12307,0x12307,0x12307},
{0x12308,Lo,0x12308,0x12308,0x12308},
{0x12309,Lo,0x12309,0x12309,0x12309},
{0x1230A,Lo,0x1230A,0x1230A,0x1230A},
{0x1230B,Lo,0x1230B,0x1230B,0x1230B},
{0x1230C,Lo,0x1230C,0x1230C,0x1230C},
{0x1230D,Lo,0x1230D,0x1230D,0x1230D},
{0x1230E,Lo,0x1230E,0x1230E,0x1230E},
{0x1230F,Lo,0x1230F,0x1230F,0x1230F},
{0x12310,Lo,0x12310,0x12310,0x12310},
{0x12311,Lo,0x12311,0x12311,0x12311},
{0x12312,Lo,0x12312,0x12312,0x12312},
{0x12313,Lo,0x12313,0x12313,0x12313},
{0x12314,Lo,0x12314,0x12314,0x12314},
{0x12315,Lo,0x12315,0x12315,0x12315},
{0x12316,Lo,0x12316,0x12316,0x12316},
{0x12317,Lo,0x12317,0x12317,0x12317},
{0x12318,Lo,0x12318,0x12318,0x12318},
{0x12319,Lo,0x12319,0x12319,0x12319},
{0x1231A,Lo,0x1231A,0x1231A,0x1231A},
{0x1231B,Lo,0x1231B,0x1231B,0x1231B},
{0x1231C,Lo,0x1231C,0x1231C,0x1231C},
{0x1231D,Lo,0x1231D,0x1231D,0x1231D},
{0x1231E,Lo,0x1231E,0x1231E,0x1231E},
{0x1231F,Lo,0x1231F,0x1231F,0x1231F},
{0x12320,Lo,0x12320,0x12320,0x12320},
{0x12321,Lo,0x12321,0x12321,0x12321},
{0x12322,Lo,0x12322,0x12322,0x12322},
{0x12323,Lo,0x12323,0x12323,0x12323},
{0x12324,Lo,0x12324,0x12324,0x12324},
{0x12325,Lo,0x12325,0x12325,0x12325},
{0x12326,Lo,0x12326,0x12326,0x12326},
{0x12327,Lo,0x12327,0x12327,0x12327},
{0x12328,Lo,0x12328,0x12328,0x12328},
{0x12329,Lo,0x12329,0x12329,0x12329},
{0x1232A,Lo,0x1232A,0x1232A,0x1232A},
{0x1232B,Lo,0x1232B,0x1232B,0x1232B},
{0x1232C,Lo,0x1232C,0x1232C,0x1232C},
{0x1232D,Lo,0x1232D,0x1232D,0x1232D},
{0x1232E,Lo,0x1232E,0x1232E,0x1232E},
{0x1232F,Lo,0x1232F,0x1232F,0x1232F},
{0x12330,Lo,0x12330,0x12330,0x12330},
{0x12331,Lo,0x12331,0x12331,0x12331},
{0x12332,Lo,0x12332,0x12332,0x12332},
{0x12333,Lo,0x12333,0x12333,0x12333},
{0x12334,Lo,0x12334,0x12334,0x12334},
{0x12335,Lo,0x12335,0x12335,0x12335},
{0x12336,Lo,0x12336,0x12336,0x12336},
{0x12337,Lo,0x12337,0x12337,0x12337},
{0x12338,Lo,0x12338,0x12338,0x12338},
{0x12339,Lo,0x12339,0x12339,0x12339},
{0x1233A,Lo,0x1233A,0x1233A,0x1233A},
{0x1233B,Lo,0x1233B,0x1233B,0x1233B},
{0x1233C,Lo,0x1233C,0x1233C,0x1233C},
{0x1233D,Lo,0x1233D,0x1233D,0x1233D},
{0x1233E,Lo,0x1233E,0x1233E,0x1233E},
{0x1233F,Lo,0x1233F,0x1233F,0x1233F},
{0x12340,Lo,0x12340,0x12340,0x12340},
{0x12341,Lo,0x12341,0x12341,0x12341},
{0x12342,Lo,0x12342,0x12342,0x12342},
{0x12343,Lo,0x12343,0x12343,0x12343},
{0x12344,Lo,0x12344,0x12344,0x12344},
{0x12345,Lo,0x12345,0x12345,0x12345},
{0x12346,Lo,0x12346,0x12346,0x12346},
{0x12347,Lo,0x12347,0x12347,0x12347},
{0x12348,Lo,0x12348,0x12348,0x12348},
{0x12349,Lo,0x12349,0x12349,0x12349},
{0x1234A,Lo,0x1234A,0x1234A,0x1234A},
{0x1234B,Lo,0x1234B,0x1234B,0x1234B},
{0x1234C,Lo,0x1234C,0x1234C,0x1234C},
{0x1234D,Lo,0x1234D,0x1234D,0x1234D},
{0x1234E,Lo,0x1234E,0x1234E,0x1234E},
{0x1234F,Lo,0x1234F,0x1234F,0x1234F},
{0x12350,Lo,0x12350,0x12350,0x12350},
{0x12351,Lo,0x12351,0x12351,0x12351},
{0x12352,Lo,0x12352,0x12352,0x12352},
{0x12353,Lo,0x12353,0x12353,0x12353},
{0x12354,Lo,0x12354,0x12354,0x12354},
{0x12355,Lo,0x12355,0x12355,0x12355},
{0x12356,Lo,0x12356,0x12356,0x12356},
{0x12357,Lo,0x12357,0x12357,0x12357},
{0x12358,Lo,0x12358,0x12358,0x12358},
{0x12359,Lo,0x12359,0x12359,0x12359},
{0x1235A,Lo,0x1235A,0x1235A,0x1235A},
{0x1235B,Lo,0x1235B,0x1235B,0x1235B},
{0x1235C,Lo,0x1235C,0x1235C,0x1235C},
{0x1235D,Lo,0x1235D,0x1235D,0x1235D},
{0x1235E,Lo,0x1235E,0x1235E,0x1235E},
{0x1235F,Lo,0x1235F,0x1235F,0x1235F},
{0x12360,Lo,0x12360,0x12360,0x12360},
{0x12361,Lo,0x12361,0x12361,0x12361},
{0x12362,Lo,0x12362,0x12362,0x12362},
{0x12363,Lo,0x12363,0x12363,0x12363},
{0x12364,Lo,0x12364,0x12364,0x12364},
{0x12365,Lo,0x12365,0x12365,0x12365},
{0x12366,Lo,0x12366,0x12366,0x12366},
{0x12367,Lo,0x12367,0x12367,0x12367},
{0x12368,Lo,0x12368,0x12368,0x12368},
{0x12369,Lo,0x12369,0x12369,0x12369},
{0x1236A,Lo,0x1236A,0x1236A,0x1236A},
{0x1236B,Lo,0x1236B,0x1236B,0x1236B},
{0x1236C,Lo,0x1236C,0x1236C,0x1236C},
{0x1236D,Lo,0x1236D,0x1236D,0x1236D},
{0x1236E,Lo,0x1236E,0x1236E,0x1236E},
{0x12400,Nl,0x12400,0x12400,0x12400},
{0x12401,Nl,0x12401,0x12401,0x12401},
{0x12402,Nl,0x12402,0x12402,0x12402},
{0x12403,Nl,0x12403,0x12403,0x12403},
{0x12404,Nl,0x12404,0x12404,0x12404},
{0x12405,Nl,0x12405,0x12405,0x12405},
{0x12406,Nl,0x12406,0x12406,0x12406},
{0x12407,Nl,0x12407,0x12407,0x12407},
{0x12408,Nl,0x12408,0x12408,0x12408},
{0x12409,Nl,0x12409,0x12409,0x12409},
{0x1240A,Nl,0x1240A,0x1240A,0x1240A},
{0x1240B,Nl,0x1240B,0x1240B,0x1240B},
{0x1240C,Nl,0x1240C,0x1240C,0x1240C},
{0x1240D,Nl,0x1240D,0x1240D,0x1240D},
{0x1240E,Nl,0x1240E,0x1240E,0x1240E},
{0x1240F,Nl,0x1240F,0x1240F,0x1240F},
{0x12410,Nl,0x12410,0x12410,0x12410},
{0x12411,Nl,0x12411,0x12411,0x12411},
{0x12412,Nl,0x12412,0x12412,0x12412},
{0x12413,Nl,0x12413,0x12413,0x12413},
{0x12414,Nl,0x12414,0x12414,0x12414},
{0x12415,Nl,0x12415,0x12415,0x12415},
{0x12416,Nl,0x12416,0x12416,0x12416},
{0x12417,Nl,0x12417,0x12417,0x12417},
{0x12418,Nl,0x12418,0x12418,0x12418},
{0x12419,Nl,0x12419,0x12419,0x12419},
{0x1241A,Nl,0x1241A,0x1241A,0x1241A},
{0x1241B,Nl,0x1241B,0x1241B,0x1241B},
{0x1241C,Nl,0x1241C,0x1241C,0x1241C},
{0x1241D,Nl,0x1241D,0x1241D,0x1241D},
{0x1241E,Nl,0x1241E,0x1241E,0x1241E},
{0x1241F,Nl,0x1241F,0x1241F,0x1241F},
{0x12420,Nl,0x12420,0x12420,0x12420},
{0x12421,Nl,0x12421,0x12421,0x12421},
{0x12422,Nl,0x12422,0x12422,0x12422},
{0x12423,Nl,0x12423,0x12423,0x12423},
{0x12424,Nl,0x12424,0x12424,0x12424},
{0x12425,Nl,0x12425,0x12425,0x12425},
{0x12426,Nl,0x12426,0x12426,0x12426},
{0x12427,Nl,0x12427,0x12427,0x12427},
{0x12428,Nl,0x12428,0x12428,0x12428},
{0x12429,Nl,0x12429,0x12429,0x12429},
{0x1242A,Nl,0x1242A,0x1242A,0x1242A},
{0x1242B,Nl,0x1242B,0x1242B,0x1242B},
{0x1242C,Nl,0x1242C,0x1242C,0x1242C},
{0x1242D,Nl,0x1242D,0x1242D,0x1242D},
{0x1242E,Nl,0x1242E,0x1242E,0x1242E},
{0x1242F,Nl,0x1242F,0x1242F,0x1242F},
{0x12430,Nl,0x12430,0x12430,0x12430},
{0x12431,Nl,0x12431,0x12431,0x12431},
{0x12432,Nl,0x12432,0x12432,0x12432},
{0x12433,Nl,0x12433,0x12433,0x12433},
{0x12434,Nl,0x12434,0x12434,0x12434},
{0x12435,Nl,0x12435,0x12435,0x12435},
{0x12436,Nl,0x12436,0x12436,0x12436},
{0x12437,Nl,0x12437,0x12437,0x12437},
{0x12438,Nl,0x12438,0x12438,0x12438},
{0x12439,Nl,0x12439,0x12439,0x12439},
{0x1243A,Nl,0x1243A,0x1243A,0x1243A},
{0x1243B,Nl,0x1243B,0x1243B,0x1243B},
{0x1243C,Nl,0x1243C,0x1243C,0x1243C},
{0x1243D,Nl,0x1243D,0x1243D,0x1243D},
{0x1243E,Nl,0x1243E,0x1243E,0x1243E},
{0x1243F,Nl,0x1243F,0x1243F,0x1243F},
{0x12440,Nl,0x12440,0x12440,0x12440},
{0x12441,Nl,0x12441,0x12441,0x12441},
{0x12442,Nl,0x12442,0x12442,0x12442},
{0x12443,Nl,0x12443,0x12443,0x12443},
{0x12444,Nl,0x12444,0x12444,0x12444},
{0x12445,Nl,0x12445,0x12445,0x12445},
{0x12446,Nl,0x12446,0x12446,0x12446},
{0x12447,Nl,0x12447,0x12447,0x12447},
{0x12448,Nl,0x12448,0x12448,0x12448},
{0x12449,Nl,0x12449,0x12449,0x12449},
{0x1244A,Nl,0x1244A,0x1244A,0x1244A},
{0x1244B,Nl,0x1244B,0x1244B,0x1244B},
{0x1244C,Nl,0x1244C,0x1244C,0x1244C},
{0x1244D,Nl,0x1244D,0x1244D,0x1244D},
{0x1244E,Nl,0x1244E,0x1244E,0x1244E},
{0x1244F,Nl,0x1244F,0x1244F,0x1244F},
{0x12450,Nl,0x12450,0x12450,0x12450},
{0x12451,Nl,0x12451,0x12451,0x12451},
{0x12452,Nl,0x12452,0x12452,0x12452},
{0x12453,Nl,0x12453,0x12453,0x12453},
{0x12454,Nl,0x12454,0x12454,0x12454},
{0x12455,Nl,0x12455,0x12455,0x12455},
{0x12456,Nl,0x12456,0x12456,0x12456},
{0x12457,Nl,0x12457,0x12457,0x12457},
{0x12458,Nl,0x12458,0x12458,0x12458},
{0x12459,Nl,0x12459,0x12459,0x12459},
{0x1245A,Nl,0x1245A,0x1245A,0x1245A},
{0x1245B,Nl,0x1245B,0x1245B,0x1245B},
{0x1245C,Nl,0x1245C,0x1245C,0x1245C},
{0x1245D,Nl,0x1245D,0x1245D,0x1245D},
{0x1245E,Nl,0x1245E,0x1245E,0x1245E},
{0x1245F,Nl,0x1245F,0x1245F,0x1245F},
{0x12460,Nl,0x12460,0x12460,0x12460},
{0x12461,Nl,0x12461,0x12461,0x12461},
{0x12462,Nl,0x12462,0x12462,0x12462},
{0x12470,Po,0x12470,0x12470,0x12470},
{0x12471,Po,0x12471,0x12471,0x12471},
{0x12472,Po,0x12472,0x12472,0x12472},
{0x12473,Po,0x12473,0x12473,0x12473},
{0x1D000,So,0x1D000,0x1D000,0x1D000},
{0x1D001,So,0x1D001,0x1D001,0x1D001},
{0x1D002,So,0x1D002,0x1D002,0x1D002},
{0x1D003,So,0x1D003,0x1D003,0x1D003},
{0x1D004,So,0x1D004,0x1D004,0x1D004},
{0x1D005,So,0x1D005,0x1D005,0x1D005},
{0x1D006,So,0x1D006,0x1D006,0x1D006},
{0x1D007,So,0x1D007,0x1D007,0x1D007},
{0x1D008,So,0x1D008,0x1D008,0x1D008},
{0x1D009,So,0x1D009,0x1D009,0x1D009},
{0x1D00A,So,0x1D00A,0x1D00A,0x1D00A},
{0x1D00B,So,0x1D00B,0x1D00B,0x1D00B},
{0x1D00C,So,0x1D00C,0x1D00C,0x1D00C},
{0x1D00D,So,0x1D00D,0x1D00D,0x1D00D},
{0x1D00E,So,0x1D00E,0x1D00E,0x1D00E},
{0x1D00F,So,0x1D00F,0x1D00F,0x1D00F},
{0x1D010,So,0x1D010,0x1D010,0x1D010},
{0x1D011,So,0x1D011,0x1D011,0x1D011},
{0x1D012,So,0x1D012,0x1D012,0x1D012},
{0x1D013,So,0x1D013,0x1D013,0x1D013},
{0x1D014,So,0x1D014,0x1D014,0x1D014},
{0x1D015,So,0x1D015,0x1D015,0x1D015},
{0x1D016,So,0x1D016,0x1D016,0x1D016},
{0x1D017,So,0x1D017,0x1D017,0x1D017},
{0x1D018,So,0x1D018,0x1D018,0x1D018},
{0x1D019,So,0x1D019,0x1D019,0x1D019},
{0x1D01A,So,0x1D01A,0x1D01A,0x1D01A},
{0x1D01B,So,0x1D01B,0x1D01B,0x1D01B},
{0x1D01C,So,0x1D01C,0x1D01C,0x1D01C},
{0x1D01D,So,0x1D01D,0x1D01D,0x1D01D},
{0x1D01E,So,0x1D01E,0x1D01E,0x1D01E},
{0x1D01F,So,0x1D01F,0x1D01F,0x1D01F},
{0x1D020,So,0x1D020,0x1D020,0x1D020},
{0x1D021,So,0x1D021,0x1D021,0x1D021},
{0x1D022,So,0x1D022,0x1D022,0x1D022},
{0x1D023,So,0x1D023,0x1D023,0x1D023},
{0x1D024,So,0x1D024,0x1D024,0x1D024},
{0x1D025,So,0x1D025,0x1D025,0x1D025},
{0x1D026,So,0x1D026,0x1D026,0x1D026},
{0x1D027,So,0x1D027,0x1D027,0x1D027},
{0x1D028,So,0x1D028,0x1D028,0x1D028},
{0x1D029,So,0x1D029,0x1D029,0x1D029},
{0x1D02A,So,0x1D02A,0x1D02A,0x1D02A},
{0x1D02B,So,0x1D02B,0x1D02B,0x1D02B},
{0x1D02C,So,0x1D02C,0x1D02C,0x1D02C},
{0x1D02D,So,0x1D02D,0x1D02D,0x1D02D},
{0x1D02E,So,0x1D02E,0x1D02E,0x1D02E},
{0x1D02F,So,0x1D02F,0x1D02F,0x1D02F},
{0x1D030,So,0x1D030,0x1D030,0x1D030},
{0x1D031,So,0x1D031,0x1D031,0x1D031},
{0x1D032,So,0x1D032,0x1D032,0x1D032},
{0x1D033,So,0x1D033,0x1D033,0x1D033},
{0x1D034,So,0x1D034,0x1D034,0x1D034},
{0x1D035,So,0x1D035,0x1D035,0x1D035},
{0x1D036,So,0x1D036,0x1D036,0x1D036},
{0x1D037,So,0x1D037,0x1D037,0x1D037},
{0x1D038,So,0x1D038,0x1D038,0x1D038},
{0x1D039,So,0x1D039,0x1D039,0x1D039},
{0x1D03A,So,0x1D03A,0x1D03A,0x1D03A},
{0x1D03B,So,0x1D03B,0x1D03B,0x1D03B},
{0x1D03C,So,0x1D03C,0x1D03C,0x1D03C},
{0x1D03D,So,0x1D03D,0x1D03D,0x1D03D},
{0x1D03E,So,0x1D03E,0x1D03E,0x1D03E},
{0x1D03F,So,0x1D03F,0x1D03F,0x1D03F},
{0x1D040,So,0x1D040,0x1D040,0x1D040},
{0x1D041,So,0x1D041,0x1D041,0x1D041},
{0x1D042,So,0x1D042,0x1D042,0x1D042},
{0x1D043,So,0x1D043,0x1D043,0x1D043},
{0x1D044,So,0x1D044,0x1D044,0x1D044},
{0x1D045,So,0x1D045,0x1D045,0x1D045},
{0x1D046,So,0x1D046,0x1D046,0x1D046},
{0x1D047,So,0x1D047,0x1D047,0x1D047},
{0x1D048,So,0x1D048,0x1D048,0x1D048},
{0x1D049,So,0x1D049,0x1D049,0x1D049},
{0x1D04A,So,0x1D04A,0x1D04A,0x1D04A},
{0x1D04B,So,0x1D04B,0x1D04B,0x1D04B},
{0x1D04C,So,0x1D04C,0x1D04C,0x1D04C},
{0x1D04D,So,0x1D04D,0x1D04D,0x1D04D},
{0x1D04E,So,0x1D04E,0x1D04E,0x1D04E},
{0x1D04F,So,0x1D04F,0x1D04F,0x1D04F},
{0x1D050,So,0x1D050,0x1D050,0x1D050},
{0x1D051,So,0x1D051,0x1D051,0x1D051},
{0x1D052,So,0x1D052,0x1D052,0x1D052},
{0x1D053,So,0x1D053,0x1D053,0x1D053},
{0x1D054,So,0x1D054,0x1D054,0x1D054},
{0x1D055,So,0x1D055,0x1D055,0x1D055},
{0x1D056,So,0x1D056,0x1D056,0x1D056},
{0x1D057,So,0x1D057,0x1D057,0x1D057},
{0x1D058,So,0x1D058,0x1D058,0x1D058},
{0x1D059,So,0x1D059,0x1D059,0x1D059},
{0x1D05A,So,0x1D05A,0x1D05A,0x1D05A},
{0x1D05B,So,0x1D05B,0x1D05B,0x1D05B},
{0x1D05C,So,0x1D05C,0x1D05C,0x1D05C},
{0x1D05D,So,0x1D05D,0x1D05D,0x1D05D},
{0x1D05E,So,0x1D05E,0x1D05E,0x1D05E},
{0x1D05F,So,0x1D05F,0x1D05F,0x1D05F},
{0x1D060,So,0x1D060,0x1D060,0x1D060},
{0x1D061,So,0x1D061,0x1D061,0x1D061},
{0x1D062,So,0x1D062,0x1D062,0x1D062},
{0x1D063,So,0x1D063,0x1D063,0x1D063},
{0x1D064,So,0x1D064,0x1D064,0x1D064},
{0x1D065,So,0x1D065,0x1D065,0x1D065},
{0x1D066,So,0x1D066,0x1D066,0x1D066},
{0x1D067,So,0x1D067,0x1D067,0x1D067},
{0x1D068,So,0x1D068,0x1D068,0x1D068},
{0x1D069,So,0x1D069,0x1D069,0x1D069},
{0x1D06A,So,0x1D06A,0x1D06A,0x1D06A},
{0x1D06B,So,0x1D06B,0x1D06B,0x1D06B},
{0x1D06C,So,0x1D06C,0x1D06C,0x1D06C},
{0x1D06D,So,0x1D06D,0x1D06D,0x1D06D},
{0x1D06E,So,0x1D06E,0x1D06E,0x1D06E},
{0x1D06F,So,0x1D06F,0x1D06F,0x1D06F},
{0x1D070,So,0x1D070,0x1D070,0x1D070},
{0x1D071,So,0x1D071,0x1D071,0x1D071},
{0x1D072,So,0x1D072,0x1D072,0x1D072},
{0x1D073,So,0x1D073,0x1D073,0x1D073},
{0x1D074,So,0x1D074,0x1D074,0x1D074},
{0x1D075,So,0x1D075,0x1D075,0x1D075},
{0x1D076,So,0x1D076,0x1D076,0x1D076},
{0x1D077,So,0x1D077,0x1D077,0x1D077},
{0x1D078,So,0x1D078,0x1D078,0x1D078},
{0x1D079,So,0x1D079,0x1D079,0x1D079},
{0x1D07A,So,0x1D07A,0x1D07A,0x1D07A},
{0x1D07B,So,0x1D07B,0x1D07B,0x1D07B},
{0x1D07C,So,0x1D07C,0x1D07C,0x1D07C},
{0x1D07D,So,0x1D07D,0x1D07D,0x1D07D},
{0x1D07E,So,0x1D07E,0x1D07E,0x1D07E},
{0x1D07F,So,0x1D07F,0x1D07F,0x1D07F},
{0x1D080,So,0x1D080,0x1D080,0x1D080},
{0x1D081,So,0x1D081,0x1D081,0x1D081},
{0x1D082,So,0x1D082,0x1D082,0x1D082},
{0x1D083,So,0x1D083,0x1D083,0x1D083},
{0x1D084,So,0x1D084,0x1D084,0x1D084},
{0x1D085,So,0x1D085,0x1D085,0x1D085},
{0x1D086,So,0x1D086,0x1D086,0x1D086},
{0x1D087,So,0x1D087,0x1D087,0x1D087},
{0x1D088,So,0x1D088,0x1D088,0x1D088},
{0x1D089,So,0x1D089,0x1D089,0x1D089},
{0x1D08A,So,0x1D08A,0x1D08A,0x1D08A},
{0x1D08B,So,0x1D08B,0x1D08B,0x1D08B},
{0x1D08C,So,0x1D08C,0x1D08C,0x1D08C},
{0x1D08D,So,0x1D08D,0x1D08D,0x1D08D},
{0x1D08E,So,0x1D08E,0x1D08E,0x1D08E},
{0x1D08F,So,0x1D08F,0x1D08F,0x1D08F},
{0x1D090,So,0x1D090,0x1D090,0x1D090},
{0x1D091,So,0x1D091,0x1D091,0x1D091},
{0x1D092,So,0x1D092,0x1D092,0x1D092},
{0x1D093,So,0x1D093,0x1D093,0x1D093},
{0x1D094,So,0x1D094,0x1D094,0x1D094},
{0x1D095,So,0x1D095,0x1D095,0x1D095},
{0x1D096,So,0x1D096,0x1D096,0x1D096},
{0x1D097,So,0x1D097,0x1D097,0x1D097},
{0x1D098,So,0x1D098,0x1D098,0x1D098},
{0x1D099,So,0x1D099,0x1D099,0x1D099},
{0x1D09A,So,0x1D09A,0x1D09A,0x1D09A},
{0x1D09B,So,0x1D09B,0x1D09B,0x1D09B},
{0x1D09C,So,0x1D09C,0x1D09C,0x1D09C},
{0x1D09D,So,0x1D09D,0x1D09D,0x1D09D},
{0x1D09E,So,0x1D09E,0x1D09E,0x1D09E},
{0x1D09F,So,0x1D09F,0x1D09F,0x1D09F},
{0x1D0A0,So,0x1D0A0,0x1D0A0,0x1D0A0},
{0x1D0A1,So,0x1D0A1,0x1D0A1,0x1D0A1},
{0x1D0A2,So,0x1D0A2,0x1D0A2,0x1D0A2},
{0x1D0A3,So,0x1D0A3,0x1D0A3,0x1D0A3},
{0x1D0A4,So,0x1D0A4,0x1D0A4,0x1D0A4},
{0x1D0A5,So,0x1D0A5,0x1D0A5,0x1D0A5},
{0x1D0A6,So,0x1D0A6,0x1D0A6,0x1D0A6},
{0x1D0A7,So,0x1D0A7,0x1D0A7,0x1D0A7},
{0x1D0A8,So,0x1D0A8,0x1D0A8,0x1D0A8},
{0x1D0A9,So,0x1D0A9,0x1D0A9,0x1D0A9},
{0x1D0AA,So,0x1D0AA,0x1D0AA,0x1D0AA},
{0x1D0AB,So,0x1D0AB,0x1D0AB,0x1D0AB},
{0x1D0AC,So,0x1D0AC,0x1D0AC,0x1D0AC},
{0x1D0AD,So,0x1D0AD,0x1D0AD,0x1D0AD},
{0x1D0AE,So,0x1D0AE,0x1D0AE,0x1D0AE},
{0x1D0AF,So,0x1D0AF,0x1D0AF,0x1D0AF},
{0x1D0B0,So,0x1D0B0,0x1D0B0,0x1D0B0},
{0x1D0B1,So,0x1D0B1,0x1D0B1,0x1D0B1},
{0x1D0B2,So,0x1D0B2,0x1D0B2,0x1D0B2},
{0x1D0B3,So,0x1D0B3,0x1D0B3,0x1D0B3},
{0x1D0B4,So,0x1D0B4,0x1D0B4,0x1D0B4},
{0x1D0B5,So,0x1D0B5,0x1D0B5,0x1D0B5},
{0x1D0B6,So,0x1D0B6,0x1D0B6,0x1D0B6},
{0x1D0B7,So,0x1D0B7,0x1D0B7,0x1D0B7},
{0x1D0B8,So,0x1D0B8,0x1D0B8,0x1D0B8},
{0x1D0B9,So,0x1D0B9,0x1D0B9,0x1D0B9},
{0x1D0BA,So,0x1D0BA,0x1D0BA,0x1D0BA},
{0x1D0BB,So,0x1D0BB,0x1D0BB,0x1D0BB},
{0x1D0BC,So,0x1D0BC,0x1D0BC,0x1D0BC},
{0x1D0BD,So,0x1D0BD,0x1D0BD,0x1D0BD},
{0x1D0BE,So,0x1D0BE,0x1D0BE,0x1D0BE},
{0x1D0BF,So,0x1D0BF,0x1D0BF,0x1D0BF},
{0x1D0C0,So,0x1D0C0,0x1D0C0,0x1D0C0},
{0x1D0C1,So,0x1D0C1,0x1D0C1,0x1D0C1},
{0x1D0C2,So,0x1D0C2,0x1D0C2,0x1D0C2},
{0x1D0C3,So,0x1D0C3,0x1D0C3,0x1D0C3},
{0x1D0C4,So,0x1D0C4,0x1D0C4,0x1D0C4},
{0x1D0C5,So,0x1D0C5,0x1D0C5,0x1D0C5},
{0x1D0C6,So,0x1D0C6,0x1D0C6,0x1D0C6},
{0x1D0C7,So,0x1D0C7,0x1D0C7,0x1D0C7},
{0x1D0C8,So,0x1D0C8,0x1D0C8,0x1D0C8},
{0x1D0C9,So,0x1D0C9,0x1D0C9,0x1D0C9},
{0x1D0CA,So,0x1D0CA,0x1D0CA,0x1D0CA},
{0x1D0CB,So,0x1D0CB,0x1D0CB,0x1D0CB},
{0x1D0CC,So,0x1D0CC,0x1D0CC,0x1D0CC},
{0x1D0CD,So,0x1D0CD,0x1D0CD,0x1D0CD},
{0x1D0CE,So,0x1D0CE,0x1D0CE,0x1D0CE},
{0x1D0CF,So,0x1D0CF,0x1D0CF,0x1D0CF},
{0x1D0D0,So,0x1D0D0,0x1D0D0,0x1D0D0},
{0x1D0D1,So,0x1D0D1,0x1D0D1,0x1D0D1},
{0x1D0D2,So,0x1D0D2,0x1D0D2,0x1D0D2},
{0x1D0D3,So,0x1D0D3,0x1D0D3,0x1D0D3},
{0x1D0D4,So,0x1D0D4,0x1D0D4,0x1D0D4},
{0x1D0D5,So,0x1D0D5,0x1D0D5,0x1D0D5},
{0x1D0D6,So,0x1D0D6,0x1D0D6,0x1D0D6},
{0x1D0D7,So,0x1D0D7,0x1D0D7,0x1D0D7},
{0x1D0D8,So,0x1D0D8,0x1D0D8,0x1D0D8},
{0x1D0D9,So,0x1D0D9,0x1D0D9,0x1D0D9},
{0x1D0DA,So,0x1D0DA,0x1D0DA,0x1D0DA},
{0x1D0DB,So,0x1D0DB,0x1D0DB,0x1D0DB},
{0x1D0DC,So,0x1D0DC,0x1D0DC,0x1D0DC},
{0x1D0DD,So,0x1D0DD,0x1D0DD,0x1D0DD},
{0x1D0DE,So,0x1D0DE,0x1D0DE,0x1D0DE},
{0x1D0DF,So,0x1D0DF,0x1D0DF,0x1D0DF},
{0x1D0E0,So,0x1D0E0,0x1D0E0,0x1D0E0},
{0x1D0E1,So,0x1D0E1,0x1D0E1,0x1D0E1},
{0x1D0E2,So,0x1D0E2,0x1D0E2,0x1D0E2},
{0x1D0E3,So,0x1D0E3,0x1D0E3,0x1D0E3},
{0x1D0E4,So,0x1D0E4,0x1D0E4,0x1D0E4},
{0x1D0E5,So,0x1D0E5,0x1D0E5,0x1D0E5},
{0x1D0E6,So,0x1D0E6,0x1D0E6,0x1D0E6},
{0x1D0E7,So,0x1D0E7,0x1D0E7,0x1D0E7},
{0x1D0E8,So,0x1D0E8,0x1D0E8,0x1D0E8},
{0x1D0E9,So,0x1D0E9,0x1D0E9,0x1D0E9},
{0x1D0EA,So,0x1D0EA,0x1D0EA,0x1D0EA},
{0x1D0EB,So,0x1D0EB,0x1D0EB,0x1D0EB},
{0x1D0EC,So,0x1D0EC,0x1D0EC,0x1D0EC},
{0x1D0ED,So,0x1D0ED,0x1D0ED,0x1D0ED},
{0x1D0EE,So,0x1D0EE,0x1D0EE,0x1D0EE},
{0x1D0EF,So,0x1D0EF,0x1D0EF,0x1D0EF},
{0x1D0F0,So,0x1D0F0,0x1D0F0,0x1D0F0},
{0x1D0F1,So,0x1D0F1,0x1D0F1,0x1D0F1},
{0x1D0F2,So,0x1D0F2,0x1D0F2,0x1D0F2},
{0x1D0F3,So,0x1D0F3,0x1D0F3,0x1D0F3},
{0x1D0F4,So,0x1D0F4,0x1D0F4,0x1D0F4},
{0x1D0F5,So,0x1D0F5,0x1D0F5,0x1D0F5},
{0x1D100,So,0x1D100,0x1D100,0x1D100},
{0x1D101,So,0x1D101,0x1D101,0x1D101},
{0x1D102,So,0x1D102,0x1D102,0x1D102},
{0x1D103,So,0x1D103,0x1D103,0x1D103},
{0x1D104,So,0x1D104,0x1D104,0x1D104},
{0x1D105,So,0x1D105,0x1D105,0x1D105},
{0x1D106,So,0x1D106,0x1D106,0x1D106},
{0x1D107,So,0x1D107,0x1D107,0x1D107},
{0x1D108,So,0x1D108,0x1D108,0x1D108},
{0x1D109,So,0x1D109,0x1D109,0x1D109},
{0x1D10A,So,0x1D10A,0x1D10A,0x1D10A},
{0x1D10B,So,0x1D10B,0x1D10B,0x1D10B},
{0x1D10C,So,0x1D10C,0x1D10C,0x1D10C},
{0x1D10D,So,0x1D10D,0x1D10D,0x1D10D},
{0x1D10E,So,0x1D10E,0x1D10E,0x1D10E},
{0x1D10F,So,0x1D10F,0x1D10F,0x1D10F},
{0x1D110,So,0x1D110,0x1D110,0x1D110},
{0x1D111,So,0x1D111,0x1D111,0x1D111},
{0x1D112,So,0x1D112,0x1D112,0x1D112},
{0x1D113,So,0x1D113,0x1D113,0x1D113},
{0x1D114,So,0x1D114,0x1D114,0x1D114},
{0x1D115,So,0x1D115,0x1D115,0x1D115},
{0x1D116,So,0x1D116,0x1D116,0x1D116},
{0x1D117,So,0x1D117,0x1D117,0x1D117},
{0x1D118,So,0x1D118,0x1D118,0x1D118},
{0x1D119,So,0x1D119,0x1D119,0x1D119},
{0x1D11A,So,0x1D11A,0x1D11A,0x1D11A},
{0x1D11B,So,0x1D11B,0x1D11B,0x1D11B},
{0x1D11C,So,0x1D11C,0x1D11C,0x1D11C},
{0x1D11D,So,0x1D11D,0x1D11D,0x1D11D},
{0x1D11E,So,0x1D11E,0x1D11E,0x1D11E},
{0x1D11F,So,0x1D11F,0x1D11F,0x1D11F},
{0x1D120,So,0x1D120,0x1D120,0x1D120},
{0x1D121,So,0x1D121,0x1D121,0x1D121},
{0x1D122,So,0x1D122,0x1D122,0x1D122},
{0x1D123,So,0x1D123,0x1D123,0x1D123},
{0x1D124,So,0x1D124,0x1D124,0x1D124},
{0x1D125,So,0x1D125,0x1D125,0x1D125},
{0x1D126,So,0x1D126,0x1D126,0x1D126},
{0x1D12A,So,0x1D12A,0x1D12A,0x1D12A},
{0x1D12B,So,0x1D12B,0x1D12B,0x1D12B},
{0x1D12C,So,0x1D12C,0x1D12C,0x1D12C},
{0x1D12D,So,0x1D12D,0x1D12D,0x1D12D},
{0x1D12E,So,0x1D12E,0x1D12E,0x1D12E},
{0x1D12F,So,0x1D12F,0x1D12F,0x1D12F},
{0x1D130,So,0x1D130,0x1D130,0x1D130},
{0x1D131,So,0x1D131,0x1D131,0x1D131},
{0x1D132,So,0x1D132,0x1D132,0x1D132},
{0x1D133,So,0x1D133,0x1D133,0x1D133},
{0x1D134,So,0x1D134,0x1D134,0x1D134},
{0x1D135,So,0x1D135,0x1D135,0x1D135},
{0x1D136,So,0x1D136,0x1D136,0x1D136},
{0x1D137,So,0x1D137,0x1D137,0x1D137},
{0x1D138,So,0x1D138,0x1D138,0x1D138},
{0x1D139,So,0x1D139,0x1D139,0x1D139},
{0x1D13A,So,0x1D13A,0x1D13A,0x1D13A},
{0x1D13B,So,0x1D13B,0x1D13B,0x1D13B},
{0x1D13C,So,0x1D13C,0x1D13C,0x1D13C},
{0x1D13D,So,0x1D13D,0x1D13D,0x1D13D},
{0x1D13E,So,0x1D13E,0x1D13E,0x1D13E},
{0x1D13F,So,0x1D13F,0x1D13F,0x1D13F},
{0x1D140,So,0x1D140,0x1D140,0x1D140},
{0x1D141,So,0x1D141,0x1D141,0x1D141},
{0x1D142,So,0x1D142,0x1D142,0x1D142},
{0x1D143,So,0x1D143,0x1D143,0x1D143},
{0x1D144,So,0x1D144,0x1D144,0x1D144},
{0x1D145,So,0x1D145,0x1D145,0x1D145},
{0x1D146,So,0x1D146,0x1D146,0x1D146},
{0x1D147,So,0x1D147,0x1D147,0x1D147},
{0x1D148,So,0x1D148,0x1D148,0x1D148},
{0x1D149,So,0x1D149,0x1D149,0x1D149},
{0x1D14A,So,0x1D14A,0x1D14A,0x1D14A},
{0x1D14B,So,0x1D14B,0x1D14B,0x1D14B},
{0x1D14C,So,0x1D14C,0x1D14C,0x1D14C},
{0x1D14D,So,0x1D14D,0x1D14D,0x1D14D},
{0x1D14E,So,0x1D14E,0x1D14E,0x1D14E},
{0x1D14F,So,0x1D14F,0x1D14F,0x1D14F},
{0x1D150,So,0x1D150,0x1D150,0x1D150},
{0x1D151,So,0x1D151,0x1D151,0x1D151},
{0x1D152,So,0x1D152,0x1D152,0x1D152},
{0x1D153,So,0x1D153,0x1D153,0x1D153},
{0x1D154,So,0x1D154,0x1D154,0x1D154},
{0x1D155,So,0x1D155,0x1D155,0x1D155},
{0x1D156,So,0x1D156,0x1D156,0x1D156},
{0x1D157,So,0x1D157,0x1D157,0x1D157},
{0x1D158,So,0x1D158,0x1D158,0x1D158},
{0x1D159,So,0x1D159,0x1D159,0x1D159},
{0x1D15A,So,0x1D15A,0x1D15A,0x1D15A},
{0x1D15B,So,0x1D15B,0x1D15B,0x1D15B},
{0x1D15C,So,0x1D15C,0x1D15C,0x1D15C},
{0x1D15D,So,0x1D15D,0x1D15D,0x1D15D},
{0x1D15E,So,0x1D15E,0x1D15E,0x1D15E},
{0x1D15F,So,0x1D15F,0x1D15F,0x1D15F},
{0x1D160,So,0x1D160,0x1D160,0x1D160},
{0x1D161,So,0x1D161,0x1D161,0x1D161},
{0x1D162,So,0x1D162,0x1D162,0x1D162},
{0x1D163,So,0x1D163,0x1D163,0x1D163},
{0x1D164,So,0x1D164,0x1D164,0x1D164},
{0x1D165,Mc,0x1D165,0x1D165,0x1D165},
{0x1D166,Mc,0x1D166,0x1D166,0x1D166},
{0x1D167,Mn,0x1D167,0x1D167,0x1D167},
{0x1D168,Mn,0x1D168,0x1D168,0x1D168},
{0x1D169,Mn,0x1D169,0x1D169,0x1D169},
{0x1D16A,So,0x1D16A,0x1D16A,0x1D16A},
{0x1D16B,So,0x1D16B,0x1D16B,0x1D16B},
{0x1D16C,So,0x1D16C,0x1D16C,0x1D16C},
{0x1D16D,Mc,0x1D16D,0x1D16D,0x1D16D},
{0x1D16E,Mc,0x1D16E,0x1D16E,0x1D16E},
{0x1D16F,Mc,0x1D16F,0x1D16F,0x1D16F},
{0x1D170,Mc,0x1D170,0x1D170,0x1D170},
{0x1D171,Mc,0x1D171,0x1D171,0x1D171},
{0x1D172,Mc,0x1D172,0x1D172,0x1D172},
{0x1D173,Cf,0x1D173,0x1D173,0x1D173},
{0x1D174,Cf,0x1D174,0x1D174,0x1D174},
{0x1D175,Cf,0x1D175,0x1D175,0x1D175},
{0x1D176,Cf,0x1D176,0x1D176,0x1D176},
{0x1D177,Cf,0x1D177,0x1D177,0x1D177},
{0x1D178,Cf,0x1D178,0x1D178,0x1D178},
{0x1D179,Cf,0x1D179,0x1D179,0x1D179},
{0x1D17A,Cf,0x1D17A,0x1D17A,0x1D17A},
{0x1D17B,Mn,0x1D17B,0x1D17B,0x1D17B},
{0x1D17C,Mn,0x1D17C,0x1D17C,0x1D17C},
{0x1D17D,Mn,0x1D17D,0x1D17D,0x1D17D},
{0x1D17E,Mn,0x1D17E,0x1D17E,0x1D17E},
{0x1D17F,Mn,0x1D17F,0x1D17F,0x1D17F},
{0x1D180,Mn,0x1D180,0x1D180,0x1D180},
{0x1D181,Mn,0x1D181,0x1D181,0x1D181},
{0x1D182,Mn,0x1D182,0x1D182,0x1D182},
{0x1D183,So,0x1D183,0x1D183,0x1D183},
{0x1D184,So,0x1D184,0x1D184,0x1D184},
{0x1D185,Mn,0x1D185,0x1D185,0x1D185},
{0x1D186,Mn,0x1D186,0x1D186,0x1D186},
{0x1D187,Mn,0x1D187,0x1D187,0x1D187},
{0x1D188,Mn,0x1D188,0x1D188,0x1D188},
{0x1D189,Mn,0x1D189,0x1D189,0x1D189},
{0x1D18A,Mn,0x1D18A,0x1D18A,0x1D18A},
{0x1D18B,Mn,0x1D18B,0x1D18B,0x1D18B},
{0x1D18C,So,0x1D18C,0x1D18C,0x1D18C},
{0x1D18D,So,0x1D18D,0x1D18D,0x1D18D},
{0x1D18E,So,0x1D18E,0x1D18E,0x1D18E},
{0x1D18F,So,0x1D18F,0x1D18F,0x1D18F},
{0x1D190,So,0x1D190,0x1D190,0x1D190},
{0x1D191,So,0x1D191,0x1D191,0x1D191},
{0x1D192,So,0x1D192,0x1D192,0x1D192},
{0x1D193,So,0x1D193,0x1D193,0x1D193},
{0x1D194,So,0x1D194,0x1D194,0x1D194},
{0x1D195,So,0x1D195,0x1D195,0x1D195},
{0x1D196,So,0x1D196,0x1D196,0x1D196},
{0x1D197,So,0x1D197,0x1D197,0x1D197},
{0x1D198,So,0x1D198,0x1D198,0x1D198},
{0x1D199,So,0x1D199,0x1D199,0x1D199},
{0x1D19A,So,0x1D19A,0x1D19A,0x1D19A},
{0x1D19B,So,0x1D19B,0x1D19B,0x1D19B},
{0x1D19C,So,0x1D19C,0x1D19C,0x1D19C},
{0x1D19D,So,0x1D19D,0x1D19D,0x1D19D},
{0x1D19E,So,0x1D19E,0x1D19E,0x1D19E},
{0x1D19F,So,0x1D19F,0x1D19F,0x1D19F},
{0x1D1A0,So,0x1D1A0,0x1D1A0,0x1D1A0},
{0x1D1A1,So,0x1D1A1,0x1D1A1,0x1D1A1},
{0x1D1A2,So,0x1D1A2,0x1D1A2,0x1D1A2},
{0x1D1A3,So,0x1D1A3,0x1D1A3,0x1D1A3},
{0x1D1A4,So,0x1D1A4,0x1D1A4,0x1D1A4},
{0x1D1A5,So,0x1D1A5,0x1D1A5,0x1D1A5},
{0x1D1A6,So,0x1D1A6,0x1D1A6,0x1D1A6},
{0x1D1A7,So,0x1D1A7,0x1D1A7,0x1D1A7},
{0x1D1A8,So,0x1D1A8,0x1D1A8,0x1D1A8},
{0x1D1A9,So,0x1D1A9,0x1D1A9,0x1D1A9},
{0x1D1AA,Mn,0x1D1AA,0x1D1AA,0x1D1AA},
{0x1D1AB,Mn,0x1D1AB,0x1D1AB,0x1D1AB},
{0x1D1AC,Mn,0x1D1AC,0x1D1AC,0x1D1AC},
{0x1D1AD,Mn,0x1D1AD,0x1D1AD,0x1D1AD},
{0x1D1AE,So,0x1D1AE,0x1D1AE,0x1D1AE},
{0x1D1AF,So,0x1D1AF,0x1D1AF,0x1D1AF},
{0x1D1B0,So,0x1D1B0,0x1D1B0,0x1D1B0},
{0x1D1B1,So,0x1D1B1,0x1D1B1,0x1D1B1},
{0x1D1B2,So,0x1D1B2,0x1D1B2,0x1D1B2},
{0x1D1B3,So,0x1D1B3,0x1D1B3,0x1D1B3},
{0x1D1B4,So,0x1D1B4,0x1D1B4,0x1D1B4},
{0x1D1B5,So,0x1D1B5,0x1D1B5,0x1D1B5},
{0x1D1B6,So,0x1D1B6,0x1D1B6,0x1D1B6},
{0x1D1B7,So,0x1D1B7,0x1D1B7,0x1D1B7},
{0x1D1B8,So,0x1D1B8,0x1D1B8,0x1D1B8},
{0x1D1B9,So,0x1D1B9,0x1D1B9,0x1D1B9},
{0x1D1BA,So,0x1D1BA,0x1D1BA,0x1D1BA},
{0x1D1BB,So,0x1D1BB,0x1D1BB,0x1D1BB},
{0x1D1BC,So,0x1D1BC,0x1D1BC,0x1D1BC},
{0x1D1BD,So,0x1D1BD,0x1D1BD,0x1D1BD},
{0x1D1BE,So,0x1D1BE,0x1D1BE,0x1D1BE},
{0x1D1BF,So,0x1D1BF,0x1D1BF,0x1D1BF},
{0x1D1C0,So,0x1D1C0,0x1D1C0,0x1D1C0},
{0x1D1C1,So,0x1D1C1,0x1D1C1,0x1D1C1},
{0x1D1C2,So,0x1D1C2,0x1D1C2,0x1D1C2},
{0x1D1C3,So,0x1D1C3,0x1D1C3,0x1D1C3},
{0x1D1C4,So,0x1D1C4,0x1D1C4,0x1D1C4},
{0x1D1C5,So,0x1D1C5,0x1D1C5,0x1D1C5},
{0x1D1C6,So,0x1D1C6,0x1D1C6,0x1D1C6},
{0x1D1C7,So,0x1D1C7,0x1D1C7,0x1D1C7},
{0x1D1C8,So,0x1D1C8,0x1D1C8,0x1D1C8},
{0x1D1C9,So,0x1D1C9,0x1D1C9,0x1D1C9},
{0x1D1CA,So,0x1D1CA,0x1D1CA,0x1D1CA},
{0x1D1CB,So,0x1D1CB,0x1D1CB,0x1D1CB},
{0x1D1CC,So,0x1D1CC,0x1D1CC,0x1D1CC},
{0x1D1CD,So,0x1D1CD,0x1D1CD,0x1D1CD},
{0x1D1CE,So,0x1D1CE,0x1D1CE,0x1D1CE},
{0x1D1CF,So,0x1D1CF,0x1D1CF,0x1D1CF},
{0x1D1D0,So,0x1D1D0,0x1D1D0,0x1D1D0},
{0x1D1D1,So,0x1D1D1,0x1D1D1,0x1D1D1},
{0x1D1D2,So,0x1D1D2,0x1D1D2,0x1D1D2},
{0x1D1D3,So,0x1D1D3,0x1D1D3,0x1D1D3},
{0x1D1D4,So,0x1D1D4,0x1D1D4,0x1D1D4},
{0x1D1D5,So,0x1D1D5,0x1D1D5,0x1D1D5},
{0x1D1D6,So,0x1D1D6,0x1D1D6,0x1D1D6},
{0x1D1D7,So,0x1D1D7,0x1D1D7,0x1D1D7},
{0x1D1D8,So,0x1D1D8,0x1D1D8,0x1D1D8},
{0x1D1D9,So,0x1D1D9,0x1D1D9,0x1D1D9},
{0x1D1DA,So,0x1D1DA,0x1D1DA,0x1D1DA},
{0x1D1DB,So,0x1D1DB,0x1D1DB,0x1D1DB},
{0x1D1DC,So,0x1D1DC,0x1D1DC,0x1D1DC},
{0x1D1DD,So,0x1D1DD,0x1D1DD,0x1D1DD},
{0x1D200,So,0x1D200,0x1D200,0x1D200},
{0x1D201,So,0x1D201,0x1D201,0x1D201},
{0x1D202,So,0x1D202,0x1D202,0x1D202},
{0x1D203,So,0x1D203,0x1D203,0x1D203},
{0x1D204,So,0x1D204,0x1D204,0x1D204},
{0x1D205,So,0x1D205,0x1D205,0x1D205},
{0x1D206,So,0x1D206,0x1D206,0x1D206},
{0x1D207,So,0x1D207,0x1D207,0x1D207},
{0x1D208,So,0x1D208,0x1D208,0x1D208},
{0x1D209,So,0x1D209,0x1D209,0x1D209},
{0x1D20A,So,0x1D20A,0x1D20A,0x1D20A},
{0x1D20B,So,0x1D20B,0x1D20B,0x1D20B},
{0x1D20C,So,0x1D20C,0x1D20C,0x1D20C},
{0x1D20D,So,0x1D20D,0x1D20D,0x1D20D},
{0x1D20E,So,0x1D20E,0x1D20E,0x1D20E},
{0x1D20F,So,0x1D20F,0x1D20F,0x1D20F},
{0x1D210,So,0x1D210,0x1D210,0x1D210},
{0x1D211,So,0x1D211,0x1D211,0x1D211},
{0x1D212,So,0x1D212,0x1D212,0x1D212},
{0x1D213,So,0x1D213,0x1D213,0x1D213},
{0x1D214,So,0x1D214,0x1D214,0x1D214},
{0x1D215,So,0x1D215,0x1D215,0x1D215},
{0x1D216,So,0x1D216,0x1D216,0x1D216},
{0x1D217,So,0x1D217,0x1D217,0x1D217},
{0x1D218,So,0x1D218,0x1D218,0x1D218},
{0x1D219,So,0x1D219,0x1D219,0x1D219},
{0x1D21A,So,0x1D21A,0x1D21A,0x1D21A},
{0x1D21B,So,0x1D21B,0x1D21B,0x1D21B},
{0x1D21C,So,0x1D21C,0x1D21C,0x1D21C},
{0x1D21D,So,0x1D21D,0x1D21D,0x1D21D},
{0x1D21E,So,0x1D21E,0x1D21E,0x1D21E},
{0x1D21F,So,0x1D21F,0x1D21F,0x1D21F},
{0x1D220,So,0x1D220,0x1D220,0x1D220},
{0x1D221,So,0x1D221,0x1D221,0x1D221},
{0x1D222,So,0x1D222,0x1D222,0x1D222},
{0x1D223,So,0x1D223,0x1D223,0x1D223},
{0x1D224,So,0x1D224,0x1D224,0x1D224},
{0x1D225,So,0x1D225,0x1D225,0x1D225},
{0x1D226,So,0x1D226,0x1D226,0x1D226},
{0x1D227,So,0x1D227,0x1D227,0x1D227},
{0x1D228,So,0x1D228,0x1D228,0x1D228},
{0x1D229,So,0x1D229,0x1D229,0x1D229},
{0x1D22A,So,0x1D22A,0x1D22A,0x1D22A},
{0x1D22B,So,0x1D22B,0x1D22B,0x1D22B},
{0x1D22C,So,0x1D22C,0x1D22C,0x1D22C},
{0x1D22D,So,0x1D22D,0x1D22D,0x1D22D},
{0x1D22E,So,0x1D22E,0x1D22E,0x1D22E},
{0x1D22F,So,0x1D22F,0x1D22F,0x1D22F},
{0x1D230,So,0x1D230,0x1D230,0x1D230},
{0x1D231,So,0x1D231,0x1D231,0x1D231},
{0x1D232,So,0x1D232,0x1D232,0x1D232},
{0x1D233,So,0x1D233,0x1D233,0x1D233},
{0x1D234,So,0x1D234,0x1D234,0x1D234},
{0x1D235,So,0x1D235,0x1D235,0x1D235},
{0x1D236,So,0x1D236,0x1D236,0x1D236},
{0x1D237,So,0x1D237,0x1D237,0x1D237},
{0x1D238,So,0x1D238,0x1D238,0x1D238},
{0x1D239,So,0x1D239,0x1D239,0x1D239},
{0x1D23A,So,0x1D23A,0x1D23A,0x1D23A},
{0x1D23B,So,0x1D23B,0x1D23B,0x1D23B},
{0x1D23C,So,0x1D23C,0x1D23C,0x1D23C},
{0x1D23D,So,0x1D23D,0x1D23D,0x1D23D},
{0x1D23E,So,0x1D23E,0x1D23E,0x1D23E},
{0x1D23F,So,0x1D23F,0x1D23F,0x1D23F},
{0x1D240,So,0x1D240,0x1D240,0x1D240},
{0x1D241,So,0x1D241,0x1D241,0x1D241},
{0x1D242,Mn,0x1D242,0x1D242,0x1D242},
{0x1D243,Mn,0x1D243,0x1D243,0x1D243},
{0x1D244,Mn,0x1D244,0x1D244,0x1D244},
{0x1D245,So,0x1D245,0x1D245,0x1D245},
{0x1D300,So,0x1D300,0x1D300,0x1D300},
{0x1D301,So,0x1D301,0x1D301,0x1D301},
{0x1D302,So,0x1D302,0x1D302,0x1D302},
{0x1D303,So,0x1D303,0x1D303,0x1D303},
{0x1D304,So,0x1D304,0x1D304,0x1D304},
{0x1D305,So,0x1D305,0x1D305,0x1D305},
{0x1D306,So,0x1D306,0x1D306,0x1D306},
{0x1D307,So,0x1D307,0x1D307,0x1D307},
{0x1D308,So,0x1D308,0x1D308,0x1D308},
{0x1D309,So,0x1D309,0x1D309,0x1D309},
{0x1D30A,So,0x1D30A,0x1D30A,0x1D30A},
{0x1D30B,So,0x1D30B,0x1D30B,0x1D30B},
{0x1D30C,So,0x1D30C,0x1D30C,0x1D30C},
{0x1D30D,So,0x1D30D,0x1D30D,0x1D30D},
{0x1D30E,So,0x1D30E,0x1D30E,0x1D30E},
{0x1D30F,So,0x1D30F,0x1D30F,0x1D30F},
{0x1D310,So,0x1D310,0x1D310,0x1D310},
{0x1D311,So,0x1D311,0x1D311,0x1D311},
{0x1D312,So,0x1D312,0x1D312,0x1D312},
{0x1D313,So,0x1D313,0x1D313,0x1D313},
{0x1D314,So,0x1D314,0x1D314,0x1D314},
{0x1D315,So,0x1D315,0x1D315,0x1D315},
{0x1D316,So,0x1D316,0x1D316,0x1D316},
{0x1D317,So,0x1D317,0x1D317,0x1D317},
{0x1D318,So,0x1D318,0x1D318,0x1D318},
{0x1D319,So,0x1D319,0x1D319,0x1D319},
{0x1D31A,So,0x1D31A,0x1D31A,0x1D31A},
{0x1D31B,So,0x1D31B,0x1D31B,0x1D31B},
{0x1D31C,So,0x1D31C,0x1D31C,0x1D31C},
{0x1D31D,So,0x1D31D,0x1D31D,0x1D31D},
{0x1D31E,So,0x1D31E,0x1D31E,0x1D31E},
{0x1D31F,So,0x1D31F,0x1D31F,0x1D31F},
{0x1D320,So,0x1D320,0x1D320,0x1D320},
{0x1D321,So,0x1D321,0x1D321,0x1D321},
{0x1D322,So,0x1D322,0x1D322,0x1D322},
{0x1D323,So,0x1D323,0x1D323,0x1D323},
{0x1D324,So,0x1D324,0x1D324,0x1D324},
{0x1D325,So,0x1D325,0x1D325,0x1D325},
{0x1D326,So,0x1D326,0x1D326,0x1D326},
{0x1D327,So,0x1D327,0x1D327,0x1D327},
{0x1D328,So,0x1D328,0x1D328,0x1D328},
{0x1D329,So,0x1D329,0x1D329,0x1D329},
{0x1D32A,So,0x1D32A,0x1D32A,0x1D32A},
{0x1D32B,So,0x1D32B,0x1D32B,0x1D32B},
{0x1D32C,So,0x1D32C,0x1D32C,0x1D32C},
{0x1D32D,So,0x1D32D,0x1D32D,0x1D32D},
{0x1D32E,So,0x1D32E,0x1D32E,0x1D32E},
{0x1D32F,So,0x1D32F,0x1D32F,0x1D32F},
{0x1D330,So,0x1D330,0x1D330,0x1D330},
{0x1D331,So,0x1D331,0x1D331,0x1D331},
{0x1D332,So,0x1D332,0x1D332,0x1D332},
{0x1D333,So,0x1D333,0x1D333,0x1D333},
{0x1D334,So,0x1D334,0x1D334,0x1D334},
{0x1D335,So,0x1D335,0x1D335,0x1D335},
{0x1D336,So,0x1D336,0x1D336,0x1D336},
{0x1D337,So,0x1D337,0x1D337,0x1D337},
{0x1D338,So,0x1D338,0x1D338,0x1D338},
{0x1D339,So,0x1D339,0x1D339,0x1D339},
{0x1D33A,So,0x1D33A,0x1D33A,0x1D33A},
{0x1D33B,So,0x1D33B,0x1D33B,0x1D33B},
{0x1D33C,So,0x1D33C,0x1D33C,0x1D33C},
{0x1D33D,So,0x1D33D,0x1D33D,0x1D33D},
{0x1D33E,So,0x1D33E,0x1D33E,0x1D33E},
{0x1D33F,So,0x1D33F,0x1D33F,0x1D33F},
{0x1D340,So,0x1D340,0x1D340,0x1D340},
{0x1D341,So,0x1D341,0x1D341,0x1D341},
{0x1D342,So,0x1D342,0x1D342,0x1D342},
{0x1D343,So,0x1D343,0x1D343,0x1D343},
{0x1D344,So,0x1D344,0x1D344,0x1D344},
{0x1D345,So,0x1D345,0x1D345,0x1D345},
{0x1D346,So,0x1D346,0x1D346,0x1D346},
{0x1D347,So,0x1D347,0x1D347,0x1D347},
{0x1D348,So,0x1D348,0x1D348,0x1D348},
{0x1D349,So,0x1D349,0x1D349,0x1D349},
{0x1D34A,So,0x1D34A,0x1D34A,0x1D34A},
{0x1D34B,So,0x1D34B,0x1D34B,0x1D34B},
{0x1D34C,So,0x1D34C,0x1D34C,0x1D34C},
{0x1D34D,So,0x1D34D,0x1D34D,0x1D34D},
{0x1D34E,So,0x1D34E,0x1D34E,0x1D34E},
{0x1D34F,So,0x1D34F,0x1D34F,0x1D34F},
{0x1D350,So,0x1D350,0x1D350,0x1D350},
{0x1D351,So,0x1D351,0x1D351,0x1D351},
{0x1D352,So,0x1D352,0x1D352,0x1D352},
{0x1D353,So,0x1D353,0x1D353,0x1D353},
{0x1D354,So,0x1D354,0x1D354,0x1D354},
{0x1D355,So,0x1D355,0x1D355,0x1D355},
{0x1D356,So,0x1D356,0x1D356,0x1D356},
{0x1D360,No,0x1D360,0x1D360,0x1D360},
{0x1D361,No,0x1D361,0x1D361,0x1D361},
{0x1D362,No,0x1D362,0x1D362,0x1D362},
{0x1D363,No,0x1D363,0x1D363,0x1D363},
{0x1D364,No,0x1D364,0x1D364,0x1D364},
{0x1D365,No,0x1D365,0x1D365,0x1D365},
{0x1D366,No,0x1D366,0x1D366,0x1D366},
{0x1D367,No,0x1D367,0x1D367,0x1D367},
{0x1D368,No,0x1D368,0x1D368,0x1D368},
{0x1D369,No,0x1D369,0x1D369,0x1D369},
{0x1D36A,No,0x1D36A,0x1D36A,0x1D36A},
{0x1D36B,No,0x1D36B,0x1D36B,0x1D36B},
{0x1D36C,No,0x1D36C,0x1D36C,0x1D36C},
{0x1D36D,No,0x1D36D,0x1D36D,0x1D36D},
{0x1D36E,No,0x1D36E,0x1D36E,0x1D36E},
{0x1D36F,No,0x1D36F,0x1D36F,0x1D36F},
{0x1D370,No,0x1D370,0x1D370,0x1D370},
{0x1D371,No,0x1D371,0x1D371,0x1D371},
{0x1D400,Lu,0x1D400,0x1D400,0x1D400},
{0x1D401,Lu,0x1D401,0x1D401,0x1D401},
{0x1D402,Lu,0x1D402,0x1D402,0x1D402},
{0x1D403,Lu,0x1D403,0x1D403,0x1D403},
{0x1D404,Lu,0x1D404,0x1D404,0x1D404},
{0x1D405,Lu,0x1D405,0x1D405,0x1D405},
{0x1D406,Lu,0x1D406,0x1D406,0x1D406},
{0x1D407,Lu,0x1D407,0x1D407,0x1D407},
{0x1D408,Lu,0x1D408,0x1D408,0x1D408},
{0x1D409,Lu,0x1D409,0x1D409,0x1D409},
{0x1D40A,Lu,0x1D40A,0x1D40A,0x1D40A},
{0x1D40B,Lu,0x1D40B,0x1D40B,0x1D40B},
{0x1D40C,Lu,0x1D40C,0x1D40C,0x1D40C},
{0x1D40D,Lu,0x1D40D,0x1D40D,0x1D40D},
{0x1D40E,Lu,0x1D40E,0x1D40E,0x1D40E},
{0x1D40F,Lu,0x1D40F,0x1D40F,0x1D40F},
{0x1D410,Lu,0x1D410,0x1D410,0x1D410},
{0x1D411,Lu,0x1D411,0x1D411,0x1D411},
{0x1D412,Lu,0x1D412,0x1D412,0x1D412},
{0x1D413,Lu,0x1D413,0x1D413,0x1D413},
{0x1D414,Lu,0x1D414,0x1D414,0x1D414},
{0x1D415,Lu,0x1D415,0x1D415,0x1D415},
{0x1D416,Lu,0x1D416,0x1D416,0x1D416},
{0x1D417,Lu,0x1D417,0x1D417,0x1D417},
{0x1D418,Lu,0x1D418,0x1D418,0x1D418},
{0x1D419,Lu,0x1D419,0x1D419,0x1D419},
{0x1D41A,Ll,0x1D41A,0x1D41A,0x1D41A},
{0x1D41B,Ll,0x1D41B,0x1D41B,0x1D41B},
{0x1D41C,Ll,0x1D41C,0x1D41C,0x1D41C},
{0x1D41D,Ll,0x1D41D,0x1D41D,0x1D41D},
{0x1D41E,Ll,0x1D41E,0x1D41E,0x1D41E},
{0x1D41F,Ll,0x1D41F,0x1D41F,0x1D41F},
{0x1D420,Ll,0x1D420,0x1D420,0x1D420},
{0x1D421,Ll,0x1D421,0x1D421,0x1D421},
{0x1D422,Ll,0x1D422,0x1D422,0x1D422},
{0x1D423,Ll,0x1D423,0x1D423,0x1D423},
{0x1D424,Ll,0x1D424,0x1D424,0x1D424},
{0x1D425,Ll,0x1D425,0x1D425,0x1D425},
{0x1D426,Ll,0x1D426,0x1D426,0x1D426},
{0x1D427,Ll,0x1D427,0x1D427,0x1D427},
{0x1D428,Ll,0x1D428,0x1D428,0x1D428},
{0x1D429,Ll,0x1D429,0x1D429,0x1D429},
{0x1D42A,Ll,0x1D42A,0x1D42A,0x1D42A},
{0x1D42B,Ll,0x1D42B,0x1D42B,0x1D42B},
{0x1D42C,Ll,0x1D42C,0x1D42C,0x1D42C},
{0x1D42D,Ll,0x1D42D,0x1D42D,0x1D42D},
{0x1D42E,Ll,0x1D42E,0x1D42E,0x1D42E},
{0x1D42F,Ll,0x1D42F,0x1D42F,0x1D42F},
{0x1D430,Ll,0x1D430,0x1D430,0x1D430},
{0x1D431,Ll,0x1D431,0x1D431,0x1D431},
{0x1D432,Ll,0x1D432,0x1D432,0x1D432},
{0x1D433,Ll,0x1D433,0x1D433,0x1D433},
{0x1D434,Lu,0x1D434,0x1D434,0x1D434},
{0x1D435,Lu,0x1D435,0x1D435,0x1D435},
{0x1D436,Lu,0x1D436,0x1D436,0x1D436},
{0x1D437,Lu,0x1D437,0x1D437,0x1D437},
{0x1D438,Lu,0x1D438,0x1D438,0x1D438},
{0x1D439,Lu,0x1D439,0x1D439,0x1D439},
{0x1D43A,Lu,0x1D43A,0x1D43A,0x1D43A},
{0x1D43B,Lu,0x1D43B,0x1D43B,0x1D43B},
{0x1D43C,Lu,0x1D43C,0x1D43C,0x1D43C},
{0x1D43D,Lu,0x1D43D,0x1D43D,0x1D43D},
{0x1D43E,Lu,0x1D43E,0x1D43E,0x1D43E},
{0x1D43F,Lu,0x1D43F,0x1D43F,0x1D43F},
{0x1D440,Lu,0x1D440,0x1D440,0x1D440},
{0x1D441,Lu,0x1D441,0x1D441,0x1D441},
{0x1D442,Lu,0x1D442,0x1D442,0x1D442},
{0x1D443,Lu,0x1D443,0x1D443,0x1D443},
{0x1D444,Lu,0x1D444,0x1D444,0x1D444},
{0x1D445,Lu,0x1D445,0x1D445,0x1D445},
{0x1D446,Lu,0x1D446,0x1D446,0x1D446},
{0x1D447,Lu,0x1D447,0x1D447,0x1D447},
{0x1D448,Lu,0x1D448,0x1D448,0x1D448},
{0x1D449,Lu,0x1D449,0x1D449,0x1D449},
{0x1D44A,Lu,0x1D44A,0x1D44A,0x1D44A},
{0x1D44B,Lu,0x1D44B,0x1D44B,0x1D44B},
{0x1D44C,Lu,0x1D44C,0x1D44C,0x1D44C},
{0x1D44D,Lu,0x1D44D,0x1D44D,0x1D44D},
{0x1D44E,Ll,0x1D44E,0x1D44E,0x1D44E},
{0x1D44F,Ll,0x1D44F,0x1D44F,0x1D44F},
{0x1D450,Ll,0x1D450,0x1D450,0x1D450},
{0x1D451,Ll,0x1D451,0x1D451,0x1D451},
{0x1D452,Ll,0x1D452,0x1D452,0x1D452},
{0x1D453,Ll,0x1D453,0x1D453,0x1D453},
{0x1D454,Ll,0x1D454,0x1D454,0x1D454},
{0x1D456,Ll,0x1D456,0x1D456,0x1D456},
{0x1D457,Ll,0x1D457,0x1D457,0x1D457},
{0x1D458,Ll,0x1D458,0x1D458,0x1D458},
{0x1D459,Ll,0x1D459,0x1D459,0x1D459},
{0x1D45A,Ll,0x1D45A,0x1D45A,0x1D45A},
{0x1D45B,Ll,0x1D45B,0x1D45B,0x1D45B},
{0x1D45C,Ll,0x1D45C,0x1D45C,0x1D45C},
{0x1D45D,Ll,0x1D45D,0x1D45D,0x1D45D},
{0x1D45E,Ll,0x1D45E,0x1D45E,0x1D45E},
{0x1D45F,Ll,0x1D45F,0x1D45F,0x1D45F},
{0x1D460,Ll,0x1D460,0x1D460,0x1D460},
{0x1D461,Ll,0x1D461,0x1D461,0x1D461},
{0x1D462,Ll,0x1D462,0x1D462,0x1D462},
{0x1D463,Ll,0x1D463,0x1D463,0x1D463},
{0x1D464,Ll,0x1D464,0x1D464,0x1D464},
{0x1D465,Ll,0x1D465,0x1D465,0x1D465},
{0x1D466,Ll,0x1D466,0x1D466,0x1D466},
{0x1D467,Ll,0x1D467,0x1D467,0x1D467},
{0x1D468,Lu,0x1D468,0x1D468,0x1D468},
{0x1D469,Lu,0x1D469,0x1D469,0x1D469},
{0x1D46A,Lu,0x1D46A,0x1D46A,0x1D46A},
{0x1D46B,Lu,0x1D46B,0x1D46B,0x1D46B},
{0x1D46C,Lu,0x1D46C,0x1D46C,0x1D46C},
{0x1D46D,Lu,0x1D46D,0x1D46D,0x1D46D},
{0x1D46E,Lu,0x1D46E,0x1D46E,0x1D46E},
{0x1D46F,Lu,0x1D46F,0x1D46F,0x1D46F},
{0x1D470,Lu,0x1D470,0x1D470,0x1D470},
{0x1D471,Lu,0x1D471,0x1D471,0x1D471},
{0x1D472,Lu,0x1D472,0x1D472,0x1D472},
{0x1D473,Lu,0x1D473,0x1D473,0x1D473},
{0x1D474,Lu,0x1D474,0x1D474,0x1D474},
{0x1D475,Lu,0x1D475,0x1D475,0x1D475},
{0x1D476,Lu,0x1D476,0x1D476,0x1D476},
{0x1D477,Lu,0x1D477,0x1D477,0x1D477},
{0x1D478,Lu,0x1D478,0x1D478,0x1D478},
{0x1D479,Lu,0x1D479,0x1D479,0x1D479},
{0x1D47A,Lu,0x1D47A,0x1D47A,0x1D47A},
{0x1D47B,Lu,0x1D47B,0x1D47B,0x1D47B},
{0x1D47C,Lu,0x1D47C,0x1D47C,0x1D47C},
{0x1D47D,Lu,0x1D47D,0x1D47D,0x1D47D},
{0x1D47E,Lu,0x1D47E,0x1D47E,0x1D47E},
{0x1D47F,Lu,0x1D47F,0x1D47F,0x1D47F},
{0x1D480,Lu,0x1D480,0x1D480,0x1D480},
{0x1D481,Lu,0x1D481,0x1D481,0x1D481},
{0x1D482,Ll,0x1D482,0x1D482,0x1D482},
{0x1D483,Ll,0x1D483,0x1D483,0x1D483},
{0x1D484,Ll,0x1D484,0x1D484,0x1D484},
{0x1D485,Ll,0x1D485,0x1D485,0x1D485},
{0x1D486,Ll,0x1D486,0x1D486,0x1D486},
{0x1D487,Ll,0x1D487,0x1D487,0x1D487},
{0x1D488,Ll,0x1D488,0x1D488,0x1D488},
{0x1D489,Ll,0x1D489,0x1D489,0x1D489},
{0x1D48A,Ll,0x1D48A,0x1D48A,0x1D48A},
{0x1D48B,Ll,0x1D48B,0x1D48B,0x1D48B},
{0x1D48C,Ll,0x1D48C,0x1D48C,0x1D48C},
{0x1D48D,Ll,0x1D48D,0x1D48D,0x1D48D},
{0x1D48E,Ll,0x1D48E,0x1D48E,0x1D48E},
{0x1D48F,Ll,0x1D48F,0x1D48F,0x1D48F},
{0x1D490,Ll,0x1D490,0x1D490,0x1D490},
{0x1D491,Ll,0x1D491,0x1D491,0x1D491},
{0x1D492,Ll,0x1D492,0x1D492,0x1D492},
{0x1D493,Ll,0x1D493,0x1D493,0x1D493},
{0x1D494,Ll,0x1D494,0x1D494,0x1D494},
{0x1D495,Ll,0x1D495,0x1D495,0x1D495},
{0x1D496,Ll,0x1D496,0x1D496,0x1D496},
{0x1D497,Ll,0x1D497,0x1D497,0x1D497},
{0x1D498,Ll,0x1D498,0x1D498,0x1D498},
{0x1D499,Ll,0x1D499,0x1D499,0x1D499},
{0x1D49A,Ll,0x1D49A,0x1D49A,0x1D49A},
{0x1D49B,Ll,0x1D49B,0x1D49B,0x1D49B},
{0x1D49C,Lu,0x1D49C,0x1D49C,0x1D49C},
{0x1D49E,Lu,0x1D49E,0x1D49E,0x1D49E},
{0x1D49F,Lu,0x1D49F,0x1D49F,0x1D49F},
{0x1D4A2,Lu,0x1D4A2,0x1D4A2,0x1D4A2},
{0x1D4A5,Lu,0x1D4A5,0x1D4A5,0x1D4A5},
{0x1D4A6,Lu,0x1D4A6,0x1D4A6,0x1D4A6},
{0x1D4A9,Lu,0x1D4A9,0x1D4A9,0x1D4A9},
{0x1D4AA,Lu,0x1D4AA,0x1D4AA,0x1D4AA},
{0x1D4AB,Lu,0x1D4AB,0x1D4AB,0x1D4AB},
{0x1D4AC,Lu,0x1D4AC,0x1D4AC,0x1D4AC},
{0x1D4AE,Lu,0x1D4AE,0x1D4AE,0x1D4AE},
{0x1D4AF,Lu,0x1D4AF,0x1D4AF,0x1D4AF},
{0x1D4B0,Lu,0x1D4B0,0x1D4B0,0x1D4B0},
{0x1D4B1,Lu,0x1D4B1,0x1D4B1,0x1D4B1},
{0x1D4B2,Lu,0x1D4B2,0x1D4B2,0x1D4B2},
{0x1D4B3,Lu,0x1D4B3,0x1D4B3,0x1D4B3},
{0x1D4B4,Lu,0x1D4B4,0x1D4B4,0x1D4B4},
{0x1D4B5,Lu,0x1D4B5,0x1D4B5,0x1D4B5},
{0x1D4B6,Ll,0x1D4B6,0x1D4B6,0x1D4B6},
{0x1D4B7,Ll,0x1D4B7,0x1D4B7,0x1D4B7},
{0x1D4B8,Ll,0x1D4B8,0x1D4B8,0x1D4B8},
{0x1D4B9,Ll,0x1D4B9,0x1D4B9,0x1D4B9},
{0x1D4BB,Ll,0x1D4BB,0x1D4BB,0x1D4BB},
{0x1D4BD,Ll,0x1D4BD,0x1D4BD,0x1D4BD},
{0x1D4BE,Ll,0x1D4BE,0x1D4BE,0x1D4BE},
{0x1D4BF,Ll,0x1D4BF,0x1D4BF,0x1D4BF},
{0x1D4C0,Ll,0x1D4C0,0x1D4C0,0x1D4C0},
{0x1D4C1,Ll,0x1D4C1,0x1D4C1,0x1D4C1},
{0x1D4C2,Ll,0x1D4C2,0x1D4C2,0x1D4C2},
{0x1D4C3,Ll,0x1D4C3,0x1D4C3,0x1D4C3},
{0x1D4C5,Ll,0x1D4C5,0x1D4C5,0x1D4C5},
{0x1D4C6,Ll,0x1D4C6,0x1D4C6,0x1D4C6},
{0x1D4C7,Ll,0x1D4C7,0x1D4C7,0x1D4C7},
{0x1D4C8,Ll,0x1D4C8,0x1D4C8,0x1D4C8},
{0x1D4C9,Ll,0x1D4C9,0x1D4C9,0x1D4C9},
{0x1D4CA,Ll,0x1D4CA,0x1D4CA,0x1D4CA},
{0x1D4CB,Ll,0x1D4CB,0x1D4CB,0x1D4CB},
{0x1D4CC,Ll,0x1D4CC,0x1D4CC,0x1D4CC},
{0x1D4CD,Ll,0x1D4CD,0x1D4CD,0x1D4CD},
{0x1D4CE,Ll,0x1D4CE,0x1D4CE,0x1D4CE},
{0x1D4CF,Ll,0x1D4CF,0x1D4CF,0x1D4CF},
{0x1D4D0,Lu,0x1D4D0,0x1D4D0,0x1D4D0},
{0x1D4D1,Lu,0x1D4D1,0x1D4D1,0x1D4D1},
{0x1D4D2,Lu,0x1D4D2,0x1D4D2,0x1D4D2},
{0x1D4D3,Lu,0x1D4D3,0x1D4D3,0x1D4D3},
{0x1D4D4,Lu,0x1D4D4,0x1D4D4,0x1D4D4},
{0x1D4D5,Lu,0x1D4D5,0x1D4D5,0x1D4D5},
{0x1D4D6,Lu,0x1D4D6,0x1D4D6,0x1D4D6},
{0x1D4D7,Lu,0x1D4D7,0x1D4D7,0x1D4D7},
{0x1D4D8,Lu,0x1D4D8,0x1D4D8,0x1D4D8},
{0x1D4D9,Lu,0x1D4D9,0x1D4D9,0x1D4D9},
{0x1D4DA,Lu,0x1D4DA,0x1D4DA,0x1D4DA},
{0x1D4DB,Lu,0x1D4DB,0x1D4DB,0x1D4DB},
{0x1D4DC,Lu,0x1D4DC,0x1D4DC,0x1D4DC},
{0x1D4DD,Lu,0x1D4DD,0x1D4DD,0x1D4DD},
{0x1D4DE,Lu,0x1D4DE,0x1D4DE,0x1D4DE},
{0x1D4DF,Lu,0x1D4DF,0x1D4DF,0x1D4DF},
{0x1D4E0,Lu,0x1D4E0,0x1D4E0,0x1D4E0},
{0x1D4E1,Lu,0x1D4E1,0x1D4E1,0x1D4E1},
{0x1D4E2,Lu,0x1D4E2,0x1D4E2,0x1D4E2},
{0x1D4E3,Lu,0x1D4E3,0x1D4E3,0x1D4E3},
{0x1D4E4,Lu,0x1D4E4,0x1D4E4,0x1D4E4},
{0x1D4E5,Lu,0x1D4E5,0x1D4E5,0x1D4E5},
{0x1D4E6,Lu,0x1D4E6,0x1D4E6,0x1D4E6},
{0x1D4E7,Lu,0x1D4E7,0x1D4E7,0x1D4E7},
{0x1D4E8,Lu,0x1D4E8,0x1D4E8,0x1D4E8},
{0x1D4E9,Lu,0x1D4E9,0x1D4E9,0x1D4E9},
{0x1D4EA,Ll,0x1D4EA,0x1D4EA,0x1D4EA},
{0x1D4EB,Ll,0x1D4EB,0x1D4EB,0x1D4EB},
{0x1D4EC,Ll,0x1D4EC,0x1D4EC,0x1D4EC},
{0x1D4ED,Ll,0x1D4ED,0x1D4ED,0x1D4ED},
{0x1D4EE,Ll,0x1D4EE,0x1D4EE,0x1D4EE},
{0x1D4EF,Ll,0x1D4EF,0x1D4EF,0x1D4EF},
{0x1D4F0,Ll,0x1D4F0,0x1D4F0,0x1D4F0},
{0x1D4F1,Ll,0x1D4F1,0x1D4F1,0x1D4F1},
{0x1D4F2,Ll,0x1D4F2,0x1D4F2,0x1D4F2},
{0x1D4F3,Ll,0x1D4F3,0x1D4F3,0x1D4F3},
{0x1D4F4,Ll,0x1D4F4,0x1D4F4,0x1D4F4},
{0x1D4F5,Ll,0x1D4F5,0x1D4F5,0x1D4F5},
{0x1D4F6,Ll,0x1D4F6,0x1D4F6,0x1D4F6},
{0x1D4F7,Ll,0x1D4F7,0x1D4F7,0x1D4F7},
{0x1D4F8,Ll,0x1D4F8,0x1D4F8,0x1D4F8},
{0x1D4F9,Ll,0x1D4F9,0x1D4F9,0x1D4F9},
{0x1D4FA,Ll,0x1D4FA,0x1D4FA,0x1D4FA},
{0x1D4FB,Ll,0x1D4FB,0x1D4FB,0x1D4FB},
{0x1D4FC,Ll,0x1D4FC,0x1D4FC,0x1D4FC},
{0x1D4FD,Ll,0x1D4FD,0x1D4FD,0x1D4FD},
{0x1D4FE,Ll,0x1D4FE,0x1D4FE,0x1D4FE},
{0x1D4FF,Ll,0x1D4FF,0x1D4FF,0x1D4FF},
{0x1D500,Ll,0x1D500,0x1D500,0x1D500},
{0x1D501,Ll,0x1D501,0x1D501,0x1D501},
{0x1D502,Ll,0x1D502,0x1D502,0x1D502},
{0x1D503,Ll,0x1D503,0x1D503,0x1D503},
{0x1D504,Lu,0x1D504,0x1D504,0x1D504},
{0x1D505,Lu,0x1D505,0x1D505,0x1D505},
{0x1D507,Lu,0x1D507,0x1D507,0x1D507},
{0x1D508,Lu,0x1D508,0x1D508,0x1D508},
{0x1D509,Lu,0x1D509,0x1D509,0x1D509},
{0x1D50A,Lu,0x1D50A,0x1D50A,0x1D50A},
{0x1D50D,Lu,0x1D50D,0x1D50D,0x1D50D},
{0x1D50E,Lu,0x1D50E,0x1D50E,0x1D50E},
{0x1D50F,Lu,0x1D50F,0x1D50F,0x1D50F},
{0x1D510,Lu,0x1D510,0x1D510,0x1D510},
{0x1D511,Lu,0x1D511,0x1D511,0x1D511},
{0x1D512,Lu,0x1D512,0x1D512,0x1D512},
{0x1D513,Lu,0x1D513,0x1D513,0x1D513},
{0x1D514,Lu,0x1D514,0x1D514,0x1D514},
{0x1D516,Lu,0x1D516,0x1D516,0x1D516},
{0x1D517,Lu,0x1D517,0x1D517,0x1D517},
{0x1D518,Lu,0x1D518,0x1D518,0x1D518},
{0x1D519,Lu,0x1D519,0x1D519,0x1D519},
{0x1D51A,Lu,0x1D51A,0x1D51A,0x1D51A},
{0x1D51B,Lu,0x1D51B,0x1D51B,0x1D51B},
{0x1D51C,Lu,0x1D51C,0x1D51C,0x1D51C},
{0x1D51E,Ll,0x1D51E,0x1D51E,0x1D51E},
{0x1D51F,Ll,0x1D51F,0x1D51F,0x1D51F},
{0x1D520,Ll,0x1D520,0x1D520,0x1D520},
{0x1D521,Ll,0x1D521,0x1D521,0x1D521},
{0x1D522,Ll,0x1D522,0x1D522,0x1D522},
{0x1D523,Ll,0x1D523,0x1D523,0x1D523},
{0x1D524,Ll,0x1D524,0x1D524,0x1D524},
{0x1D525,Ll,0x1D525,0x1D525,0x1D525},
{0x1D526,Ll,0x1D526,0x1D526,0x1D526},
{0x1D527,Ll,0x1D527,0x1D527,0x1D527},
{0x1D528,Ll,0x1D528,0x1D528,0x1D528},
{0x1D529,Ll,0x1D529,0x1D529,0x1D529},
{0x1D52A,Ll,0x1D52A,0x1D52A,0x1D52A},
{0x1D52B,Ll,0x1D52B,0x1D52B,0x1D52B},
{0x1D52C,Ll,0x1D52C,0x1D52C,0x1D52C},
{0x1D52D,Ll,0x1D52D,0x1D52D,0x1D52D},
{0x1D52E,Ll,0x1D52E,0x1D52E,0x1D52E},
{0x1D52F,Ll,0x1D52F,0x1D52F,0x1D52F},
{0x1D530,Ll,0x1D530,0x1D530,0x1D530},
{0x1D531,Ll,0x1D531,0x1D531,0x1D531},
{0x1D532,Ll,0x1D532,0x1D532,0x1D532},
{0x1D533,Ll,0x1D533,0x1D533,0x1D533},
{0x1D534,Ll,0x1D534,0x1D534,0x1D534},
{0x1D535,Ll,0x1D535,0x1D535,0x1D535},
{0x1D536,Ll,0x1D536,0x1D536,0x1D536},
{0x1D537,Ll,0x1D537,0x1D537,0x1D537},
{0x1D538,Lu,0x1D538,0x1D538,0x1D538},
{0x1D539,Lu,0x1D539,0x1D539,0x1D539},
{0x1D53B,Lu,0x1D53B,0x1D53B,0x1D53B},
{0x1D53C,Lu,0x1D53C,0x1D53C,0x1D53C},
{0x1D53D,Lu,0x1D53D,0x1D53D,0x1D53D},
{0x1D53E,Lu,0x1D53E,0x1D53E,0x1D53E},
{0x1D540,Lu,0x1D540,0x1D540,0x1D540},
{0x1D541,Lu,0x1D541,0x1D541,0x1D541},
{0x1D542,Lu,0x1D542,0x1D542,0x1D542},
{0x1D543,Lu,0x1D543,0x1D543,0x1D543},
{0x1D544,Lu,0x1D544,0x1D544,0x1D544},
{0x1D546,Lu,0x1D546,0x1D546,0x1D546},
{0x1D54A,Lu,0x1D54A,0x1D54A,0x1D54A},
{0x1D54B,Lu,0x1D54B,0x1D54B,0x1D54B},
{0x1D54C,Lu,0x1D54C,0x1D54C,0x1D54C},
{0x1D54D,Lu,0x1D54D,0x1D54D,0x1D54D},
{0x1D54E,Lu,0x1D54E,0x1D54E,0x1D54E},
{0x1D54F,Lu,0x1D54F,0x1D54F,0x1D54F},
{0x1D550,Lu,0x1D550,0x1D550,0x1D550},
{0x1D552,Ll,0x1D552,0x1D552,0x1D552},
{0x1D553,Ll,0x1D553,0x1D553,0x1D553},
{0x1D554,Ll,0x1D554,0x1D554,0x1D554},
{0x1D555,Ll,0x1D555,0x1D555,0x1D555},
{0x1D556,Ll,0x1D556,0x1D556,0x1D556},
{0x1D557,Ll,0x1D557,0x1D557,0x1D557},
{0x1D558,Ll,0x1D558,0x1D558,0x1D558},
{0x1D559,Ll,0x1D559,0x1D559,0x1D559},
{0x1D55A,Ll,0x1D55A,0x1D55A,0x1D55A},
{0x1D55B,Ll,0x1D55B,0x1D55B,0x1D55B},
{0x1D55C,Ll,0x1D55C,0x1D55C,0x1D55C},
{0x1D55D,Ll,0x1D55D,0x1D55D,0x1D55D},
{0x1D55E,Ll,0x1D55E,0x1D55E,0x1D55E},
{0x1D55F,Ll,0x1D55F,0x1D55F,0x1D55F},
{0x1D560,Ll,0x1D560,0x1D560,0x1D560},
{0x1D561,Ll,0x1D561,0x1D561,0x1D561},
{0x1D562,Ll,0x1D562,0x1D562,0x1D562},
{0x1D563,Ll,0x1D563,0x1D563,0x1D563},
{0x1D564,Ll,0x1D564,0x1D564,0x1D564},
{0x1D565,Ll,0x1D565,0x1D565,0x1D565},
{0x1D566,Ll,0x1D566,0x1D566,0x1D566},
{0x1D567,Ll,0x1D567,0x1D567,0x1D567},
{0x1D568,Ll,0x1D568,0x1D568,0x1D568},
{0x1D569,Ll,0x1D569,0x1D569,0x1D569},
{0x1D56A,Ll,0x1D56A,0x1D56A,0x1D56A},
{0x1D56B,Ll,0x1D56B,0x1D56B,0x1D56B},
{0x1D56C,Lu,0x1D56C,0x1D56C,0x1D56C},
{0x1D56D,Lu,0x1D56D,0x1D56D,0x1D56D},
{0x1D56E,Lu,0x1D56E,0x1D56E,0x1D56E},
{0x1D56F,Lu,0x1D56F,0x1D56F,0x1D56F},
{0x1D570,Lu,0x1D570,0x1D570,0x1D570},
{0x1D571,Lu,0x1D571,0x1D571,0x1D571},
{0x1D572,Lu,0x1D572,0x1D572,0x1D572},
{0x1D573,Lu,0x1D573,0x1D573,0x1D573},
{0x1D574,Lu,0x1D574,0x1D574,0x1D574},
{0x1D575,Lu,0x1D575,0x1D575,0x1D575},
{0x1D576,Lu,0x1D576,0x1D576,0x1D576},
{0x1D577,Lu,0x1D577,0x1D577,0x1D577},
{0x1D578,Lu,0x1D578,0x1D578,0x1D578},
{0x1D579,Lu,0x1D579,0x1D579,0x1D579},
{0x1D57A,Lu,0x1D57A,0x1D57A,0x1D57A},
{0x1D57B,Lu,0x1D57B,0x1D57B,0x1D57B},
{0x1D57C,Lu,0x1D57C,0x1D57C,0x1D57C},
{0x1D57D,Lu,0x1D57D,0x1D57D,0x1D57D},
{0x1D57E,Lu,0x1D57E,0x1D57E,0x1D57E},
{0x1D57F,Lu,0x1D57F,0x1D57F,0x1D57F},
{0x1D580,Lu,0x1D580,0x1D580,0x1D580},
{0x1D581,Lu,0x1D581,0x1D581,0x1D581},
{0x1D582,Lu,0x1D582,0x1D582,0x1D582},
{0x1D583,Lu,0x1D583,0x1D583,0x1D583},
{0x1D584,Lu,0x1D584,0x1D584,0x1D584},
{0x1D585,Lu,0x1D585,0x1D585,0x1D585},
{0x1D586,Ll,0x1D586,0x1D586,0x1D586},
{0x1D587,Ll,0x1D587,0x1D587,0x1D587},
{0x1D588,Ll,0x1D588,0x1D588,0x1D588},
{0x1D589,Ll,0x1D589,0x1D589,0x1D589},
{0x1D58A,Ll,0x1D58A,0x1D58A,0x1D58A},
{0x1D58B,Ll,0x1D58B,0x1D58B,0x1D58B},
{0x1D58C,Ll,0x1D58C,0x1D58C,0x1D58C},
{0x1D58D,Ll,0x1D58D,0x1D58D,0x1D58D},
{0x1D58E,Ll,0x1D58E,0x1D58E,0x1D58E},
{0x1D58F,Ll,0x1D58F,0x1D58F,0x1D58F},
{0x1D590,Ll,0x1D590,0x1D590,0x1D590},
{0x1D591,Ll,0x1D591,0x1D591,0x1D591},
{0x1D592,Ll,0x1D592,0x1D592,0x1D592},
{0x1D593,Ll,0x1D593,0x1D593,0x1D593},
{0x1D594,Ll,0x1D594,0x1D594,0x1D594},
{0x1D595,Ll,0x1D595,0x1D595,0x1D595},
{0x1D596,Ll,0x1D596,0x1D596,0x1D596},
{0x1D597,Ll,0x1D597,0x1D597,0x1D597},
{0x1D598,Ll,0x1D598,0x1D598,0x1D598},
{0x1D599,Ll,0x1D599,0x1D599,0x1D599},
{0x1D59A,Ll,0x1D59A,0x1D59A,0x1D59A},
{0x1D59B,Ll,0x1D59B,0x1D59B,0x1D59B},
{0x1D59C,Ll,0x1D59C,0x1D59C,0x1D59C},
{0x1D59D,Ll,0x1D59D,0x1D59D,0x1D59D},
{0x1D59E,Ll,0x1D59E,0x1D59E,0x1D59E},
{0x1D59F,Ll,0x1D59F,0x1D59F,0x1D59F},
{0x1D5A0,Lu,0x1D5A0,0x1D5A0,0x1D5A0},
{0x1D5A1,Lu,0x1D5A1,0x1D5A1,0x1D5A1},
{0x1D5A2,Lu,0x1D5A2,0x1D5A2,0x1D5A2},
{0x1D5A3,Lu,0x1D5A3,0x1D5A3,0x1D5A3},
{0x1D5A4,Lu,0x1D5A4,0x1D5A4,0x1D5A4},
{0x1D5A5,Lu,0x1D5A5,0x1D5A5,0x1D5A5},
{0x1D5A6,Lu,0x1D5A6,0x1D5A6,0x1D5A6},
{0x1D5A7,Lu,0x1D5A7,0x1D5A7,0x1D5A7},
{0x1D5A8,Lu,0x1D5A8,0x1D5A8,0x1D5A8},
{0x1D5A9,Lu,0x1D5A9,0x1D5A9,0x1D5A9},
{0x1D5AA,Lu,0x1D5AA,0x1D5AA,0x1D5AA},
{0x1D5AB,Lu,0x1D5AB,0x1D5AB,0x1D5AB},
{0x1D5AC,Lu,0x1D5AC,0x1D5AC,0x1D5AC},
{0x1D5AD,Lu,0x1D5AD,0x1D5AD,0x1D5AD},
{0x1D5AE,Lu,0x1D5AE,0x1D5AE,0x1D5AE},
{0x1D5AF,Lu,0x1D5AF,0x1D5AF,0x1D5AF},
{0x1D5B0,Lu,0x1D5B0,0x1D5B0,0x1D5B0},
{0x1D5B1,Lu,0x1D5B1,0x1D5B1,0x1D5B1},
{0x1D5B2,Lu,0x1D5B2,0x1D5B2,0x1D5B2},
{0x1D5B3,Lu,0x1D5B3,0x1D5B3,0x1D5B3},
{0x1D5B4,Lu,0x1D5B4,0x1D5B4,0x1D5B4},
{0x1D5B5,Lu,0x1D5B5,0x1D5B5,0x1D5B5},
{0x1D5B6,Lu,0x1D5B6,0x1D5B6,0x1D5B6},
{0x1D5B7,Lu,0x1D5B7,0x1D5B7,0x1D5B7},
{0x1D5B8,Lu,0x1D5B8,0x1D5B8,0x1D5B8},
{0x1D5B9,Lu,0x1D5B9,0x1D5B9,0x1D5B9},
{0x1D5BA,Ll,0x1D5BA,0x1D5BA,0x1D5BA},
{0x1D5BB,Ll,0x1D5BB,0x1D5BB,0x1D5BB},
{0x1D5BC,Ll,0x1D5BC,0x1D5BC,0x1D5BC},
{0x1D5BD,Ll,0x1D5BD,0x1D5BD,0x1D5BD},
{0x1D5BE,Ll,0x1D5BE,0x1D5BE,0x1D5BE},
{0x1D5BF,Ll,0x1D5BF,0x1D5BF,0x1D5BF},
{0x1D5C0,Ll,0x1D5C0,0x1D5C0,0x1D5C0},
{0x1D5C1,Ll,0x1D5C1,0x1D5C1,0x1D5C1},
{0x1D5C2,Ll,0x1D5C2,0x1D5C2,0x1D5C2},
{0x1D5C3,Ll,0x1D5C3,0x1D5C3,0x1D5C3},
{0x1D5C4,Ll,0x1D5C4,0x1D5C4,0x1D5C4},
{0x1D5C5,Ll,0x1D5C5,0x1D5C5,0x1D5C5},
{0x1D5C6,Ll,0x1D5C6,0x1D5C6,0x1D5C6},
{0x1D5C7,Ll,0x1D5C7,0x1D5C7,0x1D5C7},
{0x1D5C8,Ll,0x1D5C8,0x1D5C8,0x1D5C8},
{0x1D5C9,Ll,0x1D5C9,0x1D5C9,0x1D5C9},
{0x1D5CA,Ll,0x1D5CA,0x1D5CA,0x1D5CA},
{0x1D5CB,Ll,0x1D5CB,0x1D5CB,0x1D5CB},
{0x1D5CC,Ll,0x1D5CC,0x1D5CC,0x1D5CC},
{0x1D5CD,Ll,0x1D5CD,0x1D5CD,0x1D5CD},
{0x1D5CE,Ll,0x1D5CE,0x1D5CE,0x1D5CE},
{0x1D5CF,Ll,0x1D5CF,0x1D5CF,0x1D5CF},
{0x1D5D0,Ll,0x1D5D0,0x1D5D0,0x1D5D0},
{0x1D5D1,Ll,0x1D5D1,0x1D5D1,0x1D5D1},
{0x1D5D2,Ll,0x1D5D2,0x1D5D2,0x1D5D2},
{0x1D5D3,Ll,0x1D5D3,0x1D5D3,0x1D5D3},
{0x1D5D4,Lu,0x1D5D4,0x1D5D4,0x1D5D4},
{0x1D5D5,Lu,0x1D5D5,0x1D5D5,0x1D5D5},
{0x1D5D6,Lu,0x1D5D6,0x1D5D6,0x1D5D6},
{0x1D5D7,Lu,0x1D5D7,0x1D5D7,0x1D5D7},
{0x1D5D8,Lu,0x1D5D8,0x1D5D8,0x1D5D8},
{0x1D5D9,Lu,0x1D5D9,0x1D5D9,0x1D5D9},
{0x1D5DA,Lu,0x1D5DA,0x1D5DA,0x1D5DA},
{0x1D5DB,Lu,0x1D5DB,0x1D5DB,0x1D5DB},
{0x1D5DC,Lu,0x1D5DC,0x1D5DC,0x1D5DC},
{0x1D5DD,Lu,0x1D5DD,0x1D5DD,0x1D5DD},
{0x1D5DE,Lu,0x1D5DE,0x1D5DE,0x1D5DE},
{0x1D5DF,Lu,0x1D5DF,0x1D5DF,0x1D5DF},
{0x1D5E0,Lu,0x1D5E0,0x1D5E0,0x1D5E0},
{0x1D5E1,Lu,0x1D5E1,0x1D5E1,0x1D5E1},
{0x1D5E2,Lu,0x1D5E2,0x1D5E2,0x1D5E2},
{0x1D5E3,Lu,0x1D5E3,0x1D5E3,0x1D5E3},
{0x1D5E4,Lu,0x1D5E4,0x1D5E4,0x1D5E4},
{0x1D5E5,Lu,0x1D5E5,0x1D5E5,0x1D5E5},
{0x1D5E6,Lu,0x1D5E6,0x1D5E6,0x1D5E6},
{0x1D5E7,Lu,0x1D5E7,0x1D5E7,0x1D5E7},
{0x1D5E8,Lu,0x1D5E8,0x1D5E8,0x1D5E8},
{0x1D5E9,Lu,0x1D5E9,0x1D5E9,0x1D5E9},
{0x1D5EA,Lu,0x1D5EA,0x1D5EA,0x1D5EA},
{0x1D5EB,Lu,0x1D5EB,0x1D5EB,0x1D5EB},
{0x1D5EC,Lu,0x1D5EC,0x1D5EC,0x1D5EC},
{0x1D5ED,Lu,0x1D5ED,0x1D5ED,0x1D5ED},
{0x1D5EE,Ll,0x1D5EE,0x1D5EE,0x1D5EE},
{0x1D5EF,Ll,0x1D5EF,0x1D5EF,0x1D5EF},
{0x1D5F0,Ll,0x1D5F0,0x1D5F0,0x1D5F0},
{0x1D5F1,Ll,0x1D5F1,0x1D5F1,0x1D5F1},
{0x1D5F2,Ll,0x1D5F2,0x1D5F2,0x1D5F2},
{0x1D5F3,Ll,0x1D5F3,0x1D5F3,0x1D5F3},
{0x1D5F4,Ll,0x1D5F4,0x1D5F4,0x1D5F4},
{0x1D5F5,Ll,0x1D5F5,0x1D5F5,0x1D5F5},
{0x1D5F6,Ll,0x1D5F6,0x1D5F6,0x1D5F6},
{0x1D5F7,Ll,0x1D5F7,0x1D5F7,0x1D5F7},
{0x1D5F8,Ll,0x1D5F8,0x1D5F8,0x1D5F8},
{0x1D5F9,Ll,0x1D5F9,0x1D5F9,0x1D5F9},
{0x1D5FA,Ll,0x1D5FA,0x1D5FA,0x1D5FA},
{0x1D5FB,Ll,0x1D5FB,0x1D5FB,0x1D5FB},
{0x1D5FC,Ll,0x1D5FC,0x1D5FC,0x1D5FC},
{0x1D5FD,Ll,0x1D5FD,0x1D5FD,0x1D5FD},
{0x1D5FE,Ll,0x1D5FE,0x1D5FE,0x1D5FE},
{0x1D5FF,Ll,0x1D5FF,0x1D5FF,0x1D5FF},
{0x1D600,Ll,0x1D600,0x1D600,0x1D600},
{0x1D601,Ll,0x1D601,0x1D601,0x1D601},
{0x1D602,Ll,0x1D602,0x1D602,0x1D602},
{0x1D603,Ll,0x1D603,0x1D603,0x1D603},
{0x1D604,Ll,0x1D604,0x1D604,0x1D604},
{0x1D605,Ll,0x1D605,0x1D605,0x1D605},
{0x1D606,Ll,0x1D606,0x1D606,0x1D606},
{0x1D607,Ll,0x1D607,0x1D607,0x1D607},
{0x1D608,Lu,0x1D608,0x1D608,0x1D608},
{0x1D609,Lu,0x1D609,0x1D609,0x1D609},
{0x1D60A,Lu,0x1D60A,0x1D60A,0x1D60A},
{0x1D60B,Lu,0x1D60B,0x1D60B,0x1D60B},
{0x1D60C,Lu,0x1D60C,0x1D60C,0x1D60C},
{0x1D60D,Lu,0x1D60D,0x1D60D,0x1D60D},
{0x1D60E,Lu,0x1D60E,0x1D60E,0x1D60E},
{0x1D60F,Lu,0x1D60F,0x1D60F,0x1D60F},
{0x1D610,Lu,0x1D610,0x1D610,0x1D610},
{0x1D611,Lu,0x1D611,0x1D611,0x1D611},
{0x1D612,Lu,0x1D612,0x1D612,0x1D612},
{0x1D613,Lu,0x1D613,0x1D613,0x1D613},
{0x1D614,Lu,0x1D614,0x1D614,0x1D614},
{0x1D615,Lu,0x1D615,0x1D615,0x1D615},
{0x1D616,Lu,0x1D616,0x1D616,0x1D616},
{0x1D617,Lu,0x1D617,0x1D617,0x1D617},
{0x1D618,Lu,0x1D618,0x1D618,0x1D618},
{0x1D619,Lu,0x1D619,0x1D619,0x1D619},
{0x1D61A,Lu,0x1D61A,0x1D61A,0x1D61A},
{0x1D61B,Lu,0x1D61B,0x1D61B,0x1D61B},
{0x1D61C,Lu,0x1D61C,0x1D61C,0x1D61C},
{0x1D61D,Lu,0x1D61D,0x1D61D,0x1D61D},
{0x1D61E,Lu,0x1D61E,0x1D61E,0x1D61E},
{0x1D61F,Lu,0x1D61F,0x1D61F,0x1D61F},
{0x1D620,Lu,0x1D620,0x1D620,0x1D620},
{0x1D621,Lu,0x1D621,0x1D621,0x1D621},
{0x1D622,Ll,0x1D622,0x1D622,0x1D622},
{0x1D623,Ll,0x1D623,0x1D623,0x1D623},
{0x1D624,Ll,0x1D624,0x1D624,0x1D624},
{0x1D625,Ll,0x1D625,0x1D625,0x1D625},
{0x1D626,Ll,0x1D626,0x1D626,0x1D626},
{0x1D627,Ll,0x1D627,0x1D627,0x1D627},
{0x1D628,Ll,0x1D628,0x1D628,0x1D628},
{0x1D629,Ll,0x1D629,0x1D629,0x1D629},
{0x1D62A,Ll,0x1D62A,0x1D62A,0x1D62A},
{0x1D62B,Ll,0x1D62B,0x1D62B,0x1D62B},
{0x1D62C,Ll,0x1D62C,0x1D62C,0x1D62C},
{0x1D62D,Ll,0x1D62D,0x1D62D,0x1D62D},
{0x1D62E,Ll,0x1D62E,0x1D62E,0x1D62E},
{0x1D62F,Ll,0x1D62F,0x1D62F,0x1D62F},
{0x1D630,Ll,0x1D630,0x1D630,0x1D630},
{0x1D631,Ll,0x1D631,0x1D631,0x1D631},
{0x1D632,Ll,0x1D632,0x1D632,0x1D632},
{0x1D633,Ll,0x1D633,0x1D633,0x1D633},
{0x1D634,Ll,0x1D634,0x1D634,0x1D634},
{0x1D635,Ll,0x1D635,0x1D635,0x1D635},
{0x1D636,Ll,0x1D636,0x1D636,0x1D636},
{0x1D637,Ll,0x1D637,0x1D637,0x1D637},
{0x1D638,Ll,0x1D638,0x1D638,0x1D638},
{0x1D639,Ll,0x1D639,0x1D639,0x1D639},
{0x1D63A,Ll,0x1D63A,0x1D63A,0x1D63A},
{0x1D63B,Ll,0x1D63B,0x1D63B,0x1D63B},
{0x1D63C,Lu,0x1D63C,0x1D63C,0x1D63C},
{0x1D63D,Lu,0x1D63D,0x1D63D,0x1D63D},
{0x1D63E,Lu,0x1D63E,0x1D63E,0x1D63E},
{0x1D63F,Lu,0x1D63F,0x1D63F,0x1D63F},
{0x1D640,Lu,0x1D640,0x1D640,0x1D640},
{0x1D641,Lu,0x1D641,0x1D641,0x1D641},
{0x1D642,Lu,0x1D642,0x1D642,0x1D642},
{0x1D643,Lu,0x1D643,0x1D643,0x1D643},
{0x1D644,Lu,0x1D644,0x1D644,0x1D644},
{0x1D645,Lu,0x1D645,0x1D645,0x1D645},
{0x1D646,Lu,0x1D646,0x1D646,0x1D646},
{0x1D647,Lu,0x1D647,0x1D647,0x1D647},
{0x1D648,Lu,0x1D648,0x1D648,0x1D648},
{0x1D649,Lu,0x1D649,0x1D649,0x1D649},
{0x1D64A,Lu,0x1D64A,0x1D64A,0x1D64A},
{0x1D64B,Lu,0x1D64B,0x1D64B,0x1D64B},
{0x1D64C,Lu,0x1D64C,0x1D64C,0x1D64C},
{0x1D64D,Lu,0x1D64D,0x1D64D,0x1D64D},
{0x1D64E,Lu,0x1D64E,0x1D64E,0x1D64E},
{0x1D64F,Lu,0x1D64F,0x1D64F,0x1D64F},
{0x1D650,Lu,0x1D650,0x1D650,0x1D650},
{0x1D651,Lu,0x1D651,0x1D651,0x1D651},
{0x1D652,Lu,0x1D652,0x1D652,0x1D652},
{0x1D653,Lu,0x1D653,0x1D653,0x1D653},
{0x1D654,Lu,0x1D654,0x1D654,0x1D654},
{0x1D655,Lu,0x1D655,0x1D655,0x1D655},
{0x1D656,Ll,0x1D656,0x1D656,0x1D656},
{0x1D657,Ll,0x1D657,0x1D657,0x1D657},
{0x1D658,Ll,0x1D658,0x1D658,0x1D658},
{0x1D659,Ll,0x1D659,0x1D659,0x1D659},
{0x1D65A,Ll,0x1D65A,0x1D65A,0x1D65A},
{0x1D65B,Ll,0x1D65B,0x1D65B,0x1D65B},
{0x1D65C,Ll,0x1D65C,0x1D65C,0x1D65C},
{0x1D65D,Ll,0x1D65D,0x1D65D,0x1D65D},
{0x1D65E,Ll,0x1D65E,0x1D65E,0x1D65E},
{0x1D65F,Ll,0x1D65F,0x1D65F,0x1D65F},
{0x1D660,Ll,0x1D660,0x1D660,0x1D660},
{0x1D661,Ll,0x1D661,0x1D661,0x1D661},
{0x1D662,Ll,0x1D662,0x1D662,0x1D662},
{0x1D663,Ll,0x1D663,0x1D663,0x1D663},
{0x1D664,Ll,0x1D664,0x1D664,0x1D664},
{0x1D665,Ll,0x1D665,0x1D665,0x1D665},
{0x1D666,Ll,0x1D666,0x1D666,0x1D666},
{0x1D667,Ll,0x1D667,0x1D667,0x1D667},
{0x1D668,Ll,0x1D668,0x1D668,0x1D668},
{0x1D669,Ll,0x1D669,0x1D669,0x1D669},
{0x1D66A,Ll,0x1D66A,0x1D66A,0x1D66A},
{0x1D66B,Ll,0x1D66B,0x1D66B,0x1D66B},
{0x1D66C,Ll,0x1D66C,0x1D66C,0x1D66C},
{0x1D66D,Ll,0x1D66D,0x1D66D,0x1D66D},
{0x1D66E,Ll,0x1D66E,0x1D66E,0x1D66E},
{0x1D66F,Ll,0x1D66F,0x1D66F,0x1D66F},
{0x1D670,Lu,0x1D670,0x1D670,0x1D670},
{0x1D671,Lu,0x1D671,0x1D671,0x1D671},
{0x1D672,Lu,0x1D672,0x1D672,0x1D672},
{0x1D673,Lu,0x1D673,0x1D673,0x1D673},
{0x1D674,Lu,0x1D674,0x1D674,0x1D674},
{0x1D675,Lu,0x1D675,0x1D675,0x1D675},
{0x1D676,Lu,0x1D676,0x1D676,0x1D676},
{0x1D677,Lu,0x1D677,0x1D677,0x1D677},
{0x1D678,Lu,0x1D678,0x1D678,0x1D678},
{0x1D679,Lu,0x1D679,0x1D679,0x1D679},
{0x1D67A,Lu,0x1D67A,0x1D67A,0x1D67A},
{0x1D67B,Lu,0x1D67B,0x1D67B,0x1D67B},
{0x1D67C,Lu,0x1D67C,0x1D67C,0x1D67C},
{0x1D67D,Lu,0x1D67D,0x1D67D,0x1D67D},
{0x1D67E,Lu,0x1D67E,0x1D67E,0x1D67E},
{0x1D67F,Lu,0x1D67F,0x1D67F,0x1D67F},
{0x1D680,Lu,0x1D680,0x1D680,0x1D680},
{0x1D681,Lu,0x1D681,0x1D681,0x1D681},
{0x1D682,Lu,0x1D682,0x1D682,0x1D682},
{0x1D683,Lu,0x1D683,0x1D683,0x1D683},
{0x1D684,Lu,0x1D684,0x1D684,0x1D684},
{0x1D685,Lu,0x1D685,0x1D685,0x1D685},
{0x1D686,Lu,0x1D686,0x1D686,0x1D686},
{0x1D687,Lu,0x1D687,0x1D687,0x1D687},
{0x1D688,Lu,0x1D688,0x1D688,0x1D688},
{0x1D689,Lu,0x1D689,0x1D689,0x1D689},
{0x1D68A,Ll,0x1D68A,0x1D68A,0x1D68A},
{0x1D68B,Ll,0x1D68B,0x1D68B,0x1D68B},
{0x1D68C,Ll,0x1D68C,0x1D68C,0x1D68C},
{0x1D68D,Ll,0x1D68D,0x1D68D,0x1D68D},
{0x1D68E,Ll,0x1D68E,0x1D68E,0x1D68E},
{0x1D68F,Ll,0x1D68F,0x1D68F,0x1D68F},
{0x1D690,Ll,0x1D690,0x1D690,0x1D690},
{0x1D691,Ll,0x1D691,0x1D691,0x1D691},
{0x1D692,Ll,0x1D692,0x1D692,0x1D692},
{0x1D693,Ll,0x1D693,0x1D693,0x1D693},
{0x1D694,Ll,0x1D694,0x1D694,0x1D694},
{0x1D695,Ll,0x1D695,0x1D695,0x1D695},
{0x1D696,Ll,0x1D696,0x1D696,0x1D696},
{0x1D697,Ll,0x1D697,0x1D697,0x1D697},
{0x1D698,Ll,0x1D698,0x1D698,0x1D698},
{0x1D699,Ll,0x1D699,0x1D699,0x1D699},
{0x1D69A,Ll,0x1D69A,0x1D69A,0x1D69A},
{0x1D69B,Ll,0x1D69B,0x1D69B,0x1D69B},
{0x1D69C,Ll,0x1D69C,0x1D69C,0x1D69C},
{0x1D69D,Ll,0x1D69D,0x1D69D,0x1D69D},
{0x1D69E,Ll,0x1D69E,0x1D69E,0x1D69E},
{0x1D69F,Ll,0x1D69F,0x1D69F,0x1D69F},
{0x1D6A0,Ll,0x1D6A0,0x1D6A0,0x1D6A0},
{0x1D6A1,Ll,0x1D6A1,0x1D6A1,0x1D6A1},
{0x1D6A2,Ll,0x1D6A2,0x1D6A2,0x1D6A2},
{0x1D6A3,Ll,0x1D6A3,0x1D6A3,0x1D6A3},
{0x1D6A4,Ll,0x1D6A4,0x1D6A4,0x1D6A4},
{0x1D6A5,Ll,0x1D6A5,0x1D6A5,0x1D6A5},
{0x1D6A8,Lu,0x1D6A8,0x1D6A8,0x1D6A8},
{0x1D6A9,Lu,0x1D6A9,0x1D6A9,0x1D6A9},
{0x1D6AA,Lu,0x1D6AA,0x1D6AA,0x1D6AA},
{0x1D6AB,Lu,0x1D6AB,0x1D6AB,0x1D6AB},
{0x1D6AC,Lu,0x1D6AC,0x1D6AC,0x1D6AC},
{0x1D6AD,Lu,0x1D6AD,0x1D6AD,0x1D6AD},
{0x1D6AE,Lu,0x1D6AE,0x1D6AE,0x1D6AE},
{0x1D6AF,Lu,0x1D6AF,0x1D6AF,0x1D6AF},
{0x1D6B0,Lu,0x1D6B0,0x1D6B0,0x1D6B0},
{0x1D6B1,Lu,0x1D6B1,0x1D6B1,0x1D6B1},
{0x1D6B2,Lu,0x1D6B2,0x1D6B2,0x1D6B2},
{0x1D6B3,Lu,0x1D6B3,0x1D6B3,0x1D6B3},
{0x1D6B4,Lu,0x1D6B4,0x1D6B4,0x1D6B4},
{0x1D6B5,Lu,0x1D6B5,0x1D6B5,0x1D6B5},
{0x1D6B6,Lu,0x1D6B6,0x1D6B6,0x1D6B6},
{0x1D6B7,Lu,0x1D6B7,0x1D6B7,0x1D6B7},
{0x1D6B8,Lu,0x1D6B8,0x1D6B8,0x1D6B8},
{0x1D6B9,Lu,0x1D6B9,0x1D6B9,0x1D6B9},
{0x1D6BA,Lu,0x1D6BA,0x1D6BA,0x1D6BA},
{0x1D6BB,Lu,0x1D6BB,0x1D6BB,0x1D6BB},
{0x1D6BC,Lu,0x1D6BC,0x1D6BC,0x1D6BC},
{0x1D6BD,Lu,0x1D6BD,0x1D6BD,0x1D6BD},
{0x1D6BE,Lu,0x1D6BE,0x1D6BE,0x1D6BE},
{0x1D6BF,Lu,0x1D6BF,0x1D6BF,0x1D6BF},
{0x1D6C0,Lu,0x1D6C0,0x1D6C0,0x1D6C0},
{0x1D6C1,Sm,0x1D6C1,0x1D6C1,0x1D6C1},
{0x1D6C2,Ll,0x1D6C2,0x1D6C2,0x1D6C2},
{0x1D6C3,Ll,0x1D6C3,0x1D6C3,0x1D6C3},
{0x1D6C4,Ll,0x1D6C4,0x1D6C4,0x1D6C4},
{0x1D6C5,Ll,0x1D6C5,0x1D6C5,0x1D6C5},
{0x1D6C6,Ll,0x1D6C6,0x1D6C6,0x1D6C6},
{0x1D6C7,Ll,0x1D6C7,0x1D6C7,0x1D6C7},
{0x1D6C8,Ll,0x1D6C8,0x1D6C8,0x1D6C8},
{0x1D6C9,Ll,0x1D6C9,0x1D6C9,0x1D6C9},
{0x1D6CA,Ll,0x1D6CA,0x1D6CA,0x1D6CA},
{0x1D6CB,Ll,0x1D6CB,0x1D6CB,0x1D6CB},
{0x1D6CC,Ll,0x1D6CC,0x1D6CC,0x1D6CC},
{0x1D6CD,Ll,0x1D6CD,0x1D6CD,0x1D6CD},
{0x1D6CE,Ll,0x1D6CE,0x1D6CE,0x1D6CE},
{0x1D6CF,Ll,0x1D6CF,0x1D6CF,0x1D6CF},
{0x1D6D0,Ll,0x1D6D0,0x1D6D0,0x1D6D0},
{0x1D6D1,Ll,0x1D6D1,0x1D6D1,0x1D6D1},
{0x1D6D2,Ll,0x1D6D2,0x1D6D2,0x1D6D2},
{0x1D6D3,Ll,0x1D6D3,0x1D6D3,0x1D6D3},
{0x1D6D4,Ll,0x1D6D4,0x1D6D4,0x1D6D4},
{0x1D6D5,Ll,0x1D6D5,0x1D6D5,0x1D6D5},
{0x1D6D6,Ll,0x1D6D6,0x1D6D6,0x1D6D6},
{0x1D6D7,Ll,0x1D6D7,0x1D6D7,0x1D6D7},
{0x1D6D8,Ll,0x1D6D8,0x1D6D8,0x1D6D8},
{0x1D6D9,Ll,0x1D6D9,0x1D6D9,0x1D6D9},
{0x1D6DA,Ll,0x1D6DA,0x1D6DA,0x1D6DA},
{0x1D6DB,Sm,0x1D6DB,0x1D6DB,0x1D6DB},
{0x1D6DC,Ll,0x1D6DC,0x1D6DC,0x1D6DC},
{0x1D6DD,Ll,0x1D6DD,0x1D6DD,0x1D6DD},
{0x1D6DE,Ll,0x1D6DE,0x1D6DE,0x1D6DE},
{0x1D6DF,Ll,0x1D6DF,0x1D6DF,0x1D6DF},
{0x1D6E0,Ll,0x1D6E0,0x1D6E0,0x1D6E0},
{0x1D6E1,Ll,0x1D6E1,0x1D6E1,0x1D6E1},
{0x1D6E2,Lu,0x1D6E2,0x1D6E2,0x1D6E2},
{0x1D6E3,Lu,0x1D6E3,0x1D6E3,0x1D6E3},
{0x1D6E4,Lu,0x1D6E4,0x1D6E4,0x1D6E4},
{0x1D6E5,Lu,0x1D6E5,0x1D6E5,0x1D6E5},
{0x1D6E6,Lu,0x1D6E6,0x1D6E6,0x1D6E6},
{0x1D6E7,Lu,0x1D6E7,0x1D6E7,0x1D6E7},
{0x1D6E8,Lu,0x1D6E8,0x1D6E8,0x1D6E8},
{0x1D6E9,Lu,0x1D6E9,0x1D6E9,0x1D6E9},
{0x1D6EA,Lu,0x1D6EA,0x1D6EA,0x1D6EA},
{0x1D6EB,Lu,0x1D6EB,0x1D6EB,0x1D6EB},
{0x1D6EC,Lu,0x1D6EC,0x1D6EC,0x1D6EC},
{0x1D6ED,Lu,0x1D6ED,0x1D6ED,0x1D6ED},
{0x1D6EE,Lu,0x1D6EE,0x1D6EE,0x1D6EE},
{0x1D6EF,Lu,0x1D6EF,0x1D6EF,0x1D6EF},
{0x1D6F0,Lu,0x1D6F0,0x1D6F0,0x1D6F0},
{0x1D6F1,Lu,0x1D6F1,0x1D6F1,0x1D6F1},
{0x1D6F2,Lu,0x1D6F2,0x1D6F2,0x1D6F2},
{0x1D6F3,Lu,0x1D6F3,0x1D6F3,0x1D6F3},
{0x1D6F4,Lu,0x1D6F4,0x1D6F4,0x1D6F4},
{0x1D6F5,Lu,0x1D6F5,0x1D6F5,0x1D6F5},
{0x1D6F6,Lu,0x1D6F6,0x1D6F6,0x1D6F6},
{0x1D6F7,Lu,0x1D6F7,0x1D6F7,0x1D6F7},
{0x1D6F8,Lu,0x1D6F8,0x1D6F8,0x1D6F8},
{0x1D6F9,Lu,0x1D6F9,0x1D6F9,0x1D6F9},
{0x1D6FA,Lu,0x1D6FA,0x1D6FA,0x1D6FA},
{0x1D6FB,Sm,0x1D6FB,0x1D6FB,0x1D6FB},
{0x1D6FC,Ll,0x1D6FC,0x1D6FC,0x1D6FC},
{0x1D6FD,Ll,0x1D6FD,0x1D6FD,0x1D6FD},
{0x1D6FE,Ll,0x1D6FE,0x1D6FE,0x1D6FE},
{0x1D6FF,Ll,0x1D6FF,0x1D6FF,0x1D6FF},
{0x1D700,Ll,0x1D700,0x1D700,0x1D700},
{0x1D701,Ll,0x1D701,0x1D701,0x1D701},
{0x1D702,Ll,0x1D702,0x1D702,0x1D702},
{0x1D703,Ll,0x1D703,0x1D703,0x1D703},
{0x1D704,Ll,0x1D704,0x1D704,0x1D704},
{0x1D705,Ll,0x1D705,0x1D705,0x1D705},
{0x1D706,Ll,0x1D706,0x1D706,0x1D706},
{0x1D707,Ll,0x1D707,0x1D707,0x1D707},
{0x1D708,Ll,0x1D708,0x1D708,0x1D708},
{0x1D709,Ll,0x1D709,0x1D709,0x1D709},
{0x1D70A,Ll,0x1D70A,0x1D70A,0x1D70A},
{0x1D70B,Ll,0x1D70B,0x1D70B,0x1D70B},
{0x1D70C,Ll,0x1D70C,0x1D70C,0x1D70C},
{0x1D70D,Ll,0x1D70D,0x1D70D,0x1D70D},
{0x1D70E,Ll,0x1D70E,0x1D70E,0x1D70E},
{0x1D70F,Ll,0x1D70F,0x1D70F,0x1D70F},
{0x1D710,Ll,0x1D710,0x1D710,0x1D710},
{0x1D711,Ll,0x1D711,0x1D711,0x1D711},
{0x1D712,Ll,0x1D712,0x1D712,0x1D712},
{0x1D713,Ll,0x1D713,0x1D713,0x1D713},
{0x1D714,Ll,0x1D714,0x1D714,0x1D714},
{0x1D715,Sm,0x1D715,0x1D715,0x1D715},
{0x1D716,Ll,0x1D716,0x1D716,0x1D716},
{0x1D717,Ll,0x1D717,0x1D717,0x1D717},
{0x1D718,Ll,0x1D718,0x1D718,0x1D718},
{0x1D719,Ll,0x1D719,0x1D719,0x1D719},
{0x1D71A,Ll,0x1D71A,0x1D71A,0x1D71A},
{0x1D71B,Ll,0x1D71B,0x1D71B,0x1D71B},
{0x1D71C,Lu,0x1D71C,0x1D71C,0x1D71C},
{0x1D71D,Lu,0x1D71D,0x1D71D,0x1D71D},
{0x1D71E,Lu,0x1D71E,0x1D71E,0x1D71E},
{0x1D71F,Lu,0x1D71F,0x1D71F,0x1D71F},
{0x1D720,Lu,0x1D720,0x1D720,0x1D720},
{0x1D721,Lu,0x1D721,0x1D721,0x1D721},
{0x1D722,Lu,0x1D722,0x1D722,0x1D722},
{0x1D723,Lu,0x1D723,0x1D723,0x1D723},
{0x1D724,Lu,0x1D724,0x1D724,0x1D724},
{0x1D725,Lu,0x1D725,0x1D725,0x1D725},
{0x1D726,Lu,0x1D726,0x1D726,0x1D726},
{0x1D727,Lu,0x1D727,0x1D727,0x1D727},
{0x1D728,Lu,0x1D728,0x1D728,0x1D728},
{0x1D729,Lu,0x1D729,0x1D729,0x1D729},
{0x1D72A,Lu,0x1D72A,0x1D72A,0x1D72A},
{0x1D72B,Lu,0x1D72B,0x1D72B,0x1D72B},
{0x1D72C,Lu,0x1D72C,0x1D72C,0x1D72C},
{0x1D72D,Lu,0x1D72D,0x1D72D,0x1D72D},
{0x1D72E,Lu,0x1D72E,0x1D72E,0x1D72E},
{0x1D72F,Lu,0x1D72F,0x1D72F,0x1D72F},
{0x1D730,Lu,0x1D730,0x1D730,0x1D730},
{0x1D731,Lu,0x1D731,0x1D731,0x1D731},
{0x1D732,Lu,0x1D732,0x1D732,0x1D732},
{0x1D733,Lu,0x1D733,0x1D733,0x1D733},
{0x1D734,Lu,0x1D734,0x1D734,0x1D734},
{0x1D735,Sm,0x1D735,0x1D735,0x1D735},
{0x1D736,Ll,0x1D736,0x1D736,0x1D736},
{0x1D737,Ll,0x1D737,0x1D737,0x1D737},
{0x1D738,Ll,0x1D738,0x1D738,0x1D738},
{0x1D739,Ll,0x1D739,0x1D739,0x1D739},
{0x1D73A,Ll,0x1D73A,0x1D73A,0x1D73A},
{0x1D73B,Ll,0x1D73B,0x1D73B,0x1D73B},
{0x1D73C,Ll,0x1D73C,0x1D73C,0x1D73C},
{0x1D73D,Ll,0x1D73D,0x1D73D,0x1D73D},
{0x1D73E,Ll,0x1D73E,0x1D73E,0x1D73E},
{0x1D73F,Ll,0x1D73F,0x1D73F,0x1D73F},
{0x1D740,Ll,0x1D740,0x1D740,0x1D740},
{0x1D741,Ll,0x1D741,0x1D741,0x1D741},
{0x1D742,Ll,0x1D742,0x1D742,0x1D742},
{0x1D743,Ll,0x1D743,0x1D743,0x1D743},
{0x1D744,Ll,0x1D744,0x1D744,0x1D744},
{0x1D745,Ll,0x1D745,0x1D745,0x1D745},
{0x1D746,Ll,0x1D746,0x1D746,0x1D746},
{0x1D747,Ll,0x1D747,0x1D747,0x1D747},
{0x1D748,Ll,0x1D748,0x1D748,0x1D748},
{0x1D749,Ll,0x1D749,0x1D749,0x1D749},
{0x1D74A,Ll,0x1D74A,0x1D74A,0x1D74A},
{0x1D74B,Ll,0x1D74B,0x1D74B,0x1D74B},
{0x1D74C,Ll,0x1D74C,0x1D74C,0x1D74C},
{0x1D74D,Ll,0x1D74D,0x1D74D,0x1D74D},
{0x1D74E,Ll,0x1D74E,0x1D74E,0x1D74E},
{0x1D74F,Sm,0x1D74F,0x1D74F,0x1D74F},
{0x1D750,Ll,0x1D750,0x1D750,0x1D750},
{0x1D751,Ll,0x1D751,0x1D751,0x1D751},
{0x1D752,Ll,0x1D752,0x1D752,0x1D752},
{0x1D753,Ll,0x1D753,0x1D753,0x1D753},
{0x1D754,Ll,0x1D754,0x1D754,0x1D754},
{0x1D755,Ll,0x1D755,0x1D755,0x1D755},
{0x1D756,Lu,0x1D756,0x1D756,0x1D756},
{0x1D757,Lu,0x1D757,0x1D757,0x1D757},
{0x1D758,Lu,0x1D758,0x1D758,0x1D758},
{0x1D759,Lu,0x1D759,0x1D759,0x1D759},
{0x1D75A,Lu,0x1D75A,0x1D75A,0x1D75A},
{0x1D75B,Lu,0x1D75B,0x1D75B,0x1D75B},
{0x1D75C,Lu,0x1D75C,0x1D75C,0x1D75C},
{0x1D75D,Lu,0x1D75D,0x1D75D,0x1D75D},
{0x1D75E,Lu,0x1D75E,0x1D75E,0x1D75E},
{0x1D75F,Lu,0x1D75F,0x1D75F,0x1D75F},
{0x1D760,Lu,0x1D760,0x1D760,0x1D760},
{0x1D761,Lu,0x1D761,0x1D761,0x1D761},
{0x1D762,Lu,0x1D762,0x1D762,0x1D762},
{0x1D763,Lu,0x1D763,0x1D763,0x1D763},
{0x1D764,Lu,0x1D764,0x1D764,0x1D764},
{0x1D765,Lu,0x1D765,0x1D765,0x1D765},
{0x1D766,Lu,0x1D766,0x1D766,0x1D766},
{0x1D767,Lu,0x1D767,0x1D767,0x1D767},
{0x1D768,Lu,0x1D768,0x1D768,0x1D768},
{0x1D769,Lu,0x1D769,0x1D769,0x1D769},
{0x1D76A,Lu,0x1D76A,0x1D76A,0x1D76A},
{0x1D76B,Lu,0x1D76B,0x1D76B,0x1D76B},
{0x1D76C,Lu,0x1D76C,0x1D76C,0x1D76C},
{0x1D76D,Lu,0x1D76D,0x1D76D,0x1D76D},
{0x1D76E,Lu,0x1D76E,0x1D76E,0x1D76E},
{0x1D76F,Sm,0x1D76F,0x1D76F,0x1D76F},
{0x1D770,Ll,0x1D770,0x1D770,0x1D770},
{0x1D771,Ll,0x1D771,0x1D771,0x1D771},
{0x1D772,Ll,0x1D772,0x1D772,0x1D772},
{0x1D773,Ll,0x1D773,0x1D773,0x1D773},
{0x1D774,Ll,0x1D774,0x1D774,0x1D774},
{0x1D775,Ll,0x1D775,0x1D775,0x1D775},
{0x1D776,Ll,0x1D776,0x1D776,0x1D776},
{0x1D777,Ll,0x1D777,0x1D777,0x1D777},
{0x1D778,Ll,0x1D778,0x1D778,0x1D778},
{0x1D779,Ll,0x1D779,0x1D779,0x1D779},
{0x1D77A,Ll,0x1D77A,0x1D77A,0x1D77A},
{0x1D77B,Ll,0x1D77B,0x1D77B,0x1D77B},
{0x1D77C,Ll,0x1D77C,0x1D77C,0x1D77C},
{0x1D77D,Ll,0x1D77D,0x1D77D,0x1D77D},
{0x1D77E,Ll,0x1D77E,0x1D77E,0x1D77E},
{0x1D77F,Ll,0x1D77F,0x1D77F,0x1D77F},
{0x1D780,Ll,0x1D780,0x1D780,0x1D780},
{0x1D781,Ll,0x1D781,0x1D781,0x1D781},
{0x1D782,Ll,0x1D782,0x1D782,0x1D782},
{0x1D783,Ll,0x1D783,0x1D783,0x1D783},
{0x1D784,Ll,0x1D784,0x1D784,0x1D784},
{0x1D785,Ll,0x1D785,0x1D785,0x1D785},
{0x1D786,Ll,0x1D786,0x1D786,0x1D786},
{0x1D787,Ll,0x1D787,0x1D787,0x1D787},
{0x1D788,Ll,0x1D788,0x1D788,0x1D788},
{0x1D789,Sm,0x1D789,0x1D789,0x1D789},
{0x1D78A,Ll,0x1D78A,0x1D78A,0x1D78A},
{0x1D78B,Ll,0x1D78B,0x1D78B,0x1D78B},
{0x1D78C,Ll,0x1D78C,0x1D78C,0x1D78C},
{0x1D78D,Ll,0x1D78D,0x1D78D,0x1D78D},
{0x1D78E,Ll,0x1D78E,0x1D78E,0x1D78E},
{0x1D78F,Ll,0x1D78F,0x1D78F,0x1D78F},
{0x1D790,Lu,0x1D790,0x1D790,0x1D790},
{0x1D791,Lu,0x1D791,0x1D791,0x1D791},
{0x1D792,Lu,0x1D792,0x1D792,0x1D792},
{0x1D793,Lu,0x1D793,0x1D793,0x1D793},
{0x1D794,Lu,0x1D794,0x1D794,0x1D794},
{0x1D795,Lu,0x1D795,0x1D795,0x1D795},
{0x1D796,Lu,0x1D796,0x1D796,0x1D796},
{0x1D797,Lu,0x1D797,0x1D797,0x1D797},
{0x1D798,Lu,0x1D798,0x1D798,0x1D798},
{0x1D799,Lu,0x1D799,0x1D799,0x1D799},
{0x1D79A,Lu,0x1D79A,0x1D79A,0x1D79A},
{0x1D79B,Lu,0x1D79B,0x1D79B,0x1D79B},
{0x1D79C,Lu,0x1D79C,0x1D79C,0x1D79C},
{0x1D79D,Lu,0x1D79D,0x1D79D,0x1D79D},
{0x1D79E,Lu,0x1D79E,0x1D79E,0x1D79E},
{0x1D79F,Lu,0x1D79F,0x1D79F,0x1D79F},
{0x1D7A0,Lu,0x1D7A0,0x1D7A0,0x1D7A0},
{0x1D7A1,Lu,0x1D7A1,0x1D7A1,0x1D7A1},
{0x1D7A2,Lu,0x1D7A2,0x1D7A2,0x1D7A2},
{0x1D7A3,Lu,0x1D7A3,0x1D7A3,0x1D7A3},
{0x1D7A4,Lu,0x1D7A4,0x1D7A4,0x1D7A4},
{0x1D7A5,Lu,0x1D7A5,0x1D7A5,0x1D7A5},
{0x1D7A6,Lu,0x1D7A6,0x1D7A6,0x1D7A6},
{0x1D7A7,Lu,0x1D7A7,0x1D7A7,0x1D7A7},
{0x1D7A8,Lu,0x1D7A8,0x1D7A8,0x1D7A8},
{0x1D7A9,Sm,0x1D7A9,0x1D7A9,0x1D7A9},
{0x1D7AA,Ll,0x1D7AA,0x1D7AA,0x1D7AA},
{0x1D7AB,Ll,0x1D7AB,0x1D7AB,0x1D7AB},
{0x1D7AC,Ll,0x1D7AC,0x1D7AC,0x1D7AC},
{0x1D7AD,Ll,0x1D7AD,0x1D7AD,0x1D7AD},
{0x1D7AE,Ll,0x1D7AE,0x1D7AE,0x1D7AE},
{0x1D7AF,Ll,0x1D7AF,0x1D7AF,0x1D7AF},
{0x1D7B0,Ll,0x1D7B0,0x1D7B0,0x1D7B0},
{0x1D7B1,Ll,0x1D7B1,0x1D7B1,0x1D7B1},
{0x1D7B2,Ll,0x1D7B2,0x1D7B2,0x1D7B2},
{0x1D7B3,Ll,0x1D7B3,0x1D7B3,0x1D7B3},
{0x1D7B4,Ll,0x1D7B4,0x1D7B4,0x1D7B4},
{0x1D7B5,Ll,0x1D7B5,0x1D7B5,0x1D7B5},
{0x1D7B6,Ll,0x1D7B6,0x1D7B6,0x1D7B6},
{0x1D7B7,Ll,0x1D7B7,0x1D7B7,0x1D7B7},
{0x1D7B8,Ll,0x1D7B8,0x1D7B8,0x1D7B8},
{0x1D7B9,Ll,0x1D7B9,0x1D7B9,0x1D7B9},
{0x1D7BA,Ll,0x1D7BA,0x1D7BA,0x1D7BA},
{0x1D7BB,Ll,0x1D7BB,0x1D7BB,0x1D7BB},
{0x1D7BC,Ll,0x1D7BC,0x1D7BC,0x1D7BC},
{0x1D7BD,Ll,0x1D7BD,0x1D7BD,0x1D7BD},
{0x1D7BE,Ll,0x1D7BE,0x1D7BE,0x1D7BE},
{0x1D7BF,Ll,0x1D7BF,0x1D7BF,0x1D7BF},
{0x1D7C0,Ll,0x1D7C0,0x1D7C0,0x1D7C0},
{0x1D7C1,Ll,0x1D7C1,0x1D7C1,0x1D7C1},
{0x1D7C2,Ll,0x1D7C2,0x1D7C2,0x1D7C2},
{0x1D7C3,Sm,0x1D7C3,0x1D7C3,0x1D7C3},
{0x1D7C4,Ll,0x1D7C4,0x1D7C4,0x1D7C4},
{0x1D7C5,Ll,0x1D7C5,0x1D7C5,0x1D7C5},
{0x1D7C6,Ll,0x1D7C6,0x1D7C6,0x1D7C6},
{0x1D7C7,Ll,0x1D7C7,0x1D7C7,0x1D7C7},
{0x1D7C8,Ll,0x1D7C8,0x1D7C8,0x1D7C8},
{0x1D7C9,Ll,0x1D7C9,0x1D7C9,0x1D7C9},
{0x1D7CA,Lu,0x1D7CA,0x1D7CA,0x1D7CA},
{0x1D7CB,Ll,0x1D7CB,0x1D7CB,0x1D7CB},
{0x1D7CE,Nd,0x1D7CE,0x1D7CE,0x1D7CE},
{0x1D7CF,Nd,0x1D7CF,0x1D7CF,0x1D7CF},
{0x1D7D0,Nd,0x1D7D0,0x1D7D0,0x1D7D0},
{0x1D7D1,Nd,0x1D7D1,0x1D7D1,0x1D7D1},
{0x1D7D2,Nd,0x1D7D2,0x1D7D2,0x1D7D2},
{0x1D7D3,Nd,0x1D7D3,0x1D7D3,0x1D7D3},
{0x1D7D4,Nd,0x1D7D4,0x1D7D4,0x1D7D4},
{0x1D7D5,Nd,0x1D7D5,0x1D7D5,0x1D7D5},
{0x1D7D6,Nd,0x1D7D6,0x1D7D6,0x1D7D6},
{0x1D7D7,Nd,0x1D7D7,0x1D7D7,0x1D7D7},
{0x1D7D8,Nd,0x1D7D8,0x1D7D8,0x1D7D8},
{0x1D7D9,Nd,0x1D7D9,0x1D7D9,0x1D7D9},
{0x1D7DA,Nd,0x1D7DA,0x1D7DA,0x1D7DA},
{0x1D7DB,Nd,0x1D7DB,0x1D7DB,0x1D7DB},
{0x1D7DC,Nd,0x1D7DC,0x1D7DC,0x1D7DC},
{0x1D7DD,Nd,0x1D7DD,0x1D7DD,0x1D7DD},
{0x1D7DE,Nd,0x1D7DE,0x1D7DE,0x1D7DE},
{0x1D7DF,Nd,0x1D7DF,0x1D7DF,0x1D7DF},
{0x1D7E0,Nd,0x1D7E0,0x1D7E0,0x1D7E0},
{0x1D7E1,Nd,0x1D7E1,0x1D7E1,0x1D7E1},
{0x1D7E2,Nd,0x1D7E2,0x1D7E2,0x1D7E2},
{0x1D7E3,Nd,0x1D7E3,0x1D7E3,0x1D7E3},
{0x1D7E4,Nd,0x1D7E4,0x1D7E4,0x1D7E4},
{0x1D7E5,Nd,0x1D7E5,0x1D7E5,0x1D7E5},
{0x1D7E6,Nd,0x1D7E6,0x1D7E6,0x1D7E6},
{0x1D7E7,Nd,0x1D7E7,0x1D7E7,0x1D7E7},
{0x1D7E8,Nd,0x1D7E8,0x1D7E8,0x1D7E8},
{0x1D7E9,Nd,0x1D7E9,0x1D7E9,0x1D7E9},
{0x1D7EA,Nd,0x1D7EA,0x1D7EA,0x1D7EA},
{0x1D7EB,Nd,0x1D7EB,0x1D7EB,0x1D7EB},
{0x1D7EC,Nd,0x1D7EC,0x1D7EC,0x1D7EC},
{0x1D7ED,Nd,0x1D7ED,0x1D7ED,0x1D7ED},
{0x1D7EE,Nd,0x1D7EE,0x1D7EE,0x1D7EE},
{0x1D7EF,Nd,0x1D7EF,0x1D7EF,0x1D7EF},
{0x1D7F0,Nd,0x1D7F0,0x1D7F0,0x1D7F0},
{0x1D7F1,Nd,0x1D7F1,0x1D7F1,0x1D7F1},
{0x1D7F2,Nd,0x1D7F2,0x1D7F2,0x1D7F2},
{0x1D7F3,Nd,0x1D7F3,0x1D7F3,0x1D7F3},
{0x1D7F4,Nd,0x1D7F4,0x1D7F4,0x1D7F4},
{0x1D7F5,Nd,0x1D7F5,0x1D7F5,0x1D7F5},
{0x1D7F6,Nd,0x1D7F6,0x1D7F6,0x1D7F6},
{0x1D7F7,Nd,0x1D7F7,0x1D7F7,0x1D7F7},
{0x1D7F8,Nd,0x1D7F8,0x1D7F8,0x1D7F8},
{0x1D7F9,Nd,0x1D7F9,0x1D7F9,0x1D7F9},
{0x1D7FA,Nd,0x1D7FA,0x1D7FA,0x1D7FA},
{0x1D7FB,Nd,0x1D7FB,0x1D7FB,0x1D7FB},
{0x1D7FC,Nd,0x1D7FC,0x1D7FC,0x1D7FC},
{0x1D7FD,Nd,0x1D7FD,0x1D7FD,0x1D7FD},
{0x1D7FE,Nd,0x1D7FE,0x1D7FE,0x1D7FE},
{0x1D7FF,Nd,0x1D7FF,0x1D7FF,0x1D7FF},
{0x20000,Lo,0x20000,0x20000,0x20000},
{0x2A6D6,Lo,0x2A6D6,0x2A6D6,0x2A6D6},
{0x2F800,Lo,0x2F800,0x2F800,0x2F800},
{0x2F801,Lo,0x2F801,0x2F801,0x2F801},
{0x2F802,Lo,0x2F802,0x2F802,0x2F802},
{0x2F803,Lo,0x2F803,0x2F803,0x2F803},
{0x2F804,Lo,0x2F804,0x2F804,0x2F804},
{0x2F805,Lo,0x2F805,0x2F805,0x2F805},
{0x2F806,Lo,0x2F806,0x2F806,0x2F806},
{0x2F807,Lo,0x2F807,0x2F807,0x2F807},
{0x2F808,Lo,0x2F808,0x2F808,0x2F808},
{0x2F809,Lo,0x2F809,0x2F809,0x2F809},
{0x2F80A,Lo,0x2F80A,0x2F80A,0x2F80A},
{0x2F80B,Lo,0x2F80B,0x2F80B,0x2F80B},
{0x2F80C,Lo,0x2F80C,0x2F80C,0x2F80C},
{0x2F80D,Lo,0x2F80D,0x2F80D,0x2F80D},
{0x2F80E,Lo,0x2F80E,0x2F80E,0x2F80E},
{0x2F80F,Lo,0x2F80F,0x2F80F,0x2F80F},
{0x2F810,Lo,0x2F810,0x2F810,0x2F810},
{0x2F811,Lo,0x2F811,0x2F811,0x2F811},
{0x2F812,Lo,0x2F812,0x2F812,0x2F812},
{0x2F813,Lo,0x2F813,0x2F813,0x2F813},
{0x2F814,Lo,0x2F814,0x2F814,0x2F814},
{0x2F815,Lo,0x2F815,0x2F815,0x2F815},
{0x2F816,Lo,0x2F816,0x2F816,0x2F816},
{0x2F817,Lo,0x2F817,0x2F817,0x2F817},
{0x2F818,Lo,0x2F818,0x2F818,0x2F818},
{0x2F819,Lo,0x2F819,0x2F819,0x2F819},
{0x2F81A,Lo,0x2F81A,0x2F81A,0x2F81A},
{0x2F81B,Lo,0x2F81B,0x2F81B,0x2F81B},
{0x2F81C,Lo,0x2F81C,0x2F81C,0x2F81C},
{0x2F81D,Lo,0x2F81D,0x2F81D,0x2F81D},
{0x2F81E,Lo,0x2F81E,0x2F81E,0x2F81E},
{0x2F81F,Lo,0x2F81F,0x2F81F,0x2F81F},
{0x2F820,Lo,0x2F820,0x2F820,0x2F820},
{0x2F821,Lo,0x2F821,0x2F821,0x2F821},
{0x2F822,Lo,0x2F822,0x2F822,0x2F822},
{0x2F823,Lo,0x2F823,0x2F823,0x2F823},
{0x2F824,Lo,0x2F824,0x2F824,0x2F824},
{0x2F825,Lo,0x2F825,0x2F825,0x2F825},
{0x2F826,Lo,0x2F826,0x2F826,0x2F826},
{0x2F827,Lo,0x2F827,0x2F827,0x2F827},
{0x2F828,Lo,0x2F828,0x2F828,0x2F828},
{0x2F829,Lo,0x2F829,0x2F829,0x2F829},
{0x2F82A,Lo,0x2F82A,0x2F82A,0x2F82A},
{0x2F82B,Lo,0x2F82B,0x2F82B,0x2F82B},
{0x2F82C,Lo,0x2F82C,0x2F82C,0x2F82C},
{0x2F82D,Lo,0x2F82D,0x2F82D,0x2F82D},
{0x2F82E,Lo,0x2F82E,0x2F82E,0x2F82E},
{0x2F82F,Lo,0x2F82F,0x2F82F,0x2F82F},
{0x2F830,Lo,0x2F830,0x2F830,0x2F830},
{0x2F831,Lo,0x2F831,0x2F831,0x2F831},
{0x2F832,Lo,0x2F832,0x2F832,0x2F832},
{0x2F833,Lo,0x2F833,0x2F833,0x2F833},
{0x2F834,Lo,0x2F834,0x2F834,0x2F834},
{0x2F835,Lo,0x2F835,0x2F835,0x2F835},
{0x2F836,Lo,0x2F836,0x2F836,0x2F836},
{0x2F837,Lo,0x2F837,0x2F837,0x2F837},
{0x2F838,Lo,0x2F838,0x2F838,0x2F838},
{0x2F839,Lo,0x2F839,0x2F839,0x2F839},
{0x2F83A,Lo,0x2F83A,0x2F83A,0x2F83A},
{0x2F83B,Lo,0x2F83B,0x2F83B,0x2F83B},
{0x2F83C,Lo,0x2F83C,0x2F83C,0x2F83C},
{0x2F83D,Lo,0x2F83D,0x2F83D,0x2F83D},
{0x2F83E,Lo,0x2F83E,0x2F83E,0x2F83E},
{0x2F83F,Lo,0x2F83F,0x2F83F,0x2F83F},
{0x2F840,Lo,0x2F840,0x2F840,0x2F840},
{0x2F841,Lo,0x2F841,0x2F841,0x2F841},
{0x2F842,Lo,0x2F842,0x2F842,0x2F842},
{0x2F843,Lo,0x2F843,0x2F843,0x2F843},
{0x2F844,Lo,0x2F844,0x2F844,0x2F844},
{0x2F845,Lo,0x2F845,0x2F845,0x2F845},
{0x2F846,Lo,0x2F846,0x2F846,0x2F846},
{0x2F847,Lo,0x2F847,0x2F847,0x2F847},
{0x2F848,Lo,0x2F848,0x2F848,0x2F848},
{0x2F849,Lo,0x2F849,0x2F849,0x2F849},
{0x2F84A,Lo,0x2F84A,0x2F84A,0x2F84A},
{0x2F84B,Lo,0x2F84B,0x2F84B,0x2F84B},
{0x2F84C,Lo,0x2F84C,0x2F84C,0x2F84C},
{0x2F84D,Lo,0x2F84D,0x2F84D,0x2F84D},
{0x2F84E,Lo,0x2F84E,0x2F84E,0x2F84E},
{0x2F84F,Lo,0x2F84F,0x2F84F,0x2F84F},
{0x2F850,Lo,0x2F850,0x2F850,0x2F850},
{0x2F851,Lo,0x2F851,0x2F851,0x2F851},
{0x2F852,Lo,0x2F852,0x2F852,0x2F852},
{0x2F853,Lo,0x2F853,0x2F853,0x2F853},
{0x2F854,Lo,0x2F854,0x2F854,0x2F854},
{0x2F855,Lo,0x2F855,0x2F855,0x2F855},
{0x2F856,Lo,0x2F856,0x2F856,0x2F856},
{0x2F857,Lo,0x2F857,0x2F857,0x2F857},
{0x2F858,Lo,0x2F858,0x2F858,0x2F858},
{0x2F859,Lo,0x2F859,0x2F859,0x2F859},
{0x2F85A,Lo,0x2F85A,0x2F85A,0x2F85A},
{0x2F85B,Lo,0x2F85B,0x2F85B,0x2F85B},
{0x2F85C,Lo,0x2F85C,0x2F85C,0x2F85C},
{0x2F85D,Lo,0x2F85D,0x2F85D,0x2F85D},
{0x2F85E,Lo,0x2F85E,0x2F85E,0x2F85E},
{0x2F85F,Lo,0x2F85F,0x2F85F,0x2F85F},
{0x2F860,Lo,0x2F860,0x2F860,0x2F860},
{0x2F861,Lo,0x2F861,0x2F861,0x2F861},
{0x2F862,Lo,0x2F862,0x2F862,0x2F862},
{0x2F863,Lo,0x2F863,0x2F863,0x2F863},
{0x2F864,Lo,0x2F864,0x2F864,0x2F864},
{0x2F865,Lo,0x2F865,0x2F865,0x2F865},
{0x2F866,Lo,0x2F866,0x2F866,0x2F866},
{0x2F867,Lo,0x2F867,0x2F867,0x2F867},
{0x2F868,Lo,0x2F868,0x2F868,0x2F868},
{0x2F869,Lo,0x2F869,0x2F869,0x2F869},
{0x2F86A,Lo,0x2F86A,0x2F86A,0x2F86A},
{0x2F86B,Lo,0x2F86B,0x2F86B,0x2F86B},
{0x2F86C,Lo,0x2F86C,0x2F86C,0x2F86C},
{0x2F86D,Lo,0x2F86D,0x2F86D,0x2F86D},
{0x2F86E,Lo,0x2F86E,0x2F86E,0x2F86E},
{0x2F86F,Lo,0x2F86F,0x2F86F,0x2F86F},
{0x2F870,Lo,0x2F870,0x2F870,0x2F870},
{0x2F871,Lo,0x2F871,0x2F871,0x2F871},
{0x2F872,Lo,0x2F872,0x2F872,0x2F872},
{0x2F873,Lo,0x2F873,0x2F873,0x2F873},
{0x2F874,Lo,0x2F874,0x2F874,0x2F874},
{0x2F875,Lo,0x2F875,0x2F875,0x2F875},
{0x2F876,Lo,0x2F876,0x2F876,0x2F876},
{0x2F877,Lo,0x2F877,0x2F877,0x2F877},
{0x2F878,Lo,0x2F878,0x2F878,0x2F878},
{0x2F879,Lo,0x2F879,0x2F879,0x2F879},
{0x2F87A,Lo,0x2F87A,0x2F87A,0x2F87A},
{0x2F87B,Lo,0x2F87B,0x2F87B,0x2F87B},
{0x2F87C,Lo,0x2F87C,0x2F87C,0x2F87C},
{0x2F87D,Lo,0x2F87D,0x2F87D,0x2F87D},
{0x2F87E,Lo,0x2F87E,0x2F87E,0x2F87E},
{0x2F87F,Lo,0x2F87F,0x2F87F,0x2F87F},
{0x2F880,Lo,0x2F880,0x2F880,0x2F880},
{0x2F881,Lo,0x2F881,0x2F881,0x2F881},
{0x2F882,Lo,0x2F882,0x2F882,0x2F882},
{0x2F883,Lo,0x2F883,0x2F883,0x2F883},
{0x2F884,Lo,0x2F884,0x2F884,0x2F884},
{0x2F885,Lo,0x2F885,0x2F885,0x2F885},
{0x2F886,Lo,0x2F886,0x2F886,0x2F886},
{0x2F887,Lo,0x2F887,0x2F887,0x2F887},
{0x2F888,Lo,0x2F888,0x2F888,0x2F888},
{0x2F889,Lo,0x2F889,0x2F889,0x2F889},
{0x2F88A,Lo,0x2F88A,0x2F88A,0x2F88A},
{0x2F88B,Lo,0x2F88B,0x2F88B,0x2F88B},
{0x2F88C,Lo,0x2F88C,0x2F88C,0x2F88C},
{0x2F88D,Lo,0x2F88D,0x2F88D,0x2F88D},
{0x2F88E,Lo,0x2F88E,0x2F88E,0x2F88E},
{0x2F88F,Lo,0x2F88F,0x2F88F,0x2F88F},
{0x2F890,Lo,0x2F890,0x2F890,0x2F890},
{0x2F891,Lo,0x2F891,0x2F891,0x2F891},
{0x2F892,Lo,0x2F892,0x2F892,0x2F892},
{0x2F893,Lo,0x2F893,0x2F893,0x2F893},
{0x2F894,Lo,0x2F894,0x2F894,0x2F894},
{0x2F895,Lo,0x2F895,0x2F895,0x2F895},
{0x2F896,Lo,0x2F896,0x2F896,0x2F896},
{0x2F897,Lo,0x2F897,0x2F897,0x2F897},
{0x2F898,Lo,0x2F898,0x2F898,0x2F898},
{0x2F899,Lo,0x2F899,0x2F899,0x2F899},
{0x2F89A,Lo,0x2F89A,0x2F89A,0x2F89A},
{0x2F89B,Lo,0x2F89B,0x2F89B,0x2F89B},
{0x2F89C,Lo,0x2F89C,0x2F89C,0x2F89C},
{0x2F89D,Lo,0x2F89D,0x2F89D,0x2F89D},
{0x2F89E,Lo,0x2F89E,0x2F89E,0x2F89E},
{0x2F89F,Lo,0x2F89F,0x2F89F,0x2F89F},
{0x2F8A0,Lo,0x2F8A0,0x2F8A0,0x2F8A0},
{0x2F8A1,Lo,0x2F8A1,0x2F8A1,0x2F8A1},
{0x2F8A2,Lo,0x2F8A2,0x2F8A2,0x2F8A2},
{0x2F8A3,Lo,0x2F8A3,0x2F8A3,0x2F8A3},
{0x2F8A4,Lo,0x2F8A4,0x2F8A4,0x2F8A4},
{0x2F8A5,Lo,0x2F8A5,0x2F8A5,0x2F8A5},
{0x2F8A6,Lo,0x2F8A6,0x2F8A6,0x2F8A6},
{0x2F8A7,Lo,0x2F8A7,0x2F8A7,0x2F8A7},
{0x2F8A8,Lo,0x2F8A8,0x2F8A8,0x2F8A8},
{0x2F8A9,Lo,0x2F8A9,0x2F8A9,0x2F8A9},
{0x2F8AA,Lo,0x2F8AA,0x2F8AA,0x2F8AA},
{0x2F8AB,Lo,0x2F8AB,0x2F8AB,0x2F8AB},
{0x2F8AC,Lo,0x2F8AC,0x2F8AC,0x2F8AC},
{0x2F8AD,Lo,0x2F8AD,0x2F8AD,0x2F8AD},
{0x2F8AE,Lo,0x2F8AE,0x2F8AE,0x2F8AE},
{0x2F8AF,Lo,0x2F8AF,0x2F8AF,0x2F8AF},
{0x2F8B0,Lo,0x2F8B0,0x2F8B0,0x2F8B0},
{0x2F8B1,Lo,0x2F8B1,0x2F8B1,0x2F8B1},
{0x2F8B2,Lo,0x2F8B2,0x2F8B2,0x2F8B2},
{0x2F8B3,Lo,0x2F8B3,0x2F8B3,0x2F8B3},
{0x2F8B4,Lo,0x2F8B4,0x2F8B4,0x2F8B4},
{0x2F8B5,Lo,0x2F8B5,0x2F8B5,0x2F8B5},
{0x2F8B6,Lo,0x2F8B6,0x2F8B6,0x2F8B6},
{0x2F8B7,Lo,0x2F8B7,0x2F8B7,0x2F8B7},
{0x2F8B8,Lo,0x2F8B8,0x2F8B8,0x2F8B8},
{0x2F8B9,Lo,0x2F8B9,0x2F8B9,0x2F8B9},
{0x2F8BA,Lo,0x2F8BA,0x2F8BA,0x2F8BA},
{0x2F8BB,Lo,0x2F8BB,0x2F8BB,0x2F8BB},
{0x2F8BC,Lo,0x2F8BC,0x2F8BC,0x2F8BC},
{0x2F8BD,Lo,0x2F8BD,0x2F8BD,0x2F8BD},
{0x2F8BE,Lo,0x2F8BE,0x2F8BE,0x2F8BE},
{0x2F8BF,Lo,0x2F8BF,0x2F8BF,0x2F8BF},
{0x2F8C0,Lo,0x2F8C0,0x2F8C0,0x2F8C0},
{0x2F8C1,Lo,0x2F8C1,0x2F8C1,0x2F8C1},
{0x2F8C2,Lo,0x2F8C2,0x2F8C2,0x2F8C2},
{0x2F8C3,Lo,0x2F8C3,0x2F8C3,0x2F8C3},
{0x2F8C4,Lo,0x2F8C4,0x2F8C4,0x2F8C4},
{0x2F8C5,Lo,0x2F8C5,0x2F8C5,0x2F8C5},
{0x2F8C6,Lo,0x2F8C6,0x2F8C6,0x2F8C6},
{0x2F8C7,Lo,0x2F8C7,0x2F8C7,0x2F8C7},
{0x2F8C8,Lo,0x2F8C8,0x2F8C8,0x2F8C8},
{0x2F8C9,Lo,0x2F8C9,0x2F8C9,0x2F8C9},
{0x2F8CA,Lo,0x2F8CA,0x2F8CA,0x2F8CA},
{0x2F8CB,Lo,0x2F8CB,0x2F8CB,0x2F8CB},
{0x2F8CC,Lo,0x2F8CC,0x2F8CC,0x2F8CC},
{0x2F8CD,Lo,0x2F8CD,0x2F8CD,0x2F8CD},
{0x2F8CE,Lo,0x2F8CE,0x2F8CE,0x2F8CE},
{0x2F8CF,Lo,0x2F8CF,0x2F8CF,0x2F8CF},
{0x2F8D0,Lo,0x2F8D0,0x2F8D0,0x2F8D0},
{0x2F8D1,Lo,0x2F8D1,0x2F8D1,0x2F8D1},
{0x2F8D2,Lo,0x2F8D2,0x2F8D2,0x2F8D2},
{0x2F8D3,Lo,0x2F8D3,0x2F8D3,0x2F8D3},
{0x2F8D4,Lo,0x2F8D4,0x2F8D4,0x2F8D4},
{0x2F8D5,Lo,0x2F8D5,0x2F8D5,0x2F8D5},
{0x2F8D6,Lo,0x2F8D6,0x2F8D6,0x2F8D6},
{0x2F8D7,Lo,0x2F8D7,0x2F8D7,0x2F8D7},
{0x2F8D8,Lo,0x2F8D8,0x2F8D8,0x2F8D8},
{0x2F8D9,Lo,0x2F8D9,0x2F8D9,0x2F8D9},
{0x2F8DA,Lo,0x2F8DA,0x2F8DA,0x2F8DA},
{0x2F8DB,Lo,0x2F8DB,0x2F8DB,0x2F8DB},
{0x2F8DC,Lo,0x2F8DC,0x2F8DC,0x2F8DC},
{0x2F8DD,Lo,0x2F8DD,0x2F8DD,0x2F8DD},
{0x2F8DE,Lo,0x2F8DE,0x2F8DE,0x2F8DE},
{0x2F8DF,Lo,0x2F8DF,0x2F8DF,0x2F8DF},
{0x2F8E0,Lo,0x2F8E0,0x2F8E0,0x2F8E0},
{0x2F8E1,Lo,0x2F8E1,0x2F8E1,0x2F8E1},
{0x2F8E2,Lo,0x2F8E2,0x2F8E2,0x2F8E2},
{0x2F8E3,Lo,0x2F8E3,0x2F8E3,0x2F8E3},
{0x2F8E4,Lo,0x2F8E4,0x2F8E4,0x2F8E4},
{0x2F8E5,Lo,0x2F8E5,0x2F8E5,0x2F8E5},
{0x2F8E6,Lo,0x2F8E6,0x2F8E6,0x2F8E6},
{0x2F8E7,Lo,0x2F8E7,0x2F8E7,0x2F8E7},
{0x2F8E8,Lo,0x2F8E8,0x2F8E8,0x2F8E8},
{0x2F8E9,Lo,0x2F8E9,0x2F8E9,0x2F8E9},
{0x2F8EA,Lo,0x2F8EA,0x2F8EA,0x2F8EA},
{0x2F8EB,Lo,0x2F8EB,0x2F8EB,0x2F8EB},
{0x2F8EC,Lo,0x2F8EC,0x2F8EC,0x2F8EC},
{0x2F8ED,Lo,0x2F8ED,0x2F8ED,0x2F8ED},
{0x2F8EE,Lo,0x2F8EE,0x2F8EE,0x2F8EE},
{0x2F8EF,Lo,0x2F8EF,0x2F8EF,0x2F8EF},
{0x2F8F0,Lo,0x2F8F0,0x2F8F0,0x2F8F0},
{0x2F8F1,Lo,0x2F8F1,0x2F8F1,0x2F8F1},
{0x2F8F2,Lo,0x2F8F2,0x2F8F2,0x2F8F2},
{0x2F8F3,Lo,0x2F8F3,0x2F8F3,0x2F8F3},
{0x2F8F4,Lo,0x2F8F4,0x2F8F4,0x2F8F4},
{0x2F8F5,Lo,0x2F8F5,0x2F8F5,0x2F8F5},
{0x2F8F6,Lo,0x2F8F6,0x2F8F6,0x2F8F6},
{0x2F8F7,Lo,0x2F8F7,0x2F8F7,0x2F8F7},
{0x2F8F8,Lo,0x2F8F8,0x2F8F8,0x2F8F8},
{0x2F8F9,Lo,0x2F8F9,0x2F8F9,0x2F8F9},
{0x2F8FA,Lo,0x2F8FA,0x2F8FA,0x2F8FA},
{0x2F8FB,Lo,0x2F8FB,0x2F8FB,0x2F8FB},
{0x2F8FC,Lo,0x2F8FC,0x2F8FC,0x2F8FC},
{0x2F8FD,Lo,0x2F8FD,0x2F8FD,0x2F8FD},
{0x2F8FE,Lo,0x2F8FE,0x2F8FE,0x2F8FE},
{0x2F8FF,Lo,0x2F8FF,0x2F8FF,0x2F8FF},
{0x2F900,Lo,0x2F900,0x2F900,0x2F900},
{0x2F901,Lo,0x2F901,0x2F901,0x2F901},
{0x2F902,Lo,0x2F902,0x2F902,0x2F902},
{0x2F903,Lo,0x2F903,0x2F903,0x2F903},
{0x2F904,Lo,0x2F904,0x2F904,0x2F904},
{0x2F905,Lo,0x2F905,0x2F905,0x2F905},
{0x2F906,Lo,0x2F906,0x2F906,0x2F906},
{0x2F907,Lo,0x2F907,0x2F907,0x2F907},
{0x2F908,Lo,0x2F908,0x2F908,0x2F908},
{0x2F909,Lo,0x2F909,0x2F909,0x2F909},
{0x2F90A,Lo,0x2F90A,0x2F90A,0x2F90A},
{0x2F90B,Lo,0x2F90B,0x2F90B,0x2F90B},
{0x2F90C,Lo,0x2F90C,0x2F90C,0x2F90C},
{0x2F90D,Lo,0x2F90D,0x2F90D,0x2F90D},
{0x2F90E,Lo,0x2F90E,0x2F90E,0x2F90E},
{0x2F90F,Lo,0x2F90F,0x2F90F,0x2F90F},
{0x2F910,Lo,0x2F910,0x2F910,0x2F910},
{0x2F911,Lo,0x2F911,0x2F911,0x2F911},
{0x2F912,Lo,0x2F912,0x2F912,0x2F912},
{0x2F913,Lo,0x2F913,0x2F913,0x2F913},
{0x2F914,Lo,0x2F914,0x2F914,0x2F914},
{0x2F915,Lo,0x2F915,0x2F915,0x2F915},
{0x2F916,Lo,0x2F916,0x2F916,0x2F916},
{0x2F917,Lo,0x2F917,0x2F917,0x2F917},
{0x2F918,Lo,0x2F918,0x2F918,0x2F918},
{0x2F919,Lo,0x2F919,0x2F919,0x2F919},
{0x2F91A,Lo,0x2F91A,0x2F91A,0x2F91A},
{0x2F91B,Lo,0x2F91B,0x2F91B,0x2F91B},
{0x2F91C,Lo,0x2F91C,0x2F91C,0x2F91C},
{0x2F91D,Lo,0x2F91D,0x2F91D,0x2F91D},
{0x2F91E,Lo,0x2F91E,0x2F91E,0x2F91E},
{0x2F91F,Lo,0x2F91F,0x2F91F,0x2F91F},
{0x2F920,Lo,0x2F920,0x2F920,0x2F920},
{0x2F921,Lo,0x2F921,0x2F921,0x2F921},
{0x2F922,Lo,0x2F922,0x2F922,0x2F922},
{0x2F923,Lo,0x2F923,0x2F923,0x2F923},
{0x2F924,Lo,0x2F924,0x2F924,0x2F924},
{0x2F925,Lo,0x2F925,0x2F925,0x2F925},
{0x2F926,Lo,0x2F926,0x2F926,0x2F926},
{0x2F927,Lo,0x2F927,0x2F927,0x2F927},
{0x2F928,Lo,0x2F928,0x2F928,0x2F928},
{0x2F929,Lo,0x2F929,0x2F929,0x2F929},
{0x2F92A,Lo,0x2F92A,0x2F92A,0x2F92A},
{0x2F92B,Lo,0x2F92B,0x2F92B,0x2F92B},
{0x2F92C,Lo,0x2F92C,0x2F92C,0x2F92C},
{0x2F92D,Lo,0x2F92D,0x2F92D,0x2F92D},
{0x2F92E,Lo,0x2F92E,0x2F92E,0x2F92E},
{0x2F92F,Lo,0x2F92F,0x2F92F,0x2F92F},
{0x2F930,Lo,0x2F930,0x2F930,0x2F930},
{0x2F931,Lo,0x2F931,0x2F931,0x2F931},
{0x2F932,Lo,0x2F932,0x2F932,0x2F932},
{0x2F933,Lo,0x2F933,0x2F933,0x2F933},
{0x2F934,Lo,0x2F934,0x2F934,0x2F934},
{0x2F935,Lo,0x2F935,0x2F935,0x2F935},
{0x2F936,Lo,0x2F936,0x2F936,0x2F936},
{0x2F937,Lo,0x2F937,0x2F937,0x2F937},
{0x2F938,Lo,0x2F938,0x2F938,0x2F938},
{0x2F939,Lo,0x2F939,0x2F939,0x2F939},
{0x2F93A,Lo,0x2F93A,0x2F93A,0x2F93A},
{0x2F93B,Lo,0x2F93B,0x2F93B,0x2F93B},
{0x2F93C,Lo,0x2F93C,0x2F93C,0x2F93C},
{0x2F93D,Lo,0x2F93D,0x2F93D,0x2F93D},
{0x2F93E,Lo,0x2F93E,0x2F93E,0x2F93E},
{0x2F93F,Lo,0x2F93F,0x2F93F,0x2F93F},
{0x2F940,Lo,0x2F940,0x2F940,0x2F940},
{0x2F941,Lo,0x2F941,0x2F941,0x2F941},
{0x2F942,Lo,0x2F942,0x2F942,0x2F942},
{0x2F943,Lo,0x2F943,0x2F943,0x2F943},
{0x2F944,Lo,0x2F944,0x2F944,0x2F944},
{0x2F945,Lo,0x2F945,0x2F945,0x2F945},
{0x2F946,Lo,0x2F946,0x2F946,0x2F946},
{0x2F947,Lo,0x2F947,0x2F947,0x2F947},
{0x2F948,Lo,0x2F948,0x2F948,0x2F948},
{0x2F949,Lo,0x2F949,0x2F949,0x2F949},
{0x2F94A,Lo,0x2F94A,0x2F94A,0x2F94A},
{0x2F94B,Lo,0x2F94B,0x2F94B,0x2F94B},
{0x2F94C,Lo,0x2F94C,0x2F94C,0x2F94C},
{0x2F94D,Lo,0x2F94D,0x2F94D,0x2F94D},
{0x2F94E,Lo,0x2F94E,0x2F94E,0x2F94E},
{0x2F94F,Lo,0x2F94F,0x2F94F,0x2F94F},
{0x2F950,Lo,0x2F950,0x2F950,0x2F950},
{0x2F951,Lo,0x2F951,0x2F951,0x2F951},
{0x2F952,Lo,0x2F952,0x2F952,0x2F952},
{0x2F953,Lo,0x2F953,0x2F953,0x2F953},
{0x2F954,Lo,0x2F954,0x2F954,0x2F954},
{0x2F955,Lo,0x2F955,0x2F955,0x2F955},
{0x2F956,Lo,0x2F956,0x2F956,0x2F956},
{0x2F957,Lo,0x2F957,0x2F957,0x2F957},
{0x2F958,Lo,0x2F958,0x2F958,0x2F958},
{0x2F959,Lo,0x2F959,0x2F959,0x2F959},
{0x2F95A,Lo,0x2F95A,0x2F95A,0x2F95A},
{0x2F95B,Lo,0x2F95B,0x2F95B,0x2F95B},
{0x2F95C,Lo,0x2F95C,0x2F95C,0x2F95C},
{0x2F95D,Lo,0x2F95D,0x2F95D,0x2F95D},
{0x2F95E,Lo,0x2F95E,0x2F95E,0x2F95E},
{0x2F95F,Lo,0x2F95F,0x2F95F,0x2F95F},
{0x2F960,Lo,0x2F960,0x2F960,0x2F960},
{0x2F961,Lo,0x2F961,0x2F961,0x2F961},
{0x2F962,Lo,0x2F962,0x2F962,0x2F962},
{0x2F963,Lo,0x2F963,0x2F963,0x2F963},
{0x2F964,Lo,0x2F964,0x2F964,0x2F964},
{0x2F965,Lo,0x2F965,0x2F965,0x2F965},
{0x2F966,Lo,0x2F966,0x2F966,0x2F966},
{0x2F967,Lo,0x2F967,0x2F967,0x2F967},
{0x2F968,Lo,0x2F968,0x2F968,0x2F968},
{0x2F969,Lo,0x2F969,0x2F969,0x2F969},
{0x2F96A,Lo,0x2F96A,0x2F96A,0x2F96A},
{0x2F96B,Lo,0x2F96B,0x2F96B,0x2F96B},
{0x2F96C,Lo,0x2F96C,0x2F96C,0x2F96C},
{0x2F96D,Lo,0x2F96D,0x2F96D,0x2F96D},
{0x2F96E,Lo,0x2F96E,0x2F96E,0x2F96E},
{0x2F96F,Lo,0x2F96F,0x2F96F,0x2F96F},
{0x2F970,Lo,0x2F970,0x2F970,0x2F970},
{0x2F971,Lo,0x2F971,0x2F971,0x2F971},
{0x2F972,Lo,0x2F972,0x2F972,0x2F972},
{0x2F973,Lo,0x2F973,0x2F973,0x2F973},
{0x2F974,Lo,0x2F974,0x2F974,0x2F974},
{0x2F975,Lo,0x2F975,0x2F975,0x2F975},
{0x2F976,Lo,0x2F976,0x2F976,0x2F976},
{0x2F977,Lo,0x2F977,0x2F977,0x2F977},
{0x2F978,Lo,0x2F978,0x2F978,0x2F978},
{0x2F979,Lo,0x2F979,0x2F979,0x2F979},
{0x2F97A,Lo,0x2F97A,0x2F97A,0x2F97A},
{0x2F97B,Lo,0x2F97B,0x2F97B,0x2F97B},
{0x2F97C,Lo,0x2F97C,0x2F97C,0x2F97C},
{0x2F97D,Lo,0x2F97D,0x2F97D,0x2F97D},
{0x2F97E,Lo,0x2F97E,0x2F97E,0x2F97E},
{0x2F97F,Lo,0x2F97F,0x2F97F,0x2F97F},
{0x2F980,Lo,0x2F980,0x2F980,0x2F980},
{0x2F981,Lo,0x2F981,0x2F981,0x2F981},
{0x2F982,Lo,0x2F982,0x2F982,0x2F982},
{0x2F983,Lo,0x2F983,0x2F983,0x2F983},
{0x2F984,Lo,0x2F984,0x2F984,0x2F984},
{0x2F985,Lo,0x2F985,0x2F985,0x2F985},
{0x2F986,Lo,0x2F986,0x2F986,0x2F986},
{0x2F987,Lo,0x2F987,0x2F987,0x2F987},
{0x2F988,Lo,0x2F988,0x2F988,0x2F988},
{0x2F989,Lo,0x2F989,0x2F989,0x2F989},
{0x2F98A,Lo,0x2F98A,0x2F98A,0x2F98A},
{0x2F98B,Lo,0x2F98B,0x2F98B,0x2F98B},
{0x2F98C,Lo,0x2F98C,0x2F98C,0x2F98C},
{0x2F98D,Lo,0x2F98D,0x2F98D,0x2F98D},
{0x2F98E,Lo,0x2F98E,0x2F98E,0x2F98E},
{0x2F98F,Lo,0x2F98F,0x2F98F,0x2F98F},
{0x2F990,Lo,0x2F990,0x2F990,0x2F990},
{0x2F991,Lo,0x2F991,0x2F991,0x2F991},
{0x2F992,Lo,0x2F992,0x2F992,0x2F992},
{0x2F993,Lo,0x2F993,0x2F993,0x2F993},
{0x2F994,Lo,0x2F994,0x2F994,0x2F994},
{0x2F995,Lo,0x2F995,0x2F995,0x2F995},
{0x2F996,Lo,0x2F996,0x2F996,0x2F996},
{0x2F997,Lo,0x2F997,0x2F997,0x2F997},
{0x2F998,Lo,0x2F998,0x2F998,0x2F998},
{0x2F999,Lo,0x2F999,0x2F999,0x2F999},
{0x2F99A,Lo,0x2F99A,0x2F99A,0x2F99A},
{0x2F99B,Lo,0x2F99B,0x2F99B,0x2F99B},
{0x2F99C,Lo,0x2F99C,0x2F99C,0x2F99C},
{0x2F99D,Lo,0x2F99D,0x2F99D,0x2F99D},
{0x2F99E,Lo,0x2F99E,0x2F99E,0x2F99E},
{0x2F99F,Lo,0x2F99F,0x2F99F,0x2F99F},
{0x2F9A0,Lo,0x2F9A0,0x2F9A0,0x2F9A0},
{0x2F9A1,Lo,0x2F9A1,0x2F9A1,0x2F9A1},
{0x2F9A2,Lo,0x2F9A2,0x2F9A2,0x2F9A2},
{0x2F9A3,Lo,0x2F9A3,0x2F9A3,0x2F9A3},
{0x2F9A4,Lo,0x2F9A4,0x2F9A4,0x2F9A4},
{0x2F9A5,Lo,0x2F9A5,0x2F9A5,0x2F9A5},
{0x2F9A6,Lo,0x2F9A6,0x2F9A6,0x2F9A6},
{0x2F9A7,Lo,0x2F9A7,0x2F9A7,0x2F9A7},
{0x2F9A8,Lo,0x2F9A8,0x2F9A8,0x2F9A8},
{0x2F9A9,Lo,0x2F9A9,0x2F9A9,0x2F9A9},
{0x2F9AA,Lo,0x2F9AA,0x2F9AA,0x2F9AA},
{0x2F9AB,Lo,0x2F9AB,0x2F9AB,0x2F9AB},
{0x2F9AC,Lo,0x2F9AC,0x2F9AC,0x2F9AC},
{0x2F9AD,Lo,0x2F9AD,0x2F9AD,0x2F9AD},
{0x2F9AE,Lo,0x2F9AE,0x2F9AE,0x2F9AE},
{0x2F9AF,Lo,0x2F9AF,0x2F9AF,0x2F9AF},
{0x2F9B0,Lo,0x2F9B0,0x2F9B0,0x2F9B0},
{0x2F9B1,Lo,0x2F9B1,0x2F9B1,0x2F9B1},
{0x2F9B2,Lo,0x2F9B2,0x2F9B2,0x2F9B2},
{0x2F9B3,Lo,0x2F9B3,0x2F9B3,0x2F9B3},
{0x2F9B4,Lo,0x2F9B4,0x2F9B4,0x2F9B4},
{0x2F9B5,Lo,0x2F9B5,0x2F9B5,0x2F9B5},
{0x2F9B6,Lo,0x2F9B6,0x2F9B6,0x2F9B6},
{0x2F9B7,Lo,0x2F9B7,0x2F9B7,0x2F9B7},
{0x2F9B8,Lo,0x2F9B8,0x2F9B8,0x2F9B8},
{0x2F9B9,Lo,0x2F9B9,0x2F9B9,0x2F9B9},
{0x2F9BA,Lo,0x2F9BA,0x2F9BA,0x2F9BA},
{0x2F9BB,Lo,0x2F9BB,0x2F9BB,0x2F9BB},
{0x2F9BC,Lo,0x2F9BC,0x2F9BC,0x2F9BC},
{0x2F9BD,Lo,0x2F9BD,0x2F9BD,0x2F9BD},
{0x2F9BE,Lo,0x2F9BE,0x2F9BE,0x2F9BE},
{0x2F9BF,Lo,0x2F9BF,0x2F9BF,0x2F9BF},
{0x2F9C0,Lo,0x2F9C0,0x2F9C0,0x2F9C0},
{0x2F9C1,Lo,0x2F9C1,0x2F9C1,0x2F9C1},
{0x2F9C2,Lo,0x2F9C2,0x2F9C2,0x2F9C2},
{0x2F9C3,Lo,0x2F9C3,0x2F9C3,0x2F9C3},
{0x2F9C4,Lo,0x2F9C4,0x2F9C4,0x2F9C4},
{0x2F9C5,Lo,0x2F9C5,0x2F9C5,0x2F9C5},
{0x2F9C6,Lo,0x2F9C6,0x2F9C6,0x2F9C6},
{0x2F9C7,Lo,0x2F9C7,0x2F9C7,0x2F9C7},
{0x2F9C8,Lo,0x2F9C8,0x2F9C8,0x2F9C8},
{0x2F9C9,Lo,0x2F9C9,0x2F9C9,0x2F9C9},
{0x2F9CA,Lo,0x2F9CA,0x2F9CA,0x2F9CA},
{0x2F9CB,Lo,0x2F9CB,0x2F9CB,0x2F9CB},
{0x2F9CC,Lo,0x2F9CC,0x2F9CC,0x2F9CC},
{0x2F9CD,Lo,0x2F9CD,0x2F9CD,0x2F9CD},
{0x2F9CE,Lo,0x2F9CE,0x2F9CE,0x2F9CE},
{0x2F9CF,Lo,0x2F9CF,0x2F9CF,0x2F9CF},
{0x2F9D0,Lo,0x2F9D0,0x2F9D0,0x2F9D0},
{0x2F9D1,Lo,0x2F9D1,0x2F9D1,0x2F9D1},
{0x2F9D2,Lo,0x2F9D2,0x2F9D2,0x2F9D2},
{0x2F9D3,Lo,0x2F9D3,0x2F9D3,0x2F9D3},
{0x2F9D4,Lo,0x2F9D4,0x2F9D4,0x2F9D4},
{0x2F9D5,Lo,0x2F9D5,0x2F9D5,0x2F9D5},
{0x2F9D6,Lo,0x2F9D6,0x2F9D6,0x2F9D6},
{0x2F9D7,Lo,0x2F9D7,0x2F9D7,0x2F9D7},
{0x2F9D8,Lo,0x2F9D8,0x2F9D8,0x2F9D8},
{0x2F9D9,Lo,0x2F9D9,0x2F9D9,0x2F9D9},
{0x2F9DA,Lo,0x2F9DA,0x2F9DA,0x2F9DA},
{0x2F9DB,Lo,0x2F9DB,0x2F9DB,0x2F9DB},
{0x2F9DC,Lo,0x2F9DC,0x2F9DC,0x2F9DC},
{0x2F9DD,Lo,0x2F9DD,0x2F9DD,0x2F9DD},
{0x2F9DE,Lo,0x2F9DE,0x2F9DE,0x2F9DE},
{0x2F9DF,Lo,0x2F9DF,0x2F9DF,0x2F9DF},
{0x2F9E0,Lo,0x2F9E0,0x2F9E0,0x2F9E0},
{0x2F9E1,Lo,0x2F9E1,0x2F9E1,0x2F9E1},
{0x2F9E2,Lo,0x2F9E2,0x2F9E2,0x2F9E2},
{0x2F9E3,Lo,0x2F9E3,0x2F9E3,0x2F9E3},
{0x2F9E4,Lo,0x2F9E4,0x2F9E4,0x2F9E4},
{0x2F9E5,Lo,0x2F9E5,0x2F9E5,0x2F9E5},
{0x2F9E6,Lo,0x2F9E6,0x2F9E6,0x2F9E6},
{0x2F9E7,Lo,0x2F9E7,0x2F9E7,0x2F9E7},
{0x2F9E8,Lo,0x2F9E8,0x2F9E8,0x2F9E8},
{0x2F9E9,Lo,0x2F9E9,0x2F9E9,0x2F9E9},
{0x2F9EA,Lo,0x2F9EA,0x2F9EA,0x2F9EA},
{0x2F9EB,Lo,0x2F9EB,0x2F9EB,0x2F9EB},
{0x2F9EC,Lo,0x2F9EC,0x2F9EC,0x2F9EC},
{0x2F9ED,Lo,0x2F9ED,0x2F9ED,0x2F9ED},
{0x2F9EE,Lo,0x2F9EE,0x2F9EE,0x2F9EE},
{0x2F9EF,Lo,0x2F9EF,0x2F9EF,0x2F9EF},
{0x2F9F0,Lo,0x2F9F0,0x2F9F0,0x2F9F0},
{0x2F9F1,Lo,0x2F9F1,0x2F9F1,0x2F9F1},
{0x2F9F2,Lo,0x2F9F2,0x2F9F2,0x2F9F2},
{0x2F9F3,Lo,0x2F9F3,0x2F9F3,0x2F9F3},
{0x2F9F4,Lo,0x2F9F4,0x2F9F4,0x2F9F4},
{0x2F9F5,Lo,0x2F9F5,0x2F9F5,0x2F9F5},
{0x2F9F6,Lo,0x2F9F6,0x2F9F6,0x2F9F6},
{0x2F9F7,Lo,0x2F9F7,0x2F9F7,0x2F9F7},
{0x2F9F8,Lo,0x2F9F8,0x2F9F8,0x2F9F8},
{0x2F9F9,Lo,0x2F9F9,0x2F9F9,0x2F9F9},
{0x2F9FA,Lo,0x2F9FA,0x2F9FA,0x2F9FA},
{0x2F9FB,Lo,0x2F9FB,0x2F9FB,0x2F9FB},
{0x2F9FC,Lo,0x2F9FC,0x2F9FC,0x2F9FC},
{0x2F9FD,Lo,0x2F9FD,0x2F9FD,0x2F9FD},
{0x2F9FE,Lo,0x2F9FE,0x2F9FE,0x2F9FE},
{0x2F9FF,Lo,0x2F9FF,0x2F9FF,0x2F9FF},
{0x2FA00,Lo,0x2FA00,0x2FA00,0x2FA00},
{0x2FA01,Lo,0x2FA01,0x2FA01,0x2FA01},
{0x2FA02,Lo,0x2FA02,0x2FA02,0x2FA02},
{0x2FA03,Lo,0x2FA03,0x2FA03,0x2FA03},
{0x2FA04,Lo,0x2FA04,0x2FA04,0x2FA04},
{0x2FA05,Lo,0x2FA05,0x2FA05,0x2FA05},
{0x2FA06,Lo,0x2FA06,0x2FA06,0x2FA06},
{0x2FA07,Lo,0x2FA07,0x2FA07,0x2FA07},
{0x2FA08,Lo,0x2FA08,0x2FA08,0x2FA08},
{0x2FA09,Lo,0x2FA09,0x2FA09,0x2FA09},
{0x2FA0A,Lo,0x2FA0A,0x2FA0A,0x2FA0A},
{0x2FA0B,Lo,0x2FA0B,0x2FA0B,0x2FA0B},
{0x2FA0C,Lo,0x2FA0C,0x2FA0C,0x2FA0C},
{0x2FA0D,Lo,0x2FA0D,0x2FA0D,0x2FA0D},
{0x2FA0E,Lo,0x2FA0E,0x2FA0E,0x2FA0E},
{0x2FA0F,Lo,0x2FA0F,0x2FA0F,0x2FA0F},
{0x2FA10,Lo,0x2FA10,0x2FA10,0x2FA10},
{0x2FA11,Lo,0x2FA11,0x2FA11,0x2FA11},
{0x2FA12,Lo,0x2FA12,0x2FA12,0x2FA12},
{0x2FA13,Lo,0x2FA13,0x2FA13,0x2FA13},
{0x2FA14,Lo,0x2FA14,0x2FA14,0x2FA14},
{0x2FA15,Lo,0x2FA15,0x2FA15,0x2FA15},
{0x2FA16,Lo,0x2FA16,0x2FA16,0x2FA16},
{0x2FA17,Lo,0x2FA17,0x2FA17,0x2FA17},
{0x2FA18,Lo,0x2FA18,0x2FA18,0x2FA18},
{0x2FA19,Lo,0x2FA19,0x2FA19,0x2FA19},
{0x2FA1A,Lo,0x2FA1A,0x2FA1A,0x2FA1A},
{0x2FA1B,Lo,0x2FA1B,0x2FA1B,0x2FA1B},
{0x2FA1C,Lo,0x2FA1C,0x2FA1C,0x2FA1C},
{0x2FA1D,Lo,0x2FA1D,0x2FA1D,0x2FA1D},
{0xE0001,Cf,0xE0001,0xE0001,0xE0001},
{0xE0020,Cf,0xE0020,0xE0020,0xE0020},
{0xE0021,Cf,0xE0021,0xE0021,0xE0021},
{0xE0022,Cf,0xE0022,0xE0022,0xE0022},
{0xE0023,Cf,0xE0023,0xE0023,0xE0023},
{0xE0024,Cf,0xE0024,0xE0024,0xE0024},
{0xE0025,Cf,0xE0025,0xE0025,0xE0025},
{0xE0026,Cf,0xE0026,0xE0026,0xE0026},
{0xE0027,Cf,0xE0027,0xE0027,0xE0027},
{0xE0028,Cf,0xE0028,0xE0028,0xE0028},
{0xE0029,Cf,0xE0029,0xE0029,0xE0029},
{0xE002A,Cf,0xE002A,0xE002A,0xE002A},
{0xE002B,Cf,0xE002B,0xE002B,0xE002B},
{0xE002C,Cf,0xE002C,0xE002C,0xE002C},
{0xE002D,Cf,0xE002D,0xE002D,0xE002D},
{0xE002E,Cf,0xE002E,0xE002E,0xE002E},
{0xE002F,Cf,0xE002F,0xE002F,0xE002F},
{0xE0030,Cf,0xE0030,0xE0030,0xE0030},
{0xE0031,Cf,0xE0031,0xE0031,0xE0031},
{0xE0032,Cf,0xE0032,0xE0032,0xE0032},
{0xE0033,Cf,0xE0033,0xE0033,0xE0033},
{0xE0034,Cf,0xE0034,0xE0034,0xE0034},
{0xE0035,Cf,0xE0035,0xE0035,0xE0035},
{0xE0036,Cf,0xE0036,0xE0036,0xE0036},
{0xE0037,Cf,0xE0037,0xE0037,0xE0037},
{0xE0038,Cf,0xE0038,0xE0038,0xE0038},
{0xE0039,Cf,0xE0039,0xE0039,0xE0039},
{0xE003A,Cf,0xE003A,0xE003A,0xE003A},
{0xE003B,Cf,0xE003B,0xE003B,0xE003B},
{0xE003C,Cf,0xE003C,0xE003C,0xE003C},
{0xE003D,Cf,0xE003D,0xE003D,0xE003D},
{0xE003E,Cf,0xE003E,0xE003E,0xE003E},
{0xE003F,Cf,0xE003F,0xE003F,0xE003F},
{0xE0040,Cf,0xE0040,0xE0040,0xE0040},
{0xE0041,Cf,0xE0041,0xE0041,0xE0041},
{0xE0042,Cf,0xE0042,0xE0042,0xE0042},
{0xE0043,Cf,0xE0043,0xE0043,0xE0043},
{0xE0044,Cf,0xE0044,0xE0044,0xE0044},
{0xE0045,Cf,0xE0045,0xE0045,0xE0045},
{0xE0046,Cf,0xE0046,0xE0046,0xE0046},
{0xE0047,Cf,0xE0047,0xE0047,0xE0047},
{0xE0048,Cf,0xE0048,0xE0048,0xE0048},
{0xE0049,Cf,0xE0049,0xE0049,0xE0049},
{0xE004A,Cf,0xE004A,0xE004A,0xE004A},
{0xE004B,Cf,0xE004B,0xE004B,0xE004B},
{0xE004C,Cf,0xE004C,0xE004C,0xE004C},
{0xE004D,Cf,0xE004D,0xE004D,0xE004D},
{0xE004E,Cf,0xE004E,0xE004E,0xE004E},
{0xE004F,Cf,0xE004F,0xE004F,0xE004F},
{0xE0050,Cf,0xE0050,0xE0050,0xE0050},
{0xE0051,Cf,0xE0051,0xE0051,0xE0051},
{0xE0052,Cf,0xE0052,0xE0052,0xE0052},
{0xE0053,Cf,0xE0053,0xE0053,0xE0053},
{0xE0054,Cf,0xE0054,0xE0054,0xE0054},
{0xE0055,Cf,0xE0055,0xE0055,0xE0055},
{0xE0056,Cf,0xE0056,0xE0056,0xE0056},
{0xE0057,Cf,0xE0057,0xE0057,0xE0057},
{0xE0058,Cf,0xE0058,0xE0058,0xE0058},
{0xE0059,Cf,0xE0059,0xE0059,0xE0059},
{0xE005A,Cf,0xE005A,0xE005A,0xE005A},
{0xE005B,Cf,0xE005B,0xE005B,0xE005B},
{0xE005C,Cf,0xE005C,0xE005C,0xE005C},
{0xE005D,Cf,0xE005D,0xE005D,0xE005D},
{0xE005E,Cf,0xE005E,0xE005E,0xE005E},
{0xE005F,Cf,0xE005F,0xE005F,0xE005F},
{0xE0060,Cf,0xE0060,0xE0060,0xE0060},
{0xE0061,Cf,0xE0061,0xE0061,0xE0061},
{0xE0062,Cf,0xE0062,0xE0062,0xE0062},
{0xE0063,Cf,0xE0063,0xE0063,0xE0063},
{0xE0064,Cf,0xE0064,0xE0064,0xE0064},
{0xE0065,Cf,0xE0065,0xE0065,0xE0065},
{0xE0066,Cf,0xE0066,0xE0066,0xE0066},
{0xE0067,Cf,0xE0067,0xE0067,0xE0067},
{0xE0068,Cf,0xE0068,0xE0068,0xE0068},
{0xE0069,Cf,0xE0069,0xE0069,0xE0069},
{0xE006A,Cf,0xE006A,0xE006A,0xE006A},
{0xE006B,Cf,0xE006B,0xE006B,0xE006B},
{0xE006C,Cf,0xE006C,0xE006C,0xE006C},
{0xE006D,Cf,0xE006D,0xE006D,0xE006D},
{0xE006E,Cf,0xE006E,0xE006E,0xE006E},
{0xE006F,Cf,0xE006F,0xE006F,0xE006F},
{0xE0070,Cf,0xE0070,0xE0070,0xE0070},
{0xE0071,Cf,0xE0071,0xE0071,0xE0071},
{0xE0072,Cf,0xE0072,0xE0072,0xE0072},
{0xE0073,Cf,0xE0073,0xE0073,0xE0073},
{0xE0074,Cf,0xE0074,0xE0074,0xE0074},
{0xE0075,Cf,0xE0075,0xE0075,0xE0075},
{0xE0076,Cf,0xE0076,0xE0076,0xE0076},
{0xE0077,Cf,0xE0077,0xE0077,0xE0077},
{0xE0078,Cf,0xE0078,0xE0078,0xE0078},
{0xE0079,Cf,0xE0079,0xE0079,0xE0079},
{0xE007A,Cf,0xE007A,0xE007A,0xE007A},
{0xE007B,Cf,0xE007B,0xE007B,0xE007B},
{0xE007C,Cf,0xE007C,0xE007C,0xE007C},
{0xE007D,Cf,0xE007D,0xE007D,0xE007D},
{0xE007E,Cf,0xE007E,0xE007E,0xE007E},
{0xE007F,Cf,0xE007F,0xE007F,0xE007F},
{0xE0100,Mn,0xE0100,0xE0100,0xE0100},
{0xE0101,Mn,0xE0101,0xE0101,0xE0101},
{0xE0102,Mn,0xE0102,0xE0102,0xE0102},
{0xE0103,Mn,0xE0103,0xE0103,0xE0103},
{0xE0104,Mn,0xE0104,0xE0104,0xE0104},
{0xE0105,Mn,0xE0105,0xE0105,0xE0105},
{0xE0106,Mn,0xE0106,0xE0106,0xE0106},
{0xE0107,Mn,0xE0107,0xE0107,0xE0107},
{0xE0108,Mn,0xE0108,0xE0108,0xE0108},
{0xE0109,Mn,0xE0109,0xE0109,0xE0109},
{0xE010A,Mn,0xE010A,0xE010A,0xE010A},
{0xE010B,Mn,0xE010B,0xE010B,0xE010B},
{0xE010C,Mn,0xE010C,0xE010C,0xE010C},
{0xE010D,Mn,0xE010D,0xE010D,0xE010D},
{0xE010E,Mn,0xE010E,0xE010E,0xE010E},
{0xE010F,Mn,0xE010F,0xE010F,0xE010F},
{0xE0110,Mn,0xE0110,0xE0110,0xE0110},
{0xE0111,Mn,0xE0111,0xE0111,0xE0111},
{0xE0112,Mn,0xE0112,0xE0112,0xE0112},
{0xE0113,Mn,0xE0113,0xE0113,0xE0113},
{0xE0114,Mn,0xE0114,0xE0114,0xE0114},
{0xE0115,Mn,0xE0115,0xE0115,0xE0115},
{0xE0116,Mn,0xE0116,0xE0116,0xE0116},
{0xE0117,Mn,0xE0117,0xE0117,0xE0117},
{0xE0118,Mn,0xE0118,0xE0118,0xE0118},
{0xE0119,Mn,0xE0119,0xE0119,0xE0119},
{0xE011A,Mn,0xE011A,0xE011A,0xE011A},
{0xE011B,Mn,0xE011B,0xE011B,0xE011B},
{0xE011C,Mn,0xE011C,0xE011C,0xE011C},
{0xE011D,Mn,0xE011D,0xE011D,0xE011D},
{0xE011E,Mn,0xE011E,0xE011E,0xE011E},
{0xE011F,Mn,0xE011F,0xE011F,0xE011F},
{0xE0120,Mn,0xE0120,0xE0120,0xE0120},
{0xE0121,Mn,0xE0121,0xE0121,0xE0121},
{0xE0122,Mn,0xE0122,0xE0122,0xE0122},
{0xE0123,Mn,0xE0123,0xE0123,0xE0123},
{0xE0124,Mn,0xE0124,0xE0124,0xE0124},
{0xE0125,Mn,0xE0125,0xE0125,0xE0125},
{0xE0126,Mn,0xE0126,0xE0126,0xE0126},
{0xE0127,Mn,0xE0127,0xE0127,0xE0127},
{0xE0128,Mn,0xE0128,0xE0128,0xE0128},
{0xE0129,Mn,0xE0129,0xE0129,0xE0129},
{0xE012A,Mn,0xE012A,0xE012A,0xE012A},
{0xE012B,Mn,0xE012B,0xE012B,0xE012B},
{0xE012C,Mn,0xE012C,0xE012C,0xE012C},
{0xE012D,Mn,0xE012D,0xE012D,0xE012D},
{0xE012E,Mn,0xE012E,0xE012E,0xE012E},
{0xE012F,Mn,0xE012F,0xE012F,0xE012F},
{0xE0130,Mn,0xE0130,0xE0130,0xE0130},
{0xE0131,Mn,0xE0131,0xE0131,0xE0131},
{0xE0132,Mn,0xE0132,0xE0132,0xE0132},
{0xE0133,Mn,0xE0133,0xE0133,0xE0133},
{0xE0134,Mn,0xE0134,0xE0134,0xE0134},
{0xE0135,Mn,0xE0135,0xE0135,0xE0135},
{0xE0136,Mn,0xE0136,0xE0136,0xE0136},
{0xE0137,Mn,0xE0137,0xE0137,0xE0137},
{0xE0138,Mn,0xE0138,0xE0138,0xE0138},
{0xE0139,Mn,0xE0139,0xE0139,0xE0139},
{0xE013A,Mn,0xE013A,0xE013A,0xE013A},
{0xE013B,Mn,0xE013B,0xE013B,0xE013B},
{0xE013C,Mn,0xE013C,0xE013C,0xE013C},
{0xE013D,Mn,0xE013D,0xE013D,0xE013D},
{0xE013E,Mn,0xE013E,0xE013E,0xE013E},
{0xE013F,Mn,0xE013F,0xE013F,0xE013F},
{0xE0140,Mn,0xE0140,0xE0140,0xE0140},
{0xE0141,Mn,0xE0141,0xE0141,0xE0141},
{0xE0142,Mn,0xE0142,0xE0142,0xE0142},
{0xE0143,Mn,0xE0143,0xE0143,0xE0143},
{0xE0144,Mn,0xE0144,0xE0144,0xE0144},
{0xE0145,Mn,0xE0145,0xE0145,0xE0145},
{0xE0146,Mn,0xE0146,0xE0146,0xE0146},
{0xE0147,Mn,0xE0147,0xE0147,0xE0147},
{0xE0148,Mn,0xE0148,0xE0148,0xE0148},
{0xE0149,Mn,0xE0149,0xE0149,0xE0149},
{0xE014A,Mn,0xE014A,0xE014A,0xE014A},
{0xE014B,Mn,0xE014B,0xE014B,0xE014B},
{0xE014C,Mn,0xE014C,0xE014C,0xE014C},
{0xE014D,Mn,0xE014D,0xE014D,0xE014D},
{0xE014E,Mn,0xE014E,0xE014E,0xE014E},
{0xE014F,Mn,0xE014F,0xE014F,0xE014F},
{0xE0150,Mn,0xE0150,0xE0150,0xE0150},
{0xE0151,Mn,0xE0151,0xE0151,0xE0151},
{0xE0152,Mn,0xE0152,0xE0152,0xE0152},
{0xE0153,Mn,0xE0153,0xE0153,0xE0153},
{0xE0154,Mn,0xE0154,0xE0154,0xE0154},
{0xE0155,Mn,0xE0155,0xE0155,0xE0155},
{0xE0156,Mn,0xE0156,0xE0156,0xE0156},
{0xE0157,Mn,0xE0157,0xE0157,0xE0157},
{0xE0158,Mn,0xE0158,0xE0158,0xE0158},
{0xE0159,Mn,0xE0159,0xE0159,0xE0159},
{0xE015A,Mn,0xE015A,0xE015A,0xE015A},
{0xE015B,Mn,0xE015B,0xE015B,0xE015B},
{0xE015C,Mn,0xE015C,0xE015C,0xE015C},
{0xE015D,Mn,0xE015D,0xE015D,0xE015D},
{0xE015E,Mn,0xE015E,0xE015E,0xE015E},
{0xE015F,Mn,0xE015F,0xE015F,0xE015F},
{0xE0160,Mn,0xE0160,0xE0160,0xE0160},
{0xE0161,Mn,0xE0161,0xE0161,0xE0161},
{0xE0162,Mn,0xE0162,0xE0162,0xE0162},
{0xE0163,Mn,0xE0163,0xE0163,0xE0163},
{0xE0164,Mn,0xE0164,0xE0164,0xE0164},
{0xE0165,Mn,0xE0165,0xE0165,0xE0165},
{0xE0166,Mn,0xE0166,0xE0166,0xE0166},
{0xE0167,Mn,0xE0167,0xE0167,0xE0167},
{0xE0168,Mn,0xE0168,0xE0168,0xE0168},
{0xE0169,Mn,0xE0169,0xE0169,0xE0169},
{0xE016A,Mn,0xE016A,0xE016A,0xE016A},
{0xE016B,Mn,0xE016B,0xE016B,0xE016B},
{0xE016C,Mn,0xE016C,0xE016C,0xE016C},
{0xE016D,Mn,0xE016D,0xE016D,0xE016D},
{0xE016E,Mn,0xE016E,0xE016E,0xE016E},
{0xE016F,Mn,0xE016F,0xE016F,0xE016F},
{0xE0170,Mn,0xE0170,0xE0170,0xE0170},
{0xE0171,Mn,0xE0171,0xE0171,0xE0171},
{0xE0172,Mn,0xE0172,0xE0172,0xE0172},
{0xE0173,Mn,0xE0173,0xE0173,0xE0173},
{0xE0174,Mn,0xE0174,0xE0174,0xE0174},
{0xE0175,Mn,0xE0175,0xE0175,0xE0175},
{0xE0176,Mn,0xE0176,0xE0176,0xE0176},
{0xE0177,Mn,0xE0177,0xE0177,0xE0177},
{0xE0178,Mn,0xE0178,0xE0178,0xE0178},
{0xE0179,Mn,0xE0179,0xE0179,0xE0179},
{0xE017A,Mn,0xE017A,0xE017A,0xE017A},
{0xE017B,Mn,0xE017B,0xE017B,0xE017B},
{0xE017C,Mn,0xE017C,0xE017C,0xE017C},
{0xE017D,Mn,0xE017D,0xE017D,0xE017D},
{0xE017E,Mn,0xE017E,0xE017E,0xE017E},
{0xE017F,Mn,0xE017F,0xE017F,0xE017F},
{0xE0180,Mn,0xE0180,0xE0180,0xE0180},
{0xE0181,Mn,0xE0181,0xE0181,0xE0181},
{0xE0182,Mn,0xE0182,0xE0182,0xE0182},
{0xE0183,Mn,0xE0183,0xE0183,0xE0183},
{0xE0184,Mn,0xE0184,0xE0184,0xE0184},
{0xE0185,Mn,0xE0185,0xE0185,0xE0185},
{0xE0186,Mn,0xE0186,0xE0186,0xE0186},
{0xE0187,Mn,0xE0187,0xE0187,0xE0187},
{0xE0188,Mn,0xE0188,0xE0188,0xE0188},
{0xE0189,Mn,0xE0189,0xE0189,0xE0189},
{0xE018A,Mn,0xE018A,0xE018A,0xE018A},
{0xE018B,Mn,0xE018B,0xE018B,0xE018B},
{0xE018C,Mn,0xE018C,0xE018C,0xE018C},
{0xE018D,Mn,0xE018D,0xE018D,0xE018D},
{0xE018E,Mn,0xE018E,0xE018E,0xE018E},
{0xE018F,Mn,0xE018F,0xE018F,0xE018F},
{0xE0190,Mn,0xE0190,0xE0190,0xE0190},
{0xE0191,Mn,0xE0191,0xE0191,0xE0191},
{0xE0192,Mn,0xE0192,0xE0192,0xE0192},
{0xE0193,Mn,0xE0193,0xE0193,0xE0193},
{0xE0194,Mn,0xE0194,0xE0194,0xE0194},
{0xE0195,Mn,0xE0195,0xE0195,0xE0195},
{0xE0196,Mn,0xE0196,0xE0196,0xE0196},
{0xE0197,Mn,0xE0197,0xE0197,0xE0197},
{0xE0198,Mn,0xE0198,0xE0198,0xE0198},
{0xE0199,Mn,0xE0199,0xE0199,0xE0199},
{0xE019A,Mn,0xE019A,0xE019A,0xE019A},
{0xE019B,Mn,0xE019B,0xE019B,0xE019B},
{0xE019C,Mn,0xE019C,0xE019C,0xE019C},
{0xE019D,Mn,0xE019D,0xE019D,0xE019D},
{0xE019E,Mn,0xE019E,0xE019E,0xE019E},
{0xE019F,Mn,0xE019F,0xE019F,0xE019F},
{0xE01A0,Mn,0xE01A0,0xE01A0,0xE01A0},
{0xE01A1,Mn,0xE01A1,0xE01A1,0xE01A1},
{0xE01A2,Mn,0xE01A2,0xE01A2,0xE01A2},
{0xE01A3,Mn,0xE01A3,0xE01A3,0xE01A3},
{0xE01A4,Mn,0xE01A4,0xE01A4,0xE01A4},
{0xE01A5,Mn,0xE01A5,0xE01A5,0xE01A5},
{0xE01A6,Mn,0xE01A6,0xE01A6,0xE01A6},
{0xE01A7,Mn,0xE01A7,0xE01A7,0xE01A7},
{0xE01A8,Mn,0xE01A8,0xE01A8,0xE01A8},
{0xE01A9,Mn,0xE01A9,0xE01A9,0xE01A9},
{0xE01AA,Mn,0xE01AA,0xE01AA,0xE01AA},
{0xE01AB,Mn,0xE01AB,0xE01AB,0xE01AB},
{0xE01AC,Mn,0xE01AC,0xE01AC,0xE01AC},
{0xE01AD,Mn,0xE01AD,0xE01AD,0xE01AD},
{0xE01AE,Mn,0xE01AE,0xE01AE,0xE01AE},
{0xE01AF,Mn,0xE01AF,0xE01AF,0xE01AF},
{0xE01B0,Mn,0xE01B0,0xE01B0,0xE01B0},
{0xE01B1,Mn,0xE01B1,0xE01B1,0xE01B1},
{0xE01B2,Mn,0xE01B2,0xE01B2,0xE01B2},
{0xE01B3,Mn,0xE01B3,0xE01B3,0xE01B3},
{0xE01B4,Mn,0xE01B4,0xE01B4,0xE01B4},
{0xE01B5,Mn,0xE01B5,0xE01B5,0xE01B5},
{0xE01B6,Mn,0xE01B6,0xE01B6,0xE01B6},
{0xE01B7,Mn,0xE01B7,0xE01B7,0xE01B7},
{0xE01B8,Mn,0xE01B8,0xE01B8,0xE01B8},
{0xE01B9,Mn,0xE01B9,0xE01B9,0xE01B9},
{0xE01BA,Mn,0xE01BA,0xE01BA,0xE01BA},
{0xE01BB,Mn,0xE01BB,0xE01BB,0xE01BB},
{0xE01BC,Mn,0xE01BC,0xE01BC,0xE01BC},
{0xE01BD,Mn,0xE01BD,0xE01BD,0xE01BD},
{0xE01BE,Mn,0xE01BE,0xE01BE,0xE01BE},
{0xE01BF,Mn,0xE01BF,0xE01BF,0xE01BF},
{0xE01C0,Mn,0xE01C0,0xE01C0,0xE01C0},
{0xE01C1,Mn,0xE01C1,0xE01C1,0xE01C1},
{0xE01C2,Mn,0xE01C2,0xE01C2,0xE01C2},
{0xE01C3,Mn,0xE01C3,0xE01C3,0xE01C3},
{0xE01C4,Mn,0xE01C4,0xE01C4,0xE01C4},
{0xE01C5,Mn,0xE01C5,0xE01C5,0xE01C5},
{0xE01C6,Mn,0xE01C6,0xE01C6,0xE01C6},
{0xE01C7,Mn,0xE01C7,0xE01C7,0xE01C7},
{0xE01C8,Mn,0xE01C8,0xE01C8,0xE01C8},
{0xE01C9,Mn,0xE01C9,0xE01C9,0xE01C9},
{0xE01CA,Mn,0xE01CA,0xE01CA,0xE01CA},
{0xE01CB,Mn,0xE01CB,0xE01CB,0xE01CB},
{0xE01CC,Mn,0xE01CC,0xE01CC,0xE01CC},
{0xE01CD,Mn,0xE01CD,0xE01CD,0xE01CD},
{0xE01CE,Mn,0xE01CE,0xE01CE,0xE01CE},
{0xE01CF,Mn,0xE01CF,0xE01CF,0xE01CF},
{0xE01D0,Mn,0xE01D0,0xE01D0,0xE01D0},
{0xE01D1,Mn,0xE01D1,0xE01D1,0xE01D1},
{0xE01D2,Mn,0xE01D2,0xE01D2,0xE01D2},
{0xE01D3,Mn,0xE01D3,0xE01D3,0xE01D3},
{0xE01D4,Mn,0xE01D4,0xE01D4,0xE01D4},
{0xE01D5,Mn,0xE01D5,0xE01D5,0xE01D5},
{0xE01D6,Mn,0xE01D6,0xE01D6,0xE01D6},
{0xE01D7,Mn,0xE01D7,0xE01D7,0xE01D7},
{0xE01D8,Mn,0xE01D8,0xE01D8,0xE01D8},
{0xE01D9,Mn,0xE01D9,0xE01D9,0xE01D9},
{0xE01DA,Mn,0xE01DA,0xE01DA,0xE01DA},
{0xE01DB,Mn,0xE01DB,0xE01DB,0xE01DB},
{0xE01DC,Mn,0xE01DC,0xE01DC,0xE01DC},
{0xE01DD,Mn,0xE01DD,0xE01DD,0xE01DD},
{0xE01DE,Mn,0xE01DE,0xE01DE,0xE01DE},
{0xE01DF,Mn,0xE01DF,0xE01DF,0xE01DF},
{0xE01E0,Mn,0xE01E0,0xE01E0,0xE01E0},
{0xE01E1,Mn,0xE01E1,0xE01E1,0xE01E1},
{0xE01E2,Mn,0xE01E2,0xE01E2,0xE01E2},
{0xE01E3,Mn,0xE01E3,0xE01E3,0xE01E3},
{0xE01E4,Mn,0xE01E4,0xE01E4,0xE01E4},
{0xE01E5,Mn,0xE01E5,0xE01E5,0xE01E5},
{0xE01E6,Mn,0xE01E6,0xE01E6,0xE01E6},
{0xE01E7,Mn,0xE01E7,0xE01E7,0xE01E7},
{0xE01E8,Mn,0xE01E8,0xE01E8,0xE01E8},
{0xE01E9,Mn,0xE01E9,0xE01E9,0xE01E9},
{0xE01EA,Mn,0xE01EA,0xE01EA,0xE01EA},
{0xE01EB,Mn,0xE01EB,0xE01EB,0xE01EB},
{0xE01EC,Mn,0xE01EC,0xE01EC,0xE01EC},
{0xE01ED,Mn,0xE01ED,0xE01ED,0xE01ED},
{0xE01EE,Mn,0xE01EE,0xE01EE,0xE01EE},
{0xE01EF,Mn,0xE01EF,0xE01EF,0xE01EF},
{0xF0000,Co,0xF0000,0xF0000,0xF0000},
{0xFFFFD,Co,0xFFFFD,0xFFFFD,0xFFFFD},
{0x100000,Co,0x100000,0x100000,0x100000},
{0x10FFFD,Co,0x10FFFD,0x10FFFD,0x10FFFD}
];

SpecialCaseData internalSpecialCaseData[] = [
{
code: 0xDF,
upperCaseMapping: [0x53, 0x53],
lowerCaseMapping: [0xDF],
titleCaseMapping: [0x53, 0x73]
},
{
code: 0x130,
upperCaseMapping: [0x130],
lowerCaseMapping: [0x69, 0x307],
titleCaseMapping: [0x130]
},
{
code: 0x149,
upperCaseMapping: [0x2BC, 0x4E],
lowerCaseMapping: [0x149],
titleCaseMapping: [0x2BC, 0x4E]
},
{
code: 0x1F0,
upperCaseMapping: [0x4A, 0x30C],
lowerCaseMapping: [0x1F0],
titleCaseMapping: [0x4A, 0x30C]
},
{
code: 0x390,
upperCaseMapping: [0x399, 0x308, 0x301],
lowerCaseMapping: [0x390],
titleCaseMapping: [0x399, 0x308, 0x301]
},
{
code: 0x3B0,
upperCaseMapping: [0x3A5, 0x308, 0x301],
lowerCaseMapping: [0x3B0],
titleCaseMapping: [0x3A5, 0x308, 0x301]
},
{
code: 0x587,
upperCaseMapping: [0x535, 0x552],
lowerCaseMapping: [0x587],
titleCaseMapping: [0x535, 0x582]
},
{
code: 0x1E96,
upperCaseMapping: [0x48, 0x331],
lowerCaseMapping: [0x1E96],
titleCaseMapping: [0x48, 0x331]
},
{
code: 0x1E97,
upperCaseMapping: [0x54, 0x308],
lowerCaseMapping: [0x1E97],
titleCaseMapping: [0x54, 0x308]
},
{
code: 0x1E98,
upperCaseMapping: [0x57, 0x30A],
lowerCaseMapping: [0x1E98],
titleCaseMapping: [0x57, 0x30A]
},
{
code: 0x1E99,
upperCaseMapping: [0x59, 0x30A],
lowerCaseMapping: [0x1E99],
titleCaseMapping: [0x59, 0x30A]
},
{
code: 0x1E9A,
upperCaseMapping: [0x41, 0x2BE],
lowerCaseMapping: [0x1E9A],
titleCaseMapping: [0x41, 0x2BE]
},
{
code: 0x1F50,
upperCaseMapping: [0x3A5, 0x313],
lowerCaseMapping: [0x1F50],
titleCaseMapping: [0x3A5, 0x313]
},
{
code: 0x1F52,
upperCaseMapping: [0x3A5, 0x313, 0x300],
lowerCaseMapping: [0x1F52],
titleCaseMapping: [0x3A5, 0x313, 0x300]
},
{
code: 0x1F54,
upperCaseMapping: [0x3A5, 0x313, 0x301],
lowerCaseMapping: [0x1F54],
titleCaseMapping: [0x3A5, 0x313, 0x301]
},
{
code: 0x1F56,
upperCaseMapping: [0x3A5, 0x313, 0x342],
lowerCaseMapping: [0x1F56],
titleCaseMapping: [0x3A5, 0x313, 0x342]
},
{
code: 0x1F80,
upperCaseMapping: [0x1F08, 0x399],
lowerCaseMapping: [0x1F80],
titleCaseMapping: [0x1F88]
},
{
code: 0x1F81,
upperCaseMapping: [0x1F09, 0x399],
lowerCaseMapping: [0x1F81],
titleCaseMapping: [0x1F89]
},
{
code: 0x1F82,
upperCaseMapping: [0x1F0A, 0x399],
lowerCaseMapping: [0x1F82],
titleCaseMapping: [0x1F8A]
},
{
code: 0x1F83,
upperCaseMapping: [0x1F0B, 0x399],
lowerCaseMapping: [0x1F83],
titleCaseMapping: [0x1F8B]
},
{
code: 0x1F84,
upperCaseMapping: [0x1F0C, 0x399],
lowerCaseMapping: [0x1F84],
titleCaseMapping: [0x1F8C]
},
{
code: 0x1F85,
upperCaseMapping: [0x1F0D, 0x399],
lowerCaseMapping: [0x1F85],
titleCaseMapping: [0x1F8D]
},
{
code: 0x1F86,
upperCaseMapping: [0x1F0E, 0x399],
lowerCaseMapping: [0x1F86],
titleCaseMapping: [0x1F8E]
},
{
code: 0x1F87,
upperCaseMapping: [0x1F0F, 0x399],
lowerCaseMapping: [0x1F87],
titleCaseMapping: [0x1F8F]
},
{
code: 0x1F88,
upperCaseMapping: [0x1F08, 0x399],
lowerCaseMapping: [0x1F80],
titleCaseMapping: [0x1F88]
},
{
code: 0x1F89,
upperCaseMapping: [0x1F09, 0x399],
lowerCaseMapping: [0x1F81],
titleCaseMapping: [0x1F89]
},
{
code: 0x1F8A,
upperCaseMapping: [0x1F0A, 0x399],
lowerCaseMapping: [0x1F82],
titleCaseMapping: [0x1F8A]
},
{
code: 0x1F8B,
upperCaseMapping: [0x1F0B, 0x399],
lowerCaseMapping: [0x1F83],
titleCaseMapping: [0x1F8B]
},
{
code: 0x1F8C,
upperCaseMapping: [0x1F0C, 0x399],
lowerCaseMapping: [0x1F84],
titleCaseMapping: [0x1F8C]
},
{
code: 0x1F8D,
upperCaseMapping: [0x1F0D, 0x399],
lowerCaseMapping: [0x1F85],
titleCaseMapping: [0x1F8D]
},
{
code: 0x1F8E,
upperCaseMapping: [0x1F0E, 0x399],
lowerCaseMapping: [0x1F86],
titleCaseMapping: [0x1F8E]
},
{
code: 0x1F8F,
upperCaseMapping: [0x1F0F, 0x399],
lowerCaseMapping: [0x1F87],
titleCaseMapping: [0x1F8F]
},
{
code: 0x1F90,
upperCaseMapping: [0x1F28, 0x399],
lowerCaseMapping: [0x1F90],
titleCaseMapping: [0x1F98]
},
{
code: 0x1F91,
upperCaseMapping: [0x1F29, 0x399],
lowerCaseMapping: [0x1F91],
titleCaseMapping: [0x1F99]
},
{
code: 0x1F92,
upperCaseMapping: [0x1F2A, 0x399],
lowerCaseMapping: [0x1F92],
titleCaseMapping: [0x1F9A]
},
{
code: 0x1F93,
upperCaseMapping: [0x1F2B, 0x399],
lowerCaseMapping: [0x1F93],
titleCaseMapping: [0x1F9B]
},
{
code: 0x1F94,
upperCaseMapping: [0x1F2C, 0x399],
lowerCaseMapping: [0x1F94],
titleCaseMapping: [0x1F9C]
},
{
code: 0x1F95,
upperCaseMapping: [0x1F2D, 0x399],
lowerCaseMapping: [0x1F95],
titleCaseMapping: [0x1F9D]
},
{
code: 0x1F96,
upperCaseMapping: [0x1F2E, 0x399],
lowerCaseMapping: [0x1F96],
titleCaseMapping: [0x1F9E]
},
{
code: 0x1F97,
upperCaseMapping: [0x1F2F, 0x399],
lowerCaseMapping: [0x1F97],
titleCaseMapping: [0x1F9F]
},
{
code: 0x1F98,
upperCaseMapping: [0x1F28, 0x399],
lowerCaseMapping: [0x1F90],
titleCaseMapping: [0x1F98]
},
{
code: 0x1F99,
upperCaseMapping: [0x1F29, 0x399],
lowerCaseMapping: [0x1F91],
titleCaseMapping: [0x1F99]
},
{
code: 0x1F9A,
upperCaseMapping: [0x1F2A, 0x399],
lowerCaseMapping: [0x1F92],
titleCaseMapping: [0x1F9A]
},
{
code: 0x1F9B,
upperCaseMapping: [0x1F2B, 0x399],
lowerCaseMapping: [0x1F93],
titleCaseMapping: [0x1F9B]
},
{
code: 0x1F9C,
upperCaseMapping: [0x1F2C, 0x399],
lowerCaseMapping: [0x1F94],
titleCaseMapping: [0x1F9C]
},
{
code: 0x1F9D,
upperCaseMapping: [0x1F2D, 0x399],
lowerCaseMapping: [0x1F95],
titleCaseMapping: [0x1F9D]
},
{
code: 0x1F9E,
upperCaseMapping: [0x1F2E, 0x399],
lowerCaseMapping: [0x1F96],
titleCaseMapping: [0x1F9E]
},
{
code: 0x1F9F,
upperCaseMapping: [0x1F2F, 0x399],
lowerCaseMapping: [0x1F97],
titleCaseMapping: [0x1F9F]
},
{
code: 0x1FA0,
upperCaseMapping: [0x1F68, 0x399],
lowerCaseMapping: [0x1FA0],
titleCaseMapping: [0x1FA8]
},
{
code: 0x1FA1,
upperCaseMapping: [0x1F69, 0x399],
lowerCaseMapping: [0x1FA1],
titleCaseMapping: [0x1FA9]
},
{
code: 0x1FA2,
upperCaseMapping: [0x1F6A, 0x399],
lowerCaseMapping: [0x1FA2],
titleCaseMapping: [0x1FAA]
},
{
code: 0x1FA3,
upperCaseMapping: [0x1F6B, 0x399],
lowerCaseMapping: [0x1FA3],
titleCaseMapping: [0x1FAB]
},
{
code: 0x1FA4,
upperCaseMapping: [0x1F6C, 0x399],
lowerCaseMapping: [0x1FA4],
titleCaseMapping: [0x1FAC]
},
{
code: 0x1FA5,
upperCaseMapping: [0x1F6D, 0x399],
lowerCaseMapping: [0x1FA5],
titleCaseMapping: [0x1FAD]
},
{
code: 0x1FA6,
upperCaseMapping: [0x1F6E, 0x399],
lowerCaseMapping: [0x1FA6],
titleCaseMapping: [0x1FAE]
},
{
code: 0x1FA7,
upperCaseMapping: [0x1F6F, 0x399],
lowerCaseMapping: [0x1FA7],
titleCaseMapping: [0x1FAF]
},
{
code: 0x1FA8,
upperCaseMapping: [0x1F68, 0x399],
lowerCaseMapping: [0x1FA0],
titleCaseMapping: [0x1FA8]
},
{
code: 0x1FA9,
upperCaseMapping: [0x1F69, 0x399],
lowerCaseMapping: [0x1FA1],
titleCaseMapping: [0x1FA9]
},
{
code: 0x1FAA,
upperCaseMapping: [0x1F6A, 0x399],
lowerCaseMapping: [0x1FA2],
titleCaseMapping: [0x1FAA]
},
{
code: 0x1FAB,
upperCaseMapping: [0x1F6B, 0x399],
lowerCaseMapping: [0x1FA3],
titleCaseMapping: [0x1FAB]
},
{
code: 0x1FAC,
upperCaseMapping: [0x1F6C, 0x399],
lowerCaseMapping: [0x1FA4],
titleCaseMapping: [0x1FAC]
},
{
code: 0x1FAD,
upperCaseMapping: [0x1F6D, 0x399],
lowerCaseMapping: [0x1FA5],
titleCaseMapping: [0x1FAD]
},
{
code: 0x1FAE,
upperCaseMapping: [0x1F6E, 0x399],
lowerCaseMapping: [0x1FA6],
titleCaseMapping: [0x1FAE]
},
{
code: 0x1FAF,
upperCaseMapping: [0x1F6F, 0x399],
lowerCaseMapping: [0x1FA7],
titleCaseMapping: [0x1FAF]
},
{
code: 0x1FB2,
upperCaseMapping: [0x1FBA, 0x399],
lowerCaseMapping: [0x1FB2],
titleCaseMapping: [0x1FBA, 0x345]
},
{
code: 0x1FB3,
upperCaseMapping: [0x391, 0x399],
lowerCaseMapping: [0x1FB3],
titleCaseMapping: [0x1FBC]
},
{
code: 0x1FB4,
upperCaseMapping: [0x386, 0x399],
lowerCaseMapping: [0x1FB4],
titleCaseMapping: [0x386, 0x345]
},
{
code: 0x1FB6,
upperCaseMapping: [0x391, 0x342],
lowerCaseMapping: [0x1FB6],
titleCaseMapping: [0x391, 0x342]
},
{
code: 0x1FB7,
upperCaseMapping: [0x391, 0x342, 0x399],
lowerCaseMapping: [0x1FB7],
titleCaseMapping: [0x391, 0x342, 0x345]
},
{
code: 0x1FBC,
upperCaseMapping: [0x391, 0x399],
lowerCaseMapping: [0x1FB3],
titleCaseMapping: [0x1FBC]
},
{
code: 0x1FC2,
upperCaseMapping: [0x1FCA, 0x399],
lowerCaseMapping: [0x1FC2],
titleCaseMapping: [0x1FCA, 0x345]
},
{
code: 0x1FC3,
upperCaseMapping: [0x397, 0x399],
lowerCaseMapping: [0x1FC3],
titleCaseMapping: [0x1FCC]
},
{
code: 0x1FC4,
upperCaseMapping: [0x389, 0x399],
lowerCaseMapping: [0x1FC4],
titleCaseMapping: [0x389, 0x345]
},
{
code: 0x1FC6,
upperCaseMapping: [0x397, 0x342],
lowerCaseMapping: [0x1FC6],
titleCaseMapping: [0x397, 0x342]
},
{
code: 0x1FC7,
upperCaseMapping: [0x397, 0x342, 0x399],
lowerCaseMapping: [0x1FC7],
titleCaseMapping: [0x397, 0x342, 0x345]
},
{
code: 0x1FCC,
upperCaseMapping: [0x397, 0x399],
lowerCaseMapping: [0x1FC3],
titleCaseMapping: [0x1FCC]
},
{
code: 0x1FD2,
upperCaseMapping: [0x399, 0x308, 0x300],
lowerCaseMapping: [0x1FD2],
titleCaseMapping: [0x399, 0x308, 0x300]
},
{
code: 0x1FD3,
upperCaseMapping: [0x399, 0x308, 0x301],
lowerCaseMapping: [0x1FD3],
titleCaseMapping: [0x399, 0x308, 0x301]
},
{
code: 0x1FD6,
upperCaseMapping: [0x399, 0x342],
lowerCaseMapping: [0x1FD6],
titleCaseMapping: [0x399, 0x342]
},
{
code: 0x1FD7,
upperCaseMapping: [0x399, 0x308, 0x342],
lowerCaseMapping: [0x1FD7],
titleCaseMapping: [0x399, 0x308, 0x342]
},
{
code: 0x1FE2,
upperCaseMapping: [0x3A5, 0x308, 0x300],
lowerCaseMapping: [0x1FE2],
titleCaseMapping: [0x3A5, 0x308, 0x300]
},
{
code: 0x1FE3,
upperCaseMapping: [0x3A5, 0x308, 0x301],
lowerCaseMapping: [0x1FE3],
titleCaseMapping: [0x3A5, 0x308, 0x301]
},
{
code: 0x1FE4,
upperCaseMapping: [0x3A1, 0x313],
lowerCaseMapping: [0x1FE4],
titleCaseMapping: [0x3A1, 0x313]
},
{
code: 0x1FE6,
upperCaseMapping: [0x3A5, 0x342],
lowerCaseMapping: [0x1FE6],
titleCaseMapping: [0x3A5, 0x342]
},
{
code: 0x1FE7,
upperCaseMapping: [0x3A5, 0x308, 0x342],
lowerCaseMapping: [0x1FE7],
titleCaseMapping: [0x3A5, 0x308, 0x342]
},
{
code: 0x1FF2,
upperCaseMapping: [0x1FFA, 0x399],
lowerCaseMapping: [0x1FF2],
titleCaseMapping: [0x1FFA, 0x345]
},
{
code: 0x1FF3,
upperCaseMapping: [0x3A9, 0x399],
lowerCaseMapping: [0x1FF3],
titleCaseMapping: [0x1FFC]
},
{
code: 0x1FF4,
upperCaseMapping: [0x38F, 0x399],
lowerCaseMapping: [0x1FF4],
titleCaseMapping: [0x38F, 0x345]
},
{
code: 0x1FF6,
upperCaseMapping: [0x3A9, 0x342],
lowerCaseMapping: [0x1FF6],
titleCaseMapping: [0x3A9, 0x342]
},
{
code: 0x1FF7,
upperCaseMapping: [0x3A9, 0x342, 0x399],
lowerCaseMapping: [0x1FF7],
titleCaseMapping: [0x3A9, 0x342, 0x345]
},
{
code: 0x1FFC,
upperCaseMapping: [0x3A9, 0x399],
lowerCaseMapping: [0x1FF3],
titleCaseMapping: [0x1FFC]
},
{
code: 0xFB00,
upperCaseMapping: [0x46, 0x46],
lowerCaseMapping: [0xFB00],
titleCaseMapping: [0x46, 0x66]
},
{
code: 0xFB01,
upperCaseMapping: [0x46, 0x49],
lowerCaseMapping: [0xFB01],
titleCaseMapping: [0x46, 0x69]
},
{
code: 0xFB02,
upperCaseMapping: [0x46, 0x4C],
lowerCaseMapping: [0xFB02],
titleCaseMapping: [0x46, 0x6C]
},
{
code: 0xFB03,
upperCaseMapping: [0x46, 0x46, 0x49],
lowerCaseMapping: [0xFB03],
titleCaseMapping: [0x46, 0x66, 0x69]
},
{
code: 0xFB04,
upperCaseMapping: [0x46, 0x46, 0x4C],
lowerCaseMapping: [0xFB04],
titleCaseMapping: [0x46, 0x66, 0x6C]
},
{
code: 0xFB05,
upperCaseMapping: [0x53, 0x54],
lowerCaseMapping: [0xFB05],
titleCaseMapping: [0x53, 0x74]
},
{
code: 0xFB06,
upperCaseMapping: [0x53, 0x54],
lowerCaseMapping: [0xFB06],
titleCaseMapping: [0x53, 0x74]
},
{
code: 0xFB13,
upperCaseMapping: [0x544, 0x546],
lowerCaseMapping: [0xFB13],
titleCaseMapping: [0x544, 0x576]
},
{
code: 0xFB14,
upperCaseMapping: [0x544, 0x535],
lowerCaseMapping: [0xFB14],
titleCaseMapping: [0x544, 0x565]
},
{
code: 0xFB15,
upperCaseMapping: [0x544, 0x53B],
lowerCaseMapping: [0xFB15],
titleCaseMapping: [0x544, 0x56B]
},
{
code: 0xFB16,
upperCaseMapping: [0x54E, 0x546],
lowerCaseMapping: [0xFB16],
titleCaseMapping: [0x54E, 0x576]
},
{
code: 0xFB17,
upperCaseMapping: [0x544, 0x53D],
lowerCaseMapping: [0xFB17],
titleCaseMapping: [0x544, 0x56D]
}
];

FoldingCaseData internalFoldingCaseData[] = [
{
code: 0x41,
mapping: [0x61]
},
{
code: 0x42,
mapping: [0x62]
},
{
code: 0x43,
mapping: [0x63]
},
{
code: 0x44,
mapping: [0x64]
},
{
code: 0x45,
mapping: [0x65]
},
{
code: 0x46,
mapping: [0x66]
},
{
code: 0x47,
mapping: [0x67]
},
{
code: 0x48,
mapping: [0x68]
},
{
code: 0x49,
mapping: [0x69]
},
{
code: 0x4A,
mapping: [0x6A]
},
{
code: 0x4B,
mapping: [0x6B]
},
{
code: 0x4C,
mapping: [0x6C]
},
{
code: 0x4D,
mapping: [0x6D]
},
{
code: 0x4E,
mapping: [0x6E]
},
{
code: 0x4F,
mapping: [0x6F]
},
{
code: 0x50,
mapping: [0x70]
},
{
code: 0x51,
mapping: [0x71]
},
{
code: 0x52,
mapping: [0x72]
},
{
code: 0x53,
mapping: [0x73]
},
{
code: 0x54,
mapping: [0x74]
},
{
code: 0x55,
mapping: [0x75]
},
{
code: 0x56,
mapping: [0x76]
},
{
code: 0x57,
mapping: [0x77]
},
{
code: 0x58,
mapping: [0x78]
},
{
code: 0x59,
mapping: [0x79]
},
{
code: 0x5A,
mapping: [0x7A]
},
{
code: 0xB5,
mapping: [0x3BC]
},
{
code: 0xC0,
mapping: [0xE0]
},
{
code: 0xC1,
mapping: [0xE1]
},
{
code: 0xC2,
mapping: [0xE2]
},
{
code: 0xC3,
mapping: [0xE3]
},
{
code: 0xC4,
mapping: [0xE4]
},
{
code: 0xC5,
mapping: [0xE5]
},
{
code: 0xC6,
mapping: [0xE6]
},
{
code: 0xC7,
mapping: [0xE7]
},
{
code: 0xC8,
mapping: [0xE8]
},
{
code: 0xC9,
mapping: [0xE9]
},
{
code: 0xCA,
mapping: [0xEA]
},
{
code: 0xCB,
mapping: [0xEB]
},
{
code: 0xCC,
mapping: [0xEC]
},
{
code: 0xCD,
mapping: [0xED]
},
{
code: 0xCE,
mapping: [0xEE]
},
{
code: 0xCF,
mapping: [0xEF]
},
{
code: 0xD0,
mapping: [0xF0]
},
{
code: 0xD1,
mapping: [0xF1]
},
{
code: 0xD2,
mapping: [0xF2]
},
{
code: 0xD3,
mapping: [0xF3]
},
{
code: 0xD4,
mapping: [0xF4]
},
{
code: 0xD5,
mapping: [0xF5]
},
{
code: 0xD6,
mapping: [0xF6]
},
{
code: 0xD8,
mapping: [0xF8]
},
{
code: 0xD9,
mapping: [0xF9]
},
{
code: 0xDA,
mapping: [0xFA]
},
{
code: 0xDB,
mapping: [0xFB]
},
{
code: 0xDC,
mapping: [0xFC]
},
{
code: 0xDD,
mapping: [0xFD]
},
{
code: 0xDE,
mapping: [0xFE]
},
{
code: 0xDF,
mapping: [0x73, 0x73]
},
{
code: 0x100,
mapping: [0x101]
},
{
code: 0x102,
mapping: [0x103]
},
{
code: 0x104,
mapping: [0x105]
},
{
code: 0x106,
mapping: [0x107]
},
{
code: 0x108,
mapping: [0x109]
},
{
code: 0x10A,
mapping: [0x10B]
},
{
code: 0x10C,
mapping: [0x10D]
},
{
code: 0x10E,
mapping: [0x10F]
},
{
code: 0x110,
mapping: [0x111]
},
{
code: 0x112,
mapping: [0x113]
},
{
code: 0x114,
mapping: [0x115]
},
{
code: 0x116,
mapping: [0x117]
},
{
code: 0x118,
mapping: [0x119]
},
{
code: 0x11A,
mapping: [0x11B]
},
{
code: 0x11C,
mapping: [0x11D]
},
{
code: 0x11E,
mapping: [0x11F]
},
{
code: 0x120,
mapping: [0x121]
},
{
code: 0x122,
mapping: [0x123]
},
{
code: 0x124,
mapping: [0x125]
},
{
code: 0x126,
mapping: [0x127]
},
{
code: 0x128,
mapping: [0x129]
},
{
code: 0x12A,
mapping: [0x12B]
},
{
code: 0x12C,
mapping: [0x12D]
},
{
code: 0x12E,
mapping: [0x12F]
},
{
code: 0x130,
mapping: [0x69, 0x307]
},
{
code: 0x132,
mapping: [0x133]
},
{
code: 0x134,
mapping: [0x135]
},
{
code: 0x136,
mapping: [0x137]
},
{
code: 0x139,
mapping: [0x13A]
},
{
code: 0x13B,
mapping: [0x13C]
},
{
code: 0x13D,
mapping: [0x13E]
},
{
code: 0x13F,
mapping: [0x140]
},
{
code: 0x141,
mapping: [0x142]
},
{
code: 0x143,
mapping: [0x144]
},
{
code: 0x145,
mapping: [0x146]
},
{
code: 0x147,
mapping: [0x148]
},
{
code: 0x149,
mapping: [0x2BC, 0x6E]
},
{
code: 0x14A,
mapping: [0x14B]
},
{
code: 0x14C,
mapping: [0x14D]
},
{
code: 0x14E,
mapping: [0x14F]
},
{
code: 0x150,
mapping: [0x151]
},
{
code: 0x152,
mapping: [0x153]
},
{
code: 0x154,
mapping: [0x155]
},
{
code: 0x156,
mapping: [0x157]
},
{
code: 0x158,
mapping: [0x159]
},
{
code: 0x15A,
mapping: [0x15B]
},
{
code: 0x15C,
mapping: [0x15D]
},
{
code: 0x15E,
mapping: [0x15F]
},
{
code: 0x160,
mapping: [0x161]
},
{
code: 0x162,
mapping: [0x163]
},
{
code: 0x164,
mapping: [0x165]
},
{
code: 0x166,
mapping: [0x167]
},
{
code: 0x168,
mapping: [0x169]
},
{
code: 0x16A,
mapping: [0x16B]
},
{
code: 0x16C,
mapping: [0x16D]
},
{
code: 0x16E,
mapping: [0x16F]
},
{
code: 0x170,
mapping: [0x171]
},
{
code: 0x172,
mapping: [0x173]
},
{
code: 0x174,
mapping: [0x175]
},
{
code: 0x176,
mapping: [0x177]
},
{
code: 0x178,
mapping: [0xFF]
},
{
code: 0x179,
mapping: [0x17A]
},
{
code: 0x17B,
mapping: [0x17C]
},
{
code: 0x17D,
mapping: [0x17E]
},
{
code: 0x17F,
mapping: [0x73]
},
{
code: 0x181,
mapping: [0x253]
},
{
code: 0x182,
mapping: [0x183]
},
{
code: 0x184,
mapping: [0x185]
},
{
code: 0x186,
mapping: [0x254]
},
{
code: 0x187,
mapping: [0x188]
},
{
code: 0x189,
mapping: [0x256]
},
{
code: 0x18A,
mapping: [0x257]
},
{
code: 0x18B,
mapping: [0x18C]
},
{
code: 0x18E,
mapping: [0x1DD]
},
{
code: 0x18F,
mapping: [0x259]
},
{
code: 0x190,
mapping: [0x25B]
},
{
code: 0x191,
mapping: [0x192]
},
{
code: 0x193,
mapping: [0x260]
},
{
code: 0x194,
mapping: [0x263]
},
{
code: 0x196,
mapping: [0x269]
},
{
code: 0x197,
mapping: [0x268]
},
{
code: 0x198,
mapping: [0x199]
},
{
code: 0x19C,
mapping: [0x26F]
},
{
code: 0x19D,
mapping: [0x272]
},
{
code: 0x19F,
mapping: [0x275]
},
{
code: 0x1A0,
mapping: [0x1A1]
},
{
code: 0x1A2,
mapping: [0x1A3]
},
{
code: 0x1A4,
mapping: [0x1A5]
},
{
code: 0x1A6,
mapping: [0x280]
},
{
code: 0x1A7,
mapping: [0x1A8]
},
{
code: 0x1A9,
mapping: [0x283]
},
{
code: 0x1AC,
mapping: [0x1AD]
},
{
code: 0x1AE,
mapping: [0x288]
},
{
code: 0x1AF,
mapping: [0x1B0]
},
{
code: 0x1B1,
mapping: [0x28A]
},
{
code: 0x1B2,
mapping: [0x28B]
},
{
code: 0x1B3,
mapping: [0x1B4]
},
{
code: 0x1B5,
mapping: [0x1B6]
},
{
code: 0x1B7,
mapping: [0x292]
},
{
code: 0x1B8,
mapping: [0x1B9]
},
{
code: 0x1BC,
mapping: [0x1BD]
},
{
code: 0x1C4,
mapping: [0x1C6]
},
{
code: 0x1C5,
mapping: [0x1C6]
},
{
code: 0x1C7,
mapping: [0x1C9]
},
{
code: 0x1C8,
mapping: [0x1C9]
},
{
code: 0x1CA,
mapping: [0x1CC]
},
{
code: 0x1CB,
mapping: [0x1CC]
},
{
code: 0x1CD,
mapping: [0x1CE]
},
{
code: 0x1CF,
mapping: [0x1D0]
},
{
code: 0x1D1,
mapping: [0x1D2]
},
{
code: 0x1D3,
mapping: [0x1D4]
},
{
code: 0x1D5,
mapping: [0x1D6]
},
{
code: 0x1D7,
mapping: [0x1D8]
},
{
code: 0x1D9,
mapping: [0x1DA]
},
{
code: 0x1DB,
mapping: [0x1DC]
},
{
code: 0x1DE,
mapping: [0x1DF]
},
{
code: 0x1E0,
mapping: [0x1E1]
},
{
code: 0x1E2,
mapping: [0x1E3]
},
{
code: 0x1E4,
mapping: [0x1E5]
},
{
code: 0x1E6,
mapping: [0x1E7]
},
{
code: 0x1E8,
mapping: [0x1E9]
},
{
code: 0x1EA,
mapping: [0x1EB]
},
{
code: 0x1EC,
mapping: [0x1ED]
},
{
code: 0x1EE,
mapping: [0x1EF]
},
{
code: 0x1F0,
mapping: [0x6A, 0x30C]
},
{
code: 0x1F1,
mapping: [0x1F3]
},
{
code: 0x1F2,
mapping: [0x1F3]
},
{
code: 0x1F4,
mapping: [0x1F5]
},
{
code: 0x1F6,
mapping: [0x195]
},
{
code: 0x1F7,
mapping: [0x1BF]
},
{
code: 0x1F8,
mapping: [0x1F9]
},
{
code: 0x1FA,
mapping: [0x1FB]
},
{
code: 0x1FC,
mapping: [0x1FD]
},
{
code: 0x1FE,
mapping: [0x1FF]
},
{
code: 0x200,
mapping: [0x201]
},
{
code: 0x202,
mapping: [0x203]
},
{
code: 0x204,
mapping: [0x205]
},
{
code: 0x206,
mapping: [0x207]
},
{
code: 0x208,
mapping: [0x209]
},
{
code: 0x20A,
mapping: [0x20B]
},
{
code: 0x20C,
mapping: [0x20D]
},
{
code: 0x20E,
mapping: [0x20F]
},
{
code: 0x210,
mapping: [0x211]
},
{
code: 0x212,
mapping: [0x213]
},
{
code: 0x214,
mapping: [0x215]
},
{
code: 0x216,
mapping: [0x217]
},
{
code: 0x218,
mapping: [0x219]
},
{
code: 0x21A,
mapping: [0x21B]
},
{
code: 0x21C,
mapping: [0x21D]
},
{
code: 0x21E,
mapping: [0x21F]
},
{
code: 0x220,
mapping: [0x19E]
},
{
code: 0x222,
mapping: [0x223]
},
{
code: 0x224,
mapping: [0x225]
},
{
code: 0x226,
mapping: [0x227]
},
{
code: 0x228,
mapping: [0x229]
},
{
code: 0x22A,
mapping: [0x22B]
},
{
code: 0x22C,
mapping: [0x22D]
},
{
code: 0x22E,
mapping: [0x22F]
},
{
code: 0x230,
mapping: [0x231]
},
{
code: 0x232,
mapping: [0x233]
},
{
code: 0x23A,
mapping: [0x2C65]
},
{
code: 0x23B,
mapping: [0x23C]
},
{
code: 0x23D,
mapping: [0x19A]
},
{
code: 0x23E,
mapping: [0x2C66]
},
{
code: 0x241,
mapping: [0x242]
},
{
code: 0x243,
mapping: [0x180]
},
{
code: 0x244,
mapping: [0x289]
},
{
code: 0x245,
mapping: [0x28C]
},
{
code: 0x246,
mapping: [0x247]
},
{
code: 0x248,
mapping: [0x249]
},
{
code: 0x24A,
mapping: [0x24B]
},
{
code: 0x24C,
mapping: [0x24D]
},
{
code: 0x24E,
mapping: [0x24F]
},
{
code: 0x345,
mapping: [0x3B9]
},
{
code: 0x386,
mapping: [0x3AC]
},
{
code: 0x388,
mapping: [0x3AD]
},
{
code: 0x389,
mapping: [0x3AE]
},
{
code: 0x38A,
mapping: [0x3AF]
},
{
code: 0x38C,
mapping: [0x3CC]
},
{
code: 0x38E,
mapping: [0x3CD]
},
{
code: 0x38F,
mapping: [0x3CE]
},
{
code: 0x390,
mapping: [0x3B9, 0x308, 0x301]
},
{
code: 0x391,
mapping: [0x3B1]
},
{
code: 0x392,
mapping: [0x3B2]
},
{
code: 0x393,
mapping: [0x3B3]
},
{
code: 0x394,
mapping: [0x3B4]
},
{
code: 0x395,
mapping: [0x3B5]
},
{
code: 0x396,
mapping: [0x3B6]
},
{
code: 0x397,
mapping: [0x3B7]
},
{
code: 0x398,
mapping: [0x3B8]
},
{
code: 0x399,
mapping: [0x3B9]
},
{
code: 0x39A,
mapping: [0x3BA]
},
{
code: 0x39B,
mapping: [0x3BB]
},
{
code: 0x39C,
mapping: [0x3BC]
},
{
code: 0x39D,
mapping: [0x3BD]
},
{
code: 0x39E,
mapping: [0x3BE]
},
{
code: 0x39F,
mapping: [0x3BF]
},
{
code: 0x3A0,
mapping: [0x3C0]
},
{
code: 0x3A1,
mapping: [0x3C1]
},
{
code: 0x3A3,
mapping: [0x3C3]
},
{
code: 0x3A4,
mapping: [0x3C4]
},
{
code: 0x3A5,
mapping: [0x3C5]
},
{
code: 0x3A6,
mapping: [0x3C6]
},
{
code: 0x3A7,
mapping: [0x3C7]
},
{
code: 0x3A8,
mapping: [0x3C8]
},
{
code: 0x3A9,
mapping: [0x3C9]
},
{
code: 0x3AA,
mapping: [0x3CA]
},
{
code: 0x3AB,
mapping: [0x3CB]
},
{
code: 0x3B0,
mapping: [0x3C5, 0x308, 0x301]
},
{
code: 0x3C2,
mapping: [0x3C3]
},
{
code: 0x3D0,
mapping: [0x3B2]
},
{
code: 0x3D1,
mapping: [0x3B8]
},
{
code: 0x3D5,
mapping: [0x3C6]
},
{
code: 0x3D6,
mapping: [0x3C0]
},
{
code: 0x3D8,
mapping: [0x3D9]
},
{
code: 0x3DA,
mapping: [0x3DB]
},
{
code: 0x3DC,
mapping: [0x3DD]
},
{
code: 0x3DE,
mapping: [0x3DF]
},
{
code: 0x3E0,
mapping: [0x3E1]
},
{
code: 0x3E2,
mapping: [0x3E3]
},
{
code: 0x3E4,
mapping: [0x3E5]
},
{
code: 0x3E6,
mapping: [0x3E7]
},
{
code: 0x3E8,
mapping: [0x3E9]
},
{
code: 0x3EA,
mapping: [0x3EB]
},
{
code: 0x3EC,
mapping: [0x3ED]
},
{
code: 0x3EE,
mapping: [0x3EF]
},
{
code: 0x3F0,
mapping: [0x3BA]
},
{
code: 0x3F1,
mapping: [0x3C1]
},
{
code: 0x3F4,
mapping: [0x3B8]
},
{
code: 0x3F5,
mapping: [0x3B5]
},
{
code: 0x3F7,
mapping: [0x3F8]
},
{
code: 0x3F9,
mapping: [0x3F2]
},
{
code: 0x3FA,
mapping: [0x3FB]
},
{
code: 0x3FD,
mapping: [0x37B]
},
{
code: 0x3FE,
mapping: [0x37C]
},
{
code: 0x3FF,
mapping: [0x37D]
},
{
code: 0x400,
mapping: [0x450]
},
{
code: 0x401,
mapping: [0x451]
},
{
code: 0x402,
mapping: [0x452]
},
{
code: 0x403,
mapping: [0x453]
},
{
code: 0x404,
mapping: [0x454]
},
{
code: 0x405,
mapping: [0x455]
},
{
code: 0x406,
mapping: [0x456]
},
{
code: 0x407,
mapping: [0x457]
},
{
code: 0x408,
mapping: [0x458]
},
{
code: 0x409,
mapping: [0x459]
},
{
code: 0x40A,
mapping: [0x45A]
},
{
code: 0x40B,
mapping: [0x45B]
},
{
code: 0x40C,
mapping: [0x45C]
},
{
code: 0x40D,
mapping: [0x45D]
},
{
code: 0x40E,
mapping: [0x45E]
},
{
code: 0x40F,
mapping: [0x45F]
},
{
code: 0x410,
mapping: [0x430]
},
{
code: 0x411,
mapping: [0x431]
},
{
code: 0x412,
mapping: [0x432]
},
{
code: 0x413,
mapping: [0x433]
},
{
code: 0x414,
mapping: [0x434]
},
{
code: 0x415,
mapping: [0x435]
},
{
code: 0x416,
mapping: [0x436]
},
{
code: 0x417,
mapping: [0x437]
},
{
code: 0x418,
mapping: [0x438]
},
{
code: 0x419,
mapping: [0x439]
},
{
code: 0x41A,
mapping: [0x43A]
},
{
code: 0x41B,
mapping: [0x43B]
},
{
code: 0x41C,
mapping: [0x43C]
},
{
code: 0x41D,
mapping: [0x43D]
},
{
code: 0x41E,
mapping: [0x43E]
},
{
code: 0x41F,
mapping: [0x43F]
},
{
code: 0x420,
mapping: [0x440]
},
{
code: 0x421,
mapping: [0x441]
},
{
code: 0x422,
mapping: [0x442]
},
{
code: 0x423,
mapping: [0x443]
},
{
code: 0x424,
mapping: [0x444]
},
{
code: 0x425,
mapping: [0x445]
},
{
code: 0x426,
mapping: [0x446]
},
{
code: 0x427,
mapping: [0x447]
},
{
code: 0x428,
mapping: [0x448]
},
{
code: 0x429,
mapping: [0x449]
},
{
code: 0x42A,
mapping: [0x44A]
},
{
code: 0x42B,
mapping: [0x44B]
},
{
code: 0x42C,
mapping: [0x44C]
},
{
code: 0x42D,
mapping: [0x44D]
},
{
code: 0x42E,
mapping: [0x44E]
},
{
code: 0x42F,
mapping: [0x44F]
},
{
code: 0x460,
mapping: [0x461]
},
{
code: 0x462,
mapping: [0x463]
},
{
code: 0x464,
mapping: [0x465]
},
{
code: 0x466,
mapping: [0x467]
},
{
code: 0x468,
mapping: [0x469]
},
{
code: 0x46A,
mapping: [0x46B]
},
{
code: 0x46C,
mapping: [0x46D]
},
{
code: 0x46E,
mapping: [0x46F]
},
{
code: 0x470,
mapping: [0x471]
},
{
code: 0x472,
mapping: [0x473]
},
{
code: 0x474,
mapping: [0x475]
},
{
code: 0x476,
mapping: [0x477]
},
{
code: 0x478,
mapping: [0x479]
},
{
code: 0x47A,
mapping: [0x47B]
},
{
code: 0x47C,
mapping: [0x47D]
},
{
code: 0x47E,
mapping: [0x47F]
},
{
code: 0x480,
mapping: [0x481]
},
{
code: 0x48A,
mapping: [0x48B]
},
{
code: 0x48C,
mapping: [0x48D]
},
{
code: 0x48E,
mapping: [0x48F]
},
{
code: 0x490,
mapping: [0x491]
},
{
code: 0x492,
mapping: [0x493]
},
{
code: 0x494,
mapping: [0x495]
},
{
code: 0x496,
mapping: [0x497]
},
{
code: 0x498,
mapping: [0x499]
},
{
code: 0x49A,
mapping: [0x49B]
},
{
code: 0x49C,
mapping: [0x49D]
},
{
code: 0x49E,
mapping: [0x49F]
},
{
code: 0x4A0,
mapping: [0x4A1]
},
{
code: 0x4A2,
mapping: [0x4A3]
},
{
code: 0x4A4,
mapping: [0x4A5]
},
{
code: 0x4A6,
mapping: [0x4A7]
},
{
code: 0x4A8,
mapping: [0x4A9]
},
{
code: 0x4AA,
mapping: [0x4AB]
},
{
code: 0x4AC,
mapping: [0x4AD]
},
{
code: 0x4AE,
mapping: [0x4AF]
},
{
code: 0x4B0,
mapping: [0x4B1]
},
{
code: 0x4B2,
mapping: [0x4B3]
},
{
code: 0x4B4,
mapping: [0x4B5]
},
{
code: 0x4B6,
mapping: [0x4B7]
},
{
code: 0x4B8,
mapping: [0x4B9]
},
{
code: 0x4BA,
mapping: [0x4BB]
},
{
code: 0x4BC,
mapping: [0x4BD]
},
{
code: 0x4BE,
mapping: [0x4BF]
},
{
code: 0x4C0,
mapping: [0x4CF]
},
{
code: 0x4C1,
mapping: [0x4C2]
},
{
code: 0x4C3,
mapping: [0x4C4]
},
{
code: 0x4C5,
mapping: [0x4C6]
},
{
code: 0x4C7,
mapping: [0x4C8]
},
{
code: 0x4C9,
mapping: [0x4CA]
},
{
code: 0x4CB,
mapping: [0x4CC]
},
{
code: 0x4CD,
mapping: [0x4CE]
},
{
code: 0x4D0,
mapping: [0x4D1]
},
{
code: 0x4D2,
mapping: [0x4D3]
},
{
code: 0x4D4,
mapping: [0x4D5]
},
{
code: 0x4D6,
mapping: [0x4D7]
},
{
code: 0x4D8,
mapping: [0x4D9]
},
{
code: 0x4DA,
mapping: [0x4DB]
},
{
code: 0x4DC,
mapping: [0x4DD]
},
{
code: 0x4DE,
mapping: [0x4DF]
},
{
code: 0x4E0,
mapping: [0x4E1]
},
{
code: 0x4E2,
mapping: [0x4E3]
},
{
code: 0x4E4,
mapping: [0x4E5]
},
{
code: 0x4E6,
mapping: [0x4E7]
},
{
code: 0x4E8,
mapping: [0x4E9]
},
{
code: 0x4EA,
mapping: [0x4EB]
},
{
code: 0x4EC,
mapping: [0x4ED]
},
{
code: 0x4EE,
mapping: [0x4EF]
},
{
code: 0x4F0,
mapping: [0x4F1]
},
{
code: 0x4F2,
mapping: [0x4F3]
},
{
code: 0x4F4,
mapping: [0x4F5]
},
{
code: 0x4F6,
mapping: [0x4F7]
},
{
code: 0x4F8,
mapping: [0x4F9]
},
{
code: 0x4FA,
mapping: [0x4FB]
},
{
code: 0x4FC,
mapping: [0x4FD]
},
{
code: 0x4FE,
mapping: [0x4FF]
},
{
code: 0x500,
mapping: [0x501]
},
{
code: 0x502,
mapping: [0x503]
},
{
code: 0x504,
mapping: [0x505]
},
{
code: 0x506,
mapping: [0x507]
},
{
code: 0x508,
mapping: [0x509]
},
{
code: 0x50A,
mapping: [0x50B]
},
{
code: 0x50C,
mapping: [0x50D]
},
{
code: 0x50E,
mapping: [0x50F]
},
{
code: 0x510,
mapping: [0x511]
},
{
code: 0x512,
mapping: [0x513]
},
{
code: 0x531,
mapping: [0x561]
},
{
code: 0x532,
mapping: [0x562]
},
{
code: 0x533,
mapping: [0x563]
},
{
code: 0x534,
mapping: [0x564]
},
{
code: 0x535,
mapping: [0x565]
},
{
code: 0x536,
mapping: [0x566]
},
{
code: 0x537,
mapping: [0x567]
},
{
code: 0x538,
mapping: [0x568]
},
{
code: 0x539,
mapping: [0x569]
},
{
code: 0x53A,
mapping: [0x56A]
},
{
code: 0x53B,
mapping: [0x56B]
},
{
code: 0x53C,
mapping: [0x56C]
},
{
code: 0x53D,
mapping: [0x56D]
},
{
code: 0x53E,
mapping: [0x56E]
},
{
code: 0x53F,
mapping: [0x56F]
},
{
code: 0x540,
mapping: [0x570]
},
{
code: 0x541,
mapping: [0x571]
},
{
code: 0x542,
mapping: [0x572]
},
{
code: 0x543,
mapping: [0x573]
},
{
code: 0x544,
mapping: [0x574]
},
{
code: 0x545,
mapping: [0x575]
},
{
code: 0x546,
mapping: [0x576]
},
{
code: 0x547,
mapping: [0x577]
},
{
code: 0x548,
mapping: [0x578]
},
{
code: 0x549,
mapping: [0x579]
},
{
code: 0x54A,
mapping: [0x57A]
},
{
code: 0x54B,
mapping: [0x57B]
},
{
code: 0x54C,
mapping: [0x57C]
},
{
code: 0x54D,
mapping: [0x57D]
},
{
code: 0x54E,
mapping: [0x57E]
},
{
code: 0x54F,
mapping: [0x57F]
},
{
code: 0x550,
mapping: [0x580]
},
{
code: 0x551,
mapping: [0x581]
},
{
code: 0x552,
mapping: [0x582]
},
{
code: 0x553,
mapping: [0x583]
},
{
code: 0x554,
mapping: [0x584]
},
{
code: 0x555,
mapping: [0x585]
},
{
code: 0x556,
mapping: [0x586]
},
{
code: 0x587,
mapping: [0x565, 0x582]
},
{
code: 0x10A0,
mapping: [0x2D00]
},
{
code: 0x10A1,
mapping: [0x2D01]
},
{
code: 0x10A2,
mapping: [0x2D02]
},
{
code: 0x10A3,
mapping: [0x2D03]
},
{
code: 0x10A4,
mapping: [0x2D04]
},
{
code: 0x10A5,
mapping: [0x2D05]
},
{
code: 0x10A6,
mapping: [0x2D06]
},
{
code: 0x10A7,
mapping: [0x2D07]
},
{
code: 0x10A8,
mapping: [0x2D08]
},
{
code: 0x10A9,
mapping: [0x2D09]
},
{
code: 0x10AA,
mapping: [0x2D0A]
},
{
code: 0x10AB,
mapping: [0x2D0B]
},
{
code: 0x10AC,
mapping: [0x2D0C]
},
{
code: 0x10AD,
mapping: [0x2D0D]
},
{
code: 0x10AE,
mapping: [0x2D0E]
},
{
code: 0x10AF,
mapping: [0x2D0F]
},
{
code: 0x10B0,
mapping: [0x2D10]
},
{
code: 0x10B1,
mapping: [0x2D11]
},
{
code: 0x10B2,
mapping: [0x2D12]
},
{
code: 0x10B3,
mapping: [0x2D13]
},
{
code: 0x10B4,
mapping: [0x2D14]
},
{
code: 0x10B5,
mapping: [0x2D15]
},
{
code: 0x10B6,
mapping: [0x2D16]
},
{
code: 0x10B7,
mapping: [0x2D17]
},
{
code: 0x10B8,
mapping: [0x2D18]
},
{
code: 0x10B9,
mapping: [0x2D19]
},
{
code: 0x10BA,
mapping: [0x2D1A]
},
{
code: 0x10BB,
mapping: [0x2D1B]
},
{
code: 0x10BC,
mapping: [0x2D1C]
},
{
code: 0x10BD,
mapping: [0x2D1D]
},
{
code: 0x10BE,
mapping: [0x2D1E]
},
{
code: 0x10BF,
mapping: [0x2D1F]
},
{
code: 0x10C0,
mapping: [0x2D20]
},
{
code: 0x10C1,
mapping: [0x2D21]
},
{
code: 0x10C2,
mapping: [0x2D22]
},
{
code: 0x10C3,
mapping: [0x2D23]
},
{
code: 0x10C4,
mapping: [0x2D24]
},
{
code: 0x10C5,
mapping: [0x2D25]
},
{
code: 0x1E00,
mapping: [0x1E01]
},
{
code: 0x1E02,
mapping: [0x1E03]
},
{
code: 0x1E04,
mapping: [0x1E05]
},
{
code: 0x1E06,
mapping: [0x1E07]
},
{
code: 0x1E08,
mapping: [0x1E09]
},
{
code: 0x1E0A,
mapping: [0x1E0B]
},
{
code: 0x1E0C,
mapping: [0x1E0D]
},
{
code: 0x1E0E,
mapping: [0x1E0F]
},
{
code: 0x1E10,
mapping: [0x1E11]
},
{
code: 0x1E12,
mapping: [0x1E13]
},
{
code: 0x1E14,
mapping: [0x1E15]
},
{
code: 0x1E16,
mapping: [0x1E17]
},
{
code: 0x1E18,
mapping: [0x1E19]
},
{
code: 0x1E1A,
mapping: [0x1E1B]
},
{
code: 0x1E1C,
mapping: [0x1E1D]
},
{
code: 0x1E1E,
mapping: [0x1E1F]
},
{
code: 0x1E20,
mapping: [0x1E21]
},
{
code: 0x1E22,
mapping: [0x1E23]
},
{
code: 0x1E24,
mapping: [0x1E25]
},
{
code: 0x1E26,
mapping: [0x1E27]
},
{
code: 0x1E28,
mapping: [0x1E29]
},
{
code: 0x1E2A,
mapping: [0x1E2B]
},
{
code: 0x1E2C,
mapping: [0x1E2D]
},
{
code: 0x1E2E,
mapping: [0x1E2F]
},
{
code: 0x1E30,
mapping: [0x1E31]
},
{
code: 0x1E32,
mapping: [0x1E33]
},
{
code: 0x1E34,
mapping: [0x1E35]
},
{
code: 0x1E36,
mapping: [0x1E37]
},
{
code: 0x1E38,
mapping: [0x1E39]
},
{
code: 0x1E3A,
mapping: [0x1E3B]
},
{
code: 0x1E3C,
mapping: [0x1E3D]
},
{
code: 0x1E3E,
mapping: [0x1E3F]
},
{
code: 0x1E40,
mapping: [0x1E41]
},
{
code: 0x1E42,
mapping: [0x1E43]
},
{
code: 0x1E44,
mapping: [0x1E45]
},
{
code: 0x1E46,
mapping: [0x1E47]
},
{
code: 0x1E48,
mapping: [0x1E49]
},
{
code: 0x1E4A,
mapping: [0x1E4B]
},
{
code: 0x1E4C,
mapping: [0x1E4D]
},
{
code: 0x1E4E,
mapping: [0x1E4F]
},
{
code: 0x1E50,
mapping: [0x1E51]
},
{
code: 0x1E52,
mapping: [0x1E53]
},
{
code: 0x1E54,
mapping: [0x1E55]
},
{
code: 0x1E56,
mapping: [0x1E57]
},
{
code: 0x1E58,
mapping: [0x1E59]
},
{
code: 0x1E5A,
mapping: [0x1E5B]
},
{
code: 0x1E5C,
mapping: [0x1E5D]
},
{
code: 0x1E5E,
mapping: [0x1E5F]
},
{
code: 0x1E60,
mapping: [0x1E61]
},
{
code: 0x1E62,
mapping: [0x1E63]
},
{
code: 0x1E64,
mapping: [0x1E65]
},
{
code: 0x1E66,
mapping: [0x1E67]
},
{
code: 0x1E68,
mapping: [0x1E69]
},
{
code: 0x1E6A,
mapping: [0x1E6B]
},
{
code: 0x1E6C,
mapping: [0x1E6D]
},
{
code: 0x1E6E,
mapping: [0x1E6F]
},
{
code: 0x1E70,
mapping: [0x1E71]
},
{
code: 0x1E72,
mapping: [0x1E73]
},
{
code: 0x1E74,
mapping: [0x1E75]
},
{
code: 0x1E76,
mapping: [0x1E77]
},
{
code: 0x1E78,
mapping: [0x1E79]
},
{
code: 0x1E7A,
mapping: [0x1E7B]
},
{
code: 0x1E7C,
mapping: [0x1E7D]
},
{
code: 0x1E7E,
mapping: [0x1E7F]
},
{
code: 0x1E80,
mapping: [0x1E81]
},
{
code: 0x1E82,
mapping: [0x1E83]
},
{
code: 0x1E84,
mapping: [0x1E85]
},
{
code: 0x1E86,
mapping: [0x1E87]
},
{
code: 0x1E88,
mapping: [0x1E89]
},
{
code: 0x1E8A,
mapping: [0x1E8B]
},
{
code: 0x1E8C,
mapping: [0x1E8D]
},
{
code: 0x1E8E,
mapping: [0x1E8F]
},
{
code: 0x1E90,
mapping: [0x1E91]
},
{
code: 0x1E92,
mapping: [0x1E93]
},
{
code: 0x1E94,
mapping: [0x1E95]
},
{
code: 0x1E96,
mapping: [0x68, 0x331]
},
{
code: 0x1E97,
mapping: [0x74, 0x308]
},
{
code: 0x1E98,
mapping: [0x77, 0x30A]
},
{
code: 0x1E99,
mapping: [0x79, 0x30A]
},
{
code: 0x1E9A,
mapping: [0x61, 0x2BE]
},
{
code: 0x1E9B,
mapping: [0x1E61]
},
{
code: 0x1EA0,
mapping: [0x1EA1]
},
{
code: 0x1EA2,
mapping: [0x1EA3]
},
{
code: 0x1EA4,
mapping: [0x1EA5]
},
{
code: 0x1EA6,
mapping: [0x1EA7]
},
{
code: 0x1EA8,
mapping: [0x1EA9]
},
{
code: 0x1EAA,
mapping: [0x1EAB]
},
{
code: 0x1EAC,
mapping: [0x1EAD]
},
{
code: 0x1EAE,
mapping: [0x1EAF]
},
{
code: 0x1EB0,
mapping: [0x1EB1]
},
{
code: 0x1EB2,
mapping: [0x1EB3]
},
{
code: 0x1EB4,
mapping: [0x1EB5]
},
{
code: 0x1EB6,
mapping: [0x1EB7]
},
{
code: 0x1EB8,
mapping: [0x1EB9]
},
{
code: 0x1EBA,
mapping: [0x1EBB]
},
{
code: 0x1EBC,
mapping: [0x1EBD]
},
{
code: 0x1EBE,
mapping: [0x1EBF]
},
{
code: 0x1EC0,
mapping: [0x1EC1]
},
{
code: 0x1EC2,
mapping: [0x1EC3]
},
{
code: 0x1EC4,
mapping: [0x1EC5]
},
{
code: 0x1EC6,
mapping: [0x1EC7]
},
{
code: 0x1EC8,
mapping: [0x1EC9]
},
{
code: 0x1ECA,
mapping: [0x1ECB]
},
{
code: 0x1ECC,
mapping: [0x1ECD]
},
{
code: 0x1ECE,
mapping: [0x1ECF]
},
{
code: 0x1ED0,
mapping: [0x1ED1]
},
{
code: 0x1ED2,
mapping: [0x1ED3]
},
{
code: 0x1ED4,
mapping: [0x1ED5]
},
{
code: 0x1ED6,
mapping: [0x1ED7]
},
{
code: 0x1ED8,
mapping: [0x1ED9]
},
{
code: 0x1EDA,
mapping: [0x1EDB]
},
{
code: 0x1EDC,
mapping: [0x1EDD]
},
{
code: 0x1EDE,
mapping: [0x1EDF]
},
{
code: 0x1EE0,
mapping: [0x1EE1]
},
{
code: 0x1EE2,
mapping: [0x1EE3]
},
{
code: 0x1EE4,
mapping: [0x1EE5]
},
{
code: 0x1EE6,
mapping: [0x1EE7]
},
{
code: 0x1EE8,
mapping: [0x1EE9]
},
{
code: 0x1EEA,
mapping: [0x1EEB]
},
{
code: 0x1EEC,
mapping: [0x1EED]
},
{
code: 0x1EEE,
mapping: [0x1EEF]
},
{
code: 0x1EF0,
mapping: [0x1EF1]
},
{
code: 0x1EF2,
mapping: [0x1EF3]
},
{
code: 0x1EF4,
mapping: [0x1EF5]
},
{
code: 0x1EF6,
mapping: [0x1EF7]
},
{
code: 0x1EF8,
mapping: [0x1EF9]
},
{
code: 0x1F08,
mapping: [0x1F00]
},
{
code: 0x1F09,
mapping: [0x1F01]
},
{
code: 0x1F0A,
mapping: [0x1F02]
},
{
code: 0x1F0B,
mapping: [0x1F03]
},
{
code: 0x1F0C,
mapping: [0x1F04]
},
{
code: 0x1F0D,
mapping: [0x1F05]
},
{
code: 0x1F0E,
mapping: [0x1F06]
},
{
code: 0x1F0F,
mapping: [0x1F07]
},
{
code: 0x1F18,
mapping: [0x1F10]
},
{
code: 0x1F19,
mapping: [0x1F11]
},
{
code: 0x1F1A,
mapping: [0x1F12]
},
{
code: 0x1F1B,
mapping: [0x1F13]
},
{
code: 0x1F1C,
mapping: [0x1F14]
},
{
code: 0x1F1D,
mapping: [0x1F15]
},
{
code: 0x1F28,
mapping: [0x1F20]
},
{
code: 0x1F29,
mapping: [0x1F21]
},
{
code: 0x1F2A,
mapping: [0x1F22]
},
{
code: 0x1F2B,
mapping: [0x1F23]
},
{
code: 0x1F2C,
mapping: [0x1F24]
},
{
code: 0x1F2D,
mapping: [0x1F25]
},
{
code: 0x1F2E,
mapping: [0x1F26]
},
{
code: 0x1F2F,
mapping: [0x1F27]
},
{
code: 0x1F38,
mapping: [0x1F30]
},
{
code: 0x1F39,
mapping: [0x1F31]
},
{
code: 0x1F3A,
mapping: [0x1F32]
},
{
code: 0x1F3B,
mapping: [0x1F33]
},
{
code: 0x1F3C,
mapping: [0x1F34]
},
{
code: 0x1F3D,
mapping: [0x1F35]
},
{
code: 0x1F3E,
mapping: [0x1F36]
},
{
code: 0x1F3F,
mapping: [0x1F37]
},
{
code: 0x1F48,
mapping: [0x1F40]
},
{
code: 0x1F49,
mapping: [0x1F41]
},
{
code: 0x1F4A,
mapping: [0x1F42]
},
{
code: 0x1F4B,
mapping: [0x1F43]
},
{
code: 0x1F4C,
mapping: [0x1F44]
},
{
code: 0x1F4D,
mapping: [0x1F45]
},
{
code: 0x1F50,
mapping: [0x3C5, 0x313]
},
{
code: 0x1F52,
mapping: [0x3C5, 0x313, 0x300]
},
{
code: 0x1F54,
mapping: [0x3C5, 0x313, 0x301]
},
{
code: 0x1F56,
mapping: [0x3C5, 0x313, 0x342]
},
{
code: 0x1F59,
mapping: [0x1F51]
},
{
code: 0x1F5B,
mapping: [0x1F53]
},
{
code: 0x1F5D,
mapping: [0x1F55]
},
{
code: 0x1F5F,
mapping: [0x1F57]
},
{
code: 0x1F68,
mapping: [0x1F60]
},
{
code: 0x1F69,
mapping: [0x1F61]
},
{
code: 0x1F6A,
mapping: [0x1F62]
},
{
code: 0x1F6B,
mapping: [0x1F63]
},
{
code: 0x1F6C,
mapping: [0x1F64]
},
{
code: 0x1F6D,
mapping: [0x1F65]
},
{
code: 0x1F6E,
mapping: [0x1F66]
},
{
code: 0x1F6F,
mapping: [0x1F67]
},
{
code: 0x1F80,
mapping: [0x1F00, 0x3B9]
},
{
code: 0x1F81,
mapping: [0x1F01, 0x3B9]
},
{
code: 0x1F82,
mapping: [0x1F02, 0x3B9]
},
{
code: 0x1F83,
mapping: [0x1F03, 0x3B9]
},
{
code: 0x1F84,
mapping: [0x1F04, 0x3B9]
},
{
code: 0x1F85,
mapping: [0x1F05, 0x3B9]
},
{
code: 0x1F86,
mapping: [0x1F06, 0x3B9]
},
{
code: 0x1F87,
mapping: [0x1F07, 0x3B9]
},
{
code: 0x1F88,
mapping: [0x1F00, 0x3B9]
},
{
code: 0x1F89,
mapping: [0x1F01, 0x3B9]
},
{
code: 0x1F8A,
mapping: [0x1F02, 0x3B9]
},
{
code: 0x1F8B,
mapping: [0x1F03, 0x3B9]
},
{
code: 0x1F8C,
mapping: [0x1F04, 0x3B9]
},
{
code: 0x1F8D,
mapping: [0x1F05, 0x3B9]
},
{
code: 0x1F8E,
mapping: [0x1F06, 0x3B9]
},
{
code: 0x1F8F,
mapping: [0x1F07, 0x3B9]
},
{
code: 0x1F90,
mapping: [0x1F20, 0x3B9]
},
{
code: 0x1F91,
mapping: [0x1F21, 0x3B9]
},
{
code: 0x1F92,
mapping: [0x1F22, 0x3B9]
},
{
code: 0x1F93,
mapping: [0x1F23, 0x3B9]
},
{
code: 0x1F94,
mapping: [0x1F24, 0x3B9]
},
{
code: 0x1F95,
mapping: [0x1F25, 0x3B9]
},
{
code: 0x1F96,
mapping: [0x1F26, 0x3B9]
},
{
code: 0x1F97,
mapping: [0x1F27, 0x3B9]
},
{
code: 0x1F98,
mapping: [0x1F20, 0x3B9]
},
{
code: 0x1F99,
mapping: [0x1F21, 0x3B9]
},
{
code: 0x1F9A,
mapping: [0x1F22, 0x3B9]
},
{
code: 0x1F9B,
mapping: [0x1F23, 0x3B9]
},
{
code: 0x1F9C,
mapping: [0x1F24, 0x3B9]
},
{
code: 0x1F9D,
mapping: [0x1F25, 0x3B9]
},
{
code: 0x1F9E,
mapping: [0x1F26, 0x3B9]
},
{
code: 0x1F9F,
mapping: [0x1F27, 0x3B9]
},
{
code: 0x1FA0,
mapping: [0x1F60, 0x3B9]
},
{
code: 0x1FA1,
mapping: [0x1F61, 0x3B9]
},
{
code: 0x1FA2,
mapping: [0x1F62, 0x3B9]
},
{
code: 0x1FA3,
mapping: [0x1F63, 0x3B9]
},
{
code: 0x1FA4,
mapping: [0x1F64, 0x3B9]
},
{
code: 0x1FA5,
mapping: [0x1F65, 0x3B9]
},
{
code: 0x1FA6,
mapping: [0x1F66, 0x3B9]
},
{
code: 0x1FA7,
mapping: [0x1F67, 0x3B9]
},
{
code: 0x1FA8,
mapping: [0x1F60, 0x3B9]
},
{
code: 0x1FA9,
mapping: [0x1F61, 0x3B9]
},
{
code: 0x1FAA,
mapping: [0x1F62, 0x3B9]
},
{
code: 0x1FAB,
mapping: [0x1F63, 0x3B9]
},
{
code: 0x1FAC,
mapping: [0x1F64, 0x3B9]
},
{
code: 0x1FAD,
mapping: [0x1F65, 0x3B9]
},
{
code: 0x1FAE,
mapping: [0x1F66, 0x3B9]
},
{
code: 0x1FAF,
mapping: [0x1F67, 0x3B9]
},
{
code: 0x1FB2,
mapping: [0x1F70, 0x3B9]
},
{
code: 0x1FB3,
mapping: [0x3B1, 0x3B9]
},
{
code: 0x1FB4,
mapping: [0x3AC, 0x3B9]
},
{
code: 0x1FB6,
mapping: [0x3B1, 0x342]
},
{
code: 0x1FB7,
mapping: [0x3B1, 0x342, 0x3B9]
},
{
code: 0x1FB8,
mapping: [0x1FB0]
},
{
code: 0x1FB9,
mapping: [0x1FB1]
},
{
code: 0x1FBA,
mapping: [0x1F70]
},
{
code: 0x1FBB,
mapping: [0x1F71]
},
{
code: 0x1FBC,
mapping: [0x3B1, 0x3B9]
},
{
code: 0x1FBE,
mapping: [0x3B9]
},
{
code: 0x1FC2,
mapping: [0x1F74, 0x3B9]
},
{
code: 0x1FC3,
mapping: [0x3B7, 0x3B9]
},
{
code: 0x1FC4,
mapping: [0x3AE, 0x3B9]
},
{
code: 0x1FC6,
mapping: [0x3B7, 0x342]
},
{
code: 0x1FC7,
mapping: [0x3B7, 0x342, 0x3B9]
},
{
code: 0x1FC8,
mapping: [0x1F72]
},
{
code: 0x1FC9,
mapping: [0x1F73]
},
{
code: 0x1FCA,
mapping: [0x1F74]
},
{
code: 0x1FCB,
mapping: [0x1F75]
},
{
code: 0x1FCC,
mapping: [0x3B7, 0x3B9]
},
{
code: 0x1FD2,
mapping: [0x3B9, 0x308, 0x300]
},
{
code: 0x1FD3,
mapping: [0x3B9, 0x308, 0x301]
},
{
code: 0x1FD6,
mapping: [0x3B9, 0x342]
},
{
code: 0x1FD7,
mapping: [0x3B9, 0x308, 0x342]
},
{
code: 0x1FD8,
mapping: [0x1FD0]
},
{
code: 0x1FD9,
mapping: [0x1FD1]
},
{
code: 0x1FDA,
mapping: [0x1F76]
},
{
code: 0x1FDB,
mapping: [0x1F77]
},
{
code: 0x1FE2,
mapping: [0x3C5, 0x308, 0x300]
},
{
code: 0x1FE3,
mapping: [0x3C5, 0x308, 0x301]
},
{
code: 0x1FE4,
mapping: [0x3C1, 0x313]
},
{
code: 0x1FE6,
mapping: [0x3C5, 0x342]
},
{
code: 0x1FE7,
mapping: [0x3C5, 0x308, 0x342]
},
{
code: 0x1FE8,
mapping: [0x1FE0]
},
{
code: 0x1FE9,
mapping: [0x1FE1]
},
{
code: 0x1FEA,
mapping: [0x1F7A]
},
{
code: 0x1FEB,
mapping: [0x1F7B]
},
{
code: 0x1FEC,
mapping: [0x1FE5]
},
{
code: 0x1FF2,
mapping: [0x1F7C, 0x3B9]
},
{
code: 0x1FF3,
mapping: [0x3C9, 0x3B9]
},
{
code: 0x1FF4,
mapping: [0x3CE, 0x3B9]
},
{
code: 0x1FF6,
mapping: [0x3C9, 0x342]
},
{
code: 0x1FF7,
mapping: [0x3C9, 0x342, 0x3B9]
},
{
code: 0x1FF8,
mapping: [0x1F78]
},
{
code: 0x1FF9,
mapping: [0x1F79]
},
{
code: 0x1FFA,
mapping: [0x1F7C]
},
{
code: 0x1FFB,
mapping: [0x1F7D]
},
{
code: 0x1FFC,
mapping: [0x3C9, 0x3B9]
},
{
code: 0x2126,
mapping: [0x3C9]
},
{
code: 0x212A,
mapping: [0x6B]
},
{
code: 0x212B,
mapping: [0xE5]
},
{
code: 0x2132,
mapping: [0x214E]
},
{
code: 0x2160,
mapping: [0x2170]
},
{
code: 0x2161,
mapping: [0x2171]
},
{
code: 0x2162,
mapping: [0x2172]
},
{
code: 0x2163,
mapping: [0x2173]
},
{
code: 0x2164,
mapping: [0x2174]
},
{
code: 0x2165,
mapping: [0x2175]
},
{
code: 0x2166,
mapping: [0x2176]
},
{
code: 0x2167,
mapping: [0x2177]
},
{
code: 0x2168,
mapping: [0x2178]
},
{
code: 0x2169,
mapping: [0x2179]
},
{
code: 0x216A,
mapping: [0x217A]
},
{
code: 0x216B,
mapping: [0x217B]
},
{
code: 0x216C,
mapping: [0x217C]
},
{
code: 0x216D,
mapping: [0x217D]
},
{
code: 0x216E,
mapping: [0x217E]
},
{
code: 0x216F,
mapping: [0x217F]
},
{
code: 0x2183,
mapping: [0x2184]
},
{
code: 0x24B6,
mapping: [0x24D0]
},
{
code: 0x24B7,
mapping: [0x24D1]
},
{
code: 0x24B8,
mapping: [0x24D2]
},
{
code: 0x24B9,
mapping: [0x24D3]
},
{
code: 0x24BA,
mapping: [0x24D4]
},
{
code: 0x24BB,
mapping: [0x24D5]
},
{
code: 0x24BC,
mapping: [0x24D6]
},
{
code: 0x24BD,
mapping: [0x24D7]
},
{
code: 0x24BE,
mapping: [0x24D8]
},
{
code: 0x24BF,
mapping: [0x24D9]
},
{
code: 0x24C0,
mapping: [0x24DA]
},
{
code: 0x24C1,
mapping: [0x24DB]
},
{
code: 0x24C2,
mapping: [0x24DC]
},
{
code: 0x24C3,
mapping: [0x24DD]
},
{
code: 0x24C4,
mapping: [0x24DE]
},
{
code: 0x24C5,
mapping: [0x24DF]
},
{
code: 0x24C6,
mapping: [0x24E0]
},
{
code: 0x24C7,
mapping: [0x24E1]
},
{
code: 0x24C8,
mapping: [0x24E2]
},
{
code: 0x24C9,
mapping: [0x24E3]
},
{
code: 0x24CA,
mapping: [0x24E4]
},
{
code: 0x24CB,
mapping: [0x24E5]
},
{
code: 0x24CC,
mapping: [0x24E6]
},
{
code: 0x24CD,
mapping: [0x24E7]
},
{
code: 0x24CE,
mapping: [0x24E8]
},
{
code: 0x24CF,
mapping: [0x24E9]
},
{
code: 0x2C00,
mapping: [0x2C30]
},
{
code: 0x2C01,
mapping: [0x2C31]
},
{
code: 0x2C02,
mapping: [0x2C32]
},
{
code: 0x2C03,
mapping: [0x2C33]
},
{
code: 0x2C04,
mapping: [0x2C34]
},
{
code: 0x2C05,
mapping: [0x2C35]
},
{
code: 0x2C06,
mapping: [0x2C36]
},
{
code: 0x2C07,
mapping: [0x2C37]
},
{
code: 0x2C08,
mapping: [0x2C38]
},
{
code: 0x2C09,
mapping: [0x2C39]
},
{
code: 0x2C0A,
mapping: [0x2C3A]
},
{
code: 0x2C0B,
mapping: [0x2C3B]
},
{
code: 0x2C0C,
mapping: [0x2C3C]
},
{
code: 0x2C0D,
mapping: [0x2C3D]
},
{
code: 0x2C0E,
mapping: [0x2C3E]
},
{
code: 0x2C0F,
mapping: [0x2C3F]
},
{
code: 0x2C10,
mapping: [0x2C40]
},
{
code: 0x2C11,
mapping: [0x2C41]
},
{
code: 0x2C12,
mapping: [0x2C42]
},
{
code: 0x2C13,
mapping: [0x2C43]
},
{
code: 0x2C14,
mapping: [0x2C44]
},
{
code: 0x2C15,
mapping: [0x2C45]
},
{
code: 0x2C16,
mapping: [0x2C46]
},
{
code: 0x2C17,
mapping: [0x2C47]
},
{
code: 0x2C18,
mapping: [0x2C48]
},
{
code: 0x2C19,
mapping: [0x2C49]
},
{
code: 0x2C1A,
mapping: [0x2C4A]
},
{
code: 0x2C1B,
mapping: [0x2C4B]
},
{
code: 0x2C1C,
mapping: [0x2C4C]
},
{
code: 0x2C1D,
mapping: [0x2C4D]
},
{
code: 0x2C1E,
mapping: [0x2C4E]
},
{
code: 0x2C1F,
mapping: [0x2C4F]
},
{
code: 0x2C20,
mapping: [0x2C50]
},
{
code: 0x2C21,
mapping: [0x2C51]
},
{
code: 0x2C22,
mapping: [0x2C52]
},
{
code: 0x2C23,
mapping: [0x2C53]
},
{
code: 0x2C24,
mapping: [0x2C54]
},
{
code: 0x2C25,
mapping: [0x2C55]
},
{
code: 0x2C26,
mapping: [0x2C56]
},
{
code: 0x2C27,
mapping: [0x2C57]
},
{
code: 0x2C28,
mapping: [0x2C58]
},
{
code: 0x2C29,
mapping: [0x2C59]
},
{
code: 0x2C2A,
mapping: [0x2C5A]
},
{
code: 0x2C2B,
mapping: [0x2C5B]
},
{
code: 0x2C2C,
mapping: [0x2C5C]
},
{
code: 0x2C2D,
mapping: [0x2C5D]
},
{
code: 0x2C2E,
mapping: [0x2C5E]
},
{
code: 0x2C60,
mapping: [0x2C61]
},
{
code: 0x2C62,
mapping: [0x26B]
},
{
code: 0x2C63,
mapping: [0x1D7D]
},
{
code: 0x2C64,
mapping: [0x27D]
},
{
code: 0x2C67,
mapping: [0x2C68]
},
{
code: 0x2C69,
mapping: [0x2C6A]
},
{
code: 0x2C6B,
mapping: [0x2C6C]
},
{
code: 0x2C75,
mapping: [0x2C76]
},
{
code: 0x2C80,
mapping: [0x2C81]
},
{
code: 0x2C82,
mapping: [0x2C83]
},
{
code: 0x2C84,
mapping: [0x2C85]
},
{
code: 0x2C86,
mapping: [0x2C87]
},
{
code: 0x2C88,
mapping: [0x2C89]
},
{
code: 0x2C8A,
mapping: [0x2C8B]
},
{
code: 0x2C8C,
mapping: [0x2C8D]
},
{
code: 0x2C8E,
mapping: [0x2C8F]
},
{
code: 0x2C90,
mapping: [0x2C91]
},
{
code: 0x2C92,
mapping: [0x2C93]
},
{
code: 0x2C94,
mapping: [0x2C95]
},
{
code: 0x2C96,
mapping: [0x2C97]
},
{
code: 0x2C98,
mapping: [0x2C99]
},
{
code: 0x2C9A,
mapping: [0x2C9B]
},
{
code: 0x2C9C,
mapping: [0x2C9D]
},
{
code: 0x2C9E,
mapping: [0x2C9F]
},
{
code: 0x2CA0,
mapping: [0x2CA1]
},
{
code: 0x2CA2,
mapping: [0x2CA3]
},
{
code: 0x2CA4,
mapping: [0x2CA5]
},
{
code: 0x2CA6,
mapping: [0x2CA7]
},
{
code: 0x2CA8,
mapping: [0x2CA9]
},
{
code: 0x2CAA,
mapping: [0x2CAB]
},
{
code: 0x2CAC,
mapping: [0x2CAD]
},
{
code: 0x2CAE,
mapping: [0x2CAF]
},
{
code: 0x2CB0,
mapping: [0x2CB1]
},
{
code: 0x2CB2,
mapping: [0x2CB3]
},
{
code: 0x2CB4,
mapping: [0x2CB5]
},
{
code: 0x2CB6,
mapping: [0x2CB7]
},
{
code: 0x2CB8,
mapping: [0x2CB9]
},
{
code: 0x2CBA,
mapping: [0x2CBB]
},
{
code: 0x2CBC,
mapping: [0x2CBD]
},
{
code: 0x2CBE,
mapping: [0x2CBF]
},
{
code: 0x2CC0,
mapping: [0x2CC1]
},
{
code: 0x2CC2,
mapping: [0x2CC3]
},
{
code: 0x2CC4,
mapping: [0x2CC5]
},
{
code: 0x2CC6,
mapping: [0x2CC7]
},
{
code: 0x2CC8,
mapping: [0x2CC9]
},
{
code: 0x2CCA,
mapping: [0x2CCB]
},
{
code: 0x2CCC,
mapping: [0x2CCD]
},
{
code: 0x2CCE,
mapping: [0x2CCF]
},
{
code: 0x2CD0,
mapping: [0x2CD1]
},
{
code: 0x2CD2,
mapping: [0x2CD3]
},
{
code: 0x2CD4,
mapping: [0x2CD5]
},
{
code: 0x2CD6,
mapping: [0x2CD7]
},
{
code: 0x2CD8,
mapping: [0x2CD9]
},
{
code: 0x2CDA,
mapping: [0x2CDB]
},
{
code: 0x2CDC,
mapping: [0x2CDD]
},
{
code: 0x2CDE,
mapping: [0x2CDF]
},
{
code: 0x2CE0,
mapping: [0x2CE1]
},
{
code: 0x2CE2,
mapping: [0x2CE3]
},
{
code: 0xFB00,
mapping: [0x66, 0x66]
},
{
code: 0xFB01,
mapping: [0x66, 0x69]
},
{
code: 0xFB02,
mapping: [0x66, 0x6C]
},
{
code: 0xFB03,
mapping: [0x66, 0x66, 0x69]
},
{
code: 0xFB04,
mapping: [0x66, 0x66, 0x6C]
},
{
code: 0xFB05,
mapping: [0x73, 0x74]
},
{
code: 0xFB06,
mapping: [0x73, 0x74]
},
{
code: 0xFB13,
mapping: [0x574, 0x576]
},
{
code: 0xFB14,
mapping: [0x574, 0x565]
},
{
code: 0xFB15,
mapping: [0x574, 0x56B]
},
{
code: 0xFB16,
mapping: [0x57E, 0x576]
},
{
code: 0xFB17,
mapping: [0x574, 0x56D]
},
{
code: 0xFF21,
mapping: [0xFF41]
},
{
code: 0xFF22,
mapping: [0xFF42]
},
{
code: 0xFF23,
mapping: [0xFF43]
},
{
code: 0xFF24,
mapping: [0xFF44]
},
{
code: 0xFF25,
mapping: [0xFF45]
},
{
code: 0xFF26,
mapping: [0xFF46]
},
{
code: 0xFF27,
mapping: [0xFF47]
},
{
code: 0xFF28,
mapping: [0xFF48]
},
{
code: 0xFF29,
mapping: [0xFF49]
},
{
code: 0xFF2A,
mapping: [0xFF4A]
},
{
code: 0xFF2B,
mapping: [0xFF4B]
},
{
code: 0xFF2C,
mapping: [0xFF4C]
},
{
code: 0xFF2D,
mapping: [0xFF4D]
},
{
code: 0xFF2E,
mapping: [0xFF4E]
},
{
code: 0xFF2F,
mapping: [0xFF4F]
},
{
code: 0xFF30,
mapping: [0xFF50]
},
{
code: 0xFF31,
mapping: [0xFF51]
},
{
code: 0xFF32,
mapping: [0xFF52]
},
{
code: 0xFF33,
mapping: [0xFF53]
},
{
code: 0xFF34,
mapping: [0xFF54]
},
{
code: 0xFF35,
mapping: [0xFF55]
},
{
code: 0xFF36,
mapping: [0xFF56]
},
{
code: 0xFF37,
mapping: [0xFF57]
},
{
code: 0xFF38,
mapping: [0xFF58]
},
{
code: 0xFF39,
mapping: [0xFF59]
},
{
code: 0xFF3A,
mapping: [0xFF5A]
},
{
code: 0x10400,
mapping: [0x10428]
},
{
code: 0x10401,
mapping: [0x10429]
},
{
code: 0x10402,
mapping: [0x1042A]
},
{
code: 0x10403,
mapping: [0x1042B]
},
{
code: 0x10404,
mapping: [0x1042C]
},
{
code: 0x10405,
mapping: [0x1042D]
},
{
code: 0x10406,
mapping: [0x1042E]
},
{
code: 0x10407,
mapping: [0x1042F]
},
{
code: 0x10408,
mapping: [0x10430]
},
{
code: 0x10409,
mapping: [0x10431]
},
{
code: 0x1040A,
mapping: [0x10432]
},
{
code: 0x1040B,
mapping: [0x10433]
},
{
code: 0x1040C,
mapping: [0x10434]
},
{
code: 0x1040D,
mapping: [0x10435]
},
{
code: 0x1040E,
mapping: [0x10436]
},
{
code: 0x1040F,
mapping: [0x10437]
},
{
code: 0x10410,
mapping: [0x10438]
},
{
code: 0x10411,
mapping: [0x10439]
},
{
code: 0x10412,
mapping: [0x1043A]
},
{
code: 0x10413,
mapping: [0x1043B]
},
{
code: 0x10414,
mapping: [0x1043C]
},
{
code: 0x10415,
mapping: [0x1043D]
},
{
code: 0x10416,
mapping: [0x1043E]
},
{
code: 0x10417,
mapping: [0x1043F]
},
{
code: 0x10418,
mapping: [0x10440]
},
{
code: 0x10419,
mapping: [0x10441]
},
{
code: 0x1041A,
mapping: [0x10442]
},
{
code: 0x1041B,
mapping: [0x10443]
},
{
code: 0x1041C,
mapping: [0x10444]
},
{
code: 0x1041D,
mapping: [0x10445]
},
{
code: 0x1041E,
mapping: [0x10446]
},
{
code: 0x1041F,
mapping: [0x10447]
},
{
code: 0x10420,
mapping: [0x10448]
},
{
code: 0x10421,
mapping: [0x10449]
},
{
code: 0x10422,
mapping: [0x1044A]
},
{
code: 0x10423,
mapping: [0x1044B]
},
{
code: 0x10424,
mapping: [0x1044C]
},
{
code: 0x10425,
mapping: [0x1044D]
},
{
code: 0x10426,
mapping: [0x1044E]
},
{
code: 0x10427,
mapping: [0x1044F]
}
];

}
